/*
       Licensed to the Apache Software Foundation (ASF) under one
       or more contributor license agreements.  See the NOTICE file
       distributed with this work for additional information
       regarding copyright ownership.  The ASF licenses this file
       to you under the Apache License, Version 2.0 (the
       "License"); you may not use this file except in compliance
       with the License.  You may obtain a copy of the License at

         http://www.apache.org/licenses/LICENSE-2.0

       Unless required by applicable law or agreed to in writing,
       software distributed under the License is distributed on an
       "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
       KIND, either express or implied.  See the License for the
       specific language governing permissions and limitations
       under the License.
 */

package com.microsoft.fuselabs.dlife;

import java.util.Arrays;

import org.apache.cordova.Config;
import org.apache.cordova.CordovaActivity;
import org.apache.cordova.CordovaPlugin;
import org.apache.cordova.CordovaWebView;
import org.json.JSONException;
import org.json.JSONObject;

import android.annotation.SuppressLint;
import android.app.NotificationManager;
import android.content.Context;
import android.content.Intent;
import android.content.pm.ApplicationInfo;
import android.os.Build;
import android.os.Bundle;
import android.util.Log;

import com.facebook.Session;
import com.microsoft.fuselab.cordova.FuseBroadcastReceiver;
import com.microsoft.fuselab.cordova.PushNotification;
import com.microsoft.fuselabs.cordova.ShareTarget;

public class xim extends CordovaActivity {
	@SuppressLint("NewApi")
	@Override
	public void onCreate(Bundle savedInstanceState) {

		super.onCreate(savedInstanceState);
		super.init();

		// The following does the trick to allow viewport setting
		super.appView.getSettings().setUseWideViewPort(true);
		super.appView.getSettings().setLoadWithOverviewMode(true);
		boolean isDebuggable =( 0 != ( getApplicationInfo().flags & ApplicationInfo.FLAG_DEBUGGABLE ) );
		Log.d("XIM_MAIN", "application isDebuggable: " + String.valueOf(isDebuggable));

		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
			CordovaWebView.setWebContentsDebuggingEnabled(isDebuggable);
		}

		this.handleIntentIfNotification(getIntent(), true);
		
		// Set by <content src="index.html" /> in config.xml
		super.loadUrl(Config.getStartUrl());
	}

	@Override
	public void setActivityResultCallback(CordovaPlugin plugin) {
		this.activityResultCallback = plugin;
	}

	/**
	 * Launch an activity for which you would like a result when it finished.
	 * When this activity exits, your onActivityResult() method will be called.
	 * 
	 * @param command
	 *            The command object
	 * @param intent
	 *            The intent to start
	 * @param requestCode
	 *            The request code that is passed to callback to identify the
	 *            activity
	 */
	public void startActivityForResult(CordovaPlugin command, Intent intent,
			int requestCode) {
		this.activityResultCallback = command;
		this.activityResultKeepRunning = this.keepRunning;

		// If multitasking turned on, then disable it for activities that return
		// results
		if (command != null) {
			this.keepRunning = false;
		}

		// Start activity
		super.startActivityForResult(intent, requestCode);
	}

	@Override
	/**
	 * Called when an activity you launched exits, giving you the requestCode you started it with,
	 * the resultCode it returned, and any additional data from it.
	 *
	 * @param requestCode       The request code originally supplied to startActivityForResult(),
	 *                          allowing you to identify who this result came from.
	 * @param resultCode        The integer result code returned by the child activity through its setResult().
	 * @param data              An Intent, which can return result data to the caller (various data can be attached to Intent "extras").
	 */
	protected void onActivityResult(int requestCode, int resultCode,
			Intent intent) {
		super.onActivityResult(requestCode, resultCode, intent);
		CordovaPlugin callback = this.activityResultCallback;
		if (callback != null) {
			callback.onActivityResult(requestCode, resultCode, intent);
		}
		Session activeSession = Session.getActiveSession();
		if (activeSession != null) {
			activeSession.onActivityResult(this, requestCode, resultCode,
					intent);
		}
	}

    @Override
    /** Caputing incoming intents and sent to appropriate plugin */
    protected void onNewIntent(Intent intent) 
    {
        super.onNewIntent(intent);
 
        if (!this.handleIntentIfNotification(intent, false)) {
			// Give ShareTarget plugin and opporuntity to handle in incoming
			// intent
			ShareTarget shareTarget = new ShareTarget();
			shareTarget.processIncomingIntent(this, intent);
		}
    }

	@Override
	protected void onStop() {
		super.onStop();
		com.flurry.android.FlurryAgent.onEndSession(this);
	}
	
	/** check if a received intent is from push notification, if
	 * so handle the notification and returns true
	 * otherwise, return false */
	private boolean handleIntentIfNotification(Intent intent, boolean isLaunch) {
		boolean result = false;
		Bundle extras = intent.getExtras();
		if (extras != null) {
			boolean isDebuggable =( 0 != ( getApplicationInfo().flags & ApplicationInfo.FLAG_DEBUGGABLE ) );
			
			if (extras.containsKey(FuseBroadcastReceiver.NOTIFICATION_ID_KEY)) {
				NotificationManager notificationManager = (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);
				int notificationID = extras
						.getInt(FuseBroadcastReceiver.NOTIFICATION_ID_KEY);
				Log.d("LauchedFromPush",
						"Launched from notification, notification ID: "
								+ Integer.toString(notificationID));

				if (!isDebuggable) {
					notificationManager.cancel(notificationID);
				}
				result = true;
			}

			if (extras.containsKey(FuseBroadcastReceiver.PUSH_BUNDLE)) {
				Bundle pushBundle = extras
						.getBundle(FuseBroadcastReceiver.PUSH_BUNDLE);
				pushBundle.putBoolean("isLaunch", isLaunch);
				Log.d("ACT", Arrays.toString(extras.keySet().toArray()));

				if (pushBundle != null) {
					Log.d("ACTPUSH",
							Arrays.toString(pushBundle.keySet().toArray()));
					PushNotification.setPushBundle(pushBundle);

					result = true;
				}
			}
		}
		return result;
	}
}
