/// <reference path='../../types/node/node.d.ts' />
/// <reference path='../../types/underscore/underscore.d.ts' />
/// <reference path='../../servers/shared/types.d.ts' />
/// <reference path='../../types/cordova/userprofile.d.ts' />
/**
   Need to override localStorage for windows phone
   On Windows Phone 8, If user invokes our app by double-tap notification on a PIN locked screen,
   the web browser's localStorage is empty (it is as if the localStorage is for a different domain)
   Work around this by stubbing out Storage method and re-route it to our own IsolatedStorage based
   implementation
*/
var client;
(function (client) {
    var appStorage;
    (function (appStorage) {
        var _storage = null;
        /**
            update/insert the key value pair, this will flush the whole storage back async
        */
        function setItem(key, val) {
            var dirty = !_storage.hasOwnProperty(key) || _storage[key] != val;
            _storage[key] = val;
            if (dirty) {
                debouncedFlush();
            }
        }
        appStorage.setItem = setItem;
        /**
            query the storage, this doesn't cross js native boundry
        */
        function getItem(key) {
            return _storage[key];
        }
        appStorage.getItem = getItem;
        /**
            remove the key from store, if there, this will flush the whole storage back async
        */
        function removeItem(key) {
            var dirty = _storage.hasOwnProperty(key);
            delete _storage[key];
            if (dirty) {
                debouncedFlush();
            }
        }
        appStorage.removeItem = removeItem;
        /**
            Load the app storage from native side, this should be part of app start sequence
        */
        function init(callback) {
            if (window.userprofile && window.userprofile.getAppStorage) {
                window.userprofile.getAppStorage(function (obj) {
                    if (_.isString(obj)) {
                        obj = JSON.parse(obj);
                    }
                    _storage = obj;
                    Storage.prototype.setItem = client.appStorage.setItem;
                    Storage.prototype.getItem = client.appStorage.getItem;
                    Storage.prototype.removeItem = client.appStorage.removeItem;
                    if (callback)
                        callback(null, !!window.localStorage);
                }, function (err) {
                    if (callback)
                        callback(new Error(err || 'getAppStorge failed'), false);
                });
            }
            else {
                // if the plugin doesn't support getAppStorage use window.localStorage as is
                if (callback)
                    callback(null, !!window.localStorage);
            }
        }
        appStorage.init = init;
        function flush(callback) {
            if (window.userprofile && window.userprofile.setAppStorage) {
                window.userprofile.setAppStorage(_storage, function () {
                    if (callback)
                        callback(null, true);
                }, function (err) {
                    if (callback)
                        callback(new Error(err || 'setAppStorage failed'), false);
                });
            }
            else {
                if (callback)
                    callback(new Error('no compatible plugin found'), false);
            }
        }
        // only call flush at most once every 200 msec
        var debouncedFlush = _.debounce(flush, 200, false);
    })(appStorage = client.appStorage || (client.appStorage = {}));
})(client || (client = {}));
/// <reference path='../../types/jquery/jquery.d.ts'/>
/// <reference path='../../types/clientJade/clientJade.d.ts'/>
// Implement plugin
(function ($) {
    $.fn.render = function jadeRender(templateName, data) {
        return this.each(function (index, el) {
            jade.render(el, templateName, data);
        });
    };
})(jQuery);
/// <reference path='types.ts'/>
/// <reference path='../../types/clientJade/clientJade.d.ts'/>
var client;
(function (client) {
    /** Common fields supported by most tiles. */
    var TileData = (function () {
        function TileData() {
        }
        TileData.tileClick = function (event) {
            var tile = event.data.tile;
            if (tile.selected) {
                TileData.setSelect(tile, false);
            }
            else {
                TileData.setSelect(tile, true);
            }
            if (tile.clickCallback) {
                tile.clickCallback(tile);
            }
        };
        TileData.setSelect = function (tile, visible) {
            if (tile.selectable) {
                var curTile = $('#' + tile.id);
                if (tile.selected && !visible) {
                    curTile.removeClass('tile-selected');
                    tile.selected = false;
                }
                else if (!tile.selected && visible) {
                    curTile.addClass('tile-selected');
                    tile.selected = true;
                }
            }
        };
        return TileData;
    })();
    client.TileData = TileData;
})(client || (client = {}));
// Implement plugin
(function ($) {
    $.fn.appendTiles = function appendTiles(tiles) {
        return this.each(function (index, el) {
            var $el = $(el);
            for (var i = 0; i < tiles.length; i++) {
                var tile = tiles[i];
                $el.append(jade.templates[tile.view](tile));
                var tileElement = $el.find('#' + tile.id);
                if (tile.selected) {
                    tileElement.addClass('tile-selected');
                }
                tileElement.click({ tile: tile }, function (event) {
                    client.TileData.tileClick(event);
                });
            }
        });
    };
})(jQuery);
/// <reference path='../../types/underscore/underscore.d.ts'/>
var client;
(function (client) {
    function subtract(left, right, test) {
        return _.filter(left, function (leftElement, leftIndex) {
            return _.find(right, function (rightElement, leftIndex) {
                return test ? test(leftElement, rightElement) : (leftElement === rightElement);
            }) === undefined;
        });
    }
    function findMin(list, calc) {
        var delta = Number.MAX_VALUE;
        var cur = null;
        _.each(list, function (item, i) {
            var t = calc(item, i);
            if (t < delta) {
                cur = item;
                delta = t;
            }
        });
        return cur;
    }
    function findMax(list, calc) {
        var delta = Number.MIN_VALUE;
        var cur = null;
        _.each(list, function (item, i) {
            var t = calc(item, i);
            if (t > delta) {
                cur = item;
                delta = t;
            }
        });
        return cur;
    }
    function escapeForJQuery(id) {
        return encodeURIComponent(id).replace(/[%\.!()'"\s+>~]/g, '_');
    }
    var mixins = _.mixin({ subtract: subtract, findMin: findMin, findMax: findMax, escapeForJQuery: escapeForJQuery });
})(client || (client = {}));
/// <reference path='../shared/types.ts'/>
var client;
(function (client) {
    var logger;
    (function (logger) {
        /** Number of entries to log. */
        logger.logLength = 200;
        /** Last n logged messages. */
        logger.log = [];
        /** Logs an error message. */
        function error(errorCode, err) {
            var args = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                args[_i - 2] = arguments[_i];
            }
            var msg = format(err, args);
            _log('ERROR', errorCode + ': ' + msg, []);
            // collecting errors in analytics infrastructure
            client.analytics.logError(errorCode, msg);
        }
        logger.error = error;
        /** Logs a trace message. */
        function info(t) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            _log('INFO', t, args);
        }
        logger.info = info;
        function warn(err) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            _log('WARN', err, args);
        }
        logger.warn = warn;
        function debug(err) {
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments[_i];
            }
            _log('DEBUG', err, args);
        }
        logger.debug = debug;
        /** Starts a stopwatch timer. */
        function start() {
            return new Date().getTime();
        }
        logger.start = start;
        /** Stops a stopwatch timer and optionally logs a message. */
        function stop(start, msg) {
            var args = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                args[_i - 2] = arguments[_i];
            }
            var elapsed = new Date().getTime() - start;
            if (msg)
                _log('INFO', '[' + elapsed + 'ms] ' + msg, args);
            return elapsed;
        }
        logger.stop = stop;
        /** Stops a stopwatch timer after a warning and optionally logs a message. */
        function stopWarn(start, msg) {
            var args = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                args[_i - 2] = arguments[_i];
            }
            var elapsed = new Date().getTime() - start;
            if (msg)
                _log('WARN', '[' + elapsed + 'ms] ' + msg, args);
            return elapsed;
        }
        logger.stopWarn = stopWarn;
        /** Stops a stopwatch timer after an error and optionally logs a message. */
        function stopError(start, errorCode, msg) {
            var args = [];
            for (var _i = 3; _i < arguments.length; _i++) {
                args[_i - 3] = arguments[_i];
            }
            var elapsed = new Date().getTime() - start;
            if (msg)
                _log('ERROR', '[' + elapsed + 'ms] ' + msg, args);
            // collecting errors in analytics infrastructure
            client.analytics.logError(errorCode, msg);
            return elapsed;
        }
        logger.stopError = stopError;
        /** Formats a message to include a dump of its params. */
        function format(t, args) {
            if (args) {
                for (var i = 0; i < args.length; i++) {
                    t += ', ';
                    var v = args[i];
                    switch (typeof v) {
                        case 'object':
                        case 'array':
                            t += JSON.stringify(v);
                            break;
                        default:
                            t += (v === undefined ? 'undefined' : v.toString());
                            break;
                    }
                }
            }
            return t;
        }
        logger.format = format;
        function flattenAndInvert(obj, result) {
            _.each(obj, function (val) {
                if (_.isObject(val)) {
                    flattenAndInvert(val, result);
                }
                else {
                    result[val] = true;
                }
            });
        }
        var tagMap = null;
        /** collect instrumentation data */
        function collect(t, params) {
            if (!tagMap) {
                tagMap = _.extend({}, consts.contentSourceTags);
                if (consts && consts.tags) {
                    flattenAndInvert(consts.tags, tagMap);
                }
                if (consts) {
                    _.each([consts.tabs, consts.commands, consts.debugCommands], function (cmdMap) {
                        if (cmdMap) {
                            _.each(cmdMap, function (cmd) {
                                if (cmd && cmd.id) {
                                    tagMap[cmd.id] = true;
                                }
                            });
                        }
                    });
                }
            }
            // changing to info
            if (tagMap.hasOwnProperty(t)) {
                if (params) {
                    client.logger.info(t, params);
                }
                else {
                    client.logger.info(t);
                }
            }
            else {
                error(consts.errorCode.Analytics, t, params);
            }
        }
        logger.collect = collect;
        function collectNamedEvent(ev, params) {
            if (params) {
                client.analytics.logEvent(ev, params);
            }
            else {
                client.analytics.logEvent(ev);
            }
        }
        logger.collectNamedEvent = collectNamedEvent;
        /** set */
        (function (Levels) {
            Levels[Levels["info"] = 0] = "info";
            Levels[Levels["debug"] = 1] = "debug";
            Levels[Levels["warning"] = 2] = "warning";
            Levels[Levels["error"] = 3] = "error"; // only error
        })(logger.Levels || (logger.Levels = {}));
        var Levels = logger.Levels;
        logger.level = localStorage.getItem('logger-level') ? Levels[localStorage.getItem('logger-level')] : 0 /* info */; // Changed to default info level logging
        // export var level: Levels = Levels.debug; // default to log only error
        var logTypeToLevel = {
            INFO: 0 /* info */,
            DEBUG: 1 /* debug */,
            WARN: 2 /* warning */,
            ERROR: 3 /* error */
        };
        function _log(t, m, a) {
            var logLevel = logTypeToLevel[t];
            if (logLevel >= logger.level) {
                var msg = format(t + '> ' + m, a);
                logger.log.push({ type: t, msg: m, args: a });
                if (logger.log.length > logger.logLength)
                    logger.log.splice(0, 1);
                // to work around missing console.error and console.warn for WP
                if (client.startupMode && client.startupMode == 3 /* wp */) {
                    console.error = console.log;
                    console.warn = console.log;
                }
                if (console) {
                    switch (logLevel) {
                        case 3 /* error */:
                            console.error(msg);
                            break;
                        case 2 /* warning */:
                            console.warn(msg);
                            break;
                        default:
                            console.log(msg);
                            break;
                    }
                }
                // for windows phone, this is how console log gets dumped to debug output
                if (window.external && client.startupMode && window.location.protocol == 'https:') {
                    (window.external).notify(msg);
                }
            }
        }
    })(logger = client.logger || (client.logger = {}));
})(client || (client = {}));
var consts;
(function (consts) {
    /** Shortcuts for specifying time in milliseconds. */
    consts.time = {
        seconds: function (n) {
            return n * 1000;
        },
        minutes: function (n) {
            return n * 60000;
        },
        hours: function (n) {
            return n * 3600000;
        },
        days: function (n) {
            return n * 86400000;
        },
        now: function () {
            return new Date().getTime();
        },
        unixNow: function () {
            return Math.round(new Date().getTime() / 1000);
        },
        age: function (ts) {
            return new Date().getTime() - ts;
        }
    };
    /** Shortcuts for converting distances. */
    consts.distance = {
        kilometers: {
            toMiles: function (km) {
                return km * 0.621371;
            },
            toFeet: function (km) {
                return km * 3280.84;
            }
        },
        miles: {
            toKilometers: function (m) {
                return m * 1.60934;
            },
            toFeet: function (m) {
                return m * 5280;
            }
        }
    };
    /** Client error codes. */
    consts.errorCode = {
        CastViewer: 'CastViewer',
        ContentPicking: 'ContentPicking',
        PhotoCasting: 'PhotoCasting',
        Routing: 'Routing',
        PeoplePicking: 'PeoplePicking',
        CastInit: 'CastInit',
        Contacts: 'Contacts',
        FileDownload: 'FileDownload',
        UnhandledException: 'UnhandledException',
        Scanner: 'Scanner',
        Timeline: 'Timeline',
        Analytics: 'Analytics',
        Multicast: 'Multicast',
        UrlHandling: 'UrlHandling',
        Parsing: 'Parsing',
        Navigation: 'Navigation',
        Initialization: 'Initialization',
        Connection: 'Connection',
        NoMoreBack: 'NoMoreBack',
        ChromeCast: 'ChromeCast',
        Other: 'Other'
    };
    /** Client named events. based on flurry analytics quick spec */
    consts.flurryEvent = {
        castViewerAdd: 'castViewerAdd',
        castViewerFAQ: 'castViewerFAQ',
        cogCredits: 'cogCredits',
        cogFaq: 'cogFaq',
        cogFeedback: 'cogFeedback',
        cogPrivacy: 'cogPrivacy',
        cogProfile: 'cogProfile',
        cogSettings: 'cogSettings',
        cogTerms: 'cogTerms',
        contactsAccessDenied: 'contactsAccessDenied',
        ftuxDisplay: 'ftuxDisplay',
        ftuxLogin: 'ftuxLogin',
        pickpeopleChooseInviteMethod: 'pickpeopleChooseInviteMethod',
        pickpeopleLongTapPerson: 'pickpeopleLongTapPerson',
        pickpeoplePreviousPeople: 'pickpeoplePreviousPeople',
        pickpeopleTapPerson: 'pickpeopleTapPerson',
        picksourceBack: 'picksourceBack',
        picksourceType: 'picksourceType',
        picksourceNext: 'picksourceNext',
        pickerAllButton: 'pickerAllButton',
        profileAutoFill: 'profileAutoFill',
        profileBack: 'profileBack',
        profileUpdate: 'profileUpdate',
        registrationPhoneBack: 'registrationPhoneBack',
        registrationPhoneNext: 'registrationPhoneNext',
        registrationPrivacy: 'registrationPrivacy',
        registrationTerms: 'registrationTerms',
        registrationVerificationBack: 'registrationVerificationBack',
        registrationVerificationNext: 'registrationVerificationNext',
        registrationVerificationNextResult: 'registrationVerificationNextResult',
        registrationVerificationResend: 'registrationVerificationResend',
        registrationVerificationSuccess: 'registrationVerificationSuccess',
        registrationProfileDisplay: 'registrationProfileDisplay',
        registrationProfileAutoFill: 'registrationProfileAutoFill',
        registrationProfileCreate: 'registrationProfileCreate',
        settingsDeauthorize: 'settingsDeauthorize',
        settingsDeauthorizeConfirm: 'settingsDeauthorizeConfirm',
        settingsDelete: 'settingsDelete',
        settingsDeleteConfirm: 'settingsDeleteConfirm',
        ximCog: 'ximCog',
        ximCreateNew: 'ximCreateNew',
        ximCreateNewTimed: 'ximCreateNewTimed',
        ximPreviousXim: 'ximPreviousXim',
        ximStartXim: 'ximStartXim',
        ximsessionAddpeople: 'ximsessionAddpeople',
        ximsessionAddPicture: 'ximsessionAddPicture',
        ximsessionAddComment: 'ximsessionAddComment',
        ximsessionSaveImage: 'ximsessionSaveImage',
        ximsessionSendMessage: 'ximsessionSendMessage',
        ximsessionSlide: 'ximsessionSlide',
        ximsessionUpsell: 'ximsessionUpsell',
        ximsessionZoomIn: 'ximsessionZoomIn',
        ximsessionZoomOut: 'ximsessionZoomOut'
    };
})(consts || (consts = {}));
var consts;
(function (consts) {
    /** geographic regions bigger than country*/
    (function (GeoRegions) {
        /** North and south america also the default*/
        GeoRegions[GeoRegions["america"] = 1] = "america";
        /** Asia pacific */
        GeoRegions[GeoRegions["pacific"] = 2] = "pacific";
        /** Europe Middle-East & Africa */
        GeoRegions[GeoRegions["emea"] = 3] = "emea";
    })(consts.GeoRegions || (consts.GeoRegions = {}));
    var GeoRegions = consts.GeoRegions;
    ;
    ;
    /** country code and phone validation data */
    consts.iso3166_data = [
        {
            alpha2: "US",
            alpha3: "USA",
            country_code: "1",
            country_name: "United States",
            twilio: true,
            twilioVoice: true,
            area_codes: ["205", "251", "256", "334", "659", "938", "907", "480", "520", "602", "623", "928", "327", "479", "501", "870", "209", "213", "310", "323", "341", "369", "408", "415", "424", "442", "510", "530", "559", "562", "619", "626", "627", "628", "650", "657", "661", "669", "707", "714", "747", "760", "764", "805", "818", "831", "858", "909", "916", "925", "935", "949", "951", "303", "719", "720", "970", "203", "475", "860", "959", "302", "202", "239", "305", "321", "352", "386", "407", "561", "689", "727", "754", "772", "786", "813", "850", "863", "904", "927", "941", "954", "229", "404", "470", "478", "678", "706", "762", "770", "912", "808", "208", "217", "224", "309", "312", "331", "447", "464", "618", "630", "708", "730", "773", "779", "815", "847", "872", "219", "260", "317", "574", "765", "812", "930", "319", "515", "563", "641", "712", "316", "620", "785", "913", "270", "364", "502", "606", "859", "225", "318", "337", "504", "985", "207", "227", "240", "301", "410", "443", "667", "339", "351", "413", "508", "617", "774", "781", "857", "978", "231", "248", "269", "278", "313", "517", "586", "616", "679", "734", "810", "906", "947", "989", "218", "320", "507", "612", "651", "763", "952", "228", "601", "662", "769", "314", "417", "557", "573", "636", "660", "816", "975", "406", "308", "402", "531", "702", "725", "775", "603", "201", "551", "609", "732", "848", "856", "862", "908", "973", "505", "575", "212", "315", "347", "516", "518", "585", "607", "631", "646", "716", "718", "845", "914", "917", "929", "252", "336", "704", "828", "910", "919", "980", "984", "701", "216", "220", "234", "283", "330", "380", "419", "440", "513", "567", "614", "740", "937", "405", "539", "580", "918", "458", "503", "541", "971", "215", "267", "272", "412", "484", "570", "582", "610", "717", "724", "814", "878", "401", "803", "843", "854", "864", "605", "423", "615", "629", "731", "865", "901", "931", "210", "214", "254", "281", "325", "346", "361", "409", "430", "432", "469", "512", "682", "713", "737", "806", "817", "830", "832", "903", "915", "936", "940", "956", "972", "979", "385", "435", "801", "802", "276", "434", "540", "571", "703", "757", "804", "206", "253", "360", "425", "509", "564", "304", "681", "262", "274", "414", "534", "608", "715", "920", "307"],
            min_number_length: 7,
            phone_number_lengths: [10],
            region: 1 /* america */
        },
        {
            alpha2: "CA",
            alpha3: "CAN",
            country_code: "1",
            country_name: "Canada",
            twilio: true,
            area_codes: ["403", "587", "780", "825", "236", "250", "604", "672", "778", "204", "431", "506", "709", "782", "902", "548", "249", "289", "343", "365", "387", "416", "437", "519", "226", "613", "647", "705", "742", "807", "905", "782", "902", "418", "438", "450", "514", "579", "581", "819", "873", "306", "639", "867"],
            min_number_length: 7,
            phone_number_lengths: [10],
            region: 1 /* america */
        },
        { alpha2: "AI", alpha3: "AIA", country_code: "1", country_name: "Anguilla", twilio: true, area_codes: ["264"], min_number_length: 7, phone_number_lengths: [10], region: 1 /* america */ },
        { alpha2: "AS", alpha3: "ASM", country_code: "1", country_name: "American Samoa", twilio: true, area_codes: ["684"], min_number_length: 7, phone_number_lengths: [10], region: 1 /* america */ },
        { alpha2: "AG", alpha3: "ATG", country_code: "1", country_name: "Antigua and Barbuda", twilio: true, area_codes: ["268"], min_number_length: 7, phone_number_lengths: [10], region: 1 /* america */ },
        { alpha2: "BS", alpha3: "BHS", country_code: "1", country_name: "Bahamas, The", twilio: true, area_codes: ["242"], min_number_length: 7, phone_number_lengths: [10], region: 1 /* america */ },
        { alpha2: "BB", alpha3: "BRB", country_code: "1", country_name: "Barbados", twilio: true, area_codes: ["246"], min_number_length: 7, phone_number_lengths: [10], region: 1 /* america */ },
        { alpha2: "BM", alpha3: "BMU", country_code: "1", country_name: "Bermuda", twilio: true, area_codes: ["441"], min_number_length: 7, phone_number_lengths: [10], region: 1 /* america */ },
        { alpha2: "KY", alpha3: "CYM", country_code: "1", country_name: "Cayman Islands", twilio: true, area_codes: ["345"], min_number_length: 7, phone_number_lengths: [10], region: 1 /* america */ },
        { alpha2: "DM", alpha3: "DMA", country_code: "1", country_name: "Dominica", twilio: true, area_codes: ["767"], min_number_length: 7, phone_number_lengths: [10], region: 1 /* america */ },
        { alpha2: "DO", alpha3: "DOM", country_code: "1", country_name: "Dominican Republic", twilio: true, twilioVoice: true, area_codes: ["809", "829", "849"], min_number_length: 7, phone_number_lengths: [10], region: 1 /* america */ },
        { alpha2: "GD", alpha3: "GRD", country_code: "1", country_name: "Grenada", twilio: true, area_codes: ["473"], min_number_length: 7, phone_number_lengths: [10], region: 1 /* america */ },
        { alpha2: "GU", alpha3: "GUM", country_code: "1", country_name: "Guam", twilio: true, area_codes: ["671"], min_number_length: 7, phone_number_lengths: [10], region: 2 /* pacific */ },
        { alpha2: "JM", alpha3: "JAM", country_code: "1", country_name: "Jamaica", twilio: true, area_codes: ["876"], min_number_length: 7, phone_number_lengths: [10], region: 1 /* america */ },
        { alpha2: "KN", alpha3: "KNA", country_code: "1", country_name: "Saint Kitts and Nevis", twilio: true, area_codes: ["869"], min_number_length: 7, phone_number_lengths: [10], region: 1 /* america */ },
        { alpha2: "LC", alpha3: "LCA", country_code: "1", country_name: "Saint Lucia", twilio: true, area_codes: ["758"], min_number_length: 7, phone_number_lengths: [10], region: 1 /* america */ },
        { alpha2: "MP", alpha3: "MNP", country_code: "1", country_name: "Northern Mariana Islands", twilio: true, area_codes: ["670"], min_number_length: 7, phone_number_lengths: [10], region: 2 /* pacific */ },
        { alpha2: "MS", alpha3: "MSR", country_code: "1", country_name: "Montserrat", twilio: true, area_codes: ["664"], min_number_length: 7, phone_number_lengths: [10], region: 1 /* america */ },
        { alpha2: "PR", alpha3: "PRI", country_code: "1", country_name: "Puerto Rico", twilio: true, twilioVoice: true, area_codes: ["787", "939"], min_number_length: 7, phone_number_lengths: [10], region: 1 /* america */ },
        { alpha2: "TC", alpha3: "TCA", country_code: "1", country_name: "Turks and Caicos Islands", twilio: true, area_codes: ["649"], min_number_length: 7, phone_number_lengths: [10], region: 1 /* america */ },
        { alpha2: "TT", alpha3: "TTO", country_code: "1", country_name: "Trinidad and Tobago", twilio: true, area_codes: ["868"], min_number_length: 7, phone_number_lengths: [10], region: 1 /* america */ },
        { alpha2: "VC", alpha3: "VCT", country_code: "1", country_name: "Saint Vincent and the Grenadines", twilio: true, area_codes: ["784"], min_number_length: 7, phone_number_lengths: [10], region: 1 /* america */ },
        { alpha2: "VG", alpha3: "VGB", country_code: "1", country_name: "British Virgin Islands", twilio: true, area_codes: ["284"], min_number_length: 7, phone_number_lengths: [10], region: 1 /* america */ },
        { alpha2: "VI", alpha3: "VIR", country_code: "1", country_name: "US Virgin Islands", twilio: true, area_codes: ["340"], min_number_length: 7, phone_number_lengths: [10], region: 1 /* america */ },
        { alpha2: "SX", alpha3: "SXM", country_code: "1", country_name: "Sint Maarten", twilio: true, area_codes: ["721"], min_number_length: 7, phone_number_lengths: [10], region: 1 /* america */ },
        { alpha2: "AW", alpha3: "ABW", country_code: "297", country_name: "Aruba", twilio: true, mobile_begin_with: ["5", "6", "7", "9"], min_number_length: 7, phone_number_lengths: [7], region: 1 /* america */ },
        { alpha2: "AF", alpha3: "AFG", country_code: "93", country_name: "Afghanistan", twilio: true, mobile_begin_with: ["7"], min_number_length: 9, phone_number_lengths: [9], region: 2 /* pacific */ },
        { alpha2: "AO", alpha3: "AGO", country_code: "244", country_name: "Angola", twilio: true, mobile_begin_with: ["9"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "AX", alpha3: "ALA", country_code: "358", country_name: "Åland Islands", twilio: true, mobile_begin_with: ["18"], min_number_length: 6, phone_number_lengths: [6, 7, 8], region: 3 /* emea */ },
        { alpha2: "AL", alpha3: "ALB", country_code: "355", country_name: "Albania", mobile_begin_with: ["6"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "AD", alpha3: "AND", country_code: "376", country_name: "Andorra", mobile_begin_with: ["3", "4", "6"], min_number_length: 6, phone_number_lengths: [6], region: 3 /* emea */ },
        { alpha2: "AE", alpha3: "ARE", country_code: "971", country_name: "United Arab Emirates", twilio: true, mobile_begin_with: ["5"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "AR", alpha3: "ARG", country_code: "54", country_name: "Argentina", twilio: true, mobile_begin_with: [], min_number_length: 10, phone_number_lengths: [10], region: 1 /* america */ },
        { alpha2: "AM", alpha3: "ARM", country_code: "374", country_name: "Armenia", twilio: true, mobile_begin_with: ["5", "7", "9"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "AU", alpha3: "AUS", country_code: "61", country_name: "Australia", twilio: true, twilioVoice: true, mobile_begin_with: ["4"], min_number_length: 9, phone_number_lengths: [9], region: 2 /* pacific */ },
        { alpha2: "AT", alpha3: "AUT", country_code: "43", country_name: "Austria", twilio: true, twilioVoice: true, mobile_begin_with: ["6"], min_number_length: 10, phone_number_lengths: [10], region: 3 /* emea */ },
        { alpha2: "AZ", alpha3: "AZE", country_code: "994", country_name: "Azerbaijan", twilio: true, mobile_begin_with: ["4", "5", "6", "7"], min_number_length: 9, phone_number_lengths: [9], region: 2 /* pacific */ },
        { alpha2: "BI", alpha3: "BDI", country_code: "257", country_name: "Burundi", twilio: true, mobile_begin_with: ["7", "29"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "BE", alpha3: "BEL", country_code: "32", country_name: "Belgium", twilio: true, twilioVoice: true, mobile_begin_with: ["4"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "BJ", alpha3: "BEN", country_code: "229", country_name: "Benin", twilio: true, mobile_begin_with: ["9"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "BF", alpha3: "BFA", country_code: "226", country_name: "Burkina Faso", twilio: true, mobile_begin_with: ["6", "7"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "BD", alpha3: "BGD", country_code: "880", country_name: "Bangladesh", twilio: true, mobile_begin_with: ["1"], min_number_length: 8, phone_number_lengths: [8, 9, 10], region: 2 /* pacific */ },
        { alpha2: "BG", alpha3: "BGR", country_code: "359", country_name: "Bulgaria", twilio: true, twilioVoice: true, mobile_begin_with: ["87", "88", "89", "98", "99", "43"], min_number_length: 8, phone_number_lengths: [8, 9], region: 3 /* emea */ },
        { alpha2: "BH", alpha3: "BHR", country_code: "973", country_name: "Bahrain", twilio: true, twilioVoice: true, mobile_begin_with: ["3"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "BA", alpha3: "BIH", country_code: "387", country_name: "Bosnia and Herzegovina", twilio: true, mobile_begin_with: ["6"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "BY", alpha3: "BLR", country_code: "375", country_name: "Belarus", twilio: true, mobile_begin_with: ["25", "29", "33", "44"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "BZ", alpha3: "BLZ", country_code: "501", country_name: "Belize", twilio: true, mobile_begin_with: ["6"], min_number_length: 7, phone_number_lengths: [7], region: 1 /* america */ },
        { alpha2: "BO", alpha3: "BOL", country_code: "591", country_name: "Bolivia", twilio: true, mobile_begin_with: ["7"], min_number_length: 8, phone_number_lengths: [8], region: 1 /* america */ },
        {
            alpha2: "BR",
            alpha3: "BRA",
            country_code: "55",
            country_name: "Brazil",
            twilio: true,
            twilioVoice: true,
            mobile_begin_with: [
                "119",
                "129",
                "139",
                "149",
                "159",
                "169",
                "179",
                "189",
                "199",
                "219",
                "229",
                "249",
                "279",
                "289",
                "31",
                "32",
                "34",
                "38",
                "41",
                "43",
                "44",
                "45",
                "47",
                "48",
                "51",
                "53",
                "54",
                "55",
                "61",
                "62",
                "65",
                "67",
                "68",
                "69",
                "71",
                "73",
                "75",
                "77",
                "79",
                "81",
                "82",
                "83",
                "84",
                "85",
                "86",
                "91",
                "92",
                "95",
                "96",
                "98"
            ],
            min_number_length: 10,
            phone_number_lengths: [10, 11],
            region: 1 /* america */
        },
        { alpha2: "BN", alpha3: "BRN", country_code: "673", country_name: "Brunei", twilio: true, mobile_begin_with: ["7", "8"], min_number_length: 7, phone_number_lengths: [7], region: 2 /* pacific */ },
        { alpha2: "BT", alpha3: "BTN", country_code: "975", country_name: "Bhutan", mobile_begin_with: ["17"], min_number_length: 8, phone_number_lengths: [8], region: 2 /* pacific */ },
        { alpha2: "BW", alpha3: "BWA", country_code: "267", country_name: "Botswana", twilio: true, mobile_begin_with: ["71", "72", "73", "74", "75", "76"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "CF", alpha3: "CAF", country_code: "236", country_name: "Central African Republic", twilio: true, mobile_begin_with: ["7"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "CH", alpha3: "CHE", country_code: "41", country_name: "Switzerland", twilio: true, twilioVoice: true, mobile_begin_with: ["7"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "CL", alpha3: "CHL", country_code: "56", country_name: "Chile", twilio: true, mobile_begin_with: ["9"], min_number_length: 9, phone_number_lengths: [9], region: 1 /* america */ },
        { alpha2: "CN", alpha3: "CHN", country_code: "86", country_name: "China", twilio: true, twilioVoice: true, mobile_begin_with: ["13", "14", "15", "18"], min_number_length: 11, phone_number_lengths: [11], region: 2 /* pacific */ },
        { alpha2: "CI", alpha3: "CIV", country_code: "225", country_name: "Côte d'Ivoire", twilio: true, mobile_begin_with: ["0", "4", "5", "6"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "CM", alpha3: "CMR", country_code: "237", country_name: "Cameroon", twilio: true, mobile_begin_with: ["7", "9"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "CD", alpha3: "COD", country_code: "243", country_name: "Congo (DRC)", mobile_begin_with: ["8", "9"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "CG", alpha3: "COG", country_code: "242", country_name: "Congo", twilio: true, mobile_begin_with: ["0"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "CK", alpha3: "COK", country_code: "682", country_name: "Cook Islands", mobile_begin_with: ["5", "7"], min_number_length: 5, phone_number_lengths: [5], region: 2 /* pacific */ },
        { alpha2: "CO", alpha3: "COL", country_code: "57", country_name: "Colombia", twilio: true, mobile_begin_with: ["3"], min_number_length: 10, phone_number_lengths: [10], region: 1 /* america */ },
        { alpha2: "KM", alpha3: "COM", country_code: "269", country_name: "Comoros", twilio: true, mobile_begin_with: ["3", "76"], min_number_length: 7, phone_number_lengths: [7], region: 3 /* emea */ },
        { alpha2: "CV", alpha3: "CPV", country_code: "238", country_name: "Cabo Verde", twilio: true, mobile_begin_with: ["5", "9"], min_number_length: 7, phone_number_lengths: [7], region: 3 /* emea */ },
        { alpha2: "CR", alpha3: "CRI", country_code: "506", country_name: "Costa Rica", twilio: true, mobile_begin_with: ["5", "6", "7", "8"], min_number_length: 8, phone_number_lengths: [8], region: 1 /* america */ },
        { alpha2: "CY", alpha3: "CYP", country_code: "357", country_name: "Cyprus", twilio: true, twilioVoice: true, mobile_begin_with: ["9"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "CZ", alpha3: "CZE", country_code: "420", country_name: "Czech Republic", twilio: true, twilioVoice: true, mobile_begin_with: ["6", "7"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "DE", alpha3: "DEU", country_code: "49", country_name: "Germany", twilio: true, twilioVoice: true, mobile_begin_with: ["15", "16", "17"], min_number_length: 10, phone_number_lengths: [10, 11], region: 3 /* emea */ },
        { alpha2: "DJ", alpha3: "DJI", country_code: "253", country_name: "Djibouti", twilio: true, mobile_begin_with: ["77"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "DK", alpha3: "DNK", country_code: "45", country_name: "Denmark", twilio: true, twilioVoice: true, mobile_begin_with: [], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "DZ", alpha3: "DZA", country_code: "213", country_name: "Algeria", twilio: true, mobile_begin_with: ["5", "6", "7"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "EC", alpha3: "ECU", country_code: "593", country_name: "Ecuador", mobile_begin_with: ["9"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "EG", alpha3: "EGY", country_code: "20", country_name: "Egypt", twilio: true, mobile_begin_with: ["1"], min_number_length: 10, phone_number_lengths: [10], region: 3 /* emea */ },
        { alpha2: "ER", alpha3: "ERI", country_code: "291", country_name: "Eritrea", mobile_begin_with: ["1", "7", "8"], min_number_length: 7, phone_number_lengths: [7], region: 3 /* emea */ },
        { alpha2: "ES", alpha3: "ESP", country_code: "34", country_name: "Spain", twilio: true, twilioVoice: true, mobile_begin_with: ["6", "7"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "EE", alpha3: "EST", country_code: "372", country_name: "Estonia", twilio: true, twilioVoice: true, mobile_begin_with: ["5"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "ET", alpha3: "ETH", country_code: "251", country_name: "Ethiopia", twilio: true, mobile_begin_with: ["9"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "FI", alpha3: "FIN", country_code: "358", country_name: "Finland", twilio: true, twilioVoice: true, mobile_begin_with: ["4", "5"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "FJ", alpha3: "FJI", country_code: "679", country_name: "Fiji", twilio: true, mobile_begin_with: ["7", "9"], min_number_length: 7, phone_number_lengths: [7], region: 2 /* pacific */ },
        { alpha2: "FK", alpha3: "FLK", country_code: "500", country_name: "Falkland Islands", twilio: true, mobile_begin_with: ["5", "6"], min_number_length: 5, phone_number_lengths: [5], region: 1 /* america */ },
        { alpha2: "FR", alpha3: "FRA", country_code: "33", country_name: "France", twilio: true, twilioVoice: true, mobile_begin_with: ["6", "7"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "FO", alpha3: "FRO", country_code: "298", country_name: "Faroe Islands", twilio: true, mobile_begin_with: [], min_number_length: 6, phone_number_lengths: [6], region: 3 /* emea */ },
        { alpha2: "FM", alpha3: "FSM", country_code: "691", country_name: "Micronesia", mobile_begin_with: [], min_number_length: 7, phone_number_lengths: [7], region: 2 /* pacific */ },
        { alpha2: "GA", alpha3: "GAB", country_code: "241", country_name: "Gabon", twilio: true, mobile_begin_with: ["05", "06", "07"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "GB", alpha3: "GBR", country_code: "44", country_name: "United Kingdom", twilio: true, twilioVoice: true, mobile_begin_with: ["7"], min_number_length: 10, phone_number_lengths: [10], region: 3 /* emea */ },
        { alpha2: "GE", alpha3: "GEO", country_code: "995", country_name: "Georgia", twilio: true, mobile_begin_with: ["5", "7"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "GH", alpha3: "GHA", country_code: "233", country_name: "Ghana", twilio: true, mobile_begin_with: ["2", "5"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "GI", alpha3: "GIB", country_code: "350", country_name: "Gibraltar", twilio: true, mobile_begin_with: ["5"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "GN", alpha3: "GIN", country_code: "224", country_name: "Guinea", twilio: true, mobile_begin_with: ["6"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "GP", alpha3: "GLP", country_code: "590", country_name: "Guadeloupe", twilio: true, mobile_begin_with: ["690"], min_number_length: 9, phone_number_lengths: [9], region: 1 /* america */ },
        { alpha2: "GM", alpha3: "GMB", country_code: "220", country_name: "Gambia, The", twilio: true, mobile_begin_with: ["7", "9"], min_number_length: 7, phone_number_lengths: [7], region: 3 /* emea */ },
        { alpha2: "GW", alpha3: "GNB", country_code: "245", country_name: "Guinea-Bissau", mobile_begin_with: ["5", "6", "7"], min_number_length: 7, phone_number_lengths: [7], region: 3 /* emea */ },
        { alpha2: "GQ", alpha3: "GNQ", country_code: "240", country_name: "Equatorial Guinea", twilio: true, mobile_begin_with: ["222", "551"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "GR", alpha3: "GRC", country_code: "30", country_name: "Greece", twilio: true, twilioVoice: true, mobile_begin_with: ["6"], min_number_length: 10, phone_number_lengths: [10], region: 3 /* emea */ },
        { alpha2: "GL", alpha3: "GRL", country_code: "299", country_name: "Greenland", twilio: true, mobile_begin_with: ["4", "5"], min_number_length: 6, phone_number_lengths: [6], region: 3 /* emea */ },
        { alpha2: "GT", alpha3: "GTM", country_code: "502", country_name: "Guatemala", twilio: true, mobile_begin_with: ["3", "4", "5"], min_number_length: 8, phone_number_lengths: [8], region: 1 /* america */ },
        { alpha2: "GF", alpha3: "GUF", country_code: "594", country_name: "French Guiana", twilio: true, mobile_begin_with: ["694"], min_number_length: 9, phone_number_lengths: [9], region: 1 /* america */ },
        { alpha2: "GY", alpha3: "GUY", country_code: "592", country_name: "Guyana", twilio: true, mobile_begin_with: ["6"], min_number_length: 7, phone_number_lengths: [7], region: 1 /* america */ },
        { alpha2: "HK", alpha3: "HKG", country_code: "852", country_name: "Hong Kong SAR", twilio: true, twilioVoice: true, mobile_begin_with: ["5", "6", "9"], min_number_length: 8, phone_number_lengths: [8], region: 2 /* pacific */ },
        { alpha2: "HN", alpha3: "HND", country_code: "504", country_name: "Honduras", twilio: true, mobile_begin_with: ["3", "7", "8", "9"], min_number_length: 8, phone_number_lengths: [8], region: 1 /* america */ },
        { alpha2: "HR", alpha3: "HRV", country_code: "385", country_name: "Croatia", twilio: true, mobile_begin_with: ["9"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "HT", alpha3: "HTI", country_code: "509", country_name: "Haiti", twilio: true, mobile_begin_with: ["3", "4"], min_number_length: 8, phone_number_lengths: [8], region: 1 /* america */ },
        { alpha2: "HU", alpha3: "HUN", country_code: "36", country_name: "Hungary", twilio: true, mobile_begin_with: ["20", "30", "31", "70"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "ID", alpha3: "IDN", country_code: "62", country_name: "Indonesia", twilio: true, mobile_begin_with: ["8"], min_number_length: 9, phone_number_lengths: [9, 10, 11], region: 2 /* pacific */ },
        { alpha2: "IN", alpha3: "IND", country_code: "91", country_name: "India", twilio: true, twilioVoice: true, mobile_begin_with: ["7", "8", "9"], min_number_length: 10, phone_number_lengths: [10], region: 2 /* pacific */ },
        { alpha2: "IE", alpha3: "IRL", country_code: "353", country_name: "Ireland", twilio: true, twilioVoice: true, mobile_begin_with: ["82", "83", "84", "85", "86", "87", "88", "89"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "IR", alpha3: "IRN", country_code: "98", country_name: "Iran", twilio: true, mobile_begin_with: ["9"], min_number_length: 10, phone_number_lengths: [10], region: 3 /* emea */ },
        { alpha2: "IQ", alpha3: "IRQ", country_code: "964", country_name: "Iraq", twilio: true, mobile_begin_with: ["7"], min_number_length: 10, phone_number_lengths: [10], region: 3 /* emea */ },
        { alpha2: "IS", alpha3: "ISL", country_code: "354", country_name: "Iceland", twilio: true, mobile_begin_with: ["6", "7", "8"], min_number_length: 7, phone_number_lengths: [7], region: 3 /* emea */ },
        { alpha2: "IL", alpha3: "ISR", country_code: "972", country_name: "Israel", twilio: true, twilioVoice: true, mobile_begin_with: ["5"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "IT", alpha3: "ITA", country_code: "39", country_name: "Italy", twilio: true, twilioVoice: true, mobile_begin_with: ["3"], min_number_length: 10, phone_number_lengths: [10], region: 3 /* emea */ },
        { alpha2: "JO", alpha3: "JOR", country_code: "962", country_name: "Jordan", twilio: true, mobile_begin_with: ["7"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "JP", alpha3: "JPN", country_code: "81", country_name: "Japan", twilio: true, twilioVoice: true, mobile_begin_with: ["70", "80", "90"], min_number_length: 10, phone_number_lengths: [10], region: 2 /* pacific */ },
        { alpha2: "KZ", alpha3: "KAZ", country_code: "7", country_name: "Kazakhstan", twilio: true, mobile_begin_with: ["70", "77"], min_number_length: 10, phone_number_lengths: [10], region: 3 /* emea */ },
        { alpha2: "KE", alpha3: "KEN", country_code: "254", country_name: "Kenya", twilio: true, mobile_begin_with: ["7"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "KG", alpha3: "KGZ", country_code: "996", country_name: "Kyrgyzstan", twilio: true, mobile_begin_with: ["5", "7"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "KH", alpha3: "KHM", country_code: "855", country_name: "Cambodia", twilio: true, mobile_begin_with: ["1", "6", "7", "8", "9"], min_number_length: 8, phone_number_lengths: [8, 9], region: 2 /* pacific */ },
        { alpha2: "KI", alpha3: "KIR", country_code: "686", country_name: "Kiribati", mobile_begin_with: ["9", "30"], min_number_length: 5, phone_number_lengths: [5], region: 2 /* pacific */ },
        { alpha2: "KR", alpha3: "KOR", country_code: "82", country_name: "Korea", twilio: true, mobile_begin_with: ["1"], min_number_length: 9, phone_number_lengths: [9, 10], region: 2 /* pacific */ },
        { alpha2: "KW", alpha3: "KWT", country_code: "965", country_name: "Kuwait", twilio: true, mobile_begin_with: ["5", "6", "9"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "LA", alpha3: "LAO", country_code: "856", country_name: "Laos", twilio: true, mobile_begin_with: ["20"], min_number_length: 10, phone_number_lengths: [10], region: 2 /* pacific */ },
        { alpha2: "LB", alpha3: "LBN", country_code: "961", country_name: "Lebanon", twilio: true, mobile_begin_with: ["3", "7"], min_number_length: 7, phone_number_lengths: [7, 8], region: 3 /* emea */ },
        { alpha2: "LR", alpha3: "LBR", country_code: "231", country_name: "Liberia", twilio: true, mobile_begin_with: ["4", "5", "6", "7"], min_number_length: 7, phone_number_lengths: [7, 8], region: 3 /* emea */ },
        { alpha2: "LY", alpha3: "LBY", country_code: "218", country_name: "Libya", twilio: true, mobile_begin_with: ["9"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "LI", alpha3: "LIE", country_code: "423", country_name: "Liechtenstein", twilio: true, mobile_begin_with: ["7"], min_number_length: 7, phone_number_lengths: [7], region: 3 /* emea */ },
        { alpha2: "LK", alpha3: "LKA", country_code: "94", country_name: "Sri Lanka", twilio: true, mobile_begin_with: ["7"], min_number_length: 9, phone_number_lengths: [9], region: 2 /* pacific */ },
        { alpha2: "LS", alpha3: "LSO", country_code: "266", country_name: "Lesotho", twilio: true, mobile_begin_with: ["5", "6"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "LT", alpha3: "LTU", country_code: "370", country_name: "Lithuania", twilio: true, twilioVoice: true, mobile_begin_with: ["6"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "LU", alpha3: "LUX", country_code: "352", country_name: "Luxembourg", twilio: true, twilioVoice: true, mobile_begin_with: ["6"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "LV", alpha3: "LVA", country_code: "371", country_name: "Latvia", twilio: true, twilioVoice: true, mobile_begin_with: ["2"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "MO", alpha3: "MAC", country_code: "853", country_name: "Macao SAR", twilio: true, mobile_begin_with: ["6"], min_number_length: 8, phone_number_lengths: [8], region: 2 /* pacific */ },
        { alpha2: "MA", alpha3: "MAR", country_code: "212", country_name: "Morocco", twilio: true, mobile_begin_with: ["6"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "MC", alpha3: "MCO", country_code: "377", country_name: "Monaco", twilio: true, mobile_begin_with: ["4", "6"], min_number_length: 8, phone_number_lengths: [8, 9], region: 3 /* emea */ },
        { alpha2: "MD", alpha3: "MDA", country_code: "373", country_name: "Moldova", mobile_begin_with: ["6", "7"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "MG", alpha3: "MDG", country_code: "261", country_name: "Madagascar", twilio: true, mobile_begin_with: ["3"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "MV", alpha3: "MDV", country_code: "960", country_name: "Maldives", twilio: true, mobile_begin_with: ["7", "9"], min_number_length: 7, phone_number_lengths: [7], region: 2 /* pacific */ },
        { alpha2: "MX", alpha3: "MEX", country_code: "52", country_name: "Mexico", twilio: true, twilioVoice: true, mobile_begin_with: [""], min_number_length: 10, phone_number_lengths: [10, 11], region: 1 /* america */ },
        { alpha2: "MH", alpha3: "MHL", country_code: "692", country_name: "Marshall Islands", mobile_begin_with: [], min_number_length: 7, phone_number_lengths: [7], region: 2 /* pacific */ },
        { alpha2: "MK", alpha3: "MKD", country_code: "389", country_name: "Macedonia, FYRO", twilio: true, mobile_begin_with: ["7"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "ML", alpha3: "MLI", country_code: "223", country_name: "Mali", twilio: true, mobile_begin_with: ["6", "7"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "MT", alpha3: "MLT", country_code: "356", country_name: "Malta", twilio: true, twilioVoice: true, mobile_begin_with: ["79", "99"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "MM", alpha3: "MMR", country_code: "95", country_name: "Myanmar", mobile_begin_with: ["9"], min_number_length: 8, phone_number_lengths: [8], region: 2 /* pacific */ },
        { alpha2: "ME", alpha3: "MNE", country_code: "382", country_name: "Montenegro", twilio: true, mobile_begin_with: ["6"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "MN", alpha3: "MNG", country_code: "976", country_name: "Mongolia", twilio: true, mobile_begin_with: ["5", "8", "9"], min_number_length: 8, phone_number_lengths: [8], region: 2 /* pacific */ },
        { alpha2: "MZ", alpha3: "MOZ", country_code: "258", country_name: "Mozambique", twilio: true, mobile_begin_with: ["8"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "MR", alpha3: "MRT", country_code: "222", country_name: "Mauritania", twilio: true, mobile_begin_with: [], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "MQ", alpha3: "MTQ", country_code: "596", country_name: "Martinique", twilio: true, mobile_begin_with: ["696"], min_number_length: 9, phone_number_lengths: [9], region: 1 /* america */ },
        { alpha2: "MU", alpha3: "MUS", country_code: "230", country_name: "Mauritius", twilio: true, mobile_begin_with: [], min_number_length: 7, phone_number_lengths: [7], region: 3 /* emea */ },
        { alpha2: "MW", alpha3: "MWI", country_code: "265", country_name: "Malawi", twilio: true, mobile_begin_with: ["77", "88", "99"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "MY", alpha3: "MYS", country_code: "60", country_name: "Malaysia", twilio: true, mobile_begin_with: ["1"], min_number_length: 9, phone_number_lengths: [9, 10], region: 2 /* pacific */ },
        { alpha2: "YT", alpha3: "MYT", country_code: "269", country_name: "Mayotte", mobile_begin_with: ["639"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "NA", alpha3: "NAM", country_code: "264", country_name: "Namibia", twilio: true, mobile_begin_with: ["60", "81", "82", "85"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "NC", alpha3: "NCL", country_code: "687", country_name: "New Caledonia", twilio: true, mobile_begin_with: [], min_number_length: 6, phone_number_lengths: [6], region: 2 /* pacific */ },
        { alpha2: "NE", alpha3: "NER", country_code: "227", country_name: "Niger", twilio: true, mobile_begin_with: ["9"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "NF", alpha3: "NFK", country_code: "672", country_name: "Norfolk Island", mobile_begin_with: ["5", "8"], min_number_length: 5, phone_number_lengths: [5], region: 2 /* pacific */ },
        { alpha2: "NG", alpha3: "NGA", country_code: "234", country_name: "Nigeria", twilio: true, mobile_begin_with: ["70", "80", "81"], min_number_length: 10, phone_number_lengths: [10], region: 3 /* emea */ },
        { alpha2: "NI", alpha3: "NIC", country_code: "505", country_name: "Nicaragua", twilio: true, mobile_begin_with: ["8"], min_number_length: 8, phone_number_lengths: [8], region: 1 /* america */ },
        { alpha2: "NU", alpha3: "NIU", country_code: "683", country_name: "Niue", mobile_begin_with: [], min_number_length: 4, phone_number_lengths: [4], region: 2 /* pacific */ },
        { alpha2: "NL", alpha3: "NLD", country_code: "31", country_name: "Netherlands", twilio: true, twilioVoice: true, mobile_begin_with: ["6"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "NO", alpha3: "NOR", country_code: "47", country_name: "Norway", twilio: true, twilioVoice: true, mobile_begin_with: ["4", "9"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "NP", alpha3: "NPL", country_code: "977", country_name: "Nepal", twilio: true, mobile_begin_with: ["97", "98"], min_number_length: 10, phone_number_lengths: [10], region: 2 /* pacific */ },
        { alpha2: "NR", alpha3: "NRU", country_code: "674", country_name: "Nauru", twilio: true, mobile_begin_with: ["555"], min_number_length: 7, phone_number_lengths: [7], region: 2 /* pacific */ },
        { alpha2: "NZ", alpha3: "NZL", country_code: "64", country_name: "New Zealand", twilio: true, twilioVoice: true, mobile_begin_with: ["2"], min_number_length: 8, phone_number_lengths: [8, 9, 10], region: 2 /* pacific */ },
        { alpha2: "OM", alpha3: "OMN", country_code: "968", country_name: "Oman", twilio: true, mobile_begin_with: ["9"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "PK", alpha3: "PAK", country_code: "92", country_name: "Pakistan", twilio: true, mobile_begin_with: ["3"], min_number_length: 10, phone_number_lengths: [10], region: 2 /* pacific */ },
        { alpha2: "PA", alpha3: "PAN", country_code: "507", country_name: "Panama", twilio: true, mobile_begin_with: ["5", "6"], min_number_length: 8, phone_number_lengths: [8], region: 1 /* america */ },
        { alpha2: "PE", alpha3: "PER", country_code: "51", country_name: "Peru", twilio: true, twilioVoice: true, mobile_begin_with: ["9"], min_number_length: 9, phone_number_lengths: [9], region: 1 /* america */ },
        { alpha2: "PH", alpha3: "PHL", country_code: "63", country_name: "Philippines", twilio: true, mobile_begin_with: ["9"], min_number_length: 10, phone_number_lengths: [10], region: 2 /* pacific */ },
        { alpha2: "PW", alpha3: "PLW", country_code: "680", country_name: "Palau", mobile_begin_with: [], min_number_length: 7, phone_number_lengths: [7], region: 2 /* pacific */ },
        { alpha2: "PG", alpha3: "PNG", country_code: "675", country_name: "Papua New Guinea", twilio: true, mobile_begin_with: ["7"], min_number_length: 8, phone_number_lengths: [8], region: 2 /* pacific */ },
        { alpha2: "PL", alpha3: "POL", country_code: "48", country_name: "Poland", twilio: true, twilioVoice: true, mobile_begin_with: ["5", "6", "7", "8"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "PT", alpha3: "PRT", country_code: "351", country_name: "Portugal", twilio: true, twilioVoice: true, mobile_begin_with: ["9"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "PY", alpha3: "PRY", country_code: "595", country_name: "Paraguay", twilio: true, mobile_begin_with: ["9"], min_number_length: 9, phone_number_lengths: [9], region: 1 /* america */ },
        { alpha2: "PS", alpha3: "PSE", country_code: "970", country_name: "Palestinian Authority", twilio: true, mobile_begin_with: ["5"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "PF", alpha3: "PYF", country_code: "689", country_name: "French Polynesia", mobile_begin_with: [], min_number_length: 6, phone_number_lengths: [6], region: 2 /* pacific */ },
        { alpha2: "QA", alpha3: "QAT", country_code: "974", country_name: "Qatar", twilio: true, mobile_begin_with: ["33", "55", "66", "77"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "RE", alpha3: "REU", country_code: "262", country_name: "Reunion", mobile_begin_with: ["692", "693"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "RO", alpha3: "ROU", country_code: "40", country_name: "Romania", twilio: true, twilioVoice: true, mobile_begin_with: ["7"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "RU", alpha3: "RUS", country_code: "7", country_name: "Russia", twilio: true, mobile_begin_with: ["9"], min_number_length: 10, phone_number_lengths: [10], region: 3 /* emea */ },
        { alpha2: "RW", alpha3: "RWA", country_code: "250", country_name: "Rwanda", twilio: true, mobile_begin_with: ["7"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "SA", alpha3: "SAU", country_code: "966", country_name: "Saudi Arabia", twilio: true, mobile_begin_with: ["5"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "SN", alpha3: "SEN", country_code: "221", country_name: "Senegal", twilio: true, mobile_begin_with: ["7"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "SG", alpha3: "SGP", country_code: "65", country_name: "Singapore", twilio: true, mobile_begin_with: ["8", "9"], min_number_length: 8, phone_number_lengths: [8], region: 2 /* pacific */ },
        { alpha2: "SH", alpha3: "SHN", country_code: "290", country_name: "St Helena, Ascension, Tristan da Cunha", mobile_begin_with: [], min_number_length: 4, phone_number_lengths: [4], region: 3 /* emea */ },
        { alpha2: "SJ", alpha3: "SJM", country_code: "47", country_name: "Svalbard", mobile_begin_with: [], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "SB", alpha3: "SLB", country_code: "677", country_name: "Solomon Islands", mobile_begin_with: ["7", "8"], min_number_length: 7, phone_number_lengths: [7], region: 2 /* pacific */ },
        { alpha2: "SL", alpha3: "SLE", country_code: "232", country_name: "Sierra Leone", mobile_begin_with: ["21", "25", "30", "33", "34", "40", "44", "50", "55", "76", "77", "78", "79", "88"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "SV", alpha3: "SLV", country_code: "503", country_name: "El Salvador", twilio: true, twilioVoice: true, mobile_begin_with: ["7"], min_number_length: 8, phone_number_lengths: [8], region: 1 /* america */ },
        { alpha2: "SM", alpha3: "SMR", country_code: "378", country_name: "San Marino", twilio: true, mobile_begin_with: ["3", "6"], min_number_length: 10, phone_number_lengths: [10], region: 3 /* emea */ },
        { alpha2: "SO", alpha3: "SOM", country_code: "252", country_name: "Somalia", mobile_begin_with: ["9"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "PM", alpha3: "SPM", country_code: "508", country_name: "Saint Pierre And Miquelon", mobile_begin_with: ["55"], min_number_length: 6, phone_number_lengths: [6], region: 1 /* america */ },
        { alpha2: "RS", alpha3: "SRB", country_code: "381", country_name: "Serbia", twilio: true, mobile_begin_with: ["6"], min_number_length: 8, phone_number_lengths: [8, 9], region: 3 /* emea */ },
        { alpha2: "ST", alpha3: "STP", country_code: "239", country_name: "São Tomé and Príncipe", twilio: true, mobile_begin_with: ["98", "99"], min_number_length: 7, phone_number_lengths: [7], region: 3 /* emea */ },
        { alpha2: "SR", alpha3: "SUR", country_code: "597", country_name: "Suriname", twilio: true, mobile_begin_with: ["6", "7", "8"], min_number_length: 7, phone_number_lengths: [7], region: 1 /* america */ },
        { alpha2: "SK", alpha3: "SVK", country_code: "421", country_name: "Slovakia", twilio: true, twilioVoice: true, mobile_begin_with: ["9"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "SI", alpha3: "SVN", country_code: "386", country_name: "Slovenia", twilio: true, mobile_begin_with: ["3", "4", "5", "6", "7"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "SE", alpha3: "SWE", country_code: "46", country_name: "Sweden", twilio: true, twilioVoice: true, mobile_begin_with: ["7"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "SZ", alpha3: "SWZ", country_code: "268", country_name: "Swaziland", twilio: true, mobile_begin_with: ["7"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "SC", alpha3: "SYC", country_code: "248", country_name: "Seychelles", twilio: true, mobile_begin_with: ["2"], min_number_length: 7, phone_number_lengths: [7], region: 3 /* emea */ },
        { alpha2: "TD", alpha3: "TCD", country_code: "235", country_name: "Chad", twilio: true, mobile_begin_with: ["6", "7", "9"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "TG", alpha3: "TGO", country_code: "228", country_name: "Togo", twilio: true, mobile_begin_with: ["9"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "TH", alpha3: "THA", country_code: "66", country_name: "Thailand", twilio: true, mobile_begin_with: ["8", "9"], min_number_length: 9, phone_number_lengths: [9], region: 2 /* pacific */ },
        { alpha2: "TJ", alpha3: "TJK", country_code: "992", country_name: "Tajikistan", twilio: true, mobile_begin_with: ["9"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "TK", alpha3: "TKL", country_code: "690", country_name: "Tokelau", mobile_begin_with: [], min_number_length: 4, phone_number_lengths: [4], region: 2 /* pacific */ },
        { alpha2: "TM", alpha3: "TKM", country_code: "993", country_name: "Turkmenistan", mobile_begin_with: [], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "TL", alpha3: "TLS", country_code: "670", country_name: "Timor-Leste", twilio: true, mobile_begin_with: ["7"], min_number_length: 8, phone_number_lengths: [8], region: 2 /* pacific */ },
        { alpha2: "TO", alpha3: "TON", country_code: "676", country_name: "Tonga", twilio: true, mobile_begin_with: [], min_number_length: 5, phone_number_lengths: [5], region: 2 /* pacific */ },
        { alpha2: "TN", alpha3: "TUN", country_code: "216", country_name: "Tunisia", twilio: true, mobile_begin_with: ["2", "9"], min_number_length: 8, phone_number_lengths: [8], region: 3 /* emea */ },
        { alpha2: "TR", alpha3: "TUR", country_code: "90", country_name: "Turkey", twilio: true, mobile_begin_with: ["5"], min_number_length: 10, phone_number_lengths: [10], region: 3 /* emea */ },
        { alpha2: "TV", alpha3: "TUV", country_code: "688", country_name: "Tuvalu", mobile_begin_with: [], min_number_length: 5, phone_number_lengths: [5], region: 2 /* pacific */ },
        { alpha2: "TW", alpha3: "TWN", country_code: "886", country_name: "Taiwan", twilio: true, mobile_begin_with: ["9"], min_number_length: 9, phone_number_lengths: [9], region: 2 /* pacific */ },
        { alpha2: "TZ", alpha3: "TZA", country_code: "255", country_name: "Tanzania", twilio: true, mobile_begin_with: ["7"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "UG", alpha3: "UGA", country_code: "256", country_name: "Uganda", twilio: true, mobile_begin_with: ["7"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "UA", alpha3: "UKR", country_code: "380", country_name: "Ukraine", twilio: true, mobile_begin_with: ["39", "50", "63", "66", "67", "68", "9"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "UY", alpha3: "URY", country_code: "598", country_name: "Uruguay", twilio: true, mobile_begin_with: ["9"], min_number_length: 8, phone_number_lengths: [8], region: 1 /* america */ },
        { alpha2: "UZ", alpha3: "UZB", country_code: "998", country_name: "Uzbekistan", twilio: true, mobile_begin_with: ["9"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "VE", alpha3: "VEN", country_code: "58", country_name: "Venezuela", twilio: true, mobile_begin_with: ["4"], min_number_length: 10, phone_number_lengths: [10], region: 1 /* america */ },
        { alpha2: "VN", alpha3: "VNM", country_code: "84", country_name: "Vietnam", twilio: true, mobile_begin_with: ["9", "1"], min_number_length: 9, phone_number_lengths: [9, 10], region: 2 /* pacific */ },
        { alpha2: "VU", alpha3: "VUT", country_code: "678", country_name: "Vanuatu", mobile_begin_with: ["5", "7"], min_number_length: 7, phone_number_lengths: [7], region: 2 /* pacific */ },
        { alpha2: "WF", alpha3: "WLF", country_code: "681", country_name: "Wallis and Futuna", mobile_begin_with: [], min_number_length: 6, phone_number_lengths: [6], region: 2 /* pacific */ },
        { alpha2: "WS", alpha3: "WSM", country_code: "685", country_name: "Samoa", twilio: true, mobile_begin_with: ["7"], min_number_length: 7, phone_number_lengths: [7], region: 2 /* pacific */ },
        { alpha2: "YE", alpha3: "YEM", country_code: "967", country_name: "Yemen", twilio: true, mobile_begin_with: ["7"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "ZA", alpha3: "ZAF", country_code: "27", country_name: "South Africa", twilio: true, twilioVoice: true, mobile_begin_with: ["6", "7", "8"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "ZM", alpha3: "ZMB", country_code: "260", country_name: "Zambia", twilio: true, mobile_begin_with: ["9"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ },
        { alpha2: "ZW", alpha3: "ZWE", country_code: "263", country_name: "Zimbabwe", twilio: true, mobile_begin_with: ["71", "73", "77"], min_number_length: 9, phone_number_lengths: [9], region: 3 /* emea */ }
    ];
    /**
      * Returns countries indexed by 2 character country code.
      * @param twilio If true only numbers supported by twilio will be returned.
      */
    function countriesByAlpha2(twilio) {
        if (twilio === void 0) { twilio = false; }
        if (!byAlpha2) {
            byAlpha2 = {};
            byAlpha2Twilio = {};
            for (var i = 0, l = consts.iso3166_data.length; i < l; i++) {
                var data = consts.iso3166_data[i];
                byAlpha2[data.alpha2] = data;
                if (data.twilio)
                    byAlpha2Twilio[data.alpha2] = data;
            }
        }
        return twilio ? byAlpha2Twilio : byAlpha2;
    }
    consts.countriesByAlpha2 = countriesByAlpha2;
    var byAlpha2;
    var byAlpha2Twilio;
    /**
      * Returns countries indexed by their country code.
      */
    function countriesByCountryCode() {
        if (!byCountryCode) {
            byCountryCode = {};
            for (var i = 0, l = consts.iso3166_data.length; i < l; i++) {
                var data = consts.iso3166_data[i];
                if (!byCountryCode[data.country_code])
                    byCountryCode[data.country_code] = data;
                if (data.area_codes) {
                    for (var j = 0, jl = data.area_codes.length; j < jl; j++) {
                        var key = data.country_code + data.area_codes[j];
                        if (!byCountryCode[key])
                            byCountryCode[key] = data;
                    }
                }
            }
        }
        return byCountryCode;
    }
    consts.countriesByCountryCode = countriesByCountryCode;
    var byCountryCode;
    /** Parses a phone number into its component parts. A 2-character country code is used for context. */
    function parsePhoneNumberForCountry(phone, alpha2) {
        var countries = countriesByAlpha2();
        var iso3166 = countries[alpha2];
        return parsePhoneNumber(phone, iso3166 ? iso3166.country_code : '1');
    }
    consts.parsePhoneNumberForCountry = parsePhoneNumberForCountry;
    /** Parses a phone number into its component parts. */
    function parsePhoneNumber(phone, defaultCountryCode) {
        // Create fail response
        var fail = {};
        fail.isValid = false;
        // Make sure phone is not null
        if (!phone) {
            return fail;
        }
        // Strip down to digits with optional leading '+'
        var match = phone.match(phoneNumberExp);
        if (!match) {
            return fail;
        }
        var num = match.join('');
        // Determine if number is an international or relative number.
        var isRelative = false;
        if (num.indexOf('000') == 0) {
            // Test number without country code
            isRelative = true;
        }
        else if (num.indexOf('0011') == 0) {
            num = num.substr(4);
        }
        else if (num.indexOf('011') == 0 || num.indexOf('010') == 0 || num.indexOf('+00') == 0) {
            num = num.substr(3);
        }
        else if (num.indexOf('00') == 0) {
            num = num.substr(2);
        }
        else if (num.indexOf('+') == 0) {
            num = num.substr(1);
        }
        else if (defaultCountryCode === '1' && num.length == 11 && num.indexOf('1') == 0) {
        }
        else if (defaultCountryCode === '1' && num.length == 10) {
            // 10 digit north american number but we want to make it 11 digits to catch the case where someone in US is 10 digit dialing someone in canada
            num = '1' + num;
        }
        else {
            // Relative number without country code
            isRelative = true;
        }
        // Parse phone number
        var codes = countriesByCountryCode();
        var parsed = {};
        if (isRelative) {
            parsed.countryCode = defaultCountryCode;
            parsed.address = num;
            parsed.iso3166 = codes[defaultCountryCode];
        }
        else {
            // International number
            // - Try 4-digit code first and reduce until match
            var c;
            var l = 4;
            do {
                c = codes[num.substr(0, l)];
            } while (!c && --l > 0);
            if (c) {
                parsed.countryCode = c.country_code;
                parsed.address = num.substr(c.country_code.length);
                parsed.iso3166 = c;
            }
            else {
                parsed.address = num;
            }
        }
        // Validate phone number
        parsed.isValid = false;
        if (parsed.iso3166) {
            parsed.isTest = (parsed.address.indexOf('000') == 0 || (parsed.countryCode == '1' && parsed.address.indexOf('555') == 0));
            parsed.isValid = (parsed.address.length >= parsed.iso3166.min_number_length);
            if (!parsed.isTest && parsed.isValid && parsed.iso3166.area_codes) {
                // Has valid length so validate area code
                parsed.isValid = false;
                for (var i = 0, l = parsed.iso3166.area_codes.length; i < l; i++) {
                    if (parsed.address.indexOf(parsed.iso3166.area_codes[i]) == 0) {
                        parsed.isValid = true;
                        break;
                    }
                }
            }
        }
        return parsed;
    }
    consts.parsePhoneNumber = parsePhoneNumber;
    var phoneNumberExp = /^\+|\d+/g;
    /** Returns a properly formatted phone number of the form "+<country><address>" */
    function toString(parsed) {
        return parsed ? '+' + (parsed.countryCode ? parsed.countryCode : '') + parsed.address : '';
    }
    consts.toString = toString;
    /** Returns the iso 3166 record for a country code. */
    function getIso3166(country) {
        var countries = countriesByAlpha2();
        return countries[country];
    }
    consts.getIso3166 = getIso3166;
})(consts || (consts = {}));
/// <reference path='../shared/consts.ts'/>
/// <reference path='./phoneNumber.ts' />
// the phoneNumber.ts is generated from ../../servers/shared/phoneNumber.ts build grunt moduleWrap
var consts;
(function (consts) {
    /** If true the server data center won't be checked for. Used only to test data center sensing.  */
    consts.disableServerDataCenter = false;
    /** If not empty string the client will be forced to connect to the datacenter specified (int, europe, etc). */
    consts.forceDataCenter = ''; // __grunt_flavor_forceDataCentner
    consts.applicationName = "XIM";
    consts.applicationVersion = "1.5.3";
    /* the ID that application insights assing to the project */
    consts.applicationAppInsightsID = '498d40e9-91b1-47e5-ae41-fd55c3065dac';
    /** Visibility of the users advertised profile. */
    /** NOTE: if you change this, change the server side. */
    consts.NearbyDiscoveryMode = {
        incognito: 'incognito',
        friendsOnly: 'friends',
        anyone: 'public'
    };
    /** App store app id for various platform/store */
    consts.mobileAppId = {
        google: '777860782824',
        apple: 'tbd',
        wp: 'tbd'
    };
    /** Flurry app id **/
    consts.flurryAppId = {
        google: 'XXWTRTRGKKKTNWZMVWJW',
        apple: '8GS8SCD4GF33VSZ9DYBY',
        wp: 'NQN3PD2R4KQ5GTMVNNCG',
        HTML5: '7SCJRQS2P3TRVGBBDMSP'
    };
    /** build flavor */
    consts.buildFlavor = ''; // __grunt_flavor_name
    // For "mailto" formatting
    consts.linebreak = '%0D%0A';
    consts.externalLinks = {
        faq: 'https://aka.ms/ximfaq',
        faqdisplay: 'https://aka.ms/ximdisplay',
        terms: 'https://aka.ms/ximterms',
        privacy: 'https://aka.ms/ximprivacy',
        appstore: 'https://aka.ms/ximupgrade',
        credits: 'https://aka.ms/ximcredits',
        email: 'https://aka.ms/ximfeedback',
        getxim: 'https://www.getxim.com',
        bug: 'mailto:ximtalk@microsoft.com?subject=Xim Error'
    };
    /** !!NOTE: Changes must be replicated in server consts file */
    consts.InviteType = {
        none: '',
        email: 'email',
        sms: 'sms',
        code: 'code',
        xim: 'xim'
    };
    /** Preferred screen locations when casting to multiple devices in 'span' mode.*/
    consts.SpanPositions = {
        center: 'center',
        left: 'left',
        right: 'right',
        top: 'top',
        bottom: 'bottom',
        topLeft: 'topLeft',
        topRight: 'topRight',
        bottomLeft: 'bottomLeft',
        bottomRight: 'bottomRight'
    };
    /** various urn for identification purpose */
    consts.urns = {
        chromecastNamespace: 'urn:x-cast:com.getxim.chromecast'
    };
    /** content provider app ids */
    consts.cloudProviders = {
        instagram: {
            appKey: '1b990cec852f427e8d08658464cb511f'
        },
        dropbox: {
            appKey: 'uqlbszi3t65xcbk'
        },
        flickr: {
            appKey: '06c8bd21647c2ae6b36c9fb65325c77a'
        },
        onedrive: {
            appKey: '',
            dev: {
                appKey: '0000000048127167'
            },
            int: {
                appKey: '0000000048127168'
            },
            prod: {
                appKey: '000000004C123B3C'
            },
            asia: {
                appKey: '000000004C12C255'
            },
            europe: {
                appKey: '0000000040132B4E'
            }
        },
        facebook: {
            appKey: '827071020660815' // __grunt_flavor_fbAppId
        },
        chromecast: {
            appKey: '73E21CD6' // __grunt_flavor_googleCastAppId
        }
    };
    (function (MenuState) {
        MenuState[MenuState["closed"] = 0] = "closed";
        MenuState[MenuState["open"] = 1] = "open";
    })(consts.MenuState || (consts.MenuState = {}));
    var MenuState = consts.MenuState;
    ;
    // Tab sets
    consts.tabs = {
        timeline: { id: "home", label: "home", emptyMessage: "no recent activity", glyph: "glyphiconXIM glyphiconXIM-rightArrow" },
        settings: { id: "settings", label: "Settings", glyph: "glyphiconXIM glyphiconXIM-rightArrow" },
        faq: { id: "faq", label: "FAQ", glyph: "glyphiconXIM glyphiconXIM-rightArrow" },
        profile: { id: "profile", label: "Profile", glyph: "glyphiconXIM glyphiconXIM-rightArrow" },
        privacy: { id: "privacy", label: "Privacy Notice", glyph: "glyphiconXIM glyphiconXIM-rightArrow" },
        terms: { id: "terms", label: "Terms of Use", glyph: "glyphiconXIM glyphiconXIM-rightArrow" },
        feeback: { id: "feedback", label: "Feedback", glyph: "glyphiconXIM glyphiconXIM-rightArrow" },
        credits: { id: "credits", label: "Credits", glyph: "glyphiconXIM glyphiconXIM-rightArrow" },
        login: { id: "login", label: "login" }
    };
    // instrumentation tags
    consts.tags = {
        boot: 'start',
        exit: 'end',
        command: 'cmd',
        cast: {
            src: 'src',
            join: 'join',
            leave: 'leave',
            invite: 'invite',
            rejoin: 'rejoin',
            startXim: 'startXim',
            text: {
                send: 'sendtxt',
                more: 'showtxt'
            }
        },
        ftux: {
            facebookAutoFill: 'facebookProfile',
            photoFromCameraRoll: 'pickProfilePhoto'
        }
    };
    consts.contentSourceTags = {
        'facebook': true,
        'skydrive': true,
        'flickr': true,
        'cameraroll': true,
        'instagram': true,
        'dropbox': true,
        'takephoto': true
    };
    // CommandBar commands
    consts.commands = {
        none: { id: "none", label: "", glyph: "" },
        addPeople: { id: "addPeople", label: "invite", glyph: "glyphicon glyphicon-plus" },
        back: { id: "back", label: "back", glyph: "glyphiconXIM glyphiconXIM-backArrowIcon" },
        cancel: { id: "cancel", label: "cancel", glyph: "glyphicon glyphicon-remove" },
        celebrate: { id: "celebrate", label: "celebrate", glyph: "glyphiconXIM glyphiconXIM-sunPress" },
        exit: { id: "exit", label: "exit xim", glyph: "glyphiconXIM glyphiconXIM-dismissIcon" },
        //link: { id: "link", label: "Unlink Screens", altLabel: "Link Screens", glyph: "glyphicon glyphicon-link" },
        messages: { id: "messages", label: "messages", glyph: "glyphiconXIM glyphiconXIM-chatIcon" },
        more: { id: "more", label: "more", glyph: "glyphicon glyphicon-list" },
        newgroup: { id: "new", label: "create new" },
        people: { id: "people", label: "people", glyph: "glyphiconXIM glyphiconXIM-peopleIcon" },
        photos: { id: "photos", label: "photos", glyph: "glyphiconXIM glyphiconXIM-photoIcon" },
        picker: { id: "picker", label: "picker", glyph: "glyphicon glyphicon-folder-open" },
        position: { id: "position", label: "Multi-Screen Offset", glyph: "glyphicon glyphicon-fullscreen" },
        present: { id: "present", label: "take control", glyph: "glyphiconXIM glyphiconXIM-takeControl" },
        refresh: { id: "refresh", label: "refresh", glyph: "glyphicon glyphicon-refresh" },
        //rename: { id: "rename", label: "rename", glyph: "glyphicon glyphicon-pencil" },
        rejoin: { id: "rejoin", label: "rejoin presenter", glyph: "glyphiconXIM glyphiconXIM-rejoinShow" },
        remove: { id: "remove", label: "remove", icon: "img/Remove-ContactIcon.png", glyph: "" },
        save: { id: "save", label: "save", glyph: "glyphiconXIM glyphiconXIM-floppySave" },
        settings: { id: "settings", label: "settings", icon: "img/SettingsIcon.png", glyph: "" },
        start: { id: "start", label: "start xim" },
        next: { id: "next", label: "next" },
        token: { id: "token", label: "token", glyph: "glyphicon glyphicon-lock" },
        addMedia: { id: "addMedia", label: "add", glyph: "glyphicon glyphicon-plus" },
        ximlist: { id: "xims", label: "xims", glyph: "glyphiconXIM glyphiconXIM-previousXims" },
        testPush: { id: "testPush", label: "Test Push", glyph: "glyphicon glyphicon-send" },
        logViewer: { id: "logViewer", label: "Log", glyph: "glyphicon glyphicon-book" },
        clear: { id: "clear", label: "clear", glyph: "glyphicon glyphicon-trash" },
        clearcache: { id: "clear", label: "clear cache", glyph: "glyphicon glyphicon-fire" },
        _template: { id: 'template', label: 'default', enabled: true }
    };
    consts.enableDebugFooter = false;
    consts.debugCommands = {
        log: { id: "log", label: "log", glyph: "" },
        home: { id: "home", label: "home", glyph: "" },
        debug: { id: "debug", label: "debug", glyph: "" },
        ftux: { id: 'ftux', label: 'FTUX', glyph: "" },
        startScreen: { id: 'startScreen', label: 'show loading', glyph: '' },
        showSplash: { id: 'showSplash', label: 'show splash', glyph: '' },
        chromecast: { id: 'chromecast', label: 'chromecast', glyph: '' },
        appletv: { id: 'appletv', label: 'apple tv', glyph: '' },
        smartglass: { id: 'smartglass', label: 'smart glass', glyph: '' },
        castToContacts: { id: 'castToContacts', label: 'cast to contacts', glyph: '' }
    };
    // Table constants
    consts.tableFilterLimit = 500;
    //--- Timeline Screen ---
    /** The number of full names we show on the timeline before we go to "and x more" */
    consts.nearbyProfileLimit = 3;
    /** max number of minutes a xim is allowed to be active*/
    consts.max_xim_age_minutes = 60;
    /** max time application allowed to be in the background before a restart */
    consts.max_background_time = 5;
    consts.ScreenLockState = {
        horizontal: 'horizontal',
        vertical: 'vertical'
    };
    // Images - LARS no longer used?
    consts.img_not_available = "img/not_available.png";
    consts.img_loading = "img/progress_loading.gif";
})(consts || (consts = {}));
/// <reference path='../../types/appinsights/appinsights.d.ts'/>
/// <reference path='../../types/flurry/flurry.d.ts'/>
/// <reference path='../mobile/consts.ts'/>
var client;
(function (client) {
    var analytics;
    (function (analytics) {
        analytics.flurry = null;
        (function (Provider) {
            Provider[Provider["appInsights"] = 0] = "appInsights";
            Provider[Provider["flurry"] = 1] = "flurry";
            Provider[Provider["unknown"] = 2] = "unknown";
        })(analytics.Provider || (analytics.Provider = {}));
        var Provider = analytics.Provider;
        var provider = 2 /* unknown */;
        function startAnalytics(analyticsProvider) {
            provider = analyticsProvider;
            var appID = '';
            if (analyticsProvider == 1 /* flurry */) {
                if (window.plugins && (window.plugins).flurry) {
                    analytics.flurry = window.plugins.flurry;
                }
                else {
                    // initializing flurry HTML5 API
                    if (window.FlurryAgent) {
                        analytics.flurry = window.FlurryAgent;
                        //stubbing in missing functions in FlurryAgent loaded from flurry.js
                        analytics.flurry.setSecureTransportEnabled = analytics.flurry.setSecureTransportEnabled || (function () {
                            return;
                        });
                        analytics.flurry.logEventWithParameters = analytics.flurry.logEventWithParameters || (function (eventName, params) {
                            var formattedParams = {};
                            try {
                                var keys = _.keys(params);
                                _.forEach(keys, function (key) {
                                    if (params[key]) {
                                        formattedParams[key] = params[key].toString();
                                    }
                                });
                            }
                            catch (e) {
                                analytics.logError(consts.errorCode.Analytics, 'Cannot convert params in logEventWithParameters: ' + e);
                            }
                            return analytics.flurry.logEvent(eventName, formattedParams);
                        });
                        analytics.flurry.logPageView = analytics.flurry.logPageView || (function () {
                        });
                        analytics.flurry.logTimedEventWithParameters = analytics.flurry.logTimedEventWithParameters || (function (eventName, params) {
                            return analytics.flurry.logEvent(eventName, params, true);
                        });
                        analytics.flurry.logTimedEvent = analytics.flurry.logTimedEvent || (function (eventName) {
                            return analytics.flurry.logEvent(eventName, {}, true);
                        });
                        analytics.flurry.endTimedEventWithParameters = analytics.flurry.endTimedEventWithParameters || (function (eventName, params) {
                            return analytics.flurry.endTimedEvent(eventName, params);
                        });
                    }
                    else {
                        client.logger.info('flurry plugin is not loaded');
                    }
                }
                switch (client.startupMode) {
                    case 4 /* ios */:
                        appID = consts.flurryAppId.apple;
                        break;
                    case 5 /* droid */:
                        appID = consts.flurryAppId.google;
                        break;
                    case 3 /* wp */:
                        appID = consts.flurryAppId.wp;
                        break;
                    case 0 /* desktop */:
                        appID = consts.flurryAppId.HTML5;
                        break;
                }
            }
            else if (analyticsProvider == 0 /* appInsights */) {
                appID = consts.applicationAppInsightsID;
            }
            start(appID, consts.applicationVersion);
        }
        analytics.startAnalytics = startAnalytics;
        function start(appID, appVersion) {
            if (provider == 0 /* appInsights */) {
                appInsights.start(appID);
            }
            else if (provider == 1 /* flurry */ && analytics.flurry) {
                analytics.flurry.setSecureTransportEnabled(true);
                if (appVersion) {
                    analytics.flurry.setAppVersion(appVersion);
                }
                analytics.flurry.startSession(appID);
            }
        }
        function end() {
            if (provider == 1 /* flurry */ && analytics.flurry) {
                analytics.flurry.endSession();
            }
        }
        analytics.end = end;
        /* set the client userID for analytics framework */
        /* appinsight uses this to measure unique users*/
        function setUserID(userID) {
            if (provider == 0 /* appInsights */) {
                appInsights.appUserId = userID;
            }
            else if (provider == 1 /* flurry */ && analytics.flurry) {
                analytics.flurry.setUserID(userID);
            }
        }
        analytics.setUserID = setUserID;
        /* log a client event */
        function logEvent(eventName, params) {
            if (provider == 0 /* appInsights */) {
                appInsights.logEvent(eventName, params);
            }
            else if (provider == 1 /* flurry */ && analytics.flurry) {
                if (params) {
                    analytics.flurry.logEventWithParameters(eventName, params);
                }
                else {
                    analytics.flurry.logEvent(eventName);
                }
            }
        }
        analytics.logEvent = logEvent;
        /* log a page view */
        function logView(viewName) {
            if (provider == 0 /* appInsights */) {
                appInsights.logPageView(viewName);
            }
            else if (provider == 1 /* flurry */ && analytics.flurry) {
                analytics.flurry.logPageView();
            }
        }
        analytics.logView = logView;
        /* start loging of a timed event */
        function logTimedEvent(eventName, params) {
            if (provider == 1 /* flurry */ && analytics.flurry) {
                if (params) {
                    analytics.flurry.logTimedEventWithParameters(eventName, params, true);
                }
                else {
                    analytics.flurry.logTimedEvent(eventName, true);
                }
            }
        }
        analytics.logTimedEvent = logTimedEvent;
        /* ending logging of a timed event */
        function endTimedEvent(eventName, params) {
            if (provider == 1 /* flurry */ && analytics.flurry) {
                if (params) {
                    analytics.flurry.endTimedEventWithParameters(eventName, params);
                }
                else {
                    analytics.flurry.endTimedEvent(eventName);
                }
            }
        }
        analytics.endTimedEvent = endTimedEvent;
        function logError(errorID, message) {
            if (provider == 1 /* flurry */ && analytics.flurry) {
                analytics.flurry.logError(errorID, message);
            }
        }
        analytics.logError = logError;
    })(analytics = client.analytics || (client.analytics = {}));
})(client || (client = {}));
/// <reference path='../../types/underscore/underscore.d.ts'/>
var client;
(function (client) {
    var Event = (function () {
        function Event() {
            this.handlers = [];
        }
        Event.prototype.add = function (handler) {
            this.handlers.push(handler);
        };
        Event.prototype.remove = function (handler) {
            this.handlers = _.reject(this.handlers, function (h, index) {
                return h === handler;
            });
        };
        Event.prototype.trigger = function (args) {
            _.each(this.handlers, function (handler, index) {
                handler(args);
            });
        };
        return Event;
    })();
    client.Event = Event;
})(client || (client = {}));
///<reference path='./types.ts'/>
var querystring;
(function (querystring) {
    function parse(query) {
        var q = {};
        if (query && query.length > 1) {
            var pairs = query.substr(0, 1) == '?' ? query.substr(1).split('&') : query.split('&');
            for (var i = 0; i < pairs.length; i++) {
                try {
                    var kv = pairs[i].split('=');
                    if (kv.length > 1) {
                        q[kv[0]] = decodeURIComponent(kv[1]);
                    }
                    else {
                        q[kv[0]] = '';
                    }
                }
                catch (e) {
                    client.logger.error(consts.errorCode.Parsing, 'Exception parsing query string', pairs[i], e);
                }
            }
        }
        return q;
    }
    querystring.parse = parse;
    function format(params) {
        var q = '';
        if (params) {
            var connector = '?';
            for (var k in params) {
                if (_.has(params, k)) {
                    var v = params[k];
                    if (v) {
                        try {
                            q += connector + k + '=' + encodeURIComponent(v.toString());
                            connector = '&';
                        }
                        catch (e) {
                            client.logger.error(consts.errorCode.Parsing, 'Exception encoding query string value', k, v);
                        }
                    }
                }
            }
        }
        return q;
    }
    querystring.format = format;
})(querystring || (querystring = {}));
///<reference path='./types.ts'/>
///<reference path='./querystring.ts'/>
var uri;
(function (uri) {
    function parse(url) {
        // find the query string if any       
        var hash = null;
        var parts = url.split('#', 2);
        if (parts.length > 1) {
            hash = parts[1];
            parts = parts[0].split('?', 2);
        }
        else {
            parts = url.split('?', 2);
        }
        var query = null;
        var protocol = url.substr(0, url.indexOf(':'));
        var hostAndPath = parts[0].substr(url.indexOf(':') + 1).split('/');
        var host = null;
        var hostname = null;
        var port = protocol == 'http' ? 80 : 443;
        while (!host) {
            host = hostAndPath.shift();
            var hostparts = host.split(':', 2);
            hostname = hostparts[0];
            if (hostparts.lenght == 2) {
                port = parseInt(hostparts[1]);
            }
        }
        var path = hostAndPath.join('/');
        if (parts.length > 1) {
            query = querystring.parse(parts[1]);
        }
        return { protocol: protocol, host: host, path: path, query: query, hash: hash, hostname: hostname, port: port };
    }
    uri.parse = parse;
    function format(url) {
        var result = url.protocol + '://' + url.host;
        if (url.path)
            result += '/' + url.path;
        if (url.query)
            result += querystring.format(url.query);
        if (url.hash)
            result += '#' + url.hash;
        return result;
    }
    uri.format = format;
    function resolve(base, rel) {
        if (base[base.length - 1] == '/' && rel[0] != '/') {
            return base + rel;
        }
        var url = parse(base);
        url.query = null;
        url.hash = null;
        if (rel[0] == '/') {
            url.path = null;
            return format(url) + rel;
        }
        else {
            return format(url) + '/' + rel;
        }
    }
    uri.resolve = resolve;
})(uri || (uri = {}));
/// <reference path='./types.ts'/>
/**
  * find, parse, format met='viewport' tag in the head section
  */
var client;
(function (client) {
    var webview;
    (function (webview) {
        ;
        (function (Orientation) {
            Orientation[Orientation["portrait"] = 0] = "portrait";
            Orientation[Orientation["landscape"] = 1] = "landscape";
        })(webview.Orientation || (webview.Orientation = {}));
        var Orientation = webview.Orientation;
        function findViewport() {
            return $('head meta[name="viewport"]');
        }
        function currentViewport() {
            var $viewport = findViewport();
            if ($viewport.length) {
                return parseViewport($viewport.attr('content'));
            }
            else {
                return { width: window.innerWidth, height: window.innerHeight };
            }
        }
        webview.currentViewport = currentViewport;
        function setCurrentViewport(viewport) {
            findViewport().attr('content', formatViewport(viewport));
        }
        webview.setCurrentViewport = setCurrentViewport;
        function parseViewport(text) {
            var parts = text ? text.split(',') : [];
            var result = { width: 0, height: 0 };
            _.each(parts, function (part, i) {
                var expr = part.split('=');
                if (expr.length == 2) {
                    result[expr[0].trim()] = expr[1].trim();
                }
            });
            // make sure width and height are numbers
            result.width = result.width ? 1 * result.width : result.width;
            result.height = result.height ? 1 * result.height : result.height;
            return result;
        }
        webview.parseViewport = parseViewport;
        function formatViewport(viewport) {
            return _.map(viewport, function (val, key) {
                return key + '=' + val;
            }).join(',');
        }
        webview.formatViewport = formatViewport;
        function syncViewportWithOrientation() {
            var orientation = window.innerHeight > window.innerWidth ? 0 /* portrait */ : 1 /* landscape */;
            setViewportOrientation(orientation);
        }
        webview.syncViewportWithOrientation = syncViewportWithOrientation;
        function setViewportOrientation(orientation) {
            var viewport = currentViewport();
            if (viewport.width > 0 && viewport.height > 0) {
                var isportrait = orientation == 0 /* portrait */;
                var bigger = Math.max(viewport.width, viewport.height);
                var smaller = Math.min(viewport.width, viewport.height);
                if (isportrait) {
                    viewport.width = smaller;
                    viewport.height = bigger;
                }
                else {
                    viewport.height = smaller;
                    viewport.width = bigger;
                }
                setCurrentViewport(viewport);
            }
        }
        webview.setViewportOrientation = setViewportOrientation;
        function getAsPortrait(size) {
            return { width: Math.min(size.width, size.height), height: Math.max(size.width, size.height) };
        }
        function isPortrait(size) {
            return size.height > size.width;
        }
        function swapDimension(size) {
            return { width: size.height, height: size.width };
        }
        /**
            scale size to match viewport size
        */
        function scaleToViewport(size) {
            var portrait = getAsPortrait(currentViewport());
            if (isPortrait(size)) {
                return portrait;
            }
            return swapDimension(portrait);
        }
        webview.scaleToViewport = scaleToViewport;
        /** Returns screen size for rendering UI */
        function getScreenSize() {
            var windowSize = { width: window.innerWidth, height: window.innerHeight };
            return client.webview.scaleToViewport(windowSize);
        }
        webview.getScreenSize = getScreenSize;
    })(webview = client.webview || (client.webview = {}));
})(client || (client = {}));
/// <reference path='./types.ts'/>
var client;
(function (client) {
    var capability;
    (function (capability) {
        (function (video) {
            video[video["cueOneVideo"] = 0] = "cueOneVideo";
            video[video["playOneVideo"] = 1] = "playOneVideo";
            video[video["cuePlayList"] = 2] = "cuePlayList";
            video[video["playPlayList"] = 3] = "playPlayList";
        })(capability.video || (capability.video = {}));
        var video = capability.video;
        function androidVersion() {
            var result = -1;
            var agents = navigator.userAgent.split(';');
            client.logger.info('user agent is', agents);
            var android = _.find(agents, function (agent, i) {
                return agent.trim().indexOf('Android') == 0;
            });
            if (android) {
                android = android.trim();
                client.logger.info('android version number is', android.substr(8));
                result = parseFloat(android.substr(8));
            }
            return result;
        }
        capability.androidVersion = androidVersion;
        capability.isJellybean = androidVersion() >= 4.3 && androidVersion() < 4.4;
        function getVideoCapbility(iswp) {
            var result = 0 /* cueOneVideo */;
            var protocol = window.location.protocol;
            var isApp = !protocol || protocol.indexOf('http') != 0;
            var agent = navigator.userAgent;
            var agents = agent.split(';');
            client.logger.info('user agent is', agents);
            if (isApp) {
                var android = _.find(agents, function (agent, i) {
                    return agent.trim().indexOf('Android') == 0;
                });
                if (android) {
                    android = android.trim();
                    client.logger.info('android version number is', android.substr(8));
                    result = parseFloat(android.substr(8)) < 4.4 ? 0 /* cueOneVideo */ : 2 /* cuePlayList */;
                }
                else {
                    result = iswp ? 1 /* playOneVideo */ : 3 /* playPlayList */;
                }
            }
            else {
                // ios safari browser can only cue video
                if (agent.match(/(iPod|iPad|iPhone)/) && agent.indexOf('AppleWebKit') > 0) {
                    result = 2 /* cuePlayList */;
                }
                else if (agent.match(/IEMobile/) || agent.match(/Android/)) {
                    result = 0 /* cueOneVideo */;
                }
                else {
                    result = 3 /* playPlayList */;
                }
            }
            return result;
        }
        capability.getVideoCapbility = getVideoCapbility;
    })(capability = client.capability || (client.capability = {}));
})(client || (client = {}));
/// <reference path='../../../types/underscore/underscore.d.ts'/>
/// <reference path='../Event.ts'/>
var client;
(function (client) {
    var ObservableArray = (function () {
        function ObservableArray(array) {
            this.added = new client.Event();
            this.removed = new client.Event();
            if (!array) {
                this._array = [];
                this.length = 0;
            }
            else {
                this._array = _.clone(array);
            }
            this.length = this._array.length;
        }
        ObservableArray.prototype.push = function (obj) {
            this._array.push(obj);
            this.length = this._array.length;
            this.added.trigger({ obj: obj, index: this.length - 1 });
            return this;
        };
        ObservableArray.prototype.pop = function () {
            var result = this._array.pop();
            this.length = this._array.length;
            this.removed.trigger({ obj: result, index: this.length });
            return result;
        };
        ObservableArray.prototype.forEach = function (callback) {
            var _this = this;
            this._array.forEach(function (obj, index, list) {
                callback(obj, index, _this);
            });
        };
        ObservableArray.prototype.get = function (index) {
            return this._array[index];
        };
        return ObservableArray;
    })();
    client.ObservableArray = ObservableArray;
})(client || (client = {}));
/// <reference path='../../servers/shared/types.d.ts'/>
/// <reference path='../../types/async/async.d.ts'/>
/// <reference path='jquery-jade.ts'/>
/// <reference path='jquery-tiles.ts'/>
/// <reference path='mixins.ts'/>
/// <reference path='logger.ts'/>
/// <reference path='analytics.ts'/>
/// <reference path='Event.ts'/>
/// <reference path='querystring.ts'/>
/// <reference path='uri.ts'/>
/// <reference path='viewport.ts'/>
/// <reference path='capability.ts'/>
/// <reference path='models/ObservableArray.ts'/>
var client;
(function (client) {
    (function (StartupMode) {
        StartupMode[StartupMode["desktop"] = 0] = "desktop";
        StartupMode[StartupMode["emulator"] = 1] = "emulator";
        StartupMode[StartupMode["mobile"] = 2] = "mobile";
        StartupMode[StartupMode["wp"] = 3] = "wp";
        StartupMode[StartupMode["ios"] = 4] = "ios";
        StartupMode[StartupMode["droid"] = 5] = "droid"; // The app is running in PhoneGape on an Android device.
    })(client.StartupMode || (client.StartupMode = {}));
    var StartupMode = client.StartupMode;
    ;
    function returnAsync(callback, err, results) {
        var opts = [];
        for (var _i = 3; _i < arguments.length; _i++) {
            opts[_i - 3] = arguments[_i];
        }
        // Log error
        var e;
        if (err) {
            e = typeof err === 'object' ? err : new Error(err);
            client.logger.error(consts.errorCode.Other, e.message);
        }
        if (callback) {
            var args = Array.prototype.slice.call(arguments, 1);
            callback.apply(null, args);
        }
    }
    client.returnAsync = returnAsync;
    /** Windows "event" style synchronization primative. */
    var AsyncEvent = (function () {
        function AsyncEvent() {
            this.waiting = []; // Waiting callers
            this.signaled = null;
        }
        /** Waits for the event to become signaled. */
        AsyncEvent.prototype.wait = function (cb) {
            if (cb) {
                if (this.signaled) {
                    cb(this.signaled.e, this.signaled.r);
                }
                else {
                    this.waiting.push(cb);
                }
            }
        };
        /** Signals that an async operation has completed, unblocking any waiting listners. */
        AsyncEvent.prototype.signal = function (e, r) {
            if (!this.signaled) {
                this.signaled = { e: e, r: r };
                for (var i = 0; i < this.waiting.length; i++) {
                    try {
                        this.waiting[i](e, r);
                    }
                    catch (e) {
                        client.logger.error('Utils', 'AsyncEvent: exception - ' + e.toString());
                    }
                }
                this.waiting = [];
            }
        };
        /** Unsignals the event blocking future calls to waitFor(). */
        AsyncEvent.prototype.reset = function () {
            this.waiting = [];
            this.signaled = null;
        };
        return AsyncEvent;
    })();
    client.AsyncEvent = AsyncEvent;
})(client || (client = {}));
var phone = navigator;
var Cordova;
(function (Cordova) {
    Cordova.Platforms = {
        Android: 'Android',
        BlackBerry: 'BlackBerry',
        iOS: 'iOS',
        webOS: 'webOS',
        Tizen: 'Tizen',
        WinPhone7: 'WinCE',
        WinPhone8: 'Win32NT'
    };
    // cordova plugin result status code
    (function (callbackStatus) {
        callbackStatus[callbackStatus["NO_RESULT"] = 0] = "NO_RESULT";
        callbackStatus[callbackStatus["OK"] = 1] = "OK";
        callbackStatus[callbackStatus["CLASS_NOT_FOUND_EXCEPTION"] = 2] = "CLASS_NOT_FOUND_EXCEPTION";
        callbackStatus[callbackStatus["ILLEGAL_ACCESS_EXCEPTION"] = 3] = "ILLEGAL_ACCESS_EXCEPTION";
        callbackStatus[callbackStatus["INSTANTIATION_EXCEPTION"] = 4] = "INSTANTIATION_EXCEPTION";
        callbackStatus[callbackStatus["MALFORMED_URL_EXCEPTION"] = 5] = "MALFORMED_URL_EXCEPTION";
        callbackStatus[callbackStatus["IO_EXCEPTION"] = 6] = "IO_EXCEPTION";
        callbackStatus[callbackStatus["INVALID_ACTION"] = 7] = "INVALID_ACTION";
        callbackStatus[callbackStatus["JSON_EXCEPTION"] = 8] = "JSON_EXCEPTION";
        callbackStatus[callbackStatus["ERROR"] = 9] = "ERROR";
    })(Cordova.callbackStatus || (Cordova.callbackStatus = {}));
    var callbackStatus = Cordova.callbackStatus;
    ;
    // This is an event that fires when Cordova is fully loaded.
    function onDeviceReady(listener) {
        document.addEventListener("deviceready", listener, false);
    }
    Cordova.onDeviceReady = onDeviceReady;
    // This is an event that fires when a Cordova application is put into the background.
    function onPause(listener) {
        document.addEventListener("pause", listener, false);
    }
    Cordova.onPause = onPause;
    // This is an event that fires when a Cordova application is retrieved from the background.
    function onResume(listener) {
        document.addEventListener("resume", listener, false);
    }
    Cordova.onResume = onResume;
    // This is an event that fires when a Cordova application is online (connected to the Internet).
    function onOnline(listener) {
        document.addEventListener("online", listener, false);
    }
    Cordova.onOnline = onOnline;
    // This is an event that fires when a Cordova application is offline (not connected to the Internet).
    function onOffline(listener) {
        document.addEventListener("offline", listener, false);
    }
    Cordova.onOffline = onOffline;
    // This is an event that fires when the user presses the back button.
    function onBackButton(listener) {
        document.addEventListener("backbutton", listener, false);
    }
    Cordova.onBackButton = onBackButton;
    // This is an event that fires when a Cordova application detects the battery has reached the critical level threshold. (No WinPhone use onBatteryStatus)
    function onBatteryCritical(listener) {
        document.addEventListener("batterycritical", listener, false);
    }
    Cordova.onBatteryCritical = onBatteryCritical;
    // This is an event that fires when a Cordova application detects the battery has reached the low level threshold. (No WinPhone use onBatteryStatus)
    function onBatteryLow(listener) {
        document.addEventListener("batterylow", listener, false);
    }
    Cordova.onBatteryLow = onBatteryLow;
    // This is an event that fires when a Cordova application detects a change in the battery status.
    function onBatteryStatus(listener) {
        document.addEventListener("batterystatus", listener, false);
    }
    Cordova.onBatteryStatus = onBatteryStatus;
    // This is an event that fires when the user presses the menu button. (Android, BlackBerry only)
    function onMenuButton(listener) {
        document.addEventListener("menubutton", listener, false);
    }
    Cordova.onMenuButton = onMenuButton;
    // This is an event that fires when the user presses the search button. (Android only)
    function onSearchButton(listener) {
        document.addEventListener("searchbutton", listener, false);
    }
    Cordova.onSearchButton = onSearchButton;
    // This is an event that fires when the user presses the start call button. (BlackBerry only)
    function onStartCallButton(listener) {
        document.addEventListener("startcallbutton", listener, false);
    }
    Cordova.onStartCallButton = onStartCallButton;
    // This is an event that fires when the user presses the end call button. (BlackBerry only)
    function onEndCallButton(listener) {
        document.addEventListener("endcallbutton", listener, false);
    }
    Cordova.onEndCallButton = onEndCallButton;
    // This is an event that fires when the user presses the volume down button. (BlackBerry only)
    function onVolumeDownButton(listener) {
        document.addEventListener("volumedownbutton", listener, false);
    }
    Cordova.onVolumeDownButton = onVolumeDownButton;
    // This is an event that fires when the user presses the volume up button. (BlackBerry only)
    function onVolumeUpButton(listener) {
        document.addEventListener("volumeupbutton", listener, false);
    }
    Cordova.onVolumeUpButton = onVolumeUpButton;
})(Cordova || (Cordova = {}));
/// <reference path='../shared/types.ts'/>
/// <reference path='./consts.ts'/>
/// <reference path='../../types/cordova/cordova.ts'/>
/// <reference path='../../types/cordova/cordova-notification.d.ts'/>
/// <reference path='../../types/cordova/cordova-contacts.d.ts'/>
/// <reference path='../../types/underscore/underscore.d.ts'/>
/// <reference path='consts.ts'/>
/// <reference path='../shared/resources.ts'/>
var resources;
(function (resources) {
    /** Global app strings. */
    resources.app = {
        title: consts.applicationName,
        readyToCast: 'Ready to cast'
    };
    /** Displayed messages */
    resources.links = {
        getxim: 'www.getxim.com',
        pairEurope: 'europe.getxim.com/pair',
        pairAsia: 'asia.getxim.com/pair',
        pairUs: 'www.getxim.com/pair'
    };
    /** Displayed messages */
    resources.messages = {
        addedCamera: 'added 1 photo',
        connectError: 'Problem connecting to realtime server. Casting disabled. Restart app to try again.',
        searchingYouTube: 'Searching for YouTube videos',
        searchingInstagram: 'Searching for Instagram photos',
        loadingInstagram: 'Loading Instagram photos',
        loadingFacebook: 'Loading Facebook albums',
        loadingOneDrive: 'Loading OneDrive items',
        loadingFlickr: 'Loading Flickr photos',
        loadingDropbox: 'Loading Dropbox items',
        loadingCamera: 'Loading camera',
        login: 'Login in with Facebook',
        peoplepicker_Hello: 'hello!',
        peoplepicker_Title1: 'Invite friends, add a display or do both!',
        tagline: 'share your photos, not your phone',
        timeline_Title1: "No active xims!",
        timeline_Title2: "Create a new one.",
        startingXim: 'Loading the xim slideshow',
        unlinkDevice: {
            title: 'Deauthorize device?',
            message: 'Deauthorizing your device will unlink your account from this device.',
            choiceLeft: 'cancel',
            choiceRight: 'deauthorize'
        },
        deleteAccount: {
            title: 'Delete account?',
            message: 'Deleting your account will permanently remove your profile and xim contact history.',
            choiceLeft: 'cancel',
            choiceRight: 'delete'
        },
        disableCastViewer: {
            title: 'Disconnect?',
            message: 'Disconnect from external display?',
            choiceLeft: 'cancel',
            choiceRight: 'ok'
        },
        serverMessage: {
            title: 'Hello!',
            choiceLeft: 'ok'
        },
        joinxim: {
            line1: 'To view photos from your phone',
            line2: 'enter the code below on your xim phone app.*',
            line3: 'Remain on this screen while we set it up!',
            line4: '* If you have not installed xim on your phone, you can get from:',
            link: resources.links.getxim
        },
        expiredxim: {
            line1: 'This xim has expired!',
            line2: 'Xims have a life of 60 minutes.'
        },
        upsell: {
            description: 'To share your own photos visit:',
            link: resources.links.getxim
        },
        castViewChoice: {
            scanning: 'Looking for devices',
            title: 'Add a display'
        },
        lobby: {
            title: 'Ready to xim!',
            content: [
                {
                    line1: 'Create your xim using more than',
                    line2: 'one photo so that you can swipe,',
                    line3: 'pan, and zoom through together.'
                },
                {
                    line1: 'Friends with xim can add their photos too!',
                    line2: 'Create and share in the moment together.',
                    line3: ''
                },
                {
                    line1: 'You can xim any image file.',
                    line2: 'And yes, that includes',
                    line3: 'a PowerPoint exported to JPEG.'
                }
            ]
        },
        viewerError: {
            title: "Oops!",
            line1: "Something has gone wrong.",
            line2: "Sorry"
        }
    };
    resources.forms = {
        discoverySetting: 'Discovery Setting',
        search: 'add name, phone or email',
        selectServerPrompt: 'Server',
        simulatedUsersPrompt: 'Simulated Users',
        simulatedUsersPlaceholder: 'number of users',
        loginOverridePrompt: 'Login Type',
        locationOverridePrompt: 'Devices Location',
        searchPlaceholder: 'Type your search here',
        loggerLevel: 'Logger level',
        useMockData: 'Use mock contacts data',
        enableCelebrations: "Enable Celebrations",
        enablePresenter: "Cast: Enable presenter mode",
        enableNearby: 'People Picker: Enable nearby mode',
        enableCompressed: 'Cast: Enable compressed views',
        showSystemMessages: 'Cast: Show system messages in text stream',
        showDebug: 'Show debug settings UI',
        highlightNearby: 'Timeline: Highlight nearby users',
        showExpired: 'Timeline: Show expired xims',
        showConnection: 'Show connection state icon',
        shownFtux: "FTUX shown already",
        showCameraRoll: "Show Camera Roll",
        addBrowser: {
            title: "browser display",
            text: "Send xim to any web browser!",
            bullet1: "1. On another device visit:",
            bullet3: "2. Enter pairing code displayed on",
            bullet3s: "the device in the space below.",
            button: "pair device",
            placeholder: "######",
            subtext: "Learn more about displays."
        },
        addFireTV: {
            title: "fireTV display",
            text: "Send xim to your fireTV!",
            bullets: [
                "Install the xim viewer on your fireTV",
                "Open the xim viewer on your fireTV",
                "Enter the pairing code displayed on your fireTV in the space below"
            ],
            button: "pair device",
            placeholder: "######",
            subtext: "Learn more about displays."
        },
        xboxHowTo: {
            title: "Xbox One display",
            text: "Trouble finding your Xbox?",
            bullets: [
                "Make sure your Xbox One is on.",
                "Go to the home screen.",
                "Press the menu button on the controller.",
                "Select settings.",
                'Under SmartGlass Connections select "from any SmartGlass device"'
            ],
            button: "okay!",
            subtext: "Learn more about displays."
        },
        airplayHowTo: {
            title: "AirPlay display",
            text: "Connect to AirPlay",
            bullets: [
                "Connect your iOS device to the same wifi network as your Apple TV.",
                "Swipe up from the bottom of your screen to access Control Center.",
                "Tap AirPlay.",
                "Tap the the device you would like to display your xim.",
                "Make sure mirroring is turned on."
            ],
            button: "okay!",
            subtext: "Learn more about displays."
        },
        photoPopUp: {
            addphotos: "add photos",
            takephoto: "take photo"
        },
        peoplePopUp: {
            addpeople: "invite more people",
            people: "People",
            present: "Present"
        },
        messagePopUp: {
            button: 'express yourself',
            placeholder: 'Express yourself!'
        },
        commentPopUp: {
            placeholder: "comment on photo",
            leftButton: "Cancel",
            rightButton: "Post"
        }
    };
    resources.login = {
        phone: {
            intro1: 'Your phone number is used to connect you',
            intro2: 'into the world of xim. ',
            intro3: '',
            inputCue: '(555) 555-5555',
            next: 'next',
            sub1: 'By continuing you agree to our',
            termsOfUse: 'Terms of Use',
            privacy: 'Privacy & Cookie Policy',
            error: 'invalid number, try again'
        },
        choice: {
            intro1: 'We need to verify your number:',
            intro2: 'How would you like to receive',
            intro3: 'your verification code?',
            sms: 'text message',
            phone: 'phone call',
            sub1: 'Change phone number'
        },
        verify: {
            intro1: 'To verify your phone number, enter the',
            intro2: '4-digit pin we just gave you.',
            inputCue: '0000',
            next: 'next',
            sub1: 'If you did not receive a pin,',
            sub2: 'tap here to retry.',
            error: 'invalid pin, try again',
            expired: 'pin has expired'
        },
        profile: {
            intro1: 'How would you like to appear in xim?',
            intro2: 'Could be a photo or just your initials!',
            inputCue1: 'first name',
            inputCue2: 'last name',
            next: 'create profile',
            edit: 'update profile',
            sub1: 'or',
            fb: 'autofill from facebook',
            error: '(required)',
            legal: '(letters only)'
        }
    };
    /** Ftux string */
    resources.ftux = {
        createNew: '1. get started',
        createNewSub: 'swipe left to take a tour',
        pickPhotos: '2. pick photos',
        pickPhotosSub: 'take a photo or choose a service',
        invitePeople: '3. invite people',
        invitePeopleSub: "they don't even need the app!",
        screens: '4. screens move together',
        screensSub: 'all screens swipe, pan and zoom in sync',
        longPress: '5. long press for options',
        longPressSub: 'add a caption, save and more&hellip;',
        ready: '6. ready, set, swipe!',
        readySub: 'tap start, have fun',
        start: 'start'
    };
    /** titles and labels */
    resources.labels = {
        loginPhoneTitle: 'phone number',
        loginCountryTitle: 'country code',
        loginChoiceTitle: 'contact method',
        loginVerifyTitle: 'verification',
        loginProfileTitle: 'profile',
        pickerTitle: 'pick source',
        pickProfilePhoto: 'pick photo',
        peopleTitle: 'xim with',
        settings: 'settings',
        pickerType: {
            photo: 'photos',
            doc: 'a document'
        },
        pickerSource: {
            fb: 'Facebook',
            sk: 'OneDrive',
            yt: 'YouTube',
            fk: 'Flickr',
            db: 'Dropbox',
            gd: 'Goggle Drive',
            en: 'Evernote',
            ps: 'Picasa',
            vm: 'Vimeo',
            bx: 'Box',
            ig: 'Instagram',
            cm: 'Take a photo',
            photos: 'Phone Photos',
            pictures: 'Phone Pictures',
            gallery: 'Phone Gallery'
        },
        login: {
            login: 'Log in',
            facebook: 'Connect'
        },
        tiles: {
            all: 'all',
            more: 'more',
            photostream: 'Photo Stream',
            likes: 'Likes',
            mine: 'Mine',
            add: 'add'
        },
        thumbnailPopUp: {
            photo: 'choose photo',
            initials: 'use initials'
        }
    };
})(resources || (resources = {}));
/// <reference path="../Event.ts"/>
/** Table related tools. */
var client;
(function (client) {
    var table;
    (function (table) {
        /**
          * Pages a set of models into memory.
          * @param page First page of the set to enumerate.
          * @param callback Async results of the operation.
          * @param options (optonal) Filtering & limit options for the operation.
          */
        function pageModels(page, callback, options) {
            if (options === void 0) { options = {}; }
            var models = [];
            function more(e, p) {
                if (p) {
                    var next = p.next;
                    for (var i = 0; i < p.models.length; i++) {
                        if (!options.filterBy || options.filterBy(p.models[i], models.length, models)) {
                            models.push(p.models[i]);
                            if (options.limit && models.length == options.limit) {
                                next = null;
                                break;
                            }
                        }
                    }
                    if (next) {
                        next(more);
                    }
                    else {
                        callback(null, models);
                    }
                }
                else {
                    callback(e, null);
                }
            }
            more(null, page);
        }
        table.pageModels = pageModels;
        /**
          * Filters & sorts a set of models returned in pages.
          * @param page First page of the set to filter.
          * @param callback Async results of the operation.
          * @param options (optonal) Filtering, sorting, & limit options for the operation.
          */
        function filterModels(page, callback, options) {
            if (options === void 0) { options = {}; }
            pageModels(page, function (err, models) {
                if (!err) {
                    if (models.length > 0 && options.sortBy) {
                        callback(null, models.sort(options.sortBy));
                    }
                    else {
                        callback(null, models);
                    }
                }
                else {
                    callback(err, null);
                }
            }, options);
        }
        table.filterModels = filterModels;
        /**
          * Returns a sortBy comparison operator for numeric sorting.
          * @param prop The property to sort on.
          */
        function sortNumberASC(prop) {
            return function (a, b) {
                var av = a[prop], bv = b[prop];
                if (av === bv) {
                    return 0;
                }
                else if (!bv || (av && av > bv)) {
                    return 1;
                }
                else if (!av || (bv && bv > av)) {
                    return -1;
                }
            };
        }
        table.sortNumberASC = sortNumberASC;
        /**
          * Returns a sortBy comparison operator for numeric sorting.
          * @param prop The property to sort on.
          */
        function sortNumberDESC(prop) {
            return function (a, b) {
                var av = a[prop], bv = b[prop];
                if (av === bv) {
                    return 0;
                }
                else if (!bv || (av && av > bv)) {
                    return -1;
                }
                else if (!av || (bv && bv > av)) {
                    return 1;
                }
            };
        }
        table.sortNumberDESC = sortNumberDESC;
        /**
          * Returns a sortBy comparison operator for string sorting.
          * @param prop The property to sort on.
          */
        function sortStringASC(prop) {
            return function (a, b) {
                var av = a[prop], bv = b[prop];
                if (!av && !bv) {
                    return 0;
                }
                else if (!av) {
                    return 1;
                }
                else if (!bv) {
                    return -1;
                }
                return av.localeCompare(bv);
            };
        }
        table.sortStringASC = sortStringASC;
        /**
          * Returns a sortBy comparison operator for string sorting.
          * @param prop The property to sort on.
          */
        function sortStringDESC(prop) {
            return function (a, b) {
                var av = a[prop], bv = b[prop];
                if (!av && !bv) {
                    return 0;
                }
                else if (!av) {
                    return 1;
                }
                else if (!bv) {
                    return -1;
                }
                return bv.localeCompare(av);
            };
        }
        table.sortStringDESC = sortStringDESC;
    })(table = client.table || (client.table = {}));
})(client || (client = {}));
/// <reference path="Table.ts"/>
var client;
(function (client) {
    var DefaultLocalTableOptions = {
        pageSize: 50,
        storage: localStorage,
        asyncResults: false,
        missingOk: false
    };
    /** Implements a Table abstraction over local storage. */
    var LocalTable = (function () {
        /**
          * Creates a new local table accessor.
          * @param name Name of the table.
          * @param options (optional) Settings for the table.
          */
        function LocalTable(name, options) {
            this.name = name;
            this.options = options;
            this.options = _.extend({}, DefaultLocalTableOptions, options);
        }
        /**
          * Returns a model from a table given its ID.
          * @param id ID of the model to retireve.
          * @param callback Async result of operation. The retrieved model is returned.
          */
        LocalTable.prototype.get = function (id, callback) {
            var m = this.options.storage.getItem(this.name + '!' + id);
            return this.returnResults(callback, m == undefined && !this.options.missingOk ? new Error('not found') : null, m ? JSON.parse(m) : null);
        };
        /**
          * Replaces an existing model in the table.
          * @param model Model to replace.
          * @param upsert (optional) If true the model will be inserted if it's missing.
          * @param callback (optional) Async result of operation. The replaced model is returned.
          */
        LocalTable.prototype.replace = function (model, upsert, callback) {
            // Ensure upsert or item exists
            var id = model['id'];
            if (!id)
                return this.returnResults(callback, new Error('missing id'));
            var key = this.name + '!' + id;
            if (!upsert && !this.options.storage.getItem(key))
                return this.returnResults(callback, new Error('not found'));
            // Replace item
            this.options.storage.setItem(key, JSON.stringify(model));
            return this.returnResults(callback, null, model);
        };
        /**
          * Merges a model with an existing model in the table.
          * @param model Model to merge.
          * @param upsert (optional) If true the model will be inserted if it's missing.
          * @param callback (optional) Async result of operation. The merged model is returned.
          */
        LocalTable.prototype.merge = function (model, upsert, callback) {
            // Validate ID
            var id = model['id'];
            if (!id)
                return this.returnResults(callback, new Error('missing id'));
            // Merge model
            var key = this.name + '!' + id;
            var m = this.options.storage.getItem(key);
            var err;
            if (m) {
                model = _.extend(JSON.parse(m), model);
                this.options.storage.setItem(key, JSON.stringify(model));
            }
            else if (upsert) {
                this.options.storage.setItem(key, JSON.stringify(model));
            }
            else {
                err = new Error('not found');
            }
            return this.returnResults(callback, err, model);
        };
        /**
          * Adds a new model to the table.
          * @param model Model to insert.
          * @param callback (optional) Async result of operation. The inserted model is returned.
          */
        LocalTable.prototype.insert = function (model, callback) {
            // Ensure unique ID
            var id = model['id'];
            if (!id)
                return this.returnResults(callback, new Error('missing id'));
            var key = this.name + '!' + id;
            if (this.options.storage.getItem(key))
                return this.returnResults(callback, new Error('already exists'));
            // Insert model
            this.options.storage.setItem(key, JSON.stringify(model));
            return this.returnResults(callback, null, model);
        };
        /**
          * Removes a model from the table.
          * @param id ID of the model to delete.
          * @param callback (optional) Async result of operation.
          */
        LocalTable.prototype.delete = function (id, callback) {
            this.options.storage.removeItem(this.name + '!' + id);
            return this.returnResults(callback, null);
        };
        /**
          * Lists the contents of the table.
          * @param callback Async result of operation. The first page is returned.
          */
        LocalTable.prototype.list = function (callback) {
            // Find tables keys
            var keys = [], prefix = this.name + '!', lPrefix = prefix.length;
            for (var i = 0, l = this.options.storage.length; i < l; i++) {
                var k = this.options.storage.key(i);
                if (k.substr(0, lPrefix) === prefix)
                    keys.push(k);
            }
            // Iterator
            var i = 0, l = keys.length, that = this;
            function nextPage(cb) {
                try {
                    var page = { models: [] };
                    while (i < l && page.models.length < that.options.pageSize) {
                        page.models.push(JSON.parse(that.options.storage.getItem(keys[i++])));
                    }
                    if (i < l)
                        page.next = nextPage;
                    that.returnResults(cb, null, page);
                }
                catch (e) {
                    that.returnResults(cb, e);
                }
            }
            nextPage(callback);
            return this;
        };
        /**
          * Filters & sorts the contents of the table.
          * @param callback Async result of operation. The first page is returned.
          * @param options (optonal) Filtering, sorting, & limit options for the operation.
          */
        LocalTable.prototype.filter = function (callback, options) {
            this.list(function (err, page) {
                if (page) {
                    client.table.filterModels(page, callback, options);
                }
                else {
                    callback(err, null);
                }
            });
            return this;
        };
        //=================================================
        // Private helpers
        //=================================================
        /** Helper funtion to simulate async callbacks. */
        LocalTable.prototype.returnResults = function (callback, err, results) {
            if (callback) {
                if (this.options.asyncResults) {
                    setTimeout(function () {
                        callback(err, results);
                    }, 0);
                }
                else {
                    callback(err, results);
                }
            }
            return this;
        };
        return LocalTable;
    })();
    client.LocalTable = LocalTable;
})(client || (client = {}));
/// <reference path="LocalTable.ts"/>
/// <reference path="../types.ts"/>
/// <reference path="../consts.ts"/>
var client;
(function (client) {
    var DefaultModelCacheOptions = {
        maxAge: consts.time.minutes(15),
        pageSize: 50,
        storage: sessionStorage,
        missingOk: true
    };
    /** Implements a local cache of models. */
    var ModelCache = (function () {
        /**
          * Creates a new instance of the cache.
          * @param name Name of the cache table in local storage.
          * @param options (optional) Settings used to configure the cache and backing table.
          */
        function ModelCache(name, options) {
            if (options === void 0) { options = {}; }
            this.name = name;
            this.options = options;
            this.options = _.extend({}, DefaultModelCacheOptions, options);
            this.cache = new client.LocalTable(name, options);
        }
        /**
          * Returns a model from the cache given its ID.
          * @param id ID of the model to retireve.
          * @param callback Async result of operation. The retrieved model is returned.
          */
        ModelCache.prototype.get = function (id, callback) {
            var _this = this;
            this.cache.get(id, function (err, model) {
                if (err) {
                    callback(err, null);
                }
                else if (model && model.expires > consts.time.now()) {
                    callback(null, model.model);
                }
                else if (_this.options.retriever) {
                    _this.options.retriever(id, function (e, m) {
                        if (!e && m) {
                            // Cache model
                            _this.cache.replace(_this.newEntry(m), true);
                        }
                        callback(e, m);
                    });
                }
                else {
                    callback(null, null);
                }
            });
            return this;
        };
        /**
          * Replaces an existing model in the cache.
          * @param model Model to replace.
          * @param upsert (optional) If true the model will be inserted if it's missing.
          * @param callback (optional) Async result of operation. The replaced model is returned.
          */
        ModelCache.prototype.replace = function (model, upsert, callback) {
            this.cache.replace(this.newEntry(model), upsert, this.returnModel(callback));
            return this;
        };
        /**
          * Merges a model with an existing model in the cache.
          * @param model Model to merge.
          * @param upsert (optional) If true the model will be inserted if it's missing.
          * @param callback (optional) Async result of operation. The merged model is returned.
          */
        ModelCache.prototype.merge = function (model, upsert, callback) {
            this.cache.merge(this.newEntry(model), upsert, this.returnModel(callback));
            return this;
        };
        /**
          * Inserts a new model into the table.
          * @param model Model to insert.
          * @param callback (optional) Async result of operation. The inserted model is returned.
          */
        ModelCache.prototype.insert = function (model, callback) {
            this.cache.insert(this.newEntry(model), this.returnModel(callback));
            return this;
        };
        /**
          * Removes a model from the cache.
          * @param id ID of the model to delete.
          * @param callback (optional) Async result of operation.
          */
        ModelCache.prototype.delete = function (id, callback) {
            this.cache.delete(id, callback);
            return this;
        };
        /**
          * Manually purges the local cache.
          * @param callback (optional) Async result of the operation.
          */
        ModelCache.prototype.purge = function (callback) {
            var that = this;
            function purgeCache(next) {
                next(function (e, p) {
                    if (e) {
                        if (callback)
                            callback(e);
                    }
                    else {
                        var now = consts.time.now();
                        _.each(p.models, function (m) {
                            if (m.expires <= now)
                                that.cache.delete(m.id);
                        });
                        if (p.next) {
                            purgeCache(p.next);
                        }
                        else {
                            if (callback)
                                callback(null);
                        }
                    }
                });
            }
            purgeCache(this.cache.list);
            return this;
        };
        //=================================================
        // Private helpers
        //=================================================
        /** Injects a callback function to model from cache enrty as a result of a cache table operation. */
        ModelCache.prototype.returnModel = function (callback) {
            return function (e, ce) {
                if (callback)
                    callback(e, ce ? ce.model : null);
            };
        };
        /** Creates a new cache entry. */
        ModelCache.prototype.newEntry = function (model) {
            return {
                id: model['id'],
                model: model,
                expires: consts.time.now() + this.options.maxAge
            };
        };
        return ModelCache;
    })();
    client.ModelCache = ModelCache;
})(client || (client = {}));
var proximity;
(function (proximity) {
    /** Strongly typed event class. */
    var Event = (function () {
        function Event() {
            // Private member vars
            this.listeners = [];
        }
        /**
          * Registers a new listener for the event.
          * @param listener The callback function to register.
          */
        Event.prototype.add = function (listener) {
            this.listeners.push(listener);
        };
        /** Returns true if there are any registered listeners for the event. */
        Event.prototype.hasListeners = function () {
            return this.listeners.length > 0;
        };
        /**
          * Registers a listner for only a single occurence of the event.
          * @param listener The callback function to register.
          */
        Event.prototype.once = function (listener) {
            var that = this;
            function listen(ev) {
                that.remove(listen);
                listener(ev);
            }
            listen.fn = listener;
            this.add(listen);
        };
        /**
          * Unregisters a listener from the event.
          * @param listener (optional) Callback function that was registered. If missing then all listeners will be removed.
          */
        Event.prototype.remove = function (listener) {
            var filtered = [];
            if (listener) {
                for (var i = 0; i < this.listeners.length; i++) {
                    var l = this.listeners[i];
                    if (l !== listener && l.fn !== listener)
                        filtered.push(l);
                }
            }
            this.listeners = filtered;
        };
        /** Ensures that events are never serialized when calling JSON.stringify(). */
        Event.prototype.toJSON = function () {
            return null;
        };
        /**
          * Invokes all of the listeners for this event.
          * @param a (optional) set of arguments to pass to listners.
          */
        Event.prototype.trigger = function (ev) {
            var context = {};
            var listeners = this.listeners;
            for (var i = 0; i < listeners.length; i++) {
                listeners[i].call(context, ev);
            }
        };
        return Event;
    })();
    proximity.Event = Event;
})(proximity || (proximity = {}));
/// <reference path='types.d.ts' />
/// <reference path='Event.ts' />
/// <reference path='../shared/logger.ts'/>
/// <reference path='../shared/consts.ts'/>
/// <reference path='../shared/analytics.ts' />
var proximity;
(function (proximity) {
    (function (ClientState) {
        ClientState[ClientState["disconnected"] = 0] = "disconnected";
        ClientState[ClientState["connecting"] = 1] = "connecting";
        ClientState[ClientState["connected"] = 2] = "connected";
        ClientState[ClientState["reconnecting"] = 3] = "reconnecting";
    })(proximity.ClientState || (proximity.ClientState = {}));
    var ClientState = proximity.ClientState;
    var defaultClientOptions = {
        /** Reconnect for about a minute with a backoff policy */
        reconnectPlan: [0, 500, 500, 1000, 3000, 5000, 10000, 10000, 10000, 10000]
    };
    /** Clients connection to the server. */
    var Client = (function () {
        /**
          * Creates a new instance of the server.
          * @param server The proximity server to connect to.
          * @param options (optional) Options that control the clients connection.
          */
        function Client(server, options) {
            if (options === void 0) { options = {}; }
            this.server = server;
            this.options = options;
            this.pending = [];
            this.responses = {};
            this.nextId = 0;
            this.sessions = {};
            /** Raised when the clients state changes. */
            this.onStateChanged = new proximity.Event();
            /** The clients current connection state. */
            this.state = 0 /* disconnected */;
            /** Number of seconds before we automatically generate an error response. */
            this.responseTimeout = consts.time.seconds(30);
            this.options = _.extend({}, defaultClientOptions, this.options);
        }
        /** Manually disposes of the connection. */
        Client.prototype.dispose = function () {
            this.onStateChanged.remove();
            this.disconnect();
        };
        /** Connects to the server. */
        Client.prototype.connect = function () {
            var _this = this;
            this.disconnect(function () {
                _this.setState(1 /* connecting */);
                _this.reconnect();
            });
            return this;
        };
        /** Manually disconnects from the server. */
        Client.prototype.disconnect = function (cb) {
            var _this = this;
            if (this.socket) {
                client.logger.debug('Multicast: disconnecting from server');
                this.socket.send(JSON.stringify({ name: 'disconnect' }), function () {
                    _this.socket.close();
                    _this.socket = null;
                    _this.setState(0 /* disconnected */);
                    if (cb)
                        cb(null, true);
                });
            }
            else if (cb) {
                cb(null, false);
            }
            return this;
        };
        /**
          * Sends an event to the server.
          * @param ev The event to emit.
          * @param sent (optional) Callback to fire once the event has been successfully delivered.
          */
        Client.prototype.emit = function (ev, sent) {
            if (this.state != 0 /* disconnected */) {
                var id = this.nextId++;
                var that = this;
                function send() {
                    // Deliver event
                    client.logger.debug('Multicast: SEND ' + ev.name + '(' + (ev.session || 'null') + ', ' + (ev.responseId || 'null') + ')');
                    that.socket.send(JSON.stringify(ev), function () {
                        // Remove from pending list
                        that.pending = _.filter(that.pending, function (v) {
                            return v.id != id;
                        });
                        if (sent)
                            sent(null, true);
                    });
                }
                if (this.socket) {
                    send();
                }
                else {
                    // Push event onto pending list
                    this.pending.push({
                        id: id,
                        retry: function (err) {
                            if (!err) {
                                send();
                            }
                            else if (sent) {
                                sent(err, false);
                            }
                        }
                    });
                }
            }
            else if (sent) {
                sent(new Error('Disconnected'), false);
            }
            return this;
        };
        /**
          * Sends an event to the server and optionally waits for a response.
          * @param name Name of the event.
          * @param data (optional) Even data.
          * @param response (optional) Handler for response from server.
          */
        Client.prototype.send = function (name, data, response) {
            var _this = this;
            var ev = { name: name, data: data };
            this.registerResponse(ev, response);
            return this.emit(ev, function (err, sent) {
                if (err && ev.responseId)
                    _this.handleResponse(ev.responseId, err, null);
            });
        };
        /**
          * Sends an event to the server in the context of a session and optionally waits for a response.
          * @param session ID of the session.
          * @param name Name of the event.
          * @param data (optional) Even data.
          * @param response (optional) Handler for response from server.
          */
        Client.prototype.sendForSession = function (sessionId, name, data, response) {
            var _this = this;
            var ev = { session: sessionId, name: name, data: data };
            this.registerResponse(ev, response);
            return this.emit(ev, function (err, sent) {
                if (err && ev.responseId)
                    _this.handleResponse(ev.responseId, err, null);
            });
        };
        /**
          * Registers a session with the client for message routing purposes.
          * @param session The session to rout incoming messages to.
          */
        Client.prototype.registerSession = function (session) {
            this.sessions[session.sessionId] = session;
            return this;
        };
        /**
          * Unregisters a session with the client.
          * @param sessionId ID of the session to unregister.
          */
        Client.prototype.unregisterSession = function (sessionId) {
            if (sessionId in this.sessions)
                delete this.sessions[sessionId];
            return this;
        };
        //=======================================
        // Private Helpers
        //=======================================
        Client.prototype.setState = function (state) {
            if (state != this.state) {
                this.state = state;
                this.onStateChanged.trigger({ state: state });
            }
        };
        /** Registers response handler for a message. */
        Client.prototype.registerResponse = function (ev, response) {
            var _this = this;
            if (response) {
                var now = consts.time.now();
                ev.responseId = ev.name + this.nextId++;
                var ht = setTimeout(function () { return _this.handleResponse(ev.responseId, new Error('Timeout'), null); }, this.responseTimeout);
                this.responses[ev.responseId] = { tStart: now, cb: response, hTimeout: ht };
            }
        };
        /** Processes a recieved response. */
        Client.prototype.handleResponse = function (id, err, data) {
            if (id in this.responses) {
                var info = this.responses[id];
                clearTimeout(info.hTimeout);
                var time = consts.time.now() - info.tStart;
                var prefix = '[' + time + 'ms] Multicast: response[' + id + '] ';
                if (err) {
                    client.logger.error(consts.errorCode.Multicast, prefix + ' error: ' + err.message);
                }
                else if (time > consts.time.seconds(1.5)) {
                    client.logger.warn(prefix + ' recieved');
                }
                else {
                    client.logger.info(prefix + ' recieved');
                }
                info.cb(err, data);
                delete this.responses[id];
            }
            else {
                client.logger.error(consts.errorCode.Multicast, 'Multicast: response[' + id + '] not found');
            }
        };
        /** Manages clients connection to server. Will automatically attempt to reconnect on dropped connections. */
        Client.prototype.reconnect = function (attempt) {
            var _this = this;
            if (attempt === void 0) { attempt = 0; }
            client.logger.debug('Multicast: reconnect(' + attempt + ')');
            var socket = eio(this.server, this.options);
            socket.once('open', function () {
                client.logger.debug('Multicast: socket opened');
                // Succeeded
                attempt = 0;
                _this.socket = socket;
                _this.socket.on('message', function (data) {
                    // Message recieved
                    var ev = JSON.parse(data);
                    if (ev.name == 'response') {
                        // Handle response
                        if (ev.code < 400) {
                            _this.handleResponse(ev.responseId, null, ev.data);
                        }
                        else {
                            _this.handleResponse(ev.responseId, new Error(ev.data), null);
                        }
                    }
                    else if (ev.session && ev.session in _this.sessions) {
                        // Route session event
                        client.logger.debug('Multicast: RECEIVED ' + ev.name + '(' + (ev.session || 'null') + ')');
                        _this.sessions[ev.session].handleServerEvent(ev);
                    }
                    else {
                        client.logger.warn('Multicast: RECEIVED invalid message ' + ev.name + '(' + (ev.session || 'null') + ')');
                    }
                });
                _this.setState(2 /* connected */);
                _this.resumeSessions(function () {
                    _this.retryPending();
                });
            });
            socket.once('close', function (reason) {
                _this.socket = null;
                if (reason !== 'forced close') {
                    // Attempt reconnect
                    client.logger.debug('Multicast: socket closed');
                    var plan = _this.options.reconnectPlan;
                    if (plan && plan.length > attempt) {
                        if (_this.state == 2 /* connected */)
                            _this.setState(3 /* reconnecting */);
                        setTimeout(function () {
                            _this.reconnect(attempt + 1);
                        }, plan[attempt]);
                    }
                    else {
                        client.logger.debug('Multicast: disconnected');
                        _this.abortPending();
                        _this.setState(0 /* disconnected */);
                    }
                }
            });
        };
        /** Attemps to deliver any queued up events. */
        Client.prototype.retryPending = function () {
            if (this.pending.length) {
                client.logger.debug('Multicast: sending ' + this.pending.length + ' events');
                _.each(this.pending, function (v) {
                    v.retry(null);
                });
            }
        };
        /** Aborts delivery of any queued up events. */
        Client.prototype.abortPending = function () {
            if (this.pending.length) {
                var err = new Error('Aborted');
                _.each(this.pending, function (v) {
                    v.retry(err);
                });
                this.pending = [];
            }
        };
        /** Resumes any paused sessions once the client has reconnected to the server. */
        Client.prototype.resumeSessions = function (cb) {
            var _this = this;
            var list = _.map(this.sessions, function (v) { return v; });
            if (list.length) {
                async.each(list, function (session, next) {
                    client.logger.debug('Multicast: resuming session', session.sessionId);
                    _this.sendForSession(session.sessionId, 'resumeSession', { clientId: session.clientId }, function (err) {
                        // Check for error resuming session
                        if (err)
                            session.handleResumeError(err);
                        next(null);
                    });
                }, cb);
            }
            else {
                cb(null);
            }
        };
        return Client;
    })();
    proximity.Client = Client;
    /** Returns true if an object has a new version then the object it's replacing.  */
    function isNewer(newObj, oldObj) {
        return !!(newObj.version == 0 || !oldObj || !oldObj.version || newObj.version > oldObj.version);
    }
    proximity.isNewer = isNewer;
    /** Returns the first model in the set if their is one. */
    function firstModel(models) {
        return models && models.length ? models[0] : null;
    }
    proximity.firstModel = firstModel;
    /** Gets a model returned given its ID. */
    function getModel(models, id) {
        var model;
        if (models) {
            for (var i = 0; i < models.length; i++) {
                if (models[i].id == id) {
                    model = models[i];
                    break;
                }
            }
        }
        return model;
    }
    proximity.getModel = getModel;
    /** Gets a model returned given its type. */
    function getModelByType(models, type) {
        var model;
        if (models) {
            for (var i = 0; i < models.length; i++) {
                if (models[i].type == type) {
                    model = models[i];
                    break;
                }
            }
        }
        return model;
    }
    proximity.getModelByType = getModelByType;
})(proximity || (proximity = {}));
/// <reference path='Client.ts' />
var proximity;
(function (proximity) {
    var Session = (function () {
        /**
          * Creates a new session instance.
          * @param client The clients connection to the server.
          * @param sessionId The ID of the session.
          * @param clientId The clients unique connection ID for the session.
          */
        function Session(client, sessionId, clientId) {
            this.client = client;
            this.sessionId = sessionId;
            this.clientId = clientId;
            /** Fired anytime an event is recieved from the server. */
            this.onServerEvent = new proximity.Event();
            /** Fired anytime an error occurs while resuming a session. */
            this.onResumeError = new proximity.Event();
            this.client.registerSession(this);
        }
        /** Disposes of the session instance. */
        Session.prototype.dispose = function () {
            if (this.client) {
                this.onServerEvent.remove();
                this.onResumeError.remove();
                this.client.unregisterSession(this.sessionId);
                this.client = null;
            }
        };
        /**
          * Temporarily pauses the session preventing it from sending messages or recieving events. All you can do is resume the paused session.
          * @param timeout Max number of milliseconds to pause before disconnecting the client from the session.
          * @param response (optional) Handler for response from server.
          */
        Session.prototype.pause = function (timeout, response) {
            if (this.client) {
                this.client.sendForSession(this.sessionId, 'pauseSession', { clientId: this.clientId, timeout: timeout }, response);
            }
            return this;
        };
        /**
          * Resumes a paused session.
          * @param response (optional) Handler for response from server.
          */
        Session.prototype.resume = function (response) {
            if (this.client) {
                this.client.sendForSession(this.sessionId, 'resumeSession', { clientId: this.clientId }, response);
            }
            return this;
        };
        /**
          * Sends an event to the server in the context of the session and optionally waits for a response.
          * @param name Name of the event.
          * @param data (optional) Even data.
          * @param response (optional) Handler for response from server.
          */
        Session.prototype.send = function (name, data, response) {
            if (this.client) {
                this.client.sendForSession(this.sessionId, name, data, response);
            }
            return this;
        };
        /** Sends an unjoin event and disposes of the session instance. */
        Session.prototype.unjoin = function () {
            if (this.client) {
                this.client.sendForSession(this.sessionId, 'unjoinSession');
            }
            this.dispose();
        };
        Session.prototype.handleServerEvent = function (ev) {
            this.onServerEvent.trigger(ev);
        };
        Session.prototype.handleResumeError = function (err) {
            this.onResumeError.trigger(err);
            this.dispose();
        };
        /** Sends a createSession event to server. */
        Session.createSession = function (client, data, response) {
            client.send('createSession', data, response);
        };
        /** Sends a joinSession event to server. */
        Session.joinSession = function (client, data, response) {
            client.send('joinSession', data, response);
        };
        /** Sends a pairWithSession event to server. */
        Session.pairWithSession = function (client, data, response) {
            client.send('pairWithSession', data, response);
        };
        return Session;
    })();
    proximity.Session = Session;
})(proximity || (proximity = {}));
/// <reference path='Session.ts' />
/// <reference path='../shared/models/LocalTable.ts' />
/// <reference path='../../types/async/async.d.ts' />
var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
var proximity;
(function (proximity) {
    var localSettingsTable = new client.LocalTable('settings', { missingOk: true });
    var defaultLocalSettings = {
        newestGroup: 1,
        groupsByDate: [],
        partialGroups: []
    };
    var defaultUserSettings = {
        discovery: {
            setting: consts.NearbyDiscoveryMode.incognito,
            timestamp: consts.time.now()
        }
    };
    var UserSession = (function (_super) {
        __extends(UserSession, _super);
        function UserSession() {
            _super.apply(this, arguments);
            /** Number of nearby users to simulate. */
            this.simulatedUsers = 0;
            //=======================================
            // Events
            //=======================================
            /** Fired anytime the users profile changes. */
            this.onProfileUpdated = new proximity.Event();
            /** Fired anytime the users settings change. */
            this.onSettingsUpdated = new proximity.Event();
            /** Fired anytime the users list of nearby profiles changes. */
            this.onNearbyUpdated = new proximity.Event();
            /** Fired anytime there's activity around a users groups. */
            this.onGroupsUpdated = new proximity.Event();
        }
        //=======================================
        // Methods
        //=======================================
        UserSession.prototype.dispose = function () {
            this.onProfileUpdated.remove();
            this.onSettingsUpdated.remove();
            this.onNearbyUpdated.remove();
            this.onGroupsUpdated.remove();
            _super.prototype.dispose.call(this);
        };
        /** Public clear cached groups */
        UserSession.prototype.clearCachedGroups = function (userId) {
            this.localSettings = _.extend({ id: userId }, defaultLocalSettings);
            // Persist index changes
            this.updateLocalSettings({
                newestGroup: this.localSettings.newestGroup,
                groupsByDate: this.localSettings.groupsByDate,
                partialGroups: this.localSettings.partialGroups
            });
        };
        /** Updates the devices local group table. */
        UserSession.prototype.cacheGroups = function (profiles) {
            var _this = this;
            if (profiles && profiles.length) {
                this.ensureGroupsTable();
                // Cache groups & update indexes
                var ids = [];
                var newest = this.localSettings.newestGroup;
                var groupsByDate = this.localSettings.groupsByDate;
                var partialGroups = this.localSettings.partialGroups;
                _.each(profiles, function (p) {
                    if (p && p.id && p.lastActivity) {
                        ids.push(p.id);
                        var idx = p.id + ':' + p.lastActivity.toString();
                        if (p.lastActivity > newest)
                            newest = p.lastActivity;
                        if (!p.isPartial) {
                            var pos = _this.searchGroupIndex(groupsByDate, p.id);
                            if (pos >= 0) {
                                groupsByDate[pos] = idx;
                            }
                            else {
                                groupsByDate.push(idx);
                            }
                            _this.groupsTable.replace(p, true);
                        }
                        else {
                            var pos = _this.searchGroupIndex(partialGroups, p.id);
                            if (pos >= 0) {
                                partialGroups[pos] = idx;
                            }
                            else {
                                partialGroups.push(idx);
                            }
                        }
                    }
                    else {
                        // Invalid group profile
                        client.logger.error(consts.errorCode.Connection, 'UserSession.cacheGroups invalid group: ' + (p ? (p.id ? 'missing lastActivity' : 'missing id') : 'missing profile'));
                    }
                });
                // Persist index changes
                this.updateLocalSettings({
                    newestGroup: newest,
                    groupsByDate: this.sortGroupIndex(groupsByDate),
                    partialGroups: partialGroups
                });
                this.onGroupsUpdated.trigger(ids);
            }
        };
        /** Returns one or more of the users groups from the local cache. */
        UserSession.prototype.getCachedGroups = function (ids, response) {
            var _this = this;
            this.ensureGroupsTable();
            var profiles = [];
            async.forEachSeries(ids || [], function (id, next) {
                _this.groupsTable.get(id, function (err, p) {
                    if (p)
                        profiles.push(p);
                    next(err);
                });
            }, function (err) { return response(err, profiles); });
        };
        /** Returns one or more of the users groups given their ID. */
        UserSession.prototype.getGroups = function (ids, response) {
            var _this = this;
            this.send('getGroups', { ids: ids }, function (err, profiles) {
                _this.cacheGroups(profiles);
                response(err, profiles);
            });
        };
        /** Returns a list of the users groups from the local cache filtered by time. */
        UserSession.prototype.listCachedGroups = function (options, response) {
            // Get ID's
            var ids = [];
            var index = this.localSettings.groupsByDate;
            for (var i = 0; i < index.length; i++) {
                var parts = index[i].split(':', 2);
                if (options) {
                    var ts = parseInt(parts[1]);
                    if ((!options.after || ts > options.after) && (!options.before || ts < options.before))
                        ids.push(parts[0]);
                    if (options.limit && ids.length == options.limit)
                        break;
                }
                else {
                    ids.push(parts[0]);
                }
            }
            // Lookup groups
            this.getCachedGroups(ids, response);
        };
        /** Returns a list of the users groups filtered by time. */
        UserSession.prototype.listGroups = function (options, response) {
            var _this = this;
            this.send('listGroups', options || {}, function (err, profiles) {
                _this.cacheGroups(profiles);
                response(err, profiles);
            });
        };
        /** Initializes simulated users for the session. */
        UserSession.prototype.setSimulatedUser = function (cnt) {
            this.simulatedUsers = cnt;
            this.addSimulatedUsersToNearby();
        };
        /** Updates the users local settings. */
        UserSession.prototype.updateLocalSettings = function (update) {
            _.extend(this.localSettings, update);
            localSettingsTable.replace(this.localSettings, true);
        };
        /** Updates the users nearby matches. */
        UserSession.prototype.updateLocators = function (locators, ad) {
            this.send('updateLocators', { locators: locators, ad: ad });
        };
        /** Updates the users profile. */
        UserSession.prototype.updateProfile = function (transforms) {
            this.send('profileUpdate', transforms);
        };
        /** Updates the users settings. */
        UserSession.prototype.updateSettings = function (transforms) {
            this.send('settingsUpdate', transforms);
        };
        //=======================================
        // Session overrides
        //=======================================
        UserSession.prototype.handleServerEvent = function (ev) {
            switch (ev.name) {
                case 'profileUpdated':
                    this.profile = ev.data;
                    this.onProfileUpdated.trigger(this.profile);
                    break;
                case 'settingsUpdated':
                    this.settings = _.extend({}, defaultUserSettings, ev.data);
                    this.onSettingsUpdated.trigger(this.settings);
                    break;
                case 'nearbyUpdated':
                    this.nearby = ev.data;
                    client.logger.info('nearby updated: ' + JSON.stringify(this.nearby));
                    this.addSimulatedUsersToNearby();
                    this.onNearbyUpdated.trigger(this.nearby);
                    break;
                case 'newGroup':
                case 'updatedGroup':
                    this.cacheGroups([ev.data]);
                    break;
            }
            _super.prototype.handleServerEvent.call(this, ev);
        };
        //=======================================
        // Private helpers
        //=======================================
        UserSession.prototype.addSimulatedUsersToNearby = function () {
            for (var i = 1; i <= this.simulatedUsers; i++) {
                var userId = 'sim!user' + i;
                this.nearby.users[userId] = {
                    version: 0,
                    id: userId,
                    type: 'user',
                    userId: userId,
                    displayName: 'User Number' + i
                };
            }
        };
        UserSession.prototype.ensureGroupsTable = function () {
            if (!this.groupsTable) {
                this.groupsTable = new client.LocalTable(this.profile.userId + ':groups', { missingOk: true });
            }
        };
        UserSession.prototype.searchGroupIndex = function (index, id) {
            id += ':';
            for (var i = 0; i < index.length; i++) {
                if (index[i].indexOf(id) == 0)
                    return i;
            }
            return -1;
        };
        UserSession.prototype.sortGroupIndex = function (index) {
            return index.sort(function (a, b) {
                var tsA = parseInt(a.split(':', 2)[1]);
                var tsB = parseInt(b.split(':', 2)[1]);
                return tsB - tsA;
            });
        };
        //=======================================
        // Static methods
        //=======================================
        UserSession.join = function (client, accessToken, userId, cb) {
            // Get user local settings
            localSettingsTable.get(userId, function (err, localSettings) {
                if (!localSettings)
                    localSettings = _.extend({ id: userId }, defaultLocalSettings);
                proximity.Session.joinSession(client, { accessToken: accessToken, sessionId: 'user:' + userId, groupsAfter: localSettings.newestGroup }, function (err, response) {
                    if (!err) {
                        var us = new UserSession(client, response.sessionId, response.clientId);
                        us.profile = response.profile;
                        us.nearby = response.nearby;
                        us.settings = _.extend({}, defaultUserSettings, response.settings);
                        us.localSettings = localSettings;
                        us.cacheGroups(response.groups);
                        cb(null, us);
                    }
                    else {
                        cb(err, null);
                    }
                });
            });
        };
        return UserSession;
    })(proximity.Session);
    proximity.UserSession = UserSession;
})(proximity || (proximity = {}));
/// <reference path='Session.ts' />
var proximity;
(function (proximity) {
    var GroupSession = (function (_super) {
        __extends(GroupSession, _super);
        function GroupSession() {
            _super.apply(this, arguments);
            //=======================================
            // Events
            //=======================================
            /** Fired anytime the groups profile changes. */
            this.onProfileUpdated = new proximity.Event();
            /** Fired anytime a model has been added to the group. */
            this.onModelAdded = new proximity.Event();
            /** Fired anytime a model within the group has been changed. */
            this.onModelUpdated = new proximity.Event();
            /** Fired anytime a model has been removed from the group. */
            this.onModelRemoved = new proximity.Event();
        }
        //=======================================
        // Methods
        //=======================================
        /** Retrieves one or more models from the sessions cache. */
        GroupSession.prototype.cacheGet = function (ids, response) {
            this.send('cacheGet', { ids: ids }, response);
        };
        /** Retrieves all models of a particular type from the sessions cache. */
        GroupSession.prototype.cacheList = function (type, response) {
            this.send('cacheList', { type: type }, response);
        };
        /** Inserts or replaces a model within the sessions cache. */
        GroupSession.prototype.cachePut = function (model, expire, response) {
            this.send('cachePut', { model: model, expire: expire }, response);
        };
        /** Updates an existing model within the sessions cache. */
        GroupSession.prototype.cacheUpdate = function (id, transforms, upsert, response) {
            this.send('cacheUpdate', { id: id, transforms: transforms, upsert: upsert }, response);
        };
        /** Removes a model from the sessions cache. */
        GroupSession.prototype.cacheRemove = function (id, response) {
            this.send('cacheRemove', { id: id }, response);
        };
        /** Writes a new post to the group. */
        GroupSession.prototype.createPost = function (post, response) {
            this.send('createPost', post, response);
        };
        /** Ends the current cast. */
        GroupSession.prototype.endCast = function (response) {
            this.send('endCast', null, response);
        };
        /** Publishes a notification to the group using the system registered template. */
        GroupSession.prototype.groupNotify = function (properties, response) {
            this.send('groupNotify', properties, response);
        };
        /** Adds additional users to the group. */
        GroupSession.prototype.inviteUsers = function (invites, response) {
            this.send('inviteUsers', { invites: invites }, response);
        };
        /** Joins an existing screen cast. */
        GroupSession.prototype.joinCast = function (data, response) {
            this.send('joinCast', data, response);
        };
        /** Perminately removes the user from the group. */
        GroupSession.prototype.leaveGroup = function (response) {
            this.send('leaveGroup', null, response);
        };
        /** Starts a new screen cast. */
        GroupSession.prototype.startCast = function (data, response) {
            this.send('startCast', data, response);
        };
        /** Retrieves one or more models from the sessions table. */
        GroupSession.prototype.tableGet = function (ids, response) {
            this.send('tableGet', { ids: ids }, response);
        };
        /** Retrieves all models of a particular filter from the sessions table. */
        GroupSession.prototype.tableList = function (filter, response) {
            this.send('tableList', filter, response);
        };
        /** Inserts one or more model(s) into the sessions table. */
        GroupSession.prototype.tableAdd = function (models, response) {
            this.send('tableAdd', { models: models }, response);
        };
        /** Updates an existing model within the sessions table. */
        GroupSession.prototype.tableUpdate = function (id, transforms, upsert, response) {
            this.send('tableUpdate', { id: id, transforms: transforms, upsert: upsert }, response);
        };
        /** Removes one or more models from the sessions table. */
        GroupSession.prototype.tableRemove = function (ids, activity, response) {
            this.send('tableRemove', { ids: ids, activity: activity }, response);
        };
        //=======================================
        // Session overrides
        //=======================================
        GroupSession.prototype.handleServerEvent = function (ev) {
            var _this = this;
            switch (ev.name) {
                case 'profileUpdated':
                    this.profile = ev.data;
                    this.onProfileUpdated.trigger(this.profile);
                    break;
                case 'tableAdd':
                case 'cachePut':
                    _.each(ev.data, function (m) {
                        _this.onModelAdded.trigger(m);
                    });
                    break;
                case 'tableUpdate':
                case 'cacheUpdate':
                    _.each(ev.data, function (m) {
                        _this.onModelUpdated.trigger(m);
                    });
                    break;
                case 'tableRemove':
                case 'cacheRemove':
                    _.each(ev.data, function (id) {
                        _this.onModelRemoved.trigger(id);
                    });
                    break;
            }
            _super.prototype.handleServerEvent.call(this, ev);
        };
        //=======================================
        // Private helpers
        //=======================================
        //=======================================
        // Static methods
        //=======================================
        GroupSession.create = function (client, data, cb) {
            data.sessionType = 'group';
            proximity.Session.createSession(client, data, function (err, response) {
                if (!err) {
                    var gs = new GroupSession(client, response.sessionId, response.clientId);
                    gs.profile = response.profile;
                    gs.cast = response.cast;
                    cb(null, gs);
                }
                else {
                    cb(err, null);
                }
            });
        };
        GroupSession.join = function (client, data, cb) {
            proximity.Session.joinSession(client, data, function (err, response) {
                if (!err) {
                    var gs = new GroupSession(client, response.sessionId, response.clientId);
                    gs.profile = response.profile;
                    cb(null, gs);
                }
                else {
                    cb(err, null);
                }
            });
        };
        GroupSession.joinXim = function (client, data, cb) {
            proximity.Session.joinSession(client, data, function (err, response) {
                if (!err) {
                    var gs = new GroupSession(client, response.sessionId, response.clientId);
                    gs.profile = response.profile;
                    cb(null, gs);
                }
                else {
                    cb(err, null);
                }
            });
        };
        return GroupSession;
    })(proximity.Session);
    proximity.GroupSession = GroupSession;
})(proximity || (proximity = {}));
/// <reference path='Session.ts' />
var proximity;
(function (proximity) {
    proximity.ViewerSessionScreen = {
        disconnect: 'disconnect',
        pairing: 'pairing',
        splash: 'splash',
        xim: 'xim'
    };
    var ViewerSession = (function (_super) {
        __extends(ViewerSession, _super);
        //=======================================
        // Methods
        //=======================================
        function ViewerSession(c, response) {
            _super.call(this, c, response.sessionId, response.clientId);
            //=======================================
            // Events
            //=======================================
            /** Fired anytime the screen should change. */
            this.onScreenChange = new proximity.Event();
            this.accessToken = response.accessToken;
            this.pinCode = response.pinCode;
            this.pinExpires = response.pinExpires ? consts.time.now() + response.pinExpires : 0;
            vs_updateScreen(this, response.viewerState);
        }
        /** Registers a new PIN code for pairing with the viewer. The viewer will automatically be switched to the pairing screen.  */
        ViewerSession.prototype.createViewerPin = function (cb) {
            var _this = this;
            this.send('createViewerPin', {}, function (err, response) {
                var success = false;
                if (response && response.pinCode) {
                    success = true;
                    _this.pinCode = response.pinCode;
                    _this.pinExpires = response.pinExpires ? consts.time.now() + response.pinExpires : 0;
                }
                cb(err, success);
            });
        };
        /** Switches the viewer to a new xim. */
        ViewerSession.prototype.joinXim = function (accessToken, groupId, cb) {
            this.send('joinXim', { accessToken: accessToken, groupId: groupId }, cb);
        };
        /** Switches the viewer to the spash screen. */
        ViewerSession.prototype.showSplashScreen = function (cb) {
            this.send('showSplashScreen', {}, cb);
        };
        /** Switches the viewer to the disconnect screen. */
        ViewerSession.prototype.showDisconnectScreen = function (cb) {
            this.send('showDisconnectScreen', {}, cb);
        };
        //=======================================
        // Session overrides
        //=======================================
        ViewerSession.prototype.dispose = function () {
            this.onScreenChange.remove();
            _super.prototype.dispose.call(this);
        };
        ViewerSession.prototype.handleServerEvent = function (ev) {
            switch (ev.name) {
                case 'stateUpdated':
                    if (proximity.isNewer(ev.data, this.viewerState)) {
                        vs_updateScreen(this, ev.data);
                    }
                    break;
            }
            _super.prototype.handleServerEvent.call(this, ev);
        };
        //=======================================
        // Private helpers
        //=======================================
        //=======================================
        // Static methods
        //=======================================
        /** Creates a new viewer instance and optionally registers a pin. */
        ViewerSession.create = function (client, createViewerPin, cb) {
            proximity.Session.createSession(client, { sessionType: 'viewer', createViewerPin: createViewerPin }, function (err, response) {
                if (!err) {
                    try {
                        var vs = new ViewerSession(client, response);
                        cb(null, vs);
                    }
                    catch (e) {
                        cb(e, null);
                    }
                }
                else {
                    cb(err, null);
                }
            });
        };
        /** Joins an existing viewer using an access token. */
        ViewerSession.join = function (client, data, cb) {
            proximity.Session.joinSession(client, data, function (err, response) {
                if (!err) {
                    try {
                        var vs = new ViewerSession(client, response);
                        cb(null, vs);
                    }
                    catch (e) {
                        cb(e, null);
                    }
                }
                else {
                    cb(err, null);
                }
            });
        };
        /** Pairs with an existign viewer given a code. */
        ViewerSession.pair = function (client, data, cb) {
            data.sessionType = 'viewer';
            proximity.Session.pairWithSession(client, data, function (err, response) {
                if (!err) {
                    try {
                        var vs = new ViewerSession(client, response);
                        cb(null, vs);
                    }
                    catch (e) {
                        cb(e, null);
                    }
                }
                else {
                    cb(err, null);
                }
            });
        };
        return ViewerSession;
    })(proximity.Session);
    proximity.ViewerSession = ViewerSession;
    //=====================================================
    // Protected ViewrSession methods
    //=====================================================
    function vs_updateScreen(vs, newState) {
        var changed = false;
        if (vs.viewerState) {
            if (vs.viewerState.screen !== newState.screen)
                changed = true;
            if (vs.viewerState.screen == 'xim' && (vs.viewerState.joinXimToken !== newState.joinXimToken || vs.viewerState.joinXimId !== newState.joinXimId))
                changed = true;
        }
        else {
            changed = true;
        }
        vs.viewerState = newState;
        if (changed)
            vs.onScreenChange.trigger(vs.viewerState.screen);
    }
})(proximity || (proximity = {}));
/// <reference path='types.ts'/>
/// <reference path='app.ts'/>
var client;
(function (client) {
    var server;
    (function (server) {
        // Data centers
        server.dataCenters = {
            server: '*',
            int: 'https://xim-int.cloudapp.net/',
            america: 'https://www.getxim.com/',
            europe: 'https://europe.getxim.com/',
            asia: 'https://asia.getxim.com/'
        };
        /** Current data center name. */
        server.dataCenterName;
        /** Current server root. */
        server.root;
        /** Sets the clients current data center. */
        function setDataCenter(name) {
            // Initialize server root
            server.dataCenterName = name;
            server.root = server.dataCenters[name];
            if (server.root == '*') {
                server.root = location.protocol + '//' + location.hostname + (location.port && location.port !== '80' ? ':' + location.port + '/' : '/');
            }
            client.logger.debug('server.root = ', server.root);
            // Set up client.consts.cloudProviders.onedrive based on data center selected.
            if (consts.cloudProviders.onedrive.hasOwnProperty(name)) {
                consts.cloudProviders.onedrive.appKey = consts.cloudProviders.onedrive[name].appKey;
            }
            else {
                switch (name) {
                    case 'int':
                        consts.cloudProviders.onedrive.appKey = consts.cloudProviders.onedrive.int.appKey;
                        break;
                    case 'server':
                        consts.cloudProviders.onedrive.appKey = consts.cloudProviders.onedrive.dev.appKey;
                        break;
                    default:
                        consts.cloudProviders.onedrive.appKey = consts.cloudProviders.onedrive.prod.appKey;
                        break;
                }
            }
            client.logger.info('onedrive appId', consts.cloudProviders.onedrive.appKey);
        }
        server.setDataCenter = setDataCenter;
        /** Calls the server using a GET. */
        function GET(path, params, callback) {
            ajax(server.root + path + toQueryString(params), {
                type: 'GET'
            }, callback);
        }
        server.GET = GET;
        /** Calls the server using a POST. */
        function POST(path, params, data, callback) {
            ajax(server.root + path + toQueryString(params), {
                type: 'POST',
                contentType: 'application/json',
                data: JSON.stringify(data)
            }, callback);
        }
        server.POST = POST;
        /** Calls the server using a PUT. */
        function PUT(path, params, data, callback) {
            ajax(server.root + path + toQueryString(params), {
                type: 'PUT',
                contentType: 'application/json',
                data: JSON.stringify(data)
            }, callback);
        }
        server.PUT = PUT;
        /** Calls the server using a PATCH. */
        function PATCH(path, params, data, callback) {
            ajax(server.root + path + toQueryString(params), {
                type: 'PATCH',
                contentType: 'application/json',
                data: JSON.stringify(data)
            }, callback);
        }
        server.PATCH = PATCH;
        /** Calls the server using a DELETE. */
        function DELETE(path, params, callback) {
            ajax(server.root + path + toQueryString(params), {
                type: 'DELETE'
            }, callback);
        }
        server.DELETE = DELETE;
        /** Returns an reason for why a request failed. */
        function getRequestStatus(err) {
            if (err) {
                switch (err['code']) {
                    case 400:
                        return 1 /* badRequest */;
                    case 401:
                        return 2 /* notAuthorized */;
                    case 403:
                        return 3 /* forbidden */;
                    case 404:
                    case 451:
                        return 4 /* notFound */;
                    case 426:
                        return 5 /* upgradeRequired */;
                    case 460:
                        return 9 /* waitListed */;
                    case 498:
                        return 6 /* tokenExpired */;
                    case 500:
                        return 7 /* serverError */;
                    default:
                        return 10 /* otherError */;
                }
            }
            return 0 /* succeeded */;
        }
        server.getRequestStatus = getRequestStatus;
        (function (RequestStatus) {
            RequestStatus[RequestStatus["succeeded"] = 0] = "succeeded";
            RequestStatus[RequestStatus["badRequest"] = 1] = "badRequest";
            RequestStatus[RequestStatus["notAuthorized"] = 2] = "notAuthorized";
            RequestStatus[RequestStatus["forbidden"] = 3] = "forbidden";
            RequestStatus[RequestStatus["notFound"] = 4] = "notFound";
            RequestStatus[RequestStatus["upgradeRequired"] = 5] = "upgradeRequired";
            RequestStatus[RequestStatus["tokenExpired"] = 6] = "tokenExpired";
            RequestStatus[RequestStatus["serverError"] = 7] = "serverError";
            RequestStatus[RequestStatus["connectionError"] = 8] = "connectionError";
            RequestStatus[RequestStatus["waitListed"] = 9] = "waitListed";
            RequestStatus[RequestStatus["otherError"] = 10] = "otherError";
        })(server.RequestStatus || (server.RequestStatus = {}));
        var RequestStatus = server.RequestStatus;
        /** Helper to simplify processing request errors. */
        function ifRequestSucceeded(success, callback) {
            return function (err, r) {
                if (!err) {
                    success(r);
                }
                else {
                    callback(err, getRequestStatus(err));
                }
            };
        }
        server.ifRequestSucceeded = ifRequestSucceeded;
        function ajax(url, settings, callback) {
            $.ajax(url, $.extend(settings, {
                cache: false,
                dataType: 'json',
                success: function (data, status, jqXHR) {
                    callback(null, data);
                },
                error: function (jqXHR, status, err) {
                    var e = new Error(jqXHR.statusText || status);
                    e['code'] = jqXHR.status;
                    callback(e, null);
                }
            }));
        }
        server.ajax = ajax;
        function toQueryString(params) {
            // Ensure that we always at least send the clients version and os
            params = params || {};
            params['version'] = consts.applicationVersion;
            params['os'] = client.StartupMode[client.startupMode];
            if (client.user.accessToken)
                params['accessToken'] = client.user.accessToken.token;
            return client.app.toQueryString(params);
        }
        server.toQueryString = toQueryString;
    })(server = client.server || (client.server = {}));
})(client || (client = {}));
/// <reference path='types.ts'/>
/// <reference path='../proximity/UserSession.ts'/>
/// <reference path='../proximity/GroupSession.ts'/>
/// <reference path='../proximity/ViewerSession.ts'/>
/// <reference path='server.ts'/>
var client;
(function (client) {
    /** Disposable data center connection. */
    var Connection = (function () {
        //=================================================
        // Construction & Destruction
        //=================================================
        /**
          * Creates a new connection to a data center.
          * @param dataCenterName (optional) Name of the data center the connection is for. If missing the clients default data center will be used.
          */
        function Connection(dataCenterName) {
            this.dataCenterName = dataCenterName;
            /** If true the connection has been disposed of. */
            this.isDisposed = false;
            /** Raised when the state changes for the connections socket. */
            this.onSocketStateChanged = new proximity.Event();
            if (!this.dataCenterName)
                this.dataCenterName = client.server.dataCenterName;
            this.id = nextConnectionId++;
            // Set server root
            // - We'll map anything that starts with 'server' to localhost. This is for testing purposes where 
            //   we actually need multiple socket connects to the same server for our Unit Tests.
            if (this.dataCenterName.indexOf('server') == 0) {
                this.root = location.protocol + '//' + location.hostname + (location.port && location.port !== '80' ? ':' + location.port + '/' : '/');
            }
            else {
                this.root = client.server.dataCenters[this.dataCenterName];
            }
        }
        /** Terminates a connection. Any further calls to the connection will fail and all amaiting operation will be cancled. */
        Connection.prototype.dispose = function () {
            this.isDisposed = true;
            if (this.socket) {
                releaseSocket(this);
                this.socket = null;
            }
            if (this.socketConnected) {
                this.socketConnected.reset(); // Clean up any waiting listners
                this.socketConnected = null;
            }
        };
        //=================================================
        // REST Methods
        //=================================================
        /** Calls a server using a GET. */
        Connection.prototype.GET = function (path, params, cb) {
            this.ajax(path, params, {
                type: 'GET'
            }, cb);
        };
        /** Calls a server using a POST. */
        Connection.prototype.POST = function (path, params, data, cb) {
            this.ajax(path, params, {
                type: 'POST',
                contentType: 'application/json',
                data: JSON.stringify(data)
            }, cb);
        };
        /** Calls a server using a PUT. */
        Connection.prototype.PUT = function (path, params, data, cb) {
            this.ajax(path, params, {
                type: 'PUT',
                contentType: 'application/json',
                data: JSON.stringify(data)
            }, cb);
        };
        /** Calls a server using a PATCH. */
        Connection.prototype.PATCH = function (path, params, data, cb) {
            this.ajax(path, params, {
                type: 'PATCH',
                contentType: 'application/json',
                data: JSON.stringify(data)
            }, cb);
        };
        /** Calls a server using a DELETE. */
        Connection.prototype.DELETE = function (path, params, cb) {
            this.ajax(path, params, {
                type: 'DELETE'
            }, cb);
        };
        //=================================================
        // Socket Methods
        //=================================================
        /** Returns a connected socket. */
        Connection.prototype.getSocket = function (cb) {
            var _this = this;
            this.ifNotDisposed(function () {
                if (!_this.socket) {
                    _this.socket = addRefSocket(_this);
                    _this.socketConnected = new client.AsyncEvent();
                    if (_this.socket.state == 2 /* connected */)
                        _this.socketConnected.signal(null, _this.socket);
                }
                // Wait for socket to connect (will trigger immediately if connected)
                _this.socketConnected.wait(cb);
            }, cb);
            return this;
        };
        /** Joins the client to their user session. */
        Connection.prototype.joinUserSession = function (accessToken, userId, cb) {
            return this.ensureSocket(function (s) { return proximity.UserSession.join(s, accessToken, userId, cb); }, cb, 'joinUserSession');
        };
        /** Creates a new group session. */
        Connection.prototype.createGroupSession = function (data, cb) {
            return this.ensureSocket(function (s) { return proximity.GroupSession.create(s, data, cb); }, cb, 'createGroupSession');
        };
        /** Joins client to an existing group session. */
        Connection.prototype.joinGroupSession = function (data, cb) {
            return this.ensureSocket(function (s) { return proximity.GroupSession.join(s, data, cb); }, cb, 'joinGroupSession');
        };
        /** Joins client to an existing xim. */
        Connection.prototype.joinXim = function (data, cb) {
            return this.ensureSocket(function (s) { return proximity.GroupSession.joinXim(s, data, cb); }, cb, 'joinXim');
        };
        /** Creates a new viewer session. */
        Connection.prototype.createViewerSession = function (createViewerPin, cb) {
            return this.ensureSocket(function (s) { return proximity.ViewerSession.create(s, createViewerPin, cb); }, cb, 'createViewerSession');
        };
        /** Joins client to an existing viewer session. */
        Connection.prototype.joinViewerSession = function (data, cb) {
            return this.ensureSocket(function (s) { return proximity.ViewerSession.join(s, data, cb); }, cb, 'joinViewerSession');
        };
        /** Pairs client with an existing viewer session via a PIN code. */
        Connection.prototype.pairViewerSession = function (data, cb) {
            return this.ensureSocket(function (s) { return proximity.ViewerSession.pair(s, data, cb); }, cb, 'pairViewerSession');
        };
        //=================================================
        // Protected Methods
        //=================================================
        /** PROTECTED: processes a change to the state of the connections cached socket. */
        Connection.prototype._handleOnSocketStateChanged = function (ev) {
            if (this.socket) {
                switch (ev.state) {
                    case 2 /* connected */:
                        // Release anyone waiting for the socket to connect
                        this.socketConnected.signal(null, this.socket);
                        break;
                    case 0 /* disconnected */:
                        // Release anyone wiating for the socket to connect (ignored if already connected)
                        this.socketConnected.signal(new Error('Could not connect to data center'), null);
                        // Release reference
                        this.socket = null;
                        this.socketConnected = null;
                        break;
                }
                this.onSocketStateChanged.trigger(ev);
            }
        };
        //=================================================
        // Private Methods
        //=================================================
        Connection.prototype.ajax = function (path, params, settings, cb) {
            var _this = this;
            this.ifNotDisposed(function () {
                client.server.ajax(_this.fmtUrl(path, params), settings, function (err, result) {
                    if (!_this.isDisposed)
                        cb(err, result);
                });
            }, function (err) {
                client.logger.error('Connection', 'Connection.ajax: connection disposed - ' + settings.type + ' ' + path);
                cb(err, null);
            });
        };
        Connection.prototype.fmtUrl = function (path, params) {
            if (!path)
                return this.root + client.server.toQueryString(params);
            return (path.indexOf('//') == 0 || path.indexOf('http:') == 0 || path.indexOf('https:') == 0 ? path : this.root + path) + client.server.toQueryString(params);
        };
        Connection.prototype.ensureSocket = function (then, fail, method) {
            this.getSocket(function (err, s) {
                if (!err && s) {
                    try {
                        then(s);
                    }
                    catch (e) {
                        client.logger.error('Connection', 'Connection.' + method + ': exception - ' + e.toString());
                        fail(e, null);
                    }
                }
                else {
                    client.logger.error('Connection', 'Connection.' + method + ': error getting socket - ' + err.message);
                    fail(err, null);
                }
            });
            return this;
        };
        Connection.prototype.ifNotDisposed = function (then, fail) {
            if (!this.isDisposed) {
                then();
            }
            else if (fail) {
                fail(new Error('disposed'), null);
            }
        };
        return Connection;
    })();
    client.Connection = Connection;
    /** Immediately disconnects all open sockets. */
    function closeSockets() {
        for (var dc in sockets) {
            var s = sockets[dc];
            if (s && s.socket) {
                for (var id in s.connections) {
                    try {
                        s.connections[id]._handleOnSocketStateChanged({ state: 0 /* disconnected */ });
                    }
                    catch (e) {
                        client.logger.error('Connection', 'Connection.closeSockets: exception - ' + e.toString());
                    }
                }
                s.connections = {};
                // Dispose of socket and remove from cache
                s.socket.dispose();
                delete sockets[dc];
            }
        }
    }
    client.closeSockets = closeSockets;
    //=================================================
    // Internal state
    //=================================================
    var nextConnectionId = 1;
    var sockets = {};
    //=================================================
    // Internal helpers
    //=================================================
    /** Adds a reference for a connection to the socket for its data center. */
    function addRefSocket(c) {
        var s = sockets[c.dataCenterName];
        if (!s || !s.socket) {
            sockets[c.dataCenterName] = s = { socket: new proximity.Client(c.root), connections: {} };
            _dispatchSocketStateChanges(c.dataCenterName);
            s.socket.connect();
        }
        s.connections[c.id] = c;
        return s.socket;
    }
    /** Releases the connections reference from its socket. */
    function releaseSocket(c) {
        var s = sockets[c.dataCenterName];
        if (s && c.id in s.connections)
            delete s.connections[c.id];
    }
    function _dispatchSocketStateChanges(dc) {
        var s = sockets[dc];
        if (s && s.socket) {
            s.socket.onStateChanged.add(function (ev) {
                for (var id in s.connections) {
                    try {
                        s.connections[id]._handleOnSocketStateChanged(ev);
                    }
                    catch (e) {
                        client.logger.error('Connection', 'Connection._dispatchSocketStateChanges: exception - ' + e.toString());
                    }
                }
                if (ev.state == 0 /* disconnected */ && dc in sockets) {
                    s.socket.dispose();
                    delete sockets[dc];
                }
            });
        }
    }
})(client || (client = {}));
var nearcast;
(function (nearcast) {
    /** Strongly typed event class. */
    var Event = (function () {
        function Event() {
            // Private member vars
            this.listeners = [];
        }
        /**
          * Registers a new listener for the event.
          * @param listener The callback function to register.
          */
        Event.prototype.add = function (listener) {
            this.listeners.push(listener);
        };
        /** Returns true if there are any registered listeners for the event. */
        Event.prototype.hasListeners = function () {
            return this.listeners.length > 0;
        };
        /** Returns a copy of the listeners array. */
        Event.prototype.getListeners = function () {
            return this.listeners.slice(0);
        };
        /**
          * Unregisters a listener from the event.
          * @param listener (optional) Callback function that was registered. If missing then all listeners will be removed.
          */
        Event.prototype.remove = function (listener) {
            if (listener) {
                for (var i = 0; i < this.listeners.length; i++) {
                    if (this.listeners[i] === listener) {
                        this.listeners.splice(i, 1);
                        break;
                    }
                }
            }
            else {
                this.listeners = [];
            }
        };
        /** Ensures that events are never serialized when calling JSON.stringify(). */
        Event.prototype.toJSON = function () {
            return null;
        };
        /**
          * Invokes all of the listeners for this event.
          * @param a (optional) set of arguments to pass to listners.
          */
        Event.prototype.trigger = function (ev) {
            var context = {};
            var copy = this.listeners.slice(0);
            for (var i = 0; i < copy.length; i++) {
                copy[i].call(context, ev);
            }
        };
        return Event;
    })();
    nearcast.Event = Event;
})(nearcast || (nearcast = {}));
/// <reference path="../../types/engine.io/engine.io-client.d.ts"/>
/// <reference path="../../servers/shared/nearcast-schema.d.ts"/>
/// <reference path='../../types/jQuery/jQuery.d.ts'/>
/// <reference path='Event.ts'/>
/// <reference path='connection.ts'/>
var nearcast;
(function (nearcast) {
    /**
      * Returns a session object given its ID.
      * @param id ID of the session to lookup.
      */
    function findSession(id) {
        return sessions[id];
    }
    nearcast.findSession = findSession;
    var sessions = {};
    /**
      * Listens for incoming session events and routes them to the appropriate session object.
      * @param socket Server connection to communicate over.
      */
    function dispatchSessionEvents(socket) {
        curSocket = socket;
        curSocket.on('sessionMessage', function (ev) {
            if (ev.sessionId in sessions)
                sessions[ev.sessionId].handleSessionMessage(ev);
        });
    }
    nearcast.dispatchSessionEvents = dispatchSessionEvents;
    /** Deletes all of the current sessions */
    function deleteSessions() {
        nearcast.stats.sessions = 0;
        sessions = {};
    }
    nearcast.deleteSessions = deleteSessions;
    function compareProfile(x, y) {
        return x.userId == y.userId;
    }
    nearcast.compareProfile = compareProfile;
    var curSocket;
    /** mock session */
    var MockSession = (function () {
        function MockSession() {
            this.id = 'mockSession';
            this.properties = {};
            this.userCount = 0;
            this.clientCount = 0;
            this.users = {};
            this.onPropertiesChanged = new nearcast.Event();
            sessions['mockSession'] = this;
        }
        MockSession.prototype.updateProperties = function (transform) {
        };
        MockSession.prototype.handleSessionMessage = function (ev) {
        };
        MockSession.prototype.on = function (event, fn) {
        };
        MockSession.prototype.postMessage = function (ids, event, data) {
        };
        MockSession.prototype.broadcastMessage = function (event, data) {
        };
        MockSession.prototype.unjoin = function () {
        };
        return MockSession;
    })();
    nearcast.MockSession = MockSession;
    /** Session for communicating with other clients. */
    var Session = (function () {
        /**
          * Creates a new session object.
          * @param info Session info returned by the server.
          */
        function Session(info) {
            this.info = info;
            /** Fired anytime the  */
            this.onPropertiesChanged = new nearcast.Event();
            /** Number of clients in the session. */
            this.clientCount = 0;
            /** Session clients grouped by user. */
            this.users = {};
            /** Number of users in the session. */
            this.userCount = 0;
            this.handlers = {};
            // Cache session instances
            this.id = info.id;
            sessions[this.id] = this;
            // Initialize shared properties.
            this.properties = info.properties;
            this.logger = window.client.logger;
            // Sort clients
            this.clients = info.clients;
            for (var id in this.clients) {
                this.clientCount++;
                var client = this.clients[id];
                if (client.userId) {
                    // Update user map
                    var user = this.users[client.userId];
                    if (!user) {
                        user = this.users[client.userId] = { clients: {}, clientCount: 0 };
                        this.userCount++;
                    }
                    user.clients[client.id] = client;
                    user.clientCount++;
                }
            }
        }
        /**
          * Sends a message to one or more clients in the session.
          * @param ids List of clients to send the message to.
          * @param event Message type to send.
          * @param data (optional) Data to send with message.
          */
        Session.prototype.postMessage = function (ids, event, data) {
            curSocket.emit('postMessage', { sessionId: this.id, ids: ids, event: event, data: data });
        };
        /**
          * Broadcasts a message to all other clients in the session.
          * @param event Message type to send.
          * @param data (optional) Data to send with message.
          */
        Session.prototype.broadcastMessage = function (event, data) {
            curSocket.emit('broadcastMessage', { sessionId: this.id, event: event, data: data });
        };
        /**
          * Applies a set of transforms to the sessions shared properties.
          * @param transforms Set of property changes to apply
          */
        Session.prototype.updateProperties = function (transforms) {
            curSocket.emit('updateProperties', { sessionId: this.id, transforms: transforms });
        };
        /**
          * Invites one or more other users to the session.
          * @param users List of user ID's to send invite to.
          * @param type Type of session users are being invited to.
          */
        Session.prototype.inviteUsers = function (users, type) {
            curSocket.emit('inviteUsers', { sessionId: this.id, users: users, type: type });
        };
        /**
        * Update ambient locators seen by this current device
        * @param userId ID of the user to send invite to.
        * @param ambientLocators ambient locators that device is seeing
        * @param ad optional advertisement that will be broadcasted to other nearby devices by server
        */
        Session.prototype.updateLocators = function (ambientLocators, ad) {
            this.ambientLocators = ambientLocators;
            this.ad = ad;
            curSocket.emit('updateLocators', { sessionId: this.id, ambientLocators: ambientLocators, ad: ad });
        };
        /**
          * Registers a handler for an incoming message.
          * @param event Message type to handle.
          * @param fn Message handler function.
          */
        Session.prototype.on = function (event, fn) {
            var list = this.handlers[event];
            if (!list)
                list = this.handlers[event] = [];
            list.push(fn);
        };
        /**
          * Un-registers a message handler.
          * @param event Message type to un-register.
          * @param fn Message handler function.
          */
        Session.prototype.off = function (event, fn) {
            var list = this.handlers[event];
            if (list) {
                var pos = list.indexOf(fn);
                if (pos >= 0)
                    list.splice(pos, 1);
            }
        };
        /** Leaves the current session. */
        Session.prototype.unjoin = function () {
            if (this.id) {
                this.logger.info('nearcast: unjoining session(' + this.id + ')');
                curSocket.emit('unjoinSession', { sessionId: this.id });
                delete sessions[this.id];
                this.id = null;
            }
        };
        // LARS: Steve is this dead code?  It appears to be?
        Session.prototype.handleSessionMessage = function (ev) {
            try {
                switch (ev.event) {
                    case 'clientAdded':
                        var client = ev.data;
                        this.clients[client.id] = client;
                        this.clientCount++;
                        if (client.userId) {
                            // Update user map
                            var user = this.users[client.userId];
                            if (!user) {
                                user = this.users[client.userId] = { clients: {}, clientCount: 0 };
                                this.userCount++;
                            }
                            user.clients[client.id] = client;
                            user.clientCount++;
                        }
                        break;
                    case 'clientRemoved':
                        var client = ev.data;
                        if (client.userId && client.userId in this.users) {
                            // Update user map
                            var user = this.users[client.userId];
                            if (client.id in user.clients) {
                                delete user.clients[client.id];
                                user.clientCount--;
                                if (user.clientCount == 0) {
                                    delete this.users[client.userId];
                                    this.userCount--;
                                }
                            }
                        }
                        if (client.id in this.clients) {
                            delete this.clients[client.id];
                            this.clientCount--;
                        }
                        break;
                    case 'clientChanged':
                        // Mergse changes (this will automatically update user references as well)
                        var changes = ev.data;
                        if (changes.id in this.clients) {
                            $.extend(this.clients[changes.id], changes);
                        }
                        break;
                    case 'propertiesChanged':
                        for (var key in ev.data) {
                            var val = ev.data[key];
                            if (typeof val !== 'undefined') {
                                this.properties[key] = val;
                            }
                            else if (key in this.properties) {
                                delete this.properties[key];
                            }
                        }
                        this.onPropertiesChanged.trigger({ from: ev.from, data: ev.data });
                        break;
                    default:
                        // Dispatch external event
                        var list = this.handlers[ev.event];
                        if (list) {
                            var handlers = list.slice(0);
                            for (var i = 0; i < handlers.length; i++) {
                                handlers[i]({ from: ev.from, data: ev.data });
                            }
                        }
                        break;
                }
            }
            catch (e) {
                this.logger.info('Exception: ' + e.toString());
            }
        };
        /**
          * Joins an existing or creates a new session.
          * @param options Options used to control the session that's joined or created.
          * @param connected Callback to invoke once the session is connected.
          */
        Session.join = function (options, connected) {
            nearcast.socket.emit('joinSession', options, function (data) {
                var logger = window.client.logger;
                logger.info('nearcast: session(' + data.id + ') started');
                var session;
                if (data) {
                    session = new Session(data);
                }
                connected(session);
            });
        };
        return Session;
    })();
    nearcast.Session = Session;
})(nearcast || (nearcast = {}));
/// <reference path='Session.ts'/>
/// <reference path='../../types/underscore/underscore.d.ts' />
/// <reference path="../../types/engine.io/engine.io-client.d.ts"/>
var nearcast;
(function (nearcast) {
    /** Connection to nearcast server. */
    nearcast.socket;
    /** Information about the client. */
    nearcast.client;
    /** User session that tracks all of a users connected devices. */
    nearcast.userSession;
    /** Statistics for the nearcast pieces. */
    nearcast.stats = {
        connects: 0,
        sessions: 0
    };
    var logger = window.client.logger;
    /**
      * Connects the client to a nearcast server.
      * @param options The options used to control the connection to the server.
      * @param callback Called when the connect completes.  Can also be called again on reconnect.
      */
    function connect(options, callback) {
        // Connect socket and wire up event handlers.
        nearcast.client = options.client || {};
        var connected = false;
        nearcast.socket = eio(options.server, options.socketOptions);
        nearcast.socket.on('connect', function () {
            if (nearcast.socket) {
                nearcast.stats.connects++;
                var reconnecting = nearcast.stats.connects > 1;
                logger.info(reconnecting ? 'SOCKET: reconnecting' : 'SOCKET: connecting');
                // Delete any sessions
                nearcast.deleteSessions();
                // If we have a user sessions use the last published ad
                var locators = nearcast.userSession ? nearcast.userSession.ambientLocators : options.ambientLocators;
                var ad = nearcast.userSession ? nearcast.userSession.ad : options.advertisement;
                // Register client
                if (!nearcast.client.deviceId)
                    nearcast.client.deviceId = localStorage.getItem('nearcast-deviceId');
                if (!nearcast.client.type)
                    nearcast.client.type = 'device';
                logger.info('calling connectClient');
                nearcast.socket.emit('connectClient', { accessToken: options.accessToken, client: nearcast.client, ambientLocators: locators, advertisement: ad, simulatedUsers: options.simulatedUsers ? options.simulatedUsers : 0 }, function (info) {
                    if (info.authorized) {
                        logger.info(reconnecting ? 'SOCKET: reconnected' : 'SOCKET: connected');
                        // Update client info
                        nearcast.client.id = info.client.id;
                        if (info.client.deviceId !== nearcast.client.deviceId) {
                            nearcast.client.deviceId = info.client.deviceId;
                            localStorage.setItem('nearcast-deviceId', nearcast.client.deviceId);
                        }
                        // Setup session dispatcher
                        if (!reconnecting) {
                            nearcast.dispatchSessionEvents(nearcast.socket);
                        }
                        // Create user session
                        // TODO: check to see if locators.
                        nearcast.userSession = new nearcast.Session(info.userSession);
                        nearcast.userSession.ambientLocators = locators;
                        nearcast.userSession.ad = ad;
                        // Signal connect completed
                        callback(null, nearcast.userSession, reconnecting);
                    }
                    else {
                        logger.info('SOCKET: client not authorized');
                        callback(new Error('Not Authorized'), null, reconnecting);
                    }
                });
            }
            else {
                logger.info('SOCKET: connect on dead socket');
                callback(new Error('SOCKET: connect on dead socket'), null, reconnecting);
            }
        });
        nearcast.socket.on('connect_failed', function (err) {
            if (console)
                console.error('SOCKET: connection failed');
            callback(new Error('Connection Failed'), null, false);
        });
        nearcast.socket.on('reconnect_failed', function (err) {
            logger.info('SOCKET: reconnect failed');
            nearcast.userSession = null;
            nearcast.deleteSessions();
            callback(new Error('Reconnection Failed'), null, true);
        });
    }
    nearcast.connect = connect;
    /** Manually disconnects the client if it's connected. */
    function disconnect() {
        if (nearcast.socket) {
            nearcast.socket.close();
            nearcast.socket = null;
            nearcast.userSession = null;
            nearcast.deleteSessions();
        }
    }
    nearcast.disconnect = disconnect;
})(nearcast || (nearcast = {}));
/// <reference path='Session.ts'/>
var nearcast;
(function (nearcast) {
    /** Display that manages showing a second screen on the client. */
    var Display = (function () {
        /**
          * Initializes a new instance of a display.
          * @param selector jQuery selector for element to render screens to.
          * @param session Stevecast session to listen for 'show' events off.
          */
        function Display(selector, session) {
            var _this = this;
            this.session = session;
            /** Fired before a new screen is shown. */
            this.onShowScreen = new nearcast.Event();
            /** Fired after a screen has been shown. */
            this.onScreenShown = new nearcast.Event();
            /** Fired when a screen is about to be hidden. */
            this.onUnshowScreen = new nearcast.Event();
            /** Fired after a screen has been hidden. */
            this.onScreenHidden = new nearcast.Event();
            /** Fired anytime a screen message has been recieved from another client. */
            this.onScreenMessage = new nearcast.Event();
            this.$el = $(selector);
            session.on('showScreen', function (info) {
                _this.onShowScreen.trigger(info.data);
                _this.showScreen(info.data);
            });
            session.on('unshowScreen', function (info) {
                _this.onUnshowScreen.trigger(info.data);
                _this.unshowScreen(info.data);
            });
            session.on('screenMessage', function (message) {
                _this.onScreenMessage.trigger(message.data);
                _this.screenMessage(message.data);
            });
        }
        /**
          * Renders a screen to the display area.
          * @param info Information about the screen being shown.
          */
        Display.prototype.showScreen = function (info) {
            var _this = this;
            if (info.creator.id in this.session.clients) {
                // Hide current screen
                if (this.screen)
                    this.unshowScreen(this.screen);
                // Show new screen
                this.$frame = $('<iframe class="screen" src="' + encodeURI(info.screenUrl) + '" frameborder="0" scrolling="no"></iframe>');
                this.$el.append(this.$frame).addClass('fullscreen');
                this.target = this.$frame[0].contentWindow;
                this.screen = info;
                // Listen for messages
                screens.push({
                    target: this.target,
                    listner: function (ev) {
                        if (typeof ev.data === 'string' && ev.data.substr(0, 1) == '{' && ev.data.substr(-1) == '}') {
                            // Parse and validate message.
                            var message = JSON.parse(event.data);
                            if (message && message.type) {
                                switch (message.type) {
                                    case 'ready':
                                        // Call screen.showScreen() method.
                                        _this.screenMessage({
                                            event: 'showScreen',
                                            from: info.creator,
                                            data: info.data
                                        });
                                        break;
                                    case 'postMessage':
                                        _this.session.postMessage(message.ids, 'screenMessage', {
                                            from: nearcast.client,
                                            event: message.event,
                                            data: message.data
                                        });
                                        break;
                                    case 'broadcastMessage':
                                        _this.session.broadcastMessage('screenMessage', {
                                            from: nearcast.client,
                                            event: message.event,
                                            data: message.data
                                        });
                                        break;
                                }
                            }
                        }
                    }
                });
            }
        };
        /**
          * Removes the screen currently being displayed.
          * @param info Information about the screen being shown.
          */
        Display.prototype.unshowScreen = function (info) {
            if (this.screen && this.screen.creator.id === info.creator.id) {
                this.$el.removeClass('fullscreen');
                if (this.$frame) {
                    var frame = this.$frame[0];
                    try {
                        var frameDoc = frame.contentDocument || frame.contentWindow.document;
                        frameDoc.removeChild(frameDoc.documentElement);
                    }
                    catch (e) {
                    }
                    frame.parentNode.removeChild(frame);
                    delete this.$frame;
                }
                if (this.target) {
                    for (var i = 0; i < screens.length; i++) {
                        if (screens[i].target === this.target) {
                            screens.splice(i, 1);
                            break;
                        }
                    }
                    delete this.target;
                }
                this.onScreenHidden.trigger(this.screen);
                delete this.screen;
            }
        };
        /**
          * Passes the screen a message for processing.
          * @param message Message to forward to screen.
          */
        Display.prototype.screenMessage = function (message) {
            // Send the screen the message
            if (this.target) {
                var data = JSON.stringify(message);
                this.target.postMessage(data, '*');
            }
        };
        return Display;
    })();
    nearcast.Display = Display;
    // Global listner for window messages
    var screens = [];
    $(window).on('message', function (ev) {
        var event = ev.originalEvent;
        for (var i = 0; i < screens.length; i++) {
            if (screens[i].target === event.source) {
                screens[i].listner(event);
                break;
            }
        }
    });
})(nearcast || (nearcast = {}));
/// <reference path='connection.ts'/>
/// <reference path='Session.ts'/>
/// <reference path='Display.ts'/>
/// <reference path='../../types/cordova/userprofile.d.ts'/>
/// <reference path='../../types/cordova/nearby.d.ts'/>
/// <reference path='../shared/models/LocalTable.ts'/>
/// <reference path="../nearcast/nearcast.ts"/>
/// <reference path='types.ts'/>
/// <reference path='user.ts'/>
var NearBy;
(function (NearBy) {
    NearBy.Guids = {
        discoveryService: {
            uuid: '9FBD8E45-19ED-49C4-B2BA-8F5A29A8B486',
            characteristics: {
                proximity: '46B095BD-7CB1-46F2-8407-2C9E7C05654F'
            }
        }
    };
})(NearBy || (NearBy = {}));
var client;
(function (client) {
    var scanner;
    (function (scanner) {
        /** Default options for getCurrentPosition calls. */
        scanner.defaultPositionOptions = {
            enableHighAccuracy: false,
            timeout: consts.time.seconds(5),
            maximumAge: consts.time.seconds(15)
        };
        /** Overrides the devices GPS position. */
        scanner.locationOverride = localStorage.getItem('location-override') || '';
        /** Rate at which scanner polls. */
        scanner.watchInterval = consts.time.minutes(3);
        /** Fired anytime the a scan begins or ends. */
        scanner.onScanningState = new client.Event();
        /** Fired anytime the users position changes. */
        scanner.onPosition = new client.Event();
        /** Fired anytime the users locators have significantly changed.  */
        scanner.onLocatorChange = new client.Event();
        /** Last known position of the user. */
        scanner.lastPosition = null;
        /** Force triggering update **/
        scanner.forceUpdate = false;
        var locators = [];
        /** Begins advertising the current users profile via Bluetooth. */
        function startAdvertise(callback) {
            client.logger.info('scanner: startAdvertise');
            var sUUID = NearBy.Guids.discoveryService.uuid;
            if (window.nearby && window.nearby.advertise) {
                var characteristics;
                var profiles = [];
                if (client.user.profile) {
                    //populating profiles for background scanner service
                    if (client.user.settings.discovery.setting != consts.NearbyDiscoveryMode.incognito) {
                        var adProfile = {
                            type: 'user',
                            userId: client.user.profile.userId,
                            displayName: client.user.profile.displayName,
                            thumbnailUrl: client.user.profile.thumbnailUrl,
                            pictureUrl: client.user.profile.pictureUrl
                        };
                        adProfile = _.extend(adProfile, { access: client.user.settings.discovery.setting });
                        profiles = [adProfile];
                    }
                    characteristics = [{ uuid: NearBy.Guids.discoveryService.characteristics.proximity, value: _.pick(client.user.profile, 'id', 'displayName', 'type') }];
                }
                window.nearby.advertise(function (result) {
                    client.logger.info('advertising succeeded with: ' + result);
                    if (callback)
                        callback(null, sUUID);
                }, function (err) {
                    client.logger.error(consts.errorCode.Scanner, 'advertising error: ' + err);
                    if (callback)
                        callback(new Error('advertising error: ' + err), null);
                }, { uuid: sUUID, characteristics: characteristics, profiles: profiles });
            }
            else {
                client.logger.warn('no nearby plugin found');
                if (callback)
                    callback(null, null);
            }
        }
        scanner.startAdvertise = startAdvertise;
        /** Stops advertising the current profile via bluetooth. */
        function stopAdvertise(callback) {
            // TODO
            if (callback)
                callback(null, false);
        }
        scanner.stopAdvertise = stopAdvertise;
        /** Returns the age of the cached position locator.  If there isn't one cached then "Number.MAX_VALUE" is returned.*/
        function getPositionAge() {
            return (scanner.lastPosition && !scanner.forceUpdate) ? consts.time.age(scanner.lastPosition.timestamp) : Number.MAX_VALUE;
        }
        scanner.getPositionAge = getPositionAge;
        /**
          * Returns a list of the devices current locators.
          * @param filter (optional) Filters the type of locators returned.
          */
        function getLocators(filter) {
            if (filter === void 0) { filter = 0 /* all */; }
            var l = _.filter(locators, function (locator) {
                switch (filter) {
                    case 0 /* all */:
                    default:
                        return true;
                    case 1 /* ambient */:
                        return !!(!locator.selfIdentifier || locator.type === 'ssid' || locator.type === 'geo');
                    case 2 /* selfies */:
                        return !!(locator.selfIdentifier || locator.type === 'ssid' || locator.type === 'geo');
                }
            });
            if (scanner.lastPosition && (filter === 0 /* all */ || filter === 2 /* selfies */ || filter === 1 /* ambient */))
                l.push(scanner.lastPosition);
            return l;
        }
        scanner.getLocators = getLocators;
        (function (LocatorFilters) {
            LocatorFilters[LocatorFilters["all"] = 0] = "all";
            LocatorFilters[LocatorFilters["ambient"] = 1] = "ambient";
            LocatorFilters[LocatorFilters["selfies"] = 2] = "selfies";
        })(scanner.LocatorFilters || (scanner.LocatorFilters = {}));
        var LocatorFilters = scanner.LocatorFilters;
        /**
          * Requests the devices current position.
          * @param callback (optional) Async results of the operation.
          * @param options (optional) Position retrieval options to use.
          */
        function getCurrentPosition(callback, options) {
            function checkMove() {
                // Did the user move?
                var moveDist = movePos ? getDistance(movePos, scanner.lastPosition.value) : Number.MAX_VALUE;
                if (scanner.forceUpdate || consts.distance.kilometers.toFeet(moveDist) > 300) {
                    moveDist = scanner.lastPosition.value;
                    scanner.onPosition.trigger(scanner.lastPosition.value);
                }
                var locDist = locPos ? getDistance(locPos, scanner.lastPosition.value) : Number.MAX_VALUE;
                if (scanner.forceUpdate || consts.distance.kilometers.toMiles(locDist) > 0.25) {
                    locPos = scanner.lastPosition.value;
                    scanner.onLocatorChange.trigger(getLocators());
                }
            }
            client.logger.info('scanner: getCurrentPosition');
            callback = beginScan(callback);
            options = _.extend({}, scanner.defaultPositionOptions, options);
            if (scanner.locationOverride) {
                // Create locator
                var latLon = scanner.locationOverride.split(',');
                var now = consts.time.now();
                scanner.lastPosition = {
                    type: 'geo',
                    value: {
                        timestamp: now,
                        coords: {
                            latitude: parseFloat(latLon[0]),
                            longitude: parseFloat(latLon[1]),
                            accuracy: 50,
                            altitude: 0,
                            altitudeAccuracy: 0,
                            heading: 0,
                            speed: 0
                        }
                    },
                    timestamp: now,
                    selfIdentifier: true
                };
                checkMove();
                callback(null, scanner.lastPosition);
            }
            else if (getPositionAge() < options.maximumAge) {
                // Return cached position
                callback(null, scanner.lastPosition);
            }
            else {
                // Query for current position
                navigator.geolocation.getCurrentPosition(function (p) {
                    var pos = _.extend({}, p, { coords: _.extend({}, p.coords) });
                    // Create locator
                    scanner.lastPosition = {
                        type: 'geo',
                        value: pos,
                        timestamp: typeof pos.timestamp === 'number' ? pos.timestamp : consts.time.now(),
                        selfIdentifier: true
                    };
                    checkMove();
                    callback(null, scanner.lastPosition);
                }, function (err) {
                    switch (err.code) {
                        case err.PERMISSION_DENIED:
                            scanner.lastPosition = null;
                            break;
                        default:
                            // Delete cached position after 15 minutes
                            if (getPositionAge() > consts.time.minutes(15))
                                scanner.lastPosition = null;
                            break;
                    }
                    callback(new Error(err.message), scanner.lastPosition);
                }, options);
            }
        }
        scanner.getCurrentPosition = getCurrentPosition;
        var movePos;
        var locPos;
        /** True if a nearby scan is active. */
        scanner.inNearbyScan = false;
        /** Starts a new nearby scan. */
        function nearbyScan(callback) {
            client.logger.info('scanner: nearbyScan');
            callback = beginScan(callback);
            if (!scanner.inNearbyScan) {
                if (window.nearby && window.nearby.scan) {
                    scanner.inNearbyScan = true;
                    //keep the last geo location
                    locators = _.filter(locators, function (locator) {
                        return locator.type === 'geo';
                    });
                    window.nearby.scan(function (jsonText) {
                        var results = jsonText;
                        if (typeof jsonText === "string") {
                            results = JSON.parse(jsonText);
                        }
                        if (!results) {
                            //scan is done, sending the results 
                            // Signal changes
                            client.logger.info('scan is done');
                            scanner.inNearbyScan = false;
                            scanner.onLocatorChange.trigger(getLocators());
                            callback(null, locators);
                        }
                        else {
                            //client.logger.info('results: ' + JSON.stringify(results));
                            if (results instanceof Array) {
                                // Convert locator format
                                locators = locators.concat(_.map(results, function (result) {
                                    return {
                                        type: result.type,
                                        value: result.value,
                                        timestamp: consts.time.now(),
                                        selfIdentifier: result.selfIdentifier ? result.selfIdentifier : false
                                    };
                                }));
                            }
                            else {
                                locators.push({
                                    type: results.type,
                                    value: results.value,
                                    timestamp: consts.time.now(),
                                    selfIdentifier: results.selfIdentifier ? results.selfIdentifier : false
                                });
                            }
                        }
                    }, function (err) {
                        scanner.inNearbyScan = false;
                        client.logger.error(consts.errorCode.Scanner, 'scan error: ' + err);
                        callback(new Error('scan error: ' + err), null);
                    }, NearBy.Guids.discoveryService.uuid);
                }
                else {
                    callback(null, []);
                }
            }
            else if (callback) {
                callback(new Error('Scan in progress'), null);
            }
        }
        scanner.nearbyScan = nearbyScan;
        /** True if the scanner is watching. */
        scanner.isWatching = false;
        /** Starts the scanner watching for changes. */
        function startWatching() {
            client.logger.info('scanner: startWatching');
            if (!scanner.isWatching) {
                scanner.isWatching = true;
                update(); // Will scan for an update and then start timer.
            }
        }
        scanner.startWatching = startWatching;
        var watchId;
        /** Stops the scanner from watching for changes. */
        function stopWatching() {
            client.logger.info('scanner: stopWatching');
            scanner.isWatching = false;
            if (watchId) {
                clearTimeout(watchId);
                watchId = null;
            }
        }
        scanner.stopWatching = stopWatching;
        /** Immediately scans for updates. */
        function update() {
            // Cancel any pending watch call.
            if (watchId) {
                clearTimeout(watchId);
                watchId = null;
            }
            var cnt = 2;
            function completed() {
                if (--cnt == 0) {
                    if (scanner.isWatching) {
                        watchId = setTimeout(function () {
                            watchId = null;
                            update();
                        }, scanner.watchInterval);
                    }
                }
            }
            // Get a high accuracy update of the users position.
            getCurrentPosition(completed, { enableHighAccuracy: true, maximumAge: consts.time.seconds(30) });
            // Start a nearby scan
            if (!scanner.inNearbyScan) {
                nearbyScan(completed);
            }
            else {
                completed();
            }
        }
        scanner.update = update;
        /**
          * Calculates the distance between two points in kilometers using the "Spherical Law of Cosines" formula.
          * This approach is accurate down to around 1 meter and while not as accurate as the "Haversine" formula it's cheaper and good enough for our purposes.
          * @param pt1 The first lat/lon pair of coordinates to compare.
          * @param pt2 The second lat/lon pair of coordinates to compare.
          */
        function getDistance(pt1, pt2) {
            return Math.acos(Math.sin(pt1.coords.latitude) * Math.sin(pt2.coords.longitude) + Math.cos(pt1.coords.latitude) * Math.cos(pt2.coords.latitude) * Math.cos(pt2.coords.longitude - pt1.coords.longitude)) * 6371;
        }
        scanner.getDistance = getDistance;
        /** Called at start of a scan. The return result is a callback function which should be called at end of scan.   **/
        function beginScan(callback) {
            if (scanCount++ == 0)
                scanner.onScanningState.trigger(true);
            return function (err, results) {
                if (--scanCount == 0)
                    scanner.onScanningState.trigger(false);
                if (callback)
                    callback(err, results);
            };
        }
        var scanCount = 0;
    })(scanner = client.scanner || (client.scanner = {}));
})(client || (client = {}));
/// <reference path='types.ts'/>
var client;
(function (client) {
    var Util;
    (function (Util) {
        function generateGUID() {
            // The "4" specifies the UUID type (i.e. "Random")
            var base = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx';
            return base.replace(/[xy]/g, function (c) {
                var r = Math.random() * 16 | 0, v = c == 'x' ? r : (r & 0x3 | 0x8);
                return v.toString(16);
            });
        }
        Util.generateGUID = generateGUID;
        function cleanInput(document, element) {
            return Util.cleanString(document.getElementById(element).value);
        }
        Util.cleanInput = cleanInput;
        function cleanString(input) {
            if (input) {
                return input.trim().replace(/\</g, "").replace(/\>/g, "");
            }
            return input;
        }
        Util.cleanString = cleanString;
        /** Returns true is given text contains only letters (including international) */
        function isAllLetters(text) {
            if (text) {
                var regexlet = /([^\u00BF-\u1FFF\u2C00-\uD7FF\w])/;
                var regexnum = /([\d])/;
                return !regexlet.test(text) && !regexnum.test(text);
            }
            return false;
        }
        Util.isAllLetters = isAllLetters;
        /** check if user inputs valid name
        (at least 1 letter and optionally allow chars, no number, digits or other punctuation marks) */
        function isValidName(text) {
            // after stripping out allowed chars, check if the remaining is not empty and all letters
            return Util.isAllLetters(text.replace(/[-']/g, ''));
        }
        Util.isValidName = isValidName;
        /** Returns true if given string contains Chinese, Japanese or Korean Characters */
        function isCJK(text) {
            var regexCJK = /[\u3000-\u303f\u3040-\u309f\u30a0-\u30ff\uff00-\uff9f\u4e00-\u9faf\u3400-\u4dbf]/;
            return regexCJK.test(text);
        }
        Util.isCJK = isCJK;
        function toAlphaNumeric(text) {
            if (text) {
                return text.replace(/[^a-z0-9]/gi, '');
            }
            return text;
        }
        Util.toAlphaNumeric = toAlphaNumeric;
        /** Returns true is given text is a valid email */
        function validEmail(text) {
            if (text) {
                var regex = /^(([^<>()[\]\\.,;:\s@\"]+(\.[^<>()[\]\\.,;:\s@\"]+)*)|(\".+\"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
                return regex.test(text);
            }
            return false;
        }
        Util.validEmail = validEmail;
        /** Assumes number is valid, removes valid non-numric characters ")", "-", "+" and " " */
        function stripPhone(text) {
            if (text) {
                return text.replace(/[()+\-\s]/g, '');
            }
            return text;
        }
        Util.stripPhone = stripPhone;
        /** Returns true is given text is a valid Xim invite code*/
        function validCode(text) {
            if (text) {
                // Fail if less than 6 chars
                if (text.length < 6) {
                    return false;
                }
                // Check that first character is a "*"
                if (text[0] != "*") {
                    return false;
                }
                // Check that remaining is all alphabetic chars
                return (/^[a-zA-Z()]+$/.test(text.slice(1, 6)));
            }
            return false;
        }
        Util.validCode = validCode;
        function getRandomInt(max) {
            return Math.floor(Math.random() * (max + 1));
        }
        Util.getRandomInt = getRandomInt;
        /** Maps a set of changed properties to a transform set. */
        function toModelTransforms(changes) {
            var t = {};
            for (var k in changes) {
                var v = changes[k];
                if (!_.isNull(v) && !_.isUndefined(v)) {
                    if (!t.$set)
                        t.$set = {};
                    t.$set[k] = v;
                }
                else {
                    if (!t.$unset)
                        t.$unset = {};
                    t.$unset[k] = 0;
                }
            }
            return t;
        }
        Util.toModelTransforms = toModelTransforms;
        /** Hide placeholder input text on focus, restore on blur */
        function hidePlaceholderOnFocus($selector) {
            $selector.focus(function () {
                $(this).data('placeholder', $(this).attr('placeholder'));
                $(this).attr('placeholder', '');
            });
            $selector.blur(function () {
                $(this).attr('placeholder', $(this).data('placeholder'));
            });
        }
        Util.hidePlaceholderOnFocus = hidePlaceholderOnFocus;
        /** get the country code map, make it async so it can call the server for
           the data in the future  */
        var _countryCodeMap = null;
        function getCountryCodeMap(callback) {
            if (!_countryCodeMap) {
                _countryCodeMap = {};
                _.each(consts.iso3166_data, function (data) {
                    _countryCodeMap[data.alpha2] = {
                        shortName: data.alpha3,
                        displayName: data.country_name,
                        countryCode: data.country_code,
                        minNumberDigits: _.min(data.phone_number_lengths),
                        region: data.region
                    };
                });
            }
            _.defer(function () {
                callback(null, _countryCodeMap);
            });
        }
        Util.getCountryCodeMap = getCountryCodeMap;
    })(Util = client.Util || (client.Util = {}));
    /** Helper to simplify handling of async errors. */
    function ifSuccess(success, failure) {
        return function (err, result) {
            if (!err) {
                success(result);
            }
            else if (failure) {
                failure(err, false);
            }
        };
    }
    client.ifSuccess = ifSuccess;
})(client || (client = {}));
/// <reference path="Group.ts"/>
var client;
(function (client) {
    client.ScreenSyncMode = {
        synced: 'synced',
        presenter: 'presenter',
        free: 'free'
    };
    var ScreenCast = (function () {
        //=======================================
        // Methods
        //=======================================
        /** Creates a new instance of the screen cast. */
        function ScreenCast(group, cast) {
            this.group = group;
            //=======================================
            // Events
            //=======================================
            /** Fired anytime the casts settings change. */
            this.onSettingsUpdated = new client.Event();
            /** Fired anytime the casts queue changes. */
            this.onQueueUpdated = new client.Event();
            /** Fired anytime the casts screen state changes. */
            this.onStateUpdated = new client.Event();
            this.castId = cast.castId;
            this.castType = cast.castType;
            this.creatorId = cast.creatorId;
            this.created = cast.created;
            this.settings = proximity.getModelByType(cast.models, 'screenSettings');
            this.queue = proximity.getModelByType(cast.models, 'screenQueue');
            this.state = proximity.getModelByType(cast.models, 'screenState');
        }
        /** Disposes of the current casting object. */
        ScreenCast.prototype.dispose = function () {
            this.group = null;
        };
        /** Leaves the current cast. */
        ScreenCast.prototype.unjoin = function () {
            if (this.group)
                this.group.unjoinCast(this.castId);
        };
        /** Update the casts settings. */
        ScreenCast.prototype.updateSettings = function (transforms) {
            if (this.group)
                this.group.session.cacheUpdate(this.settings.id, transforms);
            return this;
        };
        /** Update the casts queue. */
        ScreenCast.prototype.updateQueue = function (transforms, response) {
            if (this.group) {
                this.group.session.cacheUpdate(this.queue.id, transforms, null, response);
            }
            else if (response) {
                response(new Error("Group not defined"), null);
            }
            return this;
        };
        /** Update the casts screen state. */
        ScreenCast.prototype.updateState = function (transforms) {
            if (this.group)
                this.group.session.cacheUpdate(this.state.id, transforms);
            return this;
        };
        //=======================================
        // Event routing
        //=======================================
        ScreenCast.prototype.handleModelUpdated = function (model) {
            switch (model.type) {
                case 'screenSettings':
                    if (proximity.isNewer(model, this.settings)) {
                        this.settings = model;
                        this.onSettingsUpdated.trigger(this.settings);
                    }
                    break;
                case 'screenQueue':
                    if (proximity.isNewer(model, this.queue)) {
                        this.queue = model;
                        this.onQueueUpdated.trigger(this.queue);
                    }
                    break;
                case 'screenState':
                    if (proximity.isNewer(model, this.state)) {
                        this.state = model;
                        this.onStateUpdated.trigger(this.state);
                    }
                    break;
            }
        };
        //=======================================
        // Static methods
        //=======================================
        ScreenCast.newSettings = function (options) {
            return _.extend({
                type: 'screenSettings',
                syncMode: client.ScreenSyncMode.synced,
                endIfCreatorUnjoins: true
            }, options);
        };
        ScreenCast.newQueue = function (options) {
            return _.extend({
                type: 'screenQueue',
                items: []
            }, options);
        };
        ScreenCast.newState = function (options) {
            return _.extend({
                type: 'screenState',
                queueIndex: 0
            }, options);
        };
        return ScreenCast;
    })();
    client.ScreenCast = ScreenCast;
})(client || (client = {}));
/// <reference path='Session.ts'/>
var proximity;
(function (proximity) {
    /** A synchronized list of models. */
    var ObservableList = (function () {
        /**
          * Creates a new synchronized list.
          * @param session The session the list is bound to.
          * @param filter Describes the source of the list.
          */
        function ObservableList(session, filter) {
            var _this = this;
            this.session = session;
            this.filter = filter;
            this.models = [];
            this.byId = {};
            /** Fired anytime a new model is added to the list. */
            this.onModelAdded = new proximity.Event();
            /** Fired anytime an existing model in the list is changed. */
            this.onModelUpdated = new proximity.Event();
            /** Fired anytime an existing model is removed from the list. */
            this.onModelRemoved = new proximity.Event();
            if (this.filter.fromCache) {
                this.listener = _.bind(function (ev) {
                    switch (ev.name) {
                        case 'cachePut':
                            _this.modelsAdded(ev.data);
                            break;
                        case 'cacheUpdate':
                            _this.modelsUpdated(ev.data);
                            break;
                        case 'cacheRemove':
                            _this.modelsRemoved(ev.data);
                            break;
                    }
                }, this);
            }
            else {
                this.listener = _.bind(function (ev) {
                    switch (ev.name) {
                        case 'tablePut':
                            _this.modelsAdded(ev.data);
                            break;
                        case 'tableUpdate':
                            _this.modelsUpdated(ev.data);
                            break;
                        case 'tableRemove':
                            _this.modelsRemoved(ev.data);
                            break;
                    }
                }, this);
            }
            this.session.onServerEvent.add(this.listener);
        }
        /** Stops synchronizing the list with the server */
        ObservableList.prototype.dispose = function () {
            if (this.session) {
                this.session.onServerEvent.remove(this.listener);
                this.session = null;
                this.filter = null;
            }
        };
        /** Synchronizes the list with the server. */
        ObservableList.prototype.sync = function (cb) {
            var _this = this;
            if (this.ensureSession(cb)) {
                var method = this.filter.fromCache ? 'cacheList' : 'tableList';
                this.session.send(method, this.filter, function (err, models) {
                    if (!err && models) {
                        _this.models = [];
                        _this.byId = {};
                        _.each(models, function (m) {
                            if (!_this.filter.custom || _this.filter.custom(m)) {
                                _this.models.push(m);
                                _this.byId[m.id] = m;
                            }
                        });
                        cb(null, true);
                    }
                    else {
                        cb(err, false);
                    }
                });
            }
            return this;
        };
        /** Enumerates all of the models in the list. */
        ObservableList.prototype.each = function (cb) {
            for (var i = 0; i < this.models.length; i++) {
                cb(this.models[i], i);
            }
            return this;
        };
        /** Returns the number of models in the list. */
        ObservableList.prototype.getCount = function () {
            return this.models.length;
        };
        /** Returns a model from the list given it's ID. */
        ObservableList.prototype.getModel = function (id) {
            return this.byId[id];
        };
        //=======================================
        // Private helpers
        //=======================================
        /** Makes sure we have a valid session. */
        ObservableList.prototype.ensureSession = function (cb) {
            if (!this.session) {
                if (cb)
                    cb(new Error('No session'), false);
                return false;
            }
            return true;
        };
        ObservableList.prototype.modelsAdded = function (models) {
            var _this = this;
            _.each(models, function (m) {
                if (m.type === _this.filter.type && !(m.id in _this.byId) && (!_this.filter.custom || _this.filter.custom(m))) {
                    // Insert model into head of list
                    // - This is optimized for date sorted lists which means the 
                    //   list could be out of order for random access lists.  We
                    //   don't think this will be an issue because in those cases
                    //   you'd likely retrieve models by their ID.
                    var pos = 0;
                    _this.models.splice(0, 0, m);
                    _this.byId[m.id] = m;
                    _this.onModelAdded.trigger(m);
                }
            });
        };
        ObservableList.prototype.modelsUpdated = function (models) {
            var _this = this;
            _.each(models, function (m) {
                if (m.id in _this.byId && proximity.isNewer(m, _this.byId[m.id])) {
                    // Replace existing model
                    var pos = 0;
                    while (pos < _this.models.length && m.id !== _this.models[pos].id)
                        pos++;
                    _this.models[pos] = _this.byId[m.id] = m;
                    _this.onModelUpdated.trigger(m);
                }
            });
        };
        ObservableList.prototype.modelsRemoved = function (ids) {
            var _this = this;
            _.each(ids, function (id) {
                var model = _this.byId[id];
                if (model) {
                    // Remove model from list
                    _this.models = _.filter(_this.models, function (m) { return m.id !== id; });
                    delete _this.byId[id];
                    _this.onModelRemoved.trigger(model);
                }
            });
        };
        return ObservableList;
    })();
    proximity.ObservableList = ObservableList;
})(proximity || (proximity = {}));
/// <reference path="user.ts"/>
/// <reference path="util.ts"/>
/// <reference path="ScreenCast.ts"/>
/// <reference path='../proximity/ObservableList.ts'/>
var client;
(function (client) {
    var MockGroup = (function () {
        function MockGroup(session) {
            this.session = session;
        }
        MockGroup.prototype.inviteToSession = function (type, memembers, callback) {
            if (callback) {
                callback(null, this.session);
            }
            return this;
        };
        MockGroup.prototype.createSession = function (properties, callback) {
            if (callback) {
                callback(null, this.session);
            }
            return this;
        };
        MockGroup.prototype.leaveSession = function () {
            return this;
        };
        MockGroup.prototype.size = function () {
            return 1;
        };
        return MockGroup;
    })();
    client.MockGroup = MockGroup;
    /** Exposes methods for working with a group. */
    var Group = (function () {
        //=======================================
        // Methods
        //=======================================
        /**
          * Creates a new group instance.
          * @param profile The groups profile info.
          * @param session (optional) The groups current session.
          */
        function Group(profile, session) {
            this.profile = profile;
            //=======================================
            // Events
            //=======================================
            /** Fired anytime a new post is added to the group. */
            this.onNewPost = new client.Event();
            if (session)
                this.attachSession(session);
        }
        /**
          * Attaches the group to an existing session.
          * @param session The session to attach to.
          */
        Group.prototype.attachSession = function (session) {
            var _this = this;
            this.session = session;
            this.session.onProfileUpdated.add(function (p) { return _this.onProfileUpdated(p); });
            this.session.onModelAdded.add(function (m) { return _this.onModelAdded(m); });
            this.session.onModelUpdated.add(function (m) { return _this.onModelUpdated(m); });
            this.session.onModelRemoved.add(function (id) { return _this.onModelRemoved(id); });
            if (session.cast) {
                this.cast = new client.ScreenCast(this, session.cast);
            }
            return this;
        };
        /**
          * Adds a new post to the group.
          * @param post The post to create.
          * @param cb (optional) Async results of the operation.
          */
        Group.prototype.createPost = function (post, cb) {
            var _this = this;
            this.ensureSession(function () {
                _this.session.createPost(post, cb);
            }, cb);
            return this;
        };
        /** Returns true if the group is currently in a casting session. */
        Group.prototype.inSession = function () {
            return this.session != null;
        };
        /** Add additional users to the group. */
        Group.prototype.inviteUsers = function (invites, cb) {
            var _this = this;
            try {
                if (this.session) {
                    this.session.inviteUsers(invites, function (err) {
                        if (cb) {
                            if (err) {
                                cb(err, null);
                            }
                            else {
                                cb(null, true);
                            }
                        }
                    });
                }
                else {
                    this.joinSession(client.ifSuccess(function () {
                        _this.session.inviteUsers(invites, function (err) {
                            if (cb) {
                                if (err) {
                                    cb(err, null);
                                }
                                else {
                                    cb(null, true);
                                }
                            }
                        });
                    }, cb));
                }
            }
            catch (e) {
                if (cb) {
                    cb(e, false);
                }
            }
            return this;
        };
        /**
          * Joins the session object for a group.
          * @param cb (optional) Async results of the operation.
          */
        Group.prototype.joinSession = function (cb) {
            var _this = this;
            if (!this.session) {
                client.user.getConnection().joinGroupSession({
                    accessToken: client.user.accessToken.token,
                    sessionId: 'group:' + this.profile.groupId
                }, function (err, session) {
                    if (session) {
                        _this.attachSession(session);
                        if (cb)
                            cb(null, true);
                    }
                    else if (cb) {
                        cb(err || new Error('Error joining GroupSession'), false);
                    }
                });
            }
            else if (cb) {
                cb(null, true);
            }
            return this;
        };
        /**
          * Joins an existing screen cast.
          * @param castId The ID of the cast to join.
          * @param cb Aysnc results of the operation.
          */
        Group.prototype.joinCast = function (castId, cb) {
            var _this = this;
            this.ensureSession(function () {
                _this.session.joinCast({ castId: castId, models: ['screenQueue', 'screenSettings', 'screenState'] }, client.ifSuccess(function (response) {
                    _this.cast = new client.ScreenCast(_this, response);
                    cb(null, _this.cast);
                }, cb));
            }, cb);
            return this;
        };
        /**
          * Leaves an existing screen cast.
          * @param castId The ID of the cast to leave.
          */
        Group.prototype.unjoinCast = function (castId) {
            // Stop tracking cast
            if (this.cast && (!castId || this.cast.castId == castId)) {
                this.cast.dispose();
                this.cast = null;
            }
            // Unjoin session
            if (this.session) {
                this.session.unjoin();
                this.session = null;
            }
            return this;
        };
        /**
          * Returns a list of posts for the group.
          * @param filter (optional) filter to restrict posts returned.
          * @param cb Async results of the operation.
          */
        Group.prototype.listPosts = function (filter, cb) {
            var _this = this;
            this.ensureSession(function () {
                var f = { type: 'post' };
                if (filter) {
                    if (filter.after)
                        f.after = filter.after;
                    if (filter.before)
                        f.before = filter.before;
                    if (filter.limit)
                        f.limit = filter.limit;
                }
                _this.session.tableList(f, client.ifSuccess(function (models) {
                    var p = {};
                    if (filter && filter.postType) {
                        p.models = [];
                        _.each(models, function (m) {
                            if (m.postType == filter.postType)
                                p.models.push(m);
                        });
                    }
                    else {
                        p.models = models;
                    }
                    if (filter && models.length >= filter.limit) {
                        p.next = function (callback) {
                            // Query next page
                            _this.listPosts(_.extend({}, filter, { after: models[models.length - 1].id }), callback);
                        };
                    }
                    cb(null, p);
                }, cb));
            }, cb);
            return this;
        };
        /**
          * Begins synchronizing a list of posts for the group.
          * @param filter (optional) filter to restrict posts returned.
          * @param cb Async results of the operation.
          */
        Group.prototype.syncPosts = function (filter, cb) {
            var _this = this;
            this.ensureSession(function () {
                var f = {
                    type: 'post',
                    custom: function (m) {
                        if (filter) {
                            if (filter.postType && m['postType'] !== filter.postType)
                                return false;
                            if (filter.after && m['id'] <= filter.after)
                                return false;
                            if (filter.before && m['id'] >= filter.before)
                                return false;
                        }
                        return true;
                    }
                };
                if (filter) {
                    if (filter.after)
                        f.after = filter.after;
                    if (filter.before)
                        f.before = filter.before;
                    if (filter.limit)
                        f.limit = filter.limit;
                }
                var list = new proximity.ObservableList(_this.session, f);
                list.sync(function (e, s) {
                    if (!e && s) {
                        cb(null, list);
                    }
                    else {
                        list.dispose();
                        cb(e, null);
                    }
                });
            }, cb);
            return this;
        };
        /**
          * Begins synchronizing presence information for the group.
          * @param cb Async results of the operation.
          */
        Group.prototype.syncPresence = function (cb) {
            var _this = this;
            this.ensureSession(function () {
                var f = {
                    fromCache: true,
                    type: 'presence'
                };
                var list = new proximity.ObservableList(_this.session, f);
                list.sync(function (e, s) {
                    if (!e && s) {
                        cb(null, list);
                    }
                    else {
                        list.dispose();
                        cb(e, null);
                    }
                });
            }, cb);
            return this;
        };
        /**
          * Starts a new screen cast.
          * @param options Initial state of the cast.
          * @param cb Aysnc results of the operation.
          */
        Group.prototype.startCast = function (options, cb) {
            var _this = this;
            this.ensureSession(function () {
                _this.session.startCast(Group.castOptionsToData(options), client.ifSuccess(function (response) {
                    _this.cast = new client.ScreenCast(_this, response);
                    cb(null, _this.cast);
                }, cb));
            }, cb);
            return this;
        };
        /** Time remaining before expiration */
        Group.prototype.minutesRemaining = function () {
            if (!this.profile) {
                return 0;
            }
            var pastDate = moment(this.profile.created);
            var now = moment();
            var diff = now.diff(pastDate, 'minutes');
            if (diff > consts.max_xim_age_minutes) {
                return 0;
            }
            else {
                return (consts.max_xim_age_minutes - diff);
            }
        };
        //=======================================
        // Event handlers
        //=======================================
        Group.prototype.onProfileUpdated = function (profile) {
            this.profile = profile;
        };
        Group.prototype.onModelAdded = function (model) {
            switch (model.type) {
                case 'post':
                    this.onNewPost.trigger(model);
                    break;
            }
        };
        Group.prototype.onModelUpdated = function (model) {
            switch (model.type) {
                case 'screenSettings':
                case 'screenQueue':
                case 'screenState':
                    if (this.cast && model['castId'] == this.cast.castId) {
                        this.cast.handleModelUpdated(model);
                    }
                    break;
            }
        };
        Group.prototype.onModelRemoved = function (id) {
        };
        //=======================================
        // Private helpers
        //=======================================
        Group.prototype.ensureSession = function (fn, cb) {
            this.joinSession(function (err, success) {
                if (success) {
                    fn();
                }
                else if (cb) {
                    cb(err, null);
                }
            });
        };
        //=======================================
        // Static methods
        //=======================================
        /**
          * Creates a new group. The created group will already be joined into a session.
          * @param data Creation info for the group.
          * @param cb Async results of the operation.
          */
        Group.create = function (data, cb) {
            client.user.getConnection().createGroupSession(data, function (err, session) {
                if (session) {
                    cb(null, new Group(session.profile, session));
                }
                else {
                    var errStr = 'Error creating GroupSession';
                    client.logger.info(errStr, err);
                    cb(err || new Error(errStr), null);
                }
            });
        };
        /**
          * Joins an existing group.
          * @param data Info for the group to join.
          * @param cb Async results of the operation.
          */
        Group.join = function (data, cb) {
            client.user.getConnection().joinGroupSession(data, function (err, session) {
                if (session) {
                    cb(null, new Group(session.profile, session));
                }
                else {
                    var errStr = 'Error joining group';
                    client.logger.info(errStr, err);
                    cb(err || new Error(errStr), null);
                }
            });
        };
        /**
          * Joins an existing xim.
          * @param data Info for the xim to join.
          * @param cb Async results of the operation.
          */
        Group.joinXim = function (data, cb) {
            client.user.getConnection().joinXim(data, function (err, session) {
                if (session) {
                    cb(null, new Group(session.profile, session));
                }
                else {
                    var errStr = 'Error joining xim';
                    client.logger.info(errStr, err);
                    cb(err || new Error(errStr), null);
                }
            });
        };
        Group.castOptionsToData = function (options) {
            var thumbnailUrl = options.queue.items[0] ? options.queue.items[0].url : "";
            return {
                castType: options.castType,
                models: [
                    client.ScreenCast.newQueue(options.queue),
                    client.ScreenCast.newSettings(options.settings),
                    client.ScreenCast.newState(options.state)
                ],
                thumbnailUrl: thumbnailUrl
            };
        };
        return Group;
    })();
    client.Group = Group;
})(client || (client = {}));
/// <reference path='user.ts'/>
/// <reference path='app.ts'/>
/// <reference path='../../types/cordova/pushnotification.d.ts'/>
var client;
(function (client) {
    var user;
    (function (user) {
        /** Fired anytime a user logs out */
        user.onToastNotification = new client.Event();
        function hasValidLiveToken() {
            var liveToken = client.user.accountTokens.live;
            var now = consts.time.unixNow();
            return liveToken && liveToken.token && liveToken.expires > now;
        }
        function hasValidFbToken() {
            var fbToken = client.user.accountTokens.fb;
            var now = consts.time.unixNow();
            return fbToken && fbToken.token && (!fbToken.expires || fbToken.expires > now);
        }
        user.hasValidFbToken = hasValidFbToken;
        function handleDeviceFacebookProfile(profileJson) {
            var profiles;
            if (_.isString(profileJson)) {
                profiles = JSON.parse(profileJson);
            }
            else {
                profiles = profileJson;
            }
            var fbProfile = _.find(profiles, function (p) {
                return p.socialNetwork.indexOf('facebook') >= 0;
            });
            if (fbProfile.accessToken) {
                var shortFuture = consts.time.unixNow() + 7200;
                // regardless of what token we used to have, always refresh our tokens
                // to the latest token (otherwise, we don't know when to refresh user.accessToken)
                client.user.accountTokens['fb'] = _.extend(client.user.accountTokens['fb'] || {}, { token: fbProfile.accessToken, expires: shortFuture });
                localStorage.setItem('client.user.accountTokens', JSON.stringify(client.user.accountTokens));
            }
            return fbProfile;
        }
        user.handleDeviceFacebookProfile = handleDeviceFacebookProfile;
        /** Log the user into facebook */
        function loginFacebookAsync(callback) {
            var fbToken = client.user.accountTokens.fb;
            var now = consts.time.unixNow();
            var client_id = consts.cloudProviders.facebook.appKey;
            var redirect_uri = 'https://www.facebook.com/connect/login_success.html';
            if (hasValidFbToken()) {
                _.defer(function () {
                    callback(null, true);
                });
            }
            else if (window.userprofile) {
                window.userprofile.getUserProfiles(function (profiles) {
                    var fbProfile = handleDeviceFacebookProfile(profiles);
                    callback(null, fbProfile != null);
                }, function (err) {
                    var url = uri.parse('https://www.facebook.com/dialog/oauth');
                    url.query = {
                        'client_id': client_id,
                        'redirect_uri': redirect_uri,
                        'scope': 'user_photos',
                        'response_type': 'token'
                    };
                    var loginUrl = uri.format(url);
                    client.logger.info('facebook login dialog at:', loginUrl);
                    // open inappbrowser to login user
                    var ref = window.open(loginUrl, '_blank', 'location=no');
                    ref.addEventListener('exit', function (e) {
                        if (!hasValidFbToken()) {
                            callback(new Error('user canceled'), false);
                        }
                    });
                    ref.addEventListener('loadstart', function (e) {
                        var url = uri.parse(e.url);
                        if (url.protocol == 'https' && url.host == 'www.facebook.com' && url.path == 'connect/login_success.html') {
                            var hashQuery = querystring.parse(url.hash);
                            var success = false;
                            client.logger.info('fb login return url', url);
                            client.logger.info('fb login results', hashQuery);
                            if (hashQuery.hasOwnProperty('access_token')) {
                                var access_token = hashQuery['access_token'];
                                client.user.accountTokens.fb = {
                                    token: access_token,
                                    expires: consts.time.unixNow() + parseInt(hashQuery['expires_in'])
                                };
                                // save in the local store
                                localStorage.setItem('client.user.accountTokens', JSON.stringify(client.user.accountTokens));
                                ref.close();
                                callback(null, true);
                            }
                            else {
                                callback(new Error('fb login failed'), false);
                            }
                        }
                        else {
                            client.logger.info('unhandled inapp browser url', url);
                        }
                    });
                });
            }
        }
        user.loginFacebookAsync = loginFacebookAsync;
        /** Logs the user into instagram */
        function hasValidInstagramToken() {
            return !!client.user.accountTokens.instagram;
        }
        function loginInstagramAsync(callback) {
            var redirect_uri = 'https://localhost/authcallback';
            var client_id = consts.cloudProviders.instagram.appKey;
            if (hasValidInstagramToken()) {
                setTimeout(function () {
                    callback(null, true);
                }, 0);
            }
            else {
                var handleLoginResult = function (e) {
                    client.logger.info('handle inappbrowser url:', e.url);
                    var url = uri.parse(e.url);
                    if (url.protocol == 'https' && url.hostname == 'localhost' && url.path == 'authcallback') {
                        var hashQuery = querystring.parse(url.hash);
                        var success = false;
                        client.logger.info('instagram login return url', url);
                        client.logger.info('instagram login results', hashQuery);
                        if (hashQuery.hasOwnProperty('access_token')) {
                            var access_token = hashQuery['access_token'];
                            client.user.accountTokens.instagram = {
                                token: access_token
                            };
                            // save in the local store
                            localStorage.setItem('client.user.accountTokens', JSON.stringify(client.user.accountTokens));
                            ref.close();
                            callback(null, true);
                        }
                        else {
                            client.logger.debug('instagram login failed');
                            callback(null, false);
                        }
                    }
                    else {
                        client.logger.info('unhandled inapp browser url', url);
                    }
                };
                var authUrl = uri.parse('https://instagram.com/oauth/authorize/');
                authUrl.query = { client_id: client_id, redirect_uri: redirect_uri, response_type: 'token' };
                var ref = window.open(uri.format(authUrl), '_blank', 'location=no');
                ref.addEventListener('exit', function (e) {
                    if (!hasValidInstagramToken()) {
                        callback(new Error('user canceled'), false);
                    }
                });
                ref.addEventListener('loaderror', handleLoginResult);
                ref.addEventListener('loadstart', handleLoginResult);
            }
        }
        user.loginInstagramAsync = loginInstagramAsync;
        /** Logs the user into google */
        function hasValidGoogleToken() {
            return !!client.user.accountTokens.google;
        }
        function loginGoogleAsync(callback) {
            var googleToken = client.user.accountTokens.google;
            var redirect_uri = 'urn:ietf:wg:oauth:2.0:oob';
            var client_id = '51082097186.apps.googleusercontent.com';
            if (hasValidGoogleToken()) {
                setTimeout(function () {
                    callback(null, true);
                }, 0);
            }
            else {
                var url = uri.parse('https://accounts.google.com/o/oauth2/auth');
                url.query = {
                    client_id: client_id,
                    redirect_uri: redirect_uri,
                    response_type: 'token',
                    state: 'googleLogin',
                    scope: ''
                };
            }
        }
        user.loginGoogleAsync = loginGoogleAsync;
        function hasValidFlickrToken() {
            return !!client.user.accountTokens.flickr && !!client.user.accountTokens.flickr.token;
        }
        function loginFlickrAsync(callback) {
            if (hasValidFlickrToken()) {
                setTimeout(function () {
                    callback(null, true);
                }, 0);
            }
            else {
                var loginUrl = uri.resolve(client.server.root, '/auth/flickr');
                var hostUrl = uri.parse(client.server.root);
                var hostname = hostUrl.hostname;
                client.logger.info('flickr login proxy at:', loginUrl);
                // open inappbrowser to login user
                var ref = window.open(loginUrl, '_blank', 'location=no');
                var handleLoginResult = function (e) {
                    client.logger.info('handle inappbrowser url:', e.url);
                    var url = uri.parse(e.url);
                    if (url.hostname == hostname && url.path == 'auth/ok') {
                        client.user.accountTokens.flickr = {
                            token: uri.resolve(client.server.root, '/services/flickr'),
                            user_id: url.query ? url.query['flickr_uid'] : null
                        };
                        // save in the local store
                        localStorage.setItem('client.user.accountTokens', JSON.stringify(client.user.accountTokens));
                        client.logger.debug('about to close in app browser on successful flickr authentication');
                        setTimeout(function () {
                            ref.close();
                            if (client.startupMode == 3 /* wp */) {
                                // wp8 doens't seem to like to get exit event fired on in-app browser
                                // call the callback ourselves
                                callback(null, true);
                            }
                        }, 100);
                    }
                    else {
                        client.logger.info('unhandled inapp browser url', url);
                    }
                };
                ref.addEventListener('exit', function (e) {
                    if (!hasValidFlickrToken()) {
                        callback(new Error('user canceled'), false);
                    }
                    else if (client.startupMode != 3 /* wp */) {
                        client.logger.debug('inapp browser closed on successful flickr authentciation');
                        callback(null, true);
                    }
                });
                ref.addEventListener('loaderror', handleLoginResult);
                var loadevent = client.startupMode != 5 /* droid */ ? 'loadstop' : 'loadstart';
                ref.addEventListener(loadevent, handleLoginResult);
            }
        }
        user.loginFlickrAsync = loginFlickrAsync;
        function hasValidDropboxToken() {
            return !!client.user.accountTokens.dropbox && !!client.user.accountTokens.dropbox.token;
        }
        /** Login user to dropbox */
        function loginDropboxAsync(callback) {
            var client_id = consts.cloudProviders.dropbox.appKey;
            var redirect_uri = 'http://localhost';
            if (hasValidDropboxToken()) {
                setTimeout(function () {
                    callback(null, true);
                }, 0);
            }
            else {
                var url = uri.parse('https://www.dropbox.com/1/oauth2/authorize');
                var handleLoginResult = function (e) {
                    var url = uri.parse(e.url);
                    if (url.protocol == 'http' && url.host == 'localhost') {
                        var hashQuery = querystring.parse(url.hash);
                        var success = false;
                        client.logger.info('dropbox login return url', url);
                        client.logger.info('dropbox login results', hashQuery);
                        if (hashQuery.hasOwnProperty('access_token')) {
                            var access_token = hashQuery['access_token'];
                            client.user.accountTokens.dropbox = {
                                token: access_token,
                                uid: hashQuery['uid']
                            };
                            // save in the local store
                            localStorage.setItem('client.user.accountTokens', JSON.stringify(client.user.accountTokens));
                            ref.close();
                            callback(null, true);
                        }
                        else {
                            callback(new Error('dropbox login failed'), false);
                        }
                    }
                    else {
                        client.logger.info('unhandled inapp browser url', url);
                    }
                };
                url.query = {
                    client_id: client_id,
                    redirect_uri: redirect_uri,
                    disable_signup: 'true',
                    response_type: 'token'
                };
                var loginUrl = uri.format(url);
                client.logger.info('dropbox login url', loginUrl);
                // open inappbrowser to login user
                var ref = window.open(loginUrl, '_blank', 'location=no');
                ref.addEventListener('exit', function (e) {
                    if (!hasValidDropboxToken()) {
                        callback(new Error('user canceled'), false);
                    }
                });
                ref.addEventListener('loaderror', handleLoginResult);
                ref.addEventListener('loadstart', handleLoginResult);
            }
        }
        user.loginDropboxAsync = loginDropboxAsync;
        /** Logs the user into windows live. */
        function loginLiveAsync(callback) {
            var liveToken = client.user.accountTokens.live;
            var unixNow = Math.round((new Date()).getTime() / 1000);
            var client_id = consts.cloudProviders.onedrive.appKey;
            var redirect_uri = client.server.root + 'api/onedrive/auth';
            client.logger.info('current live token status', liveToken, 'unix now', unixNow);
            if (hasValidLiveToken()) {
                callback(null, true);
            }
            else if (liveToken && liveToken.refreshToken) {
                // need to refresh access token
                client.logger.info('refresh onedrive access token');
                client.server.POST('api/onedrive/refresh', {}, { refresh_token: liveToken.refreshToken }, function (e, data) {
                    client.logger.info('post to oauth20_token replied with', data);
                    var unixNow = Math.round((new Date()).getTime() / 1000);
                    client.user.accountTokens.live = _.extend(client.user.accountTokens.live, {
                        token: data.access_token,
                        expires: unixNow + data.expires_in,
                        refreshToken: data.refresh_token
                    });
                    localStorage.setItem('client.user.accountTokens', JSON.stringify(client.user.accountTokens));
                    callback(null, true);
                });
            }
            else {
                // need to show login
                var url = uri.parse('https://login.live.com/oauth20_authorize.srf?client_id=CLIENT_ID&scope=SCOPES&response_type=RESPONSE_TYPE&redirect_uri=REDIRECT_URL');
                url.query = {
                    client_id: client_id,
                    scope: 'wl.photos wl.skydrive wl.offline_access',
                    response_type: 'code',
                    redirect_uri: redirect_uri,
                    display: client.startupMode == 4 /* ios */ ? 'ios_phone' : 'touch'
                };
                var loginUrl = uri.format(url);
                client.logger.info('skydrive login url is ', loginUrl);
                // open inappbrowser to login user
                var ref = window.open(loginUrl, '_blank', 'location=no');
                ref.addEventListener('exit', function (e) {
                    if (!hasValidLiveToken()) {
                        callback(new Error('user canceled'), false);
                    }
                });
                ref.addEventListener('loadstop', function (e) {
                    var url = uri.parse(e.url);
                    var serverUrl = uri.parse(client.server.root);
                    if (url.protocol == serverUrl.protocol && url.host == serverUrl.host && url.path == 'api/onedrive/auth') {
                        var hashQuery = querystring.parse(url.hash);
                        var success = false;
                        client.logger.info('live login return url', url);
                        client.logger.info('live login results', hashQuery);
                        var data = url.query;
                        client.logger.info('post to oauth20_token results', data);
                        if (data.access_token && data.refresh_token && data.expires) {
                            client.user.accountTokens.live = {
                                token: data.access_token,
                                refreshToken: data.refresh_token,
                                expires: data.expires
                            };
                            // save in the local store
                            localStorage.setItem('client.user.accountTokens', JSON.stringify(client.user.accountTokens));
                            ref.close();
                            callback(null, true);
                        }
                    }
                    else {
                        client.logger.info('unhandled inapp browser url', url);
                    }
                });
            }
        }
        user.loginLiveAsync = loginLiveAsync;
        /** register mobile device to get push notification */
        function registerForPush(callback) {
            if (window.pushnotification) {
                var senderId = '';
                switch (client.startupMode) {
                    case 4 /* ios */:
                        senderId = consts.mobileAppId.apple;
                        break;
                    case 5 /* droid */:
                        senderId = consts.mobileAppId.google;
                        break;
                    case 3 /* wp */:
                        senderId = consts.mobileAppId.wp;
                        break;
                }
                window.pushnotification.getDeviceToken(function (deviceToken) {
                    client.logger.info('got device token as ', deviceToken);
                    window.pushnotification.subscribe(function (notification) {
                        client.logger.info('got notification ', notification);
                        if (_.isString(notification)) {
                            notification = JSON.parse(notification);
                        }
                        if (notification) {
                            client.logger.info('got push notification', notification);
                            // trigger this event with wait if this is launch
                            // BUGBUG: ask Steve to look into why the userSession doesn't have
                            // the full group details (including the cast media payload) after user.connect callback is called
                            if (notification.isLaunch) {
                                // waiting until the app is finished with starting is there to trigger the event
                                // otherwise the event is not listened to
                                var watchForApp = setInterval(function () {
                                    if (client.app.getState() != 1 /* starting */) {
                                        clearInterval(watchForApp);
                                        client.user.onToastNotification.trigger(notification);
                                    }
                                }, 50);
                            }
                            else {
                                client.user.onToastNotification.trigger(notification);
                            }
                        }
                    }, function (e) {
                        client.logger.debug('subscribe error', e);
                    }, senderId);
                    var url = uri.parse(client.server.root + 'api/registerPush');
                    url.query = { deviceToken: deviceToken, accessToken: client.user.accessToken.token, os: client.StartupMode[client.startupMode] };
                    $.post(uri.format(url), function (data) {
                        var e = data && data.status == 'success' ? null : new Error('api registerPush failed');
                        callback(e, !e);
                    }).fail(function () {
                        callback(new Error('failed to call registerPush'), false);
                    });
                }, function (err) {
                    callback(new Error(err || 'failed to get push device token'), false);
                }, senderId);
            }
            else {
                callback(null, true);
            }
        }
        user.registerForPush = registerForPush;
    })(user = client.user || (client.user = {}));
})(client || (client = {}));
/// <reference path='user.ts'/>
var client;
(function (client) {
    var user;
    (function (user) {
        /** Fired anytime the users groups change. */
        user.onGroupsUpdated = new client.Event();
        /** Returns one or more of the users groups from the local cache. */
        function getCachedGroups(ids, response) {
            user.ifConnected(function () {
                user.session.getCachedGroups(ids, function (err, profiles) {
                    if (profiles) {
                        var page = { models: [] };
                        _.each(profiles, function (p) { return page.models.push(new client.Group(p)); });
                        response(null, page);
                    }
                    else {
                        response(err, null);
                    }
                });
            }, response);
        }
        user.getCachedGroups = getCachedGroups;
        /** Returns one or more of the users groups given their ID. */
        function getGroups(ids, response) {
            user.ifConnected(function () {
                user.session.getGroups(ids, function (err, profiles) {
                    if (profiles) {
                        var page = { models: [] };
                        _.each(profiles, function (p) { return page.models.push(new client.Group(p)); });
                        response(null, page);
                    }
                    else {
                        response(err, null);
                    }
                });
            }, response);
        }
        user.getGroups = getGroups;
        /** Creates a new group. */
        function createGroup(invites, cast, response) {
            user.ifConnected(function () {
                client.Group.create({ accessToken: user.accessToken.token, invites: invites, cast: cast }, response);
            }, response);
        }
        user.createGroup = createGroup;
        /** Returns a list of the users groups from the local cache filtered by time. */
        function listCachedGroups(options, response) {
            user.ifConnected(function () {
                user.session.listCachedGroups(options, function (err, profiles) {
                    if (profiles) {
                        var page = { models: [] };
                        _.each(profiles, function (p) { return page.models.push(new client.Group(p)); });
                        if (options && options.limit && profiles.length == options.limit) {
                            page.next = function (cb) {
                                var before = profiles[options.limit - 1].lastActivity;
                                listCachedGroups({ after: options.after, limit: options.limit, before: before }, cb);
                            };
                        }
                        response(null, page);
                    }
                    else {
                        response(err, null);
                    }
                });
            }, response);
        }
        user.listCachedGroups = listCachedGroups;
    })(user = client.user || (client.user = {}));
})(client || (client = {}));
/// <reference path='user.ts'/>
var client;
(function (client) {
    var user;
    (function (user) {
        var nearbyUsers = {};
        /**
          * Returns the current list of nearby users & groups.
          * @param filter (optional) Type of entries to return.
          */
        function nearby(filter) {
            if (filter === void 0) { filter = 0 /* all */; }
            if (!user.isAuthorized())
                throw "Not Authorized";
            var page = { models: [] };
            if (filter == 0 /* all */ || filter == 1 /* users */) {
                _.each(nearbyUsers, function (profile) { return page.models.push(profile); });
            }
            if (filter == 0 /* all */ || filter == 2 /* groups */) {
            }
            return page;
        }
        user.nearby = nearby;
        /** Updates the users discovery mode. */
        function setDiscoverySetting(mode, callback) {
            user.ifConnected(function () {
                if (mode != user.settings.discovery.setting) {
                    client.logger.info('Switching discovery mode to: ' + mode);
                    user.updateSettings({ discovery: { setting: mode, timestamp: consts.time.now() } }, function (err, status) {
                        stopDiscovery(mode === consts.NearbyDiscoveryMode.incognito);
                        if (mode !== consts.NearbyDiscoveryMode.incognito) {
                            startDiscovery(true, callback);
                        }
                        else {
                            // Remove any existing ad
                            user.session.updateLocators([], { locators: [], profiles: [] });
                        }
                        user.settings.discovery.setting = mode;
                    });
                }
                else {
                    if (callback)
                        callback(null, false);
                }
            }, callback);
        }
        user.setDiscoverySetting = setDiscoverySetting;
        /** Starts the discovery systems for the user. */
        function startDiscovery(forceUpdate, callback) {
            user.ifConnected(function () {
                client.scanner.forceUpdate = forceUpdate;
                $('body').addClass('discovery-' + user.settings.discovery.setting);
                if (!user.discoveryStarted && user.settings.discovery.setting !== consts.NearbyDiscoveryMode.incognito) {
                    user.discoveryStarted = true;
                    client.scanner.startAdvertise(function () {
                        client.scanner.getCurrentPosition(function () {
                            if (user.discoveryStarted) {
                                client.scanner.startWatching();
                                if (callback)
                                    callback(null, true);
                            }
                            else {
                                // stopDiscovery() called while getting position
                                if (callback)
                                    callback(null, false);
                            }
                        });
                    });
                }
                else {
                    if (callback)
                        callback(null, true);
                }
            }, callback);
        }
        user.startDiscovery = startDiscovery;
        /** Stops running discovery systems. */
        function stopDiscovery(clearNearby) {
            if (clearNearby)
                nearbyUsers = {};
            if (user.discoveryStarted) {
                user.discoveryStarted = false;
                client.scanner.stopAdvertise();
                client.scanner.stopWatching();
            }
        }
        user.stopDiscovery = stopDiscovery;
        //=====================================================
        // Protected methods
        //=====================================================
        /** Called anytime the users nearby list changes. */
        function _updateNearbyList(nearby) {
            // Update nearby users list
            client.logger.info('updating nearby list');
            var added = [];
            var removed = [];
            _.each(nearby.users, function (profile, id) {
                if (!(id in nearbyUsers))
                    added.push(profile);
            });
            _.each(nearbyUsers, function (profile, id) {
                if (!(id in nearby.users))
                    removed.push(profile);
            });
            nearbyUsers = nearby.users;
            client.logger.info('updating nearby list. added: ' + JSON.stringify(added));
            client.logger.info('removing: ' + JSON.stringify(removed));
            // Signal changes
            if (added.length > 0) {
                user.onNearbyUsersAdded.trigger(added);
            }
            if (removed.length > 0) {
                user.onNearbyUsersRemoved.trigger(removed);
            }
        }
        user._updateNearbyList = _updateNearbyList;
        //=====================================================
        // Event listners
        //=====================================================
        client.scanner.onScanningState.add(function (scanning) {
            if (scanning) {
                $('body').addClass('discovery-scanning');
            }
            else {
                $('body').removeClass('discovery-scanning');
            }
        });
        client.scanner.onLocatorChange.add(function () {
            if (user.discoveryStarted) {
                var locators = client.scanner.getLocators(1 /* ambient */);
                var ad = getConnectAdvertisment();
                //logger.info('Sending updated locators to server', locators, ad);
                user.session.updateLocators(locators, ad);
            }
        });
        //=====================================================
        // Helper functions
        //=====================================================
        /** Returns a nearcast advertisment based on the users current discovery status. */
        function getConnectAdvertisment() {
            var ad = { locators: [], profiles: [] };
            if (user.settings.discovery.setting !== consts.NearbyDiscoveryMode.incognito) {
                ad.locators = client.scanner.getLocators(2 /* selfies */);
                var adProfile = {
                    type: 'user',
                    userId: user.profile.userId,
                    displayName: user.profile.displayName,
                    thumbnailUrl: user.profile.thumbnailUrl,
                    pictureUrl: user.profile.pictureUrl
                };
                adProfile = _.extend(adProfile, { access: user.settings.discovery.setting });
                ad.profiles.push(adProfile);
            }
            return ad;
        }
    })(user = client.user || (client.user = {}));
})(client || (client = {}));
/// <reference path='../../../types/cordova/contacts.d.ts'/>
var client;
(function (client) {
    var MockContacts = (function () {
        function MockContacts() {
        }
        MockContacts.getall = function (successCallback, errorCallback) {
            if (MockContacts.contacts.length == 0) {
                MockContacts.makeContacts();
            }
            var contacts = [];
            for (var i = 0; i < MockContacts.contacts.length; i++) {
                var contact = {
                    displayName: MockContacts.contacts[i].displayName,
                    id: MockContacts.contacts[i].id
                };
                contacts.push(contact);
            }
            successCallback(contacts);
        };
        MockContacts.getdetails = function (ids, successCallback, errorCallback) {
            var contacts = [];
            for (var i = 0; i < ids.length; i++) {
                for (var j = 0; j < MockContacts.contacts.length; j++) {
                    if (MockContacts.contacts[j].id == ids[i]) {
                        var contact = {
                            id: ids[i],
                            email: MockContacts.contacts[j].email,
                            phone: MockContacts.contacts[j].phone
                        };
                        contacts.push(contact);
                        break;
                    }
                }
            }
            successCallback(contacts);
        };
        MockContacts.getbitmap = function (index, name, successCallback, errorCallback) {
            // Simulate about 1/2 to not have a profile photo
            if (client.Util.getRandomInt(2) == 1) {
                var image = "http://lorempixel.com/" + (160 + client.Util.getRandomInt(20)) + "/" + (160 + client.Util.getRandomInt(20)) + "/people/";
                var bitmapresponse = {
                    imageBase64: image,
                    index: index
                };
                successCallback(bitmapresponse);
            }
        };
        MockContacts.makeContacts = function () {
            // Mock contacts change, but contacts are cached so make id unique across generation
            var salt = new Date().getTime();
            for (var i = 0; i < MockContacts.numContacts; i++) {
                var first = MockContacts.firstNames[client.Util.getRandomInt(MockContacts.firstNames.length - 1)];
                var last = MockContacts.lastNames[client.Util.getRandomInt(MockContacts.lastNames.length - 1)];
                var contact = {
                    displayName: first + " " + last,
                    id: salt + i,
                    email: MockContacts.generateEmail(first, last),
                    phone: MockContacts.generatePhone()
                };
                MockContacts.contacts.push(contact);
            }
        };
        MockContacts.generateEmail = function (first, last) {
            var provider = MockContacts.providers[client.Util.getRandomInt(MockContacts.providers.length - 1)];
            var p1 = first.substr(0, client.Util.getRandomInt(first.length - 3) + 2);
            var p2 = last.substr(0, client.Util.getRandomInt(last.length - 3) + 2);
            return p1 + p2 + "@" + provider;
        };
        MockContacts.generatePhone = function () {
            var number = "(555) " + MockContacts.getNumberString(3) + "-" + MockContacts.getNumberString(4);
            return number;
        };
        MockContacts.getNumberString = function (length) {
            var nString = "";
            // First number must be at least 2
            nString += 2 + client.Util.getRandomInt(7);
            for (var i = 0; i < length - 1; i++) {
                nString += client.Util.getRandomInt(9);
            }
            return nString;
        };
        MockContacts.numContacts = 250;
        MockContacts.contacts = [];
        MockContacts.firstNames = ["Evangeline", "Montgomery", "Jacob", "Mason", "William", "Jayden", "Noah", "Michael", "Ethan", "Alexander", "Aiden", "Daniel", "Anthony", "Matthew", "Elijah", "Joshua", "Liam", "Andrew", "James", "David", "Benjamin", "Logan", "Christopher", "Joseph", "Jackson", "Gabriel", "Ryan", "Sophia", "Isabella", "Emma", "Olivia", "Ava", "Emily", "Abigail", "Madison", "Mia", "Chloe", "Elizabeth", "Ella", "Addison", "Natalie", "Lily", "Grace", "Samantha", "Avery", "Sofia", "Aubrey", "Brooklyn", "Lillian", "Victoria", "Evelyn", "Hannah", "Judy", "Hal", "Erik", "Dale", "Geremy", "Finn", "Oli", "Garret", "Kyle", "Evelyn", "Ruth", "Sara", "Barak", "Jen", "Jennifer", "Jenny", "陈", "蔡"];
        MockContacts.lastNames = ["Honeyblossom", "Smith", "Jones", "Taylor", "Williams", "Brown", "Davies", "Evans", "Wilson", "Thomas", "Roberts", "Johnson", "Lewis", "Walker", "Robinson", "Wood", "Thompson", "White", "Watson", "Jackson", "Wright", "Green", "Harris", "Cooper", "King", "Lee", "Martin", "Clarke", "James", "Morgan", "Hughes", "Edwards", "Hill", "Moore", "Clark", "Harrison", "Scott", "Young", "Morris", "Hall", "Ward", "Turner", "Carter", "Phillips", "Mitchell", "Patel", "Adams", "Campbell", "Anderson", "Allen", "Cook", "何", "高"];
        MockContacts.providers = ["example.com", "example.net", "example.org"];
        return MockContacts;
    })();
    client.MockContacts = MockContacts;
})(client || (client = {}));
/// <reference path='../shared/consts.ts'/>
var client;
(function (client) {
    /* Stores color/profileID map as color is client only */
    var ColorCycle = (function () {
        function ColorCycle() {
        }
        ColorCycle.GetNext = function () {
            // Initialize to random starting position
            if (!ColorCycle.curColor) {
                ColorCycle.curColor = Math.floor(Math.random() * (ColorCycle.profileColors.length));
            }
            else {
                ColorCycle.curColor++;
                if (ColorCycle.curColor >= ColorCycle.profileColors.length) {
                    ColorCycle.curColor = 0;
                }
            }
            return ColorCycle.profileColors[ColorCycle.curColor];
        };
        ColorCycle.defaultColor = "#3ca8b6";
        ColorCycle.curColor = null;
        // Profile colors
        ColorCycle.profileColors = [
            "#f9bf1c",
            "#ef355e",
            "#eb6d24",
            "#20cece",
            "#f9cf1c",
            "#ed2f58",
            "#e58729",
            "#2cc2c6",
            "#fbd81b",
            "#e82142",
            "#ea9626",
            "#2db9c4",
            "#f9df1c",
            "#db1d3d",
            "#e8a828",
            "#2fb0c1"
        ];
        return ColorCycle;
    })();
    client.ColorCycle = ColorCycle;
})(client || (client = {}));
/// <reference path='../../types/cordova/contacts.d.ts'/>
/// <reference path='./mockobjects/MockContacts.ts'/>
/// <reference path='./colorCycle.ts'/>
/// <reference path='types.ts'/>
/// <reference path='server.ts'/>
var client;
(function (client) {
    var DeviceContacts = (function () {
        function DeviceContacts() {
        }
        // Interface that provides contacts 
        DeviceContacts.contactProvider = function () {
            if (window.contacts) {
                return window.contacts;
            }
            else {
                return client.MockContacts;
            }
        };
        /** inner initialization of contacts from device */
        DeviceContacts._initContacts = function () {
            if (DeviceContacts.allContacts == null) {
                // Clear any existing data
                DeviceContacts.free();
                // Get all contacts
                DeviceContacts.contactProvider().getall(DeviceContacts.initSuccess, DeviceContacts.initFailed);
            }
        };
        // Called upon successful initialization of DeviceContacts, meaning list of all contacts has been loaded
        DeviceContacts.initSuccess = function (contacts) {
            if (_.isString(contacts)) {
                contacts = JSON.parse(contacts);
            }
            // Cache contacts
            DeviceContacts.allContacts = [];
            for (var i = 0; i < contacts.length; i++) {
                var contact = contacts[i];
                // Ignore contact if fully loaded and doesn't have an email or phone
                if (contact.loaded && !contact.email && !contact.phone) {
                    continue;
                }
                var splitName = DeviceContacts.splitName(contact);
                // Extract first & last name
                var deviceContact = {
                    displayName: contact.displayName,
                    color: client.ColorCycle.GetNext(),
                    firstName: splitName.first,
                    lastName: splitName.last,
                    thumbnailUrl: null,
                    gotThumbnail: false,
                    deviceId: contact['id'].toString(),
                    loaded: !!(contact.email || contact.phone)
                };
                if (deviceContact.loaded) {
                    deviceContact.email = contact.email;
                    deviceContact.phone = contact.phone;
                }
                // Store it
                DeviceContacts.allContacts.push(deviceContact);
            }
            // Sort it alphabetically
            DeviceContacts.allContacts.sort(function (a, b) {
                var textA = a.displayName.toUpperCase();
                var textB = b.displayName.toUpperCase();
                return (textA < textB) ? -1 : (textA > textB) ? 1 : 0;
            });
            // Perform any pending search that may have been requested during initialization
            if (DeviceContacts.pendingSearchTerm) {
                DeviceContacts.matchCount(DeviceContacts.pendingSearchTerm, DeviceContacts.matchCallback);
            }
        };
        // Called upon failed initialization of DeviceContacts
        DeviceContacts.initFailed = function (error) {
            // user might have refused access to contacts (on iOS)
            if (error && error == cordova.callbackStatus.INVALID_ACTION) {
                client.logger.warn('CONTACTS access not granted');
                client.analytics.logEvent(consts.flurryEvent.contactsAccessDenied);
            }
            else {
                client.logger.error(consts.errorCode.Contacts, 'CONTACTS Init Failed!');
            }
        };
        DeviceContacts.cancelCallbacks = function () {
            DeviceContacts.matchCallback = null;
            DeviceContacts.detailCallback = null;
            DeviceContacts.bitmapCallback = null;
        };
        // Free memory & terminate any remaining callbacks
        DeviceContacts.free = function () {
            DeviceContacts.pendingSearchTerm = null;
            DeviceContacts.lastSearchResults = null;
        };
        // Returns count of contacts matching search term and queues detailsearch request
        DeviceContacts.matchCount = function (searchTerm, callback) {
            searchTerm = searchTerm.toLowerCase().trim();
            // If device contacts haven't finished load, wait for them to complete before doing match
            if (DeviceContacts.allContacts == null) {
                DeviceContacts.pendingSearchTerm = searchTerm;
                DeviceContacts.matchCallback = callback;
                return;
            }
            else {
                DeviceContacts.pendingSearchTerm = null;
                DeviceContacts.matchCallback = null;
            }
            // Clear any existing callbacks
            DeviceContacts.cancelCallbacks();
            // Save callback so abort is possible
            DeviceContacts.matchCallback = callback;
            // Look matching contacts
            var fullnameMatch = [];
            var firstnameMatch = [];
            var lastnameMatch = [];
            // Break into three groups so can show full name matches first, then first name and finally last name
            if (DeviceContacts.allContacts) {
                for (var i = 0; i < DeviceContacts.allContacts.length; i++) {
                    var contact = DeviceContacts.allContacts[i];
                    if (contact.displayName && contact.displayName.toLowerCase().indexOf(searchTerm) == 0) {
                        fullnameMatch.push(contact);
                    }
                    else if (contact.firstName && contact.firstName.toLowerCase().indexOf(searchTerm) == 0) {
                        firstnameMatch.push(contact);
                    }
                    else if (contact.lastName && contact.lastName.toLowerCase().indexOf(searchTerm) == 0) {
                        lastnameMatch.push(contact);
                    }
                }
            }
            // Merge into single results list
            DeviceContacts.lastSearchResults = fullnameMatch.concat(firstnameMatch, lastnameMatch);
            // Return count of matches to callback
            if (DeviceContacts.matchCallback) {
                DeviceContacts.matchCallback(DeviceContacts.lastSearchResults.length);
            }
        };
        // Returns first/last name using email or sms when necessary
        DeviceContacts.displayNames = function (profile) {
            var splitname = {
                first: "",
                last: ""
            };
            // If no displayName, first name derived from inviteType
            if (!profile.displayName) {
                // SMS
                if (profile.inviteType == consts.InviteType.sms) {
                    if (profile.phone) {
                        // Show last 7 digits
                        var len = profile.phone.length;
                        splitname.first = (len >= 7) ? profile.phone.substring(len - 7, len) : profile.phone;
                    }
                    else {
                        client.logger.error(consts.errorCode.Contacts, 'Profile with SMS address type has invalid phone.');
                        splitname.first = "Unknown";
                    }
                }
                else if (profile.inviteType == consts.InviteType.email) {
                    if (profile.email) {
                        // Show everything before domain
                        var firstEmail = profile.email.indexOf("@");
                        if (firstEmail != -1) {
                            splitname.first = profile.email.substr(0, firstEmail);
                        }
                    }
                    else {
                        client.logger.error(consts.errorCode.Contacts, 'Profile with EMAIL address type has invalid email.');
                        splitname.first = "Unknown";
                    }
                }
                else if (profile.inviteType == consts.InviteType.code) {
                    if (profile.code) {
                        splitname.first = profile.code;
                    }
                    else {
                        client.logger.error(consts.errorCode.Contacts, 'Profile with CODE address type has invalid code.');
                        splitname.first = "Unknown";
                    }
                }
            }
            else {
                splitname = DeviceContacts.splitName(profile);
            }
            return splitname;
        };
        // Splits display name into first and last
        DeviceContacts.splitName = function (profile) {
            var splitname = {
                first: "",
                last: ""
            };
            // Return blanks if no profile or display name
            if (!profile || !profile.displayName) {
                return splitname;
            }
            var firstSpace = profile.displayName.indexOf(" ");
            if (firstSpace == -1) {
                splitname.first = profile.displayName;
            }
            else {
                splitname.first = profile.displayName.substr(0, firstSpace);
                splitname.last = profile.displayName.substr(firstSpace + 1);
            }
            return splitname;
        };
        // Returns intials for profiles that have no thumbnail
        DeviceContacts.initials = function (profile) {
            // Use symbols if profile doesn't have a thumbnail
            var initials = "";
            if (!profile.thumbnailUrl) {
                if (profile.displayName) {
                    // If displayName is valid phone or email, show @ or # symbol
                    if (client.Util.validEmail(profile.displayName)) {
                        return "@";
                    }
                    var parsedPhone = consts.parsePhoneNumberForCountry(profile.displayName, client.user.country);
                    if (parsedPhone.isValid) {
                        return "#";
                    }
                }
                // Attempt to extract initials from displayName
                var name = DeviceContacts.splitName(profile);
                if (name.first)
                    initials += name.first[0].toUpperCase();
                if (name.last)
                    initials += name.last[0].toUpperCase();
                if (initials) {
                    return initials;
                }
                // Fall back to symbols if no displayName
                if (profile.inviteType == consts.InviteType.sms) {
                    return "#";
                }
                else if (profile.inviteType == consts.InviteType.email) {
                    return "@";
                }
                else if (profile.inviteType == consts.InviteType.code) {
                    return "*";
                }
                client.logger.error(consts.errorCode.Contacts, 'Profile has no thumbnail and cannot generate initials');
                initials = "XX";
            }
            return initials;
        };
        DeviceContacts.inviteAddress = function (profile) {
            if (profile.inviteType == consts.InviteType.sms) {
                return profile.phone;
            }
            else if (profile.inviteType == consts.InviteType.email) {
                return profile.email;
            }
            else if (profile.inviteType == consts.InviteType.code) {
                return profile.code;
            }
            else if (profile.inviteType == consts.InviteType.xim) {
                return profile.userId;
            }
            else {
                return "";
            }
        };
        // Returns list of contacts for the previous match request in MAX_RESULTS chunks
        // Will remove contacts from cache that have neither an email or phone
        DeviceContacts.matchDetails = function (callback, bitmapCallback) {
            // Protect for failed initializtion
            if (DeviceContacts.allContacts == null)
                callback(null);
            if (DeviceContacts.lastSearchResults == null)
                callback(null);
            // Save callbacks
            DeviceContacts.detailCallback = callback;
            DeviceContacts.bitmapCallback = bitmapCallback;
            // Now start loading contact details
            DeviceContacts.loadDetails();
        };
        // Load contact details in chunks of size MAX_RESULTS
        DeviceContacts.loadDetails = function () {
            // Determine which contacts still need to have details loaded
            var unloaded = [];
            for (var i = 0; i < Math.min(DeviceContacts.MAX_RESULTS, DeviceContacts.lastSearchResults.length); i++) {
                var curContact = DeviceContacts.lastSearchResults[i];
                if (!curContact.loaded) {
                    unloaded.push(curContact);
                }
            }
            // If all contacts already have details, assemble results immediately 
            if (unloaded.length == 0) {
                DeviceContacts.AssembleResults();
                return;
            }
            // Otherwise, create list of unloaded IDs and load the details
            var unloadedIds = [];
            for (i = 0; i < unloaded.length; i++) {
                unloadedIds.push(unloaded[i].deviceId);
            }
            client.logger.info('about to call contacts plugin to getDetails');
            DeviceContacts.contactProvider().getdetails(unloadedIds, function (profiles) {
                if (_.isString(profiles)) {
                    profiles = JSON.parse(profiles);
                }
                DeviceContacts.detailSuccess(profiles, unloaded);
            }, function (err) {
                DeviceContacts.detailFailed(err, unloaded);
            });
        };
        // Find cached contact index by contact id
        DeviceContacts.indexFromId = function (deviceId) {
            for (var i = 0; i < DeviceContacts.allContacts.length; i++) {
                if (DeviceContacts.allContacts[i].deviceId == deviceId) {
                    return i;
                }
            }
            return null;
        };
        // Find cached contact by contact id
        DeviceContacts.contactFromId = function (deviceId) {
            for (var i = 0; i < DeviceContacts.allContacts.length; i++) {
                if (DeviceContacts.allContacts[i].deviceId == deviceId) {
                    return DeviceContacts.allContacts[i];
                }
            }
            return null;
        };
        // Find cached contact by contact displayName
        DeviceContacts.contactFromDisplayName = function (displayName) {
            for (var i = 0; i < DeviceContacts.allContacts.length; i++) {
                if (DeviceContacts.allContacts[i].displayName == displayName) {
                    return DeviceContacts.allContacts[i];
                }
            }
            return null;
        };
        // Called by plugin when contact details have been loaded.  Returns contacts to caller
        DeviceContacts.detailSuccess = function (detailedContacts, unloaded) {
            client.logger.info('contact details succeeded', detailedContacts ? detailedContacts.length : null);
            for (var i = 0; detailedContacts && i < detailedContacts.length; i++) {
                var curContact = detailedContacts[i];
                var cacheIndex = DeviceContacts.indexFromId(curContact['id']);
                // If contact doesn't have an email or phone remove from available list
                if (!curContact.email && !curContact.phone) {
                    DeviceContacts.allContacts.splice(cacheIndex, 1);
                    var index = -1;
                    _.find(DeviceContacts.lastSearchResults, function (c, i) {
                        index = i;
                        return c.deviceId == curContact.deviceId;
                    });
                    if (index >= 0) {
                        DeviceContacts.lastSearchResults.splice(index, 1);
                    }
                }
                else {
                    var cacheContact = DeviceContacts.allContacts[cacheIndex];
                    cacheContact.email = curContact.email;
                    cacheContact.phone = curContact.phone;
                    cacheContact.loaded = true;
                }
            }
            if (detailedContacts.length == 0) {
                // filter out all unloaded contacts
                _.each(unloaded, function (contact) {
                    var index = DeviceContacts.indexFromId(contact['id']);
                    if (index >= 0) {
                        DeviceContacts.allContacts.splice(index, 1);
                    }
                    index = -1;
                    _.find(DeviceContacts.lastSearchResults, function (c, i) {
                        index = i;
                        return c.deviceId == contact.deviceId;
                    });
                    if (index >= 0) {
                        DeviceContacts.lastSearchResults.splice(index, 1);
                    }
                });
            }
            // Now assemble results for callback
            DeviceContacts.AssembleResults();
        };
        // Called by plugin when details load has failed
        DeviceContacts.detailFailed = function (error, failedContacts) {
            client.logger.error(consts.errorCode.Contacts, 'Contact detail load failed.');
            // trim out contacts that can't be filled (missing both email and phone)            
            _.each(failedContacts, function (contact) {
                var index = DeviceContacts.indexFromId(contact['id']);
                if (index >= 0) {
                    DeviceContacts.allContacts.splice(index, 1);
                }
                _.find(DeviceContacts.lastSearchResults, function (c, i) {
                    index = i;
                    return c.deviceId == contact.deviceId;
                });
                DeviceContacts.lastSearchResults.splice(index, 1);
            });
            DeviceContacts.AssembleResults();
        };
        // Assemble found contacts and return to callback
        DeviceContacts.AssembleResults = function () {
            client.logger.info('assemble detailed contact results');
            // Make sure request hasn't been calcelled
            if (!DeviceContacts.detailCallback)
                return;
            // Assemble next chunk of results of size MAX_RESULTS
            var assembledContacts = [];
            for (var i = 0; i < Math.min(DeviceContacts.MAX_RESULTS, DeviceContacts.lastSearchResults.length); i++) {
                var curContact = DeviceContacts.lastSearchResults[i];
                if (curContact) {
                    if (!curContact.loaded) {
                        client.logger.warn('sending incomplete contact result to UI', curContact);
                    }
                    assembledContacts.push(curContact);
                }
            }
            // Remove returned items from results list
            DeviceContacts.lastSearchResults.splice(0, DeviceContacts.MAX_RESULTS);
            // Return contacts and count of remaining
            DeviceContacts.detailCallback(assembledContacts, DeviceContacts.lastSearchResults.length);
            // If there are remaining items, start loading them
            if (DeviceContacts.lastSearchResults.length > 0) {
                DeviceContacts.loadDetails();
            }
            for (i = 0; i < assembledContacts.length; i++) {
                if (!assembledContacts[i].gotThumbnail) {
                    assembledContacts[i].gotThumbnail = true;
                    DeviceContacts.contactProvider().getbitmap(assembledContacts[i].deviceId, assembledContacts[i].displayName, _.bind(DeviceContacts.bitmapSuccess, this), _.bind(DeviceContacts.bitmapFail, this));
                }
            }
        };
        // Called by plugin when contact bitmap succesfully loaded
        DeviceContacts.bitmapSuccess = function (bitmapResponse) {
            if (_.isString(bitmapResponse)) {
                if (bitmapResponse) {
                    bitmapResponse = JSON.parse(bitmapResponse);
                }
                else {
                    // empty string means that we are going to get the picture later
                    return;
                }
            }
            // Find contact
            var contact = null;
            if (bitmapResponse.index) {
                contact = DeviceContacts.contactFromId(bitmapResponse.index);
            }
            else if (bitmapResponse.displayName) {
                contact = DeviceContacts.contactFromDisplayName(bitmapResponse.displayName);
            }
            // Set thumbnailUrl
            if (!contact)
                return;
            contact.thumbnailUrl = bitmapResponse.imageBase64;
            // Return bitmap to whomever requested it
            if (DeviceContacts.bitmapCallback) {
                DeviceContacts.bitmapCallback(contact);
            }
        };
        // Called by plugin when contact bitmap failed to load.  Contact may not have a photo
        DeviceContacts.bitmapFail = function (bitmapResponse) {
        };
        DeviceContacts.allContacts = null; // Cache of all contacts
        DeviceContacts.pendingSearchTerm = null; // Requested search not yet completed
        DeviceContacts.lastSearchResults = null; // Contacts that match last search
        DeviceContacts.MAX_RESULTS = 8; // Max results to return at a time
        // Callbacks
        DeviceContacts.matchCallback = null;
        DeviceContacts.detailCallback = null;
        DeviceContacts.bitmapCallback = null;
        /** Load names & ids for all available contacts from device, can only be called once */
        DeviceContacts.initContacts = _.once(DeviceContacts._initContacts);
        return DeviceContacts;
    })();
    client.DeviceContacts = DeviceContacts;
})(client || (client = {}));
/// <reference path='../shared/consts.ts'/>
/// <reference path='DeviceContacts.ts'/>
/// <reference path='../../servers/shared/proximity-schema.d.ts'/>
var client;
(function (client) {
    /** Keeps cache of people user has recently Xim'd with */
    var RecentPeople = (function () {
        function RecentPeople() {
        }
        // Return time ordered list of recent contacts
        RecentPeople.Get = function () {
            // Make sure cache is current
            RecentPeople.refreshCache();
            // Create sorted array
            var profiles = [];
            for (var key in RecentPeople.recent) {
                var profile = RecentPeople.recent[key]['profile'];
                profile.isNearby = false;
                profiles.push(profile);
            }
            profiles.sort(function (a, b) {
                var timeA = RecentPeople.recent[a.deviceId]['time'];
                var timeB = RecentPeople.recent[b.deviceId]['time'];
                return timeA < timeB ? 1 : timeA > timeB ? -1 : 0;
            });
            // Return it
            return profiles;
        };
        RecentPeople.Match = function (memberMap) {
            RecentPeople.refreshCache();
            for (var groupId in memberMap) {
                var groupMember = memberMap[groupId];
                for (var recentId in RecentPeople.recent) {
                    var recentProfile = RecentPeople.recent[recentId].profile;
                    if (groupMember.inviteGuid != null && groupMember.inviteGuid == recentProfile.inviteGuid) {
                        recentProfile.userId = groupMember.userId;
                        recentProfile.inviteType = groupMember.inviteType;
                        if (groupMember.color) {
                            recentProfile.color = groupMember.color;
                        }
                        if (groupMember.thumbnailUrl) {
                            recentProfile.thumbnailUrl = groupMember.thumbnailUrl;
                        }
                        if (groupMember.displayName) {
                            recentProfile.displayName = groupMember.displayName;
                        }
                    }
                }
            }
            localStorage.setItem('recentpeople', JSON.stringify(RecentPeople.recent));
        };
        RecentPeople.Update = function (profiles) {
            // Make sure cache is current
            RecentPeople.refreshCache();
            // Add profiles to cache / update recency
            var time = consts.time.now();
            for (var i = 0; i < profiles.length; i++) {
                var profile = profiles[i];
                // Make sure nearby setting is cleared
                profile.isNearby = false;
                // Give profile a guid if doesn't have one yet
                if (!profile.inviteGuid) {
                    profile.inviteGuid = client.Util.generateGUID();
                }
                RecentPeople.recent[profile.deviceId] = {
                    profile: profile,
                    time: time
                };
            }
            localStorage.setItem('recentpeople', JSON.stringify(RecentPeople.recent));
            // Note that recent list has been updated
            RecentPeople.onRecentUsersChanged.trigger({});
        };
        // Clear cache of recent people
        RecentPeople.Clear = function () {
            localStorage.removeItem('recentpeople');
            RecentPeople.recent = null;
        };
        // Refresh recent people from cache if necessary
        RecentPeople.refreshCache = function () {
            // Load from local store if not inialized
            if (!RecentPeople.recent) {
                var recent = localStorage.getItem('recentpeople');
                if (recent) {
                    RecentPeople.recent = JSON.parse(recent);
                    if (RecentPeople.recent) {
                        // filter out all invalid entry
                        var validRecent = {};
                        _.each(RecentPeople.recent, function (r, id) {
                            if (r.profile && r.profile.deviceId) {
                                validRecent[id] = r;
                            }
                        });
                        RecentPeople.recent = validRecent;
                    }
                }
                else {
                    RecentPeople.recent = {};
                }
            }
        };
        /** Fired anytime new nearby users are added to recent list. */
        RecentPeople.onRecentUsersChanged = new client.Event();
        RecentPeople.recent = null;
        return RecentPeople;
    })();
    client.RecentPeople = RecentPeople;
})(client || (client = {}));
/// <reference path="types.ts"/>
/// <reference path="consts.ts"/>
/// <reference path="../proximity/types.d.ts"/>
/// <reference path="../../types/cordova/cordova-chromecast.d.ts"/>
/// <reference path="../mobile/user.ts"/>
var client;
(function (client) {
    var cast;
    (function (cast) {
        ;
        cast.googleCastClient = null;
        var GoogleCast = (function () {
            function GoogleCast(chromecastAppIdentifier) {
                this.chrome = null;
                this._session = null;
                this.onSessionUpdated = new client.Event();
                this.chromeAppID = chromecastAppIdentifier;
                if (window.chrome.cast) {
                    this.chrome = window.chrome;
                }
                this._onXimJoined = _.bind(this.onXimJoined, this);
                client.user.onXimJoined.add(this._onXimJoined);
            }
            GoogleCast.prototype.dispose = function () {
                client.user.onXimJoined.remove(this._onXimJoined);
            };
            GoogleCast.prototype.init = function (listener, callback) {
                var _this = this;
                if (this.chrome && this.chrome.cast) {
                    var sessionRequest = new this.chrome.cast.SessionRequest(this.chromeAppID.appKey);
                    this.chrome.cast.ApiConfig;
                    var apiConfig = new this.chrome.cast.ApiConfig(sessionRequest, function (session) {
                        _this.sessionListener(session);
                    }, function (available) {
                        client.logger.info('receiver callback', available);
                        listener(available == 'available' ? true : false);
                    });
                    this.chrome.cast.initialize(apiConfig, function () {
                        client.logger.info('chromecast API initialized!');
                        if (callback)
                            callback(null, true);
                    }, function (err) {
                        client.logger.error(consts.errorCode.ChromeCast, _this.getCastError(err).message);
                        if (callback)
                            callback(_this.getCastError(err), false);
                    });
                }
                else {
                    //google cast API is not available
                    if (callback)
                        callback(new Error('chromecast API not available'), false);
                }
            };
            GoogleCast.prototype.requestSession = function (callback) {
                var _this = this;
                if (this.chrome) {
                    this.chrome.cast.requestSession(function (session) {
                        _this._session = session;
                        _this.sendMessage({ dataCenterRoot: client.server.root }, callback); //launching the receiver app on google cast device
                    }, function (err) {
                        if (callback)
                            callback(_this.getCastError(err), false);
                    });
                }
                else {
                    if (callback)
                        callback(new Error('chromecast API not available'), false);
                }
            };
            GoogleCast.prototype.getRoutes = function () {
                if (this.chrome) {
                    var routes = _.map(_.values(this.chrome.cast.getRoutes()), function (route) {
                        return {
                            id: route.id.replace('/', '_b_'),
                            name: route.name
                        };
                    });
                    return routes;
                }
                else {
                    return [];
                }
            };
            GoogleCast.prototype.selectRoute = function (id, callback) {
                var _this = this;
                if (this.chrome) {
                    id = id.replace('_b_', '/');
                    this.chrome.cast.selectRoute(id, function (session) {
                        _this._session = session;
                        _this._session.addUpdateListener(function (isAlive) {
                            _this.onSessionUpdated.trigger(isAlive);
                        });
                        _this.sendMessage({ dataCenterRoot: client.server.root }, callback); //launching the receiver app on google cast device
                    }, function (err) {
                        if (callback)
                            callback(_this.getCastError(err), false);
                    });
                }
                else {
                    if (callback)
                        callback(new Error('chromecast API not available'), false);
                }
            };
            GoogleCast.prototype.leave = function (callback) {
                var _this = this;
                if (this.chrome) {
                    if (this._session) {
                        this._session.leave(function () {
                            _this._session = null;
                            if (callback) {
                                callback(null, true);
                            }
                        }, function (err) {
                            if (callback) {
                                callback(_this.getCastError(err), false);
                            }
                        });
                    }
                    else {
                        callback(null, true);
                    }
                }
                else {
                    if (callback)
                        callback(new Error('chromecast API not available'), false);
                }
            };
            GoogleCast.prototype.sendMessage = function (msg, callback) {
                var _this = this;
                if (this._session) {
                    this._session.sendMessage(consts.urns.chromecastNamespace, JSON.stringify(msg), function () {
                        if (callback)
                            callback(null, true);
                    }, function (err) {
                        if (callback)
                            callback(_this.getCastError(err), false);
                    });
                }
            };
            GoogleCast.prototype.onXimJoined = function (msg) {
                this.sendMessage(msg);
            };
            GoogleCast.prototype.sessionListener = function (session) {
                client.logger.info('chromecast session Listener called');
                this._session = session;
            };
            GoogleCast.prototype.getCastError = function (err) {
                client.logger.error(consts.errorCode.ChromeCast, '', err);
                var errMsg = err.code ? 'errCode: ' + err.code.toString() : '';
                errMsg += err.description ? ', errDesc: ' + err.description : '';
                return new Error(errMsg);
            };
            return GoogleCast;
        })();
        cast.GoogleCast = GoogleCast;
    })(cast = client.cast || (client.cast = {}));
})(client || (client = {}));
/// <reference path='types.ts'/>
/// <reference path='../shared/models/LocalTable.ts'/>
/// <reference path='../shared/models/ModelCache.ts'/>
/// <reference path='Connection.ts'/>
/// <reference path='../../servers/shared/proximity-schema.d.ts'/>
/// <reference path='scanner.ts'/>
/// <reference path='Group.ts'/>
/// <reference path='resources.ts'/>
/// <reference path='user-services.ts'/>
/// <reference path='user-groups.ts'/>
/// <reference path='user-discovery.ts'/>
/// <reference path='recentPeople.ts'/>
/// <reference path='../../types/moment/moment.d.ts'/>
/// <reference path='../shared/googleCast.ts'/>
var client;
(function (client) {
    client.urlUpsell;
    client.baseUri;
    client.pin;
    client.startupMode;
    client.isPhoneGap;
    /** Is display on a desktop */
    client.isDesktopDisplay;
    /** Is display interactive (i.e. takes user input via click or tap) */
    client.isInteractiveDisplay;
    /** If true the client is being rendered on a Fire TV. */
    client.isFireTv;
})(client || (client = {}));
var client;
(function (client) {
    var user;
    (function (user) {
        (function (LoginService) {
            LoginService[LoginService["fb"] = 0] = "fb";
            LoginService[LoginService["test"] = 1] = "test";
            LoginService[LoginService["display"] = 2] = "display";
        })(user.LoginService || (user.LoginService = {}));
        var LoginService = user.LoginService;
        (function (ConnectionStates) {
            ConnectionStates[ConnectionStates["logout"] = 0] = "logout";
            ConnectionStates[ConnectionStates["authorizing"] = 1] = "authorizing";
            ConnectionStates[ConnectionStates["authorized"] = 2] = "authorized";
            ConnectionStates[ConnectionStates["connecting"] = 3] = "connecting";
            ConnectionStates[ConnectionStates["connected"] = 4] = "connected";
            ConnectionStates[ConnectionStates["reconnecting"] = 5] = "reconnecting";
        })(user.ConnectionStates || (user.ConnectionStates = {}));
        var ConnectionStates = user.ConnectionStates;
        (function (NearbyFilters) {
            NearbyFilters[NearbyFilters["all"] = 0] = "all";
            NearbyFilters[NearbyFilters["users"] = 1] = "users";
            NearbyFilters[NearbyFilters["groups"] = 2] = "groups";
        })(user.NearbyFilters || (user.NearbyFilters = {}));
        var NearbyFilters = user.NearbyFilters;
        (function (AuthorizationState) {
            AuthorizationState[AuthorizationState["notLinked"] = 0] = "notLinked";
            AuthorizationState[AuthorizationState["notVerified"] = 1] = "notVerified";
            AuthorizationState[AuthorizationState["notAuthorized"] = 2] = "notAuthorized";
            AuthorizationState[AuthorizationState["notRegistered"] = 3] = "notRegistered";
            AuthorizationState[AuthorizationState["notConnected"] = 4] = "notConnected";
            AuthorizationState[AuthorizationState["Connected"] = 5] = "Connected";
        })(user.AuthorizationState || (user.AuthorizationState = {}));
        var AuthorizationState = user.AuthorizationState;
        (function (VerifyType) {
            VerifyType[VerifyType["sms"] = 0] = "sms";
            VerifyType[VerifyType["voice"] = 1] = "voice";
        })(user.VerifyType || (user.VerifyType = {}));
        var VerifyType = user.VerifyType;
        /** Default user settings. */
        user.defaultSettings = {
            discovery: {
                setting: consts.NearbyDiscoveryMode.incognito,
                timestamp: consts.time.now()
            }
        };
        //=====================================================
        // Events
        //=====================================================
        /** Fired anytime the current connection state changes. */
        user.onConnectionState = new client.Event();
        /** Fired anytime new nearby users are discovered. */
        user.onNearbyUsersAdded = new client.Event();
        /** Fired anytime a nearby user disapears. */
        user.onNearbyUsersRemoved = new client.Event();
        /** Fired anytime a new session invite is recieved. */
        user.onSessionInvite = new client.Event();
        /** Fired anytime a user logs out */
        user.onLogout = new client.Event();
        /** Fired anytime a user joins a cast */
        user.onXimJoined = new client.Event();
        //=====================================================
        // Properties
        //=====================================================
        /** Proximity cluster the client has been assigned to. */
        user.cluster;
        /** Clients current user session. */
        user.session = null;
        /** Number of users to simulate for testing purposes. */
        user.simulatedUsers = parseInt(localStorage.getItem('simulated-users') || '0');
        /** Overrides the type of login that occurs. */
        user.loginOverride = localStorage.getItem('login-override') || '';
        /** Type of users account. Can be 'fb', 'test', 'guest', or 'display'.  */
        user.accountType;
        /** Logged in users profile. */
        user.profile = null;
        /** Logged in users settings. */
        user.settings = null;
        /** Access tokens for the users linked accounts. */
        user.accountTokens = {};
        /** If true the discovery services are running. Will be false when user is incognito.  */
        user.discoveryStarted = false;
        /** Access token for logged in user. */
        user.accessToken = null;
        /** Authorization message sent from sever */
        user.serverMessage = null;
        /** The authorized users country. */
        user.country;
        /** The users current xim info. */
        user.currentXim = null;
        //=================================================
        // Local Settings 
        //=================================================
        (function (UserSetting) {
            UserSetting[UserSetting["USE_MOCK_DATA"] = 0] = "USE_MOCK_DATA";
            UserSetting[UserSetting["ENABLE_CELEBRATIONS"] = 1] = "ENABLE_CELEBRATIONS";
            UserSetting[UserSetting["ENABLE_PRESENTER_MODE"] = 2] = "ENABLE_PRESENTER_MODE";
            UserSetting[UserSetting["ENABLE_NEARBY_MODE"] = 3] = "ENABLE_NEARBY_MODE";
            UserSetting[UserSetting["_OPEN_SLOT_"] = 4] = "_OPEN_SLOT_";
            UserSetting[UserSetting["SHOW_SYSTEM_MESSAGES"] = 5] = "SHOW_SYSTEM_MESSAGES";
            UserSetting[UserSetting["SHOW_DEBUG"] = 6] = "SHOW_DEBUG";
            UserSetting[UserSetting["SHOW_CAMERA_AS_SOURCE"] = 7] = "SHOW_CAMERA_AS_SOURCE";
            UserSetting[UserSetting["HIGHLIGHT_NEARBY"] = 8] = "HIGHLIGHT_NEARBY";
            UserSetting[UserSetting["SHOW_EXPIRED"] = 9] = "SHOW_EXPIRED";
            UserSetting[UserSetting["CAST_MIN_TIME"] = 10] = "CAST_MIN_TIME";
            UserSetting[UserSetting["SHOWN_FTUX"] = 11] = "SHOWN_FTUX";
            UserSetting[UserSetting["SHOW_CAMERAROLL"] = 12] = "SHOW_CAMERAROLL";
            UserSetting[UserSetting["WAITING_FACEBOOK"] = 13] = "WAITING_FACEBOOK";
            UserSetting[UserSetting["DISABLE_AIRPLAY"] = 14] = "DISABLE_AIRPLAY";
            UserSetting[UserSetting["SHOW_CAMERA_AS_BUTTON"] = 15] = "SHOW_CAMERA_AS_BUTTON";
        })(user.UserSetting || (user.UserSetting = {}));
        var UserSetting = user.UserSetting;
        ;
        (function (SettingType) {
            SettingType[SettingType["BOOLEAN"] = 0] = "BOOLEAN";
            SettingType[SettingType["STRING"] = 1] = "STRING";
        })(user.SettingType || (user.SettingType = {}));
        var SettingType = user.SettingType;
        var localSettings = {};
        localSettings[0 /* USE_MOCK_DATA */] = { type: 0 /* BOOLEAN */, name: 'useMockData', description: resources.forms.useMockData, value: undefined };
        localSettings[5 /* SHOW_SYSTEM_MESSAGES */] = { type: 0 /* BOOLEAN */, name: 'showSystemMessages', description: resources.forms.showSystemMessages, value: undefined };
        localSettings[6 /* SHOW_DEBUG */] = { type: 0 /* BOOLEAN */, name: 'showDebug', description: resources.forms.showDebug, value: undefined };
        localSettings[8 /* HIGHLIGHT_NEARBY */] = { type: 0 /* BOOLEAN */, name: 'highlightNearby', description: resources.forms.highlightNearby, value: undefined };
        localSettings[9 /* SHOW_EXPIRED */] = { type: 0 /* BOOLEAN */, name: 'showExpired', description: resources.forms.showExpired, value: undefined };
        localSettings[1 /* ENABLE_CELEBRATIONS */] = { type: 0 /* BOOLEAN */, name: 'enableCelebrations', description: resources.forms.enableCelebrations, value: undefined };
        localSettings[2 /* ENABLE_PRESENTER_MODE */] = { type: 0 /* BOOLEAN */, name: 'enablePresenter', description: resources.forms.enablePresenter, value: undefined };
        localSettings[3 /* ENABLE_NEARBY_MODE */] = { type: 0 /* BOOLEAN */, name: 'enableNearby', description: resources.forms.enableNearby, value: undefined };
        localSettings[10 /* CAST_MIN_TIME */] = { type: 1 /* STRING */, name: 'castMinItme', description: '', value: undefined };
        localSettings[11 /* SHOWN_FTUX */] = { type: 0 /* BOOLEAN */, name: 'shownFtux', description: resources.forms.shownFtux, value: undefined };
        localSettings[12 /* SHOW_CAMERAROLL */] = { type: 0 /* BOOLEAN */, name: 'showCameraRoll', description: resources.forms.showCameraRoll, value: true };
        localSettings[13 /* WAITING_FACEBOOK */] = { type: 0 /* BOOLEAN */, name: 'waitingFacebook', description: 'waiting for facebook', value: undefined };
        localSettings[14 /* DISABLE_AIRPLAY */] = { type: 0 /* BOOLEAN */, name: 'disableAirplay', description: 'disable projecting to airplay', value: undefined };
        localSettings[7 /* SHOW_CAMERA_AS_SOURCE */] = { type: 0 /* BOOLEAN */, name: 'showCameraAsSource', description: 'Show camera in picker', value: true };
        localSettings[15 /* SHOW_CAMERA_AS_BUTTON */] = { type: 0 /* BOOLEAN */, name: 'showCameraAsButton', description: 'Show camera in cameraroll', value: undefined };
        var localTokens = localStorage.getItem('client.user.accountTokens');
        if (localTokens) {
            user.accountTokens = JSON.parse(localTokens);
        }
        /** Return value of a user setting */
        function GetSetting(settingType) {
            // If setting not loaded, load it
            // REVIEW: looks like we don't allow localStorage to override hard coded defaults,
            // is it intentional?
            if (localSettings[settingType].value == undefined) {
                var value = localStorage.getItem(localSettings[settingType].name);
                // Default value
                if (localSettings[settingType].type == 0 /* BOOLEAN */) {
                    localSettings[settingType].value = value && value !== 'false';
                }
                else {
                    localSettings[settingType].value = value;
                }
            }
            return localSettings[settingType].value;
        }
        user.GetSetting = GetSetting;
        /** Set a user setting */
        function SetSetting(settingType, value) {
            localSettings[settingType].value = value;
            localStorage.setItem(localSettings[settingType].name, value);
        }
        user.SetSetting = SetSetting;
        /** Return all user settings */
        function GetSettings() {
            for (var settingType in localSettings) {
                GetSetting(settingType);
            }
            return localSettings;
        }
        user.GetSettings = GetSettings;
        function ResetSettings() {
            _.each(localSettings, function (val) {
                localStorage.removeItem(val.name);
            });
        }
        //=====================================================
        // Private State
        //=====================================================
        var connectState = 0 /* logout */;
        var _connection;
        var _deviceConnecting = new client.AsyncEvent();
        var _profileCache = null;
        var dataCenter = localStorage.getItem('dataCenter') || '';
        var verifyToken = localStorage.getItem('verify-token') || '';
        var deviceToken = localStorage.getItem('device-token') || '';
        var _viewer;
        //=====================================================
        // Methods
        //=====================================================
        /** WP ONLY! re-initialize tokens from local storage. For localstorage that might have been stubbed */
        function checkTokensFromStorage() {
            dataCenter = dataCenter || localStorage.getItem('dataCenter') || '';
            verifyToken = verifyToken || localStorage.getItem('verify-token') || '';
            deviceToken = deviceToken || localStorage.getItem('device-token') || '';
            if (!localTokens) {
                localTokens = localStorage.getItem('client.user.accountTokens');
                if (localTokens) {
                    user.accountTokens = JSON.parse(localTokens);
                }
            }
            if (verifyToken && !localStorage.getItem('verify-token')) {
                client.logger.info('upgrade user from localStorage to appStorage');
                // for beta user that had localStorage and upgrade to 1.0.1 with appStorage
                localStorage.setItem('dataCenter', dataCenter);
                localStorage.setItem('verify-token', verifyToken);
                localStorage.setItem('device-token', deviceToken);
                localStorage.setItem('client.user.accountTokens', localTokens);
            }
            else {
                client.logger.info('no need to upgrade, verifyToken = ', verifyToken);
            }
        }
        /** Initializes the user object. */
        function init(cb) {
            // for windows phone, we might be invoked from push notification
            // from PIN locked screen, requery for the various tokens
            if (client.startupMode == 3 /* wp */) {
                checkTokensFromStorage();
            }
            if (deviceToken || verifyToken) {
                // Check for upgrade of a 1.1 user to a newer version.
                // - 1.1 clients were designed to talk to the america data center and need to start off doing that.
                //   For the out of country users we have that installed the old client their authorize call will fail
                //   with an error code indicating the user couldn't be found or was moved. This will trigger them to
                //   go back through teh account setup process resulting a new call to linkDevice() which will home them 
                //   to  the correct data center.
                if (!dataCenter) {
                    dataCenter = 'america';
                    localStorage.setItem('dataCenter', 'america');
                }
                // Initialize data center
                client.server.setDataCenter(dataCenter);
            }
            cb(null, true);
        }
        user.init = init;
        /** Returns data center being used */
        function getDataCenter() {
            return dataCenter;
        }
        user.getDataCenter = getDataCenter;
        /** Determines where in the authorization/device linking flow the app is. */
        function getAuthorizationState() {
            if (!verifyToken)
                return 0 /* notLinked */;
            if (!deviceToken)
                return 1 /* notVerified */;
            if (!user.accessToken || !user.profile)
                return 2 /* notAuthorized */;
            if (!user.profile.isRegistered)
                return 3 /* notRegistered */;
            if (!user.session)
                return 4 /* notConnected */;
            return 5 /* Connected */;
        }
        user.getAuthorizationState = getAuthorizationState;
        function showFTUX() {
            var authorizationState = user.getAuthorizationState();
            return (authorizationState == 0 /* notLinked */);
        }
        user.showFTUX = showFTUX;
        /** Creates the data center Connection object on first access. */
        function getConnection() {
            if (!_connection)
                _connection = new client.Connection();
            return _connection;
        }
        user.getConnection = getConnection;
        /**
          * Initiates the send of an SMS to the user with a code for linking the device to their account.
          * @param phoneNumber The phone number for the users account. An SMS with a code will be sent to this number.
          * @param deviceLabel The name of the device for linking purposes. The user may have one device with a given label linked at any one time.
          * @param country The users selected 2 character country.
          * @param verifyType Type of verification to send.
          * @param cb Callback function to eturn the results of the server call. The status param will always be filled out (even in an Error)
          * and will contain a code indicating the success or type of failure that occured.  Possible values:
          * - success: The server call was successful.
          * - badRequest: The phone number wasn't properly formed.
          * - forbidden: The number was a test number (1-555 prefixed) and isn't allowed by the server.
          * - serverError: An internal error occured while processing request. Likely a storage call failed.
          */
        function linkDevice(phoneNumber, deviceLabel, country, verifyType, cb) {
            var s = client.logger.start();
            client.logger.info('device: linking "' + deviceLabel + '"');
            // Identify data center
            if (!consts.disableServerDataCenter && location.hostname.indexOf('localhost') >= 0) {
                // User the server the page was downloaded from (typically ripple debugging)
                dataCenter = 'server';
            }
            else if (consts.forceDataCenter) {
                dataCenter = consts.forceDataCenter;
            }
            else {
                var iso3166 = consts.getIso3166(country);
                if (iso3166) {
                    switch (iso3166.region) {
                        case 1 /* america */:
                        default:
                            dataCenter = 'america';
                            break;
                        case 3 /* emea */:
                            dataCenter = 'europe';
                            break;
                        case 2 /* pacific */:
                            dataCenter = 'asia';
                            break;
                    }
                }
                else {
                    dataCenter = 'america';
                }
            }
            localStorage.setItem('dataCenter', dataCenter);
            client.server.setDataCenter(dataCenter);
            // Link device
            getConnection().GET('api/linkDevice', { label: deviceLabel || 'mobile', phone: phoneNumber, country: country, verifyType: VerifyType[verifyType] }, function (err, token) {
                if (!err) {
                    client.logger.stop(s, 'device: verification code sent');
                    verifyToken = token.token;
                    localStorage.setItem('verify-token', verifyToken);
                    cb(null, 0 /* succeeded */);
                }
                else {
                    client.logger.stopError(s, 'linkDevice', 'device: unable to link device - ' + err.message);
                    cb(err, client.server.getRequestStatus(err));
                }
            });
        }
        user.linkDevice = linkDevice;
        /**
          * Completes the process of linking a device to a users account.
          * @param code The 4-digit pin number entered by the user matching the code they recieved via SMS.
          * @param cb Callback function to eturn the results of the server call. The status param will always be filled out (even in an Error)
          * and will contain a code indicating the success or type of failure that occured.  Possible values:
          * - success: The server call was successful.
          * - tokenExpired: The user didn't enter the code within the alloted time and the verifyToken we returned has expired.
          * - notFound: The verify token was valid but we couldn't find a record for the user. Something unexpected happened and we should start over.
          * - forbidden: The user has atttempted to enter a code more than 5 times and are now blocked. We should start over and ask for a new code.
          * - notAuthorized: The user entered an invalid code and should try again.
          * - serverError: An internal error occured while processing request. Likely a storage call failed.
          */
        function verifyDevice(code, cb) {
            var s = client.logger.start();
            client.logger.info('device: verifying code');
            getConnection().GET('api/verifyDevice', { token: verifyToken, code: code, os: client.StartupMode[client.startupMode] }, function (err, token) {
                if (!err) {
                    client.logger.stop(s, 'device: code verified');
                    deviceToken = token.token;
                    localStorage.setItem('device-token', deviceToken);
                    cb(null, 0 /* succeeded */);
                }
                else {
                    client.logger.stopError(s, 'verifyDevice', 'device: verification failed - ' + err.message);
                    cb(err, client.server.getRequestStatus(err));
                }
            });
        }
        user.verifyDevice = verifyDevice;
        /**
          * Authorizes the linked device by retrieving the users access token.
          * @param cb Callback function to return the results of the server call. The status param will always be filled out (even in an Error)
          * and will contain a code indicating the success or type of failure that occured.  Possible values:
          * - success: The server call was successful.
          * - forbidden: The device token is no longer valid and the user should go through the authorization flow again.
          * - notFound: The device token was valid but we couldn't find a record for the user. Something unexpected happened and we should start over.
          * - serverError: An internal error occured while processing request. Likely a storage call failed.
          */
        function authorizeDevice(cb) {
            if (connectState == 0 /* logout */) {
                var s = client.logger.start();
                client.logger.info('device: authorizing');
                setConnectionState(1 /* authorizing */);
                getConnection().GET('api/authorizeDevice', { token: deviceToken, version: consts.applicationVersion, os: client.StartupMode[client.startupMode] }, function (err, result) { return handleAuthorization(err, result, s, cb); });
            }
            else {
                cb(null, 0 /* succeeded */);
            }
        }
        user.authorizeDevice = authorizeDevice;
        /**
          * Switches to a new account. Used in viewers to change the authorized user.
          * @param token The access token for the account to switch to.
          * @param cb Callback function to return the results of the server call. The status param will always be filled out (even in an Error)
          * and will contain a code indicating the success or type of failure that occured.  Possible values:
          * - success: The server call was successful.
          * - forbidden: The device token is no longer valid and the user should go through the authorization flow again.
          * - notFound: The device token was valid but we couldn't find a record for the user. Something unexpected happened and we should start over.
          * - serverError: An internal error occured while processing request. Likely a storage call failed.
          */
        function switchAccounts(token, cb) {
            var s = client.logger.start();
            client.logger.info('device: switching to new account');
            disconnectDevice(function () {
                setConnectionState(1 /* authorizing */);
                getConnection().GET('api/switchAccounts', { token: token }, function (err, result) { return handleAuthorization(err, result, s, function (err, status) {
                    // TEMP: Wait for connection to complete
                    // - Won't be needed once Xims start calling ensureConnected() themselves
                    if (!err) {
                        ensureConnected(cb);
                    }
                    else {
                        cb(err, status);
                    }
                }); });
            });
        }
        user.switchAccounts = switchAccounts;
        /**
          * Connects the users device to the server on first call. Future calls will block until the device is connected or complete immediately if
          * the device is already connected.
          * @param cb (optional) callback function to return the results of the server call. The status param will always be filled out (even in an Error)
          * and will contain a code indicating the success or type of failure that occured.  Possible values:
          * - success: The device was successfully connected to the server.
          * - connectionError: An Engine.io connection with the server could not be established.
          * - serverError: The connection was established but we failed to join the UserSession.
          */
        function ensureConnected(cb) {
            function _handleDisconnect(err) {
                // Dispose of connection. This makes sure we start with clean slate on re-connect.
                if (_connection) {
                    _connection.dispose();
                    _connection = null;
                }
                // Return to authorized state
                setConnectionState(2 /* authorized */);
                // Release any waiting callers (ignored if event has already been signaled) 
                _deviceConnecting.signal(err || new Error('Error joining user session'), 7 /* serverError */);
                _deviceConnecting.reset(); // Ready for any reconnect attempt
            }
            function _joinUserSession() {
                var s = client.logger.start();
                getConnection().joinUserSession(user.accessToken.token, user.profile.userId, function (err, us) {
                    if (us) {
                        // Save user session state
                        user.session = us;
                        user.profile = user.session.profile;
                        user.settings = user.session.settings;
                        user.session.setSimulatedUser(user.simulatedUsers);
                        // Subscribe to various session events 
                        user.session.onResumeError.add(function () {
                            client.logger.warn('device: UserSession warning - unable to resume session');
                            _joinUserSession();
                        });
                        user.session.onProfileUpdated.add(function (data) { return user.profile = data; });
                        user.session.onSettingsUpdated.add(function (data) { return user.settings = data; });
                        user.session.onGroupsUpdated.add(function (ids) { return user.onGroupsUpdated.trigger(ids); });
                        // Signal connected
                        client.logger.stop(s, 'device: UserSession joined');
                        setConnectionState(4 /* connected */);
                        _deviceConnecting.signal(null, 0 /* succeeded */);
                    }
                    else {
                        // Unable to join user session
                        client.logger.stopError(s, 'ensureConnected', 'device: UserSession error - ' + err ? err.toString() : '');
                        _handleDisconnect(err);
                    }
                });
            }
            var tsReco = 0;
            function _monitorClientState(ev) {
                switch (ev.state) {
                    case 3 /* reconnecting */:
                        client.logger.warn('device: reconnecting');
                        tsReco = client.logger.start();
                        break;
                    case 2 /* connected */:
                        if (tsReco) {
                            client.logger.stop(tsReco, 'device: reconnected');
                            tsReco = 0;
                        }
                        break;
                    case 0 /* disconnected */:
                        // Unable to connect/reconnect
                        if (tsReco) {
                            client.logger.stopWarn(tsReco, 'device: failed reconnection');
                            tsReco = 0;
                        }
                        else {
                            client.logger.warn('device: disconnected');
                        }
                        _handleDisconnect();
                        break;
                }
            }
            ifAuthorized(function () {
                if (cb)
                    _deviceConnecting.wait(cb);
                if (connectState == 2 /* authorized */) {
                    client.logger.info('device: connecting');
                    setConnectionState(3 /* connecting */);
                    getConnection().onSocketStateChanged.add(_monitorClientState);
                    _joinUserSession();
                }
            }, cb);
        }
        user.ensureConnected = ensureConnected;
        /**  Disconnected the users device from the server. */
        function disconnectDevice(cb) {
            // Stope any nearby discovery
            user.stopDiscovery(true);
            // Leave user session
            if (user.session) {
                user.session.unjoin();
                user.session.dispose();
                user.session = null;
            }
            setConnectionState(0 /* logout */);
            // Disconnect from server
            if (_connection) {
                _connection.dispose();
                _connection = null;
            }
            if (cb)
                cb(null, 0 /* succeeded */);
        }
        user.disconnectDevice = disconnectDevice;
        /** Unlinks the current device from a users account without deleting their account. */
        function unlinkDevice(cb) {
            async.series([disconnectDevice, function (next) {
                // Close any open sockets
                client.closeSockets();
                // Clear all client state
                if (window.userprofile) {
                    window.userprofile.logOff(function () {
                    }, function (e) {
                        client.logger.info('user log off failed', e);
                    });
                }
                user.accountType = '';
                user.profile = null;
                user.settings = null;
                user.accountTokens = {};
                user.discoveryStarted = false;
                user.accessToken = null;
                _profileCache = null;
                dataCenter = '';
                verifyToken = '';
                deviceToken = '';
                // Todo: need to clear out other local storage items in aditon to those below
                localStorage.removeItem('client.user.accountTokens');
                localStorage.removeItem('dataCenter');
                localStorage.removeItem('device-token');
                localStorage.removeItem('verify-token');
                ResetSettings();
                // Clear recent people cache
                client.RecentPeople.Clear();
                // Store current time.  This allow us to filter out old casts (until server side implimented)
                user.SetSetting(10 /* CAST_MIN_TIME */, moment());
                user.onLogout.trigger(true);
                next(null, null);
            }], function (err, r) {
                cb(err, client.server.getRequestStatus(err));
            });
        }
        user.unlinkDevice = unlinkDevice;
        /** Deletes the linked users account. */
        function deleteAccount(cb) {
            var s = client.logger.start();
            client.logger.info('device: deleting users account.');
            ifAuthorized(function () {
                getConnection().GET('api/deleteAccount', null, function (err) {
                    if (!err) {
                        client.logger.stop(s, 'device: account deleted');
                        unlinkDevice(cb);
                    }
                    else {
                        client.logger.stopError(s, 'deleteAccount', 'device: error deleting account: ' + err.message);
                        cb(err, client.server.getRequestStatus(err));
                    }
                });
            }, cb);
        }
        user.deleteAccount = deleteAccount;
        /**
          * Updates the users profile.
          * @param changes The fields to update. These changes will be submitted to the server as a set of transforms and then applied to the local object on success.
          * @param cb Callback function to eturn the results of the server call. The status param will always be filled out (even in an Error)
          * and will contain a code indicating the success or type of failure that occured.  Possible values:
          * - success: The server call was successful.
          * - notAuthorized: The users access token isn't valid and we should call authorizeDevice() to refresh it.
          * - serverError: An internal error occured while processing request. Likely a storage call failed.
          */
        function updateProfile(changes, cb) {
            ifAuthorized(function () {
                if (changes.id)
                    delete changes.id;
                if (changes.userId)
                    delete changes.userId;
                if (!user.profile.isRegistered) {
                    changes.isRegistered = true;
                }
                getConnection().POST('api/updateProfile', null, client.Util.toModelTransforms(changes), function (err) {
                    if (!err)
                        _.extend(user.profile, changes);
                    cb(err, client.server.getRequestStatus(err));
                });
            }, cb);
        }
        user.updateProfile = updateProfile;
        /**
          * Updates the users settings.
          * @param changes The fields to update. These changes will be submitted to the server as a set of transforms and then applied to the local object on success.
          * @param cb Callback function to eturn the results of the server call. The status param will always be filled out (even in an Error)
          * and will contain a code indicating the success or type of failure that occured.  Possible values:
          * - success: The server call was successful.
          * - notAuthorized: The users access token isn't valid and we should call authorizeDevice() to refresh it.
          * - serverError: An internal error occured while processing request. Likely a storage call failed.
          */
        function updateSettings(changes, cb) {
            ifAuthorized(function () {
                if (changes.id)
                    delete changes.id;
                _.extend(user.settings, changes);
                client.server.POST('api/updateSettings', null, client.Util.toModelTransforms(changes), function (err) {
                    if (!err)
                        _.extend(user.settings, changes);
                    cb(err, client.server.getRequestStatus(err));
                });
            }, cb);
        }
        user.updateSettings = updateSettings;
        /** Returns true if the user is authorized and has a valid access token. */
        function isAuthorized() {
            return !!user.accessToken;
        }
        user.isAuthorized = isAuthorized;
        /** Returns true if the user is currently connected. */
        function isConnected() {
            return !!user.session;
        }
        user.isConnected = isConnected;
        /** Executes the "authorized" callback if the client has a valid access token otherwise "unauthorized" is called with an error. */
        function ifAuthorized(authorized, unathorized) {
            if (isAuthorized()) {
                authorized();
            }
            else if (unathorized) {
                unathorized(new Error('Not Authorized'), 2 /* notAuthorized */);
            }
        }
        user.ifAuthorized = ifAuthorized;
        /** Executes the "connected" callback if the client has a valid server connection otherwise "unconnected" is called with an error. */
        function ifConnected(connected, unconnected) {
            if (isConnected()) {
                connected();
            }
            else {
                unconnected(new Error('Not Connected'), 1 /* badRequest */);
            }
        }
        user.ifConnected = ifConnected;
        /**
          * Authorizes the client without having to call back to the server. Used to pre-authorize guest & display accounts.
          * @param p The displays profile.
          * @param t The displays access token.
          */
        function preAuthorize(p, t) {
            client.logger.info('Pre-authorizing user');
            user.profile = p;
            user.settings = _.extend({}, user.defaultSettings);
            user.accessToken = t;
            user.accountType = user.profile.userId.split('!', 2)[0];
            // Set data center to server so that we just make calls back to the server we were loaded from.
            // - This works because preAuthorize() is always called before init().  When init() is called it will just use the stored server.
            dataCenter = 'server';
            // Patch ID's
            // - The profile & settings models are stored with an ID that isn't unique or comparable.
            user.profile.id = 'user:' + user.profile.userId;
            user.settings.id = 'settings:' + user.profile.userId;
            // Set state
            setConnectionState(2 /* authorized */);
        }
        user.preAuthorize = preAuthorize;
        /** Returns the curent connection state. */
        function getConnectionState() {
            return connectState;
        }
        user.getConnectionState = getConnectionState;
        /** Fired when a new xim is joined. */
        function triggerXimJoined(groupId) {
            // iAuthorized() check fixes an issue where we're getting called for the ximViewer control (there's no user in that case.)
            if (isAuthorized()) {
                // form the url that we will use to luanch airplay and xbox one ie with based on the server.root, groupID and accessToken
                var root = getConnection().root;
                user.currentXim = {
                    dataCenterRoot: root,
                    ximSessionID: groupId,
                    authToken: encodeURIComponent(user.accessToken.token),
                    viewerUrl: root + "cast/group:" + groupId + "?token=" + encodeURIComponent(user.accessToken.token)
                };
                if (_viewer) {
                    _viewer.joinXim(user.accessToken.token, groupId);
                }
                user.onXimJoined.trigger(user.currentXim);
            }
        }
        user.triggerXimJoined = triggerXimJoined;
        /** Sets the users current external viewer.  */
        function turnOnViewer(viewer) {
            _viewer = viewer;
            if (_viewer) {
                // Update viewers displayed screen
                if (user.currentXim) {
                    _viewer.joinXim(user.accessToken.token, user.currentXim.ximSessionID);
                }
                else {
                    _viewer.showSplashScreen();
                }
            }
        }
        user.turnOnViewer = turnOnViewer;
        /** Clears the users current external viewer. */
        function turnOffViewer() {
            if (_viewer) {
                _viewer.showDisconnectScreen();
            }
            _viewer = null;
        }
        user.turnOffViewer = turnOffViewer;
        //=====================================================
        // Helper functions
        //=====================================================
        /** Updates the current connection state. */
        function setConnectionState(state) {
            if (state !== connectState) {
                connectState = state;
                user.onConnectionState.trigger(state);
            }
        }
        /** Processes the response from a REST call to either api/authorize or api/switchAccounts. */
        function handleAuthorization(err, result, s, cb) {
            if (!err) {
                client.logger.stop(s, 'device: authorized');
                user.accessToken = result.accessToken;
                user.profile = result.profile;
                user.settings = result.settings;
                user.serverMessage = result.serverMessage;
                user.country = result.country;
                //xims = result.xims;
                // Start connection process
                setConnectionState(2 /* authorized */);
                ensureConnected();
                // SIgnal completion
                cb(null, 0 /* succeeded */);
            }
            else {
                client.logger.stopError(s, 'handleAuthorization', 'device: authorization error - ' + err.message);
                setConnectionState(0 /* logout */);
                cb(err, client.server.getRequestStatus(err));
            }
        }
    })(user = client.user || (client.user = {}));
})(client || (client = {}));
/// <reference path="types.ts"/>
/** poly fill histoyr state support for Android 4.3*/
var client;
(function (client) {
    var History;
    (function (History) {
        var historyData = {};
        var lastHistoryTs = (new Date()).getTime().toString();
        function init() {
            var prev = window.history.pushState;
            window.history.pushState = function (data, title, url) {
                var ts = (new Date()).getTime().toString();
                var state = data;
                if (typeof data == 'object') {
                    state = _.extend(data, { ts: ts });
                }
                historyData[ts] = data;
                lastHistoryTs = ts;
                if (typeof url != 'undefined') {
                    prev.call(window.history, state, title, url);
                }
                else {
                    if (typeof title != 'undefined') {
                        prev.call(window.history, state, title);
                    }
                    else {
                        prev.call(window.history, state);
                    }
                }
            };
            var repl = window.history.replaceState;
            window.history.replaceState = function (data, title, url) {
                client.logger.info('history replaceState start');
                var state = data;
                if (typeof data == 'object') {
                    state = _.extend(data, { ts: lastHistoryTs });
                }
                historyData[lastHistoryTs] = state;
                if (typeof url != 'undefined') {
                    client.logger.info('history call inner replaceState with title and url', title, url);
                    repl.call(window.history, state, title, url);
                }
                else {
                    client.logger.info('history call inner replaceState with title', title);
                    if (typeof title != 'undefined') {
                        repl.call(window.history, state, title);
                    }
                    else {
                        repl.call(window.history, state);
                    }
                }
                client.logger.info('history replaceState finished');
            };
        }
        History.init = init;
        function popState(ev) {
            var pstate = ev.state;
            if (pstate && pstate.hasOwnProperty('ts')) {
                var ts = pstate['ts'];
                if (historyData.hasOwnProperty(ts)) {
                    var state = historyData[ts];
                    client.logger.info('popping history state on for ts', ts);
                    cleanupAfter(ts);
                    lastHistoryTs = ts;
                    return state;
                }
            }
            client.logger.debug('no history state to pop');
            return null;
        }
        History.popState = popState;
        // Used to get screen info for debug output
        function debugGetHistoryInfo() {
            if (!historyData)
                return "";
            var desc = "";
            for (var curHis in historyData) {
                desc += "History:";
                if (historyData[curHis].path) {
                    desc += " " + historyData[curHis].path;
                }
                if ('screenName' in historyData[curHis]) {
                    desc += " " + historyData[curHis].screenName;
                }
                desc += consts.linebreak;
            }
            return desc;
        }
        History.debugGetHistoryInfo = debugGetHistoryInfo;
        function state() {
            return lastHistoryTs ? historyData[lastHistoryTs] : null;
        }
        History.state = state;
        function cleanupAfter(ts) {
            var tsVal = parseInt(ts);
            var toDel = _.filter(_.keys(historyData), function (key, i) {
                return parseInt(key) > tsVal;
            });
            _.each(toDel, function (key, i) {
                delete historyData[key];
            });
        }
    })(History = client.History || (client.History = {}));
})(client || (client = {}));
/// <reference path="app.ts"/>
// /// <reference path="cast.ts"/>
/// <reference path="../../types/cordova/cameraroll.d.ts" />
/** Share Handler
* Handles passed in urls from share targets
*/
var client;
(function (client) {
    var shareHandler;
    (function (shareHandler) {
        ;
        var shareNames = null;
        var shareUrls = null;
        /** Handle incoming shares, return false if none found or not handled */
        function processShareTargets(callback) {
            var _this = this;
            // First check if I already have share targets
            if ((this.shareNames && this.shareNames.length > 0) || (this.shareUrls && this.shareUrls.length > 0)) {
                callback(null, true);
            }
            else {
                window.sharetarget.getData(function (shareTargets) {
                    _this.startCast(shareTargets);
                    callback(null, true);
                }, function () {
                    // No share targets found
                    callback(new Error("No share targets found"), false);
                });
            }
        }
        shareHandler.processShareTargets = processShareTargets;
        /** Start cast from share targets */
        function startCast(shareTargets) {
            // Store share targets
            this.shareNames = shareTargets.ids;
            this.shareUrls = shareTargets.urls;
            client.AppMasterScreen.checkConnection(function () {
                client.casting.prepareAndStartCast(true);
            });
        }
        shareHandler.startCast = startCast;
        /** Map share targets to queue items, uploading on-device photos if any */
        function mapToScreenQueueItems(items, photoUploadedEvent, callback) {
            var _this = this;
            async.waterfall([
                function (c) {
                    // First process any on-device share targets
                    if (_this.shareNames && _this.shareNames.length > 0) {
                        items = _.map(_this.shareNames, function (name, i) {
                            return { id: i.toString(), name: name, type: 'photo' };
                        });
                        // Clear name share targets as they have been used
                        _this.shareNames = null;
                        // Uploaded on-device photos to server
                        client.CameraRollUploader.createScreenQueueItems(items, photoUploadedEvent, c);
                    }
                    else {
                        // No on device photos so move on the next step
                        c(null, []);
                    }
                },
                function (queueitems, c) {
                    // Add any off-device share targets
                    if (_this.shareUrls) {
                        for (var i = 0; i < _this.shareUrls.length; i++) {
                            queueitems.push({ url: _this.shareUrls[i].toString(), type: 'photo' });
                        }
                        // Clear url share targets as they have been used
                        _this.shareUrls = null;
                    }
                    c(null, queueitems);
                }
            ], function (e, queueitems) {
                // Returns list of items
                callback(null, queueitems);
            });
        }
        shareHandler.mapToScreenQueueItems = mapToScreenQueueItems;
    })(shareHandler = client.shareHandler || (client.shareHandler = {}));
})(client || (client = {}));
/// <reference path="../types.ts"/>
var client;
(function (client) {
    /** Base class for a dockable panel. */
    var Panel = (function () {
        /** Creates a new instance of a panel. */
        function Panel(selector, options) {
            this.onShown = new client.Event();
            // In some cases we create a panel derived class with out a selector (see CastingScreen blacklist) so we'll
            // just not do anything in that case.
            if (selector) {
                this.$el = $(selector);
                options = $.extend({}, DefaultPanelOptions, options);
                this.dock(options.dock);
            }
        }
        /** Docks a panel to a given side. */
        Panel.prototype.dock = function (side) {
            if (this.docked)
                this.$el.removeClass(this.docked);
            switch (side) {
                case 4 /* full */:
                    this.docked = 'dock-full';
                    break;
                case 0 /* left */:
                    this.docked = 'dock-left';
                    break;
                case 1 /* right */:
                    this.docked = 'dock-right';
                    break;
                case 3 /* bottom */:
                    this.docked = 'dock-bottom';
                    break;
                case 2 /* top */:
                    this.docked = 'dock-top';
                    break;
                default:
                    this.docked = '';
                    break;
            }
            if (this.docked)
                this.$el.addClass(this.docked);
            return this;
        };
        /** Empties the contents of the panel. */
        Panel.prototype.empty = function () {
            this.$el.empty();
            return this;
        };
        /** Removes the panel all together. */
        Panel.prototype.remove = function () {
            this.$el.remove();
            return this;
        };
        /** Redraws the panels contents to the screen. */
        Panel.prototype.render = function () {
            return this;
        };
        /** Displays a hidden panel. */
        Panel.prototype.show = function (transition) {
            switch (transition) {
                case 0 /* slide */:
                case 1 /* fade */:
                default:
                    this.$el.show();
                    break;
            }
            this.onShown.trigger(true);
            return this;
        };
        /** Hides a visible panel. */
        Panel.prototype.hide = function (transition) {
            switch (transition) {
                case 0 /* slide */:
                case 1 /* fade */:
                default:
                    this.$el.hide();
                    break;
            }
            this.onShown.trigger(false);
            return this;
        };
        /**
          * Begins an abortable operation.
          * @param name CSS class names that will be added to the outer DOM element for duration of operation.
          * @param op Function that does the operation.
          * @param final (optional) Function to invoke when operation is aborted or finishes.
          */
        Panel.prototype.abortable = function (name, op, final) {
            var _this = this;
            var finished = false;
            if (name)
                this.$el.addClass(name);
            var done = function (err, results) {
                if (!finished) {
                    finished = true;
                    if (name)
                        _this.$el.removeClass(name);
                    if (final)
                        final(err, results);
                }
            };
            try {
                op(done);
            }
            catch (e) {
                var msg = e.toString();
                client.logger.error(consts.errorCode.Other, 'Exception: ' + msg);
                done(new Error(msg), null);
            }
            return done;
        };
        return Panel;
    })();
    client.Panel = Panel;
    /** Docking options for a panel. */
    (function (Dock) {
        Dock[Dock["left"] = 0] = "left";
        Dock[Dock["right"] = 1] = "right";
        Dock[Dock["top"] = 2] = "top";
        Dock[Dock["bottom"] = 3] = "bottom";
        Dock[Dock["full"] = 4] = "full";
        Dock[Dock["none"] = 5] = "none";
    })(client.Dock || (client.Dock = {}));
    var Dock = client.Dock;
    /** Transition effect to apply when showing or hiding panel. */
    (function (PanelTransition) {
        PanelTransition[PanelTransition["slide"] = 0] = "slide";
        PanelTransition[PanelTransition["fade"] = 1] = "fade";
        PanelTransition[PanelTransition["none"] = 2] = "none";
    })(client.PanelTransition || (client.PanelTransition = {}));
    var PanelTransition = client.PanelTransition;
    var DefaultPanelOptions = {
        dock: 5 /* none */
    };
})(client || (client = {}));
/// <reference path="../types.ts"/>
/// <reference path="../../../types/jquery/jquery.d.ts"/>
var client;
(function (client) {
    (function (CommandBarState) {
        CommandBarState[CommandBarState["hidden"] = 0] = "hidden";
        CommandBarState[CommandBarState["open"] = 1] = "open";
    })(client.CommandBarState || (client.CommandBarState = {}));
    var CommandBarState = client.CommandBarState;
    /** Command menu that slides up from bottom. */
    var CommandBar = (function () {
        function CommandBar(selector, styles) {
            if (styles === void 0) { styles = {}; }
            /** Current state of the command bar. */
            this.state = 0 /* hidden */;
            /** Is disabled? */
            this.isDisabled = false;
            /** Current set of commands. */
            this.commands = [];
            /** Active command */
            this.activeCommand = consts.commands.none;
            this.$el = $(selector);
            // Use default style plus those passed in
            this.commandBarStyle = $.extend({}, CommandBar.CommandBarStyleDefault);
            for (var style in styles) {
                // Extend string values (classes), set the other ones
                if (typeof styles[style] == "string") {
                    this.commandBarStyle[style] += ' ' + styles[style];
                }
                else {
                    this.commandBarStyle[style] = styles[style];
                }
            }
        }
        /** toggle the command enabled flag */
        CommandBar.prototype.disableCommand = function (command, disabled) {
            var $command = this.$el.find('#' + command.id);
            if ($command.length) {
                $command.toggleClass('commandDisabled', disabled);
                if (!disabled && this.state == 0 /* hidden */) {
                    this.show(true);
                }
            }
        };
        /** Updates the menus commands */
        CommandBar.prototype.setCommands = function (commands, callback) {
            var _this = this;
            if (commands && commands.length) {
                this.commands = commands;
                var commandWidth = 100 / commands.length;
                // Render 
                var commandStates = _.map(this.commands, function (c, i) {
                    return _.extend({}, consts.commands._template, c);
                });
                this.$el.empty().render('commandBar', { commands: commandStates, width: commandWidth, style: this.commandBarStyle });
                this.show(this.commandBarStyle.bar != 'commandBarClosed');
                // Capture menu item clicks
                var $commands = this.$el.find('.commandBarNav > li');
                $commands.click(function (ev) {
                    // Prevent bubble up
                    window.event.cancelBubble = true;
                    // The hit target or actual button could have been pressed so get ID and explicitly select the button
                    var id = ev.currentTarget.id;
                    var $base = _this.$el.find(".commandBarBase");
                    var $cmd = $base.find("#" + id);
                    if (!$cmd.hasClass('commandDisabled')) {
                        $cmd.removeClass(_this.commandBarStyle['pressed']);
                        var cmdId = $cmd.attr('id');
                        var cmd = _.find(_this.commands, function (c, i) {
                            return c.id == cmdId;
                        });
                        switch (cmd.id) {
                            case consts.commands.newgroup.id:
                                client.analytics.logTimedEvent(consts.flurryEvent.ximCreateNewTimed);
                                break;
                            case consts.commands.start.id:
                                client.analytics.endTimedEvent(consts.flurryEvent.ximCreateNewTimed);
                                break;
                        }
                        client.logger.collect(cmd.id == consts.commands.start.id ? 'startXim' : cmd.id);
                        // If a tabbed control, second click deselects the tab
                        if (_this.commandBarStyle['isTabbed']) {
                            $commands.removeClass(_this.commandBarStyle['selected']);
                            // If clicking on tab that was already selected
                            if (_this.activeCommand && _this.activeCommand.id == cmdId) {
                                // Does command bar allow no tab to be active
                                if (_this.commandBarStyle['allowNoActiveTab']) {
                                    cmd = consts.commands.none;
                                    _this.fadeTimer = setTimeout(function () {
                                        _this.show(false);
                                    }, _this.commandBarStyle.autoHideDuration);
                                }
                                else {
                                    $cmd.addClass(_this.commandBarStyle['selected']);
                                }
                            }
                            else {
                                // Bar doesn't fade when tab seledted
                                clearTimeout(_this.fadeTimer);
                                $cmd.addClass(_this.commandBarStyle['selected']);
                                // Move ball if needed
                                _this.moveBall(cmd);
                            }
                            _this.activeCommand = cmd;
                        }
                        if (callback && cmd) {
                            callback(cmd.id, cmd);
                        }
                    }
                }).on('mousedown touchstart', function (ev) {
                    var $cmd = $(ev.currentTarget);
                    if (!$cmd.hasClass('commandDisabled')) {
                        $cmd.addClass(_this.commandBarStyle['pressed']);
                        // Remove pressed style after a pause in case user moved mouse off target (which is not a click)
                        setTimeout(function () {
                            $cmd.removeClass(_this.commandBarStyle['pressed']);
                        }, 500);
                    }
                });
            }
            else {
                this.show(false);
                this.commands = [];
            }
        };
        // Returns current selection of tabbed controls
        CommandBar.prototype.getSelection = function () {
            if (this.commandBarStyle['isTabbed']) {
                return this.activeCommand;
            }
            return null;
        };
        // Set selected tab (does not fire command)
        CommandBar.prototype.select = function (command) {
            if (this.commandBarStyle['isTabbed']) {
                this.deselect();
                this.activeCommand = command;
                var $cmd = this.$el.find('#' + command.id);
                $cmd.addClass(this.commandBarStyle['selected']);
                // Move ball if needed
                this.moveBall(command);
            }
        };
        // Hide command bar and disable from opening again
        CommandBar.prototype.disable = function (disable) {
            this.isDisabled = disable;
            if (this.isDisabled) {
                this.show(false);
            }
        };
        // For tabbed controls: delect a command or all commands if none given
        CommandBar.prototype.deselect = function (command) {
            if (this.commandBarStyle['isTabbed']) {
                if (!command || command == this.activeCommand) {
                    var $commands = this.$el.find('.commandBarNav > li');
                    $commands.removeClass(this.commandBarStyle['selected']);
                    this.activeCommand = consts.commands.none;
                }
            }
        };
        CommandBar.prototype.hasCommands = function () {
            return (!this.isDisabled && this.commands && this.commands.length > 0);
        };
        /** Returns true if the command bar is currently open. */
        CommandBar.prototype.isOpen = function () {
            return this.state == 1 /* open */;
        };
        /** Shows or hides the command bar. */
        CommandBar.prototype.show = function (visible, fadeout) {
            var _this = this;
            if (fadeout === void 0) { fadeout = false; }
            // Clear any previously set timer
            window.clearTimeout(this.fadeTimer);
            if (visible) {
                this.$el.find('.commandBarBase').removeClass("commandBarClosed").addClass("commandBarOpen");
                // show invisible hit target over top so can click while animating in
                this.$el.find('.commandBarBaseTarget').show();
                this.state = 1 /* open */;
                // Close after duration (if provided)
                if (fadeout) {
                    // Overlay automatically hides after a while
                    this.fadeTimer = setTimeout(function () {
                        _this.show(false);
                    }, this.commandBarStyle.autoHideDuration);
                }
            }
            else {
                // Don't close if an tab is active
                if (this.commandBarStyle['isTabbed'] && this.activeCommand != consts.commands.none) {
                    return;
                }
                // hide invisible hit target 
                this.$el.find('.commandBarBaseTarget').hide();
                this.$el.find('.commandBarBase').removeClass("commandBarOpen").addClass("commandBarClosed");
                this.state = 0 /* hidden */;
            }
        };
        // Animate bouncy ball under tab
        CommandBar.prototype.moveBall = function (command) {
            if (!this.commandBarStyle['bounceBall']) {
                return;
            }
            // Find element and index in array
            var index = this.commands.indexOf(command);
            var $ball = this.$el.find('.commandBarBounceBall');
            if (index == -1 || $ball.length == 0) {
                return;
            }
            // Calculate new ball position
            var screenWidth = client.webview.currentViewport().width; //Note: assumes commandBar is width of screen, but saves redraw of asking for parent width
            var commandWidth = (1 / this.commands.length);
            var position = screenWidth * ((0.5 * commandWidth) + (index * commandWidth)) - 10; // 10 = width of ball
            // Animate the ball
            var translate = 'translate(' + position + 'px)';
            $ball.css({
                '-ms-transform': translate,
                '-webkit-transform': translate,
                'transform': translate
            });
        };
        // Options   /
        CommandBar.CommandBarStyleDefault = {
            tab: '',
            bar: '',
            pressed: 'commandPressed',
            selected: '',
            isTabbed: false,
            allowNoActiveTab: false,
            bounceBall: false,
            glyphSize: 35,
            textSize: 0,
            autoHideDuration: 2500
        };
        CommandBar.height = 75; // Must match @COMMAND_BAR_HEIGHT in less file
        return CommandBar;
    })();
    client.CommandBar = CommandBar;
})(client || (client = {}));
/// <reference path="../controls/Panel.ts"/>
/// <reference path="../controls/CommandBar.ts"/>
/// <reference path="../app.ts"/>
/// <reference path="../resources.ts"/>
var client;
(function (client) {
    (function (ScreenState) {
        ScreenState[ScreenState["UNINITIALIZED"] = 0] = "UNINITIALIZED";
        ScreenState[ScreenState["CREATED"] = 1] = "CREATED";
        ScreenState[ScreenState["NAVIGATED"] = 2] = "NAVIGATED";
        ScreenState[ScreenState["DISPOSED"] = 3] = "DISPOSED";
    })(client.ScreenState || (client.ScreenState = {}));
    var ScreenState = client.ScreenState;
    (function (ModalStyle) {
        ModalStyle[ModalStyle["dialog"] = 0] = "dialog";
        ModalStyle[ModalStyle["popup"] = 1] = "popup";
        ModalStyle[ModalStyle["fullScreen"] = 2] = "fullScreen";
    })(client.ModalStyle || (client.ModalStyle = {}));
    var ModalStyle = client.ModalStyle;
    var DefaultModalScreenOptions = {
        params: {},
        style: 0 /* dialog */
    };
    /** A full screen panel that can be navigated to by the app. */
    var Screen = (function (_super) {
        __extends(Screen, _super);
        /**
          * Creates a new instance of the screen.
          * @param selector The element the screen is being bound to.
          * @param history (optional) Set of functions used to manage a screens navigation history.
          */
        function Screen(selector, history) {
            _super.call(this, selector, { dock: 4 /* full */ });
            this.history = history;
            this.boundOnNavigate = false;
            this.screenState = 0 /* UNINITIALIZED */;
            this.onCreated();
        }
        Screen.prototype.getScreenState = function () {
            return this.screenState;
        };
        /** Called after the screen has beed constructed but before it's navigated. */
        Screen.prototype.onCreated = function () {
            this.screenState = 1 /* CREATED */;
        };
        /** Called when the screen is being removed and should be disposed. */
        Screen.prototype.onDispose = function () {
            if (this.screenState != 2 /* NAVIGATED */) {
                client.logger.warn("Screen Disposed but never Navigated");
            }
            this.screenState = 3 /* DISPOSED */;
        };
        /**
          * Called when a navigation occurs.
          * @param route The route that was navigated to.
          */
        Screen.prototype.navigate = function (route) {
            if (this.screenState != 1 /* CREATED */) {
                client.logger.warn("Screen Navigated without onCreate");
            }
            // measure offsetLeft to force render offscreen
            var offset = this.$el.offset();
            if (this.$el.hasClass('enter-stage-right')) {
                this.$el.toggleClass('enter-stage-right');
            }
            var state = history.state || client.History.state();
            var canBack = !(state && state.isRoot);
            var $back = this.$el.find('#back');
            if (canBack) {
                $back.removeAttr('disabled');
            }
            else {
                $back.attr('disabled', 'disabled');
            }
            this.$el.on('click', '#back', function (e) {
                e.preventDefault();
                if (canBack) {
                    cordova.fireDocumentEvent('backbutton');
                }
            });
            if (this.history && route.page) {
                this.history.page = route.page;
            }
            this.screenState = 2 /* NAVIGATED */;
            return this;
        };
        // Test that screen is in valid state before invoking callback
        Screen.prototype.ifAvailable = function (callback) {
            var _this = this;
            return _.wrap(callback, function (f) {
                var args = [];
                for (var _i = 1; _i < arguments.length; _i++) {
                    args[_i - 1] = arguments[_i];
                }
                // Make sure screen hasn't been disposed
                if (_this.getScreenState() != 2 /* NAVIGATED */) {
                    return;
                }
                callback.apply(null, args);
            });
        };
        Screen.prototype.remove = function () {
            try {
                this.onDispose();
            }
            catch (e) {
                client.logger.error(consts.errorCode.Navigation, 'Exception disposing screen: ' + e.toString());
            }
            return _super.prototype.remove.call(this);
        };
        /** Returns a function for closing the dialog after an async sequence completes. */
        Screen.prototype.closeAsync = function () {
            var _this = this;
            return function (err, results) {
                if (_this.history)
                    _this.history.close(err, results);
            };
        };
        Screen.prototype.bindOnNavigate = function (sel, next) {
            if (!this.boundOnNavigate) {
                this.boundOnNavigate = true;
                this.$el.onNavigate(sel);
                if (next) {
                    next();
                }
            }
            return false;
        };
        /** is the current screen the last child of its DOM parent*/
        Screen.prototype.isLastChild = function () {
            var $parent = this.$el.parent();
            var $lastChild = $parent.find('>:last-child');
            return $lastChild.length && $lastChild[0] == this.$el[0];
        };
        Screen.prototype.syncNoPan = function () {
            var _this = this;
            // check the height and turn off panning if not tall enough
            setTimeout(function () {
                // only toggling class if we are on the top of the stack of screens
                if (_this.isLastChild()) {
                    $('body').toggleClass('noPan', _this.$el.height() <= client.webview.currentViewport().height - 150);
                }
            }, 100);
        };
        Screen.prototype.render = function () {
            return this;
        };
        return Screen;
    })(client.Panel);
    client.Screen = Screen;
    /** Base class for detail screens. */
    var DetailScreen = (function (_super) {
        __extends(DetailScreen, _super);
        function DetailScreen() {
            _super.apply(this, arguments);
            /** Use alternative header with back button and provided title */
            this.altHeader = null;
        }
        return DetailScreen;
    })(Screen);
    client.DetailScreen = DetailScreen;
    var DefaultMasterScreen = (function () {
        function DefaultMasterScreen(selector) {
            if (selector === void 0) { selector = '#screens'; }
            this.selector = selector;
            this.transitionEndEvents = 'transitionend webkitTransitionEnd oTransitionEnd MSTransitionEnd';
        }
        DefaultMasterScreen.prototype.routeTo = function (screen, path, route) {
            client.logger.info('route path', path, 'curPath', this.curPath || 'null');
            // Create new screen
            if (this.curPath !== path) {
                // Remove old screen
                var oldScreen = this.curScreen;
                // Create new screen
                this.curPath = path;
                var $screen = $('<div class="screen"/>');
                this.curScreen = new screen($screen);
                if (route.state !== 5 /* popping */) {
                    $screen.addClass('enter-stage-right');
                    $screen.appendTo(this.selector);
                    if (oldScreen) {
                        oldScreen.$el.css('z-index', -1);
                    }
                    $screen.bind(this.transitionEndEvents, function (e) {
                        if (oldScreen) {
                            oldScreen.remove();
                        }
                    });
                }
                else {
                    $screen.prependTo(this.selector);
                    oldScreen.$el.addClass('exit-stage-right');
                    oldScreen.$el.bind(this.transitionEndEvents, function () {
                        oldScreen.remove();
                    });
                }
            }
            // Navigate screen
            this.curScreen.navigate(route);
        };
        DefaultMasterScreen.prototype.beginModal = function (options, callback) {
            var _this = this;
            var isOverlay = (options.style == 1 /* popup */);
            if (this.curScreen && !isOverlay)
                this.curScreen.hide(2 /* none */);
            callback(this.selector, function () {
                if (_this.curScreen && !isOverlay)
                    _this.curScreen.show(2 /* none */);
            });
        };
        return DefaultMasterScreen;
    })();
    client.DefaultMasterScreen = DefaultMasterScreen;
    /** Manages the currently displayed screen. */
    var screens;
    (function (screens) {
        /** Current master screen. */
        screens.master = new DefaultMasterScreen();
        /** current modal screen*/
        screens.modal = null;
        /** If true, showModal will use the new showScreen function instead. */
        screens.useShowScreen = false;
        /**
          * Adds a route to the app for a new screen.
          * @param path Route path that navigates to screen.
          * @param screen Constructor of screen to create.
          */
        function addRoute(path, screen) {
            client.app.addRoute(path, function (route) {
                screens.master.routeTo(screen, path, route);
            });
        }
        screens.addRoute = addRoute;
        /**
          * A very simple screen renderer.
          * @param screen The screen to display as a dialog.
          * @param route (optional) Route info to navigate screen to.
          * @param onClose (optional) Function to invoke should the screen close.
          */
        function showScreen(screen, params, onClose) {
            // Build up history
            var hist = {
                page: 0,
                back: function back() {
                    client.logger.warn('ScreenHistory.back() called and not supported');
                },
                forward: function forward(params) {
                    client.logger.warn('ScreenHistory.forward() called and not supported');
                },
                close: function close(err, results) {
                    if (onClose)
                        onClose(err, results);
                }
            };
            // Create new screen
            var $modal = $('<div class="screen screen-modal"/>');
            var newModal = new screen($modal, hist);
            // Swap screens
            if (screens.modal)
                screens.modal.remove();
            $modal.appendTo('#screens');
            screens.modal = newModal;
            screens.modal.navigate({ id: consts.time.now(), path: '', params: params || {} });
            return screens.modal;
        }
        screens.showScreen = showScreen;
        /**
          * Displays a modal screen.
          * @param screen The screen to display as a dialog.
          * @param options (optional) The options used to control the presentation of the screen and to specify the params it's passed.
          * @param callback (optional) Function to invoke once the modal has closed.
          */
        function showModal(screen, options, callback) {
            options = $.extend({}, DefaultModalScreenOptions, options);
            // Check for switch to new showScreen method
            if (screens.useShowScreen)
                return showScreen(screen, options.params, callback);
            // Setup modal style
            var hasHistory, isOverlay, className;
            switch (options.style) {
                case 1 /* popup */:
                    hasHistory = false;
                    isOverlay = true;
                    className = 'screen-modal-popup';
                    break;
                case 0 /* dialog */:
                default:
                    hasHistory = true;
                    isOverlay = false;
                    className = 'screen-modal-dialog';
                    break;
            }
            // Create history manager
            var hasHistory = options.style === 0 /* dialog */ || options.style === 2 /* fullScreen */;
            var dialogId = new Date().getTime();
            var closing;
            var hScreen = {
                page: 1,
                back: function back() {
                    history.back();
                },
                forward: function forward(params) {
                    var loc = { id: dialogId, path: '', params: params || {} };
                    if (hasHistory)
                        history.pushState({ params: loc.params, page: hScreen.page + 1 }, null);
                    modal.navigate(loc);
                },
                close: function close(err, results) {
                    client.logger.info('closing modal hasHistory = ', hasHistory, ' location.hash = ', location.hash, 'page = ', hScreen.page);
                    closing = [err, results];
                    if (hasHistory) {
                        // TODO: should we reset client.screens.modal here?
                        history.go(0 - hScreen.page);
                    }
                    else {
                        removeScreen();
                    }
                }
            };
            var loc = { id: dialogId, path: '', params: options.params, page: 1 };
            if (hasHistory)
                history.pushState(loc, null);
            // Create new screen instance
            var onFinished;
            var $modal = $('<div class="screen screen-modal"/>');
            var modal = new screen($modal, hScreen);
            screens.master.beginModal(options, function (s, f) {
                $modal.appendTo(s);
                modal.navigate(loc);
                onFinished = f;
            });
            function removeScreen() {
                if (modal) {
                    var resetGlobal = modal == screens.modal;
                    modal.remove();
                    modal = null;
                    if (resetGlobal) {
                        screens.modal = null;
                    }
                    if (!callback || !callback.apply(null, closing || [])) {
                        if (onFinished)
                            onFinished();
                    }
                }
                else {
                    client.logger.warn('Screen.ShowModal: RemoveScreen called twice.');
                }
            }
            // Setup navigation capture.
            if (hasHistory) {
                client.app.captureNavigates(function (loc) {
                    if (loc.id === dialogId) {
                        hScreen.page = loc.page;
                    }
                    else {
                        // TODO: Add logic to confirm close
                        client.logger.info('release capture to modal');
                        client.app.releaseCapture();
                        removeScreen();
                        // show master screen again
                        $(screens.master.selector).children('.master.screen').show();
                    }
                });
            }
            screens.modal = modal;
            return modal;
        }
        screens.showModal = showModal;
        /** try to pop out of any on going modal, wizard, popup and get ready to start new xim
            callback with true if we got everything ready, otherwise false */
        function canStartNewXim(callback) {
            var hasWizard = !!client.wizard.currentWizard;
            var hasModal = !!screens.modal && screens.modal.getScreenState() != 3 /* DISPOSED */;
            var hasPopup = client.PopUp.history.length > 0;
            var atRoot = history.state && history.state['path'] == '/';
            // TODO : dismiss existing xim, dialog, wizard, pop up
            callback(null, !hasWizard && !hasModal && !hasPopup && atRoot);
        }
        screens.canStartNewXim = canStartNewXim;
    })(screens = client.screens || (client.screens = {}));
})(client || (client = {}));
/// <reference path="Screen.ts"/>
var client;
(function (client) {
    /** Displays a loading message. */
    var LoadingScreen = (function (_super) {
        __extends(LoadingScreen, _super);
        function LoadingScreen() {
            _super.apply(this, arguments);
        }
        /**
          * Updates the displayed message
          * @param msg The new message to display.
          */
        LoadingScreen.prototype.message = function (msg) {
            this._msg = msg;
            client.logger.info(msg);
            this.$el.find('#message').text(this._msg);
            return this;
        };
        LoadingScreen.prototype.navigate = function (route) {
            // Redraw message
            this._msg = route.params.message || '';
            this.render();
            return _super.prototype.navigate.call(this, route);
        };
        LoadingScreen.prototype.render = function () {
            // Make screen is in a valid state
            if (this.getScreenState() == 3 /* DISPOSED */) {
                client.logger.warn("Render called on DISPOSED screen ");
                return this;
            }
            // Render view
            this.empty();
            this.$el.addClass('splash').render('LoadingScreen', { message: this._msg });
            this.$el.addClass('loadingScreen');
            this.syncNoPan();
            return _super.prototype.render.call(this);
        };
        /**
          * Displays a new loading screen as a popup.
          * @param message The loading message to display.
          * @param callback (optional) The callback to call once the screen has been dropped.
          */
        LoadingScreen.display = function (message, callback) {
            client.logger.info(message);
            return client.screens.showModal(LoadingScreen, { style: 1 /* popup */, params: { message: message } }, function (e, results) {
                callback(e, results);
                return false;
            });
        };
        return LoadingScreen;
    })(client.Screen);
    client.LoadingScreen = LoadingScreen;
})(client || (client = {}));
/// <reference path="types.ts"/>
/// <reference path='../../types/cordova/userprofile.d.ts'/>
/// <reference path='../../types/cordova/screenlock.d.ts'/>
/// <reference path='../../types/cordova/sharetarget.d.ts'/>
/// <reference path='../../types/cordova/imagetransfer.d.ts'/>
/// <reference path='./user.ts'/>
/// <reference path='./history.ts'/>
/// <reference path='./ShareHandler.ts'/>
/// <reference path="screens/LoadingScreen.ts"/>
/** Global application singleton for client. */
var client;
(function (client) {
    var app;
    (function (app) {
        //===========================================
        // Interfaces & enums
        //===========================================
        /** Possible application states. */
        (function (State) {
            State[State["stopped"] = 0] = "stopped";
            State[State["starting"] = 1] = "starting";
            State[State["navigating"] = 2] = "navigating";
            State[State["navigated"] = 3] = "navigated";
            State[State["startupError"] = 4] = "startupError";
            State[State["popping"] = 5] = "popping";
            State[State["popped"] = 6] = "popped";
        })(app.State || (app.State = {}));
        var State = app.State;
        //===========================================
        // Methods
        //===========================================
        /** Restarts the application */
        function restartApplication() {
            // Put up loading screen during restart
            client.LoadingScreen.display(resources.messages.tagline, function () {
            });
            // Reload the application
            window.location.reload();
        }
        app.restartApplication = restartApplication;
        /**
          * Listens for an app level event to occur.
          * @param type Name of event to listen for. Valid values 'orientationchange', 'statechange', 'routenotfound'.
          * @param listener Callback function to invoke once event occurs.
          */
        function addEventListner(type, listener) {
            var t = type.toLowerCase();
            var l = events[t];
            if (!l)
                events[t] = l = [];
            l.push(listener);
        }
        app.addEventListner = addEventListner;
        /**
            remove the previously bound event handler
        */
        function removeEventListner(type, listener) {
            var t = type.toLowerCase();
            var l = events[t];
            if (l) {
                events[t] = _.reject(l, function (h, i) {
                    return h == listener;
                });
            }
        }
        app.removeEventListner = removeEventListner;
        /**
          * Adds a new route to handle.
          * @param path The path to register a handler for.
          * @param screen The name of a screen to navigate to.
          * @param handler The callback function to invoke when a given path is navigated to.
          */
        function addRoute(path, handler) {
            // Append new route table entry
            var route = { params: [], handler: handler };
            var p = path.concat('/?');
            p = p.replace(/\/\(/g, '(?:/');
            p = p.replace(/(\/)?(\.)?:(\w+)(?:(\(.*?\)))?(\?)?(\*)?/g, function (_, slash, format, key, capture, optional, star) {
                route.params.push({ name: key, optional: !!optional });
                slash = slash || '';
                return '' + (optional ? '' : slash) + '(?:' + (optional ? slash : '') + (format || '') + (capture || (format && '([^/.]+?)' || '([^/]+?)')) + ')' + (optional || '') + (star ? '(/*)?' : '');
            });
            p = p.replace(/([\/.])/g, '\\$1');
            p = p.replace(/\*/g, '(.*)');
            route.debugPath = '^' + p + '$';
            route.path = new RegExp(route.debugPath, 'i');
            routes.push(route);
        }
        app.addRoute = addRoute;
        /**
          * Adds a new async task to run at startup.  Tasks are invoked in the order they're added. Tasks should call the done() function when completed.
          * @param task The callback function to invoke when the task is run.
          */
        function addTask(task) {
            app.tasks.push(task);
        }
        app.addTask = addTask;
        /**
          * Begins capturing hashChange events.  Route processing will stop until the capture is released.
          * @param callback Function to call when a navigate occurs.
          */
        function captureNavigates(callback) {
            if (app.capture)
                throw "Navigates already captured";
            app.capture = callback;
            routeId = 0;
        }
        app.captureNavigates = captureNavigates;
        /** Returns the current persisted data for the app. */
        function getData() {
            return history.state ? history.state.data : undefined;
        }
        app.getData = getData;
        /** Returns the apps current state. */
        function getState() {
            return state;
        }
        app.getState = getState;
        /** Check if the app is in the started (not stopped, starting, startup error) state */
        function isStarted() {
            return state != 0 /* stopped */ && state != 1 /* starting */ && state != 4 /* startupError */;
        }
        app.isStarted = isStarted;
        /** Returns true if navigates are currently being captured. */
        function isCaptured() {
            return !!app.capture;
        }
        app.isCaptured = isCaptured;
        /** check if we actually running as display either as anonymous display or
            target of xim invite via SMS or email
        */
        function isDisplay() {
            return client.startupMode == 0 /* desktop */;
        }
        app.isDisplay = isDisplay;
        /** check if we are running on a mobile device (either as app or as display)
        */
        function isMobile() {
            return !isDisplay() || app.displayAgent && app.displayAgent.device && app.displayAgent.device.type == 'mobile';
        }
        app.isMobile = isMobile;
        /** iOS safari may have issue with screen rotation window.innerHeight will get scaled wrong on orientation change,
            we need to detect the case that we are running on iPhone/iPod Display
         */
        function isIPhoneDisplay() {
            return isDisplay() && isMobile() && app.displayAgent && app.displayAgent.os && app.displayAgent.os.name == 'iOS';
        }
        app.isIPhoneDisplay = isIPhoneDisplay;
        /**
          UAAgent as detected by our server in display (anonymous or invited) scenario
        */
        app.displayAgent = null;
        /**
          * Navigates the app to a new route.
          * @param path The application path to navigate to.
          * @param query (optional) Query parameters to add to path.
          * @param data (optional) Blob data to pass with navigate.
          */
        function navigate(path, query, data) {
            var loc = { id: new Date().getTime(), path: path, query: query, data: data };
            history.pushState(loc, null, '#' + path + toQueryString(query));
            navigateToRoute(loc);
        }
        app.navigate = navigate;
        /**
          * Converts a query string into a param map.
          * @param query The query string to parse.
          */
        function parseQueryString(query) {
            return querystring.parse(query);
        }
        app.parseQueryString = parseQueryString;
        /** Stops capturing navigates. */
        function releaseCapture() {
            app.capture = null;
        }
        app.releaseCapture = releaseCapture;
        /**
          * Updates the apps persisted data.
          * @param data The data to persist for the current URL.
          */
        function setData(data) {
            var loc = history.state || client.History.state();
            loc.data = data;
            history.replaceState(loc, null);
        }
        app.setData = setData;
        /**
          * Starts the application. Executes all startup tasks and then processes the initial route.
          * @param cb (optional) Callback to invoke once the app has been fully started.
          */
        function start(cb) {
            if (cb)
                appStarted.wait(cb);
            // Start app on first request
            if (state == 0 /* stopped */) {
                // Find initial location
                var path, search;
                if (location.hash && location.hash.length > 1 && location.hash.indexOf('#!') !== 0) {
                    var pos = location.hash.indexOf('?');
                    path = pos > 0 ? location.hash.substring(1, pos) : location.hash.substr(1);
                    search = pos > 0 ? location.hash.substr(pos) : '';
                }
                else {
                    path = '/'; // location.pathname;
                    search = location.search;
                }
                var loc = { id: new Date().getTime(), path: path, query: parseQueryString(search), isRoot: true };
                history.replaceState(loc, null);
                // Run startup tasks
                changeState(1 /* starting */);
                async.series(app.tasks, function (err, results) {
                    try {
                        if (err)
                            throw err;
                        // index.html has 'dark-theme' on by default (to avoid white flash between splash and app ux)
                        // since we are now finished booting, turn it off
                        $('body').toggleClass('dark-theme', false);
                        // Navigate to route
                        navigateToRoute(loc);
                        appStarted.signal(null, results);
                    }
                    catch (e) {
                        changeState(4 /* startupError */);
                        appStarted.signal(e, null);
                    }
                });
            }
        }
        app.start = start;
        /**
          * Converts a param map into a query string.
          * @param params The parameters to convert.
          */
        function toQueryString(params) {
            return querystring.format(params);
        }
        app.toQueryString = toQueryString;
        function parseUrl(urlString) {
            return uri.parse(urlString);
        }
        app.parseUrl = parseUrl;
        function formatUrl(url) {
            return uri.format(url);
        }
        app.formatUrl = formatUrl;
        function route(loc) {
            // Match routes
            var routeFound = false;
            // wizard cleanup will trigger route to loc = { wizardIndex:-1, page:1}
            // ignore it
            if (_.isUndefined(loc.path)) {
                return;
            }
            for (var i = 0; i < routes.length && !routeFound; i++) {
                // Check for matched path
                var r = routes[i];
                var m = r.path.exec(loc.path);
                if (!m)
                    continue;
                client.logger.info('matched route to path', r.debugPath, loc.path);
                routeFound = true;
                changeState(2 /* navigating */);
                try {
                    // Copy matched param values
                    var params = {};
                    for (var j = 1; j < m.length; j++) {
                        var key = r.params[j - 1];
                        if (!key)
                            continue;
                        try {
                            params[key.name] = ('string' == typeof m[j] ? decodeURIComponent(m[j]) : m[j]);
                        }
                        catch (e) {
                            client.logger.error(consts.errorCode.Routing, 'Exception decoding route param', key, m[j], e);
                        }
                    }
                    // Navigate to route
                    r.handler({ id: loc.id, path: loc.path, params: params, query: loc.query, data: loc.data, state: loc.state });
                }
                catch (e) {
                    client.logger.error(consts.errorCode.Routing, 'Unhandled exception navigating to route', loc, e);
                }
                changeState(3 /* navigated */);
            }
            // Fire routeNotFound event
            if (!routeFound) {
                client.logger.warn('no route found');
                fireEvent('routenotfound', loc);
            }
        }
        app.route = route;
        /**
          * Returns a callback that will check the async results of an operation for success and optionaly display an error message on failure.
          * @param fn Function to invoke on success.
          * @param message (optional) Message to display to user.
          */
        function ifSuccess(fn, message) {
            return function (err, result) {
                if (err) {
                    if (message)
                        window.alert(message);
                }
                else {
                    fn(result);
                }
            };
        }
        app.ifSuccess = ifSuccess;
        //===========================================
        // State
        //===========================================
        var routes = [];
        app.tasks = [];
        var state = 0 /* stopped */;
        var events = {};
        app.capture;
        var routeId;
        var appStarted = new client.AsyncEvent(); // Signaled when the app finishes starting
        client.History.init();
        //===========================================
        // Event handles
        //===========================================
        window.onpopstate = function onPopState(ev) {
            // First see if there is a popup to close, otherwise back in history
            if (client.PopUp.back()) {
                return;
            }
            client.logger.info('history state poped', ev.timeStamp);
            client.History.popState(ev);
            if (ev.state && ev.state.id !== routeId) {
                changeState(5 /* popping */);
                client.logger.info('window onpopstate to', ev.state);
                navigateToRoute(ev.state);
                changeState(6 /* popped */);
            }
            else {
                client.logger.info('pop state called with ev.state =', ev.state);
            }
        };
        window.addEventListener('orientationchange', function (ev) {
            fireEvent('orientationchange', ev);
        }, true);
        /** Trigger application restart */
        // Used for share targets that arrive to a backgrounded application
        window.addEventListener('restartapplication', function (ev) {
            restartApplication();
        });
        /** Time when application was paused */
        var pausedTime = null;
        /** Note time put into background */
        document.addEventListener('pause', function () {
            pausedTime = moment();
        });
        /** Called when app is resumed from background.  Force restart if longer than defined time  */
        document.addEventListener('resume', function () {
            if (pausedTime) {
                var diff = moment().diff(pausedTime, 'minutes');
                pausedTime = null;
                if (diff >= consts.max_background_time) {
                    // Delay slightly as per cordova docs
                    setTimeout(restartApplication, 100);
                }
            }
        });
        window.addEventListener('hashchange', function (ev) {
            if (location.hash && location.hash.indexOf('#!') == 0) {
                // Exctract command
                var pos = location.hash.indexOf('?');
                var command = pos > 0 ? location.hash.substring(2, pos) : location.hash.substr(2);
                var search = pos > 0 ? location.hash.substr(pos) : '';
                fireEvent('command', { command: command, query: parseQueryString(search) });
            }
        });
        //===========================================
        // Helpers
        //===========================================
        /** Transitions app to a new state. */
        function changeState(newState) {
            fireEvent('statechange', { newState: newState });
            state = newState;
        }
        /** Fires an event off the app object. */
        function fireEvent(type, ev) {
            var t = type.toLowerCase();
            var l = events[t];
            if (l) {
                for (var i = 0; i < l.length; i++) {
                    try {
                        l[i](ev);
                    }
                    catch (e) {
                        client.logger.error(consts.errorCode.UnhandledException, 'Unhandled app event exception', type, e);
                    }
                }
            }
        }
        /** Navigates the app to the current route. */
        function navigateToRoute(loc) {
            client.logger.info('navigateToRoute current state', state, 'loc', loc);
            if (app.capture) {
                app.capture(loc);
            }
            else if (state === 3 /* navigated */ || state === 1 /* starting */ || state === 6 /* popped */ || state === 5 /* popping */) {
                loc.state = state;
                client.app.route(loc);
            }
        }
        function navigateToRoute2(loc) {
            if (app.capture) {
                app.capture(loc);
            }
            else if (state === 3 /* navigated */ || state === 1 /* starting */) {
                // Match routes
                routeId = loc.id;
                var routeFound = false;
                for (var i = 0; i < routes.length && !routeFound; i++) {
                    // Check for matched path
                    var r = routes[i];
                    var m = r.path.exec(loc.path);
                    if (!m)
                        continue;
                    routeFound = true;
                    changeState(2 /* navigating */);
                    try {
                        // Copy matched param values
                        var params = {};
                        for (var i = 1; i < m.length; i++) {
                            var key = r.params[i - 1];
                            if (!key)
                                continue;
                            try {
                                params[key.name] = ('string' == typeof m[i] ? decodeURIComponent(m[i]) : m[i]);
                            }
                            catch (e) {
                                client.logger.error(consts.errorCode.Routing, 'Exception decoding route param', key, m[i], e);
                            }
                        }
                        // Navigate to route
                        r.handler({ id: loc.id, path: loc.path, params: params, query: loc.query, data: loc.data });
                    }
                    catch (e) {
                        client.logger.error(consts.errorCode.Routing, 'Unhandled exception navigating to route', loc, e);
                    }
                    changeState(3 /* navigated */);
                }
                // Fire routeNotFound event
                if (!routeFound)
                    fireEvent('routenotfound', loc);
            }
        }
    })(app = client.app || (client.app = {}));
})(client || (client = {}));
// Implement plugin
(function ($) {
    $.fn.onNavigate = function onNavigate(selector, namespace, callback) {
        return this.on(namespace ? 'click.' + namespace : 'click', selector || '.navigation', _.debounce(function (ev) {
            var $link = $(ev.target).attr('href') ? $(ev.target) : $(ev.currentTarget);
            var href = $link.attr('href');
            if (href && href.substr(0, 1) == '#') {
                ev.preventDefault();
                var pos = href.indexOf('?');
                var path = pos > 0 ? href.substring(1, pos) : href.substr(1);
                var search = pos > 0 ? href.substr(pos + 1) : '';
                var qs = client.app.parseQueryString(search);
                var loc = { id: 0, path: path, query: qs };
                if (!callback || callback(loc)) {
                    client.app.navigate(loc.path, loc.query);
                }
                else {
                    client.logger.info('missing handler for navigate click');
                }
            }
        }, 150, true));
    };
    $.fn.offNavigate = function offNavigate(namespace) {
        return this.off('click.' + namespace);
    };
})(jQuery || $);
var client;
(function (client) {
    var ScreenOverlay;
    (function (ScreenOverlay) {
        /** Duration of transition effect */
        // Note: Must match @SCREENOVERLAY_TRANSITION_TIME in css
        ScreenOverlay.TRANSITION_TIME = 300;
        /** Is the overlay currently visible */
        ScreenOverlay.overlayVisible = false;
        /** Callback to call when overlay is clicked */
        ScreenOverlay.clickCallback = null;
        /** Timeout that handles hiding overlay after animation */
        ScreenOverlay.hideTimeout = null;
        /** Selector for the overlay */
        var selector = null;
        /** Animate in the overlay and call callback when it is clicked */
        function show(opacity, clickCallback) {
            // Set onClick callback
            ScreenOverlay.clickCallback = clickCallback;
            // Initialize click handler
            if (!selector) {
                selector = $("#screenOverlay");
                // Hide when overlay clicked
                selector.on('click', function (event) {
                    event.preventDefault();
                    event.stopPropagation();
                    if (ScreenOverlay.clickCallback) {
                        ScreenOverlay.clickCallback();
                    }
                });
            }
            // Show if not already visible
            if (!ScreenOverlay.overlayVisible) {
                // Clear existing timeout if see
                clearTimeout(ScreenOverlay.hideTimeout);
                ScreenOverlay.hideTimeout = null;
                ScreenOverlay.overlayVisible = true;
                // Show the overlay
                selector.show();
                // Force redraw so that opacity animates in
                selector.width();
                selector.css({ 'opacity': opacity });
            }
        }
        ScreenOverlay.show = show;
        /** Animate out and hide the overlay */
        function hide() {
            if (ScreenOverlay.overlayVisible) {
                ScreenOverlay.overlayVisible = false;
                selector.css({ 'opacity': 0 });
                // Hide at end of opacity animation
                ScreenOverlay.hideTimeout = setTimeout(function () {
                    selector.hide();
                    ScreenOverlay.hideTimeout = null;
                }, ScreenOverlay.TRANSITION_TIME);
            }
        }
        ScreenOverlay.hide = hide;
    })(ScreenOverlay = client.ScreenOverlay || (client.ScreenOverlay = {}));
})(client || (client = {}));
/// <reference path="../types.ts"/>
/// <reference path="../screens/ScreenOverlay.ts"/>
/// <reference path="../../../types/jquery/jquery.d.ts"/>
var client;
(function (client) {
    /** User interface popup */
    var PopUp = (function () {
        // Expects Jade file named <popUpClass> and less file with .<popUpClass> style
        // NOTE: Intentionally does not automatically show the PopUp.  Show(true) much be a called
        function PopUp($el, popUpClass, data, popUpOptions) {
            if (popUpOptions === void 0) { popUpOptions = {}; }
            this.$el = $el;
            this.visible = false;
            this.emptyTimeout = null;
            this.options = null;
            this.onClose = new client.Event();
            /** Should the pop up be deleted when closed? */
            this.destroyOnClose = false;
            /** Should the pop up lock the screen? */
            this.lockVertical = false;
            /** Was selector an insterted one */
            this.insertedSelector = false;
            this.options = $.extend({}, PopUp.defaultOptions, popUpOptions);
            // If not given a selector insert one
            if (!this.$el) {
                this.$el = $('<div class="popUp"/>');
                this.$el.appendTo('#screens');
                this.insertedSelector = true;
                // Always destroy inserted popups when closing
                this.destroyOnClose = true;
                this.$el.on('click', '#back', function (evt) {
                    evt.preventDefault();
                    if (cordova) {
                        cordova.fireDocumentEvent('backbutton');
                    }
                    else {
                        PopUp.back();
                    }
                });
            }
            // If drag to close initialze drag capture
            if (this.options.dragToClose) {
                this.initDragToClose();
            }
            // Clear any remaining timeout 
            clearTimeout(this.emptyTimeout);
            // Render into screen
            this.$el.render(popUpClass, { data: data });
            this.$popup = this.$el.find("." + popUpClass);
            this.$popup.width();
            // Start hidden.  Show(true) much be a called 
            this.$el.hide();
        }
        // If open popup is on stack closes it and returns True. Otherwise False
        PopUp.back = function () {
            // If no popups are open, return false
            if (PopUp.history.length == 0) {
                return false;
            }
            // Otherwise close the more recent popup and return true
            var popup = PopUp.history[PopUp.history.length - 1];
            popup.show(false);
            return true;
        };
        /** Initialie ability to drag to close popup */
        PopUp.prototype.initDragToClose = function () {
            var _this = this;
            // Initialize hammer
            this.$el.hammer({ prevent_default: false, drag_min_distance: 5 });
            // Close popup on drag or tap
            this.$el.bind('drag', function (e) {
                if (Math.abs(e.distanceX) > 100) {
                    _this.show(false);
                }
            });
            this.$el.bind('tap', function (e) {
                e.stopPropagation();
                _this.show(false);
            });
        };
        /** Called when overlay is clicked.  Hides popup */
        PopUp.prototype.onOverlayClick = function () {
            this.show(false);
        };
        /** Called when popup is destroyed */
        PopUp.prototype.onDispose = function () {
            // Allow subclasses to take action
        };
        PopUp.prototype.isVisible = function () {
            return this.visible;
        };
        // Animate out and then remove from DOM
        PopUp.prototype.hide = function () {
            this.show(false);
        };
        // Subclasses may have custom rendering steps
        PopUp.prototype.render = function () {
        };
        /** Stop showing and remove from DOM */
        PopUp.prototype.destroy = function () {
            this.destroyOnClose = true;
            this.show(false);
        };
        /** Show or hide popup */
        PopUp.prototype.show = function (show) {
            var _this = this;
            // SHOW
            if (show && !this.visible) {
                this.visible = true;
                clearTimeout(this.emptyTimeout);
                // If another popup using the same element and had a close timeout 
                // running, stop the clost timeout, so I don't get closed
                if (PopUp.closingPopUp && PopUp.closingPopUp.$popup.selector == this.$popup.selector) {
                    clearTimeout(PopUp.closingPopUp.emptyTimeout);
                }
                // Lock screen if requested
                if (this.lockVertical && window.screenlock) {
                    window.screenlock.set(consts.ScreenLockState.vertical);
                }
                // Allow for custom render step
                this.render();
                this.$el.show();
                // Animate in
                this.$el.removeClass("popUpHide").addClass("popUpShow");
                // Show overlay if requested
                if (this.options.overlayOpacity > 0) {
                    client.ScreenOverlay.show(this.options.overlayOpacity, _.bind(this.onOverlayClick, this));
                }
                // Add to history (to catch back events)
                PopUp.history.push(this);
            }
            else if (!show && this.visible) {
                this.visible = false;
                //Animate out
                this.$el.addClass("popUpHide").removeClass("popUpShow");
                // Hide overlay if in use
                if (this.options.overlayOpacity > 0) {
                    client.ScreenOverlay.hide();
                }
                // Unlock screen so user can rotate again, if they are in a Xim
                if (this.lockVertical && window.screenlock && client.user.currentXim) {
                    window.screenlock.clear();
                }
                // Destroy or hide at end of animation
                this.emptyTimeout = setTimeout(function () {
                    // Destroy pop-up if requested
                    if (_this.destroyOnClose) {
                        // If built on instered selector, remove it
                        if (_this.insertedSelector) {
                            _this.$el.remove();
                        }
                        else {
                            _this.$el.empty();
                        }
                        _this.onDispose();
                    }
                    else {
                        _this.$el.hide();
                    }
                }, client.ScreenOverlay.TRANSITION_TIME);
                // Remove from history
                PopUp.closingPopUp = PopUp.history.pop();
                if (PopUp.closingPopUp != this) {
                    client.logger.error(consts.errorCode.Navigation, 'PopUp history stack is incorrect');
                }
                // Trigger close event
                this.onClose.trigger(null);
            }
        };
        PopUp.history = [];
        PopUp.closingPopUp = null;
        PopUp.defaultOptions = {
            dragToClose: false,
            overlayOpacity: 0
        };
        return PopUp;
    })();
    client.PopUp = PopUp;
})(client || (client = {}));
/// <reference path="../types.ts"/>
/// <reference path="../../../types/jquery/jquery.d.ts"/>
/// <reference path="PopUp.ts"/>
var client;
(function (client) {
    var ErrorScreens = (function () {
        function ErrorScreens() {
        }
        ErrorScreens.CONNECT_CLOUD = {
            text1top: 420,
            text2top: 500,
            buttontop: 650,
            svg: "svg/connectCloud.svg"
        };
        ErrorScreens.SLEEP_SUN = {
            text1top: 85,
            text2top: 210,
            buttontop: 380,
            svg: "svg/sleepSun.svg"
        };
        ErrorScreens.FAIL_SUN = {
            text1top: 450,
            text2top: 550,
            buttontop: 650,
            svg: "svg/failSun.svg"
        };
        ErrorScreens.CURIOUS_SUN = {
            text1top: 400,
            text2top: 530,
            buttontop: 650,
            svg: "svg/curiousSun.svg"
        };
        ErrorScreens.WAITING_SUN = {
            text1top: 400,
            text2top: 530,
            buttontop: 650,
            svg: "svg/waitingSun.svg"
        };
        return ErrorScreens;
    })();
    client.ErrorScreens = ErrorScreens;
    var ErrorMessages = (function () {
        function ErrorMessages() {
        }
        ErrorMessages.CONNECTION_ERROR = {
            text1: "We've lost connection!",
            text2: "Please check your internet connection.",
            buttonText: "Try again?",
            screenType: ErrorScreens.CONNECT_CLOUD
        };
        ErrorMessages.CONNECTION_BACK = {
            text1: "We've lost connection!",
            text2: "Please check your internet connection.",
            buttonText: "Back",
            screenType: ErrorScreens.CONNECT_CLOUD
        };
        ErrorMessages.RECONNECTING = {
            text1: "Fixing a lost connection!",
            text2: "Please try again in a moment.",
            buttonText: "Back",
            screenType: ErrorScreens.CONNECT_CLOUD
        };
        ErrorMessages.UPGRADE_REQUIRED = {
            text1: "This version of the app has retired.",
            text2: "Please install an update to get back to ximming.",
            buttonText: "Go to the app store?",
            screenType: ErrorScreens.SLEEP_SUN
        };
        ErrorMessages.USER_NOT_FOUND = {
            text1: "We can't seem to find this user.",
            text2: "",
            buttonText: "Restart?",
            screenType: ErrorScreens.CURIOUS_SUN
        };
        ErrorMessages.NOT_AUTHORIZED = {
            text1: "Device is not authorized.",
            text2: "",
            buttonText: "Restart?",
            screenType: ErrorScreens.CURIOUS_SUN
        };
        ErrorMessages.FORBIDDEN = {
            text1: "Device is not authorized.",
            text2: "",
            buttonText: "Restart?",
            screenType: ErrorScreens.CURIOUS_SUN
        };
        ErrorMessages.TOKEN_EXPIRED = {
            text1: "Device login has expired.",
            text2: "",
            buttonText: "Restart?",
            screenType: ErrorScreens.CURIOUS_SUN
        };
        ErrorMessages.BAD_REQUEST = {
            text1: "Bad request",
            text2: "",
            buttonText: "Retry?",
            screenType: ErrorScreens.CURIOUS_SUN
        };
        ErrorMessages.SERVER_ERROR = {
            text1: "Something has gone wrong.",
            text2: "Sorry",
            buttonText: "Retry?",
            screenType: ErrorScreens.FAIL_SUN
        };
        ErrorMessages.WAITLISTED = {
            text1: "You've been waitlisted!",
            text2: "Not to worry, we're unlocking accounts left and right.  We'll send a text when you've been unlocked.",
            buttonText: "",
            screenType: ErrorScreens.WAITING_SUN
        };
        ErrorMessages.XIM_EXPIRED = {
            text1: "This xim has expired!",
            text2: "Xims have a life of 60 minutes.",
            buttonText: "ok",
            screenType: ErrorScreens.FAIL_SUN
        };
        ErrorMessages.PHOTO_ADD_FAIL = {
            text1: "Unable to add photos!",
            text2: "Check your connection and try again.",
            buttonText: "ok",
            screenType: ErrorScreens.FAIL_SUN
        };
        ErrorMessages.PEOPLE_ADD_FAIL = {
            text1: "Unable to add people!",
            text2: "Check your connection and try again.",
            buttonText: "ok",
            screenType: ErrorScreens.FAIL_SUN
        };
        return ErrorMessages;
    })();
    client.ErrorMessages = ErrorMessages;
    /** App Error Messager */
    var ErrorPopUp = (function (_super) {
        __extends(ErrorPopUp, _super);
        function ErrorPopUp(errorMessage, callback) {
            var _this = this;
            // Call superclass to render Popup
            var isWp = (client.startupMode == 3 /* wp */);
            _super.call(this, $('.app-message'), "errorPopUp", { message: errorMessage, isWp: isWp });
            if (callback) {
                this.onClose.add(callback);
            }
            this.$el.on('mousedown touchstart', '.errorPopUp', function () {
                var $cmd = _this.$el.find('.errorButton');
                $cmd.addClass('errorButtonPressed');
                // Remove pressed style after a pause in case user moved mouse off target (which is not a click)
                setTimeout(function () {
                    $cmd.removeClass('errorButtonPressed');
                }, 500);
            });
            this.$el.on('click', '.errorPopUp', function () {
                _this.destroy();
            });
            $('.app-message').show();
        }
        return ErrorPopUp;
    })(client.PopUp);
    client.ErrorPopUp = ErrorPopUp;
})(client || (client = {}));
///<reference path='./screen.ts'/>
var client;
(function (client) {
    var wizard;
    (function (_wizard) {
        _wizard.USER_BACKED_OUT = 'user backed out';
        function wrapScreen(screen) {
            return (function (c) {
                c(null, screen);
            });
        }
        _wizard.wrapScreen = wrapScreen;
        _wizard.currentWizard = null;
        var screenWizard = (function () {
            function screenWizard(flow, callback, commands, commandCallback) {
                this.flow = flow;
                this.callback = callback;
                this.commands = commands;
                this.commandCallback = commandCallback;
                this.left = [];
                this.hwizard = null;
                this.bag = {};
                this.args = [];
                this.onFinished = null;
                var wizard = this;
                this.right = _.clone(flow);
                this.hwizard = {
                    page: 0,
                    back: function () {
                    },
                    forward: function (params) {
                    },
                    close: function (err, results) {
                        wizard.args = [results];
                        var lastIndex = wizard.left.length - 1;
                        if (!wizard.next()) {
                            // no more screen to show
                            _.each(wizard.left, function (screen, i) {
                                screen.remove();
                            });
                            wizard.cleanup(null, results);
                        }
                        else {
                            //
                            wizard.left[lastIndex].hide(2 /* none */);
                        }
                    }
                };
            }
            screenWizard.prototype.cleanup = function (e, result) {
                var _this = this;
                var wrapFinished = this.onFinished;
                client.app.releaseCapture();
                if (this.prevCapture) {
                    client.app.captureNavigates(this.prevCapture);
                    this.prevCapture = null;
                }
                if (this.prevModal) {
                    this.prevModal.show(2 /* none */);
                    this.prevModal = null;
                    if (wrapFinished) {
                        wrapFinished = _.wrap(wrapFinished, function (fn) {
                            fn();
                            // put the master screen back to its original state
                            var $masterScreen = $((client.screens.master).selector).children('.master.screen');
                            if (!$masterScreen.hasClass('show-dialog-flush'))
                                $masterScreen.hide();
                        });
                    }
                }
                _wizard.currentWizard = null;
                // clean up history entry
                var loc = history.state || client.History.state();
                if (loc && loc.page) {
                    client.logger.info('wizard cleanup going back', loc.page);
                    // history go might be async so wire up the callback after it is done
                    var prev = window.onpopstate;
                    var final = _.once(function (ev) {
                        client.logger.info('pop state event fired in cleanup', ev ? ev.state : null);
                        if (!ev || ev.state && ((typeof ev.state.page == 'undefined') || ev.state.page == 1)) {
                            // if the app hasn't been started, do call the prev, otherwise, we will
                            // start trigger navigate inside the app start sequence, which may cause
                            // unexpected screen states (timeline screen rendered before the loading screen
                            // is done and login sequence is finished, see bug 18547).
                            if (client.app.isStarted() && ev) {
                                prev(ev);
                            }
                            if (wrapFinished)
                                wrapFinished();
                            try {
                                _this.callback(e, result);
                            }
                            catch (ex) {
                                client.logger.debug('wizard cleanup exception', ex);
                            }
                            window.onpopstate = prev;
                        }
                        else {
                            client.logger.info('no action taken on pop history state', ev.state.page);
                        }
                    });
                    window.onpopstate = final;
                    client.logger.info('pop history by ', loc.page, ' steps');
                    var backtrack = result ? loc.page + 1 : loc.page;
                    client.logger.info('pop history using history.go');
                    _.delay(final, 200);
                    history.go(0 - Math.min(backtrack, history.length - 1));
                }
                else {
                    if (wrapFinished)
                        wrapFinished();
                    this.callback(e, result);
                }
            };
            // Extract screen name for debugging
            screenWizard.prototype.screenName = function (screen) {
                if (client.user.GetSetting(6 /* SHOW_DEBUG */) || (parseFloat(consts.applicationVersion) < 1.0)) {
                    var name = screen.toString();
                    name = name.substr('function '.length);
                    name = name.substr(0, name.indexOf('('));
                    return name;
                }
                return "";
            };
            screenWizard.prototype.nextScreen = function (callback) {
                // we are skipping the current screenGenerator
                // better push a dummy screen on the left stack
                // so that we can pop the right number of screen generator back
                // during handleNavigate
                var loc = { path: '', params: {}, wizardIndex: this.left.length, page: this.hwizard.page + 1, bag: this.bag, screenName: 'fakeScreen' };
                history.pushState(loc, 'wizard');
                this.left.push({ remove: function () {
                } });
                var screenGenerator = this.right.shift();
                if (screenGenerator) {
                    screenGenerator(callback);
                }
                else {
                    callback(new Error('no more screen generator'), null);
                }
            };
            screenWizard.prototype.next = function () {
                var _this = this;
                if (this.right.length) {
                    var screenGenerator = this.right.shift();
                    // inject the callback in front
                    this.args.unshift(
                    // AsyncCallback<IScreenConstructor>
                    function (e, screen, params) {
                        if (screen == null) {
                            var close = _.find(_this.commands, function (c, i) {
                                return c.id == consts.commands.cancel.id;
                            });
                            if (close) {
                                _this.commandCallback(close.id, close);
                            }
                            else {
                                // show the current screen again
                                _.last(_this.left).show(2 /* none */);
                                // unwind the next to the right
                                _this.right.unshift(screenGenerator);
                            }
                        }
                        else {
                            var $modal = $('<div class="screen screen-modal"/>');
                            var cur = new screen($modal, _this.hwizard, _this.commands, _this.commandCallback);
                            var screenName = _this.screenName(screen);
                            var loc = { path: '', params: params || {}, wizardIndex: _this.left.length, page: _this.hwizard.page + 1, bag: _this.bag, screenName: screenName };
                            //client.logger.info('wizard push state to ', loc);
                            history.pushState(loc, 'wizard');
                            // reset window scroll position
                            _.defer(function () {
                                window.scroll(0, 0);
                            });
                            cur.navigate(_.omit(loc, 'wizardIndex'));
                            cur['wizardIndex'] = loc.wizardIndex;
                            _this.left.push(cur);
                            $modal.appendTo(_this.selector);
                        }
                    });
                    screenGenerator.apply(null, this.args);
                    return true;
                }
                return false;
            };
            screenWizard.prototype.handleNavigate = function (loc) {
                var _this = this;
                client.logger.info('wizard captured navigate to ', loc);
                if (loc.hasOwnProperty('wizardIndex')) {
                    var wizardIndex = loc['wizardIndex'];
                    var restore = [];
                    _.each(this.left, function (screen, i) {
                        if (i > wizardIndex) {
                            screen.remove();
                            // put the screen generator back
                            restore.push(_this.flow[i]);
                        }
                    });
                    if (restore.length) {
                        this.right.unshift.apply(this.right, restore);
                        // also reset the commands
                        this.commands = [];
                    }
                    this.hwizard.page -= restore.length;
                    this.left.splice(wizardIndex + 1);
                    client.logger.info('wizard left length is now', this.left.length);
                    if (this.left.length) {
                        var last = _.last(this.left);
                        if (this.isRealScreen(last)) {
                            if (restore.length == 0) {
                                last.navigate(loc);
                            }
                            else {
                                last.show(2 /* none */);
                                last.syncNoPan();
                            }
                        }
                        else {
                            history.back();
                        }
                    }
                    else {
                        this.cleanup(new Error(_wizard.USER_BACKED_OUT), null);
                    }
                }
                else {
                    _.last(this.left).navigate(loc);
                }
            };
            screenWizard.prototype.begin = function () {
                var _this = this;
                // show the first screen
                _wizard.currentWizard = this;
                if (client.app.isCaptured()) {
                    this.prevModal = client.screens.modal;
                    this.prevCapture = client.app.capture;
                    client.app.capture = null;
                }
                client.app.captureNavigates(function (loc) {
                    _this.handleNavigate(loc);
                });
                history.pushState({ wizardIndex: -1, page: 1 }, 'wizard');
                client.screens.master.beginModal({ style: 0 /* dialog */ }, function (s, f) {
                    _this.selector = s;
                    _this.onFinished = f;
                    _this.next();
                });
                if (this.prevModal) {
                    this.prevModal.hide(2 /* none */);
                }
            };
            /** check if screen object in the left stack is for real */
            screenWizard.prototype.isRealScreen = function (screen) {
                return screen.hasOwnProperty('bag');
            };
            /** return the current active screen, ignore any fake one put up as filler for back navigation*/
            screenWizard.prototype.currentScreen = function () {
                var last = this.left ? this.left.length - 1 : -1;
                while (last >= 0) {
                    var cur = this.left[last];
                    if (this.isRealScreen(cur)) {
                        return cur;
                    }
                    last--;
                }
                return null;
            };
            screenWizard.prototype.finalize = function (result) {
                _.each(this.left, function (screen, i) {
                    screen.remove();
                });
                this.left = null;
                this.right = null;
                this.flow = null;
                client.logger.debug('clean up wizard');
                this.cleanup(null, result);
            };
            return screenWizard;
        })();
        function wizard(flow, callback, commands, commandCallback) {
            var sw = new screenWizard(flow, callback);
            sw.commands = commands;
            sw.commandCallback = commandCallback;
            return sw;
        }
        _wizard.wizard = wizard;
    })(wizard = client.wizard || (client.wizard = {}));
})(client || (client = {}));
/// <reference path="../types.ts"/>
var client;
(function (client) {
    client.SideMenuPosition = { left: "Left", right: "Right", bottom: "Bottom", none: "None" };
    /** A menu that slides in from the left, right or bottom */
    var SideMenu = (function () {
        function SideMenu(selector, callback) {
            this.state = 0 /* closed */;
            this.position = client.SideMenuPosition.left;
            this.options = { itemHeight: 50, visibleOverlay: true, opacity: 1.0 }; // Default should match @SIDE_MENU_ITEM_HEIGHT in .less file
            this.$el = $(selector);
            this.callback = callback;
        }
        SideMenu.prototype.Close = function () {
            this.show(false);
        };
        SideMenu.prototype.isOpen = function () {
            return this.state == 1 /* open */;
        };
        SideMenu.prototype.show = function (visible, active) {
            if (this.state == 1 /* open */ && !visible) {
                this.$el.find(".sideMenu" + this.position).removeClass("sideOpen" + this.position);
                this.$el.find(".sideMenuOverlay").removeClass("open").addClass("close");
                this.clearActive();
                this.state = 0 /* closed */;
            }
            else if (this.state == 0 /* closed */ && visible) {
                this.$el.find(".sideMenu" + this.position).addClass("sideOpen" + this.position);
                this.$el.find(".sideMenuOverlay").removeClass("close").addClass("open");
                if (active)
                    this.setActive(active);
                this.state = 1 /* open */;
            }
        };
        /* move menu to position */
        SideMenu.prototype.setPosition = function (pos) {
            this.$el.find('.sideMenu').css({ 'position': 'absolute', 'top': pos.y + 'px', 'left': pos.x + 'px' });
            return this;
        };
        // Clear any displayed nubs
        SideMenu.prototype.clearActive = function () {
            var selected = this.$el.find(".sideMenuSelected");
            selected.removeClass("sideMenuSelected");
        };
        // Show nub on given command ID
        SideMenu.prototype.setActive = function (command) {
            var active = this.$el.find("#" + command);
            active.addClass("sideMenuSelected");
        };
        /** Sets commands and renders */
        SideMenu.prototype.setCommands = function (commands) {
            this.commands = commands;
            this.renderCommands();
        };
        /** Use alternative label (when menu command has more than one */
        SideMenu.prototype.useAltLabel = function (command, useAlt) {
            var text = useAlt ? command.altLabel : command.label;
            this.$el.find('#' + command.id + "> span.sideLabel").text(text);
        };
        //=================================================
        // Private Methods
        //=================================================
        SideMenu.prototype.renderCommands = function () {
            // Main container
            var smHTML = '<div class="sideMenuHolder">';
            // Add overlay 
            if (this.position != client.SideMenuPosition.none) {
                var overlayStyle = (this.options.visibleOverlay) ? '' : 'style="opacity: 0;"';
                smHTML += '<div class="sideMenuOverlay"' + overlayStyle + '></div>';
            }
            // Add side menu
            var sideMenuStyle = (this.position == client.SideMenuPosition.bottom) ? 'style="bottom: 78px; height:' + (this.commands.length * this.options.itemHeight) + 'px;"' : '';
            smHTML += '<div class="sideMenu' + this.position + ' sideMenu"' + sideMenuStyle + '>';
            // Add tray and menu items
            smHTML += '<div class="sideMenuTray sideMenuTray' + this.position + '" style="opacity:' + this.options.opacity + '">';
            smHTML += '<ul class ="sideMenuNav">';
            for (var i = 0; i < this.commands.length; i++) {
                smHTML += '<li id="' + this.commands[i].id + '" class="sideMenuLi" + style="line-height:' + this.options.itemHeight + 'px;">';
                smHTML += '<span class="sideLabel">' + this.commands[i].label + '</span>';
                if (this.commands[i].icon) {
                    smHTML += '<img class ="sideMenuIcon" src="' + this.commands[i].icon + '">';
                }
                else if (this.commands[i].glyph) {
                    smHTML += '<span class="' + this.commands[i].glyph + ' sideMenuGlyph"></span>';
                }
                smHTML += '</li>';
            }
            smHTML += '</ul></div>'; // SideMenuTray
            smHTML += '</div>'; // SideMenu
            smHTML += '</div>'; // SideMenuContainer
            this.$el.find('.sideMenuHolder').remove();
            this.$el.append(smHTML);
            for (i = 0; i < this.commands.length; i++) {
                var id = this.commands[i].id;
                var callback = this.callback;
                this.$el.find('#' + id).click({ id: this.commands[i].id }, function (event) {
                    event.preventDefault();
                    callback(event.data.id);
                });
            }
            // keep the menu open if it is previously open
            if (this.state == 1 /* open */) {
                this.state = 0 /* closed */;
                this.show(true);
            }
        };
        return SideMenu;
    })();
    client.SideMenu = SideMenu;
})(client || (client = {}));
/// <reference path="../types.ts"/>
/// <reference path="../resources.ts"/>
/// <reference path="../../../types/jquery/jquery.d.ts"/>
/// <reference path="PopUp.ts"/>
var client;
(function (client) {
    /** User interface for a 2 choice pop-up */
    var ConfirmationPopUp = (function (_super) {
        __extends(ConfirmationPopUp, _super);
        function ConfirmationPopUp($el, title, message, leftChoice, rightChoice, callback) {
            var _this = this;
            // Call superclass to render Popup
            _super.call(this, $el, "confirmationPopUp", { title: title, message: message, leftChoice: leftChoice, rightChoice: rightChoice }, { overlayOpacity: 0.7 });
            this.leftChoice = leftChoice;
            this.rightChoice = rightChoice;
            // Close on click, and eat other clicks so they don't propagate upward 
            this.$popup.on('click', function (event) {
                if (_this.isVisible()) {
                    event.preventDefault();
                    event.stopPropagation();
                    _this.destroy();
                }
            });
            // Capture the users choice
            this.$popup.on('click', '#confirmationPopUpLeft', function () {
                callback(_this.leftChoice);
            });
            this.$popup.on('click', '#confirmationPopUpRight', function () {
                callback(_this.rightChoice);
            });
            // Show automatically (necessary for resize)
            this.show(true);
            // Size to center in screen
            this.resize();
        }
        // Center on screen (takes into account that it may have rotated)
        ConfirmationPopUp.prototype.resize = function () {
            var screenWidth = client.webview.currentViewport().width;
            var screenHeight = client.webview.currentViewport().height;
            var x = (screenWidth - this.$popup.width()) / 2;
            var y = -100 + (screenHeight - this.$popup.height()) / 2;
            this.$popup.css({ marginLeft: Math.round(x) + 'px', marginTop: Math.round(y) + 'px' });
        };
        return ConfirmationPopUp;
    })(client.PopUp);
    client.ConfirmationPopUp = ConfirmationPopUp;
})(client || (client = {}));
/// <reference path='./screen.ts'/>
var client;
(function (client) {
    var dialog;
    (function (dialog) {
        function cancelDialog() {
            // cancel the dialog
            var loc = history.state || client.History.state();
            if (loc && loc.page) {
                history.go(0 - loc.page);
            }
        }
        function bindDialogHeader($el) {
            $el.on('click', '.dialog.header a.action', function (e) {
                var $link = $(e.currentTarget);
                e.preventDefault();
                if ($link.hasClass('dialog-back')) {
                    history.go(-1);
                }
                else if ($link.hasClass('dialog-close')) {
                    cancelDialog();
                }
            });
        }
        dialog.bindDialogHeader = bindDialogHeader;
        function handleDialogCommand(id, command) {
            if (id == consts.commands.cancel.id) {
                cancelDialog();
                return true;
            }
            else if (id == consts.commands.back.id) {
                history.back();
                return true;
            }
            return false;
        }
        dialog.handleDialogCommand = handleDialogCommand;
        function renderDialogCommandBar($el, commands, commandCallback, commandBarStyle) {
            if (commands && commandCallback) {
                var commandBar = new client.CommandBar($el.find('.command-bar'), { textSize: 28, glyphSize: 0 });
                if (commandBarStyle) {
                    _.extend(commandBar.commandBarStyle, commandBarStyle);
                }
                commandBar.setCommands(commands, commandCallback);
                return commandBar;
            }
            return null;
        }
        dialog.renderDialogCommandBar = renderDialogCommandBar;
    })(dialog = client.dialog || (client.dialog = {}));
    var DialogScreen = (function (_super) {
        __extends(DialogScreen, _super);
        function DialogScreen(sel, history, commands, commandCallback) {
            _super.call(this, sel, history);
            this.history = history;
            this.commands = commands;
            this.commandCallback = commandCallback;
            this.bag = null;
            this.CommandBarStyle = { bar: 'commandBarClosed' };
            this.prevDarkTheme = false;
            var $body = $('body');
            this.prevDarkTheme = $body.hasClass('dark-theme');
            $body.toggleClass('dark-theme', false);
        }
        DialogScreen.prototype.onDispose = function () {
            if (this.prevDarkTheme) {
                $('body').toggleClass('dark-theme', true);
            }
            _super.prototype.onDispose.call(this);
        };
        DialogScreen.prototype.render = function () {
            if (this.commands && this.commandCallback) {
                this.commandBar = dialog.renderDialogCommandBar(this.$el, this.commands, this.commandCallback, this.CommandBarStyle);
            }
            this.syncNoPan();
            return _super.prototype.render.call(this);
        };
        DialogScreen.prototype.setDialogTitle = function (title) {
            // do nothing for now
            //this.$el.find('.dialogTitle label').text(title);
        };
        return DialogScreen;
    })(client.Screen);
    client.DialogScreen = DialogScreen;
})(client || (client = {}));
/// <reference path="./AppMasterScreen.ts"/>
/// <reference path='./DialogScreen.ts'/>
/// <reference path="../user.ts"/>
/// <reference path="./casting.ts"/>
var client;
(function (client) {
    /** Media Source Picking screen */
    var PickerScreen = (function (_super) {
        __extends(PickerScreen, _super);
        function PickerScreen() {
            _super.apply(this, arguments);
            this.photoUploaded = new client.Event();
        }
        PickerScreen.prototype.navigate = function (route) {
            var _this = this;
            // Update screens state
            client.logger.info('PickerScreen navigate start');
            var sources = [
                { type: 'auth', title: resources.labels.pickerSource.cm, id: 'takephoto', thumbnail: './img/takePhoto-logo.png' },
                { type: 'auth', title: client.CameraRollScreen.defaultTitle(), id: 'cameraroll', thumbnail: './img/camera-logo.png' },
                { type: 'auth', title: resources.labels.pickerSource.sk, id: 'skydrive', thumbnail: './img/OneDriveLogo.png' },
                { type: 'auth', title: resources.labels.pickerSource.db, id: 'dropbox', thumbnail: './img/dropbox-logo.png' },
                { type: 'auth', title: resources.labels.pickerSource.fb, id: 'albums', thumbnail: './img/facebook-logo.png' },
                { type: 'auth', title: resources.labels.pickerSource.ig, id: 'instagram', thumbnail: './img/instagram-logo.png' },
            ];
            // optionally turn off takephoto
            if (!client.user.GetSetting(7 /* SHOW_CAMERA_AS_SOURCE */)) {
                sources = _.filter(sources, function (s) { return s.id != 'takephoto'; });
            }
            if (!client.user.GetSetting(12 /* SHOW_CAMERAROLL */)) {
                sources = _.filter(sources, function (s) { return s.id != 'cameraroll'; });
            }
            // If in emultation add mock image source
            if (client.startupMode == 1 /* emulator */) {
                // Remove all camera related sources
                sources = _.filter(sources, function (s) { return s.id != 'cameraroll' && s.id != 'takephoto'; });
                // Add mock camera roll
                sources.unshift({ type: 'auth', title: "Mock Camera", id: 'mocksource', thumbnail: './img/camera-logo.png' });
            }
            this.photos = sources.slice(0);
            this.bag = this.bag || route.bag;
            // Redraw screen
            this.render();
            // bind events
            this.bindOnNavigate('a.brand', function () {
                client.dialog.bindDialogHeader(_this.$el);
                _this.$el.on('click', '#photos-connections a.tile-link, #docs-connections a.tile-link', function (e) {
                    var $link = $(e.currentTarget);
                    client.logger.info('user picked', $link.attr('href'));
                    if (_this.history) {
                        e.preventDefault();
                        client.AppMasterScreen.checkConnection(function () {
                            _this.history.close(null, $link.attr('href'));
                        });
                    }
                    else {
                        client.logger.info('not shown as dialog');
                    }
                });
            });
            return _super.prototype.navigate.call(this, route);
        };
        PickerScreen.prototype.render = function () {
            // Make screen is in a valid state
            if (this.getScreenState() == 3 /* DISPOSED */) {
                client.logger.warn("Render called on DISPOSED screen ");
                return this;
            }
            client.logger.info('picker render start');
            if (!this.$el) {
                client.logger.error(consts.errorCode.ContentPicking, 'this.$el not defined for picker screen');
            }
            // Render view
            this.empty();
            this.$el.render('picker', {
                title: resources.labels.pickerTitle,
                mode: client.StartupMode[client.startupMode],
                tiles: this.photos
            });
            _super.prototype.render.call(this);
            return this;
        };
        PickerScreen.prototype.mapToScreenQueueItems = function (items, callback) {
            var _this = this;
            if (items.length == 1) {
                var preview = (items[0]);
                client.CameraRollUploader.uploadFileUrlToBlob(preview.thumbnail.source, function (e, url) {
                    if (!e && url) {
                        _this.photoUploaded.trigger(null);
                        callback(null, [{ type: 'photo', url: url }]);
                    }
                    else {
                        callback(e, null);
                    }
                });
            }
            else {
                callback(new Error('only supporting upload one photo'), null);
            }
        };
        return PickerScreen;
    })(client.DialogScreen);
    client.PickerScreen = PickerScreen;
    client.screens.addRoute('/picker', PickerScreen);
})(client || (client = {}));
/// <reference path="AppMasterScreen.ts"/>
/// <reference path='./DialogScreen.ts'/>
var client;
(function (client) {
    var ContentPickingDialog = (function (_super) {
        __extends(ContentPickingDialog, _super);
        function ContentPickingDialog(sel, history, commands, commandCallback) {
            _super.call(this, sel, history, commands, commandCallback);
            this.history = history;
            this.commands = commands;
            this.commandCallback = commandCallback;
            this.lastX = 0;
            this.lastY = 0;
            this.items = null;
            this.photoUploaded = null;
            this.$el.on('click', '.tile', _.bind(this.onTileClicked, this));
        }
        ContentPickingDialog.prototype.toggleSelection = function (item) {
            var selectedItems = this.bag['selected'] || [];
            var foundIndex = -1;
            if (!_.find(selectedItems, function (sel, i) {
                var match = sel.id == item.id;
                if (match) {
                    foundIndex = i;
                }
                return match;
            })) {
                selectedItems.push(item);
            }
            else {
                selectedItems.splice(foundIndex, 1);
            }
            this.bag['selected'] = selectedItems;
            this.updateCommandBar();
        };
        ContentPickingDialog.prototype.isTileSelected = function ($tile) {
            return $tile.hasClass('tile-selected');
        };
        ContentPickingDialog.prototype.setTileSelected = function ($tile, selected) {
            return $tile.toggleClass('tile-selected', selected);
        };
        ContentPickingDialog.prototype.isItemSelected = function (item) {
            var selectedItems = this.bag['selected'] || [];
            return !!_.find(selectedItems, function (sel) {
                return item.id == sel.id;
            });
        };
        ContentPickingDialog.prototype.navigate = function (route) {
            // reset the selected items when we are on a new folder/album
            this.bag['selected'] = [];
            if (route.hasOwnProperty('wizardIndex')) {
                this.wizardIndex = route['wizardIndex'];
            }
            return _super.prototype.navigate.call(this, route);
        };
        /** Update command bar to disable/enable commands based on selection */
        ContentPickingDialog.prototype.updateCommandBar = function () {
            var contents = this.bag['selected'];
            var disabled = (!contents || contents.length == 0);
            this.commandBar.disableCommand(consts.commands.next, disabled);
            this.commandBar.disableCommand(consts.commands.addMedia, disabled);
            this.commandBar.disableCommand(consts.commands.start, disabled);
        };
        ContentPickingDialog.prototype.onClickAll = function ($tile) {
            var selectedItems = [];
            var selected = this.isTileSelected($tile);
            if (selected) {
                selectedItems = this.items.models;
                client.logger.collectNamedEvent(consts.flurryEvent.pickerAllButton);
            }
            this.bag['selected'] = selectedItems;
            // select all the selectable image tiles in the current view
            this.$el.find('.tile>.select-image-tile').toggleClass('tile-selected', selected);
            this.updateCommandBar();
        };
        ContentPickingDialog.prototype.getIconThumbnail = function (item) {
            return null;
        };
        ContentPickingDialog.prototype.fileClicked = function (tile) {
            // do nothing
        };
        ContentPickingDialog.prototype.setTitleFromRoute = function (route) {
            if (route.params && route.params.title) {
                this.title = route.params.title;
            }
            else {
                this.title = this.defaultTitle;
            }
        };
        ContentPickingDialog.prototype.contentClicked = function ($tile) {
            // find item
            var itemId = $tile.attr('id');
            var item = _.find(this.items.models, function (item, i) {
                return item.id == itemId;
            });
            if (item) {
                switch (item.type) {
                    case 'photo':
                        this.toggleSelection(item);
                        break;
                    case 'video':
                        this.toggleSelection(item);
                        break;
                }
            }
        };
        ContentPickingDialog.prototype.onTileClicked = function (evt) {
            var $tile = $(evt.currentTarget).find('>*:first-child');
            this.setTileSelected($tile, !this.isTileSelected($tile));
            if ($tile.attr('id') == 'selectAllPhotos') {
                evt.preventDefault();
                this.onClickAll($tile);
            }
            else if ($tile.hasClass('image-tile')) {
                evt.preventDefault();
                this.contentClicked($tile);
            }
            else if ($tile.attr('href')) {
                evt.preventDefault();
                this.linkClicked($tile);
            }
        };
        ContentPickingDialog.prototype.itemIdFromTileLink = function (link) {
            var prefix = '#/' + this.contentSource + '/';
            var itemId = _.last(link.split('/'));
            if (link.indexOf(prefix) == 0) {
                itemId = link.substr(prefix.length);
            }
            return itemId;
        };
        ContentPickingDialog.prototype.linkClicked = function ($tile) {
            var _this = this;
            client.logger.info('album link clicked');
            client.AppMasterScreen.checkConnection(function () {
                var link = $tile.attr('href');
                var itemId = _this.itemIdFromTileLink(link);
                var path = link.substr(1);
                var childLoc = { path: path, params: { itemid: itemId, title: $tile.text() }, page: _this.history.page + 1, wizardIndex: _this.wizardIndex };
                history.pushState(childLoc, _this.contentSource);
                _this.navigate(childLoc);
            });
        };
        ContentPickingDialog.prototype.getItemTileType = function (item) {
            if (item.type == 'text') {
                return 'textTile';
            }
            var tileType = this.getIconThumbnail(item) ? 'iconTile' : 'imageTile';
            return tileType;
        };
        ContentPickingDialog.prototype.getTiles = function (items, appendOnly) {
            var _this = this;
            if (!items)
                return [];
            var prependAll = !appendOnly && this.items && this.items.models && this.items.models.length > 0 && !_.find(this.items.models, function (item) {
                return item.type != 'photo';
            });
            var before = [];
            var after = [];
            if (prependAll) {
                before.push({
                    type: 'text',
                    name: 'selectAll',
                    title: resources.labels.tiles.all,
                    id: 'selectAllPhotos',
                    selected: false
                });
            }
            if (items.next) {
                after.push({
                    type: 'text',
                    name: 'next',
                    title: resources.labels.tiles.more,
                    id: this.contentSource + 'Next',
                    link: '#/' + this.contentSource + '/next',
                    selected: false
                });
            }
            return before.concat(_.map(items.models, function (m) {
                var extender = {};
                switch (m.type) {
                    case 'folder':
                    case 'album':
                        extender = {
                            type: m.type,
                            link: '#/' + _this.contentSource + '/' + m.id,
                            title: m.name
                        };
                        if (!m.thumbnail) {
                            extender['thumbnail'] = { width: 140, height: 140, source: './img/folder-icon.png' };
                        }
                        break;
                }
                return _.extend(m, extender);
            })).concat(after);
        };
        ContentPickingDialog.prototype.appendTiles = function (tiles) {
            if (this.getScreenState() == 3 /* DISPOSED */) {
                return;
            }
            client.logger.info('appending tiles ', tiles.length);
            var $c = this.$el.find('#' + this.contentSource + '-items .items');
            client.logger.info('found tile list', $c.length, tiles[0]);
            _.each(tiles, function (tile) {
                var templateName = tile.type == 'album' ? 'folderTile' : tile.type + 'Tile';
                $c.append(jade.templates[templateName](tile));
            });
            // check the height and turn off panning if not tall enough
            this.syncNoPan();
            return this;
        };
        ContentPickingDialog.prototype.showEmptyContainer = function () {
            this.$el.find('.message.loading').toggleClass('loading empty');
        };
        /** overridable method to map user picked content to screen queue items */
        ContentPickingDialog.prototype.mapToScreenQueueItems = function (items, callback) {
            client.logger.info('map photos to images =', items.length);
            function mapToImages(ps) {
                return _.map(ps, function (photo) {
                    if (photo.images && photo.images.length) {
                        var img = _.findMin(photo.images, function (img) {
                            return Math.max(Math.abs(img.width - 960), Math.abs(img.height - 640));
                        });
                        return { type: 'photo', url: img.source };
                    }
                    else {
                        return photo;
                    }
                });
            }
            ;
            setTimeout(function () {
                callback(null, mapToImages(items));
            });
        };
        return ContentPickingDialog;
    })(client.DialogScreen);
    client.ContentPickingDialog = ContentPickingDialog;
})(client || (client = {}));
///<reference path='../types.ts' />
///<reference path='Table.ts' />
var client;
(function (client) {
    var skydrive;
    (function (skydrive) {
        /** The value of input parameter 'filter' is a comma-separated list of the following values:
         'folders', 'albums', 'photos', 'videos', 'audio', 'notebooks'." */
        var filters = (function () {
            function filters() {
            }
            filters.all = 'all';
            filters.photos = 'photos';
            filters.videos = 'videos';
            filters.audio = 'audio';
            filters.folders = 'folders';
            filters.albums = 'albums';
            filters.notebooks = 'notebooks';
            return filters;
        })();
        skydrive.filters = filters;
        skydrive.photoFilters = filters.albums + ',' + filters.folders + ',' + filters.photos;
        var SkyApi = (function () {
            function SkyApi(access_token, filter, sort_order) {
                this.access_token = access_token;
                this.filter = filter;
                this.sort_order = sort_order;
            }
            SkyApi.prototype.get = function (url, callback) {
                var urlString = uri.resolve('https://apis.live.net/v5.0/', url);
                var apiUrl = uri.parse(urlString);
                apiUrl.query = _.extend(apiUrl.query || {}, { access_token: this.access_token });
                if (this.filter) {
                    apiUrl.query = _.extend(apiUrl.query, { filter: this.filter });
                }
                client.logger.info('about to call', urlString);
                $.ajax({
                    url: uri.format(apiUrl),
                    success: function (data, s, xhr) {
                        var contentType = xhr.getResponseHeader('Content-Type');
                        client.logger.info('request returned content type', contentType);
                        if (contentType.indexOf('image') < 0)
                            callback(null, data);
                    },
                    error: function (xhr) {
                        callback(new Error('failed with status code ' + xhr.status), null);
                    }
                });
            };
            SkyApi.prototype.getJson = function (url, transform, callback) {
                this.get(url, function (e, data) {
                    if (!e) {
                        callback(null, transform(data));
                    }
                });
            };
            SkyApi.prototype.me = function (callback) {
                var self = this;
                async.parallel([
                    function (c) {
                        self.get('me', c);
                    },
                    function (c) {
                        self.get('me/picture', c);
                    }
                ], function (e, results) {
                    if (!e) {
                        var nameJson = results[0];
                        client.logger.info('name object is', nameJson);
                        var pictureJson = results[1];
                        client.logger.info('picture object is', pictureJson);
                        callback(null, {
                            id: 'profie',
                            displayName: nameJson.name,
                            userId: 'live!' + nameJson.id,
                            thumbnailUrl: pictureJson.location
                        });
                    }
                    else {
                        callback(e, null);
                    }
                });
            };
            SkyApi.prototype.albums = function (callback) {
                this.get('me/albums', function (e, body) {
                    if (!e && body && body.data) {
                        callback(null, body.data);
                    }
                    else {
                        callback(e || new Error('me/albums returns no data'), null);
                    }
                });
            };
            SkyApi.prototype.filterBy = function (id, filter, callback) {
                this.get(id + '/files', function (e, body) {
                    if (!e) {
                        var data = body.data;
                        //client.logger.info('data.length', data.length);
                        var filtered = _.filter(data, function (item, i) {
                            return filter(item);
                        });
                        //client.logger.info('filtered length', filtered.length);
                        callback(null, filtered);
                    }
                    else {
                        callback(e, null);
                    }
                });
            };
            SkyApi.prototype.folders = function (id, callback) {
                this.filterBy(id, function (x) {
                    return x.type == 'folder';
                }, callback);
            };
            SkyApi.prototype.sharedReadLink = function (id, callback) {
                this.getJson(id + '/shared_read_link', function (a) {
                    return a.link;
                }, callback);
            };
            SkyApi.prototype.embedLink = function (item, callback) {
                if (item && item.is_embeddable) {
                    this.getJson(item.id + '/embed', function (a) {
                        var $frame = $(a.embed_html);
                        return $frame.attr('src');
                    }, callback);
                }
                else {
                    setInterval(function () {
                        callback(new Error('item not embeddable'), null);
                    }, 0);
                }
            };
            SkyApi.prototype.photos = function (id, callback) {
                this.filterBy(id, function (x) {
                    return x.type == 'photo';
                }, function (e, items) {
                    var photos = _.map(items, function (item, i) {
                        return {
                            name: item.name,
                            images: _.map(item.images, function (img, i) {
                                return { height: img.height, width: img.width, source: img.source };
                            })
                        };
                    });
                    callback(null, photos);
                });
            };
            SkyApi.prototype.skydrive = function (callback) {
                this.nextFiles('me/skydrive/files?limit=30', callback);
            };
            SkyApi.prototype.nextFiles = function (next, callback) {
                var _this = this;
                this.get(next, function (e, obj) {
                    if (!e && obj) {
                        callback(e, {
                            models: obj.data,
                            paging: obj.paging,
                            next: (obj.paging && obj.paging.next) ? function (c) {
                                _this.nextFiles(obj.paging.next.substr(1), c);
                            } : null
                        });
                    }
                    else {
                        client.logger.debug('skydrive nextFile failed', e);
                        callback(e || new Error(''), null);
                    }
                });
            };
            SkyApi.prototype.list = function (folder, callback) {
                var path = '/files?limit=100';
                if (this.sort_order) {
                    path += '&sort_order=' + this.sort_order;
                }
                this.nextFiles(folder.id + path, callback);
            };
            SkyApi.prototype.preview = function (id, callback) {
                var _this = this;
                async.waterfall([
                    function (c) {
                        _this.sharedReadLink(id, c);
                    }
                ], function (e, sharedLink) {
                    var url = uri.parse('https://apis.live.net/v5.0/skydrive/get_item_preview');
                    url.query = { url: sharedLink, type: 'album' };
                    callback(e, { width: 200, height: 200, source: uri.format(url) });
                });
            };
            SkyApi.prototype.fillFolders = function (folders, fill, reduce, callback) {
                var queue = async.queue(fill, 4);
                var result = [];
                queue.drain = function () {
                    callback(null, result);
                };
                _.each(folders, function (f, i) {
                    queue.push(f, (function (e, items) {
                        result.push(reduce(f, items));
                    }));
                });
            };
            SkyApi.prototype.fillAlbums = function (folders, callback) {
                var _this = this;
                this.fillFolders(folders, function (f, c) {
                    _this.photos(f.id, c);
                }, function (f, photos) {
                    return {
                        name: f.name,
                        photos: photos,
                        cover: (photos.length ? photos[0].images[0] : null)
                    };
                }, callback);
            };
            SkyApi.prototype.breadthFirstEnum = function (folders, callback) {
                var _this = this;
                var result = [];
                client.logger.info('about to start queue');
                var queue = async.queue(function (f, c) {
                    client.logger.info('queue worker process', f.name);
                    result.push(f);
                    _this.folders(f.id, c);
                }, 4);
                var itemCallback = function (e, s) {
                    _.each(s, function (sf, i) {
                        if (sf.count) {
                            queue.push(sf, queue.itemCallback);
                        }
                    });
                };
                queue.itemCallback = itemCallback;
                queue.drain = function () {
                    client.logger.info('queue drianed');
                    callback(null, result);
                };
                _.each(folders, function (f, i) {
                    queue.push(f, itemCallback);
                });
            };
            return SkyApi;
        })();
        skydrive.SkyApi = SkyApi;
    })(skydrive = client.skydrive || (client.skydrive = {}));
})(client || (client = {}));
/// <reference path="AppMasterScreen.ts"/>
/// <reference path="./ContentPickingDialog.ts"/>
/// <reference path="../../shared/models/SkyDriveApi.ts" />
/// <reference path="../user.ts" />
var client;
(function (client) {
    /** Settings screen. */
    var SkydriveScreen = (function (_super) {
        __extends(SkydriveScreen, _super);
        function SkydriveScreen(sel, history, commands, commandCallback) {
            _super.call(this, sel, history, commands, commandCallback);
            this.history = history;
            this.commands = commands;
            this.commandCallback = commandCallback;
            this.defaultTitle = resources.labels.pickerSource.sk;
            this.contentSource = 'skydrive';
        }
        SkydriveScreen.getFileExtension = function (item) {
            var startIndex = item.name.lastIndexOf('.');
            if (startIndex > 0) {
                return item.name.substr(startIndex).toLowerCase();
            }
            return null;
        };
        SkydriveScreen.prototype.navigate = function (route) {
            var _this = this;
            // Update screens state
            client.logger.info('skydrive navigate start');
            var itemId = route.params ? route.params['itemid'] : null;
            var token = client.user.accountTokens.live.token;
            var filter = itemId ? SkydriveScreen._filterMap[itemId] : null;
            if (filter) {
                itemId = null;
            }
            this.sk = new client.skydrive.SkyApi(token, filter, itemId ? 'descending' : null);
            this.lastX = 0;
            this.lastY = 0;
            this.setTitleFromRoute(route);
            if (route.data) {
                this.title = route.data.title;
                this.items = route.data.items;
                this.skyItems = this.items;
                if (this.skyItems && this.skyItems.paging && this.skyItems.paging.next && !this.skyItems.next) {
                    this.skyItems.next = function (c) {
                        _this.sk.nextFiles(_this.skyItems.paging.next.substr(1), function (e, nextPage) {
                            c(e, _this.tranformPage(nextPage));
                        });
                    };
                }
            }
            else {
                this.items = null;
            }
            this.bag = this.bag || route.bag;
            this.bag['selected'] = [];
            this.prevTileData = null;
            // Redraw screen
            this.render();
            // bind events
            this.bindOnNavigate('a.brand', function () {
                client.dialog.bindDialogHeader(_this.$el);
                _this.$el.on('click', '#skydriveNext', function (e) {
                    $(e.currentTarget).addClass('waiting');
                    // fetch more
                    _this.fetchItemsByPage('', _this.items.next, function (e, nextPage) {
                        $('#skydriveNext').parent().remove();
                        if (nextPage) {
                            _this.items.models = _this.items.models.concat(nextPage.models);
                            _this.items.next = nextPage.next;
                            _this.appendTiles(_this.getTiles(nextPage, true));
                        }
                    });
                });
            });
            // fetch the items
            if (!this.items || !this.items.models) {
                var curState = history.state || client.History.state();
                this.fetchItemsByPage(itemId, null, function (e, page) {
                    // check if user has navigated away (back or down)
                    var nowState = history.state || client.History.state();
                    if (nowState && curState && nowState.page != curState.page) {
                        client.logger.info('user navigated away');
                        return;
                    }
                    client.logger.info('done fetch skydrive items');
                    _this.items = page;
                    client.logger.info('about to render skydrive items');
                    _this.render();
                    if (!page.models || !page.models.length) {
                        _this.showEmptyContainer();
                    }
                });
            }
            return _super.prototype.navigate.call(this, route);
        };
        SkydriveScreen.prototype.fetchItemsByPage = function (nextId, next, callback) {
            var _this = this;
            async.waterfall([
                function (c) {
                    if (next) {
                        next(c);
                    }
                    else if (nextId) {
                        _this.sk.list({ id: nextId, type: 'folder', name: '', count: 0 }, c);
                    }
                    else {
                        _this.sk.skydrive(c);
                    }
                },
                function (itemPage, c) {
                    c(null, _this.tranformPage(itemPage));
                }
            ], callback);
        };
        SkydriveScreen.prototype.tranformPage = function (src) {
            var _this = this;
            if (!src)
                return null;
            var page = {
                models: this.transformItemsToPreviewItems(src.models),
                paging: src.paging,
                next: src.next ? function (c) {
                    src.next(function (e, nextPage) {
                        c(e, _this.tranformPage(nextPage));
                    });
                } : null
            };
            return page;
        };
        SkydriveScreen.prototype.getIconThumbnail = function (item) {
            var source = null;
            if (item.type == 'file') {
                var ext = item.name.substr(item.name.lastIndexOf('.') + 1).toLowerCase();
                source = client.SkydriveScreen._fileExtToIcon[ext];
                if (source) {
                    source = 'https://p.sfx.ms/icons/Large/' + source;
                }
            }
            else if (item.type == 'notebook') {
                source = 'https://p.sfx.ms/icons/Large/one.png';
            }
            return source;
        };
        SkydriveScreen.prototype.transformItemsToPreviewItems = function (items) {
            var _this = this;
            return _.map(_.filter(items, function (item, i) {
                return SkydriveScreen._allowedTypes[item.type] || SkydriveScreen._allowedExtensions[SkydriveScreen.getFileExtension(item)];
            }), function (item, i) {
                var img = { width: 140, height: 140, source: './img/folder-icon.png' };
                if (item.hasOwnProperty('images')) {
                    img = _.find(item['images'], function (image, i) {
                        return image.type == 'album';
                    });
                }
                else {
                    var iconUrl = _this.getIconThumbnail(item);
                    img.source = iconUrl || img.source;
                }
                return _.extend(item, { thumbnail: img });
            });
        };
        SkydriveScreen.prototype.fileClicked = function (tile) {
            if (!this.prevTileData) {
                this.prevTileData = tile;
            }
            else {
                if (this.prevTileData == tile) {
                    this.prevTileData = null;
                }
                else {
                    client.TileData.setSelect(this.prevTileData, false);
                    this.prevTileData = tile;
                }
            }
        };
        SkydriveScreen.prototype.render = function () {
            // Make screen is in a valid state
            if (this.getScreenState() == 3 /* DISPOSED */) {
                client.logger.warn("Render called on DISPOSED screen ");
                return this;
            }
            // Get tiles
            client.logger.info('Skydrive render start');
            var allTiles = {
                items: this.getTiles(this.items)
            };
            if (!this.$el) {
                client.logger.error(consts.errorCode.ContentPicking, 'this.$el not defined for Skydriev screen');
            }
            // Render view
            this.empty();
            this.$el.render('skydrive', _.extend({
                title: this.title,
                mode: client.StartupMode[client.startupMode],
                qrLink: 'http://stevecast.cloudapp.net'
            }, allTiles));
            _super.prototype.render.call(this);
            var tiles = allTiles.items;
            this.appendTiles(tiles);
            this.updateCommandBar();
            this.setDialogTitle("Select What");
            return this;
        };
        SkydriveScreen._filterMap = {
            'docs': client.skydrive.filters.folders,
            'photos': client.skydrive.photoFilters
        };
        SkydriveScreen._allowedTypes = {
            photo: true,
            album: true,
            folder: true
        };
        SkydriveScreen._allowedExtensions = {
            '.pptx': true
        };
        SkydriveScreen._fileExtToIcon = {
            'ppt': 'ppt.png',
            'pptx': 'ppt.png',
            'doc': 'doc.png',
            'docx': 'doc.png',
            'xls': 'xls.png',
            'xlsx': 'xls.png',
            'mdb': 'mdb.png',
            'one': 'one.png',
            'zip': 'zip.png'
        };
        return SkydriveScreen;
    })(client.ContentPickingDialog);
    client.SkydriveScreen = SkydriveScreen;
    client.screens.addRoute('/skydrive', SkydriveScreen);
    client.screens.addRoute('/skydrive/:itemid', SkydriveScreen);
})(client || (client = {}));
///<reference path='../types.ts' />
///<reference path='./Table.ts' />
var client;
(function (client) {
    var facebook;
    (function (facebook) {
        var accessToken;
        var asyncLimit = 4;
        var coverPictureSize = 120; // Previosly used 240
        var photoPictureSize = 1080;
        function init(token) {
            accessToken = token;
            //logger.info('set fb token');
        }
        facebook.init = init;
        var _albumPageOption = {
            fields: 'name, cover_photo',
            // for debugging set to smaller number
            limit: 25
        };
        var _photoPageOption = {
            // for debuggin set to smaller number
            limit: 25
        };
        var fbApi = (function () {
            function fbApi(access_token) {
                this.access_token = access_token;
            }
            fbApi.prototype.get = function (url, data, callback) {
                var apiUrl = uri.parse('https://graph.facebook.com/' + url);
                apiUrl.query = _.extend(data || {}, { access_token: this.access_token });
                $.get(uri.format(apiUrl), null, function (data) {
                    callback(null, data);
                }).fail(function (xhr, status, e) {
                    callback(new Error(status), null);
                });
            };
            /** transform facebook albums response to a Page object */
            fbApi.prototype.processAlbumResponse = function (r, callback) {
                var _this = this;
                callback(null, {
                    models: r.data,
                    next: this.hasMoreResult(r) ? function (c) {
                        _this.get('me/albums', _.extend({ after: r.paging.cursors.after }, _albumPageOption), function (e, n) {
                            if (!e) {
                                _this.processAlbumResponse(n, c);
                            }
                            else {
                                c(e, null);
                            }
                        });
                    } : null
                });
            };
            /** is the current facebook response at the end of stream? */
            fbApi.prototype.hasMoreResult = function (r) {
                return !!(r.paging && r.paging.next);
            };
            /** get user albums in paged object */
            fbApi.prototype.albums = function (callback) {
                var _this = this;
                this.get('me/albums', _albumPageOption, function (e, r) {
                    if (!e) {
                        _this.processAlbumResponse(r, callback);
                    }
                    else {
                        callback(new Error("Missing Albums"), null);
                    }
                });
            };
            /** transform facebook photos response to Page object */
            fbApi.prototype.processPhotosResponse = function (albumId, r, callback) {
                var _this = this;
                var photos = r.data;
                var after = this.hasMoreResult(r) ? r.paging.cursors.after : null;
                callback(null, {
                    models: _.map(photos, function (photo, i) {
                        return {
                            name: photo.id.toString(),
                            images: photo.images
                        };
                    }),
                    next: after ? function (c) {
                        _this.get('' + albumId + '/photos', _.extend({ after: after }, _photoPageOption), function (e, n) {
                            _this.processPhotosResponse(albumId, n, c);
                        });
                    } : null
                });
            };
            /** get photos in the album */
            fbApi.prototype.getAlbumPhotos = function (albumId, callback) {
                var _this = this;
                this.get('' + albumId + '/photos', _photoPageOption, function (e, r) {
                    if (!e) {
                        _this.processPhotosResponse(albumId, r, callback);
                    }
                    else {
                        callback(e, null);
                    }
                });
            };
            /** get the photo images in its various resolutions*/
            fbApi.prototype.getPhoto = function (photoid, callback) {
                if (!photoid) {
                    window.setTimeout(function () {
                        callback(null, {
                            name: 'default',
                            images: [
                                { width: 200, height: 200, source: 'img/display-tile.jpg' }
                            ]
                        });
                    }, 0);
                }
                else {
                    this.get('' + photoid, { fields: 'images', supress_http_code: 1 }, function (e, r) {
                        if (!e) {
                            callback(null, { name: photoid.toString(), images: r.images });
                        }
                        else {
                            callback(e, null);
                        }
                    });
                }
            };
            /** transform album json page to preview item page so we can show it in the picker */
            fbApi.prototype.transformAlbumJsonToPrevewiItem = function (albums, callback) {
                var _this = this;
                var validAlbums = _.filter(albums.models, function (album, i) {
                    return !!album.cover_photo;
                });
                async.map(validAlbums, function (album, preview) {
                    _this.getPhoto(album.cover_photo, function (e, cover) {
                        if (!e) {
                            var coverImage = _.find(cover.images, function (image, i) {
                                return image.height >= coverPictureSize && image.width >= coverPictureSize;
                            }) || _.findMax(cover.images, function (image, i) {
                                return image.height * image.width;
                            });
                            preview(null, {
                                thumbnail: coverImage,
                                id: album.id.toString(),
                                name: album.name,
                                type: 'album'
                            });
                        }
                        else {
                            preview(e, null);
                        }
                    });
                }, function (e, previews) {
                    var next = null;
                    if (albums.next) {
                        next = function (n) {
                            albums.next(function (e, nextAlbumPage) {
                                if (!e) {
                                    _this.transformAlbumJsonToPrevewiItem(nextAlbumPage, n);
                                }
                                else {
                                    n(e, null);
                                }
                            });
                        };
                    }
                    callback(e, !e ? {
                        models: previews,
                        next: next
                    } : null);
                });
            };
            /** get  albums of the user has */
            fbApi.prototype.albumItems = function (callback) {
                var _this = this;
                async.waterfall([
                    function (c) {
                        _this.albums(c);
                    },
                    function (albums, c) {
                        _this.transformAlbumJsonToPrevewiItem(albums, c);
                    }
                ], callback);
            };
            return fbApi;
        })();
        /** transform photo page to preview item page (picking the small image size) */
        function transfromPhotoPageToPreviewPage(photos) {
            return {
                models: _.map(photos.models, function (photo) {
                    var coverImage = _.find(photo.images, function (image, i) {
                        return image.height >= coverPictureSize && image.width >= coverPictureSize;
                    }) || _.first(photo.images);
                    return {
                        id: photo.name,
                        name: photo.name,
                        type: 'photo',
                        images: photo.images,
                        thumbnail: coverImage
                    };
                }),
                next: photos.next ? function (c) {
                    photos.next(function (e, photoPage) {
                        c(e, e ? null : transfromPhotoPageToPreviewPage(photoPage));
                    });
                } : null
            };
        }
        /** get photos in the user's album */
        function getAlbumPhotos(albumid, callback) {
            var fb = new fbApi(accessToken);
            fb.getAlbumPhotos(albumid, function (e, photos) {
                if (!e) {
                    callback(null, transfromPhotoPageToPreviewPage(photos));
                }
                else {
                    callback(e, null);
                }
            });
        }
        facebook.getAlbumPhotos = getAlbumPhotos;
        /** get the albums that user has on facebook */
        function getAlbums(callback) {
            var fb = new fbApi(accessToken);
            fb.albumItems(callback);
        }
        facebook.getAlbums = getAlbums;
        function fitPhotoSize(photo) {
            var dx = Math.abs(photo.width - photoPictureSize);
            var dy = Math.abs(photo.height - photoPictureSize);
            return Math.max(dx, dy);
        }
    })(facebook = client.facebook || (client.facebook = {}));
})(client || (client = {}));
/// <reference path="AppMasterScreen.ts"/>
/// <reference path='./ContentPickingDialog.ts'/>
/// <reference path="../../shared/models/FacebookApi.ts" />
var client;
(function (client) {
    /** Settings screen. */
    var AlbumsScreen = (function (_super) {
        __extends(AlbumsScreen, _super);
        function AlbumsScreen(sel, history, commands, commandCallback) {
            _super.call(this, sel, history, commands, commandCallback);
            this.history = history;
            this.commands = commands;
            this.commandCallback = commandCallback;
            this.contentSource = 'album';
            this.defaultTitle = resources.labels.pickerSource.fb;
        }
        AlbumsScreen.prototype.processItems = function (e, results) {
            client.logger.info('getAlbums called back');
            if (e || !results || !results.models || !results.models.length) {
                client.logger.error(consts.errorCode.ContentPicking, 'getAlbums error', e);
                this.render();
                this.showEmptyContainer();
            }
            else {
                client.logger.info('fetched facebook albums count ', results.models.length);
                this.items = results;
                this.render();
            }
        };
        AlbumsScreen.prototype.navigate = function (route) {
            var _this = this;
            // Update screens state
            client.logger.info('Album navigate start');
            var albumid = null;
            if (route && route.params) {
                albumid = route.params['itemid'];
            }
            else {
                client.logger.error(consts.errorCode.Routing, 'Missing route params');
            }
            var accountToken = client.user.accountTokens[0 /* fb */] || client.user.accountTokens.fb;
            var token = accountToken.token;
            client.facebook.init(token);
            this.setTitleFromRoute(route);
            if (route.data) {
                this.items = route.data.items;
                this.title = route.data.title;
            }
            else {
                this.items = null;
            }
            this.lastX = 0;
            this.lastY = 0;
            this.bag = this.bag || route.bag;
            this.bag['selected'] = [];
            if (!this.items || !this.items.models || !this.items.models.length) {
                var screen = this;
                if (!albumid) {
                    client.facebook.getAlbums(_.bind(this.processItems, this));
                }
                else {
                    client.facebook.getAlbumPhotos(albumid, _.bind(this.processItems, this));
                }
            }
            // Redraw screen
            this.render();
            this.bindOnNavigate('a.brand', function () {
                // bind events
                client.dialog.bindDialogHeader(_this.$el);
                _this.$el.on('click', '#albumNext', function (e) {
                    $(e.currentTarget).addClass('waiting');
                    // fetch more
                    _this.items.next(function (e, nextPage) {
                        $('#albumNext').parent().remove();
                        if (nextPage) {
                            _this.items.models = _this.items.models.concat(nextPage.models);
                            _this.items.next = nextPage.next;
                            _this.appendTiles(_this.getTiles(nextPage, true));
                        }
                    });
                });
            });
            return _super.prototype.navigate.call(this, route);
        };
        AlbumsScreen.prototype.render = function () {
            // Make screen is in a valid state
            if (this.getScreenState() == 3 /* DISPOSED */) {
                client.logger.warn("Render called on DISPOSED screen ");
                return this;
            }
            // Get tiles
            client.logger.info('Albums render start');
            var allTiles = {
                albums: this.getTiles(this.items)
            };
            if (!this.$el) {
                client.logger.error(consts.errorCode.ContentPicking, 'this.$el not defined for AlbumsScreen');
            }
            // Render view
            this.empty();
            this.$el.render('albums', _.extend({
                title: this.title,
                mode: client.StartupMode[client.startupMode],
                qrLink: 'http://stevecast.cloudapp.net'
            }, allTiles));
            _super.prototype.render.call(this);
            this.appendTiles(allTiles.albums);
            this.updateCommandBar();
            this.setDialogTitle("Select What");
            return this;
        };
        return AlbumsScreen;
    })(client.ContentPickingDialog);
    client.AlbumsScreen = AlbumsScreen;
    client.screens.addRoute('/albums', AlbumsScreen);
    client.screens.addRoute('/albums/:itemId', AlbumsScreen);
})(client || (client = {}));
///<reference path='../types.ts' />
///<reference path='Table.ts' />
var client;
(function (client) {
    var dropbox;
    (function (dropbox) {
        ;
        var sizeMap = {
            xs: { width: 32, height: 32 },
            s: { width: 64, height: 64 },
            m: { width: 128, height: 128 },
            l: { width: 640, height: 480 },
            xl: { width: 1024, height: 768 }
        };
        var DropboxApi = (function () {
            function DropboxApi(access_token) {
                this.access_token = access_token;
            }
            DropboxApi.prototype._send = function (verb, root, url, callback) {
                var urlString = uri.resolve(root, url);
                var apiUrl = uri.parse(urlString);
                apiUrl.query = _.extend(apiUrl.query || {}, { access_token: this.access_token });
                client.logger.info('about to call', urlString);
                $.ajax({
                    type: verb,
                    url: uri.format(apiUrl),
                    dataType: 'text',
                    success: function (data, s, xhr) {
                        var contentType = xhr.getResponseHeader('Content-Type');
                        client.logger.info('request returned content type', contentType);
                        if (contentType == 'text/javascript' || contentType == 'application/json') {
                            var jsonData = JSON.parse(data);
                            callback(null, jsonData);
                        }
                        else if (contentType.indexOf('image') >= 0) {
                            client.logger.info('dropbox returned the raw image');
                            callback(null, data);
                        }
                    },
                    error: function (xhr) {
                        callback(new Error('failed with status code ' + xhr.status), null);
                    }
                });
            };
            DropboxApi.prototype._get = function (root, url, callback) {
                this._send('GET', root, url, callback);
            };
            DropboxApi.prototype.post = function (url, callback) {
                this._send('POST', 'https://api.dropbox.com/1/', url, callback);
            };
            DropboxApi.prototype.get = function (url, callback) {
                this._get('https://api.dropbox.com/1/', url, callback);
            };
            DropboxApi.prototype.getContent = function (url, callback) {
                this._get('https://api-content.dropbox.com/1/', url, callback);
            };
            DropboxApi.prototype.getJson = function (url, transform, callback) {
                this.get(url, function (e, data) {
                    if (!e) {
                        callback(null, transform(data));
                    }
                });
            };
            DropboxApi.prototype.share = function (item, callback) {
                client.logger.info('fetch share link for ', item.id);
                this.post('shares/' + item.id, function (e, link) {
                    if (!e && link && link.url) {
                        callback(null, link.url);
                    }
                    else {
                        callback(e || new Error('failed to share ' + item.id), null);
                    }
                });
            };
            DropboxApi.prototype.media = function (item, callback) {
                this.post('media/' + item.id, function (e, link) {
                    if (!e && link && link.url) {
                        callback(null, link.url);
                    }
                    else {
                        callback(e || new Error('failed to get media for ' + item.id), null);
                    }
                });
            };
            DropboxApi.prototype.preview = function (item, size) {
                if (size === void 0) { size = 'm'; }
                if (item.thumb_exists) {
                    var url = uri.parse('https://api-content.dropbox.com/1/thumbnails/' + item.root + item.path);
                    url.query = { size: size, access_token: this.access_token };
                    return _.extend({ source: uri.format(url) }, sizeMap[size]);
                }
                else {
                    return { width: 140, height: 140, source: './img/folder-icon.png' };
                }
            };
            DropboxApi.prototype.extractShareImage = function (share, callback) {
                client.logger.info('about to load shared link', share);
                $.ajax({
                    url: share,
                    dataType: 'text',
                    crossDomain: true,
                    type: 'GET',
                    success: function (data, s, xhr) {
                        client.logger.info('got preview html for', share, ' data.length = ', data ? data.length : 'null');
                        var thtml;
                        var prev = window['on_script_loaded'];
                        var matchExp = /ShmodelPreview.init_photo\("([^"]+)"/;
                        var matches = data.match(matchExp);
                        if (matches && matches.length > 1) {
                            callback(null, matches[1]);
                            return;
                        }
                        if (!prev) {
                            window['on_script_loaded'] = function () {
                            };
                        }
                        try {
                            thtml = $.parseHTML(data);
                            client.logger.info('thtml parsed length =', thtml ? thtml.length : 'null');
                        }
                        catch (exp) {
                            // do nothing
                            client.logger.debug('parseHTML failed', exp);
                        }
                        var $dbhtml = $(thtml);
                        var $preview = $dbhtml.find('img#preview-img');
                        var preview = $preview.attr('src');
                        if (preview.indexOf('icons/icon_spacer') >= 0) {
                            preview = null;
                        }
                        if (!preview) {
                            preview = $(thtml).find('.preview-box img').attr('src');
                        }
                        if (preview && preview.indexOf('icons/icon_spacer') >= 0) {
                            preview = null;
                        }
                        if (!preview) {
                            client.logger.debug('thtml', data);
                        }
                        callback(null, preview);
                    },
                    error: function (xhr) {
                        client.logger.debug('ajax get failed for', share, 'status code', xhr.status);
                        callback(new Error('failed with status code ' + xhr.status), null);
                    }
                });
            };
            /**
                get the viewer url for dropbox items and extract the thumbnail image
               */
            DropboxApi.prototype.getThumbnails = function (items, callback) {
                var _this = this;
                async.map(items || [], function (item, photoCallback) {
                    async.waterfall([
                        function (c) {
                            _this.share(item, c);
                        },
                        function (share, c) {
                            _this.extractShareImage(share, function (e, thumbnail) {
                                if (!e && thumbnail) {
                                    c(null, _.extend(item, { images: [{ source: thumbnail, width: 1024, height: 768 }] }));
                                }
                                else {
                                    c(e || new Error('failed to extract thumbnail for ' + item.id), null);
                                }
                            });
                        }
                    ], photoCallback);
                }, function (e, photos) {
                    if (!e && photos) {
                        callback(null, photos);
                    }
                    else {
                        callback(e || new Error('fail to extract thumbnails'), null);
                    }
                });
            };
            DropboxApi.prototype.list = function (path, callback) {
                var _this = this;
                this.get(uri.resolve('metadata/', path), function (e, r) {
                    if (!e && r) {
                        async.map(r.contents || [], function (d, cb) {
                            var pi = {
                                id: d.root + d.path,
                                name: _.last(d.path.split('/')),
                                type: d.is_dir ? 'folder' : (d.thumb_exists ? 'photo' : 'file')
                            };
                            setTimeout(function () {
                                cb(null, _.extend(pi, { thumbnail: _this.preview(d, 'l') }));
                            }, 0);
                        }, function (e, items) {
                            if (!e) {
                                callback(e, { models: _.filter(items, function (pi, i) {
                                    return pi.type == 'folder' || pi.type == 'photo';
                                }) });
                            }
                            else {
                                callback(e, null);
                            }
                        });
                    }
                    else {
                        callback(e, null);
                    }
                });
            };
            return DropboxApi;
        })();
        dropbox.DropboxApi = DropboxApi;
    })(dropbox = client.dropbox || (client.dropbox = {}));
})(client || (client = {}));
/// <reference path="AppMasterScreen.ts"/>
/// <reference path='./ContentPickingDialog.ts'/>
/// <reference path="../../shared/models/DropBoxApi.ts" />
var client;
(function (client) {
    /** Settings screen. */
    var DropboxScreen = (function (_super) {
        __extends(DropboxScreen, _super);
        function DropboxScreen(sel, history, commands, commandCallback) {
            _super.call(this, sel, history, commands, commandCallback);
            this.history = history;
            this.commands = commands;
            this.commandCallback = commandCallback;
            this.contentSource = 'dropbox';
            this.defaultTitle = resources.labels.pickerSource.db;
            this.dropboxApi = new client.dropbox.DropboxApi(client.user.accountTokens.dropbox.token);
        }
        DropboxScreen.prototype.processItems = function (e, page) {
            client.logger.info('dropbox list called back');
            this.items = page;
            if (e || !page) {
                // An error occured, show an empty folder
                client.logger.error(consts.errorCode.ContentPicking, 'dropbox list error', e);
                this.render();
                this.showEmptyContainer();
            }
            else if (!page.models || !page.models.length) {
                // Show an empty folder
                this.render();
                this.showEmptyContainer();
            }
            else {
                this.render();
            }
        };
        DropboxScreen.prototype.navigate = function (route) {
            var _this = this;
            // Update screens state
            client.logger.info('Dropbox navigate start');
            var folderid = null;
            if (route && route.params) {
                folderid = route.params['itemid'];
            }
            else {
                client.logger.error(consts.errorCode.ContentPicking, "Missing route params");
            }
            this.setTitleFromRoute(route);
            if (route.data) {
                this.items = route.data.items;
                this.title = route.data.title;
            }
            else {
                this.items = null;
            }
            this.lastX = 0;
            this.lastY = 0;
            this.bag = this.bag || route.bag;
            this.bag['selected'] = [];
            if (!this.items || !this.items.models || !this.items.models.length) {
                if (!folderid) {
                    this.dropboxApi.list('dropbox/', _.bind(this.processItems, this));
                }
                else {
                    this.dropboxApi.list(folderid, _.bind(this.processItems, this));
                }
            }
            // Redraw screen
            this.render();
            this.bindOnNavigate('a.brand', function () {
                // bind events
                client.dialog.bindDialogHeader(_this.$el);
            });
            return _super.prototype.navigate.call(this, route);
        };
        DropboxScreen.prototype.render = function () {
            // Make screen is in a valid state
            if (this.getScreenState() == 3 /* DISPOSED */) {
                client.logger.warn("Render called on DISPOSED screen ");
                return this;
            }
            // Get tiles
            client.logger.info('dropbox render start');
            var allTiles = {
                albums: this.getTiles(this.items)
            };
            if (!this.$el) {
                client.logger.error(consts.errorCode.ContentPicking, 'this.$el not defined for AlbusmScreen');
            }
            // Render view
            this.empty();
            this.$el.render('dropbox', _.extend({
                title: this.title,
                mode: client.StartupMode[client.startupMode]
            }, allTiles));
            _super.prototype.render.call(this);
            this.appendTiles(allTiles.albums);
            this.updateCommandBar();
            this.setDialogTitle("Select What");
            return this;
        };
        DropboxScreen.prototype.mapToScreenQueueItems = function (items, callback) {
            if (_.all(items, function (item) {
                return item.type == 'photo' && item.url;
            })) {
                _.defer(function () {
                    callback(null, items);
                });
            }
            else {
                this.dropboxApi.getThumbnails(items, function (e, thumbnails) {
                    if (!e) {
                        callback(null, _.map(thumbnails, function (thumbnail) {
                            return { id: thumbnail.id, type: 'photo', url: thumbnail.images[0].source };
                        }));
                    }
                    else {
                        callback(e, null);
                    }
                });
            }
        };
        return DropboxScreen;
    })(client.ContentPickingDialog);
    client.DropboxScreen = DropboxScreen;
    client.screens.addRoute('/dropbox', DropboxScreen);
    client.screens.addRoute('/dropbox/:itemid', DropboxScreen);
})(client || (client = {}));
/// <reference path="AppMasterScreen.ts"/>
/// <reference path='./ContentPickingDialog.ts'/>
var client;
(function (client) {
    /** Instagram screen. */
    var InstagramScreen = (function (_super) {
        __extends(InstagramScreen, _super);
        function InstagramScreen(sel, history, commands, commandCallback) {
            _super.call(this, sel, history, commands, commandCallback);
            this.history = history;
            this.commands = commands;
            this.commandCallback = commandCallback;
            this.searchTerm = '';
            this.defaultTitle = resources.labels.pickerSource.ig;
            this.contentSource = 'instagram';
        }
        InstagramScreen.prototype.transformInstagramItems = function (images) {
            return _.map(images, function (item, i) {
                return {
                    id: item.id,
                    name: item.id,
                    type: 'photo',
                    thumbnail: { source: item.images.low_resolution.url, height: item.images.low_resolution.height, width: item.images.low_resolution.width },
                    images: [
                        {
                            source: item.images.standard_resolution.url,
                            height: item.images.standard_resolution.height,
                            width: item.images.standard_resolution.width
                        }
                    ]
                };
            });
        };
        InstagramScreen.prototype.processInstagramResponse = function (r) {
            var _this = this;
            this.$el.find('#instagram-items .message').text('').removeClass('loading');
            if (r.meta.code == 200) {
                var images = _.filter(r.data, function (item, i) {
                    return item.type == 'image';
                });
                var batch = this.transformInstagramItems(images);
                if (this.items) {
                    this.items.models = this.items.models.concat(batch);
                }
                else {
                    this.items = {
                        models: this.transformInstagramItems(images)
                    };
                }
                this.items.next = r.pagination && r.pagination.next_url ? function (c) {
                    $.ajax({
                        url: r.pagination.next_url,
                        crossDomain: true,
                        dataType: 'json',
                        success: _.bind(_this.processInstagramResponse, _this),
                        error: function () {
                            client.logger.debug('ajax failed', arguments);
                        }
                    });
                } : null;
                if (this.items.models.length == batch.length) {
                    this.render();
                }
                else {
                    $('#instagramNext').parent().remove();
                    this.appendTiles(this.getTiles({ models: batch, next: this.items.next }, true));
                }
            }
            else {
                client.logger.debug('failed to fetch instagram feed');
            }
            if (!this.items.models || !this.items.models.length) {
                this.showEmptyContainer();
            }
        };
        InstagramScreen.prototype.refreshSearch = function (q) {
            var _this = this;
            this.items = null;
            this.searchTerm = q;
            this.$el.find('#instagram-items .items').empty().css('height', 'auto');
            this.$el.find('#instagram-items .message').addClass('loading').text(resources.messages.searchingInstagram);
            var searchUrl = uri.parse('https://api.instagram.com/v1/tags/search');
            searchUrl.query = { q: q, access_token: client.user.accountTokens.instagram.token };
            $.ajax({ url: uri.format(searchUrl), crossDomain: true, dataType: 'json', success: function (r) {
                if (r.meta && r.meta.code == 200) {
                    if (r.data && r.data.length) {
                        var fetchUrl = uri.parse('https://api.instagram.com/v1/tags/' + r.data[0].name + '/media/recent');
                        fetchUrl.query = { access_token: client.user.accountTokens.instagram.token };
                        $.ajax({
                            url: uri.format(fetchUrl),
                            crossDomain: true,
                            dataType: 'json',
                            success: _.bind(_this.processInstagramResponse, _this)
                        });
                    }
                    else {
                        client.logger.debug('tag search found no result');
                    }
                }
                else {
                    client.logger.debug('tag search failed with', r.meta);
                }
            } });
        };
        InstagramScreen.prototype.getFeed = function (liked) {
            this.items = null;
            this.$el.find('#instagram-items .items').empty().css('height', 'auto');
            this.$el.find('#instagram-items .message').addClass('loading').text(resources.messages.loadingInstagram);
            var feedUrl = uri.parse('https://api.instagram.com/v1/users/self/media/' + (liked ? 'liked' : 'recent'));
            feedUrl.query = { access_token: client.user.accountTokens.instagram.token };
            $.ajax({
                url: uri.format(feedUrl),
                crossDomain: true,
                dataType: 'json',
                success: _.bind(this.processInstagramResponse, this),
                error: function () {
                    client.logger.debug('ajax failed', arguments);
                }
            });
        };
        InstagramScreen.prototype.navigate = function (route) {
            var _this = this;
            // Update screens state
            client.logger.info('instagram navigate start');
            this.setTitleFromRoute(route);
            if (route.data) {
                this.items = route.data.items;
                this.title = route.data.title;
            }
            else {
                this.items = null;
            }
            this.lastX = 0;
            this.lastY = 0;
            this.bag = this.bag || route.bag;
            this.bag['selected'] = [];
            var path = _.last(route.path.split('/'));
            switch (path) {
                case 'mine':
                    this.getFeed();
                    break;
                case 'likes':
                    this.getFeed(true);
                    break;
                default:
                    this.items = {
                        models: [
                            {
                                id: 'mine',
                                name: resources.labels.tiles.mine,
                                type: 'folder',
                                thumbnail: { width: 140, height: 140, source: './img/folder-icon.png' }
                            },
                            {
                                id: 'likes',
                                name: resources.labels.tiles.likes,
                                type: 'folder',
                                thumbnail: { width: 140, height: 140, source: './img/folder-icon.png' }
                            }
                        ]
                    };
                    break;
            }
            // Redraw screen
            this.render();
            this.bindOnNavigate('a.brand', function () {
                // bind events
                _this.$el.on('blur', '#search', function (e) {
                    var $input = $(e.currentTarget);
                    var q = client.Util.cleanString($input.val());
                    if (q) {
                        _this.refreshSearch(q);
                    }
                });
                _this.$el.on('submit', 'form', function (e) {
                    e.preventDefault();
                    var q = client.Util.cleanString($(e.currentTarget).find('#search').val());
                    _this.refreshSearch(q);
                });
                client.dialog.bindDialogHeader(_this.$el);
                _this.$el.on('click', '#instagramNext', function (e) {
                    $(e.currentTarget).addClass('waiting');
                    // fetch more
                    if (_this.items && _this.items.next) {
                        _this.items.next(function (e, page) {
                        });
                    }
                });
                var screen = _this;
                _this.$el.on('click', '#mine, #likes', function (e) {
                    client.logger.info('flickr link clicked');
                    e.preventDefault();
                    var $link = $(e.currentTarget);
                    if (screen.history) {
                        var href = $link.attr('href') || '#/instagram/' + $link.attr('id');
                        client.logger.info('user clicked on ', href);
                        var itemid = href.substr('#/instagram/'.length);
                        client.logger.info('user picked item id', itemid);
                        var item = _.find(screen.items.models, function (a, i) {
                            return a.id == itemid;
                        });
                        client.logger.info('user picked item ', item);
                        if (item && item.type == 'text' && (item.id == 'mine' || item.id == 'likes')) {
                            client.app.setData({ title: _this.title, items: _this.items });
                            var childLoc = { path: href.substr(1), params: {}, page: _this.history.page + 1 };
                            _this.title = item.name;
                            history.pushState(childLoc, 'instagram');
                            _this.navigate(childLoc);
                        }
                    }
                });
            });
            return _super.prototype.navigate.call(this, route);
        };
        InstagramScreen.prototype.render = function () {
            // Make screen is in a valid state
            if (this.getScreenState() == 3 /* DISPOSED */) {
                client.logger.warn("Render called on DISPOSED screen ");
                return this;
            }
            // Get tiles
            this.lastX = 0;
            this.lastY = 0;
            client.logger.info('flickr render start');
            var allTiles = {
                albums: this.getTiles(this.items)
            };
            if (!this.$el) {
                client.logger.error(consts.errorCode.ContentPicking, 'this.$el not defined for AlbusmScreen');
            }
            // Render view
            this.empty();
            this.$el.render('instagram', _.extend({
                searchTerm: this.searchTerm,
                title: this.title,
                mode: client.StartupMode[client.startupMode],
                qrLink: 'http://stevecast.cloudapp.net'
            }, allTiles));
            _super.prototype.render.call(this);
            this.appendTiles(allTiles.albums);
            this.updateCommandBar();
            this.setDialogTitle("Select What");
            return this;
        };
        return InstagramScreen;
    })(client.ContentPickingDialog);
    client.InstagramScreen = InstagramScreen;
    client.screens.addRoute('/instagram', InstagramScreen);
    client.screens.addRoute('/instagram/likes', InstagramScreen);
    client.screens.addRoute('/instagram/mine', InstagramScreen);
})(client || (client = {}));
/// <reference path="types.ts"/>
/// <reference path="consts.ts"/>
var client;
(function (client) {
    /** Base class for a cast viewer (i.e. xbox, display, chromecast) */
    var CastViewer = (function () {
        /** Base class constructor */
        function CastViewer(name, id) {
            this.displayName = name;
            this.id = id;
        }
        /** Attempt to display to this castViewer */
        CastViewer.prototype.turnOn = function (callback) {
            // Subclass should override
            client.logger.error(consts.errorCode.CastViewer, 'calling default turnOn');
        };
        /** Turn viewer off */
        CastViewer.prototype.turnOff = function (callback) {
            // Subclass should override
            client.logger.error(consts.errorCode.CastViewer, 'calling default turnOff');
        };
        return CastViewer;
    })();
    client.CastViewer = CastViewer;
})(client || (client = {}));
/// <reference path="types.ts"/>
/// <reference path="consts.ts"/>
/// <reference path="../proximity/types.d.ts"/>
/// <reference path="../../types/cordova/cordova-chromecast.d.ts"/>
var client;
(function (client) {
    var cast;
    (function (cast) {
        cast.browserCastClient = null;
        var BrowserCast = (function () {
            function BrowserCast() {
                this.connection = new client.Connection();
            }
            BrowserCast.prototype.dispose = function () {
                if (this.viewer) {
                    this.viewer.dispose();
                    this.viewer = null;
                }
                if (this.connection) {
                    this.connection.dispose();
                    this.connection = null;
                }
            };
            /** Start a display with the given route */
            // Takes a pairing id
            // Returns true if succesfully paired or error code
            BrowserCast.prototype.selectRoute = function (pinCode, callback) {
                var _this = this;
                // Create viewer
                this.connection.pairViewerSession({ pinCode: pinCode }, function (err, viewer) {
                    if (!err && viewer) {
                        _this.viewer = viewer;
                        if (callback)
                            callback(null, true);
                    }
                    else {
                        if (callback)
                            callback(BrowserCast.INVALID_PIN, false);
                    }
                });
            };
            BrowserCast.INVALID_PIN = new Error("Invalid Pin");
            return BrowserCast;
        })();
        cast.BrowserCast = BrowserCast;
    })(cast = client.cast || (client.cast = {}));
})(client || (client = {}));
/// <reference path="../types.ts"/>
/// <reference path="../../../types/jquery/jquery.d.ts"/>
/// <reference path="PopUp.ts"/>
/// <reference path='../resources.ts'/>
/// <reference path='../../shared/browserCast.ts'/>
var client;
(function (client) {
    /** Popup for showing castViewer instructions */
    var CastViewerHowToPopUp = (function (_super) {
        __extends(CastViewerHowToPopUp, _super);
        /** Callback returns pin for a valid browser or null */
        function CastViewerHowToPopUp($el, howToClass, resources, callback) {
            var _this = this;
            // Call superclass to render Popup
            _super.call(this, $el, howToClass, { resources: resources });
            this.callback = callback;
            // Force vertical screen when shown
            this.lockVertical = true;
            // Close window when button is clicked
            this.$el.on('click', '.howToButton', function () {
                _this.destroy();
            });
            if (this.callback) {
                this.onClose.add(function () {
                    _this.callback(new Error('user abandoned'), null);
                });
            }
            // Show display FAQ when link clicked
            this.$el.on('click', '.howToFooter', function () {
                client.logger.collectNamedEvent(consts.flurryEvent.castViewerFAQ);
                window.open(consts.externalLinks.faqdisplay, '_blank', 'location=no');
            });
            this.show(true);
        }
        return CastViewerHowToPopUp;
    })(client.PopUp);
    client.CastViewerHowToPopUp = CastViewerHowToPopUp;
})(client || (client = {}));
/// <reference path="types.ts"/>
/// <reference path="consts.ts"/>
/// <reference path="castViewer.ts"/>
/// <reference path="./controls/CastViewerHowToPopUp.ts"/>
/// <reference path="../../types/cordova/appletv.d.ts"/>
var client;
(function (client) {
    var defaultAirPlayId = "AirPlay";
    var airplayCastViewer = null;
    var ximEventsBound = false;
    /** A AirPlay based castViewer */
    var CastViewerAirPlay = (function (_super) {
        __extends(CastViewerAirPlay, _super);
        function CastViewerAirPlay(name, id, connected) {
            if (name === void 0) { name = "AirPlay"; }
            if (id === void 0) { id = defaultAirPlayId; }
            if (connected === void 0) { connected = false; }
            _super.call(this, name, id);
            this.connected = connected;
            this.connection = new client.Connection();
            this.icon = "glyphiconXIM glyphiconXIM-airPlay";
            this.iconActive = "glyphiconXIM glyphiconXIM-airPlaya";
        }
        /** Start scanning for air play devices */
        CastViewerAirPlay.startScan = function () {
            if (window.appletv) {
                if (!ximEventsBound) {
                    ximEventsBound = true;
                    // always show airplay icon since we can not 
                    // scan for airplay mirroring device on ios
                    // the appletv plugin is only on ios
                    airplayCastViewer = new CastViewerAirPlay();
                    client.CastViewManager.addCastViewer(airplayCastViewer);
                    window.appletv.registerListener(function (device) {
                        if (_.isString(device)) {
                            device = JSON.parse(device);
                        }
                        if (client.user.GetSetting(14 /* DISABLE_AIRPLAY */)) {
                            return;
                        }
                        if (device.connected) {
                            // airplay mirroring is turned on and off by user in the system tray
                            // once it is connected it is already mirroring
                            airplayCastViewer.connected = true;
                            airplayCastViewer.connectViewer(function (e, b) {
                                if (!e && b) {
                                    // set active viewer to apple tv
                                    client.CastViewManager.toggle(airplayCastViewer, function () {
                                    });
                                }
                            });
                        }
                        else {
                            // user disconnect airplay mirroring
                            if (airplayCastViewer.connected) {
                                airplayCastViewer.connected = false;
                                client.CastViewManager.toggle(airplayCastViewer, function () {
                                });
                            }
                        }
                    }, function (e) {
                        client.logger.warn("appletv register listener failed", e);
                    });
                }
                else {
                    if (airplayCastViewer) {
                        client.CastViewManager.addCastViewer(airplayCastViewer);
                    }
                }
            }
        };
        /** Attempt to display to this castViewer */
        CastViewerAirPlay.prototype.turnOn = function (callback) {
            client.logger.collectNamedEvent(consts.flurryEvent.castViewerAdd, { 'viewType': 'AirPlay' });
            if (this.connected) {
                // already connected
                if (this.viewer) {
                    _.defer(function () {
                        callback(null, true);
                    });
                }
                else {
                    this.connectViewer(callback);
                }
            }
            else {
                // airplay mirroring is controlled by user outside the app, if we
                // not already connected, means user has not turned it on, show the 'how to' instruction
                var howToPopUp = new client.CastViewerHowToPopUp(null, "howToAirPlay", resources.forms.airplayHowTo, function (e, s) {
                    callback(null, false);
                });
            }
        };
        /** Attempt to turn off this castViewer */
        CastViewerAirPlay.prototype.turnOff = function (callback) {
            // airplay mirroring is controlled outside the UI, only
            // back out to default page in turn off
            if (this.connected) {
                this.disconnectViewer();
                callback(null, true);
            }
            else {
                // otherwise just return true
                _.defer(function () {
                    callback(null, true);
                });
            }
        };
        CastViewerAirPlay.prototype.connectViewer = function (cb) {
            var _this = this;
            this.disconnectViewer();
            this.connection.createViewerSession(false, function (err, session) {
                if (!err && session) {
                    _this.viewer = session;
                    client.user.turnOnViewer(_this.viewer);
                    // Load viewer on XBOX
                    var url = _this.connection.root + 'appleReceiver' + client.app.toQueryString({ accessToken: _this.viewer.accessToken.token, sessionId: _this.viewer.sessionId });
                    client.logger.info('viewer url at', url);
                    window.appletv.loadUrl(url, function () {
                        client.logger.info('appletv load url ok');
                        cb(null, true);
                    }, function (e) {
                        client.logger.warn('appletv load url failed', e);
                        cb(new Error('failed'), false);
                    });
                }
                else {
                    cb(err, false);
                }
            });
        };
        CastViewerAirPlay.prototype.disconnectViewer = function () {
            if (this.viewer) {
                client.logger.info("appletv discounnected");
                client.user.turnOffViewer();
                this.viewer.dispose();
                this.viewer = null;
            }
        };
        return CastViewerAirPlay;
    })(client.CastViewer);
    client.CastViewerAirPlay = CastViewerAirPlay;
})(client || (client = {}));
/// <reference path="../types.ts"/>
/// <reference path="../../../types/jquery/jquery.d.ts"/>
/// <reference path="PopUp.ts"/>
/// <reference path='../resources.ts'/>
/// <reference path='../../shared/browserCast.ts'/>
var client;
(function (client) {
    /** Popup for adding a castViewBrowser */
    var AddBrowserPopUp = (function (_super) {
        __extends(AddBrowserPopUp, _super);
        /** Callback returns pin for a valid browser or null */
        function AddBrowserPopUp(resources, callback) {
            var _this = this;
            // Call superclass to render Popup
            _super.call(this, null, "addBrowserPopUp", { resources: resources, url: AddBrowserPopUp.getJoinURL() });
            this.callback = callback;
            this.pin = null;
            // Force vertical screen when shown
            this.lockVertical = true;
            this.$el.on('click', '.addBrowserButton', function () {
                var code = _this.$el.find('input[name=code]').val();
                _this.disableUserInput(true);
                _this.validatePin(code);
            });
            // Show display FAQ when link clicked
            this.$el.on('click', '.addBrowserFooter', function () {
                client.logger.collectNamedEvent(consts.flurryEvent.castViewerFAQ);
                window.open(consts.externalLinks.faqdisplay, '_blank', 'location=no');
            });
            // Clear error messages when new text is typed
            this.$el.on('keyup input', '.dialogInputText', function () {
                _this.$el.find(".dialogError").text("");
            });
            // If user hits back button notify creator that action was abandoned, so calling dialog can close
            if (this.callback) {
                this.onClose.add(function () {
                    _this.callback(new Error("Abandoned"), null);
                });
            }
            // Hide placeholder text on focus
            client.Util.hidePlaceholderOnFocus(this.$el.find('.ximformcontrol'));
            this.show(true);
        }
        /** Return pairing URL for user's region */
        AddBrowserPopUp.getJoinURL = function () {
            switch (client.user.getDataCenter()) {
                case 'europe':
                    return resources.links.pairEurope;
                case 'asia':
                    return resources.links.pairAsia;
                default:
                    return resources.links.pairUs;
            }
        };
        /** Attempt to connect to browser */
        AddBrowserPopUp.prototype.validatePin = function (code) {
            this.pin = code;
            client.cast.browserCastClient = new client.cast.BrowserCast();
            client.cast.browserCastClient.selectRoute(this.pin, _.bind(this.verifyCallback, this));
        };
        AddBrowserPopUp.prototype.disableUserInput = function (value) {
            if (value) {
                this.$el.find('.addBrowerIntro').addClass("loginDisabled");
                this.$el.find('.addBrowerTitle').addClass("loginDisabled");
                this.$el.find('.addBrowerInputBox').addClass("loginDisabled");
                this.$el.find('.addBrowserWait').show();
            }
            else {
                this.$el.find('.addBrowerIntro').removeClass("loginDisabled");
                this.$el.find('.addBrowerTitle').removeClass("loginDisabled");
                this.$el.find('.addBrowerInputBox').removeClass("loginDisabled");
                this.$el.find('.addBrowserWait').hide();
            }
        };
        AddBrowserPopUp.prototype.verifyCallback = function (err, id) {
            if (!err) {
                this.$el.find(".dialogError").text("");
                if (this.callback) {
                    // Pass back success message with pin
                    this.callback(null, this.pin);
                }
                // Close the dialog
                this.destroy();
            }
            else if (err == client.cast.BrowserCast.INVALID_PIN) {
                this.$el.find(".dialogError").text(resources.login.verify.error);
            }
            else {
                this.$el.find(".dialogError").text("Failed to connect to server.");
            }
            // Restore disabled input
            this.disableUserInput(false);
        };
        return AddBrowserPopUp;
    })(client.PopUp);
    client.AddBrowserPopUp = AddBrowserPopUp;
})(client || (client = {}));
/// <reference path="types.ts"/>
/// <reference path="consts.ts"/>
/// <reference path="castViewer.ts"/>
/// <reference path="castViewManager.ts"/>
/// <reference path="./controls/AddBrowserPopUp.ts"/>
/// <reference path="../shared/browserCast.ts"/>
/// <reference path="user.ts"/>
var client;
(function (client) {
    /** A browser based castViewer */
    var CastViewerBrowser = (function (_super) {
        __extends(CastViewerBrowser, _super);
        function CastViewerBrowser(name, id) {
            if (name === void 0) { name = "Browser"; }
            if (id === void 0) { id = "Browser"; }
            _super.call(this, name, id);
            this.icon = "glyphiconXIM glyphiconXIM-webDisplay";
            this.iconActive = "glyphiconXIM glyphiconXIM-webDisplaya";
        }
        /** Attempt to display to this castViewer */
        CastViewerBrowser.prototype.turnOn = function (callback) {
            var _this = this;
            client.logger.collectNamedEvent(consts.flurryEvent.castViewerAdd, { 'viewType': 'Browser' });
            // Collect PIN from user
            var addBrowserPopUp = new client.AddBrowserPopUp(resources.forms.addBrowser, function (err, id) {
                if (!err && client.cast.browserCastClient) {
                    _this.id = id;
                    client.user.turnOnViewer(client.cast.browserCastClient.viewer);
                    callback(null, true);
                }
                else {
                    callback(err, false);
                }
            });
        };
        /** Attempt to turn off this castViewer */
        CastViewerBrowser.prototype.turnOff = function (callback) {
            if (client.cast.browserCastClient) {
                client.user.turnOffViewer();
                client.cast.browserCastClient.dispose();
                client.cast.browserCastClient = null;
            }
            callback(null, true);
        };
        /** Start scanning for browser devices */
        CastViewerBrowser.startScan = function () {
            // For now, limit to only one castViewBrowser at a time
            // When we want to enable multiple browsers, remove the .browser code
            if (!CastViewerBrowser.browser) {
                CastViewerBrowser.browser = new CastViewerBrowser();
            }
            // There is always one browser device available
            client.CastViewManager.addCastViewer(CastViewerBrowser.browser);
        };
        //=================================================
        // Statics
        //=================================================
        CastViewerBrowser.browser = null;
        return CastViewerBrowser;
    })(client.CastViewer);
    client.CastViewerBrowser = CastViewerBrowser;
})(client || (client = {}));
/// <reference path="types.ts"/>
/// <reference path="consts.ts"/>
/// <reference path="castViewer.ts"/>
/// <reference path="castViewManager.ts"/>
/// <reference path="./controls/AddBrowserPopUp.ts"/>
/// <reference path="../shared/browserCast.ts"/>
/// <reference path="user.ts"/>
var client;
(function (client) {
    /** A browser based castViewer */
    var CastViewerFireTV = (function (_super) {
        __extends(CastViewerFireTV, _super);
        function CastViewerFireTV(name, id) {
            if (name === void 0) { name = "Fire TV"; }
            if (id === void 0) { id = "FireTV"; }
            _super.call(this, name, id);
            this.icon = "glyphiconXIM glyphiconXIM-amazon";
            this.iconActive = "glyphiconXIM glyphiconXIM-amazonActive";
        }
        /** Attempt to display to this castViewer */
        CastViewerFireTV.prototype.turnOn = function (callback) {
            var _this = this;
            client.logger.collectNamedEvent(consts.flurryEvent.castViewerAdd, { 'viewType': 'FireTV' });
            // Collect PIN from user
            var addBrowserPopUp = new client.AddBrowserPopUp(resources.forms.addFireTV, function (err, id) {
                if (!err && client.cast.browserCastClient) {
                    _this.id = id;
                    client.user.turnOnViewer(client.cast.browserCastClient.viewer);
                    callback(null, true);
                }
                else {
                    callback(err, false);
                }
            });
        };
        /** Attempt to turn off this castViewer */
        CastViewerFireTV.prototype.turnOff = function (callback) {
            if (client.cast.browserCastClient) {
                client.user.turnOffViewer();
                client.cast.browserCastClient.dispose();
                client.cast.browserCastClient = null;
            }
            callback(null, true);
        };
        /** Start scanning for browser devices */
        CastViewerFireTV.startScan = function () {
            if (!CastViewerFireTV.fireTV) {
                CastViewerFireTV.fireTV = new CastViewerFireTV();
            }
            // There is always one fireTV device available (as we can't detect them)
            client.CastViewManager.addCastViewer(CastViewerFireTV.fireTV);
        };
        //=================================================
        // Statics
        //=================================================
        CastViewerFireTV.fireTV = null;
        return CastViewerFireTV;
    })(client.CastViewer);
    client.CastViewerFireTV = CastViewerFireTV;
})(client || (client = {}));
/// <reference path="types.ts"/>
/// <reference path="consts.ts"/>
/// <reference path="castViewer.ts"/>
/// <reference path="../shared/googleCast.ts"/>
var client;
(function (client) {
    /** A ChromeCast based castViewer */
    var CastViewerChromeCast = (function (_super) {
        __extends(CastViewerChromeCast, _super);
        function CastViewerChromeCast(name, id) {
            if (name === void 0) { name = "Chrome Cast"; }
            if (id === void 0) { id = "Chrome"; }
            _super.call(this, name, id);
            this.icon = "glyphiconXIM glyphiconXIM-gCastd";
            this.iconActive = "glyphiconXIM glyphiconXIM-gCastda";
        }
        /** Start scanning for chromecast devices */
        CastViewerChromeCast.startScan = function () {
            if (window.chrome && client.startupMode != 3 /* wp */) {
                // Start scanner and set scan result callback
                if (!client.cast.googleCastClient) {
                    client.cast.googleCastClient = new client.cast.GoogleCast(consts.cloudProviders.chromecast);
                }
                client.cast.googleCastClient.init(CastViewerChromeCast.onScanResult);
            }
        };
        /** Call back function for chomecast scan */
        CastViewerChromeCast.onScanResult = function (available) {
            client.logger.info('receiver callback with result: ', available);
            // If available chromecasts, get routes
            if (available) {
                var routes = client.cast.googleCastClient.getRoutes();
                for (var i = 0; i < routes.length; i++) {
                    var route = routes[i];
                    client.CastViewManager.addCastViewer(new CastViewerChromeCast(route.name, route.id));
                }
            }
        };
        CastViewerChromeCast.prototype.onXimJoined = function (m) {
            this.groupdId = m.ximSessionID;
        };
        /** Attempt to display to this castViewer */
        CastViewerChromeCast.prototype.turnOn = function (callback) {
            var _this = this;
            if (!callback) {
                client.logger.error(consts.errorCode.CastViewer, 'missing callback');
                return;
            }
            client.logger.collectNamedEvent(consts.flurryEvent.castViewerAdd, { 'viewType': 'ChromeCast' });
            client.cast.googleCastClient.selectRoute(this.id, function (err, success) {
                if (!err && success) {
                    _this.onXimJoinedHandler = _.bind(_this.onXimJoined, _this);
                    client.user.onXimJoined.add(_this.onXimJoinedHandler);
                    // Register for disconnect events
                    client.cast.googleCastClient.onSessionUpdated.add(function (isActive) {
                        if (!isActive) {
                            client.CastViewManager.onDisconnected(_this);
                        }
                    });
                    callback(null, true);
                }
                else {
                    callback(err, false);
                }
            });
        };
        /** Attempt to turn off this castViewer */
        CastViewerChromeCast.prototype.turnOff = function (callback) {
            var _this = this;
            client.cast.googleCastClient.leave(function (e, success) {
                if (!e && success) {
                    client.user.onXimJoined.remove(_this.onXimJoinedHandler);
                    _this.onXimJoinedHandler = null;
                    _this.groupdId = null;
                }
                callback(e, success);
            });
        };
        return CastViewerChromeCast;
    })(client.CastViewer);
    client.CastViewerChromeCast = CastViewerChromeCast;
})(client || (client = {}));
/// <reference path="types.ts"/>
/// <reference path="consts.ts"/>
/// <reference path="castViewer.ts"/>
/// <reference path="castViewManager.ts"/>
/// <reference path="../../types/cordova/smartglass.d.ts"/>
/// <reference path="Connection.ts"/>
var client;
(function (client) {
    /** A xbox based castViewer */
    var CastViewerXBox = (function (_super) {
        __extends(CastViewerXBox, _super);
        function CastViewerXBox(name, id) {
            if (name === void 0) { name = "XBox One"; }
            if (id === void 0) { id = "XBox"; }
            _super.call(this, name, id);
            this.connection = new client.Connection();
            this.icon = "glyphiconXIM glyphiconXIM-xboxDisplay";
            this.iconActive = "glyphiconXIM glyphiconXIM-xboxDisplaya";
        }
        /** Start scanning for xbox devices */
        CastViewerXBox.startScan = function () {
            // Note: There is always one xbox device available that provides instructions
            if (window.smartglass) {
                window.smartglass.registerListener(function (device) {
                    if (_.isString(device)) {
                        device = JSON.parse(device);
                    }
                    client.logger.info('smartglass scan callback', device);
                    //routeId 0.0.0.0 means that the xbox discovery is finished
                    if (device.routeId == '0.0.0.0') {
                        // If no device found add placeholder for instructions
                        if (!CastViewerXBox.foundDevices) {
                            client.CastViewManager.addCastViewer(new CastViewerXBox("XBox One", device.routeId));
                        }
                    }
                    else if (device.available) {
                        // Note device found and add to castViewers
                        CastViewerXBox.foundDevices = true;
                        client.CastViewManager.addCastViewer(new CastViewerXBox(device.name, device.routeId));
                    }
                }, function (e) {
                    client.logger.warn('smartglass failed to registerListener', e);
                });
            }
        };
        /** Attempt to display to this castViewer */
        CastViewerXBox.prototype.turnOn = function (callback) {
            var _this = this;
            client.logger.collectNamedEvent(consts.flurryEvent.castViewerAdd, { 'viewType': 'XBox' });
            // If no devices were found, show how to popup
            if (this.id == '0.0.0.0') {
                var howToPopUp = new client.CastViewerHowToPopUp(null, "howToXBox", resources.forms.xboxHowTo, function (e, s) {
                    callback(null, false);
                });
                return;
            }
            // Otherwise attempt to connect
            var smartglass = window.smartglass;
            if (smartglass) {
                smartglass.selectRoute(this.id, function (connected) {
                    if (connected) {
                        _this.connectViewer(callback);
                    }
                    else {
                        _this.disconnectViewer();
                        client.CastViewManager.onDisconnected(_this);
                        callback(null, connected);
                    }
                }, function (e) {
                    client.logger.warn('smartglass selectRoute error', e);
                    callback(null, false);
                });
            }
        };
        /** Attempt to turn off this castViewer */
        CastViewerXBox.prototype.turnOff = function (callback) {
            this.disconnectViewer();
            callback(null, true);
        };
        CastViewerXBox.prototype.connectViewer = function (cb) {
            var _this = this;
            this.disconnectViewer();
            this.connection.createViewerSession(false, function (err, session) {
                if (!err && session) {
                    _this.viewer = session;
                    client.user.turnOnViewer(_this.viewer);
                    // Load viewer on XBOX
                    var url = _this.connection.root + 'xboxReceiver' + client.app.toQueryString({ accessToken: _this.viewer.accessToken.token, sessionId: _this.viewer.sessionId });
                    window.smartglass.loadUrl(url, function () {
                        client.logger.info('smartglass load url ok');
                        cb(null, true);
                    }, function (e) {
                        client.logger.warn('smartglass load url failed', e);
                        cb(new Error('failed'), false);
                    });
                }
                else {
                    cb(err, false);
                }
            });
        };
        CastViewerXBox.prototype.disconnectViewer = function () {
            if (this.viewer) {
                client.logger.info("smartglass discounnected");
                client.user.turnOffViewer();
                this.viewer.dispose();
                this.viewer = null;
            }
        };
        /** Keeps track of whether any XBox's were found during scanning */
        CastViewerXBox.foundDevices = false;
        return CastViewerXBox;
    })(client.CastViewer);
    client.CastViewerXBox = CastViewerXBox;
})(client || (client = {}));
/// <reference path="types.ts"/>
/// <reference path="consts.ts"/>
/// <reference path="castViewerAirPlay.ts"/>
/// <reference path="castViewerBrowser.ts"/>
/// <reference path="castViewerFireTV.ts"/>
/// <reference path="castViewerChromeCast.ts"/>
/// <reference path="castViewerXBox.ts"/>
/** Module for managing cast viewer (i.e. xbox, display, chromecast) */
var client;
(function (client) {
    var CastViewManager;
    (function (CastViewManager) {
        CastViewManager.onCastViewerChanged = new client.Event();
        /** Array of all castViewers */
        CastViewManager.castViewers = [];
        /** Array of all castViewers */
        CastViewManager.changeCallback = null;
        /** Active castVierwer */
        CastViewManager.activeViewer = null;
        /** Start scanning for castViewers */
        function startScans() {
            // Reset viewers list
            CastViewManager.castViewers = [];
            // Add in active viewer first if it exists
            if (CastViewManager.activeViewer) {
                CastViewManager.castViewers.push(CastViewManager.activeViewer);
            }
            client.CastViewerChromeCast.startScan();
            client.CastViewerBrowser.startScan();
            client.CastViewerFireTV.startScan();
            client.CastViewerXBox.startScan();
            client.CastViewerAirPlay.startScan();
        }
        CastViewManager.startScans = startScans;
        /** Scan for castViewers, call callback when list is updated */
        function getCastViewers(callback) {
            this.changeCallback = callback;
            CastViewManager.startScans();
            // If any cast viewers already exist, call the callback
            if (this.castViewers.length > 0) {
                this.changeCallback(this.castViewers);
            }
        }
        CastViewManager.getCastViewers = getCastViewers;
        /** Stop returning cast viewers */
        function stopScans() {
            this.changeCallback = null;
        }
        CastViewManager.stopScans = stopScans;
        /** Returns icon to display */
        function getIcon() {
            // If there is an active viewer, display icon for that viewer
            if (this.activeViewer) {
                return this.activeViewer.icon;
            }
            // Otherwise
            return "glyphiconXIM glyphiconXIM-display";
        }
        CastViewManager.getIcon = getIcon;
        /** Returns icon to display */
        function getActiveIcon() {
            // If there is an active viewer, display icon for that viewer
            if (this.activeViewer) {
                return this.activeViewer.iconActive;
            }
            // Otherwise
            return "glyphiconXIM glyphiconXIM-display";
        }
        CastViewManager.getActiveIcon = getActiveIcon;
        /** Get the active castViewer - null if none is active*/
        function getActiveViewer() {
            return CastViewManager.activeViewer;
        }
        CastViewManager.getActiveViewer = getActiveViewer;
        /** Get the active castViewer - null if none is active*/
        function addCastViewer(castViewer) {
            for (var i = 0; i < this.castViewers.length; i++) {
                if (this.castViewers[i].id == castViewer.id) {
                    return;
                }
            }
            this.castViewers.push(castViewer);
            if (this.changeCallback) {
                this.changeCallback(this.castViewers);
            }
        }
        CastViewManager.addCastViewer = addCastViewer;
        /** Called when a castViewer is disconnected (i.e. device turned off) */
        function onDisconnected(castViewer) {
            // If disconnected view is active one, clear activeViewer and trigger change event
            if (CastViewManager.activeViewer && CastViewManager.activeViewer.id == castViewer.id) {
                CastViewManager.activeViewer = null;
                this.onCastViewerChanged.trigger(null);
            }
        }
        CastViewManager.onDisconnected = onDisconnected;
        /** Select/Deselect a viewer by id, takes selector in case UI is needed */
        function toggle(castViewer, callback) {
            var _this = this;
            if (CastViewManager.castViewers && castViewer) {
                // If it was active, deselect it (NOTE: compare by id not object)
                if (CastViewManager.activeViewer && castViewer.id == CastViewManager.activeViewer.id) {
                    castViewer.turnOff(function (err, success) {
                        if (!err && success) {
                            CastViewManager.activeViewer = null;
                            _this.onCastViewerChanged.trigger(null);
                        }
                        callback(err, success);
                    });
                }
                else {
                    // If another viewer was active turn it off
                    if (CastViewManager.activeViewer) {
                        CastViewManager.activeViewer.turnOff(function (err, success) {
                            if (!err && success) {
                                CastViewManager.activeViewer = null;
                                _this.onCastViewerChanged.trigger(null);
                            }
                            callback(err, success);
                        });
                    }
                    // Try to turn the new viewer on
                    castViewer.turnOn(function (err, success) {
                        if (!err && success) {
                            CastViewManager.activeViewer = castViewer;
                            _this.onCastViewerChanged.trigger(null);
                        }
                        callback(err, success);
                    });
                }
            }
            else {
                _.defer(function () {
                    callback(null, false);
                });
            }
        }
        CastViewManager.toggle = toggle;
    })(CastViewManager = client.CastViewManager || (client.CastViewManager = {}));
})(client || (client = {}));
/// <reference path="Screen.ts"/>
/// <reference path='../resources.ts'/>
var client;
(function (client) {
    /** Lobby Screen for castViewer */
    var LobbyScreen = (function (_super) {
        __extends(LobbyScreen, _super);
        function LobbyScreen() {
            _super.apply(this, arguments);
            /** Index into rotating content */
            this.contentIndex = -1;
        }
        /** Rotate to next display message */
        LobbyScreen.prototype.nextMessage = function () {
            var _this = this;
            this.contentIndex++;
            if (this.contentIndex == resources.messages.lobby.content.length) {
                this.contentIndex = 0;
            }
            // Fade in and show message
            this.$el.find('#line1').text(resources.messages.lobby.content[this.contentIndex].line1).css('opacity', '1.0');
            this.$el.find('#line2').text(resources.messages.lobby.content[this.contentIndex].line2).css('opacity', '1.0');
            this.$el.find('#line3').text(resources.messages.lobby.content[this.contentIndex].line3).css('opacity', '1.0');
            // Fade out and rotate to next message
            setTimeout(function () {
                _this.$el.find('#line1').css('opacity', '0.0');
                _this.$el.find('#line2').css('opacity', '0.0');
                _this.$el.find('#line3').css('opacity', '0.0');
                setTimeout(_.bind(_this.nextMessage, _this), 600);
            }, 10000);
        };
        LobbyScreen.prototype.navigate = function (route) {
            this.render();
            return _super.prototype.navigate.call(this, route);
        };
        LobbyScreen.prototype.render = function () {
            // Make screen is in a valid state
            if (this.getScreenState() == 3 /* DISPOSED */) {
                client.logger.warn("Render called on DISPOSED screen ");
                return this;
            }
            // Render view
            this.empty();
            this.$el.addClass('splash').render('LobbyScreen', { resources: resources.messages.lobby, isWp: client.startupMode == 3 /* wp */ });
            this.$el.addClass('lobbyScreen');
            // Start message rotation
            this.nextMessage();
            this.syncNoPan();
            return _super.prototype.render.call(this);
        };
        /**
          * Displays a new lobby screen as a popup.
          * @param message The loading message to display.
          * @param callback (optional) The callback to call once the screen has been dropped.
          */
        LobbyScreen.display = function (callback) {
            return client.screens.showModal(LobbyScreen, { style: 1 /* popup */, params: {} }, function (e, results) {
                callback(e, results);
                return false;
            });
        };
        return LobbyScreen;
    })(client.Screen);
    client.LobbyScreen = LobbyScreen;
})(client || (client = {}));
/// <reference path="Screen.ts"/>
/// <reference path='LobbyScreen.ts'/>
var client;
(function (client) {
    /** Displays a expired screen */
    var ExpiredScreen = (function (_super) {
        __extends(ExpiredScreen, _super);
        function ExpiredScreen() {
            _super.apply(this, arguments);
        }
        ExpiredScreen.prototype.navigate = function (route) {
            this.render();
            return _super.prototype.navigate.call(this, route);
        };
        ExpiredScreen.prototype.render = function () {
            // Make screen is in a valid state
            if (this.getScreenState() == 3 /* DISPOSED */) {
                client.logger.warn("Render called on DISPOSED screen ");
                return this;
            }
            // Render view
            this.empty();
            var isWp = (client.startupMode == 3 /* wp */);
            this.$el.addClass('splash').render('ExpiredScreen', { isWp: isWp, text: resources.messages.expiredxim });
            this.syncNoPan();
            return _super.prototype.render.call(this);
        };
        /**
          * Displays an expired screen
          * @param callback (optional) The callback to call once the screen has been dropped.
          */
        ExpiredScreen.display = function (callback) {
            return client.screens.showModal(ExpiredScreen, { style: 1 /* popup */ }, function (e, results) {
                callback(e, results);
                return false;
            });
        };
        return ExpiredScreen;
    })(client.Screen);
    client.ExpiredScreen = ExpiredScreen;
})(client || (client = {}));
/// <reference path='../../../types/cordova/screenlock.d.ts'/>
/// <reference path="Screen.ts"/>
/// <reference path="../user.ts"/>
/// <reference path="../group.ts"/>
/// <reference path='../recentPeople.ts'/>
/// <reference path='../castViewManager.ts'/>
/// <reference path='ExpiredScreen.ts'/>
var client;
(function (client) {
    (function (CastingState) {
        CastingState[CastingState["joining"] = 0] = "joining";
        CastingState[CastingState["casting"] = 1] = "casting";
    })(client.CastingState || (client.CastingState = {}));
    var CastingState = client.CastingState;
    ;
    /** Base class for all castable screens. */
    var CastingScreen = (function (_super) {
        __extends(CastingScreen, _super);
        function CastingScreen() {
            _super.apply(this, arguments);
            this.tsStart = consts.time.now();
            /** Current screen state. */
            this.state = 0 /* joining */;
            /** Event triggered when Xim expires */
            this.onXimExpired = new client.Event();
        }
        CastingScreen.prototype.onDispose = function () {
            if (this.cast) {
                this.cast.unjoin();
                this.cast = null;
            }
            // Clear timers
            clearTimeout(this.expirationTimer);
            // Lock screen back to vertical
            if (window.screenlock) {
                window.screenlock.set(consts.ScreenLockState.vertical);
            }
            if (this.orientationHandler) {
                client.app.removeEventListner('orientationchange', this.orientationHandler);
                this.orientationHandler = null;
                // force viewport back to portrait mode once the casting is over
                client.webview.setViewportOrientation(0 /* portrait */);
            }
            if (this.castViewerChangeHandler) {
                client.CastViewManager.onCastViewerChanged.remove(this.castViewerChangeHandler);
            }
            _super.prototype.onDispose.call(this);
        };
        /** Starts a new group & cast in one step. */
        CastingScreen.prototype.startXim = function (invites, prepareCast, callback) {
            var _this = this;
            prepareCast(this, client.ifSuccess(function (castOptions) {
                client.user.createGroup(invites, client.Group.castOptionsToData(castOptions), client.ifSuccess(function (group) {
                    if (group && group.cast) {
                        client.logger.collectNamedEvent(consts.flurryEvent.ximStartXim, {
                            mediaLength: castOptions.queue.items.length,
                            groupSize: (invites ? invites.length : 0)
                        });
                        client.user.triggerXimJoined(group.profile.groupId);
                        _this.setCast(group.cast);
                        if (callback)
                            callback(null, true);
                    }
                    else if (callback) {
                        callback(new Error('No cast created. old server'), false);
                    }
                }, callback));
            }, callback));
            return this;
        };
        CastingScreen.prototype.startCast = function (grp, options, callback) {
            var that = this;
            function createGroup(o) {
                // Check for group factory
                if (_.isFunction(grp)) {
                    grp(client.ifSuccess(function (g) { return initCast(g, o); }, callback));
                }
                else {
                    initCast(grp, o);
                }
            }
            function initCast(g, o) {
                // Start cast
                g.startCast(o, client.ifSuccess(function (cast) {
                    client.user.triggerXimJoined(g.profile.groupId);
                    that.setCast(cast);
                    if (callback)
                        callback(null, true);
                }, callback));
            }
            ;
            // Check for options factory
            if (_.isFunction(options)) {
                options(that, client.ifSuccess(createGroup, callback));
            }
            else {
                createGroup(options);
            }
            return this;
        };
        /**
          * Joins an existing cast
          * @param grp The group to join cast for.
          * @param castId The ID of the cast to join.
          * @param callback (optional) Async completion of the operation.
          */
        CastingScreen.prototype.joinCast = function (grp, castId, callback) {
            var _this = this;
            grp.joinCast(castId, client.ifSuccess(function (cast) {
                client.user.triggerXimJoined(grp.profile.groupId);
                _this.setCast(cast);
                var mediaLength = cast.queue && cast.queue.items ? cast.queue.items.length : 0;
                var groupSize = grp && grp.profile && grp.profile.members ? _.keys(grp.profile.members).length : 0;
                client.logger.collect(consts.tags.cast.join, {
                    mediaLength: mediaLength,
                    groupSize: groupSize
                });
                if (callback)
                    callback(null, true);
            }, callback));
            return this;
        };
        /**
          * Bind the casting screen to it's cast.
          * @param cast The cast to bind to.
          */
        CastingScreen.prototype.setCast = function (cast) {
            var _this = this;
            if (this.state == 0 /* joining */) {
                this.state = 1 /* casting */;
                this.cast = cast;
                // Listen for state changes
                this.cast.onQueueUpdated.add(function (args) {
                    _this.onQueueUpdated(args);
                });
                this.cast.onSettingsUpdated.add(function (args) {
                    _this.onSettingsUpdated(args);
                });
                this.cast.onStateUpdated.add(function (args) {
                    _this.onStateUpdated(args);
                });
                this.cast.group.onNewPost.add(function (args) {
                    _this.onNewPost(args);
                });
                // Unlock screen so user can rotate
                if (window.screenlock) {
                    window.screenlock.clear();
                }
                // Update recent Xims list 
                client.RecentPeople.Match(cast.group.profile.members);
                // Signal ready
                this.onCastReady();
                // Set timer to boot user out of cast when it's expired
                this.setExpirationTimer();
                // Update UI
                var tsPauseFor = consts.time.seconds(1);
                var tsAge = consts.time.age(this.tsStart);
                if (tsAge >= tsPauseFor) {
                    this.render();
                }
                else {
                    setTimeout(function () {
                        _this.render();
                    }, tsPauseFor - tsAge);
                }
            }
            return this;
        };
        /** Set timer to exit Xim when time has expired */
        CastingScreen.prototype.setExpirationTimer = function () {
            var _this = this;
            if (this.cast && this.cast.group) {
                var msRemaining = 60000 * this.cast.group.minutesRemaining();
                // Trigger Xim expired event (which will be captured by castViewers)
                this.expirationTimer = setTimeout(function () {
                    _this.onXimExpired.trigger(null);
                }, msRemaining);
                // If not castViewer, respond to expired event
                if (!client.isDesktopDisplay) {
                    this.onXimExpired.add(function () {
                        // Display expired pop up
                        var popup = new client.ErrorPopUp(client.ErrorMessages.XIM_EXPIRED, function () {
                            // Exit cast on close of expiration message
                            history.back();
                        });
                        popup.show(true);
                    });
                }
            }
        };
        /** Handles initialization of the dialog. */
        CastingScreen.prototype.navigate = function (route) {
            this.render();
            return _super.prototype.navigate.call(this, route);
        };
        /** render a back button on ios devices for all casting screen */
        CastingScreen.prototype.render = function () {
            var _this = this;
            // Make screen is in a valid state
            if (this.getScreenState() == 3 /* DISPOSED */) {
                client.logger.warn("Render called on DISPOSED screen ");
                return _super.prototype.render.call(this);
            }
            if ($('body').hasClass('ios')) {
                $('<i class="close-casting"/>').appendTo(this.$el);
            }
            this.bindOnNavigate('', function () {
                _this.$el.on('click', '.close-casting', function () {
                    _this.history.back();
                    return false;
                });
                _this.orientationHandler = _.bind(_this.onOrientationChange, _this);
                client.app.addEventListner('orientationchange', _this.orientationHandler);
                _this.bindCast();
            });
            this.syncNoPan();
            return _super.prototype.render.call(this);
        };
        /** override to bind custom controls */
        CastingScreen.prototype.bindCast = function () {
            this.castViewerChangeHandler = _.bind(this.onCastViewerChange, this);
            client.CastViewManager.onCastViewerChanged.add(this.castViewerChangeHandler);
        };
        /** load the castviewer with the current cast */
        CastingScreen.prototype.onCastViewerChange = function () {
            var activeCastViewer = client.CastViewManager.activeViewer;
            var groupId = this.cast && this.cast.group && this.cast.group.profile && this.cast.group.profile.groupId ? this.cast.group.profile.groupId : null;
            // do nothing if there is no active viewer or the active viewer is already showing the current cast
            if (!activeCastViewer || groupId == activeCastViewer.groupdId) {
                return;
            }
            if (groupId) {
                client.user.triggerXimJoined(groupId);
            }
        };
        /** detected orientation change */
        CastingScreen.prototype.onOrientationChange = function (ev) {
            var _this = this;
            if (!client.capability.isJellybean) {
                _.delay(function () {
                    var castScreen = _this;
                    _this.$el.find('.castHeader').toggleClass('hidden', true);
                    window.setTimeout(function () {
                        castScreen.$el.find('.castHeader').toggleClass('hidden', false);
                    }, 100);
                    // iOS display will set window.innerHeight wrong, 
                    // if we tweak the viewport meta tag see jsbin example at http://jsbin.com/bafaki on iPhone for repro
                    // skip syncing viewport meta tag on iPhone display
                    if (!client.app.isIPhoneDisplay()) {
                        client.webview.syncViewportWithOrientation();
                    }
                }, 200);
            }
            else {
                window.setTimeout(function () {
                    client.webview.syncViewportWithOrientation();
                    $('.popUpDialog').css('width', client.webview.currentViewport().width + 'px');
                }, 300);
            }
        };
        /** Called when the cast is ready and about to be rendered. */
        CastingScreen.prototype.onCastReady = function () {
        };
        /** The screens queue has been updated. */
        CastingScreen.prototype.onQueueUpdated = function (queue) {
            client.logger.info('CastingScreen<' + this.cast.castId + '> onQueueUpdated', queue);
        };
        /** A new post has been created. */
        CastingScreen.prototype.onNewPost = function (post) {
            client.logger.info('CastingScreen<' + this.cast.castId + '> onNewPost ', post.id);
        };
        /** The screens settings have been updated. */
        CastingScreen.prototype.onSettingsUpdated = function (settings) {
            client.logger.info('CastingScreen<' + this.cast.castId + '> onSettingsUpdated', settings);
        };
        /** The screens state has been updated. */
        CastingScreen.prototype.onStateUpdated = function (state) {
            client.logger.info('CastingScreen<' + this.cast.castId + '> onStateUpdated', state);
        };
        //==================
        // DEPRECATED EVENTS
        //==================
        /** The sessions properties have changed. */
        CastingScreen.prototype.onPropertiesChanged = function (ev) {
        };
        return CastingScreen;
    })(client.Screen);
    client.CastingScreen = CastingScreen;
    var blacklist = new CastingScreen(null);
})(client || (client = {}));
/// <reference path="types.ts"/>
/// <reference path="consts.ts"/>
/// <reference path="castViewerAirPlay.ts"/>
/// <reference path="castViewerBrowser.ts"/>
/// <reference path="castViewerFireTV.ts"/>
/// <reference path="castViewerChromeCast.ts"/>
/// <reference path="castViewerXBox.ts"/>
/** Module for storing image meta data for a xim */
var client;
(function (client) {
    var ImageMetaData;
    (function (ImageMetaData) {
        /** Latest comment for each image */
        var comments = {};
        /** Sun count (likes) for each image */
        var suns = {};
        /** Maximum number of suns allowed */
        ImageMetaData.MAX_SUN_COUNT = 5;
        /** Clear meta data */
        function clearMetaData() {
            comments = {};
            suns = {};
        }
        ImageMetaData.clearMetaData = clearMetaData;
        /** Set comment for the given image */
        function setComment(imageId, comment) {
            comments[imageId] = comment;
        }
        ImageMetaData.setComment = setComment;
        /** Return comment for the given image */
        function getComment(imageId) {
            if (comments[imageId]) {
                return comments[imageId];
            }
            else {
                return null;
            }
        }
        ImageMetaData.getComment = getComment;
        /** Add a sun to the given image */
        function addSun(imageId, userId) {
            if (suns[imageId]) {
                suns[imageId]['TOTAL']++;
                if (suns[imageId][userId]) {
                    suns[imageId][userId]++;
                }
                else {
                    suns[imageId][userId] = 1;
                }
            }
            else {
                suns[imageId] = {};
                suns[imageId]['TOTAL'] = 1;
                suns[imageId][userId] = 1;
            }
        }
        ImageMetaData.addSun = addSun;
        /** Get sun count for the given image */
        function getSuns(imageId, userId) {
            if (suns[imageId]) {
                if (userId) {
                    if (suns[imageId][userId]) {
                        return Math.min(suns[imageId][userId], ImageMetaData.MAX_SUN_COUNT);
                    }
                }
                else {
                    return Math.min(suns[imageId]['TOTAL'], ImageMetaData.MAX_SUN_COUNT);
                }
            }
            else {
                return 0;
            }
        }
        ImageMetaData.getSuns = getSuns;
    })(ImageMetaData = client.ImageMetaData || (client.ImageMetaData = {}));
})(client || (client = {}));
/// <reference path="../../../types/jquery/jquery.d.ts" />
/// <reference path="../types.ts"/>
/// <reference path="../ScreenCast.ts"/>
/// <reference path="../imageMetaData.ts"/>
var client;
(function (client) {
    /** Animation states for the PhotoSlider */
    var AnimationStates;
    (function (AnimationStates) {
        AnimationStates[AnimationStates["AT_REST"] = 0] = "AT_REST";
        AnimationStates[AnimationStates["ANIMATING"] = 1] = "ANIMATING";
        AnimationStates[AnimationStates["USER_MOVING"] = 2] = "USER_MOVING";
        AnimationStates[AnimationStates["USER_WAITING"] = 3] = "USER_WAITING"; // Waiting after user has taken an action
    })(AnimationStates || (AnimationStates = {}));
    ;
    /** Load state for images */
    (function (ImageState) {
        ImageState[ImageState["NEW"] = 0] = "NEW";
        ImageState[ImageState["WAITING"] = 1] = "WAITING";
        ImageState[ImageState["RETRY"] = 2] = "RETRY";
        ImageState[ImageState["LOADED"] = 3] = "LOADED";
        ImageState[ImageState["ERROR"] = 4] = "ERROR"; // Encountered an error loading the image
    })(client.ImageState || (client.ImageState = {}));
    var ImageState = client.ImageState;
    ;
    var PhotoSlider = (function () {
        function PhotoSlider($el, photoCastScreen, media, sliderState, options) {
            if (options === void 0) { options = {}; }
            this.$el = $el;
            this.photoCastScreen = photoCastScreen;
            // Callback allows for interception of events
            this.eventCallback = null;
            // Slide holders
            this.prevSlide = null;
            this.nextSlide = null;
            this.curSlide = null;
            this.hammerOptions = {
                prevent_default: true,
                scale_threshold: 0,
                drag_min_distance: 5,
                hold_timeout: 450,
                tap_double_distance: 50
            };
            /** Current view state of the slider */
            this.sliderState = {
                queueIndex: 0,
                imgScale: 1.0,
                imgXOffset: 0,
                imgYOffset: 0
            };
            /** Last state of slider */
            this.lastQueueIndex = -1;
            this.lastScale = 1;
            /** Requested view state for the slider.  (Coming from other screens) */
            this.requestedState = null;
            /** Current movement state of the slider */
            this.animationState = 0 /* AT_REST */;
            /** Timeout for move waiting */
            this.waitTimeout = null;
            /** Timeout for fading flippers */
            this.flipperLeftTimeout = null;
            this.flipperRightTimeout = null;
            this.flipperEatTap = false;
            /** Has the PhotoSlider been disposed */
            this.isDisposed = false;
            // User interface 
            this.$bar = null;
            this.$images = [];
            // Used to store state when user initiates a slide or zoom action
            this.startXPos = null;
            this.startYPos = null;
            this.startXTouch = null;
            this.startYTouch = null;
            this.startScale = null;
            // Calculate screen size
            this.calcScreenSize();
            // Initialize options
            this.hammerOptions.prevent_default = options.preventDefault;
            this.$el.hammer(this.hammerOptions);
            this.useFlippers = options.useFlippers;
            // Set up event handlers
            this.initEventHandlers();
            // Compose potentially partial state change into complete state change
            this.sliderState = this.composeStateChange(sliderState);
            // Render the page
            this.render(media);
        }
        PhotoSlider.prototype.onDispose = function () {
            this.isDisposed = true;
            // Clear timers
            clearTimeout(this.waitTimeout);
            clearTimeout(this.flipperLeftTimeout);
            clearTimeout(this.flipperRightTimeout);
            // Ensure that jquery stored data is cleaned up
            this.$el.empty();
        };
        //=============================================================
        // PUBLIC
        //=============================================================
        /** Returns current photo slider state */
        PhotoSlider.prototype.getScreenState = function () {
            return this.sliderState;
        };
        /** Attempt to move to next slide via flipper or keyboard input */
        PhotoSlider.prototype.jumpToNextSlide = function () {
            var index = Math.min(this.sliderState.queueIndex + 1, this.$images.length - 1);
            this.jumpToSlide(index);
        };
        /** Attempt to move to previous slide via flipper or keyboard input */
        PhotoSlider.prototype.jumpToPreviousSlide = function () {
            var index = Math.max(this.sliderState.queueIndex - 1, 0);
            this.jumpToSlide(index);
        };
        /** Attempt to move to last slide */
        PhotoSlider.prototype.jumpToLastSlide = function () {
            var lastSlide = this.$images.length - 1;
            this.jumpToSlide(lastSlide);
        };
        /** Attempt to move to first */
        PhotoSlider.prototype.jumpToFirstSlide = function () {
            this.jumpToSlide(0);
        };
        /** Attempt to move to given slide */
        PhotoSlider.prototype.jumpToSlide = function (index) {
            if (index >= 0 && index < this.$images.length) {
                var duration = this.animationDuration(index);
                this.moveToIndex(index, duration, 0 /* AT_REST */);
                // Notify other screens of the change
                this.triggerSlideChangeEvent();
            }
        };
        /** When true zoom to full screen, When false zoom to normal scale */
        // Default to screen center
        PhotoSlider.prototype.setMaxZoom = function (zoom, x, y) {
            if (x === void 0) { x = (0.5 * this.screenWidth); }
            if (y === void 0) { y = (0.5 * this.screenHeight); }
            // Don't allow scale transforms for non-loaded images
            if (this.$images[this.sliderState.queueIndex].data('state') != 3 /* LOADED */) {
                return;
            }
            // While animating, ignore double tap
            if (this.animationState == 1 /* ANIMATING */) {
                return;
            }
            // Zoom to fill screen
            if (zoom) {
                // Set scale and offset based on tap position
                this.sliderState.imgScale = this.fullScreenScale(this.$images[this.sliderState.queueIndex]);
                this.sliderState.imgXOffset = (this.sliderState.imgScale - 1) * ((0.5 * this.screenWidth) - x);
                this.sliderState.imgYOffset = (this.sliderState.imgScale - 1) * ((0.5 * this.screenHeight) - y);
                // Adjust offset to keep on screen
                this.constrainHorizontally();
                this.constrainVertically();
                // Move slide to new position
                this.translateSlide(300, 3 /* USER_WAITING */);
            }
            else {
                this.moveToIndex(this.sliderState.queueIndex, 300, 3 /* USER_WAITING */);
            }
            // User has completed moving
            this.onUserDone();
        };
        /** Request to move to a specific slider state */
        PhotoSlider.prototype.requestMove = function (changeState) {
            // Compose potentially partial state change into complete state change
            var requestedState = this.composeStateChange(changeState);
            //DEBUG this.DebugText(this.sliderState.queueIndex.toString() + ">>" + requestedState.queueIndex.toString());  
            // If I'm at rest or performing an automated animation, move to the requested state
            if (this.animationState == 0 /* AT_REST */ || this.animationState == 1 /* ANIMATING */) {
                this.requestedState = null;
                var duration = this.animationDuration(requestedState.queueIndex);
                this.moveTo(requestedState, duration, 0 /* AT_REST */);
            }
            else {
                this.requestedState = requestedState;
            }
        };
        /** Add new media */
        PhotoSlider.prototype.addMedia = function (media) {
            // Add media to photoSlider
            this.appendMedia(media);
            // Update layout
            this.relayout();
        };
        /** Update latest comment for an image */
        PhotoSlider.prototype.addComment = function (imageId, userId, comment, hasSun) {
            if (hasSun === void 0) { hasSun = false; }
            // Update comment
            if (comment) {
                client.ImageMetaData.setComment(imageId, comment);
            }
            // Update sun count
            if (hasSun) {
                client.ImageMetaData.addSun(imageId, userId);
            }
            // If image is currently in displayed range update it
            var $slideHolder = this.$bar.find("#holder" + imageId);
            if ($slideHolder.length > 0) {
                // If there is an existing comment
                if (comment) {
                    // Clear any existing comment
                    $slideHolder.find(".photoComment").remove();
                    $slideHolder.append(this.createComment(comment));
                }
                // If sun was updated on displayed image, animate it
                if (hasSun && imageId == this.sliderState.queueIndex) {
                    this.showSun(true);
                }
            }
        };
        /** Show of hide the sun (if the image has one) */
        PhotoSlider.prototype.showSun = function (show) {
            if (show) {
                // Only animate sun when at base scale
                if (this.sliderState.imgScale != 1) {
                    return;
                }
                // Does this image have any suns
                var count = client.ImageMetaData.getSuns(this.sliderState.queueIndex);
                if (count > 0) {
                    var $holder = this.$bar.find("#holder" + this.sliderState.queueIndex);
                    var $sun = $holder.find(".photoSun");
                    // If doesn't exist add it
                    if ($sun.length == 0) {
                        var uriSrc = ((client.baseUri) ? client.baseUri : '') + 'img/sunGrow.png';
                        $sun = $('<img class="photoSun" src="' + uriSrc + '"></img>').css({
                            '-ms-transition-duration': '0ms',
                            '-webkit-transition-duration': '0ms',
                            'transition-duration': '0ms',
                            '-ms-transform': 'scale(0)',
                            '-webkit-transform': 'scale(0)',
                            'transform': 'scale(0)'
                        });
                        $holder.append($sun);
                    }
                    // Animate it into place - pause to allow redraw
                    setTimeout(function () {
                        if ($sun.length > 0) {
                            var scale = 'scale(' + (5 * count) + ')';
                            // WP (and not Android) needs 0 translation set to prevent pop
                            if (client.startupMode == 3 /* wp */) {
                                scale += ' translate(0px, 0px)';
                            }
                            $sun.css({
                                '-ms-transition-duration': '500ms',
                                '-webkit-transition-duration': '500ms',
                                'transition-duration': '500ms',
                                '-ms-transform': scale,
                                '-webkit-transform': scale,
                                'transform': scale,
                                'opacity': 1
                            });
                        }
                    });
                }
            }
            else {
                // First reset all sun
                var $suns = this.$bar.find(".photoSun");
                $suns.css({
                    '-ms-transition-duration': '500ms',
                    '-webkit-transition-duration': '500ms',
                    'transition-duration': '500ms',
                    '-ms-transform': 'scale(0)',
                    '-webkit-transform': 'scale(0)',
                    'transform': 'scale(0)',
                    'opacity': 0
                });
            }
        };
        /** Load images near curret index */
        PhotoSlider.prototype.loadNearbyImages = function () {
            var minBuffer = this.sliderState.queueIndex - PhotoSlider.IMAGE_BUFFER_SIZE;
            var maxBuffer = this.sliderState.queueIndex + PhotoSlider.IMAGE_BUFFER_SIZE;
            for (var i = minBuffer; i < maxBuffer; i++) {
                if (0 <= i && i < this.$images.length) {
                    this.loadImage(this.$images[i]);
                }
            }
        };
        /** Get URL of currently displayed image */
        PhotoSlider.prototype.currentImageURL = function () {
            return this.$images[this.sliderState.queueIndex].data('source');
        };
        /** Returns current photo index */
        PhotoSlider.prototype.getCurrentIndex = function () {
            return this.sliderState.queueIndex;
        };
        /** Returns url for given image's photo */
        PhotoSlider.prototype.getImageUrl = function (index) {
            if (index >= 0 || index < this.$images.length) {
                return this.$images[index].data('source');
            }
            return null;
        };
        /** Returns total number of images */
        PhotoSlider.prototype.getImageCount = function () {
            return this.$images.length;
        };
        /** This method must be called any time the size of PhotoSlider element changes. */
        PhotoSlider.prototype.onWindowResize = function () {
            // Browsers trigger resize differently, some on start & end, others only on start or on end
            // We must be prepare to get called once or twice, before or after rotation
            var _this = this;
            // If I don't relayout immediately
            if (!this.handleWindowResize()) {
                // Wait for screen to change size before trying relayout again
                window.setTimeout(function () {
                    _this.handleWindowResize();
                }, 300);
            }
        };
        /** Bind event string to a callback */
        PhotoSlider.prototype.bind = function (event, handler) {
            this.$el.bind.apply(this.$el, arguments);
        };
        //=============================================================
        // PRIVATE
        //=============================================================
        //---------------------------------------------------------
        // Event Handling
        //---------------------------------------------------------
        /** Set up event handling */
        PhotoSlider.prototype.initEventHandlers = function () {
            this.$el.bind('dragstart', $.proxy(this.onDragStart, this));
            this.$el.bind('drag', $.proxy(this.onDragging, this));
            this.$el.bind('dragend', $.proxy(this.onDragEnd, this));
            this.$el.bind('transformstart', $.proxy(this.onTransformStart, this));
            this.$el.bind('transform', $.proxy(this.onTransforming, this));
            this.$el.bind('transformend', $.proxy(this.onTransformEnd, this));
            this.$el.bind('doubletap', $.proxy(this.onDoubleTap, this));
            this.$el.bind('hold', $.proxy(this.onHold, this));
        };
        /** Called at start of Drag */
        PhotoSlider.prototype.onDragStart = function (e) {
            // Capture initial state
            this.startXPos = this.sliderState.imgXOffset;
            this.startYPos = this.sliderState.imgYOffset;
        };
        /** Called during a Drag */
        PhotoSlider.prototype.onDragging = function (e) {
            // Give event callback the opportunity to handle the event
            if (this.eventCallback && this.eventCallback(e)) {
                return;
            }
            // While animating, ignore drag motions
            if (this.animationState == 1 /* ANIMATING */) {
                return;
            }
            // Note that user has taken control
            this.animationState = 2 /* USER_MOVING */;
            // Adjust horizontal scroll
            this.sliderState.imgXOffset = this.startXPos + e.distanceX;
            // Only allow vertical scroll if image is zoomed and larger than screen height
            if (this.sliderState.imgScale > 1) {
                this.sliderState.imgYOffset = this.startYPos + e.distanceY;
                this.constrainVertically();
            }
            // Move the slide
            this.translateSlide(0, 2 /* USER_MOVING */);
        };
        /** Called at end of Drag */
        PhotoSlider.prototype.onDragEnd = function (e) {
            // Give event callback the opportunity to handle the event
            if (this.eventCallback && this.eventCallback(e)) {
                return;
            }
            // While animating, ignore drag motions
            if (this.animationState == 1 /* ANIMATING */) {
                return;
            }
            // Calculate nearest image based on x position and image overhang
            var nearestImage = this.nearestImage();
            // Check if user attempted to scroll off the end
            if (nearestImage == this.$images.length) {
                // Set back in range
                nearestImage = this.$images.length - 1;
                // Trigger swipe off event
                this.$el.trigger(PhotoSlider.SwipeOffEvent);
            }
            else if (nearestImage < 0) {
                // Set back in range
                nearestImage = 0;
                // Trigger swipe off event
                this.$el.trigger(PhotoSlider.SwipeOffEvent);
            }
            // Center on nearest image if not zoomed in, or if zoomed in and nearest image is not current image
            if (this.sliderState.imgScale == 1 || nearestImage != this.sliderState.queueIndex) {
                this.moveToIndex(nearestImage, 300, 3 /* USER_WAITING */);
            }
            else if (this.sliderState.imgScale > 1) {
                this.constrainHorizontally();
                this.translateSlide(300, 3 /* USER_WAITING */);
            }
            // User has completed moving
            this.onUserDone();
        };
        /* DEBUG Tool for putting text on the screen */
        PhotoSlider.prototype.DebugText = function (text, num, round) {
            if (round === void 0) { round = true; }
            var $debug = this.$el.parent().find('.debugInfo');
            if (!text && !num) {
                $debug.empty();
                return;
            }
            var lineNumber = $debug.length;
            if (lineNumber > 20) {
                $debug.empty();
                lineNumber = 1;
            }
            var $text = $('<div class="debugInfo" style="top: ' + (lineNumber * 22) + 'px;">');
            if (num) {
                if (round) {
                    text = text + ":" + Math.round(num).toString();
                }
                else {
                    text = text + ":" + num.toString();
                }
            }
            $text.text(text);
            $debug.first().append($text);
        };
        /** Called at start of transform (zoom) */
        PhotoSlider.prototype.onTransformStart = function (e) {
            // Capture initial state
            this.startXTouch = (0.5 * this.screenWidth) - (e.touches[0].x + e.touches[1].x) / 2;
            this.startYTouch = (0.5 * this.screenHeight) - (e.touches[0].y + e.touches[1].y) / 2;
            this.startXPos = this.sliderState.imgXOffset;
            this.startYPos = this.sliderState.imgYOffset;
            this.startScale = this.sliderState.imgScale;
        };
        /** Called during a transform (zoom) */
        PhotoSlider.prototype.onTransforming = function (e) {
            // Don't allow scale transforms for non-loaded images
            if (this.$images[this.sliderState.queueIndex].data('state') != 3 /* LOADED */) {
                return;
            }
            // While animating, ignore drag motions
            if (this.animationState == 1 /* ANIMATING */) {
                return;
            }
            // Note that user has taken control
            this.animationState = 2 /* USER_MOVING */;
            // Calculate new scale, no smaller than start scale size
            this.sliderState.imgScale = Math.max(1.0, this.startScale * e.scale);
            // If no change bail (i.e. trying to zoom to less than 1.0)
            if (this.sliderState.imgScale == this.startScale) {
                return;
            }
            // If new scale is over limit
            if (this.sliderState.imgScale > PhotoSlider.MAX_ZOOM) {
                e.scale = PhotoSlider.MAX_ZOOM / this.startScale;
                this.sliderState.imgScale = PhotoSlider.MAX_ZOOM;
            }
            // Calculate new offset position that zoom/pinch from center of fingers
            this.sliderState.imgXOffset = this.startXPos + (e.scale - 1) * this.startXTouch;
            this.sliderState.imgYOffset = this.startYPos + (e.scale - 1) * this.startYTouch;
            // Adjust offset to keep on screen
            this.constrainHorizontally();
            this.constrainVertically();
            // Move slide to new position
            this.translateSlide(0, 2 /* USER_MOVING */);
        };
        /** Called at end of transform (zoom) */
        PhotoSlider.prototype.onTransformEnd = function (e) {
            // While animating, ignore user motions
            if (this.animationState == 1 /* ANIMATING */) {
                return;
            }
            // User has completed moving
            this.onUserDone();
        };
        /** Called when user is done moving (pan/zoom/etc) */
        PhotoSlider.prototype.onUserDone = function () {
            var _this = this;
            //Put slider in waiting state
            this.animationState = 3 /* USER_WAITING */;
            // Set timer to trigger wait is over, clearing any existing timeout
            clearTimeout(this.waitTimeout);
            this.waitTimeout = setTimeout(function () {
                _this.onWaitOver();
            }, PhotoSlider.USER_WAIT_TIME);
            // Notify other screens of the change
            this.triggerSlideChangeEvent();
        };
        /** Called when after user move wait is over */
        PhotoSlider.prototype.onWaitOver = function () {
            // If I'm still in waiting state
            if (this.animationState == 3 /* USER_WAITING */) {
                // Check to see if I have a waiting move request from another user, if so, make that move
                if (this.requestedState) {
                    var duration = this.animationDuration(this.requestedState.queueIndex);
                    this.moveTo(this.requestedState, duration, 0 /* AT_REST */);
                    this.requestedState = null;
                }
                else {
                    this.animationState = 0 /* AT_REST */;
                }
            }
        };
        /** Called on user double tap.  Zooms or unzooms the view */
        PhotoSlider.prototype.onDoubleTap = function (e) {
            // If I'm not zoomed, zoom to fill screen
            if (this.sliderState.imgScale == 1) {
                this.setMaxZoom(true, e.position[0].x, e.position[0].y);
            }
            else {
                this.setMaxZoom(false);
            }
        };
        /** Called on user hold */
        PhotoSlider.prototype.onHold = function (e) {
            // Ignore multiple touch holds, as user is likely zooming
            if (e.touches.length == 1) {
                this.$el.trigger(PhotoSlider.LongPressEvent, e.position);
            }
        };
        /** Trigger slideChangedEvent  */
        PhotoSlider.prototype.triggerSlideChangeEvent = function () {
            // Skip if I have a requested state change as I've temporarily come out of sync with other users
            if (!this.requestedState) {
                // Trigger event
                this.$el.trigger(PhotoSlider.SlideChangedEvent, this.sliderState);
            }
        };
        //---------------------------------------------------------
        // Animation
        //---------------------------------------------------------
        /** Return animation speed to use when moving to new index */
        PhotoSlider.prototype.animationDuration = function (newIndex) {
            // Calculate hom many slides from target location
            var dist = Math.abs(this.sliderState.queueIndex - newIndex);
            // If a single slide away use single slide animation time
            if (dist <= 1) {
                return PhotoSlider.SLIDE_ANIMATION_TIME;
            }
            // Otherwise aim for multi-slide-animation time per slide
            var totalTime = PhotoSlider.MULTI_SLIDE_ANIMATION_TIME * dist;
            // But allow max 2 second total animation time
            totalTime = Math.min(PhotoSlider.MAX_SLIDE_ANIMATION_TIME, totalTime);
            // Now return per slide time
            return Math.round(totalTime / dist);
        };
        /** Return nearest image index for the sliders current position */
        // NOTE: May be outside range of allowed images.  Receiver needs to check (and needs this data)
        PhotoSlider.prototype.nearestImage = function () {
            var offset = -this.sliderState.imgXOffset;
            var threshold = (0.5 * this.screenWidth * (this.sliderState.imgScale - 1)) + (PhotoSlider.IMAGE_MARGIN * 2);
            if (offset > threshold) {
                return this.sliderState.queueIndex + 1;
            }
            else if (offset < -threshold) {
                return this.sliderState.queueIndex - 1;
            }
            return this.sliderState.queueIndex;
        };
        // Returns value of 0-1 used for animating fade/scale of images during slide
        PhotoSlider.prototype.overhang = function (offset) {
            var scale = (this.sliderState.imgScale - 1);
            var threshold = (0.5 * this.screenWidth * scale);
            var margin = (PhotoSlider.IMAGE_MARGIN * 2);
            if (offset > (threshold + margin)) {
                return ((threshold - margin + offset) / (this.screenWidth)) - scale;
            }
            else if (offset < -(threshold + margin)) {
                return ((threshold - margin - offset) / (this.screenWidth)) - scale;
            }
            return 0;
        };
        /** Adjust offsets so image doesn't slide off top or bottom of screen */
        PhotoSlider.prototype.constrainVertically = function () {
            // Calculate min/max screen bounds
            var imageHeight = this.sliderState.imgScale * this.$images[this.sliderState.queueIndex].data('size').height;
            var minY = (imageHeight - this.screenHeight) / 2;
            var maxY = minY + this.screenHeight - imageHeight;
            // If image height is less than screen height, center vertically
            if (imageHeight < this.screenHeight) {
                this.sliderState.imgYOffset = 0.5 * (minY + maxY);
            }
            else {
                this.sliderState.imgYOffset = Math.min(minY, this.sliderState.imgYOffset);
                this.sliderState.imgYOffset = Math.max(maxY, this.sliderState.imgYOffset);
            }
        };
        /** Adjust offsets so image doesn't slide off left or right of screen */
        PhotoSlider.prototype.constrainHorizontally = function () {
            var imageWidth = this.sliderState.imgScale * this.$images[this.sliderState.queueIndex].data('size').width;
            // If image width is less than screen width, center horizontally
            if (imageWidth < this.screenWidth) {
                this.sliderState.imgXOffset = 0;
            }
            else {
                var edge = 0.5 * (imageWidth - this.screenWidth);
                if (this.sliderState.imgXOffset > edge) {
                    this.sliderState.imgXOffset = edge;
                }
                else if (this.sliderState.imgXOffset < -edge) {
                    this.sliderState.imgXOffset = -edge;
                }
            }
        };
        /** Move user to given index */
        PhotoSlider.prototype.moveToIndex = function (index, duration, endState) {
            this.sliderState.queueIndex = index;
            this.sliderState.imgXOffset = 0;
            this.sliderState.imgYOffset = 0;
            this.sliderState.imgScale = 1;
            this.moveTo(this.sliderState, duration, endState);
        };
        /** Move user to requested slider state */
        PhotoSlider.prototype.moveTo = function (sliderState, duration, endState) {
            // Set new state
            this.sliderState = sliderState;
            // Animate to new position
            this.translateSlide(duration, endState);
            // Update flipper visibility
            if (this.useFlippers) {
                this.updateFlippers();
            }
            // Trigger loading of nearby images
            this.loadNearbyImages();
        };
        /** Create a comment object */
        PhotoSlider.prototype.createComment = function (comment) {
            var $comment = $('<div class="photoComment">' + comment + '</div>');
            // Adjust comment position based on orientation
            if (this.screenWidth > this.screenHeight) {
                $comment.addClass("landscape");
            }
            return $comment;
        };
        /** Add image of given index into DOM and return pointer to it */
        PhotoSlider.prototype.insertSlideHolder = function (imageIndex) {
            if (imageIndex < 0 || imageIndex >= this.$images.length) {
                return null;
            }
            // Add new slide holder to slider
            var style = "position: fixed; top: 0px; left: 0px;";
            var $slideHolder = $('<div class="photoHolder" id="holder' + imageIndex + '" style="' + style + '">');
            // If image has a comment, add it
            var comment = client.ImageMetaData.getComment(imageIndex);
            if (comment) {
                $slideHolder.append(this.createComment(comment));
            }
            this.$bar.append($slideHolder);
            // Append image to slide holder
            var $image = this.$images[imageIndex];
            $slideHolder.append($image);
            // Clear background "loading" image if image has been loaded
            if ($image.data('state') == 3 /* LOADED */) {
                $slideHolder.css({ 'background-image': 'none' });
            }
            return $slideHolder;
        };
        /** Update slide position based on sliderState */
        PhotoSlider.prototype.translateSlide = function (slideDuration, postAnimationState, continued) {
            var _this = this;
            if (continued === void 0) { continued = false; }
            // If already Animating and not continuing an animation, don't start a new translation.  It will be caught the at end of running animation
            if (this.animationState == 1 /* ANIMATING */ && !continued) {
                //DEBUG this.DebugText(this.lastQueueIndex.toString() + "-> (skip) ->" + this.sliderState.queueIndex.toString()); // DEBUG
                return;
            }
            // Set animation state, so I don't get interrupted by the user
            if (slideDuration > 0) {
                this.animationState = 1 /* ANIMATING */;
            }
            // Clear sun display upon any animation
            this.showSun(false);
            // Only show comments when not zooming
            if (this.lastScale != this.sliderState.imgScale) {
                var $comments = this.$bar.find(".photoComment");
                if (this.sliderState.imgScale == 1) {
                    $comments.show(100);
                }
                else {
                    $comments.hide(100);
                }
            }
            // Default animation transition style 
            var timing = 'ease';
            var transDuration = slideDuration;
            // Calculate distance of requested move (number of slides)
            var dist = Math.abs(this.sliderState.queueIndex - this.lastQueueIndex);
            // If a continued animation
            if (continued) {
                // If on last frame ease out slowly
                if (dist == 1) {
                    timing = 'ease-out';
                    transDuration = PhotoSlider.SLIDE_ANIMATION_TIME;
                }
                else {
                    timing = 'linear';
                }
            }
            else if (dist > 1) {
                // Ease in slowly on first frame
                timing = 'ease-in';
                transDuration = PhotoSlider.SLIDE_ANIMATION_TIME;
            }
            // Default to not changing in index
            var nextIndex = this.sliderState.queueIndex;
            // If moved to next slide
            if (this.lastQueueIndex < this.sliderState.queueIndex) {
                // Only move one slide at a time
                nextIndex = this.lastQueueIndex + 1;
                // Remove old slide
                $("#holder" + (nextIndex - 2)).detach();
                // Swap slides
                this.prevSlide = (this.curSlide) ? this.curSlide : this.insertSlideHolder(nextIndex - 1);
                this.curSlide = (this.nextSlide) ? this.nextSlide : this.insertSlideHolder(nextIndex);
                this.nextSlide = this.insertSlideHolder(nextIndex + 1);
            }
            // If moved to previous slide
            if (this.lastQueueIndex > this.sliderState.queueIndex) {
                // Only move one slide at a time
                nextIndex = this.lastQueueIndex - 1;
                // Remove old slide from DOM but leave data intact
                $("#holder" + (nextIndex + 2)).detach();
                // Swap slides
                this.nextSlide = (this.curSlide) ? this.curSlide : this.insertSlideHolder(nextIndex + 1);
                this.curSlide = (this.prevSlide) ? this.prevSlide : this.insertSlideHolder(nextIndex);
                this.prevSlide = this.insertSlideHolder(nextIndex - 1);
            }
            //DEBUG: this.DebugText(((continued) ? "+" : "-") + this.lastQueueIndex.toString() + "->" + nextIndex.toString() + "->" + this.sliderState.queueIndex.toString() + " :" + timing + " (" + transDuration.toString()+")");
            //----------------------------
            // Now animate the slides
            //----------------------------
            // CENTER SLIDE
            var xPos = Math.round(this.sliderState.imgXOffset);
            var yPos = Math.round(this.sliderState.imgYOffset);
            var translate = 'translate(' + xPos + 'px,' + yPos + 'px)';
            // Calculate scale based on image position
            var overhang = 1 - this.overhang(xPos);
            var scale = 'scale(' + overhang * this.sliderState.imgScale + ')';
            var opacity = overhang * overhang;
            var durstring = transDuration + "ms";
            this.curSlide.css({
                '-ms-transition-duration': durstring,
                '-webkit-transition-duration': durstring,
                'transition-duration': durstring,
                '-ms-transform': translate + ' ' + scale,
                '-webkit-transform': translate + ' ' + scale,
                'transform': translate + ' ' + scale,
                '-webkit-transition-timing-function': timing,
                'transition-timing-function': timing,
                'opacity': opacity
            });
            // PREVIOUS SLIDE
            if (this.prevSlide) {
                xPos = Math.round(this.sliderState.imgXOffset - (this.stepSize * this.sliderState.imgScale));
                translate = 'translate(' + xPos + 'px,' + yPos + 'px)';
                // If a new slide, set duration to zero to pop slide into place instantly
                var prevDuration = (this.lastQueueIndex > this.sliderState.queueIndex) ? "0ms" : durstring;
                // If used to be the current slide and scaling image on drag
                if (this.lastQueueIndex < this.sliderState.queueIndex) {
                    var overhang = 1 - this.overhang(xPos);
                    scale = 'scale(' + overhang * this.sliderState.imgScale + ')';
                    opacity = overhang * overhang;
                }
                else {
                    scale = 'scale(' + 1 + ')';
                    opacity = 1;
                }
                this.prevSlide.css({
                    '-ms-transition-duration': prevDuration,
                    '-webkit-transition-duration': prevDuration,
                    'transition-duration': prevDuration,
                    '-ms-transform': translate + ' ' + scale,
                    '-webkit-transform': translate + ' ' + scale,
                    'transform': translate + ' ' + scale,
                    '-webkit-transition-timing-function': timing,
                    'trasition-timing-function': timing,
                    'opacity': opacity
                });
            }
            // NEXT SLIDE
            if (this.nextSlide) {
                xPos = Math.round(this.sliderState.imgXOffset + (this.stepSize * this.sliderState.imgScale));
                translate = 'translate(' + xPos + 'px,' + yPos + 'px)';
                // If a new slide, set duration to zero to pop slide into place instantly
                var nextDuration = (this.lastQueueIndex < this.sliderState.queueIndex) ? "0ms" : durstring;
                // If used to be the current slide and scale images on drag
                if (this.lastQueueIndex > this.sliderState.queueIndex) {
                    var overhang = 1 - this.overhang(xPos);
                    //DEBUG: this.DebugText("P:", overhang, false); // DEBUG
                    scale = 'scale(' + overhang * this.sliderState.imgScale + ')';
                    opacity = overhang * overhang;
                }
                else {
                    scale = 'scale(' + 1 + ')';
                    opacity = 1;
                }
                this.nextSlide.css({
                    '-ms-transition-duration': nextDuration,
                    '-webkit-transition-duration': nextDuration,
                    'transition-duration': nextDuration,
                    '-ms-transform': translate + ' ' + scale,
                    '-webkit-transform': translate + ' ' + scale,
                    'transform': translate + ' ' + scale,
                    '-webkit-transition-timing-function': timing,
                    'trasition-timing-function': timing,
                    'opacity': opacity
                });
            }
            // Remember last state
            this.lastQueueIndex = nextIndex;
            this.lastScale = this.sliderState.imgScale;
            // If a non-zero duration requested, we have work to do at the end of the animation
            if (transDuration > 0) {
                setTimeout(function () {
                    // If haven't reached desired index/zoom, queue next slide transition
                    if (_this.lastQueueIndex != _this.sliderState.queueIndex || _this.lastScale != _this.sliderState.imgScale) {
                        _this.translateSlide(slideDuration, postAnimationState, true);
                    }
                    else {
                        // Animate sun in when no chrome
                        if (!_this.photoCastScreen || !_this.photoCastScreen.isChromeVisible()) {
                            _this.showSun(true);
                        }
                        _this.animationState = postAnimationState;
                    }
                }, transDuration);
            }
        };
        //---------------------------------------------------------
        // Image Initialization & Setup
        //---------------------------------------------------------
        /** Render the initial screen container */
        PhotoSlider.prototype.render = function (media) {
            // Clear the element
            this.$el.empty();
            // Create containers for images
            this.$bar = $('<div class="photoSliderBar">');
            // Put in DOM
            this.$el.append(this.$bar);
            // Add images
            this.appendMedia(media);
            // Insert intial slides
            this.prevSlide = this.insertSlideHolder(this.sliderState.queueIndex - 1);
            this.curSlide = this.insertSlideHolder(this.sliderState.queueIndex);
            this.nextSlide = this.insertSlideHolder(this.sliderState.queueIndex + 1);
            this.lastQueueIndex = this.sliderState.queueIndex;
            this.lastScale = this.sliderState.imgScale;
            // Relayout to set sizes
            this.relayout();
            // Add left/right flippers if requested
            if (this.useFlippers) {
                this.addFlippers();
            }
        };
        /** Add left / right flippers to the screen */
        PhotoSlider.prototype.addFlippers = function () {
            var _this = this;
            this.$el.append('<div class="castFlipperLeft">');
            this.$el.append('<div class="castFlipperRight">');
            // Set initial opacity
            this.$el.find('.castFlipperLeft').css('opacity', '0.3');
            this.$el.find('.castFlipperRight').css('opacity', '0.3');
            // Initialize flipper visibility
            this.updateFlippers();
            // Capture click events
            this.$el.on('click touchstart', '.castFlipperLeft', function (e) {
                e.stopPropagation();
                // Block tap event from propagating
                _this.flipperEatTap = true;
                // Flash flippers
                // Clear any existing timeout
                clearTimeout(_this.flipperLeftTimeout);
                // Make full opacity when clicked
                _this.$el.find('.castFlipperLeft').css('opacity', '1.0');
                // Fade after a delay
                _this.flipperLeftTimeout = setTimeout(function () {
                    _this.$el.find('.castFlipperLeft').css('opacity', '0.3');
                }, 300);
                _this.jumpToPreviousSlide();
            });
            this.$el.on('click touchstart', '.castFlipperRight', function (e) {
                e.stopPropagation();
                // Block tap event from propagating
                _this.flipperEatTap = true;
                // Flash flippers
                // Clear any existing timeout
                clearTimeout(_this.flipperRightTimeout);
                // Make full opacity when clicked
                _this.$el.find('.castFlipperRight').css('opacity', '1.0');
                // Fade after a delay
                _this.flipperRightTimeout = setTimeout(function () {
                    _this.$el.find('.castFlipperRight').css('opacity', '0.3');
                }, 300);
                _this.jumpToNextSlide();
            });
            // Eat tap events to prevent hammer from triggering chrome when flipper is touched
            this.$el.on('tap', function (e) {
                if (_this.flipperEatTap) {
                    e.stopPropagation();
                    e.preventDefault();
                }
                _this.flipperEatTap = false;
            });
        };
        /** Update visibility of flippers based on index */
        PhotoSlider.prototype.updateFlippers = function () {
            if (this.sliderState.queueIndex == 0) {
                this.$el.find('.castFlipperLeft').hide();
            }
            else {
                this.$el.find('.castFlipperLeft').show();
            }
            if (this.sliderState.queueIndex == this.$images.length - 1) {
                this.$el.find('.castFlipperRight').hide();
            }
            else {
                this.$el.find('.castFlipperRight').show();
            }
        };
        /** Add media to end of photo slider */
        PhotoSlider.prototype.appendMedia = function (media) {
            if (this.$bar) {
                for (var i = 0; i < media.length; i++) {
                    // Create image element
                    var $image = $('<img class="photoSliderImage"/>');
                    $image.data('state', 0 /* NEW */);
                    $image.data('source', media[i]);
                    this.$images.push($image);
                }
            }
        };
        /** Compose partial state change into complete state */
        PhotoSlider.prototype.composeStateChange = function (changeState) {
            // Change data could be partial (scale / index and offset / or both)
            var scale = ('imgScale' in changeState) ? changeState.imgScale : (this.sliderState.imgScale) ? this.sliderState.imgScale : 1;
            if (('queueIndex' in changeState) && ('imgXOffset' in changeState) && ('imgYOffset' in changeState)) {
                var completeState = {
                    queueIndex: changeState.queueIndex,
                    imgXOffset: changeState.imgXOffset,
                    imgYOffset: changeState.imgYOffset,
                    imgScale: scale
                };
            }
            else {
                completeState = {
                    queueIndex: this.sliderState.queueIndex,
                    imgXOffset: this.sliderState.imgXOffset,
                    imgYOffset: this.sliderState.imgYOffset,
                    imgScale: scale
                };
            }
            return completeState;
        };
        /** Handle window resize.  Return true if relayout occured */
        PhotoSlider.prototype.handleWindowResize = function () {
            var _this = this;
            var sliderPortrait = this.screenWidth < this.screenHeight;
            var windowPortrait = window.innerWidth < window.innerHeight;
            var windowMax = Math.max(window.innerWidth, window.innerHeight);
            var sliderMax = Math.max(this.screenWidth, this.screenHeight);
            var bogusWindow = sliderMax && windowMax && Math.abs(windowMax - sliderMax) > 50;
            var toRelayout = function () {
                if (sliderPortrait != windowPortrait) {
                    client.logger.info('window.innerHeight, innerWidth', window.innerHeight, window.innerWidth);
                    // Resize from rotation takes presidence over other animations
                    if (_this.animationState == 1 /* ANIMATING */) {
                        _this.animationState = 0 /* AT_REST */;
                    }
                    // Do the relayout
                    _this.relayout();
                }
            };
            if (bogusWindow) {
                _.delay(toRelayout, 300);
            }
            else {
                toRelayout();
            }
            return sliderPortrait != windowPortrait;
        };
        /** Relayout screen elements based on current screen size and rotation */
        PhotoSlider.prototype.relayout = function () {
            // Recalculate screen size
            this.calcScreenSize();
            for (var i = 0; i < this.$images.length; i++) {
                this.$images[i].data('scaled', false);
            }
            // Triggers immediate scaling of nearby images
            this.loadNearbyImages();
            // Adjust comment position based on orientation
            var $comments = this.$bar.find(".photoComment");
            if (this.screenWidth > this.screenHeight) {
                $comments.addClass("landscape");
            }
            else {
                $comments.removeClass("landscape");
            }
            // Center on nearest image
            this.moveToIndex(this.sliderState.queueIndex, 0, 0 /* AT_REST */);
        };
        /** Initialize screen width/height */
        PhotoSlider.prototype.calcScreenSize = function () {
            this.screenWidth = window.innerWidth;
            this.screenHeight = window.innerHeight;
            // Todo: not sure why this is here as it missing comment, but dont' want o break anything.
            if (!this.screenWidth || !this.screenHeight) {
                var vp = client.webview.currentViewport();
                this.screenWidth = this.screenWidth || vp.width;
                this.screenHeight = this.screenHeight || vp.height;
            }
            // Todo: needs comment to explain what is going on here
            if (client.capability.isJellybean) {
                // May get physical resolution here make sure we use the the viewport resolution
                var viewportSize = client.webview.scaleToViewport({ width: this.screenWidth, height: this.screenHeight });
                this.screenWidth = viewportSize.width;
                this.screenHeight = viewportSize.height;
            }
            // Update step size 
            this.stepSize = this.screenWidth + PhotoSlider.IMAGE_MARGIN;
        };
        /** Load an image and scale to screen as needed */
        PhotoSlider.prototype.loadImage = function ($image) {
            // Get current state of this image
            var state = $image.data('state');
            // Image has never been loaded
            // -> Set src to trigger the load
            if (state == 0 /* NEW */) {
                //logger.debug("ImageState: NEW");
                $image.attr('src', $image.data('source'));
                this.waitForImagesToLoad($image, _.bind(this.onImageLoaded, this));
                return;
            }
            else if (state == 1 /* WAITING */) {
                //logger.debug("ImageState: WAITING " + $image.data('reloadTimeout'));
                return;
            }
            else if (state == 2 /* RETRY */) {
                //logger.debug("ImageState: RETRY " + $image.data('reloadTimeout'));
                // Add query parameter to source URL to force browser to reload from same URL
                var newSource = $image.data('source') + "&ximtimeout=" + $image.data('reloadTimeout');
                // Set new src to trigger the load
                $image.attr('src', newSource);
                // Show image again
                $image.show();
                this.waitForImagesToLoad($image, _.bind(this.onImageLoaded, this));
                return;
            }
            else if (state == 4 /* ERROR */) {
                //logger.debug("ImageState: ERROR " + $image.data('reloadTimeout'));
                // Set background "fail" image 
                $image.parent().addClass("photoHolderFailed");
                return;
            }
            else if (state == 3 /* LOADED */) {
                //logger.debug("ImageState: LOADED " + ($image.data('reloadTimeout')||""));
                if (!$image.data('scaled')) {
                    this.scaleImage($image);
                }
            }
        };
        /** Callback for when image has been loaded */
        PhotoSlider.prototype.onImageLoaded = function ($image) {
            // If the image loaded correctly
            if ($image.data('state') == 3 /* LOADED */) {
                // Clear background "loading" image
                $image.parent().css({ 'background-image': 'none' });
                // Scale the image
                this.scaleImage($image);
            }
        };
        /** Returns the scale for the given image that will fill the screen */
        PhotoSlider.prototype.fullScreenScale = function ($image) {
            var size = this.$images[this.sliderState.queueIndex].data('size');
            if (this.screenWidth > this.screenHeight) {
                return Math.min(PhotoSlider.MAX_ZOOM, (this.screenWidth / size.width));
            }
            else {
                return Math.min(PhotoSlider.MAX_ZOOM, (this.screenHeight / size.height));
            }
        };
        /** Centers supplied image inside its parent element (image must already be loaded) */
        PhotoSlider.prototype.scaleImage = function ($image) {
            // Get scales in each dimension
            var img = $image[0];
            var scaleX = img.naturalWidth / this.screenWidth;
            var scaleY = img.naturalHeight / this.screenHeight;
            // Pick best fitting scale, but do not scale up
            var scale = Math.max(1, Math.max(scaleX, scaleY));
            // Calculate dimensions
            var width = Math.round(img.naturalWidth / scale);
            var height = Math.round(img.naturalHeight / scale);
            // Calculate padding so image is vertically centered
            var paddingY = Math.round((this.screenHeight - height) / 2);
            var paddingX = Math.round((this.screenWidth - width) / 2);
            $image.css({
                width: width,
                height: height,
                'margin-bottom': paddingY,
                'margin-top': paddingY,
                'margin-left': paddingX,
                'margin-right': paddingX
            });
            // Store data to avoid recalculation later
            jQuery.data($image[0], "size", { height: height, width: width });
            // Store that I've been scaled
            jQuery.data($image[0], "scaled", true);
        };
        /** When supplied images have all either loaded or errored out, calls the supplied callback function. */
        PhotoSlider.prototype.waitForImagesToLoad = function ($images, callback) {
            var _this = this;
            // Clear any existing bindings
            $images.unbind('load');
            $images.unbind('error');
            var allDone = false;
            var checkAllDone = function () {
                if (allDone) {
                    return;
                }
                for (var i = 0; i < $images.length; i++) {
                    var image = $images[i];
                    var $image = $(image);
                    // If there was a problem loading the image
                    if ($image.data('state') == 4 /* ERROR */) {
                        // Hide image so failed image icon doesn't show
                        $image.hide();
                        // Get reload delay (will increase each time we try to reload this image)
                        var delay = 2 * ($image.data('reloadTimeout') || PhotoSlider.INITIAL_RETRY_TIME);
                        // Give up retrying after a maximum amount of time
                        if (delay <= PhotoSlider.MAX_RETRY_TIME) {
                            // Put image in WAITING state, so no one else tries to load it
                            $image.data('state', 1 /* WAITING */);
                            // Update reload timeout for exponential backoff
                            $image.data('reloadTimeout', delay);
                            // Try to reload the image after the delay
                            setTimeout(function () {
                                $image.data('state', 2 /* RETRY */);
                                // Before loading check that PhotoSlider is still in use
                                if (!_this.isDisposed) {
                                    _this.loadImage($image);
                                }
                            }, delay);
                        }
                    }
                    else {
                        // If complete w/o error mark as loaded
                        if (image.complete) {
                            $image.data('state', 3 /* LOADED */);
                        }
                        else {
                            break;
                        }
                    }
                }
                if (i == $images.length && i > 0) {
                    allDone = true;
                    $images.unbind('load');
                    $images.unbind('error');
                    callback($images);
                }
            };
            $images.bind('load', function () {
                checkAllDone();
            });
            $images.bind('error', function (ev) {
                $(ev.target).data('state', 4 /* ERROR */);
                checkAllDone();
            });
            // If the image has already been loaded (or errored out) before we set the handlers then we won't see those events.
            // Set the src attribute again to make load/error fire.
            $.each($images, function (i, v) {
                var $img = $(v);
                $img.attr('src', $img.attr('src'));
            });
            checkAllDone();
        };
        /** Number of images to load on each side of current displayed image */
        PhotoSlider.IMAGE_BUFFER_SIZE = 5;
        /** Maximum that user is allowed to zoom */
        PhotoSlider.MAX_ZOOM = 5;
        /** Margin between images */
        PhotoSlider.IMAGE_MARGIN = 40;
        /** Initial time to wait before retrying an image load.  After that doubles on each retry */
        PhotoSlider.INITIAL_RETRY_TIME = 90;
        /** Maxium time to allow before giving up on retries. Just over 3 minutes, allows for 12 retry attempts */
        PhotoSlider.MAX_RETRY_TIME = 184320;
        /** How long to wait to follow other screen after user has taken their own zoom or swipe action */
        PhotoSlider.USER_WAIT_TIME = 3000;
        /** How long should animation between two slides be in ms */
        PhotoSlider.SLIDE_ANIMATION_TIME = 300;
        /** How long should animation across multiple slides be in ms */
        PhotoSlider.MULTI_SLIDE_ANIMATION_TIME = 150;
        /** Maximum time to spend animating across multiple slides in ms */
        PhotoSlider.MAX_SLIDE_ANIMATION_TIME = 2000;
        // Events
        PhotoSlider.SlideChangedEvent = 'slideChanged'; // LARS check on usage
        PhotoSlider.LongPressEvent = 'longpress';
        PhotoSlider.DoubleTapEvent = 'doubletap';
        PhotoSlider.SwipeOffEvent = 'swipeoff'; // Triggered when user swipes off end of slideshow
        return PhotoSlider;
    })();
    client.PhotoSlider = PhotoSlider;
})(client || (client = {}));
/// <reference path="../types.ts"/>
/// <reference path='../DeviceContacts.ts'/>
/// <reference path='../colorCycle.ts'/>
/// <reference path="../../../types/jquery/jquery.d.ts"/>
/// <reference path="../imageMetaData.ts"/>
var client;
(function (client) {
    /** A Message Bubble */
    var Bubble = (function () {
        function Bubble($el, bubbleMessage, immediate) {
            if (immediate === void 0) { immediate = false; }
            // Save base element
            Bubble.$el = $el;
            if (immediate) {
                Bubble.queueBubble.remove();
                Bubble.queueBubble = null;
                Bubble.showMessage(bubbleMessage);
            }
            else {
                // Add to message queue
                Bubble.bubbleQueue.unshift(bubbleMessage);
                // If no bubble being displayed, show message
                if (Bubble.queueBubble == null) {
                    Bubble.ShowFromQueue();
                }
            }
        }
        Bubble.getRenderData = function (message) {
            if (!client.user.profile || !message.profile) {
                client.logger.error(consts.errorCode.Timeline, 'GetRenderData: profile undefined.');
                return null;
            }
            // Protect against missing profile color
            var profileColor = (message.profile.color ? message.profile.color : client.ColorCycle.GetNext());
            var boxColor = profileColor;
            var boxStyle = "";
            var backColor = "#FFFFFF";
            var sunUrl = null;
            if (message.isSystem) {
                boxStyle = "System";
                boxColor = "#555555";
                backColor = boxColor;
            }
            else if (client.user.profile.userId == message.profile.userId) {
                boxStyle = "Sent";
                sunUrl = (message.hasSun) ? 'img/sunL' + client.ImageMetaData.getSuns(message.imageId, message.profile.userId) + '.png' : null;
            }
            else {
                boxStyle = "Received";
                sunUrl = (message.hasSun) ? 'img/sunR' + client.ImageMetaData.getSuns(message.imageId, message.profile.userId) + '.png' : null;
            }
            // Get image ulr if associated image
            var imageURL = (message.imageId !== null && Bubble.getImageUrlCallback) ? Bubble.getImageUrlCallback(message.imageId) : null;
            var data = {
                text: message.text,
                boxStyle: boxStyle,
                boxColor: boxColor,
                backColor: backColor,
                profileID: message.profile.userId,
                sunUrl: sunUrl,
                thumbnailUrl: message.profile.thumbnailUrl || null,
                imageUrl: imageURL,
                initials: client.DeviceContacts.initials(message.profile),
                profileColor: profileColor
            };
            return data;
        };
        /* Hides any message currently being displayed */
        Bubble.hideMessage = function () {
            if (Bubble.queueBubble) {
                Bubble.queueBubble.remove();
                Bubble.queueBubble = null;
                Bubble.ShowFromQueue();
            }
        };
        Bubble.showMessage = function (bubbleMessage) {
            if (bubbleMessage) {
                // Add to DOM
                Bubble.queueBubble = $('<div class="bubbleBox bubbleFadeInOut"></div>');
                Bubble.$el.append(Bubble.queueBubble);
                var bubbleRenderData = Bubble.getRenderData(bubbleMessage);
                if (!bubbleRenderData) {
                    return;
                }
                Bubble.queueBubble.render('messageBubble', { data: bubbleRenderData, animate: false });
                Bubble.queueBubble.find(".messageBlock").css({ width: Bubble.messageBlockWidth + 'px' });
                Bubble.queueBubble.find(".messageBoxHolder").css({ width: Bubble.messageBoxWidth + 'px' });
                if (bubbleMessage.callback) {
                    Bubble.queueBubble.on('click', function (event) {
                        // Only callback on non-system messages
                        if (!bubbleMessage.isSystem) {
                            event.preventDefault();
                            event.stopPropagation();
                            bubbleMessage.callback(bubbleMessage.imageId);
                        }
                    });
                }
                // Remove bubble after animation is over, display next in queue
                setTimeout(function () {
                    Bubble.hideMessage();
                }, 8000); // Note: Adjust transition time in .less file too
            }
        };
        Bubble.ShowFromQueue = function () {
            // Grab next message from the queue
            var bubbleMessage = Bubble.bubbleQueue.pop();
            this.showMessage(bubbleMessage);
        };
        Bubble.messageBlockWidthMax = 700;
        Bubble.messageBubbleWidth = 100;
        Bubble.messageBlockWidth = 0;
        Bubble.messageBoxWidth = 0;
        Bubble.bubbleOffset = 42;
        Bubble.baseTop = 10;
        // Bubble Slot
        Bubble.bubbleSlot = []; // Slots for slot view
        // Bubble Queue 
        Bubble.queueBubble = null; // Bubble in use in queue
        Bubble.bubbleQueue = []; // Queue for queue view
        Bubble.$el = null; // Base element for queue
        Bubble.getImageUrlCallback = null;
        return Bubble;
    })();
    client.Bubble = Bubble;
})(client || (client = {}));
var client;
(function (client) {
    /** Exposes methods for working with a group. */
    var FigureLayout = (function () {
        function FigureLayout() {
            // Public parameters with defaults
            this.marginSide = 20;
            this.marginTop = 20;
            this.marginImage = 10;
            this.rowHeight = 166;
            this.colCount = 4;
            this.useCaption = true;
            // Private parameters
            this.stepSize = null;
        }
        FigureLayout.prototype.setWidth = function (width) {
            width = width - (2 * this.marginSide);
            this.stepSize = width / this.colCount;
        };
        FigureLayout.prototype.getPosition = function (index) {
            var hpos = index % this.colCount;
            var vpos = Math.floor(index / this.colCount);
            var left = this.marginSide + (this.stepSize * hpos);
            var top = this.marginTop + (this.rowHeight * vpos);
            var imageSize = this.stepSize - this.marginImage;
            return { left: left, top: top, width: this.stepSize, imageSize: imageSize, height: this.rowHeight };
        };
        // Give the number of items in the layout return the position of the bottom of the layout
        FigureLayout.prototype.getBottomPosition = function (numItems) {
            var vpos = Math.floor(numItems / this.colCount) + 1;
            return this.marginTop + (this.rowHeight * vpos);
        };
        return FigureLayout;
    })();
    client.FigureLayout = FigureLayout;
})(client || (client = {}));
/// <reference path='types.ts'/>
/// <reference path='colorCycle.ts'/>
/// <reference path='util.ts'/>
/// <reference path="FigureLayout.ts"/>.ast
/// <reference path="deviceContacts.ts"/>
var client;
(function (client) {
    /** Creates a DOM object containing a grid a profiles */
    var ProfileGrid = (function () {
        function ProfileGrid() {
        }
        ProfileGrid.colorSVG = function (color) {
            return "data:image/svg+xml;charset=utf-8;base64," + window.btoa("<svg xmlns='http://www.w3.org/2000/svg' width='100' height='100'><rect fill='" + color + "' x='0' y='0' width='100%' height ='100%'/></svg>");
        };
        ProfileGrid.render = function (
            // Where to render the profiles
            $selector, 
            // Set of profiles to render
            profiles, 
            // Defines layout of grid
            layout, 
            // Callback when clicked
            clickCallback, 
            // Optional callback for getting selection state
            selectedCallack, 
            // Optional Index to start rendering
            startIndex, 
            // Optional caption color
            defaultCaptionColor) {
            if (selectedCallack === void 0) { selectedCallack = null; }
            if (startIndex === void 0) { startIndex = 0; }
            if (defaultCaptionColor === void 0) { defaultCaptionColor = "#666666"; }
            for (var i = 0; i < profiles.length; i++) {
                var position = layout.getPosition(startIndex++);
                var profile = profiles[i];
                if (client.user.GetSetting(2 /* ENABLE_PRESENTER_MODE */)) {
                    if (profile['presenting']) {
                        position.imageSize = 1.2 * position.imageSize;
                        position.top = position.top - 10;
                    }
                }
                // Skip profiles with no ID (used for leaving space)
                if (profile.userId == null) {
                    continue;
                }
                // Check that first / last name have been calculated
                var displayNames = client.DeviceContacts.displayNames(profile);
                // Use symbols if profile doesn't have a thumbnail
                var initials = client.DeviceContacts.initials(profile);
                var captionColor = (client.user.GetSetting(8 /* HIGHLIGHT_NEARBY */) && profile.isNearby) ? '#FF0000' : defaultCaptionColor;
                // Could be rendering device objects (pre-cast) or service objects (post-cast)
                var id = (profile.userId) ? profile.userId : profile.deviceId;
                var $content = $("<div>").render('person', {
                    image: profile.thumbnailUrl || ProfileGrid.colorSVG(profile.color),
                    initials: initials,
                    caption1: displayNames.first,
                    useCaption: layout.useCaption,
                    captionColor: captionColor,
                    position: position,
                    id: client.Util.toAlphaNumeric(id),
                    isSelected: (selectedCallack) ? selectedCallack(profile) : false,
                    backgroundColor: profile.color,
                    presenting: profile['presenting']
                });
                $content.on('click', { profile: profile }, clickCallback);
                $selector.append($content);
            }
        };
        return ProfileGrid;
    })();
    client.ProfileGrid = ProfileGrid;
})(client || (client = {}));
/// <reference path="PopUp.ts"/>
/// <reference path="../types.ts"/>
/// <reference path="../../../types/jquery/jquery.d.ts"/>
var client;
(function (client) {
    /** Extends a PopUp to show profiles when bubble heads are clicked */
    var PopUpWithProfile = (function (_super) {
        __extends(PopUpWithProfile, _super);
        function PopUpWithProfile() {
            _super.apply(this, arguments);
            this.viewingProfile = null;
            this.profilePopUp = null;
        }
        PopUpWithProfile.prototype.onProfileClick = function (event) {
            var profile = event.data.profile;
            event.preventDefault();
            event.stopPropagation();
            this.showProfile(profile);
        };
        // Update the currently show profile as data may have changed
        PopUpWithProfile.prototype.updateProfile = function () {
            if (this.viewingProfile) {
                this.hideProfile();
                this.showProfile(this.viewingProfile);
            }
        };
        PopUpWithProfile.prototype.hideProfile = function () {
            if (this.viewingProfile) {
                this.viewingProfile = null;
                this.profilePopUp.destroy();
                this.profilePopUp = null;
            }
        };
        PopUpWithProfile.prototype.showProfile = function (profile) {
            this.viewingProfile = profile;
            var $popUpContent = this.$el.find('.profilePopUpContent');
            this.profilePopUp = new client.ProfilePopUp($popUpContent, profile);
            this.profilePopUp.show(true);
        };
        // Called when window is resize
        PopUpWithProfile.prototype.resize = function () {
            if (this.profilePopUp) {
                this.profilePopUp.resize();
            }
        };
        // Overwrite base class to close any open profilePopUp
        PopUpWithProfile.prototype.show = function (show) {
            if (this.profilePopUp) {
                this.hideProfile();
            }
            _super.prototype.show.call(this, show);
        };
        return PopUpWithProfile;
    })(client.PopUp);
    client.PopUpWithProfile = PopUpWithProfile;
})(client || (client = {}));
/// <reference path="../types.ts"/>
/// <reference path="../../../types/jquery/jquery.d.ts"/>
/// <reference path="PopUp.ts"/>
var client;
(function (client) {
    /** User interface for showing a user profile */
    var ProfilePopUp = (function (_super) {
        __extends(ProfilePopUp, _super);
        function ProfilePopUp($el, profile) {
            var _this = this;
            // Remove from DOM on closing
            this.destroyOnClose = true;
            // Ensure first/last name has been extracted
            var splitName = client.DeviceContacts.splitName(profile);
            profile.firstName = splitName.first;
            profile.lastName = splitName.last;
            // Use symbols if profile doesn't have a thumbnail
            profile['initials'] = client.DeviceContacts.initials(profile);
            // Call superclass to render Popup
            _super.call(this, $el, "profilePopUp", profile, { overlayOpacity: 0.01 });
            // Close on click, and eat other clicks so they don't propagate upward 
            this.$popup.on('click', function (event) {
                if (_this.isVisible()) {
                    event.preventDefault();
                    event.stopPropagation();
                    _this.show(false);
                }
            });
            // Size to center in screen
            this.resize();
        }
        // Center on screen (takes into account that it may have rotated)
        ProfilePopUp.prototype.resize = function () {
            var screenWidth = client.webview.currentViewport().width;
            var screenHeight = client.webview.currentViewport().height;
            var x = (screenWidth - this.$popup.width()) / 2;
            var y = (screenHeight - this.$popup.height()) / 2;
            this.$popup.css({ marginLeft: Math.round(x) + 'px', marginTop: Math.round(y) + 'px' });
        };
        return ProfilePopUp;
    })(client.PopUp);
    client.ProfilePopUp = ProfilePopUp;
})(client || (client = {}));
/// <reference path="../types.ts"/>
/// <reference path="../FigureLayout.ts"/>
/// <reference path="../ProfileGrid.ts"/>
/// <reference path="../Group.ts"/>
/// <reference path="PopUpWithProfile.ts"/>
/// <reference path="ProfilePopUp.ts"/>
var client;
(function (client) {
    /** User interface for showing who is in a Xim */
    var PeoplePopUp = (function (_super) {
        __extends(PeoplePopUp, _super);
        function PeoplePopUp($el, cast, addCallback) {
            var _this = this;
            // Call superclass to render Popup
            _super.call(this, $el, "peoplePopUp", { resources: resources.forms.peoplePopUp });
            this.cast = cast;
            this.addCallback = addCallback;
            this.layout = null;
            this.presenterID = null;
            this.groupPresence = null;
            //--------------------
            // Handle user events
            //--------------------
            this.$el.on('click', '.peoplePopUpButtonAdd', function (evt) {
                evt.preventDefault();
                evt.stopPropagation();
                client.AppMasterScreen.checkConnection(function () {
                    _this.addCallback();
                });
            });
            // close the popup if user click on the background
            this.$el.on('click', '.peoplePopUp', function (evt) {
                evt.preventDefault();
                evt.stopPropagation();
                _this.show(false);
            });
        }
        // Called to rerender when data has changed 
        PeoplePopUp.prototype.updatePresence = function (groupPresence) {
            this.groupPresence = groupPresence;
            this.renderPresence();
        };
        // Called to rerender presence information
        PeoplePopUp.prototype.renderPresence = function () {
            var _this = this;
            if (this.groupPresence) {
                // Clear all presence marks
                this.$popup.find(".figuremark").removeClass("presenceMark");
                // Add back those that are present
                var presentCount = 0;
                this.groupPresence.each(function (model, index) {
                    if (model.id.indexOf(PeoplePopUp.PRESENCE_KEY) >= 0) {
                        presentCount++;
                        // Extract user ID
                        var profileId = client.Util.toAlphaNumeric(model.id.substring(PeoplePopUp.PRESENCE_KEY.length, model.id.length));
                        _this.$popup.find('#' + profileId + " .figuremark").addClass("presenceMark");
                    }
                    client.logger.debug(model.id);
                });
                // Update people count
                this.$el.find(".peoplePopUpCount").text(resources.forms.peoplePopUp.people + " - " + presentCount.toString() + " " + resources.forms.peoplePopUp.present);
            }
        };
        // Called to rerender when data has changed (such as who is presenter)
        PeoplePopUp.prototype.updateDisplay = function () {
            // Only redraw is something changed
            if (this.presenterID != this.cast.state.presenterID) {
                this.render();
                this.updateProfile();
            }
        };
        // Called when window is resize
        PeoplePopUp.prototype.resize = function () {
            this.render();
            _super.prototype.resize.call(this);
        };
        // Overwrite baseclass for custom rendering
        PeoplePopUp.prototype.render = function () {
            var screenWidth = client.webview.currentViewport().width;
            var screenHeight = client.webview.currentViewport().height;
            // Create layout object
            this.layout = new client.FigureLayout();
            this.layout.useCaption = false;
            this.layout.colCount = (screenWidth > screenHeight) ? 7 : 5;
            this.layout.rowHeight = 100;
            this.layout.marginTop = 200;
            this.layout.setWidth(screenWidth);
            // Clear previous content
            this.$popup.find(".peoplePopProfiles").empty();
            // Update presenterID
            this.presenterID = this.cast.state.presenterID;
            // Genereate profile list
            var profiles = [];
            for (var id in this.cast.group.profile.members) {
                var profile = this.cast.group.profile.members[id];
                // If using presenter mode, highlight the presenter
                if (client.user.GetSetting(2 /* ENABLE_PRESENTER_MODE */)) {
                    if (profile.userId == this.presenterID) {
                        profile['presenting'] = true;
                    }
                    else {
                        profile['presenting'] = false;
                    }
                }
                profiles.push(profile);
            }
            // Render profiles
            client.ProfileGrid.render(this.$popup.find(".peoplePopProfiles"), profiles, this.layout, _.bind(this.onProfileClick, this));
            if (client.startupMode != 3 /* wp */) {
                this.$popup.css({ height: screenHeight + 'px', width: screenWidth + 'px' });
            }
            else {
                this.$popup.css({ height: screenHeight + 'px' });
            }
            // Update presence display
            this.renderPresence();
        };
        PeoplePopUp.PRESENCE_KEY = "presence:";
        return PeoplePopUp;
    })(client.PopUpWithProfile);
    client.PeoplePopUp = PeoplePopUp;
})(client || (client = {}));
/// <reference path="../types.ts"/>
/// <reference path="../resources.ts"/>
/// <reference path="../castViewManager.ts"/>
/// <reference path="../../../types/jquery/jquery.d.ts"/>
/// <reference path="PopUp.ts"/>
var client;
(function (client) {
    /** User interface for choosing a castViewer */
    var CastViewerChoicePopUp = (function (_super) {
        __extends(CastViewerChoicePopUp, _super);
        function CastViewerChoicePopUp(closeCallback) {
            var _this = this;
            // Call superclass to render Popup
            _super.call(this, null, "castViewerChoicePopUp", {}, { overlayOpacity: 0.7 });
            this.closeCallback = closeCallback;
            if (closeCallback) {
                this.onClose.add(closeCallback);
            }
            // Start scan for viewers
            client.CastViewManager.getCastViewers(_.bind(this.onChoiceChanged, this));
            // Register for screen rotation for resizing
            window.addEventListener('orientationchange', _.bind(this.resize, this));
            // Turn off scan animation after a delay
            setTimeout(function () {
                _this.$popup.find('.viewerChoiceWait').hide();
                _this.$popup.find('.viewerChoiceTitle').text(resources.messages.castViewChoice.title);
            }, 15000);
        }
        /** Called when popup is destroyed */
        CastViewerChoicePopUp.prototype.onDispose = function () {
            // Stop receiving new cast viewers
            client.CastViewManager.stopScans();
        };
        /** Called when available castViewers has changed */
        CastViewerChoicePopUp.prototype.onChoiceChanged = function (castViewers) {
            var _this = this;
            // Generate render data
            var data = _.map(castViewers, function (castViewer) {
                return {
                    id: castViewer.id,
                    label: castViewer.displayName,
                    glyph: castViewer.icon
                };
            });
            // Empty existing popup and event handlers
            this.$popup.empty();
            this.$popup.off('click');
            // Render
            this.$popup.render("castViewerChoice", { resources: resources.messages.castViewChoice, castViewers: data });
            // Attach viewers to elements
            var activeViewer = client.CastViewManager.getActiveViewer();
            for (var i = 0; i < castViewers.length; i++) {
                var $selector = this.$popup.find('#viewer_' + i);
                $selector.data('viewer', castViewers[i]);
                // Highlight active viewer (Note: compare by ID not object) 
                if (activeViewer && activeViewer.id == castViewers[i].id) {
                    $selector.addClass("castViewerSelected");
                }
            }
            // Close on click and toggle selection 
            this.$popup.on('click', '.viewerChoiceBox', _.once(
            // click should only be called once per pop up
            function (event) {
                event.preventDefault();
                // Trigger view manager toggle
                var $cur = $(event.currentTarget);
                var selectedViewer = $cur.data('viewer');
                // update selectedness highlight
                $cur.toggleClass("castViewerSelected");
                client.CastViewManager.toggle(selectedViewer, function () {
                    _this.destroy();
                });
            }));
            // Show automatically
            this.show(true);
            // Size to center in screen
            this.resize();
        };
        // Center on screen (takes into account that it may have rotated)
        CastViewerChoicePopUp.prototype.resize = function () {
            var screenSize = client.webview.getScreenSize();
            var x = (screenSize.width - this.$popup.width()) / 2;
            var maxHeight = screenSize.height - 200;
            this.$popup.css({ marginLeft: Math.round(x) + 'px', marginTop: '100px', 'max-height': maxHeight });
        };
        return CastViewerChoicePopUp;
    })(client.PopUp);
    client.CastViewerChoicePopUp = CastViewerChoicePopUp;
})(client || (client = {}));
/// <reference path="../types.ts"/>
/// <reference path="../CastViewManager.ts"/>
/// <reference path="../controls/CastViewerChoicePopUp.ts"/>
/// <reference path="../../../types/jquery/jquery.d.ts"/>
var client;
(function (client) {
    /** An icon for viewing / changing castViewer state */
    var CastViewerIcon = (function () {
        function CastViewerIcon(selector, iconClass) {
            var _this = this;
            this.selector = selector;
            this.iconClass = iconClass;
            this.viewerChoice = null;
            this.$el = $('<div class="castViewerIcon"></div>');
            $(selector).append(this.$el);
            // Initialize the icon
            this.updateIcon();
            // Register for castViewer changes to update displayed icon
            client.CastViewManager.onCastViewerChanged.add(_.bind(this.updateIcon, this));
            // Capture click
            this.$el.on('click', function (e) {
                e.preventDefault();
                // If there is active viewer show disconnect popup
                if (client.CastViewManager.activeViewer) {
                    var confirmationPopUp = new client.ConfirmationPopUp(null, resources.messages.disableCastViewer.title, resources.messages.disableCastViewer.message, resources.messages.disableCastViewer.choiceLeft, resources.messages.disableCastViewer.choiceRight, function (choice) {
                        // Turn off viewer
                        if (choice == resources.messages.disableCastViewer.choiceRight) {
                            client.CastViewManager.toggle(client.CastViewManager.activeViewer, function () {
                            });
                        }
                    });
                }
                else if (!_this.viewerChoice) {
                    _this.viewerChoice = new client.CastViewerChoicePopUp(function () {
                        // Clear when dialog closed
                        _this.viewerChoice = null;
                    });
                }
            });
        }
        CastViewerIcon.prototype.updateIcon = function () {
            // Get the appropriate icon
            var glyph = client.CastViewManager.getActiveIcon();
            // Update the view
            this.$el.removeClass();
            this.$el.addClass("castViewerIcon " + this.iconClass + " " + glyph);
        };
        /** Show or hide the icon */
        CastViewerIcon.prototype.show = function (show) {
            if (show) {
                this.$el.fadeIn();
            }
            else {
                this.$el.fadeOut();
            }
        };
        return CastViewerIcon;
    })();
    client.CastViewerIcon = CastViewerIcon;
})(client || (client = {}));
/// <reference path="../types.ts"/>
/// <reference path="../../../types/jquery/jquery.d.ts"/>
/// <reference path="CastViewerIcon.ts"/>
var client;
(function (client) {
    (function (ChromeTopBarState) {
        ChromeTopBarState[ChromeTopBarState["hidden"] = 0] = "hidden";
        ChromeTopBarState[ChromeTopBarState["open"] = 1] = "open";
    })(client.ChromeTopBarState || (client.ChromeTopBarState = {}));
    var ChromeTopBarState = client.ChromeTopBarState;
    /** Upsell chrome that slides down from top */
    var ChromeTopBar = (function () {
        function ChromeTopBar(selector, autohideduration, upsellUrl) {
            var _this = this;
            this.upsellUrl = upsellUrl;
            /** Current state of the upsell bar. */
            this.state = 0 /* hidden */;
            this.$el = $(selector);
            this.autohideduration = autohideduration;
            // If passed an upsell URL, show upsell on top chrome
            if (this.upsellUrl) {
                this.$el.empty().render('upsellBar', { text: resources.messages.upsell });
            }
            else if (client.isDesktopDisplay) {
                return;
            }
            else {
                this.$el.empty().render('chromeTopBar', { text: resources.messages.upsell });
                // Add viewer icon
                var iconViewer = new client.CastViewerIcon(this.$el.find(".chromeTopBase"), "castscreen");
                // Enable back button if on iOS
                if (client.startupMode == 4 /* ios */) {
                    this.$el.find('.castBackArrow').show();
                    this.$el.on('click', '.castBackArrow', function () {
                        history.back();
                    });
                }
                else {
                    this.$el.find('.castBackArrow').hide();
                }
            }
            this.$el.on('click', function (e) {
                e.preventDefault();
                // If showing upsell message navigate to getXim page
                if (_this.upsellUrl) {
                    client.logger.collectNamedEvent(consts.flurryEvent.ximsessionUpsell);
                    window.open(_this.upsellUrl, '_system');
                }
            });
        }
        /** Returns true if the upsell bar is currently open. */
        ChromeTopBar.prototype.isOpen = function () {
            return this.state == 1 /* open */;
        };
        /** Shows or hides the upsell bar. */
        ChromeTopBar.prototype.show = function (visible, fadeout) {
            var _this = this;
            if (fadeout === void 0) { fadeout = false; }
            // Don't show if a desktop display and no upsell
            if (!this.upsellUrl && client.isDesktopDisplay) {
                return;
            }
            // Clear any previously set timer
            window.clearTimeout(this.fadeTimer);
            if (visible) {
                this.$el.find('.chromeTopBase').removeClass("chromeTopClosed").addClass("chromeTopOpen");
                this.state = 1 /* open */;
                // Close after duration 
                if (fadeout) {
                    // Automatically hides after a while
                    this.fadeTimer = setTimeout(function () {
                        _this.show(false);
                    }, this.autohideduration);
                }
            }
            else {
                this.$el.find('.chromeTopBase').removeClass("chromeTopOpen").addClass("chromeTopClosed");
                this.state = 0 /* hidden */;
            }
        };
        ChromeTopBar.height = 75; // Must match @COMMAND_BAR_HEIGHT in less file
        return ChromeTopBar;
    })();
    client.ChromeTopBar = ChromeTopBar;
})(client || (client = {}));
/// <reference path="../types.ts"/>
/// <reference path="../FigureLayout.ts"/>
/// <reference path="../ProfileGrid.ts"/>
/// <reference path="../screens/AppMasterScreen.ts"/>
/// <reference path="../Group.ts"/>
/// <reference path="PopUpWithProfile.ts"/>
/// <reference path="ProfilePopUp.ts"/>
var client;
(function (client) {
    /** User interface for showing who is in a Xim */
    var PhotoPopUp = (function (_super) {
        __extends(PhotoPopUp, _super);
        function PhotoPopUp($el, cast, addCallback) {
            var _this = this;
            // Call superclass to render Popup.  Allow drag to close popup
            _super.call(this, $el, "photoPopUp", { resources: resources.forms.photoPopUp }, { dragToClose: true });
            this.cast = cast;
            this.addCallback = addCallback;
            //--------------------
            // Handle user events
            //--------------------
            this.$el.on('click', '.photoPopUpButtonAdd', function (evt) {
                evt.preventDefault();
                evt.stopPropagation();
                client.AppMasterScreen.checkConnection(function () {
                    _this.addCallback(false);
                });
            });
            this.$el.on('click', '.photoPopUpButtonTake', function (evt) {
                evt.preventDefault();
                evt.stopImmediatePropagation();
                client.AppMasterScreen.checkConnection(function () {
                    _this.addCallback(true);
                });
            });
            // close the popup if user click on the background
            this.$el.on('click', '.photoPopUp', function (evt) {
                evt.preventDefault();
                evt.stopPropagation();
                _this.show(false);
            });
        }
        // Called when data has changed (such as number of photos)
        PhotoPopUp.prototype.updateDisplay = function (imageCount, curIndex) {
            // Update image count
            this.$el.find(".photoPopUpCount").text("Photo " + curIndex + "  of " + imageCount);
        };
        // Called when window is resize
        PhotoPopUp.prototype.resize = function () {
            this.render();
            _super.prototype.resize.call(this);
        };
        // Overwrite baseclass for custom rendering
        PhotoPopUp.prototype.render = function () {
            var viewport = client.webview.currentViewport();
            var screenWidth = viewport.width;
            var screenHeight = viewport.height;
            if (client.startupMode != 3 /* wp */)
                this.$popup.css({ height: screenHeight + 'px', width: screenWidth + 'px' });
            else
                this.$popup.css({ height: screenHeight + 'px' });
        };
        return PhotoPopUp;
    })(client.PopUpWithProfile);
    client.PhotoPopUp = PhotoPopUp;
})(client || (client = {}));
/// <reference path="../types.ts"/>
/// <reference path="../../../types/jquery/jquery.d.ts"/>
/// <reference path="ProfilePopUp.ts"/>
/// <reference path="PopUpWithProfile.ts"/>
/// <reference path="Bubble.ts"/>
/// <reference path="CommandBar.ts"/>
/// <reference path="../Util.ts"/>
/// <reference path="../imageMetaData.ts"/>
var client;
(function (client) {
    /** User interface for showing list of messages */
    var MessagePopUp = (function (_super) {
        __extends(MessagePopUp, _super);
        function MessagePopUp($el, sendcallback, closecallback) {
            var _this = this;
            // Call superclass to render Popup
            _super.call(this, $el, "messagePopUp", { resources: resources.forms.messagePopUp });
            this.sendcallback = sendcallback;
            this.closecallback = closecallback;
            // Keep track of sun messages so only one per user
            this.sunMessages = {};
            // Show button and not input box
            this.$el.find('.messagePopUpButton').show();
            this.$el.find('.messagePopUpInput').hide();
            // Force redraw
            this.resize();
            this.focusOnInput(false);
            //--------------------
            // Handle user events
            //--------------------
            // Eat other clicks so they don't propagate upward and close the popup
            this.$popup.on('click', function (event) {
                if (_this.isVisible()) {
                    event.preventDefault();
                    event.stopPropagation();
                }
            });
            this.$popup.on('click', '.messageBoxHolder', function (event) {
                if (_this.isVisible()) {
                    event.preventDefault();
                    event.stopPropagation();
                }
            });
            this.$popup.on('click', '.messagePopUpButton', function (evt) {
                evt.preventDefault();
                evt.stopPropagation();
                _this.focusOnInput(true);
            });
            // Close if click on message list background
            this.$popup.on('click', '.messagePopUpContent', function (event) {
                if (_this.isVisible()) {
                    event.preventDefault();
                    event.stopPropagation();
                    _this.show(false);
                }
            });
            this.$popup.on('keypress', function (event) {
                // Enter key acts as a sumbit
                if (event.keyCode == 13) {
                    event.preventDefault();
                    // Force WP8 keyboard to close
                    _this.$popup.find('textarea.form-control.messagePopUpInputForm').blur();
                    _this.$el.find('.focusdrop').focus();
                    _this.focusOnInput(false);
                    client.AppMasterScreen.checkConnection(function () {
                        _this.send();
                    });
                }
                else {
                    var $text = _this.$popup.find('textarea.form-control.messagePopUpInputForm');
                    var rawText = $text.val();
                    if (rawText.length > MessagePopUp.MAX_MESSAGE_LENGTH) {
                        event.preventDefault();
                    }
                }
            });
            this.$popup.on('click', 'input[type="submit"]', _.bind(this.send, this));
        }
        MessagePopUp.prototype.send = function () {
            var $text = this.$popup.find('textarea.form-control.messagePopUpInputForm');
            var rawText = $text.val();
            var text = client.Util.cleanString(rawText);
            // Limit text length
            text = text.substr(0, MessagePopUp.MAX_MESSAGE_LENGTH);
            client.logger.collectNamedEvent(consts.flurryEvent.ximsessionSendMessage, { textLength: text.length });
            client.logger.collect(consts.tags.cast.text.send, { textLength: text.length });
            // Call calback
            this.sendcallback(text);
            // Clear for next entry if stays open
            $text.val("");
        };
        MessagePopUp.prototype.focusOnInput = function (showInput) {
            if (showInput) {
                this.$el.find('.messagePopUpButton').hide();
                this.$el.find('.messagePopUpInput').show();
                this.$popup.find('textarea.form-control.messagePopUpInputForm').focus();
            }
            else {
                this.$el.find('.messagePopUpButton').show();
                this.$el.find('.messagePopUpInput').hide();
            }
        };
        // Hide any open profile when opening
        MessagePopUp.prototype.show = function (show) {
            if (show) {
                this.focusOnInput(false);
                this.hideProfile();
            }
            _super.prototype.show.call(this, show);
        };
        MessagePopUp.prototype.showDetail = function (show, focus) {
            if (focus === void 0) { focus = true; }
        };
        /** Returns true if sun message already exists for user and image */
        MessagePopUp.prototype.hasSunMessage = function (message) {
            // Get unique ID for message
            var sunId = message.imageId + message.profile.userId;
            if (this.sunMessages[sunId]) {
                return true;
            }
            return false;
        };
        /** Store link to sun message so I can update it later */
        MessagePopUp.prototype.saveSunMessage = function (message, $el) {
            // Get unique ID for message
            var sunId = message.imageId + message.profile.userId;
            // Replace it with new one
            this.sunMessages[sunId] = $el.find(".messageSun");
        };
        /** Update existing sun message if it exists, or return false */
        MessagePopUp.prototype.updateSunMessage = function (message, sunUrl) {
            // Get unique ID for message
            var sunId = message.imageId + message.profile.userId;
            if (!this.sunMessages[sunId]) {
                return false;
            }
            // Update image to reflect new count
            this.sunMessages[sunId].attr("src", sunUrl);
            return true;
        };
        // Add a single message
        MessagePopUp.prototype.addMessage = function (message) {
            // Are system messages included?
            if (message.isSystem && !client.user.GetSetting(5 /* SHOW_SYSTEM_MESSAGES */)) {
                return;
            }
            // Render bubble message
            var $content = this.renderBubble(message, this.isVisible());
            if ($content) {
                // Add to message display
                var $messages = this.$popup.find(".messagePopUpContent");
                $messages.prepend($content);
                this.$popup.find(".messageBlock").css({ width: client.Bubble.messageBlockWidth + 'px' });
                this.$popup.find(".messageBoxHolder").css({ width: client.Bubble.messageBoxWidth + 'px' });
                // Animate it in
                if (this.isVisible()) {
                    setTimeout(function () {
                        $content.find(".messageBlock").removeClass("messageNew");
                    }, 1);
                }
            }
        };
        // Initialize message list
        MessagePopUp.prototype.initMessages = function (messages) {
            var $messages = this.$popup.find(".messagePopUpContent");
            $messages.empty();
            // Close if click on message list background
            $messages.on('scroll', function (event) {
                if ($messages.scrollTop() != 0) {
                    $messages.addClass('scrolled');
                }
                else {
                    $messages.removeClass('scrolled');
                }
            });
            for (var i = 0; i < messages.length; i++) {
                var message = messages[i];
                // Include system messages?
                if (message.isSystem && !client.user.GetSetting(5 /* SHOW_SYSTEM_MESSAGES */)) {
                    continue;
                }
                // Render message and add to message display
                var $content = this.renderBubble(message);
                if ($content) {
                    $messages.append($content);
                }
            }
            this.$popup.find(".messageBlock").css({ width: client.Bubble.messageBlockWidth + 'px' });
            this.$popup.find(".messageBoxHolder").css({ width: client.Bubble.messageBoxWidth + 'px' });
        };
        /** Render new message bubble */
        MessagePopUp.prototype.renderBubble = function (message, animate) {
            if (animate === void 0) { animate = false; }
            // Generate render data
            var bubbleRenderData = client.Bubble.getRenderData(message);
            if (!bubbleRenderData) {
                return null;
            }
            // If sun message already exists, update it
            if (message.hasSun && this.updateSunMessage(message, bubbleRenderData.sunUrl)) {
                return null;
            }
            else {
                var $content = $("<div style='margin-top: -10px'>").render('messageBubble', { data: bubbleRenderData, animate: animate });
                $content.find('.messageFace').on('click', { profile: message.profile }, _.bind(this.onProfileClick, this));
                if (bubbleRenderData.imageUrl) {
                    $content.find('.messageImage').on('click', { imageId: message.imageId }, _.bind(this.onImageClick, this));
                }
                // Store links to sun messages for future updates
                if (message.hasSun) {
                    this.saveSunMessage(message, $content);
                }
            }
            return $content;
        };
        /** Called when user click on image */
        MessagePopUp.prototype.onImageClick = function (event) {
            event.preventDefault();
            event.stopPropagation();
            // Send image id to caller
            this.closecallback(event.data.imageId);
        };
        // Set size taking into account that screen may have rotated
        MessagePopUp.prototype.resize = function () {
            var screenWidth = window.innerWidth;
            var screenHeight = window.innerHeight;
            var listHeight = screenHeight - (client.CommandBar.height + MessagePopUp.COMMAND_OFFSET);
            this.$popup.find(".messagePopUpContent").css({ height: listHeight + 'px' });
            var inputWidth = screenWidth - 60; // 60 = 30 margin on each side
            this.$popup.find(".messagePopUpInputForm").css({ width: inputWidth + 'px' });
            client.Bubble.messageBlockWidth = Math.min(client.Bubble.messageBlockWidthMax, 0.92 * screenWidth);
            client.Bubble.messageBoxWidth = client.Bubble.messageBlockWidth - client.Bubble.messageBubbleWidth;
            this.$popup.find(".messageBlock").css({ width: client.Bubble.messageBlockWidth + 'px' });
            this.$popup.find(".messageBoxHolder").css({ width: client.Bubble.messageBoxWidth + 'px' });
            _super.prototype.resize.call(this);
        };
        MessagePopUp.MAX_MESSAGE_LENGTH = 160;
        MessagePopUp.COMMAND_OFFSET = 202;
        return MessagePopUp;
    })(client.PopUpWithProfile);
    client.MessagePopUp = MessagePopUp;
})(client || (client = {}));
/// <reference path="../types.ts"/>
/// <reference path="../resources.ts"/>
/// <reference path="../../../types/jquery/jquery.d.ts"/>
/// <reference path="PopUp.ts"/>
var client;
(function (client) {
    /** User interface for entering a comment */
    var CommentPopUp = (function (_super) {
        __extends(CommentPopUp, _super);
        function CommentPopUp(callback) {
            var _this = this;
            // Call superclass to render Popup
            _super.call(this, null, "commentPopUp", { resources: resources.forms.commentPopUp }, { overlayOpacity: 0.7 });
            this.callback = callback;
            // Capture the users choice
            this.$popup.on('click', '#confirmationPopUpLeft', function () {
                _this.callback(null);
                _this.destroy();
            });
            this.$popup.on('click', '#confirmationPopUpRight', function () {
                _this.sendAndClose();
            });
            this.$popup.on('keypress', function (event) {
                // Enter key acts as a sumbit
                if (event.keyCode == 13) {
                    event.preventDefault();
                    _this.sendAndClose();
                }
                else {
                    var $text = _this.$popup.find('textarea.form-control.commentPopUpInputForm');
                    var rawText = $text.val();
                    if (rawText.length > CommentPopUp.MAX_COMMENT_LENGTH) {
                        event.preventDefault();
                    }
                }
            });
            // Show automatically (necessary for resize)
            this.show(true);
            // Size to center in screen
            this.resize();
            // Set focus on input
            this.$popup.find('textarea.form-control.commentPopUpInputForm').focus();
        }
        /** Called when comment is sent. Create comment and closes popup */
        CommentPopUp.prototype.sendAndClose = function () {
            // Grab text, make sure it's clean
            var $text = this.$popup.find('textarea.form-control.commentPopUpInputForm');
            var rawText = $text.val();
            var text = client.Util.cleanString(rawText);
            // Limit text length
            text = text.substr(0, CommentPopUp.MAX_COMMENT_LENGTH);
            client.logger.collectNamedEvent(consts.flurryEvent.ximsessionAddComment);
            this.callback(text);
            this.destroy();
        };
        // Center on screen (takes into account that it may have rotated)
        CommentPopUp.prototype.resize = function () {
            var screenWidth = client.webview.currentViewport().width;
            var screenHeight = client.webview.currentViewport().height;
            var x = (screenWidth - this.$popup.width()) / 2;
            var y = -100 + (screenHeight - this.$popup.height()) / 2;
            this.$popup.css({ marginLeft: Math.round(x) + 'px', marginTop: Math.round(y) + 'px' });
        };
        CommentPopUp.MAX_COMMENT_LENGTH = 100;
        return CommentPopUp;
    })(client.PopUp);
    client.CommentPopUp = CommentPopUp;
})(client || (client = {}));
/// <reference path="../types.ts"/>
/// <reference path="../../../types/jquery/jquery.d.ts"/>
var client;
(function (client) {
    var HoverPosition;
    (function (HoverPosition) {
        HoverPosition[HoverPosition["CENTER"] = 0] = "CENTER";
        HoverPosition[HoverPosition["TOP"] = 1] = "TOP";
        HoverPosition[HoverPosition["TOP_LEFT"] = 2] = "TOP_LEFT";
        HoverPosition[HoverPosition["TOP_RIGHT"] = 3] = "TOP_RIGHT";
        HoverPosition[HoverPosition["BOTTOM"] = 4] = "BOTTOM";
        HoverPosition[HoverPosition["BOTTOM_LEFT"] = 5] = "BOTTOM_LEFT";
        HoverPosition[HoverPosition["BOTTOM_RIGHT"] = 6] = "BOTTOM_RIGHT";
        HoverPosition[HoverPosition["LEFT"] = 7] = "LEFT";
        HoverPosition[HoverPosition["RIGHT"] = 8] = "RIGHT";
    })(HoverPosition || (HoverPosition = {}));
    ;
    /** A menu that floats near where the user clicked */
    var HoverMenu = (function () {
        function HoverMenu(selector, callback) {
            this.state = 0 /* closed */;
            /** Base position of hover menu around which buttons are displayed */
            this.basePos = null;
            /** Which layout to use for the buttons */
            this.layout = null;
            /** Offset of each  button from the base */
            this.buttonOffset = {};
            this.$el = $(selector);
            this.callback = callback;
        }
        HoverMenu.prototype.isOpen = function () {
            return this.state == 1 /* open */;
        };
        HoverMenu.prototype.hide = function () {
            this.$el.hide();
            this.state = 0 /* closed */;
        };
        /** Trigger a touch event on the hover menu.  (For capturing Hammer.js events) */
        HoverMenu.prototype.triggerTouch = function (e) {
            var touchPos = this.getTouchPos(e);
            if (!touchPos) {
                return;
            }
            for (var i = 0; i < this.commands.length; i++) {
                // If touch is in button, trigger click callback
                if (this.inButton(touchPos, i)) {
                    this.callback(this.commands[i].id);
                }
            }
        };
        /** Trigger a hover event on the hover menu.  (For capturing Hammer.js events) */
        HoverMenu.prototype.triggerHover = function (e) {
            var touchPos = this.getTouchPos(e);
            if (!touchPos) {
                return;
            }
            for (var i = 0; i < this.commands.length; i++) {
                var $button = this.$el.find("#" + this.commands[i].id);
                // Set or clear hover display depending on whether touch is in button
                this.showHover($button, this.inButton(touchPos, i));
            }
        };
        /** Get device indepenant touch position */
        HoverMenu.prototype.getTouchPos = function (e) {
            var touchPos = null;
            if (e.originalEvent.changedTouches && e.originalEvent.changedTouches.length != 0) {
                return e.originalEvent.changedTouches[0];
            }
            else if (e.originalEvent.targetTouches && e.originalEvent.targetTouches.length != 0) {
                return e.originalEvent.targetTouches[0];
            }
            else if (e.originalEvent) {
                return e.originalEvent;
            }
            return null;
        };
        /** Show the hover menu */
        HoverMenu.prototype.show = function (clickPosition) {
            var _this = this;
            if (this.state == 0 /* closed */) {
                // Calculate base position of hover menu and layout
                this.calcBasePosition(clickPosition);
                this.calcLayout();
                // Set overlay to be clear so can animate in
                var $overlay = this.$el.find(".hoverMenuOverlay");
                $overlay.css({ 'background-color': 'rgba(0, 0, 0, 0.1)' });
                for (var i = 0; i < this.commands.length; i++) {
                    var $button = this.$el.find("#" + this.commands[i].id);
                    $button.removeClass('hoverMenuPressed');
                    var translate = 'translate(0px,0px)';
                    $button.css({
                        top: this.basePos.y,
                        left: this.basePos.x,
                        '-ms-transition-duration': 0,
                        '-webkit-transition-duration': 0,
                        'transition-duration': 0,
                        '-ms-transform': translate,
                        '-webkit-transform': translate,
                        'transform': translate
                    });
                }
                // Show it
                this.$el.show();
                this.state = 1 /* open */;
                // Animate - pause to allow redraw
                setTimeout(function () {
                    for (var i = 0; i < _this.commands.length; i++) {
                        var $button = _this.$el.find("#" + _this.commands[i].id);
                        _this.buttonOffset[i] = _this.calcButtonOffset(i, _this.layout);
                        var translate = 'translate(' + _this.buttonOffset[i].x + 'px,' + _this.buttonOffset[i].y + 'px)';
                        var duration = '200ms';
                        $button.css({
                            '-ms-transition-duration': duration,
                            '-webkit-transition-duration': duration,
                            'transition-duration': duration,
                            '-ms-transform': translate,
                            '-webkit-transform': translate,
                            'transform': translate,
                            'transition-timing-function': 'ease'
                        });
                    }
                    $overlay.css({ 'background-color': 'rgba(0, 0, 0, 0.7)' });
                });
            }
        };
        /** Sets commands and renders */
        HoverMenu.prototype.setCommands = function (commands) {
            this.commands = commands;
            this.renderCommands();
            this.hide();
        };
        //=================================================
        // Private Methods
        //=================================================
        //** Render hover menu items */
        HoverMenu.prototype.renderCommands = function () {
            var _this = this;
            var $holder = $('<div class="hoverMenuHolder">');
            // Add overlay to capture off menu clicks
            var $overlay = $('<div class="hoverMenuOverlay">');
            $holder.append($overlay);
            for (var i = 0; i < this.commands.length; i++) {
                var $button = $('<div class="hoverMenuButton" id="' + this.commands[i].id + '">');
                $button.append('<div class="' + this.commands[i].glyph + ' hoverMenuGlyph">');
                $button.data('id', i);
                $holder.append($button);
            }
            this.$el.append($holder);
            for (i = 0; i < this.commands.length; i++) {
                var id = this.commands[i].id;
                var callback = this.callback;
                var $command = this.$el.find('#' + id);
                $command.click({ id: this.commands[i].id }, function (ev) {
                    ev.stopPropagation();
                    callback(ev.data.id);
                });
                $command.on('mousedown touchstart', function (ev) {
                    var $cmd = $(ev.currentTarget);
                    _this.showHover($cmd, true);
                    // Remove pressed style after a pause in case user moved mouse off target (which is not a click)
                    setTimeout(function () {
                        _this.showHover($cmd, false);
                    }, 500);
                });
            }
            this.$el.append($holder);
            this.$el.on('click', '.hoverMenuOverlay', function (ev) {
                ev.preventDefault();
                _this.hide();
            });
        };
        /** Show hover styling on the given button */
        HoverMenu.prototype.showHover = function ($button, show) {
            var i = $button.data('id');
            var translate = 'translate(' + this.buttonOffset[i].x + 'px,' + this.buttonOffset[i].y + 'px)';
            if (show) {
                // Set pressed state 
                $button.addClass('hoverMenuPressed');
                $button.css({
                    top: this.basePos.y,
                    left: this.basePos.x,
                    '-ms-transition-duration': '100ms',
                    '-webkit-transition-duration': '100ms',
                    'transition-duration': '100ms',
                    '-ms-transform': translate + 'scale(1.5)',
                    '-webkit-transform': translate + 'scale(1.5)',
                    'transform': translate + 'scale(1.5)'
                });
            }
            else {
                // clear pressed state 
                $button.removeClass('hoverMenuPressed');
                $button.css({
                    top: this.basePos.y,
                    left: this.basePos.x,
                    '-ms-transition-duration': '100ms',
                    '-webkit-transition-duration': '100ms',
                    'transition-duration': '100ms',
                    '-ms-transform': translate + ' scale(1)',
                    '-webkit-transform': translate + 'scale(1)',
                    'transform': translate + 'scale(1)'
                });
            }
        };
        /** Returns true if touch position is inside buttone with given buttonID */
        HoverMenu.prototype.inButton = function (touchPos, id) {
            return (Math.abs(touchPos.clientX - (this.buttonOffset[id].x + this.basePos.x + HoverMenu.BUTTON_HALF_WIDTH)) < HoverMenu.TOUCH_THRESHOLD && Math.abs(touchPos.clientY - (this.buttonOffset[id].y + this.basePos.y + HoverMenu.BUTTON_HALF_WIDTH)) < HoverMenu.TOUCH_THRESHOLD);
        };
        HoverMenu.prototype.calcBasePosition = function (position) {
            // Set position for hovering menu
            var viewport = client.webview.currentViewport();
            var y = position.y - HoverMenu.TOP_MARGIN;
            var x = position.x;
            var xbase = Math.min(Math.max(0, x), (viewport.width - HoverMenu.HOVER_OFFSET));
            var ybase = Math.min(Math.max(0, y), (viewport.height - HoverMenu.HOVER_OFFSET - HoverMenu.BUTTON_HALF_WIDTH));
            this.basePos = { x: xbase, y: ybase };
        };
        /** Given a click position return which display type to use */
        HoverMenu.prototype.calcLayout = function () {
            // Set position for hovering menu
            var viewport = client.webview.currentViewport();
            var leftEdge = this.basePos.x < HoverMenu.HOVER_MARGIN;
            var rightEdge = this.basePos.x > (viewport.width - HoverMenu.HOVER_MARGIN);
            var topEdge = this.basePos.y < HoverMenu.HOVER_MARGIN;
            var bottomEdge = this.basePos.y > (viewport.height - HoverMenu.HOVER_MARGIN);
            if (leftEdge) {
                if (topEdge) {
                    this.layout = 2 /* TOP_LEFT */;
                }
                else if (bottomEdge) {
                    this.layout = 5 /* BOTTOM_LEFT */;
                }
                else {
                    this.layout = 7 /* LEFT */;
                }
            }
            else if (rightEdge) {
                if (topEdge) {
                    this.layout = 3 /* TOP_RIGHT */;
                }
                else if (bottomEdge) {
                    this.layout = 6 /* BOTTOM_RIGHT */;
                }
                else {
                    this.layout = 8 /* RIGHT */;
                }
            }
            else if (topEdge) {
                this.layout = 1 /* TOP */;
            }
            else if (bottomEdge) {
                this.layout = 4 /* BOTTOM */;
            }
            else {
                this.layout = 0 /* CENTER */;
            }
        };
        /** Determine offset for each hover menu button */
        HoverMenu.prototype.calcButtonOffset = function (index, hoverPosition) {
            switch (index) {
                case 0:
                    switch (hoverPosition) {
                        case 0 /* CENTER */:
                            return this.getHoverPosition(2 /* TOP_LEFT */);
                        case 1 /* TOP */:
                            return this.getHoverPosition(5 /* BOTTOM_LEFT */);
                        case 8 /* RIGHT */:
                        case 4 /* BOTTOM */:
                        case 3 /* TOP_RIGHT */:
                        case 6 /* BOTTOM_RIGHT */:
                            return this.getHoverPosition(7 /* LEFT */);
                        case 2 /* TOP_LEFT */:
                            return this.getHoverPosition(4 /* BOTTOM */);
                        case 7 /* LEFT */:
                        case 5 /* BOTTOM_LEFT */:
                            return this.getHoverPosition(3 /* TOP_RIGHT */);
                    }
                case 1:
                    switch (hoverPosition) {
                        case 0 /* CENTER */:
                            return this.getHoverPosition(3 /* TOP_RIGHT */);
                        case 1 /* TOP */:
                            return this.getHoverPosition(6 /* BOTTOM_RIGHT */);
                        case 4 /* BOTTOM */:
                        case 2 /* TOP_LEFT */:
                        case 5 /* BOTTOM_LEFT */:
                        case 7 /* LEFT */:
                            return this.getHoverPosition(8 /* RIGHT */);
                        case 3 /* TOP_RIGHT */:
                            return this.getHoverPosition(4 /* BOTTOM */);
                        case 8 /* RIGHT */:
                        case 6 /* BOTTOM_RIGHT */:
                            return this.getHoverPosition(2 /* TOP_LEFT */);
                    }
                case 2:
                    switch (hoverPosition) {
                        case 0 /* CENTER */:
                        case 4 /* BOTTOM */:
                        case 8 /* RIGHT */:
                        case 7 /* LEFT */:
                        case 6 /* BOTTOM_RIGHT */:
                        case 5 /* BOTTOM_LEFT */:
                            return this.getHoverPosition(1 /* TOP */);
                        case 1 /* TOP */:
                            return this.getHoverPosition(4 /* BOTTOM */);
                        case 2 /* TOP_LEFT */:
                            return this.getHoverPosition(6 /* BOTTOM_RIGHT */);
                        case 3 /* TOP_RIGHT */:
                            return this.getHoverPosition(5 /* BOTTOM_LEFT */);
                    }
            }
            return { x: 0, y: 0 };
        };
        /** Given a click position return which display type to use */
        HoverMenu.prototype.getHoverPosition = function (hoverPosition) {
            switch (hoverPosition) {
                case 1 /* TOP */:
                    return { x: 0, y: -HoverMenu.HOVER_OFFSET };
                case 2 /* TOP_LEFT */:
                    return { x: -0.75 * HoverMenu.HOVER_OFFSET, y: -0.75 * HoverMenu.HOVER_OFFSET };
                case 3 /* TOP_RIGHT */:
                    return { x: 0.75 * HoverMenu.HOVER_OFFSET, y: -0.75 * HoverMenu.HOVER_OFFSET };
                case 4 /* BOTTOM */:
                    return { x: 0, y: HoverMenu.HOVER_OFFSET };
                case 5 /* BOTTOM_LEFT */:
                    return { x: -0.75 * HoverMenu.HOVER_OFFSET, y: 0.75 * HoverMenu.HOVER_OFFSET };
                case 6 /* BOTTOM_RIGHT */:
                    return { x: 0.75 * HoverMenu.HOVER_OFFSET, y: 0.75 * HoverMenu.HOVER_OFFSET };
                case 7 /* LEFT */:
                    return { x: -HoverMenu.HOVER_OFFSET, y: 0 };
                case 8 /* RIGHT */:
                    return { x: HoverMenu.HOVER_OFFSET, y: 0 };
                default:
                    return { x: 0, y: 0 };
            }
        };
        HoverMenu.HOVER_MARGIN = 130;
        HoverMenu.HOVER_OFFSET = 110;
        // Size of target area for button
        HoverMenu.TOUCH_THRESHOLD = 45;
        // Half-width of the button
        HoverMenu.BUTTON_HALF_WIDTH = 30;
        // Amount to raise display buttons above location user pressed
        HoverMenu.TOP_MARGIN = 90;
        return HoverMenu;
    })();
    client.HoverMenu = HoverMenu;
})(client || (client = {}));
/// <reference path="../types.ts"/>
/// <reference path="../../../types/jquery/jquery.d.ts"/>
var client;
(function (client) {
    // Master canvas that can hold multiple partile effects
    var ParticleMaster = (function () {
        function ParticleMaster() {
        }
        ParticleMaster.show = function ($el, numberOfElements) {
            // Clear any running animation
            this.startTime = null;
            // Bind animate function
            if (!this.animateFunction) {
                this.animateFunction = _.bind(this.animate, this);
            }
            // Save base element
            this.$el = $el;
            // Create canvas
            var screenWidth = client.webview.currentViewport().width;
            var screenHeight = client.webview.currentViewport().height;
            this.$el.empty();
            var html = '<canvas class="particleEffect" width="' + screenWidth + '" height="' + screenHeight + '"></canvas>';
            this.$el.append(html);
            this.canvas = this.$el.find(".particleEffect")[0];
            this.context = this.canvas.getContext('2d');
            // Create particle effects
            var startDelay = 0;
            for (var i = 0; i < numberOfElements; i++) {
                this.createParticleEffect(startDelay);
                // Start each effect with a slight delay
                startDelay += 500 + (500 * Math.random());
            }
            // Record start time
            this.startTime = moment();
            this.lastTime = this.startTime;
            // Start animation loop
            this.animate();
        };
        ParticleMaster.createParticleEffect = function (startDelay) {
            var particleEffect = new ParticleEffect(40, startDelay);
            this.particleEffects.push(particleEffect);
        };
        // Animate the particle effects
        ParticleMaster.animate = function () {
            // Don't animate if I've been reset
            if (!this.startTime) {
                return;
            }
            // Time passed since last animation call (so effect can be relative to frame rate)
            var now = moment();
            var timePassed = now.diff(this.lastTime, 'milliseconds');
            // Time passed since first started animating
            var totalTimePassed = now.diff(this.startTime, 'milliseconds');
            this.lastTime = now;
            // Fade existing canvas
            var fadeRate = Math.max(0, 1 - (ParticleEffect.fadeRate * timePassed));
            this.fadeCanvas(fadeRate);
            // Itterate through each particle effect, noting if still animating
            var animating = false;
            for (var pe = 0; pe < this.particleEffects.length; pe++) {
                // Animate if past start delay
                if (totalTimePassed >= this.particleEffects[pe].startDelay) {
                    if (this.particleEffects[pe].animate(timePassed)) {
                        animating = true;
                    }
                }
                else {
                    // Still animating as haven't started yet
                    animating = true;
                }
            }
            // If still animating cue up next animation call
            if (animating) {
                requestAnimationFrame(this.animateFunction);
            }
            else {
                // Clear the canvas
                this.$el.empty();
                this.particleEffects = [];
            }
        };
        // Fill canvas with given color at given opacity
        ParticleMaster.fillCanvas = function (color, opacity) {
            this.context.rect(0, 0, this.canvas.width, this.canvas.height);
            this.context.fillStyle = color;
            this.context.globalAlpha = opacity;
            this.context.fill();
        };
        // Fade the canvas
        ParticleMaster.fadeCanvas = function (fade) {
            // Create a temp canvas 
            var tempCanvas = document.createElement('canvas');
            var tempContext = tempCanvas.getContext('2d');
            tempCanvas.width = this.canvas.width;
            tempCanvas.height = this.canvas.height;
            // Apply the old canvas to the temp one
            tempContext.drawImage(this.canvas, 0, 0);
            // Clear existing canvas and copy temp back with fade
            this.context.clearRect(0, 0, this.canvas.width, this.canvas.height);
            this.context.globalAlpha = fade;
            this.context.drawImage(tempCanvas, 0, 0, this.canvas.width, this.canvas.height);
        };
        ParticleMaster.$el = null;
        ParticleMaster.canvas = null;
        ParticleMaster.context = null;
        ParticleMaster.animateFunction = null;
        ParticleMaster.particleEffects = [];
        return ParticleMaster;
    })();
    client.ParticleMaster = ParticleMaster;
    // Controls a group of firework particles
    var ParticleEffect = (function () {
        function ParticleEffect(numParticles, startDelay) {
            this.particles = null;
            this.colorTable = null;
            this.startTime = null;
            this.startDelay = 0;
            this.startDelay = startDelay;
            this.makeColorTable();
            this.makeParticles(numParticles);
            this.initializeParticles();
        }
        // Make color table for fast look-up of shifting color
        ParticleEffect.prototype.makeColorTable = function () {
            var randomColor = Math.floor(356 * Math.random());
            this.colorTable = [];
            var colorShifted = randomColor;
            for (var i = 0; i < ParticleEffect.colorRange; i++) {
                colorShifted += 3;
                if (colorShifted > 356)
                    colorShifted = 0;
                this.colorTable.push('hsl(' + colorShifted + ',100%,50%)');
            }
        };
        // Animate and return true if still animating
        ParticleEffect.prototype.animate = function (timePassed) {
            // Note time first started
            if (!this.startTime) {
                this.startTime = moment();
                // Flash screen
                ParticleMaster.fillCanvas(this.colorTable[0], 0.4);
            }
            // Calculate fraction of time remaining
            var totalTimePassed = moment().diff(this.startTime, 'milliseconds');
            var fractionPassed = totalTimePassed / ParticleEffect.duration;
            // Done animating
            if (fractionPassed > 1) {
                return false;
            }
            // Calculate acceleration relative to time passed (for frame rate independance)
            var timeSquared = Math.pow(totalTimePassed, 2);
            var acceleration = (0.5 * timeSquared * ParticleEffect.gravity);
            // Fade particles when animation is nearing end
            var opacity = 1.0;
            if (fractionPassed > 0.8) {
                // Convert to fade value from 1 -> 0
                opacity = 5 * (1 - fractionPassed);
            }
            for (var i = 0; i < this.particles.length; i++) {
                this.particles[i].animate(this, totalTimePassed, acceleration, opacity);
            }
            return true;
        };
        ParticleEffect.prototype.initializeParticles = function () {
            var xOffset = -100 + (200 * Math.random());
            var yOffset = -(200 * Math.random());
            for (var i = 0; i < this.particles.length; i++) {
                this.particles[i].initialize(xOffset, yOffset);
            }
        };
        ParticleEffect.prototype.makeParticles = function (numParticles) {
            this.particles = [];
            for (var i = 0; i < numParticles; i++) {
                var particle = new Particle();
                this.particles.push(particle);
            }
        };
        ParticleEffect.initVelocity = 0.2;
        ParticleEffect.duration = 1500;
        ParticleEffect.gravity = 0.00005;
        ParticleEffect.fadeRate = 0.005;
        ParticleEffect.colorRange = 25;
        return ParticleEffect;
    })();
    client.ParticleEffect = ParticleEffect;
    // Fireworks particle
    var Particle = (function () {
        function Particle() {
        }
        Particle.prototype.initialize = function (xOffset, yOffset) {
            this.xInitPos = xOffset + (ParticleMaster.canvas.width * 0.5);
            this.yInitPos = yOffset + (ParticleMaster.canvas.height * 0.5);
            this.xPos = this.xInitPos;
            this.yPos = this.yInitPos;
            this.xInitVel = ParticleEffect.initVelocity * (Math.random() - 0.5);
            this.yInitVel = ParticleEffect.initVelocity * (Math.random() - 0.5);
            this.colorOffset = Math.floor(Math.random() * ParticleEffect.colorRange);
            this.colorDir = 1;
            this.width = Math.floor(Math.random() * 4) + 2;
        };
        // Animate the particle
        Particle.prototype.animate = function (particleEffect, totalTimePassed, acceleration, opacity) {
            var context = ParticleMaster.context;
            // Calculate new position pased on time passed (frame rate independant)
            var newX = this.xInitPos + (this.xInitVel * totalTimePassed);
            var newY = this.yInitPos + (this.yInitVel * totalTimePassed) + acceleration;
            // Gradient so front of particle beam is lighter
            var gradient = context.createLinearGradient(this.xPos, this.yPos, newX, newY);
            gradient.addColorStop("0", particleEffect.colorTable[this.colorOffset]);
            gradient.addColorStop("0.8", particleEffect.colorTable[this.colorOffset]);
            gradient.addColorStop("1.0", '#FFFFFF');
            context.globalAlpha = opacity;
            context.beginPath();
            context.moveTo(this.xPos, this.yPos);
            context.lineTo(newX, newY);
            context.strokeStyle = gradient;
            context.lineCap = 'round';
            context.lineWidth = this.width;
            context.stroke();
            this.xPos = newX;
            this.yPos = newY;
            // Rotate color
            this.colorOffset += this.colorDir;
            if (this.colorOffset >= ParticleEffect.colorRange) {
                this.colorOffset = 0;
                this.colorDir = -1;
            }
            else if (this.colorOffset < 0) {
                this.colorOffset = 0;
                this.colorDir = 1;
            }
        };
        return Particle;
    })();
})(client || (client = {}));
/// <reference path="types.ts"/>
/// <reference path="consts.ts"/>
/// <reference path="castViewerAirPlay.ts"/>
/// <reference path="castViewerBrowser.ts"/>
/// <reference path="castViewerFireTV.ts"/>
/// <reference path="castViewerChromeCast.ts"/>
/// <reference path="castViewerXBox.ts"/>
/** Module for managing xim specific keyboard and remote control input */
var client;
(function (client) {
    var KeyboardHandler;
    (function (KeyboardHandler) {
        KeyboardHandler.onKeypress = new client.Event();
        (function (KeyEvent) {
            KeyEvent[KeyEvent["FORWARD"] = 39] = "FORWARD";
            KeyEvent[KeyEvent["BACK"] = 37] = "BACK";
            KeyEvent[KeyEvent["REWIND"] = 36] = "REWIND";
            KeyEvent[KeyEvent["FAST_FORWARD"] = 35] = "FAST_FORWARD";
            KeyEvent[KeyEvent["ZOOM_IN"] = 38] = "ZOOM_IN";
            KeyEvent[KeyEvent["ZOOM_OUT"] = 40] = "ZOOM_OUT"; // down arrow
        })(KeyboardHandler.KeyEvent || (KeyboardHandler.KeyEvent = {}));
        var KeyEvent = KeyboardHandler.KeyEvent;
        var FireTVEvent;
        (function (FireTVEvent) {
            FireTVEvent[FireTVEvent["REWIND"] = 227] = "REWIND";
            FireTVEvent[FireTVEvent["FAST_FORWARD"] = 228] = "FAST_FORWARD";
        })(FireTVEvent || (FireTVEvent = {}));
        /** Keyboard handler */
        document.onkeyup = function (event) {
            switch (event.keyCode) {
                case 39 /* FORWARD */:
                    KeyboardHandler.onKeypress.trigger(39 /* FORWARD */);
                    return;
                case 37 /* BACK */:
                    KeyboardHandler.onKeypress.trigger(37 /* BACK */);
                    return;
                case 36 /* REWIND */:
                case 227 /* REWIND */:
                    KeyboardHandler.onKeypress.trigger(36 /* REWIND */);
                    return;
                case 35 /* FAST_FORWARD */:
                case 228 /* FAST_FORWARD */:
                    KeyboardHandler.onKeypress.trigger(35 /* FAST_FORWARD */);
                    return;
                case 38 /* ZOOM_IN */:
                    KeyboardHandler.onKeypress.trigger(38 /* ZOOM_IN */);
                    return;
                case 40 /* ZOOM_OUT */:
                    KeyboardHandler.onKeypress.trigger(40 /* ZOOM_OUT */);
                    return;
            }
        };
    })(KeyboardHandler = client.KeyboardHandler || (client.KeyboardHandler = {}));
})(client || (client = {}));
/// <reference path="types.ts"/>
/// <reference path="consts.ts"/>
/// <reference path='../../types/cordova/imagetransfer.d.ts'/>
/// <reference path='../../types/cordova/cordova-file.d.ts'/>
var client;
(function (client) {
    var FileDownload = (function () {
        function FileDownload(url, folderName, fileName) {
            this.url = url;
            this.folderName = folderName;
            this.fileName = fileName;
        }
        FileDownload.prototype.download = function (successcallback, failcallback) {
            this.successCallback = successcallback;
            this.failCallback = failcallback;
            // Won't exist in emultion / display mode
            if (typeof (LocalFileSystem) == 'undefined') {
                this.failCallback("No LocalFileSystem");
                return;
            }
            // Start download by requesting file system
            requestFileSystem(LocalFileSystem.PERSISTENT, 0, _.bind(this.fileSuccess, this), _.bind(this.downloadFail, this));
        };
        FileDownload.prototype.fileSuccess = function (filesystem) {
            var _this = this;
            // Encode url
            var uri = encodeURI(this.url);
            // Attempt to get file extention
            var ext = uri.substr(uri.lastIndexOf('.') + 1);
            // If coming from blob storage will not have a valid extention, but it will be jpg
            var validExtensions = ["jpg", "jpeg", "bmp", "gif", "png"];
            if (validExtensions.indexOf(ext.toLowerCase()) == -1) {
                ext = "jpg";
            }
            // Get root path
            var directoryEntry = filesystem.root;
            // Pick photo directory for android
            if (client.startupMode == 5 /* droid */) {
                this.folderName = "DCIM/" + this.folderName;
            }
            directoryEntry.getDirectory(this.folderName, { create: true, exclusive: false }, null, _.bind(this.downloadFail, this));
            // Generate path for saved file
            var targetPath = directoryEntry.toURL() + "/" + this.folderName + "/" + this.fileName + "." + ext;
            // Now download file
            if (window.imagetransfer) {
                window.imagetransfer.download(this.url, targetPath, function () {
                    _this.successCallback();
                }, function (error) {
                    _this.failCallback(error);
                });
            }
            else {
                client.logger.error(consts.errorCode.FileDownload, 'ImageTransfer Plugin not defined.');
            }
        };
        FileDownload.prototype.downloadFail = function (error) {
            client.logger.error(consts.errorCode.FileDownload, 'File Save Error: ' + error.code);
            this.failCallback(error.code);
        };
        return FileDownload;
    })();
    client.FileDownload = FileDownload;
})(client || (client = {}));
/// <reference path="CastingScreen.ts"/>
/// <reference path="../controls/PhotoSlider.ts"/>  
/// <reference path="../controls/Bubble.ts"/>
/// <reference path="../controls/PeoplePopUp.ts"/>
/// <reference path="../controls/ChromeTopBar.ts"/>
/// <reference path="../controls/PhotoPopUp.ts"/>
/// <reference path="../controls/MessagePopUp.ts"/>
/// <reference path="../controls/CommentPopUp.ts"/>
/// <reference path="../controls/HoverMenu.ts"/>
/// <reference path="../controls/ParticleEffect.ts"/>
/// <reference path="../Group.ts"/>
/// <reference path="../keyboardHandler.ts"/>
/// <reference path="../fileDownload.ts"/>
/// <reference path="casting.ts"/>
var client;
(function (client) {
    /** View for a photo cast session. */
    var PhotoCastScreen = (function (_super) {
        __extends(PhotoCastScreen, _super);
        function PhotoCastScreen() {
            _super.apply(this, arguments);
            this.menuCommands = [];
            this.chromeTimer = null;
            // For detecting double taps
            this.tapTimer = null;
            this.groupPresence = null;
            this.dialogScreen = null;
            this.queueLength = 0;
        }
        PhotoCastScreen.prototype.onCreated = function () {
            // Turn off panning for windows display so screen doesn't jump
            $('body').addClass("noPan");
            this._onWindowResize = _.bind(this.onWindowResize, this);
            window.addEventListener('resize', this._onWindowResize);
            // If display, capture keyboard events
            if (client.isDesktopDisplay) {
                this._onKeypressHandler = _.bind(this.onKeypressHandler, this);
                client.KeyboardHandler.onKeypress.add(this._onKeypressHandler);
            }
            // Make sure image metadata is reset
            client.ImageMetaData.clearMetaData();
            _super.prototype.onCreated.call(this);
        };
        PhotoCastScreen.prototype.onDispose = function () {
            // Note that PhotoSlider is disposed so pending image loads will stop
            if (this.photoSlider) {
                this.photoSlider.onDispose();
            }
            // remove dark theme
            $('body').toggleClass('dark-theme', false);
            // Disconnect from events 
            window.removeEventListener('resize', this._onWindowResize);
            // clear event handlers
            if (client.isDesktopDisplay) {
                client.KeyboardHandler.onKeypress.remove(this._onKeypressHandler);
            }
            _super.prototype.onDispose.call(this);
        };
        /** Renders the slideshow to the screen. */
        PhotoCastScreen.prototype.render = function () {
            var _this = this;
            // Make screen is in a valid state
            if (this.getScreenState() == 3 /* DISPOSED */) {
                client.logger.warn("Render called on DISPOSED screen ");
                return this;
            }
            if (!this.photoSlider) {
                // Render view
                this.empty();
                if (this.state !== 0 /* joining */) {
                    // Switch to dark background
                    $('body').toggleClass('dark-theme', true);
                    this.$el.toggleClass('splash', false).css({ position: 'fixed', left: '0', top: '0', right: '0', bottom: '0' }).render('CastScreen');
                    //----------------------
                    // Initialize tab command bar
                    var tabBarCommands = [
                        consts.commands.photos,
                        consts.commands.people,
                        consts.commands.messages
                    ];
                    // only allow people and message commands in display mode
                    if (client.startupMode == 0 /* desktop */) {
                        tabBarCommands = [consts.commands.people, consts.commands.messages];
                    }
                    // desktop display has no chrome at all
                    if (client.isDesktopDisplay) {
                        // Clearing commands will prevent tab bar from displaying
                        tabBarCommands = [];
                    }
                    var tabStyle = {
                        bar: 'commandBarBottom',
                        tab: 'commandBarTabBorder',
                        selected: 'commandTabSelected',
                        glyphSize: 40,
                        isTabbed: true,
                        allowNoActiveTab: true,
                        autoHideDuration: PhotoCastScreen.CHROME_DURATION
                    };
                    this.tabBar = new client.CommandBar(this.$el.find('.castCommandBar'), tabStyle);
                    this.tabBar.setCommands(tabBarCommands, _.bind(this.onTabClick, this));
                    this.chromeTopBar = new client.ChromeTopBar(this.$el.find('.castTopBar'), PhotoCastScreen.CHROME_DURATION, client.urlUpsell);
                    // LONG PRESS MENU
                    //----------------------
                    // Initialize side menu
                    if (client.startupMode != 0 /* desktop */) {
                        if (client.startupMode != 1 /* emulator */) {
                            this.menuCommands.push(consts.commands.save);
                        }
                        this.menuCommands.push(consts.commands.celebrate);
                        this.menuCommands.push(consts.commands.messages);
                    }
                    if (this.menuCommands.length > 0) {
                        this.hoverMenu = new client.HoverMenu(this.$el.find('.hoverMenuContent'), _.bind(this.onMenuClick, this));
                        this.hoverMenu.setCommands(this.menuCommands);
                    }
                    // If in emulation bring up hover menu on shift right click as can't long press in emulator
                    if (client.startupMode == 1 /* emulator */) {
                        this.$el.on('contextmenu', function (ev) {
                            if (ev.shiftKey) {
                                ev.preventDefault();
                                _this.hoverMenu.show({ x: ev.pageX, y: ev.pageY });
                            }
                        });
                    }
                    //-------------------------
                    // Initialize photo slider
                    if (!this.cast || !this.cast.queue) {
                        client.logger.warn('photocast has no cast or photos');
                        history.back();
                        return this;
                    }
                    var media = _.map(this.cast.queue.items || [], function (item) { return item.url; });
                    if (media.length) {
                        this.queueLength = media.length;
                        // Create slider
                        this.photoSlider = new client.PhotoSlider(this.$el.find(".castContent"), this, media, this.cast.state, {
                            preventDefault: client.startupMode != 4 /* ios */,
                            useFlippers: (client.startupMode == 1 /* emulator */) || (client.isDesktopDisplay && client.isInteractiveDisplay) // Add left/right flippers if desktop and interactive
                        });
                        // Listen for PhotoSlider events events
                        this.photoSlider.bind(client.PhotoSlider.SlideChangedEvent, _.bind(this.onPhotoSliderChanged, this));
                        this.photoSlider.bind(client.PhotoSlider.DoubleTapEvent, _.bind(this.onDoubleTap, this));
                        this.photoSlider.bind(client.PhotoSlider.SwipeOffEvent, _.bind(this.onSwipeOff, this));
                        if (client.startupMode != 0 /* desktop */) {
                            this.photoSlider.bind(client.PhotoSlider.LongPressEvent, _.bind(this.onLongpress, this));
                        }
                        this.photoSlider.eventCallback = _.bind(this.onSliderEvent, this);
                        // Provide call back to Bubble for rendering slider images
                        client.Bubble.getImageUrlCallback = function (id) {
                            return _this.photoSlider.getImageUrl(id);
                        };
                    }
                    //------------------------
                    // Initialize messages
                    var $popUpContent = this.$el.find('.castMessagePopUp');
                    this.messagePopUp = new client.MessagePopUp($popUpContent, _.bind(this.onMessageInput, this), _.bind(this.hideMessagePopUp, this));
                    this.messagePopUp.onClose.add(function () {
                        _this.tabBar.deselect(consts.commands.messages);
                        _this.tabBar.show(false);
                        _this.chromeTopBar.show(false);
                    });
                    this.initMessages();
                    //------------------------
                    // Initialize chrome
                    this.showChrome(true);
                    //------------------------
                    // Start tracking presence
                    this.cast.group.syncPresence(function (err, res) {
                        if (res) {
                            _this.groupPresence = res;
                            _this.groupPresence.onModelAdded.add(_.bind(_this.addPresence, _this));
                            _this.groupPresence.onModelRemoved.add(_.bind(_this.removePresence, _this));
                        }
                    });
                }
                else {
                    this.$el.toggleClass('splash', true).render('StartingCastScreen', { message: resources.messages.startingXim, isWp: client.startupMode == 3 /* wp */ });
                }
            }
            _super.prototype.render.call(this);
            this.$el.find('i.close-casting').toggleClass('hidden', true);
            return this;
        };
        /** Slider event has occured */
        PhotoCastScreen.prototype.onSliderEvent = function (ev) {
            // Capture slider events when hoverMenu is open
            if (this.hoverMenu && this.hoverMenu.isOpen()) {
                // Eat drag events to prevent slide from moving under hover menu
                if (ev.type == 'drag') {
                    this.hoverMenu.triggerHover(ev);
                    return true;
                }
                else if (ev.type == 'dragend') {
                    this.hoverMenu.triggerTouch(ev);
                    return true;
                }
            }
        };
        PhotoCastScreen.prototype.onKeypressHandler = function (keyEvent) {
            if (this.photoSlider) {
                switch (keyEvent) {
                    case 39 /* FORWARD */:
                        this.photoSlider.jumpToNextSlide();
                        break;
                    case 37 /* BACK */:
                        this.photoSlider.jumpToPreviousSlide();
                        break;
                    case 36 /* REWIND */:
                        this.photoSlider.jumpToFirstSlide();
                        break;
                    case 35 /* FAST_FORWARD */:
                        this.photoSlider.jumpToLastSlide();
                        break;
                    case 38 /* ZOOM_IN */:
                        this.photoSlider.setMaxZoom(true);
                        break;
                    case 40 /* ZOOM_OUT */:
                        this.photoSlider.setMaxZoom(false);
                        break;
                }
            }
        };
        PhotoCastScreen.prototype.addPresence = function (profile) {
            if (!profile.color) {
                client.logger.error(consts.errorCode.PhotoCasting, 'AddPresence: Profile missing color');
            }
            if (!profile.displayName) {
                client.logger.error(consts.errorCode.PhotoCasting, 'AddPresence: Profile missing display name');
            }
            // Show message to user
            var bubbleMessage = {
                text: profile.displayName + " has joined",
                profile: profile,
                isSystem: true
            };
            this.queueNewMessage(bubbleMessage);
            // Update people popup
            if (this.peoplePopUp) {
                this.peoplePopUp.updatePresence(this.groupPresence);
            }
        };
        PhotoCastScreen.prototype.removePresence = function (profile) {
            // Check to see if profile is user.
            if (profile.userId == client.user.profile.userId) {
                client.logger.error(consts.errorCode.Multicast, "User's presence removed during cast.");
                return;
            }
            // Show message to user
            var bubbleMessage = {
                text: profile.displayName + " has left",
                profile: profile,
                isSystem: true
            };
            this.queueNewMessage(bubbleMessage);
            // Update people popup
            if (this.peoplePopUp) {
                this.peoplePopUp.updatePresence(this.groupPresence);
            }
        };
        PhotoCastScreen.prototype.initMessages = function () {
            var filter = {
                limit: 50,
                postType: "message"
            };
            this.cast.group.listPosts(filter, this.ifAvailable(_.bind(this.onListPosts, this)));
        };
        /** When user clicks on celebrate */
        PhotoCastScreen.prototype.onCelebrate = function () {
            var imageId = this.photoSlider.getCurrentIndex();
            // If already as max amount skip
            if (client.ImageMetaData.getSuns(imageId, client.user.profile.userId) >= client.ImageMetaData.MAX_SUN_COUNT) {
                return;
            }
            // Post new sun comment
            var message = {
                text: null,
                creatorId: client.user.profile.userId,
                imageId: imageId,
                postType: 'message',
                type: 'post',
                hasSun: true,
                created: new Date().getTime(),
                isSystem: false
            };
            this.cast.group.createPost(message, _.bind(this.postCreated, this));
            //OLD CELEBRATION STYLE.  SAVE FOR NOW
            // this.triggerCelebrate();  
        };
        /** Called when intial list of messages is received */
        PhotoCastScreen.prototype.onListPosts = function (e, page) {
            //todo: paging.  Just getting intial 50 for now
            if (e) {
                client.logger.debug('onListPosts error = ', e.toString());
                return;
            }
            // Convert posts into bubble messages
            var messages = [];
            for (var i = 0; i < page.models.length; i++) {
                var bubbleMessage = this.messageToBubble(page.models[i]);
                if (bubbleMessage) {
                    messages.push(bubbleMessage);
                }
                // If message is image specific
                if (bubbleMessage.imageId !== null) {
                    // Comments are in reverse time order, so if comment already exists ignore older comment
                    if (client.ImageMetaData.getComment(bubbleMessage.imageId)) {
                        this.photoSlider.addComment(bubbleMessage.imageId, bubbleMessage.profile.userId, null, bubbleMessage.hasSun);
                    }
                    else {
                        this.photoSlider.addComment(bubbleMessage.imageId, bubbleMessage.profile.userId, bubbleMessage.text, bubbleMessage.hasSun);
                    }
                }
            }
            // Initialize the message popup with the bubble messages
            this.messagePopUp.initMessages(messages);
        };
        /** User has taken control */
        // OBSOLETE
        PhotoCastScreen.prototype.onTakeControl = function () {
            try {
                var state = this.photoSlider.getScreenState();
                state.presenterID = client.user.profile.userId;
                this.cast.updateState(client.Util.toModelTransforms(state));
                // People popup should show presenter change
                if (this.peoplePopUp) {
                    this.peoplePopUp.updateDisplay();
                }
                // Show message to users 
                var message = "Took control";
                this.onMessageInput(message, true);
            }
            catch (ex) {
                client.logger.debug('updateState throw', ex);
            }
            // Relink screens and hide alternative command bar, but don't move screen
            this.setScreenLink(true, false);
        };
        /** Response to PhotoImageSlider change event */
        PhotoCastScreen.prototype.onPhotoSliderChanged = function (ev, data) {
            if (data && this.cast && this.cast.group && this.cast.group.session) {
                // Clear old celebrate level
                data.celebrateLevel = 0;
                if (data.queueIndex !== null) {
                    try {
                        // NOTE: server will set 2 fields to identify user & client making the move 
                        //       when we apply this update: 
                        // data['agent'] = user.profile.userId;
                        // data['agentClient'] = this.cast.group.session.clientId;
                        this.cast.updateState(client.Util.toModelTransforms(data));
                    }
                    catch (ex) {
                        client.logger.debug('updateState throw', ex);
                    }
                }
                if (this.photoPopUp) {
                    this.updatePhotoPopup();
                }
            }
        };
        /** handle long press event*/
        PhotoCastScreen.prototype.onLongpress = function (e, position) {
            // todo: Until longPress in emulator is fixed (currently triggers on every click)
            if (client.startupMode == 1 /* emulator */) {
                return;
            }
            // Hide bars
            this.tabBar.deselect();
            this.tabBar.show(false);
            this.chromeTopBar.show(false);
            // find the right position to show the menu
            client.logger.info("user invokes long press: ", position);
            var clickPosition = _.clone(position);
            if (this.hoverMenu) {
                this.hoverMenu.show(clickPosition);
            }
        };
        PhotoCastScreen.prototype.bindCast = function () {
            var _this = this;
            _super.prototype.bindCast.call(this);
            this.$el.on('click', '.slideMenuContent', function () {
                if (_this.hoverMenu) {
                    _this.hoverMenu.hide();
                }
                _this.tabBar.deselect();
            });
            // Accept both tap and click when running in emulator, otherwise only tap
            var events = 'tap';
            if (client.startupMode == 1 /* emulator */) {
                events += ' click';
            }
            this.$el.on(events, _.bind(this.onTap, this));
        };
        /** Handle tap event, ignoring if a double click */
        PhotoCastScreen.prototype.onTap = function () {
            var _this = this;
            // Delay tap action waiting to see if user is going to double click
            if (!this.tapTimer) {
                this.tapTimer = setTimeout(function () {
                    // Clear the tap timer
                    _this.tapTimer = null;
                    // Show/hide chrome
                    if (_this.tabBar) {
                        if (_this.tabBar.isOpen()) {
                            _this.showChrome(false);
                        }
                        else {
                            _this.showChrome(true);
                        }
                    }
                    // hide context menu if shown
                    if (_this.hoverMenu) {
                        _this.hoverMenu.hide();
                    }
                }, 300);
            }
        };
        /** Clear out single tap response if double-tap is detected */
        PhotoCastScreen.prototype.onDoubleTap = function () {
            // Clear any un-triggered single-tap response
            if (this.tapTimer) {
                clearTimeout(this.tapTimer);
                this.tapTimer = null;
            }
        };
        PhotoCastScreen.prototype.showChrome = function (show, fadeOut) {
            var _this = this;
            if (fadeOut === void 0) { fadeOut = true; }
            // Clear any existing timout
            window.clearTimeout(this.chromeTimer);
            // Set footer visiblity
            if (this.tabBar) {
                this.tabBar.show(show, fadeOut);
            }
            if (this.chromeTopBar) {
                this.chromeTopBar.show(show, fadeOut);
            }
            if (this.photoSlider) {
                this.photoSlider.showSun(!show);
            }
            // Automatically hide chrome after a while
            if (show && fadeOut) {
                this.chromeTimer = setTimeout(function () {
                    _this.showChrome(false);
                }, PhotoCastScreen.CHROME_DURATION);
            }
            else {
                this.chromeTimer = null;
            }
        };
        /** Returns true is chrome is visible */
        PhotoCastScreen.prototype.isChromeVisible = function () {
            return (this.tabBar && this.tabBar.isOpen());
        };
        /** Put message in queue for display and store on message list */
        PhotoCastScreen.prototype.queueNewMessage = function (bubbleMessage) {
            // Ensure the bubble message isn't empty
            if (!bubbleMessage.hasSun && (!bubbleMessage.text || bubbleMessage.text.length == 0)) {
                client.logger.error(consts.errorCode.PhotoCasting, 'BubbleMessage has no text');
                return;
            }
            // If message is a comment on a specific image, notify the photoslider
            if (bubbleMessage.imageId !== null) {
                // Add comment
                this.photoSlider.addComment(bubbleMessage.imageId, bubbleMessage.profile.userId, bubbleMessage.text, bubbleMessage.hasSun);
            }
            // Record whether a sun message already exits, before adding it
            var hadSun = this.messagePopUp.hasSunMessage(bubbleMessage);
            // Add message the messagePopUp
            this.messagePopUp.addMessage(bubbleMessage);
            // If message is a sun 
            if (bubbleMessage.hasSun) {
                // If I'm on the current image, don't show popup
                if (bubbleMessage.imageId == this.photoSlider.getCurrentIndex()) {
                    return;
                }
                // If I've already shown a popup for this user/image before, don't do it again
                if (hadSun) {
                    return;
                }
            }
            // If messagePopup isn't open display in a message bubble
            if (!this.messagePopUp.isVisible()) {
                var $bubbleContent = this.$el.find('.bubbleContent');
                // If not a display, clicking on bubble opens message interface
                if (!client.isDesktopDisplay) {
                    bubbleMessage.callback = _.bind(this.onBubbleClicked, this);
                }
                var bubble = new client.Bubble($bubbleContent, bubbleMessage);
            }
        };
        /** Called when user has added added a new message to the Xim */
        PhotoCastScreen.prototype.onMessageInput = function (text, systemMessage) {
            if (systemMessage === void 0) { systemMessage = false; }
            if (text) {
                var message = {
                    text: text,
                    creatorId: client.user.profile.userId,
                    imageId: null,
                    postType: 'message',
                    type: 'post',
                    created: new Date().getTime(),
                    isSystem: systemMessage
                };
                this.cast.group.createPost(message, _.bind(this.postCreated, this));
            }
        };
        /** Post has been created.  Callback from cast.group.createGroup */
        PhotoCastScreen.prototype.postCreated = function (e, post) {
            if (e) {
                client.logger.error(consts.errorCode.PhotoCasting, 'post create error = ', e.message);
            }
        };
        /** Converts an IMessage to a BubbleMessage */
        PhotoCastScreen.prototype.messageToBubble = function (post) {
            if (!this.cast) {
                client.logger.warn("PhotoCastScreen: messageToBubble has no cast.");
                return null;
            }
            var author = this.cast.group.profile.members[post.creatorId];
            var bubbleMessage = {
                text: post.text,
                profile: author,
                imageId: post.imageId,
                hasSun: post.hasSun,
                isSystem: post.isSystem,
                callback: null
            };
            return bubbleMessage;
        };
        /** A new post has been created in the group */
        PhotoCastScreen.prototype.onNewPost = function (post) {
            var bubbleMessage = this.messageToBubble(post);
            if (bubbleMessage) {
                this.queueNewMessage(bubbleMessage);
            }
        };
        /* Show/hide Presenter bar */
        PhotoCastScreen.prototype.showPresenterBar = function (show) {
            if (show) {
                if (!this.presenterBar) {
                    var presenterBarCommands = [
                        consts.commands.rejoin,
                        consts.commands.present
                    ];
                    var presenterStyle = {
                        tab: 'commandBarTabBorder',
                        bar: 'commandBarBottom commandBarReverse',
                        selected: 'commandTabSelected',
                        isTabbed: true,
                        allowNoActiveTab: true,
                        glyphSize: 40,
                        textSize: 0
                    };
                    this.presenterBar = new client.CommandBar(this.$el.find('.castAltBar'), presenterStyle);
                    this.presenterBar.commandBarStyle = presenterStyle;
                    this.presenterBar.setCommands(presenterBarCommands, _.bind(this.onPresenterBarClick, this));
                    this.presenterBar.disableCommand(consts.commands.addPeople, true);
                }
                this.presenterBar.show(true);
            }
            else {
                this.presenterBar.deselect();
                this.presenterBar.show(false);
            }
        };
        /** Show/Hide Popup showing people watching the cast */
        PhotoCastScreen.prototype.showPeoplePopUp = function (show) {
            var _this = this;
            if (show) {
                if (!this.peoplePopUp) {
                    var userInputContent = this.$el.find('.castPeoplePopUp');
                    userInputContent.empty();
                    this.peoplePopUp = new client.PeoplePopUp(userInputContent, this.cast, _.bind(this.onAddPeople, this));
                    this.peoplePopUp.onClose.add(function () {
                        _this.tabBar.deselect(consts.commands.people);
                        _this.tabBar.show(false);
                        _this.chromeTopBar.show(false);
                    });
                }
                this.peoplePopUp.show(true);
                this.peoplePopUp.updatePresence(this.groupPresence);
            }
            else if (this.peoplePopUp) {
                this.peoplePopUp.show(false);
            }
        };
        /** Popup showing photos in the cast */
        PhotoCastScreen.prototype.showPhotoPopUp = function (show) {
            var _this = this;
            if (show) {
                if (!this.photoPopUp) {
                    var userInputContent = this.$el.find('.castPhotoPopUp');
                    userInputContent.empty();
                    this.photoPopUp = new client.PhotoPopUp(userInputContent, this.cast, _.bind(this.onPhotoPopUpCallback, this));
                    this.photoPopUp.onClose.add(function () {
                        _this.tabBar.deselect(consts.commands.photos);
                        _this.tabBar.show(false);
                        _this.chromeTopBar.show(false);
                    });
                }
                this.photoPopUp.show(true);
                this.updatePhotoPopup();
            }
            else if (this.photoPopUp) {
                this.photoPopUp.show(false);
            }
        };
        /** hide all popups */
        PhotoCastScreen.prototype.hidePopUps = function () {
            if (this.peoplePopUp) {
                this.peoplePopUp.show(false);
            }
            if (this.photoPopUp) {
                this.photoPopUp.show(false);
            }
            if (this.messagePopUp) {
                this.showMessagePopUp(false, false);
            }
        };
        PhotoCastScreen.prototype.showMessagePopUp = function (show, expanded) {
            if (show) {
                $('body').toggleClass('ms-overflow-none', true);
                this.messagePopUp.show(true);
                if (expanded) {
                    this.messagePopUp.showDetail(true);
                }
            }
            else {
                this.messagePopUp.show(false);
                $('body').toggleClass('ms-overflow-none', false);
            }
        };
        /** Called when message popup is closed.  imageId is non-null if user clicked on an image */
        PhotoCastScreen.prototype.hideMessagePopUp = function (imageId) {
            // Deselect active tab in command bar
            this.tabBar.deselect();
            // Close message pop up
            this.showMessagePopUp(false, false);
            // If image was returned go to that image
            if (imageId !== null) {
                this.photoSlider.jumpToSlide(imageId);
            }
        };
        /** On window resize, trigger relayout */
        PhotoCastScreen.prototype.onWindowResize = function () {
            this.photoSlider.onWindowResize();
            if (this.peoplePopUp) {
                this.peoplePopUp.resize();
            }
            if (this.photoPopUp) {
                this.photoPopUp.resize();
            }
            if (this.messagePopUp) {
                this.messagePopUp.resize();
            }
        };
        /** On user commenting on a particular photo */
        PhotoCastScreen.prototype.onComment = function (text) {
            this.commentPopUp = null;
            if (!text) {
                return;
            }
            else {
                var imageId = this.photoSlider.getCurrentIndex();
                // Add comment to display
                this.photoSlider.addComment(imageId, client.user.profile.id, text);
                // Post comment to other users
                var message = {
                    text: text,
                    creatorId: client.user.profile.userId,
                    imageId: imageId,
                    postType: 'message',
                    type: 'post',
                    created: new Date().getTime(),
                    isSystem: false
                };
                this.cast.group.createPost(message, _.bind(this.postCreated, this));
            }
        };
        /** An option on the side menu has been selected */
        PhotoCastScreen.prototype.onMenuClick = function (id) {
            // Deselect the tab
            this.tabBar.deselect();
            // Hide hover menu
            if (this.hoverMenu) {
                this.hoverMenu.hide();
            }
            switch (id) {
                case consts.commands.celebrate.id:
                    this.onCelebrate();
                    break;
                case consts.commands.save.id:
                    this.saveImage();
                    break;
                case consts.commands.messages.id:
                    this.commentPopUp = new client.CommentPopUp(_.bind(this.onComment, this));
                    this.commentPopUp.show(true);
                    break;
                default:
                    client.logger.info("Unhandled menu click: " + id);
            }
        };
        PhotoCastScreen.prototype.triggerCelebrate = function () {
            var state = this.photoSlider.getScreenState();
            state.celebrateLevel = 1;
            try {
                this.cast.updateState(client.Util.toModelTransforms(state));
            }
            catch (ex) {
                client.logger.debug('updateState throw', ex);
            }
        };
        PhotoCastScreen.prototype.celebrate = function (level) {
            var $particleEffect = this.$el.find('.debugEffect');
            // For now don't do anything with celebrate level
            client.ParticleMaster.show($particleEffect, 3);
        };
        /** Presenter bar was clicked */
        PhotoCastScreen.prototype.onPresenterBarClick = function (id) {
            switch (id) {
                case consts.commands.rejoin.id:
                    this.setScreenLink(true);
                    break;
                case consts.commands.present.id:
                    this.onTakeControl();
                    break;
                default:
                    client.logger.info("Unhandled command click: " + id);
            }
        };
        /** An item on the tab menu was selected */
        PhotoCastScreen.prototype.onTabClick = function (id) {
            var _this = this;
            // Hide any open mesasage bubble
            client.Bubble.hideMessage();
            // Hide top chrome bar 
            this.chromeTopBar.show(false);
            if (id != consts.commands.messages.id) {
                this.showMessagePopUp(false, false);
            }
            if (id != consts.commands.people.id) {
                this.showPeoplePopUp(false);
            }
            if (id != consts.commands.photos.id) {
                this.showPhotoPopUp(false);
            }
            switch (id) {
                case consts.commands.messages.id:
                    client.AppMasterScreen.checkConnection(function () {
                        _this.showMessagePopUp(true, false);
                    }, function () {
                        _this.tabBar.deselect();
                    });
                    break;
                case consts.commands.people.id:
                    client.AppMasterScreen.checkConnection(function () {
                        _this.showPeoplePopUp(true);
                    }, function () {
                        _this.tabBar.deselect();
                    });
                    break;
                case consts.commands.photos.id:
                    client.AppMasterScreen.checkConnection(function () {
                        _this.showPhotoPopUp(true);
                    }, function () {
                        _this.tabBar.deselect();
                    });
                    break;
                default:
                    client.logger.info("Unhandled command click: " + id);
            }
        };
        PhotoCastScreen.prototype.onQueueUpdated = function (queue) {
            var _this = this;
            // since we only allow queue append
            var extraPhotos = _.map(_.filter(queue.items, function (item, i) {
                return i >= _this.queueLength;
            }), function (item) {
                return item.url;
            });
            if (extraPhotos.length) {
                this.photoSlider.addMedia(extraPhotos);
                this.queueLength += extraPhotos.length;
                this.updatePhotoPopup();
            }
        };
        PhotoCastScreen.prototype.onPhotoUploaded = function () {
            if (this.photoSlider) {
                this.photoSlider.loadNearbyImages();
            }
        };
        PhotoCastScreen.prototype.addPhotos = function (callback) {
            var _this = this;
            client.casting.addMoreContent(function (e, spec) {
                if (!e) {
                    if (spec.photoUploaded) {
                        spec.photoUploaded.add(function () {
                            _this.onPhotoUploaded();
                        });
                    }
                    client.logger.collectNamedEvent(consts.flurryEvent.ximsessionAddPicture, { count: spec.items ? spec.items.length : 0 });
                    spec.mapToScreenQueueItems(spec.items, function (e, items) {
                        if (items && items.length) {
                            if (_this.cast) {
                                _this.cast.updateQueue({ $pushAll: { 'items': items } }, function (e) {
                                    if (e) {
                                        callback(e, 0);
                                    }
                                    else {
                                        callback(null, items.length);
                                    }
                                });
                            }
                            else {
                                callback(new Error("PhotoCastScreen: AddPhotos has no cast"), null);
                            }
                        }
                        else {
                            callback(new Error("PhotoCastScreen: AddPhotos has no items"), null);
                        }
                    });
                }
                else {
                    callback(e, 0);
                }
            });
        };
        /** handle when user clicks on add photo or take a single photo from camera*/
        PhotoCastScreen.prototype.onPhotoPopUpCallback = function (useCamera) {
            // Clear selected tab
            this.showPhotoPopUp(false);
            this.tabBar.deselect();
            if (useCamera) {
                this.onAddCameraPhoto();
            }
            else {
                this.onAddPhotos();
            }
        };
        /** use the camera to take a single photo and add it to the end of media queue */
        PhotoCastScreen.prototype.onAddCameraPhoto = function () {
            var _this = this;
            var $action = $(event.currentTarget);
            client.casting.selectSinglePhoto(function (e, previewItem) {
                if (!e && previewItem.thumbnail) {
                    client.CameraRollUploader.uploadFileUrlToBlob(previewItem.thumbnail.source, function (e, url) {
                        // now add the url to the media queue
                        // first convert url to a IScreenQueueItem
                        if (!e && url) {
                            var item = { type: 'photo', url: url };
                            if (_this.cast) {
                                _this.cast.updateQueue({ $pushAll: { 'items': [item] } }, function (e) {
                                    if (e) {
                                        client.logger.error(consts.errorCode.PhotoCasting, 'update cast queue', e);
                                    }
                                    else {
                                        _this.onMessageInput(resources.messages.addedCamera, true);
                                    }
                                });
                            }
                            else {
                                client.logger.error(consts.errorCode.PhotoCasting, 'no cast');
                            }
                        }
                        else {
                            client.logger.error(consts.errorCode.PhotoCasting, 'camera uploadFileUrlToBlob', e);
                        }
                    });
                }
                else {
                    // only log as info as user might just change his mind 
                    // and backed out
                    client.logger.info('addCameraPhoto failed', e);
                }
            });
        };
        // Bring up photo picking dialog
        PhotoCastScreen.prototype.onAddPhotos = function () {
            var _this = this;
            // Clear selected tab
            this.showPhotoPopUp(false);
            this.tabBar.deselect();
            client.AppMasterScreen.checkConnection(function () {
                // Lock screen back to vertical while picking photos
                if (window.screenlock) {
                    window.screenlock.set(consts.ScreenLockState.vertical);
                }
                _this.addPhotos(function (e, count) {
                    // Unlock screen so user can rotate again
                    if (window.screenlock) {
                        window.screenlock.clear();
                    }
                    if (e && e.message != client.wizard.USER_BACKED_OUT) {
                        client.logger.error(consts.errorCode.PhotoCasting, 'add more photo error = ', e.toString());
                        var popup = new client.ErrorPopUp(client.ErrorMessages.PHOTO_ADD_FAIL, function () {
                        });
                        popup.show(true);
                    }
                    else if (count > 0) {
                        var message = "Added " + count + ((count > 1) ? " images" : " image");
                        _this.onMessageInput(message, true);
                    }
                });
            });
        };
        // Bring up people picking dialog
        PhotoCastScreen.prototype.onAddPeople = function () {
            var _this = this;
            // Clear selected tab
            this.showPeoplePopUp(false);
            this.tabBar.deselect();
            // Lock screen back to vertical while picking people
            if (window.screenlock) {
                window.screenlock.set(consts.ScreenLockState.vertical);
            }
            // People picker shouldn't show people already in the cast
            var excludedProfiles = _.map(this.cast.group.profile.members, function (member) {
                return member;
            });
            client.casting.addMorePeople(excludedProfiles, function (e, spec) {
                // Unlock screen so user can rotate again
                if (window.screenlock) {
                    window.screenlock.clear();
                }
                // Handle errors
                if (e && e.message != client.wizard.USER_BACKED_OUT) {
                    client.logger.error(consts.errorCode.PhotoCasting, 'Add more people error = ', e.toString());
                    var popup = new client.ErrorPopUp(client.ErrorMessages.PEOPLE_ADD_FAIL, function () {
                    });
                    popup.show(true);
                    return;
                }
                else if (!spec || !spec.invites || !spec.invites.length) {
                    return;
                }
                var invites = spec.invites;
                // Invite users
                _this.cast.group.inviteUsers(invites, function (err) {
                    if (err) {
                        client.logger.error(consts.errorCode.PhotoCasting, 'Invite more people error = ', err.toString());
                        popup = new client.ErrorPopUp(client.ErrorMessages.PEOPLE_ADD_FAIL, function () {
                        });
                        popup.show(true);
                    }
                    else {
                        // Display toast
                        var count = invites.length;
                        var message = "Invited " + count + ((count > 1) ? " people" : " person") + " to join";
                        _this.onMessageInput(message, true);
                    }
                });
            });
        };
        PhotoCastScreen.prototype.saveImage = function () {
            var _this = this;
            client.logger.collectNamedEvent(consts.flurryEvent.ximsessionSaveImage);
            // Get current image location
            var imageUrl = this.photoSlider.currentImageURL();
            // Generate file name from date and time
            var fileName = consts.applicationName + "_" + new Date().getTime();
            var dirName = consts.applicationName;
            // Notify this user with a pop up
            this.queueNewMessage({ text: "Saving Image To Device", profile: client.user.profile, isSystem: true });
            // Download file to local device
            var filedownload = new client.FileDownload(imageUrl, dirName, fileName);
            filedownload.download(function () {
                // Share "system" message with self and others
                _this.onMessageInput("Saved an Image", true);
            }, function (error) {
                // Notify this user with pop up
                _this.queueNewMessage({ text: "Image Save Failed", profile: null, isSystem: true });
                client.logger.error(consts.errorCode.FileDownload, 'Image Save Failed: ' + error);
            });
        };
        // Returns true is command icon isn't visible
        PhotoCastScreen.prototype.isCommandVisible = function () {
            return ((this.hoverMenu && this.hoverMenu.isOpen()));
        };
        PhotoCastScreen.prototype.setScreenLink = function (mode, move) {
            if (move === void 0) { move = true; }
            // Sreen linking/unlinking is currently disabled
            return;
            /*
            if (mode == true && !this.photoSlider.opts.isLinked) {

                // Hide alternative command bar
                this.showPresenterBar(false);

                var screenState = <IPhotoCastScreenState>this.cast.state;
                this.photoSlider.opts.isLinked = true;

                if (move) {
                    if (this.photoSlider) {
                    this.photoSlider.moveTo(screenState.queueIndex, screenState.imgScale, screenState.imgXOffset, screenState.imgYOffset);
                        return;
                    } else {
                        logger.error(consts.errorCode.PhotoCasting, 'Photocast has no photoSlider');
                        return;
                    }
                }

            } else if (mode == false && this.photoSlider.opts.isLinked){
                
                // Show alternative command bar
                this.showPresenterBar(true);
                this.photoSlider.opts.isLinked = false;
            }
            */
        };
        /** The screens state has changed. */
        PhotoCastScreen.prototype.onStateUpdated = function (state) {
            // Check that photo cast screen hasn't been disposed
            if (this.getScreenState() == 3 /* DISPOSED */) {
                return;
            }
            // Ignore swipe events if hover menu is open
            if (this.hoverMenu && this.hoverMenu.isOpen()) {
                return;
            }
            // Ignore swipe events if comment UI is open
            if (this.commentPopUp) {
                return;
            }
            if (this.photoSlider) {
                // Ignore my own moves
                // - Don't need to also look at the 'agent' here because the 'agentClient' is always unique. 
                //   If you want to see if the changes came from this screen then you can simply ignore anything 
                //   where agentClient matches teh session clientId. 
                if (state['agentClient'] != this.cast.group.session.clientId) {
                    this.photoSlider.requestMove(state);
                }
            }
            else {
                client.logger.warn('PhotoCastScreen: onStateUpdated called with no PhotoSlider');
                return;
            }
            if (state.celebrateLevel > 0) {
                this.celebrate(state.celebrateLevel);
            }
            if (this.peoplePopUp) {
                this.peoplePopUp.updateDisplay();
            }
            this.updatePhotoPopup();
        };
        PhotoCastScreen.prototype.updatePhotoPopup = function () {
            if (this.photoPopUp && this.photoSlider) {
                var imageCount = this.photoSlider.getImageCount();
                var curIndex = this.photoSlider.getCurrentIndex() + 1;
                this.photoPopUp.updateDisplay(imageCount, curIndex);
            }
        };
        /** Called when message bubble is clicked.  imageId is non-null if user clicked on an image */
        PhotoCastScreen.prototype.onBubbleClicked = function (imageId) {
            // Hide bubble message
            client.Bubble.hideMessage();
            // If image was returned go to that image
            if (imageId !== null) {
                this.photoSlider.jumpToSlide(imageId);
            }
            else {
                this.tabBar.select(consts.commands.messages);
                this.tabBar.show(true);
                this.showMessagePopUp(true, true);
            }
        };
        // Triggered when user swipes off last photo, bring up photo tab
        PhotoCastScreen.prototype.onSwipeOff = function () {
            // If desktop show upsell
            if (client.startupMode == 0 /* desktop */) {
                this.chromeTopBar.show(true);
            }
            else {
                this.tabBar.select(consts.commands.photos);
                this.tabBar.show(true);
                this.chromeTopBar.show(false);
                this.showPhotoPopUp(true);
            }
        };
        /** Starts a new group & cast in one step. */
        PhotoCastScreen.startXim = function (photoSpec, onClosed) {
            var options = { castType: 'photocast', settings: {}, queue: { items: [] }, state: { queueIndex: 0, presenterID: client.user.profile.userId } };
            var prepareCast = function (screen, next) {
                photoSpec.mapToScreenQueueItems(photoSpec.items, function (e, images) {
                    if (!e) {
                        options.queue.items = images;
                        client.logger.collect(consts.tags.cast.invite, {
                            mediaLength: images ? images.length : 0,
                            groupSize: photoSpec.invites ? photoSpec.invites.length : 0
                        });
                        next(null, options);
                        if (photoSpec.photoUploaded) {
                            photoSpec.photoUploaded.add(function () {
                                screen.onPhotoUploaded();
                            });
                        }
                    }
                    else {
                        client.logger.debug('mapping photos to images failed', e.toString());
                        next(e, null);
                    }
                });
            };
            var screen = PhotoCastScreen.display(onClosed).startXim(photoSpec.invites, prepareCast, function (err, success) {
                if (err) {
                    // Xim failed to start
                    screen.closeAsync()(err, null);
                }
            });
            return screen;
        };
        PhotoCastScreen.startCast = function (grp, photoSpec, callback) {
            var options = { castType: 'photocast', settings: {}, queue: { items: [] }, state: { queueIndex: 0, presenterID: client.user.profile.userId } };
            var prepareCast = function (screen, next) {
                photoSpec.mapToScreenQueueItems(photoSpec.items, function (e, images) {
                    if (!e) {
                        options.queue.items = images;
                        client.logger.collect(consts.tags.cast.invite, {
                            mediaLength: images ? images.length : 0,
                            groupSize: photoSpec.invites ? photoSpec.invites.length : 0
                        });
                        next(null, options);
                        if (photoSpec.photoUploaded) {
                            photoSpec.photoUploaded.add(function () {
                                screen.onPhotoUploaded();
                            });
                        }
                    }
                    else {
                        client.logger.debug('mapping photos to images failed', e.toString());
                        next(e, null);
                    }
                });
            };
            var cb = _.once(callback || function () {
            });
            return PhotoCastScreen.display(cb).startCast(grp, prepareCast, function (e) {
                // TODO: Should close dialog instead
                if (e)
                    cb(e, null);
            });
        };
        PhotoCastScreen.joinCast = function (grp, castId, callback) {
            var cb = _.once(callback || function () {
            });
            return PhotoCastScreen.display(callback).joinCast(grp, castId, function (e) {
                // TODO: Should close dialog instead
                if (e)
                    cb(e, null);
            });
        };
        /**
          * Displays a new Photo Casting screen as a popup.
          * @param callback (optional) The callback to call once the screen has been dropped.
          */
        PhotoCastScreen.display = function (callback) {
            return client.screens.showModal(PhotoCastScreen, { style: 2 /* fullScreen */ }, function (e, results) {
                if (callback)
                    callback(e, results);
                return false;
            });
        };
        PhotoCastScreen.CHROME_DURATION = 5000;
        return PhotoCastScreen;
    })(client.CastingScreen);
    client.PhotoCastScreen = PhotoCastScreen;
})(client || (client = {}));
/// <reference path="CastingScreen.ts"/>
var client;
(function (client) {
    /** View for a photo cast session. */
    var DocCastScreen = (function (_super) {
        __extends(DocCastScreen, _super);
        function DocCastScreen() {
            _super.apply(this, arguments);
        }
        /** Renders the embedded doc to the screen. */
        DocCastScreen.prototype.render = function () {
            // Make screen is in a valid state
            if (this.getScreenState() == 3 /* DISPOSED */) {
                client.logger.warn("Render called on DISPOSED screen ");
                return this;
            }
            // Render view
            this.empty();
            $('body').addClass('dark-theme');
            if (this.state !== 0 /* joining */) {
                // Add media to page
                var fullScreenCss = { position: 'fixed', left: '0px', top: '0px', right: '0px', bottom: '0px' };
                this.$el.toggleClass('splash', false).css(fullScreenCss);
                var startAt = this.cast.state.queueIndex || 0;
                var media = this.cast.queue.items || [];
                for (var i = 0; i < media.length; i++) {
                    var doc = media[i].url;
                    $('<iframe class="doc-casting"></iframe>').attr('src', doc).css(fullScreenCss).appendTo(this.$el);
                    break;
                }
            }
            else {
                this.$el.toggleClass('splash', true).render('StartingCastScreen', { message: 'Starting Xim' });
            }
            return _super.prototype.render.call(this);
        };
        DocCastScreen.prototype.bindDocViewer = function (ref) {
            var _this = this;
            ref.addEventListener('exit', function (e) {
                // close the doc viewer, we need to exit the cast screen
                client.logger.info('inapp browser closing');
                client.dialog.handleDialogCommand(consts.commands.cancel.id, consts.commands.cancel);
            });
            ref.addEventListener('loadstop', function (e) {
                var urlStr = e.url;
                client.logger.info('inapp browser loaded', urlStr);
                var url = uri.parse(urlStr);
                var browser = ref;
                if (url.host == 'onedrive.live.com') {
                    setTimeout(function () {
                        browser.executeScript({ code: "document.querySelector('#embedOfficeFrame').getAttribute('src')" }, function (frameUrl) {
                            client.logger.info('inner frame url', frameUrl);
                            var innerRef = window.open(frameUrl, '_blank', 'location=no');
                            _this.bindDocViewer(innerRef);
                        });
                    }, 500);
                }
                if (url.host == 'powerpoint.officeapps.live.com') {
                    client.logger.info('about to inject script to browser for behavior modification');
                    setTimeout(function () {
                        browser.executeScript({ code: "(function(d){var s=document.querySelector('script#powerpointcast');if (!s){var e=d.createElement('script');e.setAttribute('type','text/javascript');e.setAttribute('charset','UTF-8');e.setAttribute('id', 'powerpointcast');e.setAttribute('src','//amesscratch2.blob.core.windows.net/hailiu/powerpoint-onedrive.js?r='+Math.random()*99999999);d.body.appendChild(e)} return 'inject script ok'; })(document);" });
                    }, 300);
                }
            });
        };
        /**
          * Initiates a document cast where the current user is the leader
          * @param grp Group to cast to.
          * @param url The URL of the document to cast.
          * @param callback (optional) The callback to call once the screen has been dropped.
          */
        DocCastScreen.startCast = function (grp, urlFactory, callback) {
            // Get list of URL's
            var cb = _.once(callback || function (e, r) {
            });
            var screen = DocCastScreen.display(cb);
            urlFactory(function (err, urls) {
                if (!err) {
                    var items = _.map(urls, function (url) {
                        return { type: 'doc', url: url };
                    });
                    screen.startCast(grp, { castType: 'doccast', queue: { items: items }, state: { queueIndex: 0 } }, function (e) {
                        if (e)
                            cb(e, null);
                    });
                }
                else {
                    cb(err, null);
                }
            });
            return screen;
        };
        /**
          * Joins an existing document cast
          * @param id The group to cast to.
          * @param leader The leader of the casting session.
          * @param callback (optional) The callback to call once the screen has been dropped.
          */
        DocCastScreen.joinCast = function (grp, castId, callback) {
            var cb = _.once(callback || function (e, r) {
            });
            return DocCastScreen.display(cb).joinCast(grp, castId, function (e) {
                if (e)
                    cb(e, null);
            });
        };
        /**
          * Displays a new Document Casting screen as a popup.
          * @param callback (optional) The callback to call once the screen has been dropped.
          */
        DocCastScreen.display = function (callback) {
            return client.screens.showModal(DocCastScreen, { style: 2 /* fullScreen */ }, function (e, results) {
                if (callback)
                    callback(e, results);
                return false;
            });
        };
        return DocCastScreen;
    })(client.CastingScreen);
    client.DocCastScreen = DocCastScreen;
})(client || (client = {}));
/// <reference path="app.ts"/>
/// <reference path="../../types/cordova/cameraroll.d.ts" />
var client;
(function (client) {
    /** Camera Roll Uploader
    * given a set of photoData and set of ICameraRollUrls
    * first upload the first photo and then batch upload
    * the remaing photos, raise photoUploaded event
    */
    var CameraRollUploader = (function () {
        function CameraRollUploader(photoUploaded, photos, urls) {
            this.photoUploaded = photoUploaded;
            this.photos = photos;
            this.urls = urls;
            this.ids = _.map(_.pluck(photos, 'id'), function (pid) {
                return (pid && (pid.indexOf('cameraroll_') == 0)) ? pid.substr(11) : pid;
            });
            this.assetUrls = _.filter(_.pluck(photos, 'assetUrl'), function (v) {
                return !_.isUndefined(v);
            });
            this.names = _.pluck(photos, 'name');
        }
        CameraRollUploader.prototype.batchUpload = function (batch, callback) {
            var _this = this;
            // if we have assetUrl returned from the native plugin 
            // we need to pass that back to the plugin for upload
            // otherwise, using the 'name' field
            window.window.cameraRoll.getPhotoMedias(_.map(this.urls.slice(0, batch), function (u) {
                return u.writeUrl;
            }), this.ids.slice(0, batch), this.assetUrls.length ? this.assetUrls.slice(0, batch) : this.names.slice(0, batch), function (uploaded) {
                if (_.isString(uploaded)) {
                    uploaded = JSON.parse(uploaded);
                }
                client.logger.info('photo uploads count = ', uploaded.length);
                _this.photoUploaded.trigger({
                    photos: uploaded,
                    urls: _this.urls.slice(0, batch)
                });
                _this.urls = _this.urls.slice(batch);
                _this.ids = _this.ids.slice(batch);
                _this.names = _this.names.slice(batch);
                _this.assetUrls = _this.assetUrls.slice(batch);
                callback(null, _this.urls.length);
            }, function (e) {
                client.logger.info('photo upload failed with', e);
                callback(new Error(e ? e : 'photo upload unknown error'), 0);
            });
        };
        CameraRollUploader.prototype.start = function () {
            var self = this;
            var continueUpload = function (e, r) {
                if (!e) {
                    if (r > 0) {
                        self.batchUpload(Math.min(r, 5), continueUpload);
                    }
                }
            };
            this.batchUpload(1, continueUpload);
        };
        /** Given list of image urls, return blob names to use for those urls */
        CameraRollUploader.getBlobNames = function (names, callback) {
            // makesure the blobNames are clean
            var blobNames = _.map(names, function (x) {
                return x.replace(/[%\?:\\]/g, '_');
            }).join(',');
            client.server.POST('api/cameraRollUrls', null, { blobNames: blobNames }, function (err, data) {
                if (data && !err) {
                    callback(null, data.urls);
                }
                else {
                    callback(err, null);
                }
            });
        };
        /** upload a file on the phone to blob*/
        CameraRollUploader.uploadFile = function (fileUrl, blobUrl, callback) {
            window.cameraRoll.uploadFile(fileUrl, blobUrl, function () {
                callback(null, true);
            }, function (e) {
                callback(new Error(e || 'uploadFile failed'), false);
            });
        };
        /** upload a single file to our azure blob storage*/
        CameraRollUploader.uploadFileUrlToBlob = function (fileUrl, callback) {
            // get the leaf name of the fileUrl
            var leafName = fileUrl.substr(fileUrl.lastIndexOf('/') + 1);
            var prefix = 'd_' + moment() + '_';
            async.waterfall([
                // get the url to upload
                function (c) {
                    CameraRollUploader.getBlobNames([prefix + leafName], c);
                },
                function (blobUrls, c) {
                    var blobUrl = blobUrls[0];
                    CameraRollUploader.uploadFile(fileUrl, blobUrl.writeUrl, function (e) {
                        c(e, (!e) ? blobUrl.readUrl : null);
                    });
                }
            ], callback);
        };
        /** Creates cue items, and triggers upload to server */
        CameraRollUploader.createScreenQueueItems = function (items, photoUploadedEvent, callback) {
            async.waterfall([
                function (c) {
                    var nameArray = _.pluck(items, 'name');
                    CameraRollUploader.getBlobNames(nameArray, function (err, blobNames) {
                        if (!err && blobNames) {
                            c(null, blobNames);
                        }
                        else {
                            callback(err, null);
                        }
                    });
                },
                function (urls, c) {
                    var map = {};
                    _.each(urls, function (url) {
                        map[url.writeUrl] = url.readUrl;
                    });
                    var firstUpload = _.once(function (uploaded) {
                        var photos = _.map(items, function (item, i) {
                            return {
                                id: item.id,
                                name: item.name,
                                thumbnail: {
                                    width: 0,
                                    height: 0,
                                    source: urls[i].readUrl
                                }
                            };
                        });
                        c(null, photos);
                    });
                    var uploader = new CameraRollUploader(photoUploadedEvent, items, urls);
                    uploader.photoUploaded.add(firstUpload);
                    uploader.start();
                }
            ], function (e, photos) {
                if (e) {
                    callback(e, null);
                }
                else {
                    callback(null, _.map(photos, function (photo) {
                        return {
                            type: 'photo',
                            url: photo.thumbnail.source,
                            id: photo.id,
                            name: photo.name
                        };
                    }));
                }
            });
        };
        return CameraRollUploader;
    })();
    client.CameraRollUploader = CameraRollUploader;
})(client || (client = {}));
///<reference path='../../../types/cordova/CameraRoll.d.ts'/>
var client;
(function (client) {
    // Returns mock camera roll photos
    var MockCameraRoll = (function () {
        function MockCameraRoll() {
        }
        MockCameraRoll.prototype.enumerateAlbum = function (parentId, prevId, batchSize, success, error) {
            var items = [];
            for (var i = 0; i < batchSize; i++) {
                // Vary height and width to get a variety of images
                var width = (160 + client.Util.getRandomInt(20));
                var height = (160 + client.Util.getRandomInt(20));
                var imageSrc = "http://lorempixel.com/" + width + "/" + height + "/people/";
                var image = { source: imageSrc, width: width, height: height };
                var photo = { id: parseInt(prevId) + i, thumbnail: image };
                items.push({ type: 'photo', photo: photo });
            }
            success(items);
        };
        MockCameraRoll.prototype.getPhotoMedias = function (urls, ids, names, success, error) {
            throw 'Mock not implemented';
        };
        /** upload a single file url to the specified blob url*/
        MockCameraRoll.prototype.uploadFile = function (fileUrl, blobUrl, success, error) {
            throw 'Mock not implemented';
        };
        MockCameraRoll.numContacts = 250;
        MockCameraRoll.contacts = [];
        return MockCameraRoll;
    })();
    client.MockCameraRoll = MockCameraRoll;
})(client || (client = {}));
/// <reference path="AppMasterScreen.ts"/>
/// <reference path='./ContentPickingDialog.ts'/>
/// <reference path='../CameraRollUploader.ts'/>
/// <reference path='../mockobjects/MockCameraRoll.ts'/>
/// <reference path="../../../types/cordova/cameraroll.d.ts" />
/// <reference path="../../shared/capability.ts" />
var client;
(function (client) {
    /** Camera screen. */
    var CameraRollScreen = (function (_super) {
        __extends(CameraRollScreen, _super);
        function CameraRollScreen(sel, history, commands, commandCallback) {
            _super.call(this, sel, history, commands, commandCallback);
            this.history = history;
            this.commands = commands;
            this.commandCallback = commandCallback;
            this.searchTerm = '';
            // the first batch are counting for the 'all' and 'more' button
            // besides, we already fetch 1 more than we want to show (to check if the more button should be rendered)
            this.batchSize = CameraRollScreen.rowsPerPage * CameraRollScreen.colsPerRow - 1;
            this.albumId = null;
            this.defaultTitle = CameraRollScreen.defaultTitle();
            this.contentSource = 'cameraroll';
            this.photoUploaded = new client.Event();
        }
        CameraRollScreen.defaultTitle = function () {
            var crTitle = resources.labels.pickerSource.photos;
            switch (client.startupMode) {
                case 3 /* wp */:
                    crTitle = resources.labels.pickerSource.pictures;
                    break;
                case 4 /* ios */:
                    crTitle = resources.labels.pickerSource.photos;
                    break;
                case 5 /* droid */:
                    if (client.capability.androidVersion() >= 5) {
                        crTitle = resources.labels.pickerSource.photos;
                    }
                    else {
                        crTitle = resources.labels.pickerSource.gallery;
                    }
                    break;
            }
            return crTitle;
        };
        CameraRollScreen.prototype.getNextItemBatch = function (callback) {
            var _this = this;
            var lastItem = _.last(this.items.models);
            var lastItemId = lastItem.id;
            if (lastItemId.indexOf('cameraroll_') == 0) {
                lastItemId = lastItemId.substr(11);
            }
            var cameraRollProvider = this.cameraRollProvider();
            cameraRollProvider.enumerateAlbum(this.albumId, lastItemId, this.batchSize, function (items) {
                client.logger.info('got more photos from camera roll');
                if (_.isString(items)) {
                    items = JSON.parse(items);
                }
                var models = _this.processItems(items);
                _this.$el.find('#camerarollNext').parent().remove();
                _this.appendTiles(_this.getTiles({ models: models, next: _this.items.next }, true));
            }, function (e) {
                client.logger.warn("camera roll plugin failed with", e);
            });
        };
        CameraRollScreen.prototype.processItems = function (items) {
            if (items.length == this.batchSize) {
                // we got more to show
                this.items.next = _.bind(this.getNextItemBatch, this);
                items.pop();
            }
            else {
                this.items.next = null;
            }
            var models = _.map(items, function (item) {
                if (item.type == 'photo') {
                    var photo = item.photo;
                    return {
                        type: 'photo',
                        id: _.isNumber(photo.id) ? 'cameraroll_' + photo.id : photo.id,
                        name: photo.name,
                        thumbnail: photo.thumbnail,
                        assetUrl: photo.assetUrl
                    };
                }
                else {
                    var album = item.album;
                    return {
                        type: 'folder',
                        id: album.id,
                        name: album.name,
                        thumbnail: album.thumbnail
                    };
                }
            });
            this.items.models = this.items.models.concat(models);
            return models;
        };
        CameraRollScreen.prototype.navigate = function (route) {
            var _this = this;
            // Update screens state
            client.logger.info('cameraRoll navigate start');
            this.albumId = route.params ? route.params['itemid'] : null;
            this.bag = this.bag || route.bag;
            this.setTitleFromRoute(route);
            this.items = {
                models: [
                ]
            };
            var cameraRollProvider = this.cameraRollProvider();
            this.batchSize = CameraRollScreen.rowsPerPage * CameraRollScreen.colsPerRow - 1;
            if (cameraRollProvider) {
                cameraRollProvider.enumerateAlbum(this.albumId, null, this.batchSize, function (items) {
                    if (_.isString(items)) {
                        items = JSON.parse(items);
                    }
                    _this.processItems(items);
                    _this.render();
                    if (!_this.items || !_this.items.models || !_this.items.models.length) {
                        _this.showEmptyContainer();
                    }
                    else {
                        // after the first fetch, update this.batchSize to 1 over
                        _this.batchSize = CameraRollScreen.rowsPerPage * CameraRollScreen.colsPerRow + 1;
                    }
                }, function (e) {
                    client.logger.warn("camera roll plugin failed with ", e);
                });
            }
            // Redraw screen
            this.render();
            this.bindOnNavigate('a.brand', function () {
                // bind events
                client.dialog.bindDialogHeader(_this.$el);
                _this.$el.on('click', '#takephoto', function (e) {
                    e.preventDefault();
                    e.stopPropagation();
                    e.stopImmediatePropagation();
                    client.casting.selectSinglePhoto(function (e, preview) {
                        if (!e && preview && preview.thumbnail) {
                            // reset all previously selected images
                            _this.$el.find('.tile>.select-image-tile').toggleClass('tile-selected', false);
                            _this.bag['selected'] = [_.extend({ isTakePhoto: true }, preview)];
                            // go directly to the next screen
                            _this.history.close(null, null);
                        }
                    });
                });
                _this.$el.on('click', '#camerarollNext', function (e) {
                    $(e.currentTarget).addClass('waiting');
                    client.logger.info('fetch more from cameraroll');
                    // fetch more
                    _.defer(function () {
                        if (_this.items && _this.items.next) {
                            _this.items.next(function () {
                            });
                        }
                    });
                });
            });
            return _super.prototype.navigate.call(this, route);
        };
        // Interface that provides camera roll 
        CameraRollScreen.prototype.cameraRollProvider = function () {
            if (window.cameraRoll) {
                return window.cameraRoll;
            }
            else {
                return new client.MockCameraRoll();
            }
        };
        CameraRollScreen.prototype.render = function () {
            // Make screen is in a valid state
            if (this.getScreenState() == 3 /* DISPOSED */) {
                client.logger.warn("Render called on DISPOSED screen ");
                return this;
            }
            // Get tiles
            this.lastX = 0;
            this.lastY = 0;
            client.logger.info('camera roll render start');
            var allTiles = {
                albums: this.getTiles(this.items)
            };
            if (!this.$el) {
                client.logger.error(consts.errorCode.ContentPicking, 'this.$el not defined for AlbusmScreen');
            }
            // Render view
            this.empty();
            this.$el.render('camera', _.extend({
                searchTerm: this.searchTerm,
                contentSource: this.contentSource,
                title: this.title,
                mode: client.StartupMode[client.startupMode],
                qrLink: 'http://stevecast.cloudapp.net'
            }, allTiles));
            _super.prototype.render.call(this);
            this.appendTiles(allTiles.albums);
            this.updateCommandBar();
            this.setDialogTitle("Select What");
            return this;
        };
        /** Map user picked content to screen queue items */
        CameraRollScreen.prototype.mapToScreenQueueItems = function (items, callback) {
            if (items.length == 1 && items[0].hasOwnProperty('isTakePhoto')) {
                var preview = (items[0]);
                client.CameraRollUploader.uploadFileUrlToBlob(preview.thumbnail.source, function (e, url) {
                    if (!e && url) {
                        callback(null, [{ type: 'photo', url: url }]);
                    }
                    else {
                        callback(e, null);
                    }
                });
            }
            else {
                // Let camera roll uploader generate screen queue items and trigger uploading
                client.CameraRollUploader.createScreenQueueItems(items, this.photoUploaded, callback);
            }
        };
        /** override to add the camera button if turned on */
        CameraRollScreen.prototype.getTiles = function (items, appendOnly) {
            var showCamera = client.user.GetSetting(15 /* SHOW_CAMERA_AS_BUTTON */);
            var tiles = _super.prototype.getTiles.call(this, items, appendOnly);
            if (!appendOnly && showCamera && tiles.length >= 1) {
                tiles.splice(1, 0, {
                    type: 'text',
                    name: 'takephoto',
                    title: resources.labels.tiles.add,
                    id: 'takephoto',
                    selected: false
                });
            }
            return tiles;
        };
        // number of rows to fetch per page
        CameraRollScreen.rowsPerPage = 10;
        CameraRollScreen.colsPerRow = 3;
        return CameraRollScreen;
    })(client.ContentPickingDialog);
    client.CameraRollScreen = CameraRollScreen;
    client.screens.addRoute('/cameraroll', CameraRollScreen);
})(client || (client = {}));
/// <reference path="../controls/CommandBar.ts"/>
/// <reference path="Screen.ts"/>
var client;
(function (client) {
    /** Token screen. */
    var TokenScreen = (function (_super) {
        __extends(TokenScreen, _super);
        function TokenScreen() {
            _super.apply(this, arguments);
        }
        TokenScreen.prototype.navigate = function (route) {
            var _this = this;
            // Update screens state
            client.logger.info('GroupScreen navigate start');
            // Redraw screen
            this.render();
            // bind event
            this.bindOnNavigate('a.brand', function () {
                var $textarea = _this.$el.find('textarea.token');
                var $tokenSelect = _this.$el.find('#tokenSelect');
                $tokenSelect.change(function (e) {
                    var token = client.user.accountTokens[$(e.currentTarget).val()];
                    if (token) {
                        $textarea.val(token.refreshToken || token.token);
                    }
                    else {
                        $textarea.val('');
                    }
                });
                _this.$el.on('click', '.actions button', function (e) {
                    var token = $textarea.val();
                    token = token.replace(/\s/g, '');
                    var $button = $(e.currentTarget);
                    if ($button.hasClass('set')) {
                        var tokens = client.user.accountTokens;
                        var service = $tokenSelect.val();
                        var newToken = (service == 'fb' || service == 'dropbox' || service == 'instagram') ? { token: token } : {
                            refreshToken: token,
                            token: token,
                            expires: Math.round((new Date()).getTime() / 1000)
                        };
                        tokens[service] = _.extend(tokens[service] || {}, newToken);
                        localStorage.setItem('client.user.accountTokens', JSON.stringify(tokens));
                    }
                    if (_this.history) {
                        _this.history.close();
                    }
                });
            });
            return _super.prototype.navigate.call(this, route);
        };
        TokenScreen.prototype.render = function () {
            // Make screen is in a valid state
            if (this.getScreenState() == 3 /* DISPOSED */) {
                client.logger.warn("Render called on DISPOSED screen ");
                return this;
            }
            // Render view
            this.empty();
            var token = null;
            if (client.user.accountTokens && client.user.accountTokens.fb) {
                token = client.user.accountTokens.fb.token;
            }
            client.logger.info('refresh token', token);
            this.$el.render('tokenScreen', { token: token });
            this.syncNoPan();
            return this;
        };
        return TokenScreen;
    })(client.Screen);
    client.TokenScreen = TokenScreen;
    client.screens.addRoute('/token', TokenScreen);
})(client || (client = {}));
/// <reference path="./AppMasterScreen.ts"/>
/// <reference path="../user.ts"/>
var client;
(function (client) {
    /** Log viewer screen. */
    var LogScreen = (function (_super) {
        __extends(LogScreen, _super);
        function LogScreen() {
            _super.apply(this, arguments);
        }
        LogScreen.prototype.navigate = function (route) {
            var _this = this;
            // Update screens state
            client.logger.info('LogScreen navigate start');
            // Setup command bar
            if (this.commandBar) {
                this.commandBar.setCommands([
                    consts.commands.refresh,
                    consts.commands.clear
                ], function (id) {
                    switch (id) {
                        case consts.commands.refresh.id:
                            _this.render();
                            break;
                        case consts.commands.clear.id:
                            client.logger.log = [];
                            _this.render();
                            break;
                    }
                });
            }
            // Redraw screen
            this.render();
            return _super.prototype.navigate.call(this, route);
        };
        LogScreen.prototype.render = function () {
            // Make screen is in a valid state
            if (this.getScreenState() == 3 /* DISPOSED */) {
                client.logger.warn("Render called on DISPOSED screen ");
                return this;
            }
            // Render view
            this.empty();
            this.$el.render('logScreen', { resources: resources, title: "logs" });
            // Logger level 
            var $loggerLevel = this.$el.find('#loggerLevel');
            $loggerLevel.val(client.logger.level).change(function () {
                client.logger.level = $loggerLevel.val();
                localStorage.setItem('logger-level', client.logger.Levels[client.logger.level]);
            });
            // Populate log
            var $log = this.$el.find('.section.log');
            _.each(client.logger.log, function (entry) {
                var html = '<div class="' + entry.type + ' entry">';
                html += client.Util.cleanString(entry.type + ': ' + entry.msg);
                _.each(entry.args || [], function (v) {
                    html += ', <span class="arg">';
                    switch (typeof v) {
                        case 'object':
                        case 'array':
                            html += client.Util.cleanString(JSON.stringify(v));
                            break;
                        default:
                            html += (v === undefined ? 'undefined' : client.Util.cleanString(v.toString()));
                            break;
                    }
                    html += '</span>';
                });
                html += '</div>';
                $log.prepend(html);
            });
            this.syncNoPan();
            return this;
        };
        return LogScreen;
    })(client.DetailScreen);
    client.LogScreen = LogScreen;
    client.screens.addRoute('/log', LogScreen);
    // Listen for commands
    client.app.addEventListner('command', function (command) {
        switch (command.command) {
            case 'showLogs':
                client.screens.showModal(LogScreen);
                break;
            case 'clearLogs':
                client.logger.log = [];
                client.screens.showModal(LogScreen);
                break;
        }
    });
})(client || (client = {}));
/// <reference path="./AppMasterScreen.ts"/>
/// <reference path="./PickerScreen.ts"/>
/// <reference path="./SkydriveScreen.ts"/>
/// <reference path="./AlbumsScreen.ts"/>
/// <reference path="./TokenScreen.ts"/>
/// <reference path="./LogScreen.ts"/>
/// <reference path="../user.ts"/>
/// <reference path="../controls/ParticleEffect.ts"/>
/// <reference path="./casting.ts"/>
var client;
(function (client) {
    /** Debug screen. */
    var DebugScreen = (function (_super) {
        __extends(DebugScreen, _super);
        function DebugScreen(selector) {
            _super.call(this, selector);
            this.selector = selector;
            this.altHeader = "Debug Settings";
        }
        DebugScreen.prototype.navigate = function (route) {
            var _this = this;
            // Update screens state
            client.logger.info('SettiDebugScreenngsScreen navigate start');
            // Setup command bar
            this.commandBar.commandBarStyle['textSize'] = 14;
            this.commandBar.commandBarStyle['glyphSize'] = 24;
            this.commandBar.setCommands([
                consts.commands.picker,
                consts.commands.token,
                consts.commands.testPush,
                consts.commands.logViewer,
                consts.commands.clearcache
            ], function (id) {
                switch (id) {
                    case consts.commands.picker.id:
                        //client.casting.castContent(new MockGroup(new nearcast.MockSession()));
                        client.casting.addMoreContent(function (e, spec) {
                            if (e) {
                                client.logger.info('add media error', e.toString());
                            }
                            else {
                                var items = spec.items;
                                client.logger.info('added media count is', items.length);
                                _.each(items, function (item) {
                                    client.logger.info('adding item', item);
                                });
                            }
                        });
                        break;
                    case consts.commands.token.id:
                        client.screens.showModal(client.TokenScreen);
                        break;
                    case consts.commands.testPush.id:
                        client.user.registerForPush(function (e, success) {
                            client.logger.debug('registerPush result = ', success, 'with error=', e);
                        });
                        break;
                    case consts.commands.logViewer.id:
                        client.app.navigate('/log');
                        break;
                    case consts.commands.clearcache.id:
                        var $particleEffect = _this.$el.find('.debugEffect');
                        client.ParticleMaster.show($particleEffect, 3);
                        client.user.session.clearCachedGroups(client.user.profile.userId);
                        break;
                }
            });
            // Redraw screen
            this.render();
            return _super.prototype.navigate.call(this, route);
        };
        DebugScreen.prototype.render = function () {
            // Make screen is in a valid state
            if (this.getScreenState() == 3 /* DISPOSED */) {
                client.logger.warn("Render called on DISPOSED screen ");
                return this;
            }
            // Render view
            this.empty();
            var localSettings = client.user.GetSettings();
            this.$el.render('debugScreen', { settings: localSettings, resources: resources, title: "DEBUG" });
            // Login override
            var $loginOverride = this.$el.find('#loginOverride');
            $loginOverride.val(client.user.loginOverride).change(function () {
                client.user.loginOverride = $loginOverride.val();
                localStorage.setItem('login-override', client.user.loginOverride);
            });
            // Location override
            var $locationOverride = this.$el.find('#locationOverride');
            $locationOverride.val(client.scanner.locationOverride).change(function () {
                client.scanner.locationOverride = $locationOverride.val();
                localStorage.setItem('location-override', client.scanner.locationOverride);
                client.scanner.lastPosition = null;
                client.scanner.update();
            });
            // Logger level 
            var $loggerLevel = this.$el.find('#loggerLevel');
            $loggerLevel.val(client.logger.level).change(function () {
                client.logger.level = $loggerLevel.val();
                localStorage.setItem('logger-level', client.logger.Levels[client.logger.level]);
            });
            // discovery settings
            this.$el.find('#discoverySetting').val(client.user.settings.discovery.setting).change(function (e) {
                var discoverySetting = $(e.currentTarget).val();
                client.user.setDiscoverySetting(discoverySetting);
                localStorage.setItem('discoverySetting', discoverySetting);
            });
            for (var settingName in localSettings) {
                if (localSettings[settingName].type == 0 /* BOOLEAN */) {
                    var name = localSettings[settingName].name;
                    this.$el.find('.' + name).change(function (e) {
                        client.user.SetSetting(e.currentTarget.id, $(e.currentTarget).prop('checked'));
                    });
                }
            }
            this.syncNoPan();
            return this;
        };
        return DebugScreen;
    })(client.DetailScreen);
    client.DebugScreen = DebugScreen;
})(client || (client = {}));
/// <reference path='../types.ts'/>
/// <reference path='../colorCycle.ts'/>
/// <reference path='../util.ts'/>
/// <reference path="../deviceContacts.ts"/>
/// <reference path="../screens/DebugScreen.ts"/>
var client;
(function (client) {
    /** Creates a DOM object containing search results */
    var SearchResults = (function () {
        function SearchResults() {
        }
        SearchResults.render = function (
            // Search term
            searchTerm, 
            // Where to render the profiles
            $selector, 
            // Set of profiles to render
            profiles, 
            // Callback when clicked
            clickCallback) {
            SearchResults.$selector = $selector;
            SearchResults.clickCallback = clickCallback;
            if (profiles.length == 0)
                return;
            for (var i = 0; i < profiles.length; i++) {
                var profile = profiles[i];
                // Use symbols if profile doesn't have a thumbnail
                var initials = client.DeviceContacts.initials(profile);
                var displayName = profile.displayName;
                var showChoice = true;
                // Default to contacting via phone if not a Xim user already
                var contact;
                if (profile.phone) {
                    profile.inviteType = consts.InviteType.sms;
                    contact = profile.phone;
                    if (!displayName) {
                        displayName = profile.phone;
                        showChoice = false;
                    }
                }
                else if (profile.email) {
                    profile.inviteType = consts.InviteType.email;
                    contact = profile.email;
                    if (!displayName) {
                        displayName = profile.email;
                        showChoice = false;
                    }
                }
                else if (profile.code) {
                    profile.inviteType = consts.InviteType.code;
                    contact = profile.code;
                    if (!displayName) {
                        displayName = profile.code;
                        showChoice = false;
                    }
                }
                else {
                    client.logger.warn("Contact has no valid method of invite");
                    continue;
                }
                var $content = $("<div>").render('searchResult', {
                    image: profile.thumbnailUrl ? profile.thumbnailUrl : '',
                    initials: initials,
                    name: displayName,
                    phone: profile.phone,
                    email: profile.email,
                    contact: contact,
                    id: client.Util.toAlphaNumeric(profile.deviceId),
                    backgroundColor: profile.color,
                    showChoice: showChoice
                });
                if (initials) {
                    $content.find('.searchImage').hide();
                }
                else {
                    $content.find('.searchInitials').hide();
                }
                $content.on('click', { profile: profile }, SearchResults.onProfileClick);
                $content.on('click', '.searchResultGlyph', { profile: profile }, SearchResults.showContactMenu);
                $content.on('click', '.searchResultChoice', { profile: profile }, SearchResults.onContactChoice);
                $content.on('mousedown touchstart', '.searchResultChoice', function (ev) {
                    var $cmd = $(ev.currentTarget);
                    $cmd.addClass('searchResultChoicePressed');
                    // Remove pressed style after a pause in case user moved mouse off target (which is not a click)
                    setTimeout(function () {
                        $cmd.removeClass('searchResultChoicePressed');
                    }, 500);
                });
                $selector.append($content);
            }
            // Add space at end for scrolling past keyboard
            $selector.append('<div class="keyboardpadding"></div>');
        };
        // User has clicked on a profile
        SearchResults.onProfileClick = function (event) {
            // If contact type picker is open close it
            if (SearchResults.activeProfile) {
                SearchResults.hideContactMenus();
            }
            else {
                // Otherwise return the chosen profile
                var profile = event.data.profile;
                if (profile) {
                    SearchResults.clickCallback(profile);
                }
            }
        };
        // Profile photo was loaded
        SearchResults.onThumbnailLoaded = function (profile) {
            if (profile) {
                var $profile = SearchResults.$selector.find('#' + profile.deviceId);
                $profile.find('.searchResultImage').attr('src', profile.thumbnailUrl);
                $profile.find('.searchInitials').hide();
                $profile.find('.searchImage').show();
            }
        };
        // User has chosen a contact method
        SearchResults.onContactChoice = function (event) {
            var profile = event.data.profile;
            if (!profile) {
                client.logger.error(consts.errorCode.PeoplePicking, 'missing profile');
                return;
            }
            if (event.target.id == "email") {
                profile.inviteType = consts.InviteType.email;
            }
            else {
                profile.inviteType = consts.InviteType.sms;
            }
            client.logger.collectNamedEvent(consts.flurryEvent.pickpeopleTapPerson, { 'inviteType': profile.inviteType });
            SearchResults.clickCallback(profile);
        };
        SearchResults.hideContactMenus = function () {
            // Clear out any open contact selection
            var $selector = $(".searchResultGlyph");
            $selector.removeClass('searchResultGlyphSelected');
            $selector.addClass('glyphiconXIM-unselectedOption');
            $selector.removeClass('glyphiconXIM-selectionOption');
            SearchResults.$selector.find(".searchResultPicker").hide();
            SearchResults.activeProfile = null;
        };
        SearchResults.showContactMenu = function (event) {
            client.logger.collectNamedEvent(consts.flurryEvent.pickpeopleChooseInviteMethod);
            event.preventDefault();
            event.stopPropagation();
            // Get the profile from event data
            var profile = event.data.profile;
            if (!profile) {
                client.logger.error(consts.errorCode.PeoplePicking, 'missing profile');
                return;
            }
            // If user isn't clicking on already open glyph, open it
            if (!SearchResults.activeProfile || SearchResults.activeProfile.deviceId != profile.deviceId) {
                SearchResults.hideContactMenus();
                SearchResults.activeProfile = profile;
                var $selector = $('#' + profile.deviceId);
                var $glyph = $selector.find(".searchResultGlyph");
                $glyph.addClass('searchResultGlyphSelected');
                $glyph.removeClass('glyphiconXIM-unselectedOption');
                $glyph.addClass('glyphiconXIM-selectionOption');
                $selector.find(".searchResultPicker").show();
            }
            else {
                SearchResults.hideContactMenus();
            }
        };
        return SearchResults;
    })();
    client.SearchResults = SearchResults;
})(client || (client = {}));
/// <reference path='../types.ts'/>
/// <reference path='../DeviceContacts.ts'/>
/// <reference path='CommandBar.ts'/>
/// <reference path='../ProfileGrid.ts'/>
/// <reference path='../colorCycle.ts'/>
/// <reference path='../util.ts'/>
/// <reference path='../controls/SearchResults.ts'/>
var client;
(function (client) {
    /** People Picker */
    var PeoplePicker = (function () {
        function PeoplePicker(selector, commandBar) {
            this.commandBar = commandBar;
            /** Callback for selection change */
            // REVIEW: this probably should have been an event instead
            this.selectionChangedCallback = null;
            /** Models being rendered to screen. */
            this.nearbyModels = []; // Detected nearby
            this.recentModels = [];
            this.searchModels = []; // Search results
            this.selectedModels = []; // Selected search resutls
            /** Models to exclude from the picker */
            this.excludedModels = [];
            this.selectedProfiles = [];
            this.allSelected = false;
            this.searchVisible = false;
            /** Search */
            this.searchTimeout = null;
            this.lastSearchTerm = null;
            this.$el = $(selector);
            this.$el.empty();
            // Listen for events
            if (client.user.GetSetting(3 /* ENABLE_NEARBY_MODE */)) {
                this._onNearbyUsersAdded = _.bind(this.onNearbyUsersAdded, this);
                client.user.onNearbyUsersAdded.add(this._onNearbyUsersAdded);
                this._onNearbyUsersRemoved = _.bind(this.onNearbyUsersRemoved, this);
                client.user.onNearbyUsersRemoved.add(this._onNearbyUsersRemoved);
            }
            this._onRecentUsersChanged = _.bind(this.onRecentUsersChanged, this);
            client.RecentPeople.onRecentUsersChanged.add(this._onRecentUsersChanged);
        }
        PeoplePicker.prototype.onDispose = function () {
            // Free memory for contacts
            client.DeviceContacts.free();
            // Clear callback
            this.selectionChangedCallback = null;
            // Disconnect from events and stop stuff
            client.user.onNearbyUsersAdded.remove(this._onNearbyUsersAdded);
            client.user.onNearbyUsersRemoved.remove(this._onNearbyUsersRemoved);
            client.RecentPeople.onRecentUsersChanged.remove(this._onRecentUsersChanged);
        };
        PeoplePicker.prototype.render = function () {
            var _this = this;
            // Render core screen
            this.$el.empty();
            this.$el.render('peoplePicker', { messages: resources.messages, isWp: client.startupMode == 3 /* wp */ });
            // Add castViewer Icon
            var iconViewer = new client.CastViewerIcon(this.$el, 'peoplepicker');
            // Capture key press events in search box
            this.$el.on('keyup input', '#searchTerm', _.bind(this.onSearchChanged, this));
            this.$el.find("#searchTerm").focus(function () {
                _.delay(function () {
                    if (client.startupMode == 4 /* ios */) {
                        var sTop = $(window).scrollTop();
                        if (sTop > 0) {
                            // because this the physical height of the dialogTitleBox (height + top + margin-bottom);
                            var $title = $('.dialogTitleBox');
                            var heights = $title.css(['height', 'top', 'margin-bottom']);
                            var offset = _.reduce(heights, function (sum, v) {
                                return sum + parseInt(v);
                            }, 0);
                            client.logger.info('snap window to top', sTop, 'offset', offset);
                            window.scroll(0, sTop + offset);
                        }
                    }
                }, 20);
            });
            // De-focus search bar when user scrolls to hide keyboard
            if (client.startupMode != 3 /* wp */) {
                this.$el.find('.searchresultslist').on('touchmove', function () {
                    _this.searchBlur();
                });
            }
            else {
                // windows phone doesn't fire touchmove event
                this.$el.find('.searchresultslist').on('mousemove', function () {
                    _this.searchBlur();
                });
            }
            this.showSearchView(false);
            this.updateAndRenderModels();
        };
        // Reset previous selections
        PeoplePicker.prototype.reset = function () {
            this.searchModels = [];
            this.selectedModels = [];
            this.selectedProfiles = [];
        };
        PeoplePicker.prototype.inSearchVisible = function () {
            return this.searchVisible;
        };
        // Clear selected profiles
        PeoplePicker.prototype.clearSelection = function () {
            // Itterate through copy of array so can change while looping
            var selectedCopy = this.selectedProfiles.slice(0);
            for (var i = 0; i < selectedCopy.length; i++) {
                this.selectProfile(selectedCopy[i], false);
            }
        };
        PeoplePicker.prototype.profileCount = function () {
            return this.selectedModels.length + this.recentModels.length + this.nearbyModels.length;
        };
        PeoplePicker.prototype.renderModels = function (callback) {
            if (callback === void 0) { callback = null; }
            var inCast = (this.excludedModels && this.excludedModels.length > 0);
            if (this.profileCount() == 0 && !inCast) {
                this.$el.find('.peoplepickerNoContacts').show();
                this.$el.find('.peoplelayout').hide();
                this.commandBar.show(false);
            }
            else {
                this.$el.find('.peoplepickerNoContacts').hide();
                this.$el.find('.peoplelayout').show();
                // Define layout
                var layout = new client.FigureLayout();
                // Always render in portrait, but screen may be in middle of rotation so take smallest axis
                var width = Math.min(client.webview.currentViewport().width, client.webview.currentViewport().height);
                layout.setWidth(width);
                // Find place to render
                var $nearby = this.$el.find('.peoplelayout');
                // Clear previous content
                $nearby.empty();
                // Render profiles
                client.ProfileGrid.render($nearby, this.selectedModels, layout, _.bind(this.onNearbyClick, this), _.bind(this.isSelected, this), 0);
                client.ProfileGrid.render($nearby, this.nearbyModels, layout, _.bind(this.onNearbyClick, this), _.bind(this.isSelected, this), this.selectedModels.length);
                client.ProfileGrid.render($nearby, this.recentModels, layout, _.bind(this.onNearbyClick, this), _.bind(this.isSelected, this), this.selectedModels.length + this.nearbyModels.length);
                // Add padding for command bar
                var $padding = this.$el.find('.commandBarPadding');
                var modelCount = this.selectedModels.length + this.nearbyModels.length + this.recentModels.length;
                var bottom = layout.getBottomPosition(modelCount) + 75; // 75 = Command Bar height
                $padding.css({ position: 'absolute', top: bottom });
            }
            if (callback) {
                callback(null);
            }
        };
        PeoplePicker.prototype.renderSearchModels = function () {
            // Find place to render
            var $search = this.$el.find('.searchresultslist');
            // Clear previous content
            $search.empty();
            client.SearchResults.render(this.lastSearchTerm, $search, this.searchModels, _.bind(this.onSearchClick, this));
        };
        PeoplePicker.prototype.selectProfile = function (profile, select) {
            // Find profile selector (could be by userId or deviceID depending on whether is registered user)
            var $nearby = this.$el.find('.peoplelayout');
            var $profile = $nearby.find("#" + client.Util.toAlphaNumeric(profile.deviceId));
            if ($profile.length == 0) {
                $profile = $nearby.find("#" + client.Util.toAlphaNumeric(profile.userId));
            }
            if ($profile.length == 0) {
                client.logger.error(consts.errorCode.PeoplePicking, "can't find profile");
                return;
            }
            // Select or deselect it
            var index = this.selectedProfiles.indexOf(profile);
            if (select) {
                if (index < 0) {
                    this.selectedProfiles.push(profile);
                }
                $profile.find(".personImage").addClass("personImageSelected");
                $profile.find('.figuremark').addClass('checkmark');
            }
            else {
                if (index >= 0) {
                    this.selectedProfiles.splice(index, 1);
                }
                $profile.find(".personImage").removeClass("personImageSelected");
                $profile.find('.figuremark').removeClass('checkmark');
            }
        };
        /** Returns true if profile has been selected */
        PeoplePicker.prototype.isSelected = function (profile) {
            return (this.selectedProfiles.indexOf(profile) >= 0);
        };
        PeoplePicker.prototype.onSearchClick = function (profile) {
            // Clear search term and callbacks from previous search
            this.lastSearchTerm = null;
            var $searchTerm = this.$el.find('#searchTerm');
            $searchTerm.val("");
            // If user picked a profile that has already been selected,
            // Update selected profile's contact method, but selection stays the same
            var selectedModel = PeoplePicker.inList(profile, this.selectedModels);
            if (selectedModel) {
                selectedModel.inviteType = profile.inviteType;
                selectedModel.phone = (profile.phone ? profile.phone : selectedModel.phone);
                selectedModel.email = (profile.email ? profile.email : selectedModel.email);
                this.completeSearch(null);
                return;
            }
            // If user picked a profile that matches one on recent profile list,
            // Use recent profile, as it may have extra data (i.e. Name, profile photo, etc)
            var recentModel = PeoplePicker.inList(profile, this.recentModels);
            if (recentModel) {
                recentModel.inviteType = profile.inviteType;
                recentModel.phone = (profile.phone ? profile.phone : recentModel.phone);
                recentModel.email = (profile.email ? profile.email : recentModel.email);
                this.completeSearch(recentModel);
                return;
            }
            // It's a brand new profile.  Set userId and add to selected models 
            profile.userId = profile.deviceId;
            this.selectedModels.push(profile);
            this.completeSearch(profile);
        };
        /** Complete search action.  If new selection was made, pass it in */
        PeoplePicker.prototype.completeSearch = function (newProfile) {
            // If user selected a new profile
            if (newProfile) {
                // Update display
                this.renderModels();
                // Update UI to show selection
                this.selectProfile(newProfile, true);
                // Note that selection has changed
                this.selectionChanged();
            }
            // Return to non-search view  
            this.showSearchView(false);
            if (this.searchVisible) {
                // hide the keyboard
                this.$el.find('.nearby').focus();
            }
        };
        /** Called when a nearby tile is clicked */
        PeoplePicker.prototype.onNearbyClick = function (event) {
            // Get the profile from event data
            var profile = event.data.profile;
            if (!profile) {
                client.logger.error(consts.errorCode.PeoplePicking, "missing profile");
                return;
            }
            client.logger.collectNamedEvent(consts.flurryEvent.pickpeoplePreviousPeople, { 'inviteType': profile.inviteType });
            // Toggle selection
            this.selectProfile(profile, !this.isSelected(profile));
            // Note that selection has changed
            this.selectionChanged();
        };
        /** Callback on selection change */
        PeoplePicker.prototype.selectionChanged = function () {
            if (this.selectionChangedCallback) {
                this.selectionChangedCallback(this.selectedProfiles);
            }
        };
        // Hide / show search view
        PeoplePicker.prototype.showSearchView = function (show) {
            // Reset scroll position top
            $(window).scrollTop(0);
            if (show) {
                this.$el.find('.nearby').hide();
                this.$el.find('.searchresults').show();
                if (this.commandBar)
                    this.commandBar.show(false);
            }
            else {
                // Cancel any outstanding search callbacks
                client.DeviceContacts.cancelCallbacks();
                this.$el.find('.nearby').show();
                this.$el.find('.searchresults').hide();
                if (this.commandBar && this.profileCount() > 0) {
                    this.commandBar.show(true);
                }
            }
            this.searchVisible = show;
        };
        // Returns true if search string has suffient data to begin a search
        PeoplePicker.prototype.isSearchStringSufficient = function (text) {
            // Remove spaces as (first{space}last) is valid
            text = text.replace(/\s+/g, '');
            if (client.Util.isValidName(text)) {
                // If starts with Chinese, Korean or Japanese chars one char is sufficient for a name
                if (text.length > 0 && client.Util.isCJK(text[0])) {
                    return true;
                }
                // Otherwise require 2 chars
                return (text.length > 1);
            }
            return false;
        };
        // Returns invite type if search string has suffient data for a phone number or email address
        PeoplePicker.prototype.getInviteType = function (text) {
            if (client.Util.validEmail(text))
                return consts.InviteType.email;
            var parsedPhone = consts.parsePhoneNumberForCountry(text, client.user.country);
            if (parsedPhone.isValid)
                return consts.InviteType.sms;
            if (client.Util.validCode(text))
                return consts.InviteType.code;
            return consts.InviteType.none;
        };
        PeoplePicker.prototype.onSearchChanged = function (event) {
            var _this = this;
            if (!event.target) {
                return;
            }
            // Enter event acts as select if only a single search match
            if (event.keyCode == 13 && this.searchModels.length == 1) {
                this.$el.find('#searchTerm').blur();
                this.onSearchClick(this.searchModels[0]);
                return;
            }
            var text = client.Util.cleanString(event.target.value);
            // If no longer searching
            if (text.length == 0) {
                this.showSearchView(false);
                return;
            }
            // If serach term hasn't changed do nothing
            if (text == this.lastSearchTerm) {
                return;
            }
            this.lastSearchTerm = text;
            // Clear search models
            this.searchModels = [];
            // Hide nearby and show search results
            this.showSearchView(true);
            // Terminate any existing search delay
            if (this.searchTimeout) {
                window.clearTimeout(this.searchTimeout);
            }
            // If search string is complete email or phone, display card for this item
            var inviteType = this.getInviteType(text);
            if (inviteType != consts.InviteType.none) {
                this.searchModels = [
                    {
                        deviceId: text,
                        displayName: null,
                        color: client.ColorCycle.GetNext(),
                        phone: (inviteType == consts.InviteType.sms) ? text : null,
                        email: (inviteType == consts.InviteType.email) ? text : null,
                        code: (inviteType == consts.InviteType.code) ? text : null,
                        inviteType: inviteType
                    }
                ];
                // Clear search message
                this.setSearchMessage(null);
            }
            else if (this.isSearchStringSufficient(text)) {
                // Delay search slightly in case user is still typing
                this.searchTimeout = window.setTimeout(function () {
                    _this.setSearchMessage("Searching contacts...");
                    client.DeviceContacts.matchCount(text, _.bind(_this.searchMatchComplete, _this));
                }, 300);
            }
            else if (text.length >= 2) {
                this.setSearchMessage("No matching contacts found.");
            }
            this.renderSearchModels();
        };
        PeoplePicker.prototype.setSearchMessage = function (message) {
            var $selector = this.$el.find('.searchmessage');
            if (message == null) {
                $selector.hide();
            }
            else {
                $selector.show();
                $selector.text(message);
            }
        };
        // Contact search match has completed and returned number of positive matches
        PeoplePicker.prototype.searchMatchComplete = function (matchCount) {
            if (matchCount == 0) {
                this.setSearchMessage("No matching contacts found.");
            }
            else {
                this.setSearchMessage("Loading " + matchCount + " matching contacts...");
                client.DeviceContacts.matchDetails(_.bind(this.searchDetailsComplete, this), _.bind(client.SearchResults.onThumbnailLoaded, this));
            }
        };
        // Contact details have been loaded
        PeoplePicker.prototype.searchDetailsComplete = function (contacts, remaining) {
            if (remaining == 0) {
                this.setSearchMessage(null);
            }
            else {
                this.setSearchMessage("Loading " + remaining + " more contacts...");
            }
            // Render the returned results
            this.searchModels = this.searchModels.concat(contacts);
            this.renderSearchModels();
        };
        PeoplePicker.prototype.FailedContacts = function (error) {
            client.logger.error(consts.errorCode.Contacts, 'Failed to load contacts.');
            this.renderSearchModels();
        };
        // Scroll events should take focus away from inputbox
        PeoplePicker.prototype.searchBlur = function () {
            this.$el.find('#searchTerm').blur();
        };
        PeoplePicker.prototype.onNearbyUsersAdded = function (profiles) {
            this.updateAndRenderModels();
        };
        PeoplePicker.prototype.onNearbyUsersRemoved = function (profiles) {
            this.updateAndRenderModels();
        };
        PeoplePicker.prototype.onRecentUsersChanged = function () {
            this.updateAndRenderModels();
        };
        /** Updates the models and render */
        PeoplePicker.prototype.updateAndRenderModels = function () {
            async.series([
                _.bind(this.updateModels, this),
                _.bind(this.renderModels, this)
            ], null);
        };
        /** Returns matching model if exists in the given list */
        PeoplePicker.inList = function (model, list) {
            if (!list)
                return false;
            var phone = client.Util.stripPhone(model.phone);
            var email = (model.email ? model.email.toLowerCase() : "");
            var code = (model.code ? model.code : "");
            var duplicate = _.filter(list, function (t) {
                // If deviceID, userID or code match it is the same profile
                if ((t.deviceId && (t.deviceId == model.deviceId)) || (t.userId && (t.userId == model.userId)) || (t.code && (t.code == code))) {
                    return true;
                }
                // If phones match
                if (t.phone && (client.Util.stripPhone(t.phone) == phone)) {
                    // Check that emails aren't unmatched as some people share a phone number
                    if (!t.email || !email || (t.email.toLowerCase() == email)) {
                        return true;
                    }
                }
                // If emails match
                if (t.email && (t.email.toLowerCase() == email)) {
                    // Check that phones aren't unmatched, as some people share an email
                    if (!t.phone || !phone || (client.Util.stripPhone(t.phone) == phone)) {
                        return true;
                    }
                }
                return false;
            });
            if (duplicate.length > 0) {
                return duplicate[0];
            }
            return null;
        };
        PeoplePicker.prototype.updateModels = function (callback) {
            var models = [];
            if (client.user.GetSetting(3 /* ENABLE_NEARBY_MODE */)) {
                models = client.user.nearby().models;
            }
            // If there are any models to exclude, do so
            if (this.excludedModels) {
                this.nearbyModels = [];
                for (var i = 0; i < models.length; i++) {
                    if (!PeoplePicker.inList(models[i], this.excludedModels)) {
                        var profile = models[i];
                        profile.isNearby = true;
                        this.nearbyModels.push(profile);
                    }
                }
            }
            else {
                this.nearbyModels = models;
            }
            // Add recent models that aren't duplicates or excluded
            var recent = client.RecentPeople.Get();
            this.recentModels = [];
            for (i = 0; i < recent.length; i++) {
                if (!PeoplePicker.inList(recent[i], this.excludedModels) && !PeoplePicker.inList(recent[i], this.selectedModels) && !PeoplePicker.inList(recent[i], this.nearbyModels)) {
                    profile = recent[i];
                    profile.isNearby = false;
                    this.recentModels.push(profile);
                }
            }
            callback(null);
        };
        return PeoplePicker;
    })();
    client.PeoplePicker = PeoplePicker;
})(client || (client = {}));
/// <reference path="./AppMasterScreen.ts"/>
/// <reference path='./DialogScreen.ts'/>
/// <reference path='./ScreenWizard.ts'/>
/// <reference path="../controls/PeoplePicker.ts"/>
/// <reference path="../user.ts"/>
var client;
(function (client) {
    /** Media Source Picking screen */
    var PeopleScreen = (function (_super) {
        __extends(PeopleScreen, _super);
        function PeopleScreen(sel, history, commands, commandCallback) {
            _super.call(this, sel, history, commands, commandCallback);
            this.history = history;
            this.commands = commands;
            this.commandCallback = commandCallback;
            /** People Picker */
            this.peoplePicker = null;
            this.excludedModels = [];
            this.CommandBarStyle.bar = 'commandBarOpen';
        }
        PeopleScreen.prototype.navigate = function (route) {
            // Update screens state
            client.logger.info('PeopleScreen navigate start');
            this.bag = this.bag || route.bag;
            if (this.bag && this.bag['exclude']) {
                this.excludedModels = this.bag['exclude'];
            }
            // Redraw screen
            this.render();
            // bind events
            this.bindOnNavigate('a.brand', function () {
            });
            return _super.prototype.navigate.call(this, route);
        };
        PeopleScreen.prototype.onDispose = function () {
            // Clear people picker if it exists
            if (this.peoplePicker) {
                this.peoplePicker.onDispose();
            }
            _super.prototype.onDispose.call(this);
        };
        PeopleScreen.prototype.render = function () {
            // Make screen is in a valid state
            if (this.getScreenState() == 3 /* DISPOSED */) {
                client.logger.warn("Render called on DISPOSED screen ");
                return this;
            }
            // Render view
            this.empty();
            if (this.peoplePicker) {
                this.peoplePicker.onDispose();
            }
            this.peoplePicker = null;
            this.$el.render('people', { title: resources.labels.peopleTitle });
            _super.prototype.render.call(this);
            this.showPeoplePicker(true);
            this.setDialogTitle(resources.labels.peopleTitle);
            this.syncNoPan();
            return this;
        };
        PeopleScreen.prototype.showPeoplePicker = function (show) {
            if (show) {
                if (!this.peoplePicker) {
                    this.peoplePicker = new client.PeoplePicker(this.$el.find('#peoplePicker'), this.commandBar);
                    this.peoplePicker.excludedModels = this.excludedModels;
                    this.peoplePicker.selectionChangedCallback = _.bind(this.onPeoplePickerSelectionChanged, this);
                    this.peoplePicker.render();
                    // Register for castViewer changes to update command bar appropriately
                    client.CastViewManager.onCastViewerChanged.add(_.bind(this.onCastViewerChanged, this));
                }
                this.$el.find('.timelinePeoplePicker').show();
                this.$el.find('.timelineGroups').hide();
                this.commandBar.disable(false);
                this.updateCommandBar();
                // Show command bar only if not currently searching
                if (!this.peoplePicker.inSearchVisible() && this.peoplePicker.profileCount() > 0) {
                    this.commandBar.show(true);
                }
            }
            else {
                // Update groups
                this.commandBar.disable(true);
            }
        };
        /** start the cast if there is a viewer connected already*/
        PeopleScreen.prototype.onCastViewerChanged = function () {
            if (client.CastViewManager.activeViewer) {
                // trigger the start cast command right away
                if (client.wizard.currentWizard && client.wizard.currentWizard.commandCallback) {
                    client.wizard.currentWizard.commandCallback(consts.commands.start.id, consts.commands.start);
                }
            }
            else {
                // no active cast viwer, just update the command bar status
                this.updateCommandBar();
            }
        };
        /** Update enable state of command bar based on user selection */
        PeopleScreen.prototype.updateCommandBar = function () {
            // Can only start a xim if at least one person is selected or a viewManager is active
            var cantStart = (this.peoplePicker.selectedProfiles.length <= 0) && (client.CastViewManager.activeViewer == null);
            this.commandBar.disableCommand(consts.commands.start, cantStart);
            this.commandBar.disableCommand(consts.commands.addPeople, cantStart);
        };
        PeopleScreen.prototype.onPeoplePickerSelectionChanged = function (selection) {
            this.bag['invites'] = selection;
            this.updateCommandBar();
            this.syncNoPan();
        };
        return PeopleScreen;
    })(client.DialogScreen);
    client.PeopleScreen = PeopleScreen;
    client.screens.addRoute('/people', PeopleScreen);
})(client || (client = {}));
/// <reference path="../screens/AppMasterScreen.ts"/>
/// <reference path='../screens/ContentPickingDialog.ts'/>
var client;
(function (client) {
    /** Settings screen. */
    var MockSourceScreen = (function (_super) {
        __extends(MockSourceScreen, _super);
        function MockSourceScreen(sel, history, commands, commandCallback) {
            _super.call(this, sel, history, commands, commandCallback);
            this.history = history;
            this.commands = commands;
            this.commandCallback = commandCallback;
            this.contentSource = 'mocksource';
            this.defaultTitle = resources.labels.pickerSource.fb;
        }
        MockSourceScreen.prototype.getItems = function () {
            var categories = ["abstract", "animals", "business", "cats", "city", "food", "nightlife", "fashion", "people", "nature", "sports", "technics", "transport"];
            var category = categories[Math.floor(Math.random() * categories.length)];
            var models = [];
            for (var i = 1; i < 11; i++) {
                var width = (200 + client.Util.getRandomInt(400));
                var height = (200 + client.Util.getRandomInt(400));
                var url = "http://lorempixel.com/" + width + "/" + height + "/" + category + "/" + i;
                var newPhoto = {
                    type: 'photo',
                    id: i,
                    name: i,
                    thumbnail: {
                        source: url,
                        width: width,
                        height: height
                    },
                    url: url
                };
                models.push(newPhoto);
            }
            return { models: models };
        };
        MockSourceScreen.prototype.navigate = function (route) {
            var _this = this;
            // Update screens state
            client.logger.info('MockSource navigate start');
            this.lastX = 0;
            this.lastY = 0;
            this.bag = this.bag || route.bag;
            this.bag['selected'] = [];
            this.items = this.getItems();
            this.title = "Mock Photos";
            // Redraw screen
            this.render();
            this.bindOnNavigate('a.brand', function () {
                // bind events
                client.dialog.bindDialogHeader(_this.$el);
                _this.$el.on('click', '#albumNext', function (e) {
                    $(e.currentTarget).addClass('waiting');
                    // fetch more
                    _this.items.next(function (e, nextPage) {
                        $('#ablumNext').remove();
                        if (nextPage) {
                            _this.items.models = _this.items.models.concat(nextPage.models);
                            _this.items.next = nextPage.next;
                            _this.appendTiles(_this.getTiles(nextPage, true));
                        }
                    });
                });
            });
            return _super.prototype.navigate.call(this, route);
        };
        MockSourceScreen.prototype.render = function () {
            // Make screen is in a valid state
            if (this.getScreenState() == 3 /* DISPOSED */) {
                client.logger.warn("Render called on DISPOSED screen ");
                return this;
            }
            // Get tiles
            client.logger.info('MockSource render start');
            var allTiles = {
                albums: this.getTiles(this.items)
            };
            if (!this.$el) {
                client.logger.error(consts.errorCode.ContentPicking, 'this.$el not defined for MockSourceScreen');
            }
            // Render view
            this.empty();
            this.$el.render('albums', _.extend({
                title: this.title
            }, allTiles));
            _super.prototype.render.call(this);
            this.contentSource = 'album'; // Override so can use albums.jade
            this.appendTiles(allTiles.albums);
            this.updateCommandBar();
            this.setDialogTitle("Select What");
            return this;
        };
        return MockSourceScreen;
    })(client.ContentPickingDialog);
    client.MockSourceScreen = MockSourceScreen;
    client.screens.addRoute('/mocksource', client.AlbumsScreen);
})(client || (client = {}));
/// <reference path='../../../types/cordova/cordova-camera.d.ts'/>
/// <reference path='./ScreenWizard.ts'/>
/// <reference path='./PickerScreen.ts'/>
/// <reference path='./SkydriveScreen.ts'/>
/// <reference path='./AlbumsScreen.ts'/>
/// <reference path='./DropboxScreen.ts'/>
/// <reference path='./InstagramScreen.ts'/>
/// <reference path='./PhotoCastScreen.ts'/>
/// <reference path='./DocCastScreen.ts'/>
/// <reference path='./CameraRollScreen.ts'/>
/// <reference path='./PeopleScreen.ts'/>
/// <reference path='../mockobjects/MockSourceScreen.ts'/>
/// <reference path='../Group.ts'/>
/// <reference path='../recentPeople.ts'/>
/// <reference path='../deviceContacts.ts'/>
/// <reference path='../../shared/models/SkyDriveApi.ts'/>
/// <reference path='../CameraRollUploader.ts'/>
var client;
(function (client) {
    var casting;
    (function (casting) {
        /** Types of sessions supported. */
        casting.SessionTypes = {
            photoCast: 'photocast',
            docCast: 'doccast',
            videoCast: 'videocast'
        };
        var itemTypeToSessionType = {
            'photo': casting.SessionTypes.photoCast,
            'video': casting.SessionTypes.videoCast,
            'default': casting.SessionTypes.docCast
        };
        /** Prepare and start a new Xim.  If have share targets, skip item collection step) */
        function prepareAndStartCast(haveShareTargets) {
            if (haveShareTargets === void 0) { haveShareTargets = false; }
            // Function to call when spec is returned, if successful, start Xim
            var specCallback = client.ifSuccess(function (spec) {
                client.PhotoCastScreen.startXim(spec, null);
            }, null);
            // if the activeViewer is already connected, don't show the people picking screen
            if (client.CastViewManager.activeViewer) {
                // If I already have items, just start the xim already
                if (haveShareTargets) {
                    // TODO: this can never happen since share target launch will 
                    // reload our app (and lose any previously connected viewer).
                    // Once this becomes possible, start the xim right away, no wizard screen will show
                    prepareCastingSpecFromShareTarget(specCallback);
                }
                else {
                    prepareAndStartCastForContacts([]);
                }
            }
            else {
                // If I already have items, just get people
                if (haveShareTargets) {
                    addPeopleToSpec(specCallback);
                }
                else {
                    addContentAndPeopleToSpec(specCallback);
                }
            }
        }
        casting.prepareAndStartCast = prepareAndStartCast;
        /** Prepare and start a new xim, show only the content picker screens since we already have contacts*/
        function prepareAndStartCastForContacts(contacts) {
            var invites = _.map(contacts, function (contact) {
                var inviteType = contact.phoneNumber ? consts.InviteType.sms : (contact.email ? consts.InviteType.email : consts.InviteType.none);
                var inviteAddress = contact.phoneNumber || contact.email;
                var displayName = contact.name || (inviteType == consts.InviteType.sms ? contact.phoneNumber.substr(contact.phoneNumber.length - 4) : contact.email.substr(0, contact.email.indexOf('@'))); // email name before '@'
                return {
                    inviteType: inviteType,
                    inviteAddress: inviteAddress,
                    color: client.ColorCycle.GetNext(),
                    displayName: displayName
                };
            });
            var spec = {
                invites: invites,
                items: [],
                mapToScreenQueueItems: null
            };
            addContentToSpec(spec, client.ifSuccess(function (spec) {
                client.PhotoCastScreen.startXim(spec, null);
            }, null));
        }
        casting.prepareAndStartCastForContacts = prepareAndStartCastForContacts;
        /** add content to spec */
        function addContentToSpec(spec, callback) {
            var screens = [];
            screens.push(client.wizard.wrapScreen(client.PickerScreen), (function (cb, href) { return sourceScreenGenerator(cb, href, consts.commands.start); }));
            var wizard = client.wizard.wizard(screens, callback);
            wizard.commands = [
                _.extend({}, consts.commands.start, { enabled: false })
            ];
            wizard.commandCallback = function (id, command) {
                if (id == consts.commands.start.id) {
                    var currentScreen = (wizard.currentScreen());
                    wizard.finalize({
                        invites: spec.invites,
                        items: wizard.bag['selected'],
                        mapToScreenQueueItems: _.bind(currentScreen.mapToScreenQueueItems, currentScreen),
                        photoUploaded: currentScreen.photoUploaded
                    });
                }
                else {
                    client.dialog.handleDialogCommand(id, command);
                }
            };
            wizard.begin();
        }
        casting.addContentToSpec = addContentToSpec;
        casting.screenQueueItemsEvent = new client.Event();
        var _screenQueueItemsEventHandler = null;
        var _photoUploadedEvent;
        /** Displays a people & content picking wizard to the user, returns casting spec. */
        function addContentAndPeopleToSpec(callback) {
            // Wizard screens
            var screens = [];
            screens.push(client.wizard.wrapScreen(client.PickerScreen), (function (cb, href) { return sourceScreenGenerator(cb, href, consts.commands.next); }), _.wrap(client.wizard.wrapScreen(client.PeopleScreen), function (f) {
                var args = [];
                Array.prototype.push.apply(args, arguments);
                args.shift();
                // Get content provider
                var contentPicker = (wizard.currentScreen());
                // Process selected content
                processSelectedContent(wizard, contentPicker);
                f.apply(null, args);
            }));
            var wizard = client.wizard.wizard(screens, callback);
            // command bar commands
            wizard.commands = [
                _.extend({}, consts.commands.next, { enabled: false })
            ];
            // command bar callback
            wizard.commandCallback = function (id, command) {
                handleCommandCallback(wizard, id, command);
            };
            wizard.begin();
        }
        casting.addContentAndPeopleToSpec = addContentAndPeopleToSpec;
        /** ask the shareHandler to prepare the items in our casting spec */
        function prepareCastingSpecFromShareTarget(callback) {
            var photoUploaded = new client.Event();
            var contentProvider = {
                photoUploaded: photoUploaded,
                mapToScreenQueueItems: function (contentItems, queueItemsCallback) {
                    // Use shareHandler
                    client.shareHandler.mapToScreenQueueItems(contentItems, photoUploaded, queueItemsCallback);
                }
            };
            _photoUploadedEvent = contentProvider.photoUploaded;
            // Map users selection to screen queue items
            // This will start any uploading of content in preparation for casting   
            contentProvider.mapToScreenQueueItems(null, function (e, items) {
                // notifying any screen that might be interested in this
                casting.screenQueueItemsEvent.trigger(items);
                callback(e, e ? null : _.extend({ items: items }, contentProvider));
            });
        }
        /** Displays only people picking wizard to the user, returns casting spec.  Used for share targets */
        function addPeopleToSpec(callback) {
            // Wizard screens
            var screens = [];
            screens.push(_.wrap(client.wizard.wrapScreen(client.PeopleScreen), function (f) {
                var args = [];
                Array.prototype.push.apply(args, arguments);
                args.shift();
                prepareCastingSpecFromShareTarget(function (e, spec) {
                    wizard.bag['selected'] = e ? [] : spec.items;
                });
                // UI should switch to start cast command
                client.wizard.currentWizard.commands = [_.extend({}, consts.commands.start, { enabled: false })];
                f.apply(null, args);
            }));
            var wizard = client.wizard.wizard(screens, callback);
            // UI should display start cast command
            wizard.commands = [_.extend({}, consts.commands.start, { enabled: false })];
            // command bar callback
            wizard.commandCallback = function (id, command) {
                handleCommandCallback(wizard, id, command);
            };
            wizard.begin();
        }
        casting.addPeopleToSpec = addPeopleToSpec;
        function handleCommandCallback(wizard, id, command) {
            if (id == consts.commands.next.id) {
                // Record that user has selected next on contenct picking screen
                client.logger.collectNamedEvent(consts.flurryEvent.picksourceNext);
                // goto next screen
                wizard.currentScreen().history.close();
            }
            else if (id == consts.commands.start.id) {
                client.AppMasterScreen.checkConnection(function () {
                    client.logger.debug('about to cast');
                    // Generate invites (if there are any)
                    var invites = wizard.bag['invites'] ? getInvites(wizard.bag['invites'].slice(0)) : null;
                    wizard.finalize({
                        items: wizard.bag['selected'],
                        invites: invites,
                        photoUploaded: _photoUploadedEvent,
                        mapToScreenQueueItems: function (x, c) {
                            // All items may have already been mapped, so do nothing
                            if (x && x.length) {
                                _.defer(function () {
                                    c(null, x);
                                });
                            }
                            else {
                                // wait until our content picking screen finishes the mapping
                                if (!_screenQueueItemsEventHandler) {
                                    _screenQueueItemsEventHandler = _.once(function (items) {
                                        casting.screenQueueItemsEvent.remove(_screenQueueItemsEventHandler);
                                        _screenQueueItemsEventHandler = null;
                                        c(null, items);
                                    });
                                    casting.screenQueueItemsEvent.add(_screenQueueItemsEventHandler);
                                }
                                else {
                                    client.logger.debug('still have a pending cast');
                                    c(new Error('overlapping pending cast'), []);
                                }
                            }
                        }
                    });
                });
            }
            else {
                client.dialog.handleDialogCommand(id, command);
            }
        }
        casting.handleCommandCallback = handleCommandCallback;
        /** Process user selected content in preparation to start cast (from contentpicking or share target) */
        function processSelectedContent(wizard, contentProvider) {
            // we may need to first get the right url for cast
            var selectedItems = wizard.bag['selected'];
            wizard.bag['selected'] = [];
            _photoUploadedEvent = contentProvider.photoUploaded;
            // Map users selection to screen queue items
            // This will start any uploading of content in preparation for casting   
            // TODO: remove TEMP FIX that trimmed the seleced set of image to 50
            contentProvider.mapToScreenQueueItems(selectedItems, function (e, items) {
                client.logger.info('content picker mapped items.length', items ? items.length : e);
                wizard.bag['selected'] = items;
                // notifying any screen that might be interested in this
                casting.screenQueueItemsEvent.trigger(items);
            });
            // UI should switch to start cast command
            client.wizard.currentWizard.commands = [_.extend({}, consts.commands.start, { enabled: false })];
        }
        casting.processSelectedContent = processSelectedContent;
        function getInvites(data) {
            // Get device profiles
            var deviceProfiles = data;
            // Update list of recently xim'd people
            client.RecentPeople.Update(deviceProfiles);
            // Convert to user invites
            var userInvites = [];
            for (var i = 0; i < deviceProfiles.length; i++) {
                var userInvite = {
                    displayName: deviceProfiles[i].displayName,
                    color: deviceProfiles[i].color,
                    inviteAddress: client.DeviceContacts.inviteAddress(deviceProfiles[i]),
                    inviteType: deviceProfiles[i].inviteType,
                    inviteGuid: deviceProfiles[i].inviteGuid
                };
                userInvites.push(userInvite);
            }
            return userInvites;
        }
        casting.getInvites = getInvites;
        /** add additional content */
        function addMoreContent(callback) {
            var screens = [];
            screens.push(client.wizard.wrapScreen(client.PickerScreen), (function (cb, href) { return sourceScreenGenerator(cb, href, consts.commands.addMedia); }));
            var wizard = client.wizard.wizard(screens, callback);
            wizard.commands = [
                _.extend({}, consts.commands.addMedia, { enabled: false })
            ];
            wizard.commandCallback = function (id, command) {
                if (id == consts.commands.addMedia.id) {
                    var currentScreen = (wizard.currentScreen());
                    wizard.finalize({
                        invites: null,
                        items: wizard.bag['selected'],
                        mapToScreenQueueItems: _.bind(currentScreen.mapToScreenQueueItems, currentScreen),
                        photoUploaded: currentScreen.photoUploaded
                    });
                }
                else {
                    client.dialog.handleDialogCommand(id, command);
                }
            };
            wizard.begin();
        }
        casting.addMoreContent = addMoreContent;
        function addMorePeople(exclude, callback) {
            var screens = [];
            screens.push(client.wizard.wrapScreen(client.PeopleScreen));
            var wizard = client.wizard.wizard(screens, callback);
            wizard.bag['exclude'] = exclude;
            wizard.bag['selected'] = ['dummy'];
            wizard.commands = [
                _.extend({}, consts.commands.addPeople, { enabled: false })
            ];
            wizard.commandCallback = function (id, command) {
                if (id == consts.commands.addPeople.id) {
                    var invites = getInvites(wizard.bag['invites'].slice(0));
                    client.logger.collectNamedEvent(consts.flurryEvent.ximsessionAddpeople, { count: invites ? invites.length : 0 });
                    wizard.finalize({
                        invites: invites,
                        items: [],
                        mapToScreenQueueItems: null
                    });
                }
                else {
                    client.dialog.handleDialogCommand(id, command);
                }
            };
            wizard.begin();
        }
        casting.addMorePeople = addMorePeople;
        /** Setsup the content picker screen for the selected source. */
        function sourceScreenGenerator(cb, href, command) {
            var itemid = null;
            var tokens = href.split('/');
            var src = tokens.length > 1 ? tokens[1] : 'unknownSrc';
            src = src == 'albums' ? 'facebook' : src;
            if (src != 'mocksource') {
                client.logger.collect(src);
            }
            var firstCommand = _.first(client.wizard.currentWizard.commands);
            // if there is no firstCommand use the command passed in
            if (!firstCommand) {
                client.wizard.currentWizard.commands = [
                    _.extend({}, command, { enabled: false })
                ];
                firstCommand = _.first(client.wizard.currentWizard.commands);
            }
            var castingToViewer = firstCommand && firstCommand.id == consts.commands.start.id;
            var insideXim = firstCommand && firstCommand.id == consts.commands.addMedia.id;
            switch (href) {
                case '#/skydrive/docs':
                case '#/skydrive/photos':
                    itemid = _.last(href.split('/'));
                case '#/skydrive':
                    client.logger.collectNamedEvent(consts.flurryEvent.picksourceType, { 'type': 'One Drive' });
                    client.logger.info('need to return skydrive screen generator');
                    client.user.loginLiveAsync(function (e, success) {
                        client.logger.info('live login callback', success);
                        if (success) {
                            cb(null, client.SkydriveScreen, { itemid: itemid });
                        }
                        else {
                            cb(new Error('live id not logged in'), null);
                        }
                    });
                    break;
                case '#/albums':
                    client.logger.collectNamedEvent(consts.flurryEvent.picksourceType, { 'type': 'Facebook' });
                    client.user.loginFacebookAsync(function (e, success) {
                        if (success) {
                            cb(null, client.AlbumsScreen);
                        }
                        else {
                            cb(new Error('facebook not logged in'), null);
                        }
                    });
                    break;
                case '#/dropbox':
                    client.logger.collectNamedEvent(consts.flurryEvent.picksourceType, { 'type': 'Dropbox' });
                    client.user.loginDropboxAsync(function (e, success) {
                        if (success) {
                            cb(null, client.DropboxScreen);
                        }
                        else {
                            cb(new Error('dropbox not logged in'), null);
                        }
                    });
                    break;
                case '#/instagram':
                    client.logger.collectNamedEvent(consts.flurryEvent.picksourceType, { 'type': 'Instagram' });
                    client.user.loginInstagramAsync(function (e, success) {
                        if (success) {
                            cb(null, client.InstagramScreen);
                        }
                        else {
                            cb(new Error('Instagram not logged in'), null);
                        }
                    });
                    break;
                case '#/cameraroll':
                    client.logger.collectNamedEvent(consts.flurryEvent.picksourceType, { 'type': 'Camera Roll' });
                    setTimeout(function () {
                        cb(null, client.CameraRollScreen);
                    }, 0);
                    break;
                case '#/takephoto':
                    client.logger.collectNamedEvent(consts.flurryEvent.picksourceType, { 'type': 'Take Photo' });
                    casting.selectSinglePhoto(function (err, preview) {
                        if (!err && preview && preview.thumbnail) {
                            var curWizard = client.wizard.currentWizard;
                            var curScreen = curWizard.currentScreen();
                            // skip to next screen
                            curWizard.bag['selected'] = [preview];
                            if (insideXim || castingToViewer) {
                                curWizard.finalize({
                                    invites: insideXim ? null : [],
                                    items: curWizard.bag['selected'],
                                    mapToScreenQueueItems: _.bind(curScreen.mapToScreenQueueItems, curScreen),
                                    photoUploaded: curScreen.photoUploaded
                                });
                            }
                            else {
                                curWizard.nextScreen(cb);
                            }
                        }
                        else {
                            cb(null, null);
                        }
                    });
                    break;
                case '#/mocksource':
                    client.logger.collectNamedEvent(consts.flurryEvent.picksourceType, { 'type': 'Mock Camera' });
                    if (client.startupMode == 1 /* emulator */) {
                        cb(null, client.MockSourceScreen);
                    }
                    else {
                        cb(new Error('Mock photos only allowed in emulator'), null);
                    }
                    break;
                default:
                    cb(new Error('unknown href' + href), null);
            }
        }
        function fileUrlToPreviewItem(fileUrl) {
            client.logger.info('user picked local file', fileUrl);
            var leaf = _.last(fileUrl.split('/'));
            leaf = _.last(leaf.split('\\'));
            return {
                id: leaf,
                name: leaf,
                type: 'photo',
                thumbnail: { source: fileUrl, width: 1080, height: 1080 },
                images: [{ source: fileUrl, width: 1080, height: 1080 }]
            };
        }
        /** invoke camera plugin to take a single photo*/
        function selectSinglePhoto(callback) {
            var nav = navigator;
            // on Samsung S4 (most popular android phone), we are seeing viewport
            // issue (window.innerWidth = 480, innerHeight=270) when coming back
            // from camera launcher, the wrapped callback below forces the browser to re-render
            var wrappedCallback = _.wrap(callback, function (inner, err, preview) {
                inner(err, preview);
                var $screen = $('#screens .screen-modal:last-child');
                $screen.toggleClass('hidden', true);
                _.delay(function () {
                    client.logger.info('camera take photo post process');
                    $screen.toggleClass('hidden', false);
                }, 200);
            });
            if (nav.camera) {
                var camera = nav.camera;
                camera.getPicture(function (fileUrl) {
                    wrappedCallback(null, fileUrlToPreviewItem(fileUrl));
                }, function (message) {
                    client.logger.info('camera take photo failed');
                    wrappedCallback(new Error(message || "Unknown camera failure"), null);
                }, {
                    quality: 75,
                    sourceType: Camera.PictureSourceType.CAMERA,
                    destinationType: Camera.DestinationType.FILE_URI,
                    mediaType: Camera.MediaType.PICTURE,
                    targetWidth: 1080,
                    targetHeight: 1080,
                    encodingType: Camera.EncodingType.JPEG,
                    correctOrientation: true
                });
            }
            else {
                _.defer(function () {
                    callback(new Error('no camera plugin found'), null);
                });
            }
        }
        casting.selectSinglePhoto = selectSinglePhoto;
        function startCast(grp, spec, finished) {
            var session_done = function (e, group, result) {
                // TODO: leave cast
                if (group) {
                }
                if (finished) {
                    finished(e, result);
                }
            };
            var items = spec.items;
            var first = _.first(items);
            var type = first && first.type ? itemTypeToSessionType[first.type] : itemTypeToSessionType['photo'];
            switch (type) {
                case casting.SessionTypes.photoCast:
                    client.PhotoCastScreen.startCast(grp, spec, function (e) {
                        session_done(e, grp, true);
                    });
                    break;
                case casting.SessionTypes.videoCast:
                    break;
                case casting.SessionTypes.docCast:
                    break;
            }
        }
    })(casting = client.casting || (client.casting = {}));
})(client || (client = {}));
/// <reference path="Screen.ts"/>
/// <reference path="../controls/ErrorPopUp.ts"/>
/// <refernece path='./DialogScreen.ts'/>
/// <reference path='./ScreenWizard.ts'/>
/// <reference path='../../../types/cordova/screenlock.d.ts'/>
/// <reference path="../controls/SideMenu.ts"/>
/// <reference path="../controls/CommandBar.ts"/>
/// <reference path="../controls/ConfirmationPopUp.ts"/>
/// <reference path='../user.ts'/>
/// <reference path='../resources.ts'/>
/// <reference path='casting.ts'/>
var client;
(function (client) {
    /** Applications custom master screen implementation. */
    var AppMasterScreen = (function () {
        function AppMasterScreen(selector) {
            if (selector === void 0) { selector = '#screens'; }
            this.selector = selector;
            // Command setup
            this.commands = [];
            this.commandHandlers = {};
            this.commandPaths = {};
            // Debug settings enable
            this.debugClickCount = 0;
            client.user.onToastNotification.add(_.bind(this.onToastNotification, this));
        }
        /** Check that user has a conection before initiating an action */
        AppMasterScreen.checkConnection = function (action, failaction) {
            if (failaction === void 0) { failaction = function () {
            }; }
            switch (client.user.getConnectionState()) {
                case 0 /* logout */:
                case 1 /* authorizing */:
                default:
                    // Invalid state
                    failaction();
                    break;
                case 2 /* authorized */:
                    // Show pop up and try to initiate a reconnect
                    new client.ErrorPopUp(client.ErrorMessages.CONNECTION_BACK, failaction).show(true);
                    client.user.ensureConnected();
                    break;
                case 3 /* connecting */:
                case 5 /* reconnecting */:
                    // Already trying reconnect to just show pop up
                    new client.ErrorPopUp(client.ErrorMessages.RECONNECTING, failaction).show(true);
                    break;
                case 4 /* connected */:
                    // Connected so take action
                    action();
                    break;
            }
        };
        // Used to get screen info for debug output
        AppMasterScreen.prototype.debugGetScreenInfo = function () {
            var desc = "";
            if (this.curScreen) {
                desc += "Screen: " + (this.curScreen.constructor ? this.curScreen.constructor.name + consts.linebreak : '');
            }
            if (this.curPath) {
                desc += "Path: " + this.curPath + consts.linebreak;
            }
            return desc;
        };
        AppMasterScreen.prototype.addDetailScreen = function (path, screen, menuCommand) {
            var _this = this;
            client.app.addRoute(path, function (route) {
                _this.routeTo(screen, path, route);
            });
            if (menuCommand) {
                this.commandPaths[path] = menuCommand.id;
                this.addMenuCommand(menuCommand, function () {
                    // Navigate to screen
                    if (_this.curPath != path) {
                        client.app.navigate(path);
                    }
                });
            }
            return this;
        };
        AppMasterScreen.prototype.addMenuCommand = function (command, callback) {
            this.commands.push(command);
            if (callback) {
                this.commandHandlers[command.id] = callback;
            }
            return this;
        };
        AppMasterScreen.prototype.removeMenuCommand = function (command) {
            this.commands = _.filter(this.commands, function (c) {
                return c.id != command.id;
            });
        };
        // Closes side menu if open, and returns true
        AppMasterScreen.prototype.back = function () {
            if (this.navMenu && this.navMenu.isOpen()) {
                this.navMenu.show(false);
                return true;
            }
            return false;
        };
        /** context sensitive menu command visibility, debug commands only shown at debug screen*/
        AppMasterScreen.prototype.shouldShowMenuCommand = function (command) {
            var isOnDebugScreen = this.curPath == '/debug';
            var show = false;
            if (isOnDebugScreen) {
                // on debug screen we only show debug commands
                show = !!consts.debugCommands[command.id];
            }
            else {
                // on non-debug screen only show non debug command plus 
                // if user has set show_debug, we will only show entry point to debug screen or log screen
                show = !consts.debugCommands[command.id] || (client.user.GetSetting(6 /* SHOW_DEBUG */) && (consts.debugCommands.debug.id == command.id || consts.debugCommands.log.id == command.id));
            }
            return show;
        };
        AppMasterScreen.prototype.refreshNavMenu = function () {
            if (this.navMenu && this.commands && this.commands.length) {
                this.navMenu.setCommands(_.filter(this.commands, _.bind(this.shouldShowMenuCommand, this)));
            }
        };
        /** Called anytime a screen has been navigated within or to. */
        AppMasterScreen.prototype.routeTo = function (screen, path, route) {
            // Find master page
            var $master = this.findMaster();
            if ($master.length) {
                // Create new screen
                if (this.curPath !== path || !this.curScreen) {
                    // Remove old screen
                    if (this.curScreen) {
                        this.curScreen.remove();
                        this.curScreen = null;
                        this.curPath = '';
                    }
                    // Cleanup command bar state
                    this.commandBar.setCommands();
                    // Create new screen
                    this.curPath = path;
                    var $screen = $('<div class="detail screen"/>');
                    this.curScreen = new screen($screen);
                    this.curScreen.commandBar = this.commandBar;
                    $screen.appendTo($master.find('.detail-screens'));
                    $(".header").toggleClass('hidden', false);
                    // refresh the nav menu since we are now on a different screen
                    this.refreshNavMenu();
                }
                // Default is that screen is locked to vertical - screens can overwrite
                if (window.screenlock) {
                    window.screenlock.set(consts.ScreenLockState.vertical);
                }
                this.curScreen.navigate(route);
            }
        };
        /** Called anytime a modal is about to be displayed. */
        AppMasterScreen.prototype.beginModal = function (options, callback) {
            var _this = this;
            //var isOverlay = (options.style == ModalStyle.popup);
            var $master = $(this.selector).children('.master.screen');
            //if ($master.length > 0 && !isOverlay) $master.hide();
            var showDialogClass = 'show-dialog';
            var prevDialogClasses = $master.length ? _.filter($master.attr('class').split(' '), function (c) {
                return c.indexOf('show-dialog') >= 0;
            }) : [];
            if (options && options.style == 2 /* fullScreen */) {
                showDialogClass = 'show-dialog show-dialog-flush';
                $master.toggleClass('show-dialog show-dialog-flush', true);
            }
            else {
                $master.toggleClass('show-dialog', true).toggleClass('show-dialog-flush', false);
            }
            callback(this.selector, function () {
                if (_this.curScreen)
                    _this.curScreen.render();
                //if ($master.length > 0 && !isOverlay) $master.show();
                $master.toggleClass(showDialogClass, false).toggleClass(prevDialogClasses.join(' '), true);
            });
        };
        /** Finds and creates as needed the master screen. */
        AppMasterScreen.prototype.findMaster = function () {
            var _this = this;
            var $master = $(this.selector).children('.master.screen');
            if ($master.length == 0) {
                // Create master screen
                $master = $('<div class="master screen"/>');
                $master.render('appMasterScreen', { resources: resources });
                $master.appendTo(this.selector);
                // Change heading color when int
                if (consts.buildFlavor == 'beta') {
                    $master.find(".header").css({ 'background-color': '#CC126F' });
                }
                // Setup nav menu
                this.navMenu = new client.SideMenu($master.find('.navMenuContent'), _.bind(this.onNavClick, this));
                this.navMenu.position = client.SideMenuPosition.left;
                this.navMenu.setCommands(_.filter(this.commands, function (command) {
                    return client.user.GetSetting(6 /* SHOW_DEBUG */) || command.id != consts.tabs.settings.id;
                }));
                // Setup command bar
                this.commandBar = new client.CommandBar($master.find('.commandBarContent'));
                // Add castViewer Icon
                var $header = $master.find(".header");
                this.iconViewer = new client.CastViewerIcon($header, 'timeline');
                // Listen for events
                $master.on('click', '.backglyph', function () {
                    _this.commandHandlers["home"]();
                });
                // Listen for events
                $master.on('click', '.cogglyph', function () {
                    if (!_this.navMenu.isOpen()) {
                        client.logger.collectNamedEvent(consts.flurryEvent.ximCog, { 'opening': true });
                        var id = _this.commandPaths[_this.curPath];
                        _this.commandBar.show(false);
                        _this.navMenu.show(true, id);
                        _this.iconViewer.show(false);
                    }
                    else {
                        client.logger.collectNamedEvent(consts.flurryEvent.ximCog, { 'opening': false });
                        _this.navMenu.show(false);
                        _this.restoreCommandBar();
                        _this.iconViewer.show(true);
                    }
                    //-------------------------------------------------
                    // Enable debug settings if user clicks 7 times in a row
                    clearTimeout(_this.debugClickTimer);
                    _this.debugClickCount++;
                    _this.debugClickTimer = setTimeout(function () {
                        _this.debugClickCount = 0;
                    }, 1000);
                    if (_this.debugClickCount >= 7) {
                        if (!client.user.GetSetting(6 /* SHOW_DEBUG */)) {
                            client.user.SetSetting(6 /* SHOW_DEBUG */, true);
                            (client.screens.master).refreshNavMenu();
                        }
                    }
                });
                $master.find('.navMenuContent').on('click', function () {
                    _this.showSideMenu(false);
                    _this.restoreCommandBar();
                });
            }
            return $master;
        };
        // Hide/Show nav menu
        AppMasterScreen.prototype.showSideMenu = function (show) {
            this.navMenu.show(show);
            this.iconViewer.show(!show);
        };
        AppMasterScreen.prototype.restoreCommandBar = function () {
            if (this.commandBar.hasCommands()) {
                this.commandBar.show(true);
            }
        };
        /** An option under the nav menu has been selected. */
        AppMasterScreen.prototype.onNavClick = function (id) {
            // Close the nav menu
            this.showSideMenu(false);
            // Invoke command
            if (id in this.commandHandlers) {
                this.commandHandlers[id]();
            }
        };
        /** wait for group.cast to be downloaded */
        AppMasterScreen.prototype.waitForCastId = function (group, next) {
            function _waitForCastId(next, attempt) {
                if (attempt === void 0) { attempt = 0; }
                // Wait up to 15 seconds for castId to show up.Actual timeout screen 
                // will be drawn after "Connecting to server" screen is closed.
                if (attempt == 15) {
                    next(new Error("Cast not started"), null);
                }
                else if (group && group.profile && group.profile.castId) {
                    next(null, group.profile.castId);
                }
                else {
                    setTimeout(function () { return _waitForCastId(next, attempt + 1); }, 1000);
                }
            }
            _waitForCastId(next);
        };
        /** convert toast notification to sessionInvite */
        AppMasterScreen.prototype.onToastNotification = function (notification) {
            var _this = this;
            var qs = querystring.parse(notification.param);
            var sessionId = qs["sessionId"];
            client.logger.info('handle push notification with sessionID ' + sessionId);
            var hasWizard = !!client.wizard.currentWizard;
            var hasModal = !!client.screens.modal && client.screens.modal.getScreenState() != 3 /* DISPOSED */;
            var hasPopup = client.PopUp.history.length > 0;
            if (notification.isLaunch || (!hasWizard && !hasModal && !hasPopup && !notification.isForeground)) {
                // wait until screens.master is up, the toast plugin
                // may call us while inside the deviceReady sequence
                // wait until the master screen is up
                var t = setInterval(function () {
                    client.logger.info('check if we are ready to join cast yet');
                    if (client.screens.master && client.app.isStarted()) {
                        _this.joinInvitedCast(sessionId, client.casting.SessionTypes.photoCast);
                        clearInterval(t);
                    }
                    else {
                        client.logger.info('waiting on app state', client.app.State[client.app.getState()]);
                    }
                }, 50);
            }
            else {
                // we are already in app, check if we are not on the timeline screen
                // then prompt user that something new has happened
                // TODO: since the push could come in after user saw 
                // the new xim and enters it already, we may want to
                // check any existing xim matching
                if (hasModal || hasPopup || hasWizard) {
                    if (phone && phone.notification && phone.notification.vibrate) {
                        phone.notification.vibrate(500);
                    }
                }
            }
        };
        /**  join the cast session */
        AppMasterScreen.prototype.joinInvitedCast = function (sessionId, type) {
            var _this = this;
            client.logger.info('Joining cast with session id =', sessionId);
            // End any current wizard as well (note: ideally we would want to suspend and resume)
            if (client.wizard.currentWizard) {
                client.wizard.currentWizard.finalize(null);
            }
            switch (type) {
                case client.casting.SessionTypes.photoCast:
                    client.logger.info('Joining photo cast with group id =', sessionId);
                    if (sessionId && sessionId.length > 6) {
                        client.user.getGroups([sessionId.substr(6)], function (e, groups) {
                            client.logger.info('getGroups result = ', groups, 'with eror = ', e);
                            if (!e && groups && groups.models && groups.models.length) {
                                var g = groups.models[0];
                                // check if the group has expired
                                var lastActivity = (g && g.profile && g.profile.lastActivity) ? moment(g.profile.lastActivity) : moment();
                                if (moment().diff(lastActivity, 'minutes') >= consts.max_xim_age_minutes) {
                                    // this xim is now considered expired
                                    var popup = new client.ErrorPopUp(client.ErrorMessages.XIM_EXPIRED, function () {
                                    });
                                    popup.show(true);
                                }
                                else {
                                    _this.waitForCastId(g, function (e, castId) {
                                        if (!e) {
                                            client.PhotoCastScreen.joinCast(g, castId, function () {
                                                client.logger.info('Casting session finished');
                                            });
                                        }
                                        else {
                                            client.logger.error(consts.errorCode.CastInit, 'waitForCast error', e);
                                        }
                                    });
                                }
                            }
                            else {
                                // we didn't get a valid group
                                client.logger.error(consts.errorCode.CastInit, "client.user.getGroups failed for sessionId", sessionId);
                                var popup = new client.ErrorPopUp(client.ErrorMessages.SERVER_ERROR, function () {
                                });
                                popup.show(true);
                            }
                        });
                    }
                    else {
                        client.logger.error(consts.errorCode.CastInit, 'unexpected sessionId', sessionId);
                    }
                    break;
                case client.casting.SessionTypes.videoCast:
                    client.logger.info('Joining video cast');
                    break;
                case client.casting.SessionTypes.docCast:
                default:
                    client.logger.info('Joining document cast');
                    break;
            }
        };
        return AppMasterScreen;
    })();
    client.AppMasterScreen = AppMasterScreen;
})(client || (client = {}));
var client;
(function (client) {
    /** Login: Get country */
    var CountryPopUp = (function (_super) {
        __extends(CountryPopUp, _super);
        function CountryPopUp($el, countryCodeMap, callback) {
            var _this = this;
            // Generate sorted country array
            var countryArray = this.generateCounryArray(countryCodeMap);
            // Call superclass to render Popup
            _super.call(this, $el, "countryPopUp", { title: resources.labels.loginCountryTitle, countryArray: countryArray });
            // On selection, return selected country code to callback and close self
            this.$el.on('click', '.countryPopUpItem', function (event) {
                if (callback) {
                    callback(event.currentTarget.id);
                }
                _this.destroy();
            });
            this.show(true);
        }
        /** Return alphabetically sorted array of county name /country code pairs */
        CountryPopUp.prototype.generateCounryArray = function (countryCodeMap) {
            var countryArray = _.map(countryCodeMap, function (item) {
                return {
                    name: item.country_name,
                    code: item.alpha2
                };
            });
            countryArray = countryArray.sort(function (a, b) {
                return (b.name < a.name) ? 1 : -1;
            });
            return countryArray;
        };
        return CountryPopUp;
    })(client.PopUp);
    client.CountryPopUp = CountryPopUp;
})(client || (client = {}));
/// <reference path="../consts.ts"/>
/// <reference path="./AppMasterScreen.ts"/>
/// <reference path='./DialogScreen.ts'/>
/// <reference path='../util.ts'/>
/// <reference path='../../../types/cordova/userprofile.d.ts' />
/// <reference path='../controls/CountryPopUp.ts'/>
var client;
(function (client) {
    /** Login Dialog: Get phone number */
    var LoginPhoneScreen = (function (_super) {
        __extends(LoginPhoneScreen, _super);
        function LoginPhoneScreen() {
            _super.apply(this, arguments);
            // default to US number in case the user is using a WIFI only device
            // or a phone with no SIM card, this will still present user the (+1)
            // in the phone number input field, as a cue for user to correct
            this.isoCountry = 'US';
            this.countryCodeMap = null;
        }
        LoginPhoneScreen.prototype.navigate = function (route) {
            var _this = this;
            // Get country code
            if (window.userprofile && window.userprofile.isoCountryCode) {
                window.userprofile.isoCountryCode(function (code) {
                    _this.isoCountry = code || _this.isoCountry;
                }, function (err) {
                    client.logger.warn('failed to get iso country code');
                });
            }
            // Redraw screen
            this.render();
            return _super.prototype.navigate.call(this, route);
        };
        /** Update UI to reflect country code */
        LoginPhoneScreen.prototype.updateCountryCode = function () {
            if (this.countryCodeMap && this.isoCountry && (this.isoCountry in this.countryCodeMap)) {
                var name = this.countryCodeMap[this.isoCountry].alpha3;
                var code = '+' + this.countryCodeMap[this.isoCountry].country_code;
                var placeholder = "";
                if (this.isoCountry == 'US') {
                    // For us use (555) 555-5555 placeholder
                    placeholder = resources.login.phone.inputCue;
                }
                else {
                    for (var i = 0; i < this.countryCodeMap[this.isoCountry].min_number_length; i++) {
                        placeholder += "5";
                    }
                }
                this.$el.find('.loginLabelCountry').text(name);
                this.$el.find('.loginButtonCountry').prop('value', code);
                this.$el.find('.loginInputTextPhone').prop('placeholder', placeholder);
            }
        };
        LoginPhoneScreen.prototype.render = function () {
            var _this = this;
            // Make screen is in a valid state
            if (this.getScreenState() == 3 /* DISPOSED */) {
                client.logger.warn("Render called on DISPOSED screen ");
                return this;
            }
            this.$el.addClass('splash').render('loginPhoneScreen', { title: resources.labels.loginPhoneTitle, resources: resources.login.phone, countryname: this.isoCountry });
            _super.prototype.render.call(this);
            // Get country code map and update ui
            this.countryCodeMap = consts.countriesByAlpha2();
            this.setCountry(this.isoCountry);
            // Update UI
            this.updateCountryCode();
            this.bindOnNavigate('', function () {
                var _submitPhone = _.bind(_this.sumbitPhone, _this);
                _this.$el.on('click', '.loginButton', function () {
                    _submitPhone();
                });
                // Capture android "next" press events on keyboard (must be on keydown, comes as tab)
                _this.$el.on('keydown', '.loginInputTextPhone', function (event) {
                    if (event.keyCode == 9) {
                        _submitPhone();
                    }
                });
                _this.$el.on('click', '.loginButtonCountry', function () {
                    var $popUpContent = _this.$el.find('.loginCountryPopUp');
                    var countryPopUp = new client.CountryPopUp($popUpContent, _this.countryCodeMap, _.bind(_this.setCountry, _this));
                });
                _this.$el.on('click', '.loginLink#terms', function () {
                    client.logger.collectNamedEvent(consts.flurryEvent.registrationTerms);
                    window.open(consts.externalLinks.terms, '_system');
                });
                _this.$el.on('click', '.loginLink#privacy', function () {
                    client.logger.collectNamedEvent(consts.flurryEvent.registrationPrivacy);
                    window.open(consts.externalLinks.privacy, '_blank', 'location=no');
                });
                // Hide placeholder text on focus
                client.Util.hidePlaceholderOnFocus(_this.$el.find('.ximformcontrol'));
                // Clear error messages when new text is typed
                _this.$el.on('keyup input', '.loginInputTextPhone', function () {
                    _this.$el.find(".loginErrorPhone").text("");
                });
            });
            // Add ability to choose server response, view logs
            client.Login.EnableDebugFooter(this.$el.find('.loginPhone'));
            return this;
        };
        /** Sumbit phone number form */
        LoginPhoneScreen.prototype.sumbitPhone = function () {
            var phone = this.$el.find('input[name=phone]').val();
            this.setPhone(phone);
        };
        /** Set phone number ensuring that given phone string is valid for country */
        LoginPhoneScreen.prototype.setPhone = function (phone) {
            if (this.countryCodeMap && this.isoCountry) {
                var parsedPhone = consts.parsePhoneNumberForCountry(phone, this.isoCountry);
                if (parsedPhone.isValid) {
                    client.logger.collectNamedEvent(consts.flurryEvent.registrationPhoneNext, { 'valid': true });
                    client.Login.setPhone(phone);
                    this.$el.find(".loginErrorPhone").text("");
                    client.wizard.currentWizard.currentScreen().history.close();
                }
                else {
                    client.logger.collectNamedEvent(consts.flurryEvent.registrationPhoneNext, { 'valid': false });
                    this.$el.find(".loginErrorPhone").text(resources.login.phone.error);
                }
            }
        };
        LoginPhoneScreen.prototype.setCountry = function (isoCountry) {
            if (this.countryCodeMap && isoCountry) {
                this.isoCountry = isoCountry;
                var code = this.countryCodeMap[this.isoCountry].alpha2;
                client.Login.setCountryAlpha2(code);
                this.updateCountryCode();
            }
        };
        return LoginPhoneScreen;
    })(client.DialogScreen);
    client.LoginPhoneScreen = LoginPhoneScreen;
    client.screens.addRoute('/loginphone', LoginPhoneScreen);
})(client || (client = {}));
/// <reference path="./AppMasterScreen.ts"/>
/// <reference path='./DialogScreen.ts'/>
/// <reference path='./Login.ts'/>
var client;
(function (client) {
    /** Login Dialog: Choose SMS or Phone call verification  */
    var LoginChoiceScreen = (function (_super) {
        __extends(LoginChoiceScreen, _super);
        function LoginChoiceScreen() {
            _super.apply(this, arguments);
        }
        LoginChoiceScreen.prototype.navigate = function (route) {
            // Redraw screen
            this.render();
            return _super.prototype.navigate.call(this, route);
        };
        LoginChoiceScreen.prototype.render = function () {
            var _this = this;
            // Make screen is in a valid state
            if (this.getScreenState() == 3 /* DISPOSED */) {
                client.logger.warn("Render called on DISPOSED screen ");
                return this;
            }
            this.$el.addClass('splash').render('loginChoiceScreen', { title: resources.labels.loginChoiceTitle, resources: resources.login.choice, phone: client.Login.getPhone() });
            _super.prototype.render.call(this);
            // Hide phone number button if twilio doesn't support voice verification
            var countryCodeMap = consts.countriesByAlpha2();
            var voiceSupport = countryCodeMap[client.Login.getCountryAlpha2()].twilioVoice;
            if (!voiceSupport) {
                this.$el.find(".loginButtonPhone").hide();
            }
            this.bindOnNavigate('', function () {
                _this.$el.on('click', '.loginButtonSMS', function () {
                    _this.disableUserInput(true);
                    client.Login.setVerificationType(0 /* sms */);
                    client.Login.submitPhone(_.bind(_this.phoneCallback, _this));
                });
                _this.$el.on('click', '.loginButtonPhone', function () {
                    _this.disableUserInput(true);
                    client.Login.setVerificationType(1 /* voice */);
                    client.Login.submitPhone(_.bind(_this.phoneCallback, _this));
                });
                // Go to phone screen to re-enter phone number
                _this.$el.on('click', '.loginChangePhone', function () {
                    history.back();
                });
            });
            // Add ability to choose server response, view logs
            client.Login.EnableDebugFooter(this.$el.find('.loginPhone'));
            return this;
        };
        LoginChoiceScreen.prototype.disableUserInput = function (value) {
            if (value) {
                this.$el.find('.loginIntro').addClass("loginDisabled");
                this.$el.find('.loginChoiceInputbox').addClass("loginDisabled");
                this.$el.find('.loginChangePhone').addClass("loginDisabled");
                this.$el.find('.loginChoiceWait').show();
            }
            else {
                this.$el.find('.loginIntro').removeClass("loginDisabled");
                this.$el.find('.loginChoiceInputbox').removeClass("loginDisabled");
                this.$el.find('.loginChangePhone').removeClass("loginDisabled");
                this.$el.find('.loginChoiceWait').hide();
            }
        };
        LoginChoiceScreen.prototype.phoneCallback = function (err, loginToken) {
            client.logger.info('phone number submit result', err);
            if (!err) {
                this.$el.find(".dialogError").text("");
                client.wizard.currentWizard.currentScreen().history.close();
            }
            else if (err == client.LoginError.HANDLED) {
                // Error messaging was already handled by caller
                this.$el.find(".dialogError").text("");
            }
            else if (err == client.LoginError.INVALID_PHONE) {
                this.$el.find(".dialogError").text(resources.login.phone.error);
            }
            else {
                this.$el.find(".dialogError").text("Failed to connect to server.");
            }
            // Restore disabled input
            this.disableUserInput(false);
        };
        return LoginChoiceScreen;
    })(client.DialogScreen);
    client.LoginChoiceScreen = LoginChoiceScreen;
    client.screens.addRoute('/loginchoice', LoginChoiceScreen);
})(client || (client = {}));
/// <reference path="./AppMasterScreen.ts"/>
/// <reference path='./DialogScreen.ts'/>
var client;
(function (client) {
    /** Login Dialog: Verify SMS */
    var LoginVerifyScreen = (function (_super) {
        __extends(LoginVerifyScreen, _super);
        function LoginVerifyScreen() {
            _super.apply(this, arguments);
        }
        LoginVerifyScreen.prototype.navigate = function (route) {
            // Redraw screen
            this.render();
            return _super.prototype.navigate.call(this, route);
        };
        LoginVerifyScreen.prototype.render = function () {
            var _this = this;
            // Make screen is in a valid state
            if (this.getScreenState() == 3 /* DISPOSED */) {
                client.logger.warn("Render called on DISPOSED screen ");
                return this;
            }
            this.$el.addClass('splash').render('loginVerifyScreen', { title: resources.labels.loginVerifyTitle, resources: resources.login.verify });
            _super.prototype.render.call(this);
            this.bindOnNavigate('', function () {
                var _sumbitPin = _.bind(_this.sumbitPin, _this);
                _this.$el.on('click', '.loginButton', function () {
                    _sumbitPin();
                });
                // Capture android "next" press events on keyboard (must be on keydown, comes as tab)
                _this.$el.on('keydown', '.dialogInputText', function (event) {
                    if (event.keyCode == 9) {
                        _sumbitPin();
                    }
                });
                // Go to phone screen to resend invite
                _this.$el.on('click', '.loginResentSMS', function () {
                    client.logger.collectNamedEvent(consts.flurryEvent.registrationVerificationResend);
                    // Go back two screens to phone number entry
                    history.go(-2);
                });
                // Clear error messages when new text is typed
                _this.$el.on('keyup input', '.dialogInputText', function () {
                    _this.$el.find(".dialogError").text("");
                });
                // Hide placeholder text on focus
                client.Util.hidePlaceholderOnFocus(_this.$el.find('.ximformcontrol'));
            });
            // Add ability to choose server response, view logs
            client.Login.EnableDebugFooter(this.$el.find('.loginPhone'));
            return this;
        };
        /** Sumbit pin number form */
        LoginVerifyScreen.prototype.sumbitPin = function () {
            var code = this.$el.find('input[name=code]').val();
            this.disableUserInput(true);
            client.logger.collectNamedEvent(consts.flurryEvent.registrationVerificationNext);
            client.Login.validatePin(code, _.bind(this.verifyCallback, this));
        };
        LoginVerifyScreen.prototype.disableUserInput = function (value) {
            if (value) {
                this.$el.find('.loginIntro').addClass("loginDisabled");
                this.$el.find('.loginInput').addClass("loginDisabled");
                this.$el.find('.loginResentSMS').addClass("loginDisabled");
                this.$el.find('.loginWait').show();
            }
            else {
                this.$el.find('.loginIntro').removeClass("loginDisabled");
                this.$el.find('.loginInput').removeClass("loginDisabled");
                this.$el.find('.loginResentSMS').removeClass("loginDisabled");
                this.$el.find('.loginWait').hide();
            }
        };
        LoginVerifyScreen.prototype.verifyCallback = function (err, loginToken) {
            if (!err) {
                // todo: save the token to local storage
                this.$el.find(".dialogError").text("");
                client.wizard.currentWizard.currentScreen().history.close();
            }
            else if (err == client.LoginError.INVALID_PIN) {
                this.$el.find(".dialogError").text(resources.login.verify.error);
            }
            else if (err == client.LoginError.EXPIRED_PIN) {
                this.$el.find(".dialogError").text(resources.login.verify.expired);
            }
            else {
                this.$el.find(".dialogError").text("Failed to connect to server.");
            }
            // Restore disabled input
            this.disableUserInput(false);
        };
        return LoginVerifyScreen;
    })(client.DialogScreen);
    client.LoginVerifyScreen = LoginVerifyScreen;
    client.screens.addRoute('/loginverify', LoginVerifyScreen);
})(client || (client = {}));
/// <reference path="../types.ts"/>
/// <reference path="../resources.ts"/>
/// <reference path="../../../types/jquery/jquery.d.ts"/>
/// <reference path="PopUp.ts"/>
var client;
(function (client) {
    (function (ThumbnailType) {
        ThumbnailType[ThumbnailType["PHOTO"] = 0] = "PHOTO";
        ThumbnailType[ThumbnailType["INITIALS"] = 1] = "INITIALS";
    })(client.ThumbnailType || (client.ThumbnailType = {}));
    var ThumbnailType = client.ThumbnailType;
    /** User interface for choosing thumbnail type */
    var ThumbnailPopUp = (function (_super) {
        __extends(ThumbnailPopUp, _super);
        function ThumbnailPopUp(initials, thumbnail, color, callback) {
            var _this = this;
            // Call superclass to render Popup
            _super.call(this, null, "thumbnailPopUp", { initials: initials, thumbnail: thumbnail, color: color, resources: resources.labels.thumbnailPopUp }, { overlayOpacity: 0.7 });
            // Close on click, and eat other clicks so they don't propagate upward 
            this.$popup.on('click', function (event) {
                if (_this.isVisible()) {
                    event.preventDefault();
                    event.stopPropagation();
                    _this.destroy();
                }
            });
            // Capture the users choice
            this.$popup.on('click', '#thumbnailPopUpPhoto', function () {
                callback(0 /* PHOTO */);
            });
            this.$popup.on('click', '#thumbnailPopUpInitials', function () {
                callback(1 /* INITIALS */);
            });
            // Size to center in screen
            this.resize();
        }
        // Center on screen (takes into account that it may have rotated)
        ThumbnailPopUp.prototype.resize = function () {
            var screenWidth = client.webview.currentViewport().width;
            var screenHeight = client.webview.currentViewport().height;
            var x = (screenWidth - this.$popup.width()) / 2;
            var y = (screenHeight - this.$popup.height()) / 2;
            this.$popup.css({ marginLeft: Math.round(x) + 'px', marginTop: Math.round(y) + 'px' });
        };
        return ThumbnailPopUp;
    })(client.PopUp);
    client.ThumbnailPopUp = ThumbnailPopUp;
})(client || (client = {}));
/// <reference path="../screens/Login.ts"/>
var client;
(function (client) {
    /** Login Dialog: Set up profile */
    var ProfileEditor = (function () {
        function ProfileEditor(selector, getThumbnail, onClose, profile) {
            if (profile === void 0) { profile = null; }
            this.getThumbnail = getThumbnail;
            this.onClose = onClose;
            this.profile = profile;
            this.thumbnail = null;
            this.color = client.ColorCycle.GetNext();
            this.$el = $(selector);
            this.$el.empty();
        }
        ProfileEditor.prototype.render = function () {
            // Extract first / last name
            var firstName = "";
            var lastName = "";
            if (this.profile) {
                var firstSpace = this.profile.displayName.indexOf(" ");
                if (firstSpace == -1) {
                    firstName = this.profile.displayName;
                }
                else {
                    firstName = this.profile.displayName.substr(0, firstSpace);
                    lastName = this.profile.displayName.substr(firstSpace + 1);
                }
                if (!this.profile.color) {
                    this.profile.color = client.ColorCycle.GetNext();
                }
                this.color = this.profile.color;
            }
            this.$el.render('profileEditor', {
                title: resources.labels.loginProfileTitle,
                resources: resources.login.profile,
                thumbnail: ProfileEditor.defaultThumbnail,
                backcolor: this.color,
                profile: this.profile,
                firstName: firstName,
                lastName: lastName
            });
            // Init thumbnail view
            var thumbnail = this.thumbnail ? this.thumbnail : (this.profile ? this.profile.thumbnailUrl : null);
            this.setThumbnail(thumbnail);
            if (client.user.GetSetting(13 /* WAITING_FACEBOOK */)) {
                client.user.SetSetting(13 /* WAITING_FACEBOOK */, false);
                this.onFacebookAutoFill();
            }
        };
        ProfileEditor.prototype.initEvents = function () {
            this.$el.on('click', '.profileButton', _.bind(this.createProfile, this));
            this.$el.on('click', '.loginThumbnailImage', _.bind(this.onThumbnailClick, this));
            this.$el.on('click', '.loginThumbnailInitial', _.bind(this.onThumbnailClick, this));
            this.$el.on('keyup input', '.dialogInputText', _.bind(this.onNameChanged, this));
            this.$el.on('click', '.loginProfileAutofill', _.bind(this.onFacebookAutoFillClicked, this));
            // Hide placeholder text on focus
            client.Util.hidePlaceholderOnFocus(this.$el.find('.ximformcontrol'));
        };
        ProfileEditor.prototype.disableUserInput = function (value) {
            if (value) {
                this.$el.find('.loginIntro').addClass("loginDisabled");
                this.$el.find('.loginInputProfile').addClass("loginDisabled");
                this.$el.find('.loginProfileAutofill').addClass("loginDisabled");
                this.$el.find('.loginProfileWait').show();
            }
            else {
                this.$el.find('.loginIntro').removeClass("loginDisabled");
                this.$el.find('.loginInputProfile').removeClass("loginDisabled");
                this.$el.find('.loginProfileAutofill').removeClass("loginDisabled");
                this.$el.find('.loginProfileWait').hide();
            }
        };
        ProfileEditor.prototype.createProfile = function () {
            var _this = this;
            if (this.profile) {
                client.logger.collectNamedEvent(consts.flurryEvent.profileUpdate);
            }
            else {
                client.logger.collectNamedEvent(consts.flurryEvent.registrationProfileCreate);
            }
            var firstName = this.$el.find('input[name=firstname]').val().trim();
            var lastName = this.$el.find('input[name=lastname]').val().trim();
            var error = false;
            if (!firstName || firstName.length == 0) {
                this.$el.find("#firstName").text(resources.login.profile.error);
                error = true;
            }
            else if (!client.Util.isValidName(firstName)) {
                this.$el.find("#firstName").text(resources.login.profile.legal);
                error = true;
            }
            else {
                this.$el.find("#firstName").text("");
            }
            if (!lastName || lastName.length == 0) {
                this.$el.find("#lastName").text(resources.login.profile.error);
                error = true;
            }
            else if (!client.Util.isValidName(lastName)) {
                this.$el.find("#lastName").text(resources.login.profile.legal);
                error = true;
            }
            else {
                this.$el.find("#lastName").text("");
            }
            if (error) {
                return;
            }
            this.disableUserInput(true);
            if (this.thumbnail) {
                // get user thumbnail data (if it is from camera roll)
                async.waterfall([
                    function (c) {
                        if (_this.thumbnail.indexOf('/') == 0) {
                            if (window.imagetransfer) {
                                window.imagetransfer.bitmapfromuri('file://' + _this.thumbnail, 0, function (r) {
                                    if (_.isString(r)) {
                                        r = JSON.parse(r);
                                    }
                                    c(null, r.imageBase64);
                                }, function (e) {
                                    client.logger.debug('image transfer error', e);
                                    c(new Error(e || 'imagetransfer error'), null);
                                });
                            }
                        }
                        else {
                            _.defer(function () {
                                c(null, _this.thumbnail);
                            });
                        }
                    },
                    // set user profile thumbnailUrl (optionally upload the thumbnail binary data)
                    function (thumbnailUrl, c) {
                        if (thumbnailUrl.indexOf('data:') == 0) {
                            client.server.POST('api/profilePhoto', null, { photoUrl: thumbnailUrl }, function (e, data) {
                                if (e || data && data.error) {
                                    client.logger.error(consts.errorCode.Other, 'upload thumbnail error', e || data.error);
                                    c(e || new Error(data.error), null);
                                }
                                else {
                                    c(null, data.photoUrl);
                                }
                            });
                        }
                        else {
                            _.defer(function () {
                                c(null, thumbnailUrl);
                            });
                        }
                    },
                    // update user profile photo
                    function (url, c) {
                        client.Login.submitProfile(firstName, lastName, url, _this.color, c);
                    }
                ], this.onClose);
            }
            else {
                client.Login.submitProfile(firstName, lastName, null, this.color, this.onClose);
            }
        };
        ProfileEditor.prototype.setThumbnail = function (thumbnail) {
            this.thumbnail = thumbnail;
            if (this.thumbnail) {
                this.$el.find('.loginThumbnailImage').show();
                this.$el.find('.loginThumbnailInitial').hide();
                this.$el.find('.loginProfileFigure').attr('src', this.thumbnail);
                // user might have scrolled to pick the profile photo, snap it back
                window.scroll(0, 0);
            }
            else {
                var initials = this.getInitials();
                if (initials) {
                    this.$el.find('.loginThumbnailImage').hide();
                    this.$el.find('.loginThumbnailInitial').show();
                    this.$el.find('.loginProfileLabel').text(initials);
                }
                else {
                    this.$el.find('.loginThumbnailImage').show();
                    this.$el.find('.loginThumbnailInitial').hide();
                    this.$el.find('.loginProfileFigure').attr('src', ProfileEditor.defaultThumbnail);
                }
            }
        };
        // Update initials when name changes
        ProfileEditor.prototype.onNameChanged = function () {
            // Extract names
            var firstName = this.$el.find('input[name=firstname]').val();
            var lastName = this.$el.find('input[name=lastname]').val();
            // Clear error messages
            if (firstName && firstName.length > 0) {
                this.$el.find("#firstName").text("");
            }
            if (lastName && lastName.length > 0) {
                this.$el.find("#lastName").text("");
            }
            this.setThumbnail(this.thumbnail);
        };
        ProfileEditor.prototype.getInitials = function () {
            var firstName = this.$el.find('input[name=firstname]').val().trim();
            var lastName = this.$el.find('input[name=lastname]').val().trim();
            if (!firstName || firstName.length == 0 || !lastName || lastName.length == 0) {
                return null;
            }
            else {
                return (firstName[0].toUpperCase() + lastName[0].toUpperCase());
            }
        };
        ProfileEditor.prototype.onThumbnailClick = function () {
            // If no thumbnail imange and initial, go to camera roll
            if (!this.thumbnail && !this.getInitials()) {
                this.getThumbnail();
            }
            else {
                // Otherwise let user choose between image and initials
                this.showThumbnailPopUp(true);
            }
        };
        ProfileEditor.prototype.showThumbnailPopUp = function (value) {
            var thumbnailPopUp = new client.ThumbnailPopUp(this.getInitials(), this.thumbnail, this.color, _.bind(this.pickThumbType, this));
            thumbnailPopUp.show(true);
        };
        ProfileEditor.prototype.pickThumbType = function (thumbType) {
            if (thumbType == 0 /* PHOTO */) {
                client.logger.collect(consts.tags.ftux.photoFromCameraRoll);
                this.getThumbnail();
            }
            else {
                this.setThumbnail(null);
            }
        };
        ProfileEditor.prototype.onFacebookAutoFillClicked = function () {
            if (this.profile) {
                client.logger.collectNamedEvent(consts.flurryEvent.profileAutoFill);
            }
            else {
                client.logger.collectNamedEvent(consts.flurryEvent.registrationProfileAutoFill);
            }
            var $button = this.$el.find('.loginProfileAutofill');
            if ($button.length && !$button.hasClass('disabled')) {
                $button.toggleClass('disabled', true);
                this.onFacebookAutoFill();
            }
        };
        ProfileEditor.prototype.onFacebookAutoFillDone = function () {
            var $button = this.$el.find('.loginProfileAutofill');
            if ($button.length) {
                $button.toggleClass('disabled', false);
            }
        };
        ProfileEditor.prototype.onFacebookAutoFill = function () {
            var _this = this;
            if (window.userprofile) {
                client.logger.collect(consts.tags.ftux.facebookAutoFill);
                window.userprofile.getUserProfiles(function (profileJson) {
                    _this.onFacebookAutoFillDone();
                    if (!profileJson) {
                        client.user.SetSetting(13 /* WAITING_FACEBOOK */, true);
                        return;
                    }
                    var fbProfile = client.user.handleDeviceFacebookProfile(profileJson);
                    if (fbProfile && fbProfile.name) {
                        var names = fbProfile.name.split(' ');
                        names = _.filter(names, function (name) {
                            return !!name.trim();
                        });
                        var firstName = fbProfile.firstName || names[0];
                        var lastName = fbProfile.lastName || _.last(names);
                        _this.$el.find('input[name=firstname]').val(firstName);
                        _this.$el.find('input[name=lastname]').val(lastName);
                        _this.thumbnail = fbProfile.thumbnailUrl;
                        _this.onNameChanged();
                    }
                }, function (err) {
                    _this.onFacebookAutoFillDone();
                    client.logger.debug('fb auto fill failed', err);
                });
            }
        };
        ProfileEditor.defaultThumbnail = './img/personBubble.png';
        return ProfileEditor;
    })();
    client.ProfileEditor = ProfileEditor;
})(client || (client = {}));
/// <reference path="./AppMasterScreen.ts"/>
/// <reference path='./DialogScreen.ts'/>
/// <reference path='../controls/ThumbnailPopUp.ts'/>
/// <reference path='../controls/ProfileEditor.ts'/>
var client;
(function (client) {
    /** Login Dialog: Set up profile */
    var LoginProfileScreen = (function (_super) {
        __extends(LoginProfileScreen, _super);
        function LoginProfileScreen() {
            _super.apply(this, arguments);
            this.profileEditor = null;
        }
        LoginProfileScreen.prototype.navigate = function (route) {
            // Record that new profile setup has been shown
            client.logger.collectNamedEvent(consts.flurryEvent.registrationProfileDisplay);
            this.render();
            return _super.prototype.navigate.call(this, route);
        };
        LoginProfileScreen.prototype.render = function () {
            var _this = this;
            // Make screen is in a valid state
            if (this.getScreenState() == 3 /* DISPOSED */) {
                client.logger.warn("Render called on DISPOSED screen ");
                return this;
            }
            if (!this.profileEditor) {
                this.profileEditor = new client.ProfileEditor(this.$el.addClass('splash'), _.bind(this.getThumbnail, this), _.bind(this.onClose, this));
            }
            this.profileEditor.render();
            this.bindOnNavigate('', function () {
                _this.profileEditor.initEvents();
                // Exit app on back as can't reverse in login step
                document.addEventListener("backbutton", function (e) {
                    // only exit the app if we are not hidden 
                    // (this screen will be hidden if we are showing profile picture picker)
                    if (_this.$el.css('display') != 'none') {
                        if (phone.app && phone.app.exitApp) {
                            client.exitApp();
                        }
                    }
                }, false);
            });
            // Add ability to choose server response, view logs
            client.Login.EnableDebugFooter(this.$el.find('.loginPhone'));
            // no more back from this point on
            this.$el.find('#back').toggleClass('hidden', true);
            return this;
        };
        LoginProfileScreen.prototype.getThumbnail = function () {
            // Goto next screen (camera roll picker)
            client.wizard.currentWizard.currentScreen().history.close();
        };
        LoginProfileScreen.prototype.onClose = function () {
            // Close wizard which triggers new login step
            client.wizard.currentWizard.finalize({});
        };
        // Overwrite superclass to retreive thumbnail from wizard
        LoginProfileScreen.prototype.show = function (transition) {
            if (client.wizard.currentWizard.bag['selected'] && client.wizard.currentWizard.bag['selected'].length > 0) {
                this.profileEditor.setThumbnail(client.wizard.currentWizard.bag['selected'][0].source);
            }
            return _super.prototype.show.call(this, transition);
        };
        return LoginProfileScreen;
    })(client.DialogScreen);
    client.LoginProfileScreen = LoginProfileScreen;
})(client || (client = {}));
/// <reference path="../../../types/jquery/jquery.d.ts" />
/// <reference path="../types.ts"/>
/// <reference path="../ScreenCast.ts"/>
var client;
(function (client) {
    (function (ImageType) {
        ImageType[ImageType["LOADED"] = 0] = "LOADED";
        ImageType[ImageType["ERROR"] = 1] = "ERROR";
        ImageType[ImageType["RETRY"] = 2] = "RETRY";
    })(client.ImageType || (client.ImageType = {}));
    var ImageType = client.ImageType;
    ;
    (function (AnimationType) {
        AnimationType[AnimationType["INSTANT"] = 0] = "INSTANT";
        AnimationType[AnimationType["GRADUAL"] = 1] = "GRADUAL";
    })(client.AnimationType || (client.AnimationType = {}));
    var AnimationType = client.AnimationType;
    ;
    (function (SliderEvent) {
        SliderEvent[SliderEvent["SWIPE"] = 0] = "SWIPE";
        SliderEvent[SliderEvent["ZOOM"] = 1] = "ZOOM";
    })(client.SliderEvent || (client.SliderEvent = {}));
    var SliderEvent = client.SliderEvent;
    ;
    var VisibleType;
    (function (VisibleType) {
        VisibleType[VisibleType["VISIBLE"] = 0] = "VISIBLE";
        VisibleType[VisibleType["HIDDEN"] = 1] = "HIDDEN";
    })(VisibleType || (VisibleType = {}));
    ;
    var MoveType;
    (function (MoveType) {
        MoveType[MoveType["PREVIOUS"] = 0] = "PREVIOUS";
        MoveType[MoveType["NEXT"] = 1] = "NEXT";
        MoveType[MoveType["RETURN"] = 2] = "RETURN";
        MoveType[MoveType["INIT"] = 3] = "INIT";
        MoveType[MoveType["NONE"] = 4] = "NONE";
    })(MoveType || (MoveType = {}));
    ;
    var TouchSlider = (function () {
        function TouchSlider($el, $slides, options) {
            if (options === void 0) { options = {}; }
            this.$el = $el;
            this.$slides = $slides;
            // Current display state
            this.currentIndex = 0;
            this.currentScale = 1;
            this.currentXOffset = 0;
            this.currentYOffset = 0;
            this.animating = false;
            this.animateTimeout = null;
            this.lastScaleFactor = 1;
            this.previousScaleFactor = 1;
            this.cssOrigin = '';
            this.prevWindowSize = { innerWidth: window.innerWidth, innerHeight: window.innerHeight };
            // Pan vars
            this.panAndZoomMode = false;
            // Drag vars
            this.dragStartX = 0;
            this.dragStartY = 0;
            this.dragComplete = false;
            this.dragInProgress = false;
            this.hammerOptions = {
                prevent_default: true,
                scale_threshold: 0,
                drag_min_distance: 5,
                hold_timeout: 600
            };
            this.opts = $.extend({}, client.DefaultTouchSliderOptions, options);
            this.hammerOptions.prevent_default = this.opts.preventDefault;
            this.initOffsets();
            // Calculate initial position for progress image
            // NOTE: relies on progress image being 240x160
            var x = (this.screenWidth - 240) / 2;
            var y = (this.screenHeight - 160) / 2;
            var style = ''; //"style='margin-left:"+ Math.round(x) + "px; margin-top:"+ Math.round(y) + "px;'";
            // Create slide elements
            $slides.toggleClass('touch-slider-hidden', true).appendTo(this.$el);
            this.$el.addClass('touch-slider');
            this.handleEvents();
            this.$el.hammer(this.hammerOptions);
        }
        // Returns number of images
        TouchSlider.prototype.getImageCount = function () {
            return this.$slides.length;
        };
        // Returns current index
        TouchSlider.prototype.getCurrentIndex = function () {
            return this.currentIndex;
        };
        TouchSlider.prototype.appendSlide = function ($slide, skipLayoyt) {
            $slide.toggleClass('touch-slider-hidden').attr('draggable', 'false').appendTo(this.$el);
            this.$slides = this.$slides.add($slide);
            if (!skipLayoyt) {
                this.relayoutImages(0 /* INSTANT */);
            }
        };
        TouchSlider.prototype.bind = function (event, handler) {
            this.$el.bind.apply(this.$el, arguments);
        };
        // Initialize first slide shown (may be joining the cast late so could be zoomed and not first slide)
        TouchSlider.prototype.initStartSlide = function (properties, loaded, waiter) {
            var _this = this;
            // Get properties
            this.currentIndex = this.indexToSpanPosition(properties.queueIndex || 0);
            this.loadNearbyImages(this.currentIndex);
            this.currentScale = properties.imgScale || this.opts.minScale;
            this.currentXOffset = properties.imgXOffset || 0;
            this.currentYOffset = properties.imgYOffset || 0;
            // Get current/previous/next index.
            var pIndex = this.prevIndex(this.currentIndex);
            var nIndex = this.nextIndex(this.currentIndex);
            // Get current/previous/next images
            this.$currentItem = this.getSlide(this.currentIndex);
            this.$previousItem = this.getSlide(pIndex);
            this.$nextItem = this.getSlide(nIndex);
            // Place initial items after they have all loaded
            var $startImages = $().add(this.$currentItem).add(this.$previousItem).add(this.$previousItem);
            waiter($startImages, function () {
                var offset = _this.offsetFromSpanPosition();
                _.each($startImages, function (img, i) {
                    loaded($(img));
                });
                _this.translateSlide(_this.$currentItem, 0 /* INSTANT */, 0 /* VISIBLE */, _this.currentScale, _this.currentXOffset + offset.x, _this.currentYOffset + offset.y);
                _this.translateSlide(_this.$previousItem, 0 /* INSTANT */, 0 /* VISIBLE */, _this.opts.minScale, -_this.imageOffset, 0);
                _this.translateSlide(_this.$nextItem, 0 /* INSTANT */, 0 /* VISIBLE */, _this.opts.minScale, _this.imageOffset, 0);
                // Note the first slide has happened
                _this.$el.trigger(TouchSlider.SlideChangedEvent, null);
            });
        };
        TouchSlider.prototype.relayoutImages = function (animationType) {
            this.prevWindowSize = { innerWidth: window.innerWidth, innerHeight: window.innerHeight };
            // Recalculate offset from screen size
            this.initOffsets();
            // Reload nearby images to trigger rescale
            this.loadNearbyImages(this.currentIndex);
            // Arrange show items
            this.arrangeImages(animationType, 3 /* INIT */, this.currentIndex);
        };
        TouchSlider.prototype.refreshFullScreenSize = function () {
            var windowSize = { width: window.innerWidth, height: window.innerHeight };
            var viewportSize = client.webview.scaleToViewport(windowSize);
            this.screenWidth = viewportSize.width;
            this.screenHeight = viewportSize.height;
        };
        // This method must be called any time the size of touchslider element changes. 
        TouchSlider.prototype.relayout = function (animationType) {
            var _this = this;
            if (animationType === void 0) { animationType = 1 /* GRADUAL */; }
            // Transform out the current images
            this.translateSlide(this.$previousItem, animationType, 0 /* VISIBLE */, 0, -this.imageOffset, 0);
            this.translateSlide(this.$nextItem, animationType, 0 /* VISIBLE */, 0, this.imageOffset, 0);
            this.translateSlide(this.$currentItem, animationType, 0 /* VISIBLE */, 0, 0, 0);
            // Mark all images as needing to be rescaled (done on demand)
            this.$slides.attr('data-rescale', 'true');
            var delayedAnimationType = animationType;
            if (!client.capability.isJellybean && (this.prevWindowSize.innerWidth != window.innerWidth || this.prevWindowSize.innerHeight != window.innerHeight)) {
                this.relayoutImages(delayedAnimationType);
            }
            else {
                // Delay as rotation event occurs before new screen size is available on some platforms
                window.setTimeout(function () {
                    _this.relayoutImages(delayedAnimationType);
                }, 300);
            }
        };
        // Jumps to a specific slide. 
        TouchSlider.prototype.moveTo = function (slideIndex, scale, xOffset, yOffset) {
            // If screens aren't linked, ignore
            if (!this.opts.isLinked) {
                return;
            }
            // Check that index is valid
            if ((typeof (slideIndex) != "number") || slideIndex < 0 || slideIndex >= this.$slides.length) {
                return;
            }
            // If not zooming
            if (scale == this.opts.minScale) {
                this.panAndZoomMode = false;
                // Convert index based on span position
                slideIndex = this.indexToSpanPosition(slideIndex);
                var lastIndex = this.$slides.length - 1;
                // Animate to new slide if needed
                if (this.currentIndex != slideIndex) {
                    var moveType;
                    if (slideIndex == 0 && this.currentIndex == lastIndex) {
                        moveType = 1 /* NEXT */;
                    }
                    else if (slideIndex == lastIndex && this.currentIndex == 0) {
                        moveType = 0 /* PREVIOUS */;
                    }
                    else if (this.currentIndex > slideIndex) {
                        moveType = 0 /* PREVIOUS */;
                    }
                    else {
                        moveType = 1 /* NEXT */;
                    }
                    this.loadNearbyImages(slideIndex);
                    // Provide prev & next as move may be greater than one slide 
                    this.arrangeImages(1 /* GRADUAL */, moveType, slideIndex);
                }
                else if (this.currentScale != this.opts.minScale) {
                    this.currentScale = this.opts.minScale;
                    this.previousScaleFactor = this.opts.minScale;
                    this.translateSlide(this.$currentItem, 1 /* GRADUAL */, 0 /* VISIBLE */, this.currentScale, 0, 0);
                }
            }
            else {
                if (this.currentIndex != slideIndex) {
                    var newIndex = slideIndex;
                    this.loadNearbyImages(newIndex);
                    this.arrangeImages(0 /* INSTANT */, 3 /* INIT */, newIndex);
                }
                this.panAndZoomMode = true;
                // Animate current slide to given scale & offset
                var offset = this.offsetFromSpanPosition();
                this.currentScale = scale;
                this.currentXOffset = xOffset + offset.x;
                this.currentYOffset = yOffset + offset.y;
                this.previousScaleFactor = scale;
                this.translateSlide(this.$currentItem, 1 /* GRADUAL */, 0 /* VISIBLE */, this.currentScale, this.currentXOffset, this.currentYOffset);
            }
        };
        //--------------------------------------------------------
        // PRIVATE 
        //--------------------------------------------------------
        TouchSlider.prototype.loadNearbyImages = function (index) {
            // do nothing
        };
        // Initialize screen width/height and offset
        TouchSlider.prototype.initOffsets = function () {
            var androidVersion = client.capability.androidVersion();
            if (this.opts.fullScreen && (androidVersion >= 4.4 || client.startupMode == 4 /* ios */)) {
                this.refreshFullScreenSize();
            }
            else {
                this.screenWidth = this.$el.width();
                this.screenHeight = this.$el.height();
                if (!this.screenWidth || !this.screenHeight) {
                    var vp = client.webview.currentViewport();
                    this.screenWidth = this.screenWidth || vp.width;
                    this.screenHeight = this.screenHeight || vp.height;
                }
                if (client.capability.isJellybean) {
                    // we may got phsycial resolution here
                    // make sure we use the the viewport resolution
                    var viewportSize = client.webview.scaleToViewport({ width: this.screenWidth, height: this.screenHeight });
                    this.screenWidth = viewportSize.width;
                    this.screenHeight = viewportSize.height;
                }
            }
            this.imageOffset = this.screenWidth + this.opts.gapBetweenImages;
        };
        // Set up event handling
        TouchSlider.prototype.handleEvents = function () {
            this.$el.bind('dragstart', $.proxy(this.onDragStart, this));
            this.$el.bind('drag', $.proxy(this.onDrag, this));
            this.$el.bind('dragend', $.proxy(this.onDragEnd, this));
            this.$el.bind('transformstart', $.proxy(this.onTransformStart, this));
            this.$el.bind('transform', $.proxy(this.onTransform, this));
            this.$el.bind('transformend', $.proxy(this.onTransformEnd, this));
            this.$el.bind('tap', $.proxy(this.onTap, this));
            this.$el.bind('transitionend webkitTransitionEnd oTransitionEnd MSTransitionEnd', { _this: this }, this.translationDone);
            this.$el.bind('hold', $.proxy(this.onHold, this));
        };
        // Is slider allowed to loop
        TouchSlider.prototype.canLoop = function () {
            // Disable as we are getting rid of looping in next version
            return false;
            // At least 4 images are needed to enable loop mode.
            //return this.opts.loop && this.$slides.length >= 4;
        };
        TouchSlider.prototype.canDoNextSlide = function () {
            if (this.currentIndex < this.$slides.length - 1)
                return true;
            if (this.canLoop())
                return true;
            return false;
        };
        TouchSlider.prototype.canDoPreviousSlide = function () {
            if (this.currentIndex > 0)
                return true;
            if (this.canLoop())
                return true;
            return false;
        };
        // Move to next slide
        TouchSlider.prototype.nextSlide = function () {
            if (!this.canDoNextSlide()) {
                return;
            }
            var newIndex = (this.currentIndex + 1) % this.$slides.length;
            this.arrangeImages(1 /* GRADUAL */, 1 /* NEXT */, newIndex);
            return newIndex;
        };
        // Move to previous slide
        TouchSlider.prototype.previousSlide = function () {
            if (!this.canDoPreviousSlide()) {
                return;
            }
            var newIndex = this.currentIndex - 1;
            if (newIndex < 0) {
                newIndex = this.$slides.length - 1;
            }
            this.arrangeImages(1 /* GRADUAL */, 0 /* PREVIOUS */, newIndex);
            return newIndex;
        };
        // Converts index accounting for span position
        TouchSlider.prototype.indexToSpanPosition = function (index) {
            // If in pan and zoom all screens share the same index
            if (this.panAndZoomMode) {
                return index;
            }
            switch (this.opts.spanPosition) {
                case consts.SpanPositions.left:
                case consts.SpanPositions.topLeft:
                case consts.SpanPositions.bottomLeft:
                    return this.prevIndex(index);
                case consts.SpanPositions.right:
                case consts.SpanPositions.topRight:
                case consts.SpanPositions.bottomRight:
                    return this.nextIndex(index);
                default:
                    return index;
            }
        };
        // Determines x/y offset from span position
        TouchSlider.prototype.offsetFromSpanPosition = function () {
            // If not in pan and zoom mode all screens are on same offset
            if (!this.panAndZoomMode) {
                return { x: 0, y: 0 };
            }
            switch (this.opts.spanPosition) {
                case consts.SpanPositions.left:
                    return { x: this.screenWidth, y: 0 };
                case consts.SpanPositions.topLeft:
                    return { x: this.screenWidth, y: this.screenHeight };
                case consts.SpanPositions.bottomLeft:
                    return { x: this.screenWidth, y: -this.screenHeight };
                case consts.SpanPositions.right:
                    return { x: -this.screenWidth, y: 0 };
                case consts.SpanPositions.topRight:
                    return { x: -this.screenWidth, y: this.screenHeight };
                case consts.SpanPositions.bottomRight:
                    return { x: -this.screenWidth, y: -this.screenHeight };
                case consts.SpanPositions.top:
                    return { x: 0, y: this.screenHeight };
                case consts.SpanPositions.bottom:
                    return { x: 0, y: -this.screenHeight };
                case consts.SpanPositions.center:
                default:
                    return { x: 0, y: 0 };
            }
        };
        // Converts index back to base using span position
        TouchSlider.prototype.indexFromSpanPosition = function (index) {
            // If in pan and zoom mode all screens are on same image
            if (this.panAndZoomMode) {
                return index;
            }
            switch (this.opts.spanPosition) {
                case consts.SpanPositions.left:
                case consts.SpanPositions.topLeft:
                case consts.SpanPositions.bottomLeft:
                    return this.nextIndex(index);
                case consts.SpanPositions.right:
                case consts.SpanPositions.topRight:
                case consts.SpanPositions.bottomRight:
                    return this.prevIndex(index);
                default:
                    return index;
            }
        };
        TouchSlider.prototype.resetTransform = function () {
            this.previousScaleFactor = 1;
            this.cssOrigin = '';
        };
        TouchSlider.prototype.skipIndex = function (index, skip) {
            var result = index + skip;
            var count = this.$slides.length;
            if (result < 0 || result >= count) {
                if (this.canLoop()) {
                    result = result % count;
                    if (result < 0) {
                        result = count + result;
                    }
                }
            }
            return result;
        };
        TouchSlider.prototype.nextIndex = function (index) {
            return this.skipIndex(index, 1);
        };
        TouchSlider.prototype.prevIndex = function (index) {
            return this.skipIndex(index, -1);
        };
        // Returns image by index or null if index is invalid
        TouchSlider.prototype.getSlide = function (index) {
            return (index < 0 || index >= this.$slides.length) ? null : this.$slides.eq(index);
        };
        // Arranges current/prev/next images for display
        TouchSlider.prototype.arrangeImages = function (animationType, moveType, newIndex) {
            this.currentXOffset = 0;
            this.currentYOffset = 0;
            this.currentScale = this.opts.minScale;
            // Note which item is moving out of the display
            // May not be prev or next if moving more than one slide
            var $exitItem = this.getSlide(this.currentIndex);
            // Update current index
            this.currentIndex = newIndex;
            // Get current/previous/next 
            var pIndex = this.prevIndex(this.currentIndex);
            var nIndex = this.nextIndex(this.currentIndex);
            // Get current/previous/next images
            this.$currentItem = this.getSlide(this.currentIndex);
            this.$previousItem = this.getSlide(pIndex);
            this.$nextItem = this.getSlide(nIndex);
            // Cue up next tier
            var ppIndex = this.prevIndex(pIndex);
            var nnIndex = this.nextIndex(nIndex);
            var $ppreviousItem = this.getSlide(ppIndex);
            var $nnextItem = this.getSlide(nnIndex);
            // Show current image
            this.$currentItem.removeClass('touch-slider-hidden');
            // Animate items
            if (moveType == 1 /* NEXT */) {
                this.translateSlide($exitItem, 1 /* GRADUAL */, 0 /* VISIBLE */, this.opts.minScale, -this.imageOffset, 0);
                this.translateSlide(this.$nextItem, 0 /* INSTANT */, 1 /* HIDDEN */, this.opts.minScale, this.imageOffset, 0);
                // Account for jumps of move than one slide
                if ($exitItem && this.$previousItem && $exitItem[0] != this.$previousItem[0]) {
                    this.translateSlide(this.$previousItem, 0 /* INSTANT */, 1 /* HIDDEN */, this.opts.minScale, -this.imageOffset, 0);
                }
                this.translateSlide($nnextItem, 0 /* INSTANT */, 1 /* HIDDEN */, this.opts.minScale, this.imageOffset, 0);
                this.translateSlide($ppreviousItem, 0 /* INSTANT */, 1 /* HIDDEN */, this.opts.minScale, -this.imageOffset, 0);
            }
            else if (moveType == 0 /* PREVIOUS */) {
                this.translateSlide($exitItem, 1 /* GRADUAL */, 0 /* VISIBLE */, this.opts.minScale, this.imageOffset, 0);
                this.translateSlide(this.$previousItem, 0 /* INSTANT */, 1 /* HIDDEN */, this.opts.minScale, -this.imageOffset, 0);
                // Account for jumps of move than one slide
                if ($exitItem && this.$nextItem && $exitItem[0] != this.$nextItem[0]) {
                    this.translateSlide(this.$nextItem, 0 /* INSTANT */, 1 /* HIDDEN */, this.opts.minScale, this.imageOffset, 0);
                }
                this.translateSlide($nnextItem, 0 /* INSTANT */, 1 /* HIDDEN */, this.opts.minScale, this.imageOffset, 0);
                this.translateSlide($ppreviousItem, 0 /* INSTANT */, 1 /* HIDDEN */, this.opts.minScale, -this.imageOffset, 0);
            }
            else if (moveType == 2 /* RETURN */) {
                this.translateSlide(this.$previousItem, 1 /* GRADUAL */, 0 /* VISIBLE */, this.opts.minScale, -this.imageOffset, 0);
                this.translateSlide(this.$nextItem, 1 /* GRADUAL */, 0 /* VISIBLE */, this.opts.minScale, this.imageOffset, 0);
            }
            else if (moveType == 3 /* INIT */) {
                this.translateSlide(this.$previousItem, 0 /* INSTANT */, 0 /* VISIBLE */, this.opts.minScale, -this.imageOffset, 0);
                this.translateSlide(this.$nextItem, 0 /* INSTANT */, 0 /* VISIBLE */, this.opts.minScale, this.imageOffset, 0);
                this.translateSlide($nnextItem, 0 /* INSTANT */, 1 /* HIDDEN */, this.opts.minScale, this.imageOffset, 0);
                this.translateSlide($ppreviousItem, 0 /* INSTANT */, 1 /* HIDDEN */, this.opts.minScale, -this.imageOffset, 0);
            }
            this.translateSlide(this.$currentItem, animationType, 0 /* VISIBLE */, this.opts.minScale, 0, 0);
        };
        // Used to hide image at end of transition
        TouchSlider.prototype.translationDone = function (event) {
            // Hide image at end of transition unless it is the displayed image
            if (event.data._this.$currentItem[0] != $(event.target)[0]) {
                $(event.target).addClass('touch-slider-hidden');
            }
        };
        // Translate a slide
        TouchSlider.prototype.translateSlide = function (slide, animationType, visibleType, scale, xOffset, yOffset) {
            var _this = this;
            if (!slide) {
                return;
            }
            if (slide.length) {
                // Don't allow scale transforms for non-loaded images
                if (slide.data('state') != 0 /* LOADED */) {
                    scale = this.opts.minScale;
                }
                // Terminate any existing animation
                if (this.animateTimeout) {
                    window.clearTimeout(this.animateTimeout);
                    slide.removeClass('touch-slider-duration');
                }
                // Set visibility
                if (visibleType == 0 /* VISIBLE */) {
                    slide.removeClass('touch-slider-hidden');
                }
                else {
                    slide.addClass('touch-slider-hidden');
                }
                // Set duration
                if (animationType == 1 /* GRADUAL */) {
                    slide.addClass('touch-slider-duration');
                    this.animating = true;
                    this.animateTimeout = window.setTimeout(function () {
                        _this.animating = false;
                        slide.removeClass('touch-slider-duration');
                    }, 300); // @TOUCH_SLIDER_DURATION in CSS
                }
                // Set Transform visibleType
                var transX = Math.round(xOffset / scale);
                var transY = Math.round(yOffset / scale);
                var translate = 'scale(' + scale + ') translate(' + transX + 'px,' + transY + 'px)';
                slide.css({
                    '-ms-transform': translate,
                    '-webkit-transform': translate,
                    'transform': translate
                });
            }
        };
        // Called at start of drag event
        TouchSlider.prototype.onDragStart = function (e) {
            this.referenceDistanceX = 0;
            this.dragStartX = this.currentXOffset;
            this.dragStartY = this.currentYOffset;
            this.dragComplete = false;
        };
        // Max X Offset before image leaves sceen
        TouchSlider.prototype.maxXOffset = function () {
            var screenWidth = this.$el.width();
            var imageWidth = this.$currentItem[0].clientWidth;
            var maxWidth = ((imageWidth * this.currentScale) - screenWidth) / 2;
            return Math.abs(maxWidth);
        };
        // Max Y Offset before image leaves sceen
        TouchSlider.prototype.maxYOffset = function () {
            var screenHeight = this.$el.height();
            var imageHeight = this.$currentItem[0].clientHeight;
            var maxHeight = ((imageHeight * this.currentScale) - screenHeight) / 2;
            return Math.abs(maxHeight);
        };
        // Called at start of a transform event
        TouchSlider.prototype.onTransformStart = function (e) {
            if (this.dragInProgress) {
                return;
            }
            // If item isn't loaded, disallow pan and zoom
            if (this.$currentItem.data('state') != 0 /* LOADED */) {
                return;
            }
            this.panAndZoomMode = true;
            // The initial midpoint of the first two touches is our transform origin.
            var centerX = (e.touches[0].x + e.touches[1].x) / 2;
            var centerY = (e.touches[0].y + e.touches[1].y) / 2;
            var offs = this.$currentItem.offset();
            this.cssOrigin = ((centerX - offs.left) / this.currentScale) + "px " + ((centerY - offs.top) / this.currentScale) + "px";
        };
        // Called during a drag
        TouchSlider.prototype.onDrag = function (e) {
            if (this.animating) {
                this.referenceDistanceX = e.distanceX;
                return;
            }
            var distanceY = this.panAndZoomMode ? e.distanceY : 0;
            var distanceX = e.distanceX - this.referenceDistanceX;
            if (!this.panAndZoomMode) {
                // Max draggable distance should be limited based on whether previous/next is permitted.
                if (!this.canDoNextSlide()) {
                    distanceX = Math.max(distanceX, -this.opts.maxPlay);
                }
                if (!this.canDoPreviousSlide()) {
                    distanceX = Math.min(distanceX, this.opts.maxPlay);
                }
            }
            if (!this.dragComplete) {
                this.currentXOffset = this.dragStartX + distanceX;
                this.currentYOffset = this.dragStartY + distanceY;
                // If in pan and zoom don't let image slide off screen
                if (this.panAndZoomMode) {
                    this.constrainToScreen();
                }
                this.translateSlide(this.$currentItem, 0 /* INSTANT */, 0 /* VISIBLE */, this.currentScale, this.currentXOffset, this.currentYOffset);
                if (!this.panAndZoomMode) {
                    this.dragInProgress = true;
                    // Drag previous/next slides.
                    this.translateSlide(this.$previousItem, 0 /* INSTANT */, 0 /* VISIBLE */, this.opts.minScale, -this.imageOffset + distanceX, distanceY);
                    this.translateSlide(this.$nextItem, 0 /* INSTANT */, 0 /* VISIBLE */, this.opts.minScale, this.imageOffset + distanceX, distanceY);
                    // Trigger previous/next slide if appropriate.
                    var w = this.$el.width();
                    if (distanceX > w / TouchSlider.swipeDetectRatio) {
                        this.referenceDistanceX = e.distanceX;
                        this.dragComplete = this.canDoPreviousSlide();
                        this.previousSlide();
                        this.triggerChangedEvent(0 /* SWIPE */);
                    }
                    else if (-distanceX > w / TouchSlider.swipeDetectRatio) {
                        this.referenceDistanceX = e.distanceX;
                        this.dragComplete = this.canDoNextSlide();
                        this.nextSlide();
                        this.triggerChangedEvent(0 /* SWIPE */);
                    }
                }
            }
        };
        // Don't let image slide off screen
        TouchSlider.prototype.constrainToScreen = function () {
            var maxXOffset = this.maxXOffset();
            var maxYOffset = this.maxYOffset();
            this.currentXOffset = Math.min(this.currentXOffset, maxXOffset);
            this.currentXOffset = Math.max(this.currentXOffset, -maxXOffset);
            this.currentYOffset = Math.min(this.currentYOffset, maxYOffset);
            this.currentYOffset = Math.max(this.currentYOffset, -maxYOffset);
        };
        // Called at end of drag event
        TouchSlider.prototype.onDragEnd = function (e) {
            this.dragInProgress = false;
            if (this.animating) {
                return false;
            }
            if (this.panAndZoomMode) {
                this.triggerChangedEvent(1 /* ZOOM */);
            }
            else if (!this.dragComplete) {
                this.arrangeImages(1 /* GRADUAL */, 2 /* RETURN */, this.currentIndex);
            }
        };
        // Called during a transform event
        TouchSlider.prototype.onTransform = function (e) {
            if (!this.panAndZoomMode) {
                return;
            }
            this.currentScale = this.previousScaleFactor * e.scale;
            this.currentScale = Math.max(this.opts.minScale, Math.min(this.currentScale, this.opts.maxScale));
            // Adjust offset to keep on screen
            this.constrainToScreen();
            this.translateSlide(this.$currentItem, 0 /* INSTANT */, 0 /* VISIBLE */, this.currentScale, this.currentXOffset, this.currentYOffset);
        };
        /** user long pressed on the surface*/
        TouchSlider.prototype.onHold = function (e) {
            client.logger.info("long press detected at:", e.position);
            this.$el.trigger(TouchSlider.LongPressEvent, e.position);
        };
        // Exit pan and zoom mode
        TouchSlider.prototype.exitPanAndZoomMode = function () {
            if (this.panAndZoomMode) {
                this.panAndZoomMode = false;
                this.resetTransform();
                this.arrangeImages(1 /* GRADUAL */, 4 /* NONE */, this.currentIndex);
                this.triggerChangedEvent(1 /* ZOOM */);
            }
        };
        // Called on end of transform event
        TouchSlider.prototype.onTransformEnd = function (e) {
            if (this.dragInProgress) {
                this.arrangeImages(1 /* GRADUAL */, 4 /* NONE */, this.currentIndex);
                this.dragInProgress = false;
            }
            if (this.currentScale <= 1) {
                this.exitPanAndZoomMode();
            }
            this.previousScaleFactor = this.currentScale;
            this.triggerChangedEvent(1 /* ZOOM */);
        };
        // Used to trigger a change event for other displays
        TouchSlider.prototype.triggerChangedEvent = function (eventType) {
            // Only send back change if screens are linked
            if (this.opts.isLinked) {
                var state = this.getScreenState();
                state.event = eventType;
                this.$el.trigger(TouchSlider.SlideChangedEvent, state);
            }
            else {
                // Note the event happened but send no data
                this.$el.trigger(TouchSlider.SlideChangedEvent, { event: eventType });
            }
        };
        TouchSlider.prototype.getScreenState = function () {
            // Convert my index back to base index accouting for spanPosition
            var index = this.indexFromSpanPosition(this.currentIndex);
            var offset = this.offsetFromSpanPosition();
            var props = {
                queueIndex: index,
                imgScale: this.currentScale,
                imgXOffset: this.currentXOffset - offset.x,
                imgYOffset: this.currentYOffset - offset.y
            };
            return props;
        };
        // Called on a tap event
        TouchSlider.prototype.onTap = function (e) {
            this.exitPanAndZoomMode();
        };
        TouchSlider.swipeDetectRatio = 6; // What fraction of screen (1/x) to drag before swipe triggered
        // Events
        TouchSlider.SlideChangedEvent = 'slideChanged';
        TouchSlider.UnlinkEvent = 'userUnlinked';
        TouchSlider.LongPressEvent = 'longpress';
        return TouchSlider;
    })();
    client.TouchSlider = TouchSlider;
    client.DefaultTouchSliderOptions = {
        loop: true,
        gapBetweenImages: 20,
        minScale: 1,
        maxScale: 4,
        animEasing: 'swing',
        allowRotation: false,
        maxPlay: 50,
        preventDefault: true,
        isLinked: true,
        spanPosition: consts.SpanPositions.center,
        fullScreen: true
    };
})(client || (client = {}));
/// <reference path="../types.ts" />
/// <reference path="./TouchSlider.ts" />
var client;
(function (client) {
    var TouchFtuxSlider = (function (_super) {
        __extends(TouchFtuxSlider, _super);
        function TouchFtuxSlider($el, $slides, options) {
            if (options === void 0) { options = {}; }
            _super.call(this, $el, $slides, _.extend({
                loop: false,
                minScale: 1,
                maxScale: 1,
                fullScreen: true
            }, options));
            this.$slides = $slides;
            this.$pager = $('<div class="slider-pager"></div>').append(_.map(this.$slides, function () {
                return '<i></i>';
            }).join('')).appendTo($el);
            $(this.$pager.find('i')[0]).toggleClass('active', true);
            this.bind(client.TouchSlider.SlideChangedEvent, _.bind(this.onSlideChanged, this));
        }
        TouchFtuxSlider.prototype.onSlideChanged = function (ev, data) {
            if (data) {
                $(this.$pager.find('i').toggleClass('active', false)[data.queueIndex]).toggleClass('active', true);
            }
        };
        TouchFtuxSlider.prototype.onTap = function (e) {
            var oe = e.originalEvent;
            var handled = false;
            if (oe && oe.target) {
                var $target = $(oe.target);
                if ($target.hasClass('clickable')) {
                    handled = true;
                    $target.click();
                }
            }
            if (!handled) {
                _super.prototype.onTap.call(this, e);
            }
        };
        return TouchFtuxSlider;
    })(client.TouchSlider);
    client.TouchFtuxSlider = TouchFtuxSlider;
})(client || (client = {}));
/// <reference path="Screen.ts"/>
/// <reference path='../controls/TouchFtuxSlider.ts' />
/// <reference path='./LoadingScreen.ts'/>
var client;
(function (client) {
    /** Displays a loading message. */
    var FtuxScreen = (function (_super) {
        __extends(FtuxScreen, _super);
        function FtuxScreen() {
            _super.apply(this, arguments);
        }
        FtuxScreen.prototype.navigate = function (route) {
            // Record that ftux has been shown
            client.logger.collectNamedEvent(consts.flurryEvent.ftuxDisplay);
            // Redraw message
            if (route.params && route.params['showLoading']) {
                this.$el.toggleClass('splash', true).render('StartingCastScreen', { message: resources.messages.startingXim, isWp: client.startupMode == 3 /* wp */ });
            }
            else {
                this.render();
            }
            return _super.prototype.navigate.call(this, route);
        };
        FtuxScreen.prototype.onDispose = function () {
            client.user.SetSetting(11 /* SHOWN_FTUX */, true);
            _super.prototype.onDispose.call(this);
        };
        FtuxScreen.prototype.render = function () {
            var _this = this;
            // Make screen is in a valid state
            if (this.getScreenState() == 3 /* DISPOSED */) {
                client.logger.warn("Render called on DISPOSED screen ");
                return this;
            }
            // Render view
            this.empty();
            this.$el.addClass('splash').render('FtuxScreen', { isWp: client.startupMode == 3 /* wp */ });
            var $slides = this.$el.find('.ftux-slide');
            this.slider = new client.TouchFtuxSlider(this.$el.find('.ftux-slider'), $slides);
            $(this.slider.$slides[0]).toggleClass('touch-slider-hidden', false);
            this.slider.initStartSlide({}, function (slides) {
            }, function (slide) {
            });
            this.slider.moveTo(0, 1, 0, 0);
            this.bindOnNavigate('', function () {
                _this.$el.find('#login, #start').click(function () {
                    // close the current ftux sequence and go to login screen
                    client.logger.collectNamedEvent(consts.flurryEvent.ftuxLogin, { 'curIdx': _this.slider.getCurrentIndex() });
                    if (_this.history) {
                        _this.history.close(null, { cmd: consts.tabs.login });
                    }
                });
                _this.slider.bind(client.TouchSlider.SlideChangedEvent, function (e, data) {
                    // only show start/login affordance on the last slide
                    _this.$el.find('#login, #start').toggleClass('hidden', !(data && data.queueIndex == $slides.length - 1));
                });
            });
            this.syncNoPan();
            return _super.prototype.render.call(this);
        };
        /**
          * Displays a new loading screen as a popup.
          * @param message The loading message to display.
          * @param callback (optional) The callback to call once the screen has been dropped.
          */
        FtuxScreen.display = function (callback) {
            return client.screens.showModal(FtuxScreen, { style: 2 /* fullScreen */, params: { showLoading: true } }, function (e, results) {
                callback(e, results);
                return false;
            });
        };
        return FtuxScreen;
    })(client.Screen);
    client.FtuxScreen = FtuxScreen;
    client.screens.addRoute('/ftux', FtuxScreen);
})(client || (client = {}));
/// <reference path='./CameraRollScreen.ts'/>
var client;
(function (client) {
    /** Thumbnail selection screen. Allows seleciton of one photo*/
    var ThumbnailSelectionScreen = (function (_super) {
        __extends(ThumbnailSelectionScreen, _super);
        function ThumbnailSelectionScreen(sel, history, commands, commandCallback) {
            _super.call(this, sel, history, commands, commandCallback);
            this.history = history;
            this.commands = commands;
            this.commandCallback = commandCallback;
            this.defaultTitle = resources.labels.pickProfilePhoto;
        }
        // Overwrite superclass to limit to single image selected
        ThumbnailSelectionScreen.prototype.contentClicked = function ($tile) {
            // find item
            var itemId = $tile.attr('id');
            var item = _.find(this.items.models, function (item, i) {
                return item.id == itemId;
            });
            if (item) {
                this.bag['selected'] = [item.thumbnail];
                // If command callback use to notify thumbnail selection
                if (client.wizard.currentWizard.commandCallback) {
                    client.wizard.currentWizard.commandCallback(null, null);
                }
                history.back();
            }
        };
        // Overwrite superclass to never show "all" tile
        ThumbnailSelectionScreen.prototype.getTiles = function (items, appendOnly) {
            return _super.prototype.getTiles.call(this, items, true);
        };
        return ThumbnailSelectionScreen;
    })(client.CameraRollScreen);
    client.ThumbnailSelectionScreen = ThumbnailSelectionScreen;
    client.screens.addRoute('/thumbnailselection', ThumbnailSelectionScreen);
})(client || (client = {}));
/// <reference path='./LoginPhoneScreen.ts'/>
/// <reference path='./LoginChoiceScreen.ts'/>
/// <reference path='./LoginVerifyScreen.ts'/>
/// <reference path='./LoginProfileScreen.ts'/>
/// <reference path="./LogScreen.ts"/>
/// <reference path='./FtuxScreen.ts'/>
/// <reference path='./ThumbnailSelectionScreen.ts'/>
/// <reference path='../controls/ErrorPopUp.ts'/>
var client;
(function (client) {
    var LoginError = (function () {
        function LoginError() {
        }
        LoginError.INVALID_PHONE = new Error("Invalid Phone");
        LoginError.INVALID_PIN = new Error("Invalid Pin");
        LoginError.EXPIRED_PIN = new Error("Expired Pin");
        LoginError.HANDLED = new Error("Handled");
        return LoginError;
    })();
    client.LoginError = LoginError;
    ;
    // Manages
    var Login = (function () {
        function Login() {
        }
        /** Displays a people & content picking wizard to the user. */
        Login.start = function (callback) {
            Login.doneCallback = callback;
            Login.nextstep();
        };
        Login.nextstep = function () {
            this.debugServerOverride = 0 /* succeeded */;
            var authorizationState = client.user.getAuthorizationState();
            // Fail on too many retries 
            if (authorizationState != Login.lastState) {
                Login.lastState = authorizationState;
                Login.repeatCount = 0;
            }
            else {
                Login.repeatCount++;
                if (Login.repeatCount > 10) {
                    // Warn user, Unlink account and re-initiate authorization
                    var popup = new client.ErrorPopUp(client.ErrorMessages.SERVER_ERROR, Login.unlinkAndRetry);
                    popup.show(true);
                    return;
                }
            }
            var screens = [];
            client.logger.info('login state =', authorizationState);
            // if we want to show UI as part of login (sign-up, verify, profile editing, profile-photo picking)
            // we better make sure the loading screen is properlly trimmed visually (so it won't poke through z-layers)
            var loading = (client.screens.modal);
            var toggleLoadingScreen = function (on) {
                if (loading && loading.$el) {
                    // don't chain toggleClass class as it will cause the browser
                    // to re-render.
                    if (loading.$el.hasClass('loadingScreen')) {
                        loading.$el.toggleClass('loadingScreen', false);
                    }
                    loading.$el.toggleClass('backdrop', on);
                }
            };
            switch (authorizationState) {
                case 0 /* notLinked */:
                    // Clear any stored phone from local storage
                    localStorage.removeItem('login.phone');
                    screens.push(client.wizard.wrapScreen(client.FtuxScreen));
                    screens.push(client.wizard.wrapScreen(client.LoginPhoneScreen));
                    screens.push(client.wizard.wrapScreen(client.LoginChoiceScreen));
                    screens.push(client.wizard.wrapScreen(client.LoginVerifyScreen));
                    this.wizard = client.wizard.wizard(screens, function () {
                        toggleLoadingScreen(false);
                        _.defer(Login.nextstep);
                    });
                    this.wizard.commands = [];
                    this.wizard.commandCallback = function () {
                    };
                    toggleLoadingScreen(true);
                    this.wizard.begin();
                    break;
                case 1 /* notVerified */:
                    screens.push(client.wizard.wrapScreen(client.LoginPhoneScreen));
                    screens.push(client.wizard.wrapScreen(client.LoginChoiceScreen));
                    screens.push(client.wizard.wrapScreen(client.LoginVerifyScreen));
                    this.wizard = client.wizard.wizard(screens, function () {
                        toggleLoadingScreen(false);
                        _.defer(Login.nextstep);
                    });
                    this.wizard.commands = [];
                    this.wizard.commandCallback = function () {
                    };
                    toggleLoadingScreen(true);
                    this.wizard.begin();
                    // Jump back two screens to Verify screen, to allow back
                    client.wizard.currentWizard.currentScreen().history.close();
                    client.wizard.currentWizard.currentScreen().history.close();
                    break;
                case 2 /* notAuthorized */:
                    // Athorize, then jump back in to make sure user is registered
                    Login.authorize(Login.nextstep);
                    break;
                case 3 /* notRegistered */:
                    screens.push(client.wizard.wrapScreen(client.LoginProfileScreen));
                    screens.push(client.wizard.wrapScreen(client.ThumbnailSelectionScreen));
                    this.wizard = client.wizard.wizard(screens, function () {
                        toggleLoadingScreen(false);
                        _.defer(Login.nextstep);
                    });
                    this.wizard.commands = [];
                    this.wizard.commandCallback = function () {
                    };
                    toggleLoadingScreen(true);
                    this.wizard.begin();
                    break;
                case 4 /* notConnected */:
                    Login.connect(Login.nextstep);
                    break;
                case 5 /* Connected */:
                    if (Login.doneCallback) {
                        client.logger.info('login complete!');
                        _.defer(Login.doneCallback);
                    }
                    break;
            }
        };
        /** Set user's phone */
        Login.setPhone = function (phone) {
            Login.phone = phone;
            // Store in local storage in case user quits application before verify step
            localStorage.setItem('login.phone', Login.phone);
        };
        /** Set user's country - 2 letter alpha */
        Login.setCountryAlpha2 = function (countryAlpha2) {
            if (countryAlpha2.length != 2 || !client.Util.isAllLetters(countryAlpha2)) {
                client.logger.error(consts.errorCode.Initialization, "Bad country code");
                Login.countryAlpha2 = "US"; // Default back to us
                return;
            }
            Login.countryAlpha2 = countryAlpha2;
        };
        /** Return user's country - 2 letter alpha */
        Login.getCountryAlpha2 = function () {
            return Login.countryAlpha2;
        };
        /** Set user's verification type */
        Login.setVerificationType = function (verificationType) {
            Login.verificationType = verificationType;
        };
        /** Return phone number in display format */
        Login.getPhone = function () {
            // Check local storage for phone (happens if user quit application before verify step) 
            if (!Login.phone) {
                Login.phone = localStorage.getItem('login.phone');
            }
            if (Login.phone) {
                var phone = consts.parsePhoneNumberForCountry(Login.phone, Login.countryAlpha2);
                return consts.toString(phone);
            }
            else {
                return "";
            }
        };
        /** Sumbit phone number to server */
        Login.submitPhone = function (callback) {
            client.logger.collectNamedEvent(consts.flurryEvent.registrationPhoneNext, { 'valid': true });
            this.retryCallback = callback;
            this.retryData = Login.phone;
            if (Login.debugServerOverride != 0 /* succeeded */) {
                var error = new Error("TEST ERROR");
                var status = Login.debugServerOverride;
                Login.submitPhoneCallback(error, status);
            }
            else {
                var deviceLabel = window.device ? window.device.model : '';
                client.user.linkDevice(Login.phone, deviceLabel, Login.countryAlpha2, Login.verificationType, Login.submitPhoneCallback);
            }
        };
        Login.submitPhoneCallback = function (error, status) {
            client.logger.info('linkDevice err', error, 'status', status);
            if (!error) {
                Login.retryCallback(null, null);
            }
            else if (status == 1 /* badRequest */) {
                // Malformed number
                Login.retryCallback(LoginError.INVALID_PHONE, null);
            }
            else if (status == 3 /* forbidden */) {
                // 555 number not allowed on server
                Login.retryCallback(LoginError.INVALID_PHONE, null);
            }
            else if (status == 7 /* serverError */) {
                // Pop user back to phone screen after showing server error message
                Login.handleServerError(status, function () {
                    Login.retryCallback(LoginError.HANDLED);
                });
            }
            else {
                // Try the server call again after showing error message
                Login.handleServerError(status, function () {
                    Login.submitPhone(Login.retryCallback);
                });
            }
        };
        // Given phone token and pin, validate
        Login.validatePin = function (pin, callback) {
            if ($.isNumeric(pin)) {
                this.retryCallback = callback;
                this.retryData = pin;
                if (Login.debugServerOverride != 0 /* succeeded */) {
                    var error = new Error("TEST ERROR");
                    var status = Login.debugServerOverride;
                    Login.validatePinCallback(error, status);
                }
                else {
                    client.user.verifyDevice(pin, Login.validatePinCallback);
                }
            }
            else {
                callback(LoginError.INVALID_PIN, null);
            }
        };
        Login.validatePinCallback = function (error, status) {
            if (!error) {
                client.logger.collectNamedEvent(consts.flurryEvent.registrationVerificationNextResult, { 'result': 'success' });
                client.logger.collectNamedEvent(consts.flurryEvent.registrationVerificationSuccess);
                Login.retryCallback(null, null);
            }
            else if (status == 2 /* notAuthorized */) {
                // The user entered an invalid code and should try again
                client.logger.collectNamedEvent(consts.flurryEvent.registrationVerificationNextResult, { 'result': 'invalid pin' });
                Login.retryCallback(LoginError.INVALID_PIN, null);
            }
            else if (status == 6 /* tokenExpired */) {
                // The user didn't enter the code within the alloted time
                client.logger.collectNamedEvent(consts.flurryEvent.registrationVerificationNextResult, { 'result': 'expired pin' });
                Login.retryCallback(LoginError.EXPIRED_PIN, null);
            }
            else if (status == 3 /* forbidden */) {
                // The user has attempted to enter a code more than 5 times
                client.logger.collectNamedEvent(consts.flurryEvent.registrationVerificationNextResult, { 'result': '5 tries' });
                Login.retryCallback(LoginError.EXPIRED_PIN, null);
            }
            else {
                Login.handleServerError(status, function () {
                    Login.validatePin(Login.retryData, Login.retryCallback);
                });
            }
        };
        Login.authorize = function (callback) {
            client.user.authorizeDevice(function (error, status) {
                if (!error) {
                    callback(null, null);
                }
                else {
                    Login.handleServerError(status, function () {
                        Login.authorize(callback);
                    });
                }
            });
        };
        Login.connect = function (callback) {
            client.user.ensureConnected(function (error, status) {
                if (!error) {
                    client.logger.info('user.connectDevice succeeded', status);
                    callback(null, null);
                }
                else {
                    Login.handleServerError(status, function () {
                        Login.connect(callback);
                    });
                }
            });
        };
        Login.submitProfile = function (firstName, lastName, thumbnailUrl, color, callback) {
            this.retryCallback = callback;
            this.retryData = {
                firstName: firstName,
                lastName: lastName,
                thumbnailUrl: thumbnailUrl,
                color: color
            };
            if (Login.debugServerOverride != 0 /* succeeded */) {
                var error = new Error("TEST ERROR");
                var status = Login.debugServerOverride;
                Login.submitProfileCallback(error, status);
            }
            else {
                var changes = {
                    displayName: firstName + ' ' + lastName,
                    thumbnailUrl: thumbnailUrl,
                    color: color
                };
                client.user.updateProfile(changes, Login.submitProfileCallback);
            }
        };
        Login.submitProfileCallback = function (error, status) {
            if (!error) {
                if (Login.retryCallback) {
                    Login.retryCallback(null, null);
                }
            }
            else {
                Login.handleServerError(status, function () {
                    Login.submitProfile(Login.retryData.firstName, Login.retryData.lastName, Login.retryData.thumbnailUrl, Login.retryData.color, Login.retryCallback);
                });
            }
        };
        Login.unlinkAndRetry = function () {
            client.user.unlinkDevice(function () {
                // Close any open wizard
                if (client.wizard.currentWizard) {
                    client.wizard.currentWizard.finalize({});
                }
                Login.nextstep();
            });
        };
        Login.handleServerError = function (status, retryAction) {
            var popup;
            client.logger.debug('login server error', status);
            if (status == 1 /* badRequest */) {
                popup = new client.ErrorPopUp(client.ErrorMessages.BAD_REQUEST, retryAction);
                client.logger.info("Server: Bad Request");
            }
            else if (status == 2 /* notAuthorized */) {
                popup = new client.ErrorPopUp(client.ErrorMessages.NOT_AUTHORIZED, Login.unlinkAndRetry);
                client.logger.info("Server: Not Authorized");
            }
            else if (status == 3 /* forbidden */) {
                popup = new client.ErrorPopUp(client.ErrorMessages.FORBIDDEN, Login.unlinkAndRetry);
                client.logger.info("Server: Forbidden");
            }
            else if (status == 4 /* notFound */) {
                popup = new client.ErrorPopUp(client.ErrorMessages.USER_NOT_FOUND, Login.unlinkAndRetry);
                client.logger.info("Server: User Not Found");
            }
            else if (status == 5 /* upgradeRequired */) {
                popup = new client.ErrorPopUp(client.ErrorMessages.UPGRADE_REQUIRED, Login.upgrade);
                client.logger.info("Server: Upgrade Required");
            }
            else if (status == 6 /* tokenExpired */) {
                popup = new client.ErrorPopUp(client.ErrorMessages.TOKEN_EXPIRED, Login.unlinkAndRetry);
                client.logger.info("Server: Token Expired");
            }
            else if (status == 7 /* serverError */) {
                popup = new client.ErrorPopUp(client.ErrorMessages.SERVER_ERROR, retryAction);
                client.logger.info("Server: Server Error");
            }
            else if (status == 8 /* connectionError */) {
                popup = new client.ErrorPopUp(client.ErrorMessages.CONNECTION_ERROR, retryAction);
                client.logger.info("Server: Connection Error");
            }
            else if (status == 10 /* otherError */) {
                popup = new client.ErrorPopUp(client.ErrorMessages.CONNECTION_ERROR, retryAction);
                client.logger.info("Server: Other Error");
            }
            else if (status == 9 /* waitListed */) {
                popup = new client.ErrorPopUp(client.ErrorMessages.WAITLISTED, function () {
                });
                client.logger.info("Server: Waitlisted");
            }
            if (popup) {
                popup.show(true);
            }
            else {
                alert('status = ' + status);
            }
        };
        Login.upgrade = function () {
            window.open(consts.externalLinks.appstore, '_system');
            var popup = new client.ErrorPopUp(client.ErrorMessages.UPGRADE_REQUIRED, Login.upgrade).show(true);
        };
        // Add debug footer with version number and hidden menu
        Login.EnableDebugFooter = function ($el) {
            var _this = this;
            // Add application verison footer
            var $content = $("<label class='applicationVersion'>Version " + consts.applicationVersion + "</label>");
            $el.append($content);
            if (!consts.enableDebugFooter) {
                return;
            }
            // If already using footer on previous page, turn on on current page
            if (Login.debugFooterOn) {
                Login.TurnOnDebugFooter($el);
            }
            else {
                // Listen for events
                $el.on('click', '.applicationVersion', function () {
                    //-------------------------------------------------
                    // Enable debug footer if user clicks 7 times in a row
                    clearTimeout(_this.debugClickTimer);
                    _this.debugClickCount++;
                    _this.debugClickTimer = setTimeout(function () {
                        _this.debugClickCount = 0;
                    }, 1000);
                    if (_this.debugClickCount >= 7) {
                        _this.TurnOnDebugFooter($el);
                    }
                });
            }
        };
        Login.TurnOnDebugFooter = function ($el) {
            Login.debugFooterOn = true;
            this.AddServerDebug($el);
            $el.find('.applicationVersion').hide();
        };
        // When in emulation mode and option to override server response for testing
        Login.AddServerDebug = function ($el) {
            var _this = this;
            var $content = $("<div>").render('debugFooter', {});
            $el.after($content);
            var $selectedServer = $content.find('#serverOverride');
            $selectedServer.change(function () {
                _this.debugServerOverride = $selectedServer.val();
            });
            $content.on('click', '#debugButtonLogs', function () {
                // Clear callback and close wizard
                if (_this.wizard) {
                    _this.wizard.callback = function () {
                        client.app.navigate('/log');
                    };
                    client.wizard.currentWizard.finalize({});
                }
            });
        };
        Login.doneCallback = null;
        Login.wizard = null;
        Login.lastState = null;
        Login.repeatCount = 0;
        Login.retryCallback = null;
        Login.retryData = null;
        // User validation 
        Login.phone = null;
        Login.countryAlpha2 = null;
        Login.verificationType = null;
        // Used for debugging
        Login.debugFooterOn = false;
        Login.debugClickCount = 0;
        Login.debugServerOverride = 0 /* succeeded */;
        return Login;
    })();
    client.Login = Login;
})(client || (client = {}));
/// <reference path="./AppMasterScreen.ts"/>
/// <reference path="./Login.ts"/>
var client;
(function (client) {
    /** Terms screen. */
    var SettingsScreen = (function (_super) {
        __extends(SettingsScreen, _super);
        function SettingsScreen(selector) {
            _super.call(this, selector);
            this.selector = selector;
            this.tookRestartAction = false;
            this.altHeader = resources.labels.settings;
        }
        SettingsScreen.prototype.navigate = function (route) {
            // Update screens state
            client.logger.info('TermsScreen navigate start');
            // Redraw screen
            this.render();
            return _super.prototype.navigate.call(this, route);
        };
        SettingsScreen.prototype.render = function () {
            var _this = this;
            // Make screen is in a valid state
            if (this.getScreenState() == 3 /* DISPOSED */) {
                client.logger.warn("Render called on DISPOSED screen ");
                return this;
            }
            // Render view
            this.empty();
            // Don't show push notifications on iOS and handled in native UI
            var showPushSettings = (client.startupMode != 4 /* ios */);
            this.$el.render('settingsScreen', { version: consts.applicationVersion, title: resources.labels.settings, showPushSettings: showPushSettings });
            // Initialize push notification button state
            if (client.user.settings.disablePush) {
                this.$el.find('.settingsToggler').toggleClass('settingsOn settingsOff');
            }
            this.$el.on('click', '.unlinkButton', function () {
                client.logger.collectNamedEvent(consts.flurryEvent.settingsDeauthorize);
                _this.showUnlinkConfirmation();
            });
            this.$el.on('click', '.deleteAccountButton', function () {
                client.logger.collectNamedEvent(consts.flurryEvent.settingsDelete);
                _this.showDeleteConfirmation();
            });
            this.$el.on('click', '.notificationsButton', function () {
                _this.$el.find('.settingsToggler').toggleClass('settingsOn settingsOff');
                _this.toggleNotifications();
            });
            // As a replacement for capturing swipes on the toggle button (consider capturing swipes in the future)
            this.$el.on('click mousedown touchstart', '.settingsToggler', function (ev) {
                ev.stopPropagation();
                _this.$el.find('.settingsToggler').toggleClass('settingsOn settingsOff');
                _this.toggleNotifications();
            });
            this.syncNoPan();
            return this;
        };
        SettingsScreen.prototype.showUnlinkConfirmation = function () {
            var $popUpContent = this.$el.find('.confirmationPopUpContent');
            $popUpContent.empty();
            var confirmationPopUp = new client.ConfirmationPopUp($popUpContent, resources.messages.unlinkDevice.title, resources.messages.unlinkDevice.message, resources.messages.unlinkDevice.choiceLeft, resources.messages.unlinkDevice.choiceRight, _.bind(this.unlinkDevice, this));
        };
        SettingsScreen.prototype.unlinkDevice = function (choice) {
            var _this = this;
            if (choice == resources.messages.unlinkDevice.choiceRight) {
                client.logger.collectNamedEvent(consts.flurryEvent.settingsDeauthorizeConfirm);
                client.AppMasterScreen.checkConnection(function () {
                    client.user.unlinkDevice(_.bind(_this.actionCallback, _this));
                });
            }
        };
        SettingsScreen.prototype.showDeleteConfirmation = function () {
            var $popUpContent = this.$el.find('.confirmationPopUpContent');
            $popUpContent.empty();
            var confirmationPopUp = new client.ConfirmationPopUp($popUpContent, resources.messages.deleteAccount.title, resources.messages.deleteAccount.message, resources.messages.deleteAccount.choiceLeft, resources.messages.deleteAccount.choiceRight, _.bind(this.deleteAccount, this));
        };
        SettingsScreen.prototype.deleteAccount = function (choice) {
            var _this = this;
            if (choice == resources.messages.deleteAccount.choiceRight) {
                client.logger.collectNamedEvent(consts.flurryEvent.settingsDeleteConfirm);
                client.AppMasterScreen.checkConnection(function () {
                    client.user.deleteAccount(_.bind(_this.actionCallback, _this));
                });
            }
        };
        SettingsScreen.prototype.toggleNotifications = function () {
            var _this = this;
            client.AppMasterScreen.checkConnection(function () {
                client.user.updateSettings({ disablePush: !client.user.settings.disablePush }, function () {
                });
            }, 
            // Reverse UI setting upon failure
            function () {
                _this.$el.find('.settingsToggler').toggleClass('settingsOn settingsOff');
            });
        };
        SettingsScreen.prototype.actionCallback = function (error, status) {
            this.tookRestartAction = true;
            if (!error) {
                client.screens.master.showSideMenu(false);
                client.app.navigate('/');
                client.Login.start(null);
            }
            else {
                client.Login.handleServerError(status, function () {
                });
            }
        };
        SettingsScreen.prototype.onDispose = function () {
            // Show side menu unless user took action that restarts the application
            if (!this.tookRestartAction) {
                client.screens.master.showSideMenu(true);
            }
            _super.prototype.onDispose.call(this);
        };
        return SettingsScreen;
    })(client.DetailScreen);
    client.SettingsScreen = SettingsScreen;
})(client || (client = {}));
/// <reference path="./AppMasterScreen.ts"/>
/// <reference path="./Login.ts"/>
/// <reference path='../controls/ProfileEditor.ts'/>
var client;
(function (client) {
    /** Profile Edit Screen. */
    var ProfileEditScreen = (function (_super) {
        __extends(ProfileEditScreen, _super);
        function ProfileEditScreen(selector) {
            _super.call(this, selector);
            this.selector = selector;
            this.profileEditor = null;
            this.altHeader = "Profile";
        }
        ProfileEditScreen.prototype.navigate = function (route) {
            // Update screens state
            client.logger.info('ProfileEditScreen navigate start');
            // Redraw screen
            this.render();
            return _super.prototype.navigate.call(this, route);
        };
        ProfileEditScreen.prototype.render = function () {
            var _this = this;
            // Make screen is in a valid state
            if (this.getScreenState() == 3 /* DISPOSED */) {
                client.logger.warn("Render called on DISPOSED screen ");
                return this;
            }
            if (!this.profileEditor) {
                this.profileEditor = new client.ProfileEditor(this.$el, _.bind(this.getThumbnail, this), this.onClose, client.user.profile);
            }
            this.profileEditor.render();
            this.bindOnNavigate('', function () {
                _this.profileEditor.initEvents();
            });
            return this;
        };
        // Create wizard to get thumbnail
        ProfileEditScreen.prototype.getThumbnail = function () {
            var _this = this;
            var screens = [];
            screens.push(client.wizard.wrapScreen(client.ThumbnailSelectionScreen));
            var wizard = client.wizard.wizard(screens, function () {
            });
            wizard.commands = [];
            // Use command callback to capture selected image
            wizard.commandCallback = function () {
                if (client.wizard.currentWizard.bag['selected'] && client.wizard.currentWizard.bag['selected'].length > 0) {
                    _this.profileEditor.setThumbnail(client.wizard.currentWizard.bag['selected'][0].source);
                }
            };
            wizard.begin();
        };
        ProfileEditScreen.prototype.onClose = function () {
            client.app.navigate('/');
        };
        ProfileEditScreen.prototype.onDispose = function () {
            client.screens.master.showSideMenu(true);
            _super.prototype.onDispose.call(this);
        };
        return ProfileEditScreen;
    })(client.DetailScreen);
    client.ProfileEditScreen = ProfileEditScreen;
})(client || (client = {}));
/// <reference path="./AppMasterScreen.ts"/>
/// <reference path='./PickerScreen.ts'/>
/// <reference path='./SkydriveScreen.ts'/>
/// <reference path='./AlbumsScreen.ts'/>
/// <reference path='./PhotoCastScreen.ts'/>
/// <reference path='../controls/ErrorPopUp.ts'/>
/// <reference path="../controls/PeoplePicker.ts"/>
/// <reference path='../Group.ts'/>
/// <reference path='../ShareHandler.ts'/>
/// <reference path='casting.ts'/>
/// <reference path='../../../types/cordova/sharetarget.d.ts'/>
/// <reference path='../../../types/moment/moment.d.ts'/>
var client;
(function (client) {
    /** Timeline screen. */
    var TimelineScreen = (function (_super) {
        __extends(TimelineScreen, _super);
        function TimelineScreen() {
            _super.apply(this, arguments);
            this.groups = [];
            /** Number of groups rendered in timeline.  Maybe smaller than number of actual groups because of time limitation */
            this.renderedGroupCount = 0;
            this.closing = false;
            /** Note first time groups are rendered */
            this.firstGroupRender = true;
            this.firstRender = true;
            /** Are there any live Xims? */
            this.anyLiveGroups = false;
            /** Timer for updating expiration display */
            this.expirationTimer = null;
        }
        TimelineScreen.prototype.onCreated = function () {
            // Listen for events
            this._onGroupsUpdated = _.bind(this.onGroupsUpdated, this);
            client.user.onGroupsUpdated.add(this._onGroupsUpdated);
            this._onLogout = _.bind(this.onLogout, this);
            client.user.onLogout.add(this._onLogout);
            _super.prototype.onCreated.call(this);
        };
        TimelineScreen.prototype.onDispose = function () {
            this.closing = true;
            // Clear timers
            if (this.expirationTimer) {
                clearTimeout(this.expirationTimer);
            }
            // Disconnect from events and stop stuff
            client.user.onGroupsUpdated.remove(this._onGroupsUpdated);
            client.user.onLogout.remove(this._onLogout);
            _super.prototype.onDispose.call(this);
        };
        TimelineScreen.prototype.navigate = function (route) {
            // Redraw screen
            this.render();
            return _super.prototype.navigate.call(this, route);
        };
        /** Called when a command has been clicked. */
        TimelineScreen.prototype.onCommandClick = function (id) {
            switch (id) {
                case consts.commands.newgroup.id:
                    client.logger.collectNamedEvent(consts.flurryEvent.ximCreateNew, { timelinecount: this.renderedGroupCount });
                    client.AppMasterScreen.checkConnection(function () {
                        client.casting.prepareAndStartCast();
                    });
                    break;
                default:
                    client.logger.info("Unhandled menu click: " + id);
            }
        };
        /** Given a group id return matching group or null */
        TimelineScreen.prototype.groupById = function (id) {
            for (var i = 0; i < this.groups.length; i++) {
                if (this.groups[i].profile.id == id) {
                    return this.groups[i];
                }
            }
            return null;
        };
        /** A Post box had been clicked. */
        TimelineScreen.prototype.onBoxClick = function (event) {
            client.logger.collectNamedEvent(consts.flurryEvent.ximPreviousXim, { timelinecount: this.renderedGroupCount });
            var group = this.groupById(event.currentTarget.id);
            if (!group) {
                client.logger.error(consts.errorCode.Timeline, 'Could not find group with given ID');
                return;
            }
            // If user click on expired group, show message and update list
            if (group.minutesRemaining() == 0) {
                var popup = new client.ErrorPopUp(client.ErrorMessages.XIM_EXPIRED, function () {
                });
                popup.show(true);
                // Update timeline
                this.onGroupsUpdated(null);
                return;
            }
            client.AppMasterScreen.checkConnection(function () {
                client.PhotoCastScreen.joinCast(group, group.profile.castId, function () {
                    client.logger.info('Casting session finished');
                });
            });
        };
        TimelineScreen.prototype.onGroupsUpdated = function (data) {
            client.logger.info('onGroupUpdated called');
            client.user.listCachedGroups({ limit: TimelineScreen.groupLimit }, _.bind(this.onListCastGroups, this));
        };
        TimelineScreen.prototype.render = function () {
            var _this = this;
            // Make screen is in a valid state
            if (this.getScreenState() == 3 /* DISPOSED */) {
                client.logger.warn("Render called on DISPOSED screen ");
                return this;
            }
            client.logger.info('timeline screen render start');
            // Clear viewers cached xim
            client.user.currentXim = null;
            var clientWidth = window.innerWidth;
            var clientHeight = window.innerHeight;
            if (clientWidth > 0 && clientHeight > 0 && clientWidth > clientHeight) {
                // we are in a trnsition from landscape to portrait
                client.logger.info('wrong aspect ratio detected, attempt to correct');
                window.setTimeout(function () {
                    // force relayout;
                    client.logger.info('delayed re-render');
                    _this.render();
                }, 50);
                return this;
            }
            // bind events
            this.bindOnNavigate('a.brand', function () {
                _this.$el.on('click', '.timelineBox', _.bind(_this.onBoxClick, _this));
            });
            client.user.listCachedGroups({ limit: TimelineScreen.groupLimit }, _.bind(this.onListCastGroups, this));
            // Initiate caching of device contacts
            client.DeviceContacts.initContacts();
            // Show any message delivered by the server
            this.showServerMessage();
            this.syncNoPan();
            // Check if sharetargets were passed in, if so start cast with them
            if (window.sharetarget) {
                client.shareHandler.processShareTargets(function () {
                });
            }
            return this;
        };
        /** Show server message as a popup, then clear it */
        TimelineScreen.prototype.showServerMessage = function () {
            if (!client.user.serverMessage || client.user.serverMessage.length == 0) {
                return;
            }
            // Limit length of server message
            client.user.serverMessage = client.user.serverMessage.substr(0, 400);
            var $popUpContent = this.$el.find('.serverMessagePopUpContent');
            $popUpContent.empty();
            var confirmationPopUp = new client.ConfirmationPopUp($popUpContent, resources.messages.serverMessage.title, client.user.serverMessage, resources.messages.serverMessage.choiceLeft, null, function () {
            });
            client.user.serverMessage = '';
        };
        TimelineScreen.prototype.getTimeString = function (timeStamp) {
            var ximDate = moment(timeStamp);
            var now = moment();
            var daysDiff = now.diff(ximDate, 'days');
            if (ximDate.day() == now.day()) {
                return "Today " + ximDate.format("h:mm a");
            }
            else if (Math.abs(now.day() - ximDate.day()) == 1) {
                return "Yesterday " + ximDate.format("h:mm a");
            }
            else if (daysDiff < 7) {
                return ximDate.format("dddd h:mm a");
            }
            else {
                return ximDate.format("dddd, MMMM Do, h:mm a");
            }
        };
        // Render command bars (done after first load of people)
        TimelineScreen.prototype.renderCommands = function () {
            // CommandBar
            this.$el.find('.timelineUL').show();
            var commandStyle = {
                tab: '',
                bar: '',
                pressed: 'commandPressed',
                selected: '',
                isTabbed: false,
                allowNoActiveTab: false,
                bounceBall: false,
                glyphSize: 0,
                textSize: 36
            };
            this.commandBar.commandBarStyle = commandStyle;
            this.commandBar.setCommands([consts.commands.newgroup], _.bind(this.onCommandClick, this));
        };
        TimelineScreen.prototype.renderGroups = function () {
            // If dialog is closing, ingore
            if (this.closing) {
                return;
            }
            var html = "<div>";
            for (var ig = 0; ig < this.groups.length; ig++) {
                var group = this.groups[ig];
                var timeRemaining = group.minutesRemaining();
                // Only show expired Xims if requested
                if (timeRemaining == 0 && !client.user.GetSetting(9 /* SHOW_EXPIRED */)) {
                    continue;
                }
                this.renderedGroupCount++;
                html += "<div class='timelineBox" + ((timeRemaining == 0) ? " timelineExpired" : "") + "' id='" + group.profile.id + "'>";
                html += "<div class='timelineThumbnail', style='background-image:url(" + group.profile.thumbnailUrl + ")'></div>";
                // Time - DISABLED
                //var timeString = this.getTimeString(group.profile.created);
                //html += '<label class="timelineTime">' + timeString + '</label>';
                // Time Remaining Label
                html += '<label class="timelineRemaining"></label>';
                // Note that a live Xim is available
                if (timeRemaining > 0) {
                    this.anyLiveGroups = true;
                }
                // Images
                var profileCount = 0;
                for (var id in group.profile.members) {
                    var profile = group.profile.members[id];
                    if (client.user.profile && (profile.userId != client.user.profile.userId)) {
                        if (profileCount < consts.nearbyProfileLimit) {
                            html += '<label class="timelineName">' + profile.displayName + '</label>';
                        }
                        profileCount++;
                    }
                }
                var remaining = profileCount - consts.nearbyProfileLimit;
                if (remaining > 0) {
                    html += '<label class="timelineName">...and ' + remaining + ' more</label>';
                }
                html += "</div>";
            }
            html += "</div>";
            if (this.renderedGroupCount == 0 || !client.user.isConnected()) {
                this.$el.find('.timelineNoXims').toggleClass('hidden', false);
                this.$el.find('.timelineGroupsXims').toggleClass('hidden', true);
            }
            else {
                this.$el.find('.timelineNoXims').toggleClass('hidden', true);
                // Now render the content
                var $content = this.$el.find('.timelineGroupsXims');
                $content.empty();
                $content.append(html);
                $content.toggleClass('hidden', false);
            }
            // Update expiration times
            this.updateExpirationTimes();
            // If first time rendering groups, also render commands
            if (this.firstGroupRender) {
                // Render command bars
                this.renderCommands();
                this.firstGroupRender = false;
            }
        };
        /** Update expiration times */
        TimelineScreen.prototype.updateExpirationTimes = function () {
            var _this = this;
            for (var ig = 0; ig < this.groups.length; ig++) {
                var group = this.groups[ig];
                var box = this.$el.find('#' + group.profile.id);
                var timeRemaining = group.minutesRemaining();
                if (timeRemaining > 0) {
                    var label = box.find('.timelineRemaining');
                    var text = 'Expiring in ' + timeRemaining + ((timeRemaining > 1) ? ' minutes' : " minute");
                    label.text(text);
                }
                else {
                    box.remove();
                }
            }
            // Update again in 60 seconds
            this.expirationTimer = setTimeout(function () {
                _this.updateExpirationTimes();
            }, 60000);
        };
        TimelineScreen.prototype.onListCastGroups = function (error, results) {
            client.logger.info('onListCastGroups error =', error);
            var connected = client.user.isConnected();
            if (error && connected) {
                client.logger.error(consts.errorCode.Timeline, 'Failed to load cast groups', error);
                return;
            }
            // If minTime set, only include Xims that occurred before then
            var minTime = client.user.GetSetting(10 /* CAST_MIN_TIME */);
            if (minTime && results && _.isObject(results)) {
                this.$el.find('.timelineGroupsXims').toggleClass('loading', false);
                this.groups = [];
                for (var i = 0; i < results.models.length; i++) {
                    var barDate = moment(minTime);
                    var pastDate = moment(results.models[i].profile.lastActivity);
                    var diff = barDate.diff(pastDate, 'seconds');
                    if (diff < 0) {
                        this.groups.push(results.models[i]);
                    }
                }
            }
            else if (results && _.isObject(results) && results.models) {
                this.groups = results.models;
            }
            // todo: Temporary work around for double render call (LARS)
            if (this.firstRender) {
                this.$el.render('timelineScreen', { messages: resources.messages, isWp: client.startupMode == 3 /* wp */ });
                this.firstRender = false;
            }
            this.renderGroups();
        };
        TimelineScreen.prototype.onLogout = function (loggedout) {
        };
        // Max number of groups to display
        TimelineScreen.groupLimit = 15;
        // Converts time stamp to string representing how long ago
        TimelineScreen.oneDay = 86400000;
        return TimelineScreen;
    })(client.DetailScreen);
    client.TimelineScreen = TimelineScreen;
})(client || (client = {}));
/// <reference path="DialogScreen.ts"/>
var client;
(function (client) {
    /** Displays a login screen. */
    var LoginScreen = (function (_super) {
        __extends(LoginScreen, _super);
        function LoginScreen(sel, history, commands, commandCallback) {
            _super.call(this, sel, history, commands || [consts.commands.cancel], commandCallback || client.dialog.handleDialogCommand);
        }
        LoginScreen.prototype.navigate = function (route) {
            var _this = this;
            this.render();
            this.bindOnNavigate('', function () {
                _this.$el.find('#login-facebook').click(function (e) {
                    e.preventDefault();
                    async.series(client.app.tasks, function (e, result) {
                        // fetch user from user userprofile plugin again
                        if (!e) {
                            _this.history.close(null, _.last(result));
                        }
                        else {
                            _this.history.close(e, false);
                        }
                    });
                });
            });
            return _super.prototype.navigate.call(this, route);
        };
        LoginScreen.prototype.render = function () {
            // Make screen is in a valid state
            if (this.getScreenState() == 3 /* DISPOSED */) {
                client.logger.warn("Render called on DISPOSED screen ");
                return this;
            }
            // Render view
            this.empty();
            this.$el.render('login', { title: resources.labels.login.login });
            return _super.prototype.render.call(this);
        };
        /**
          * Display login screen
          * @param message The loading message to display.
          * @param callback (optional) The callback to call once the screen has been dropped.
          */
        LoginScreen.display = function (callback) {
            return client.screens.showModal(LoginScreen, { style: 0 /* dialog */, params: {} }, function (e, results) {
                callback(e, results);
                return false;
            });
        };
        return LoginScreen;
    })(client.DialogScreen);
    client.LoginScreen = LoginScreen;
})(client || (client = {}));
/// <reference path="../types.ts"/>
/// <reference path="../user.ts"/>
/// <reference path="../screens/AppMasterScreen.ts"/>
/// <reference path="../../../types/jquery/jquery.d.ts"/>
/// <reference path="PopUp.ts"/>
var client;
(function (client) {
    /** App Debug Messager */
    var DebugPopUp = (function (_super) {
        __extends(DebugPopUp, _super);
        function DebugPopUp(message, uri, lineNumber, columnNumber, error) {
            var _this = this;
            // Call superclass to render Popup
            _super.call(this, $('.app-message'), "debugPopUp", { error: message, uri: uri, line: lineNumber, column: columnNumber });
            this.$el.on('click', '.errorPopUp', function () {
                var content = "";
                content += "---------------------------" + consts.linebreak;
                if (error && error.stack) {
                    content += "Detail: " + error.stack + consts.linebreak;
                }
                else {
                    content += "Error: " + message + consts.linebreak;
                    content += "File: " + uri + consts.linebreak;
                    content += "Line: " + lineNumber + consts.linebreak;
                    content += "Column: " + columnNumber + consts.linebreak;
                }
                content += "---------------------------" + consts.linebreak;
                content += "User: " + ((client.user) ? client.user.profile.userId : '') + consts.linebreak;
                content += "Application Version: " + consts.applicationVersion + "\n" + consts.linebreak;
                content += "Startup Mode: " + _this.getStartupMode() + "\n" + consts.linebreak;
                content += "Device: " + (window.device ? window.device.model : '') + consts.linebreak;
                content += "Connection State: " + _this.getConnectionState() + consts.linebreak;
                content += client.screens.master.debugGetScreenInfo() + consts.linebreak;
                content += client.History.debugGetHistoryInfo() + consts.linebreak;
                if (client.startupMode == 3 /* wp */) {
                    window.open(consts.externalLinks.bug + '&body=' + content, '_self');
                }
                else {
                    window.open(consts.externalLinks.bug + '&body=' + content, '_system');
                }
                _this.destroy();
            });
            $('.app-message').show();
        }
        DebugPopUp.prototype.getStartupMode = function () {
            switch (client.startupMode) {
                case 0:
                    return "Desktop";
                case 1:
                    return "Emulator";
                case 2:
                    return "Mobile";
                case 3:
                    return "WP";
                case 4:
                    return "iOS";
                case 5:
                    return "Android";
            }
            return "Unknown";
        };
        DebugPopUp.prototype.getConnectionState = function () {
            if (client.user) {
                switch (client.user.getConnectionState()) {
                    case 3 /* connecting */:
                        return "Connecting";
                    case 4 /* connected */:
                        return "Connected";
                    case 5 /* reconnecting */:
                        return "Reconnecting";
                    default:
                        return "Disconnected";
                }
            }
            return "Unknown";
        };
        return DebugPopUp;
    })(client.PopUp);
    client.DebugPopUp = DebugPopUp;
})(client || (client = {}));
/// <reference path="appStorage.ts"/>
/// <reference path="types.ts"/>
/// <reference path="consts.ts"/>
/// <reference path="resources.ts"/>
/// <reference path="../../types/underscore/underscore.d.ts"/>
/// <reference path="../../types/cordova/cordova-chromecast.d.ts"/>
/// <reference path="../../types/cordova/appletv.d.ts"/>
/// <reference path="../../types/cordova/smartglass.d.ts"/>
/// <reference path="app.ts"/>
/// <reference path="scanner.ts"/>
/// <reference path="controls/PopUp.ts"/>
/// <reference path="screens/Login.ts"/>
/// <reference path="screens/DebugScreen.ts"/>
/// <reference path="screens/SettingsScreen.ts"/>
/// <reference path="screens/LoadingScreen.ts"/>
/// <reference path="screens/AlbumsScreen.ts"/>
/// <reference path="screens/SkydriveScreen.ts"/>
/// <reference path="screens/PickerScreen.ts"/>
/// <reference path="screens/ProfileEditScreen.ts"/>
/// <reference path="screens/TokenScreen.ts"/>
/// <reference path="screens/TimelineScreen.ts"/>
/// <reference path="screens/LoginScreen.ts"/>
/// <reference path="screens/FtuxScreen.ts"/>
/// <reference path="controls/DebugPopUp.ts"/>
/// <reference path="../../types/cordova/urlhandler.d.ts"/>
/// <reference path="../shared/googlecast.ts"/>
/** global function to be called by native code when a deep app link was invoked */
function handleOpenURL(url) {
    client.logger.debug('deep app launch with', url);
    client.launchUrl = url;
}
var client;
(function (client) {
    client.app.addTask(function (callback) {
        var loading = client.LoadingScreen.display(resources.messages.tagline, callback);
        var done = loading.closeAsync();
        async.series([
            startupStep('client.appStorage.init', true, client.appStorage.init),
            startupStep('client.user.init', true, client.user.init),
            client.Login.start,
            startupStep('client.logger.collect', false, function (next) {
                client.logger.collect(consts.tags.boot);
                next(null, true);
            }),
            startupStep('CastViewerAirPlay.startScan', false, function (next) {
                // register for airplay mirroring
                client.CastViewerAirPlay.startScan();
                next(null, true);
            }),
            function (next) {
                // register for push asynchronously
                startupStep('client.user.registerForPush', false, function (next) {
                    if (window.pushnotification) {
                        client.user.registerForPush(next);
                    }
                    else {
                        next(null, true);
                    }
                })(function () {
                });
                next(null, true);
            }
        ], done);
    });
    function startupStep(name, required, step, timeout) {
        if (timeout === void 0) { timeout = 5000; }
        return function (next) {
            // Start timer
            var nextCalled = false;
            var ht = setTimeout(function () {
                if (required) {
                    client.logger.error('startup', name + ': timeout');
                    next(new Error('timeout'), false);
                }
                else {
                    client.logger.warn('startup', name + ': timeout');
                    next(null, true);
                }
                nextCalled = true;
            }, timeout);
            try {
                step(function (err, success) {
                    if (!nextCalled) {
                        clearTimeout(ht);
                        next(err, success);
                    }
                });
            }
            catch (e) {
                client.logger.error('startup', name + ': exception - ' + e.toString());
                next(e, false);
            }
        };
    }
    client.launchUrl;
    function exitApp(e) {
        if (e) {
            e.preventDefault();
        }
        client.analytics.end();
        if (phone.app && phone.app.exitApp) {
            phone.app.exitApp();
        }
        else {
            throw consts.errorCode.NoMoreBack;
        }
    }
    client.exitApp = exitApp;
    /** Called to start the client. */
    function start(selector, mode) {
        client.logger.info('Page loaded');
        function deviceReady() {
            window.onerror = (function (message, uri, lineNumber, columnNumber, error) {
                // windows phone doesn't have exitApp, so it quits by throwing; we don't need to log it as error, the thrown exception
                // will be caught by cordova's native handling code in CordovaView.page_BackKeyPress and the app
                // will be closed accordingly
                if (client.startupMode == 3 /* wp */ && message == consts.errorCode.NoMoreBack) {
                    return true;
                }
                if (error && error.stack) {
                    client.logger.error(consts.errorCode.UnhandledException, error.stack);
                }
                else {
                    var errMessage = message + "\n File: " + uri + "\n Line: " + lineNumber + "\n Column: " + columnNumber;
                    client.logger.error(consts.errorCode.UnhandledException, errMessage);
                }
                // Show pop up message if in debug mode
                if (client.user.GetSetting(6 /* SHOW_DEBUG */) || (parseFloat(consts.applicationVersion) < 1.0)) {
                    var popup = new client.DebugPopUp(message, uri, lineNumber, columnNumber, error);
                    popup.show(true);
                }
                return true;
            });
            // Process any switches
            client.analytics.startAnalytics(1 /* flurry */);
            var q = querystring.parse(location.search);
            if (q['simulated'])
                client.user.simulatedUsers = parseInt(q['simulated']);
            client.analytics.logEvent('deviceReady');
            // hook into deep url luanching
            if (window.urlhandler) {
                window.urlhandler.subscribe(function (url) {
                    if (url) {
                        handleOpenURL(url);
                    }
                }, function (e) {
                    client.logger.error(consts.errorCode.UrlHandling, 'error with urlhandler plugin', e);
                });
            }
            if (window.screenlock) {
                window.screenlock.set(consts.ScreenLockState.vertical);
            }
            // TODO: Fix backbutton processing.
            if (client.isPhoneGap) {
                document.addEventListener('backbutton', function (e) {
                    // First try to close open pop up
                    if (client.PopUp.back()) {
                        return;
                    }
                    var state = history.state || client.History.state();
                    if (!client.screens.master || !client.screens.master.back()) {
                        client.screens.canStartNewXim(function (e, f) {
                            if (f || state && state.isRoot) {
                                exitApp(e);
                            }
                            else {
                                history.back();
                            }
                        });
                    }
                });
            }
            // Setup master screen
            var master = new client.AppMasterScreen();
            master.addDetailScreen('/', client.TimelineScreen, consts.tabs.timeline);
            master.addDetailScreen('/editprofile', client.ProfileEditScreen, consts.tabs.profile);
            master.addDetailScreen('/settings', client.SettingsScreen, consts.tabs.settings);
            master.addMenuCommand(consts.tabs.faq, function () {
                client.logger.collectNamedEvent(consts.flurryEvent.cogFaq);
                window.open(consts.externalLinks.faq, '_system');
            });
            master.addMenuCommand(consts.tabs.terms, function () {
                client.logger.collectNamedEvent(consts.flurryEvent.cogTerms);
                window.open(consts.externalLinks.terms, '_system');
            });
            master.addMenuCommand(consts.tabs.privacy, function () {
                client.logger.collectNamedEvent(consts.flurryEvent.cogPrivacy);
                window.open(consts.externalLinks.privacy, '_blank', 'location=no');
            });
            master.addMenuCommand(consts.tabs.feeback, function () {
                client.logger.collectNamedEvent(consts.flurryEvent.cogFeedback);
                window.open(consts.externalLinks.email, '_system');
            });
            master.addMenuCommand(consts.tabs.credits, function () {
                client.logger.collectNamedEvent(consts.flurryEvent.cogCredits);
                window.open(consts.externalLinks.credits, '_system');
            });
            // build number is even, then it indicates that this is a dev build
            var isDevBuild = (parseInt(consts.applicationVersion.split('.')[2]) % 2) === 0;
            if (isDevBuild) {
                master.addDetailScreen('/debug', client.DebugScreen, consts.debugCommands.debug);
            }
            else {
                master.addDetailScreen('/log', client.DebugScreen, consts.debugCommands.log);
            }
            master.removeMenuCommand(consts.tabs.timeline);
            master.addMenuCommand(consts.debugCommands.showSplash, function () {
                // this allows dev to invoke the loading screen and hold it
                // for visual inspection
                client.LoadingScreen.display(resources.messages.tagline, function () {
                });
                // hide the '.loading' animation so we can take screen shot
                // to use as splash screen
                $('.splash .loading').toggleClass('hidden', true);
            });
            master.addMenuCommand(consts.debugCommands.home, function () {
                // since ios has no back button, we use this
                // to get us out of the debug screen
                client.app.navigate('/');
            });
            master.addMenuCommand(consts.debugCommands.ftux, function () {
                var screens = [];
                screens.push(client.wizard.wrapScreen(client.FtuxScreen));
                screens.push(client.wizard.wrapScreen(client.LoginPhoneScreen));
                screens.push(client.wizard.wrapScreen(client.LoginVerifyScreen));
                var wizard = client.wizard.wizard(screens, function () {
                });
                wizard.commands = [];
                wizard.commandCallback = function () {
                };
                wizard.begin();
            });
            master.addMenuCommand(consts.debugCommands.chromecast, function () {
                if (window.chrome) {
                    client.cast.googleCastClient = new client.cast.GoogleCast(consts.cloudProviders.chromecast);
                    async.series([
                        // initialize chromecast scan
                        function (cb) {
                            client.cast.googleCastClient.init(function (available) {
                                client.logger.info('receiver callback with result: ', available);
                                if (available) {
                                    client.logger.info('routes available: ', client.cast.googleCastClient.getRoutes());
                                }
                            }, cb);
                        },
                        // let user choose a chromecast to connect to
                        function (cb) {
                            client.cast.googleCastClient.requestSession(cb);
                        }
                    ], function (err, res) {
                        if (err) {
                            client.logger.error(consts.errorCode.ChromeCast, 'launching chromecast receiver app', err);
                        }
                    });
                }
            });
            if (window.appletv) {
                var appletv = window.appletv;
                master.addMenuCommand(consts.debugCommands.appletv, function () {
                    appletv.registerListener(function (screen) {
                        if (_.isString(screen)) {
                            screen = JSON.parse(screen);
                        }
                        if (screen.connected) {
                            client.logger.info('appletv connected to', screen.name);
                            appletv.loadUrl('http://www.getxim.com', function () {
                                client.logger.info('appletv load url ok');
                            }, function (e) {
                                client.logger.warn('appletv load url failed', e);
                            });
                            client.user.onXimJoined.add(function (ximMessage) {
                                appletv.loadUrl(ximMessage.viewerUrl, function () {
                                    client.logger.info('appletv joined xim', ximMessage.ximSessionID);
                                }, function (e) {
                                    client.logger.warn('applvetv failed to join xim', ximMessage.ximSessionID, e);
                                });
                            });
                        }
                        else {
                            client.logger.info('appletv disconnected', screen.name);
                        }
                    }, function (e) {
                        client.logger.warn('appletv failed to register listener', e);
                    });
                });
            }
            if (window.smartglass) {
                var smartglass = window.smartglass;
                master.addMenuCommand(consts.debugCommands.smartglass, function () {
                    smartglass.registerListener(function (screen) {
                        client.logger.info("discovered smartglass screen: ", screen);
                        if (_.isString(screen)) {
                            screen = JSON.parse(screen);
                        }
                        if (screen.connected) {
                            smartglass.selectRoute(screen.routeId, function (connected) {
                                if (connected) {
                                    client.logger.info('smartglass connected to', screen.name);
                                    smartglass.loadUrl('http://www.getxim.com', function () {
                                        client.logger.info('smartglass load url ok');
                                    }, function (e) {
                                        client.logger.warn('smartglass load url failed', e);
                                    });
                                    client.user.onXimJoined.add(function (ximMessage) {
                                        smartglass.loadUrl(ximMessage.viewerUrl, function () {
                                            client.logger.info('smartglass joined xim', ximMessage.ximSessionID);
                                        }, function (e) {
                                            client.logger.warn('smartglass failed to join xim', ximMessage.ximSessionID, e);
                                        });
                                    });
                                }
                            }, function (e) {
                                client.logger.warn('smartglass failed to select route', e);
                            });
                        }
                        else {
                            client.logger.info('smartglass disconnected', screen.name);
                        }
                    }, function (e) {
                        client.logger.warn('smartglass failed to register listener', e);
                    });
                });
            }
            client.screens.master = master;
            // Start app
            client.logger.info('Starting Application');
            var tStart = client.logger.start();
            client.app.start(function (err) {
                if (!err) {
                    client.logger.stop(tStart, 'Application started');
                }
                else {
                    client.logger.stop(tStart, 'Application startup error');
                    client.app.navigate('/log');
                }
            });
        }
        client.startupMode = mode;
        var isChrome = (navigator.platform == 'Win32' && navigator.vendor == 'Google Inc.');
        if (mode == 0 /* desktop */ || mode == 2 /* mobile */ || isChrome || typeof Cordova === 'undefined') {
            $(document).ready(deviceReady);
        }
        else {
            client.isPhoneGap = true;
            Cordova.onDeviceReady(deviceReady);
        }
    }
    client.start = start;
})(client || (client = {}));
