
jade = (function(exports){
/*!
 * Jade - runtime
 * Copyright(c) 2010 TJ Holowaychuk <tj@vision-media.ca>
 * MIT Licensed
 */

/**
 * Lame Array.isArray() polyfill for now.
 */

if (!Array.isArray) {
  Array.isArray = function(arr){
    return '[object Array]' == Object.prototype.toString.call(arr);
  };
}

/**
 * Lame Object.keys() polyfill for now.
 */

if (!Object.keys) {
  Object.keys = function(obj){
    var arr = [];
    for (var key in obj) {
      if (obj.hasOwnProperty(key)) {
        arr.push(key);
      }
    }
    return arr;
  }
}

/**
 * Merge two attribute objects giving precedence
 * to values in object `b`. Classes are special-cased
 * allowing for arrays and merging/joining appropriately
 * resulting in a string.
 *
 * @param {Object} a
 * @param {Object} b
 * @return {Object} a
 * @api private
 */

exports.merge = function merge(a, b) {
  var ac = a['class'];
  var bc = b['class'];

  if (ac || bc) {
    ac = ac || [];
    bc = bc || [];
    if (!Array.isArray(ac)) ac = [ac];
    if (!Array.isArray(bc)) bc = [bc];
    ac = ac.filter(nulls);
    bc = bc.filter(nulls);
    a['class'] = ac.concat(bc).join(' ');
  }

  for (var key in b) {
    if (key != 'class') {
      a[key] = b[key];
    }
  }

  return a;
};

/**
 * Filter null `val`s.
 *
 * @param {Mixed} val
 * @return {Mixed}
 * @api private
 */

function nulls(val) {
  return val != null;
}

/**
 * Render the given attributes object.
 *
 * @param {Object} obj
 * @param {Object} escaped
 * @return {String}
 * @api private
 */

exports.attrs = function attrs(obj, escaped){
  var buf = []
    , terse = obj.terse;

  delete obj.terse;
  var keys = Object.keys(obj)
    , len = keys.length;

  if (len) {
    buf.push('');
    for (var i = 0; i < len; ++i) {
      var key = keys[i]
        , val = obj[key];

      if ('boolean' == typeof val || null == val) {
        if (val) {
          terse
            ? buf.push(key)
            : buf.push(key + '="' + key + '"');
        }
      } else if (0 == key.indexOf('data') && 'string' != typeof val) {
        buf.push(key + "='" + JSON.stringify(val) + "'");
      } else if ('class' == key && Array.isArray(val)) {
        buf.push(key + '="' + exports.escape(val.join(' ')) + '"');
      } else if (escaped && escaped[key]) {
        buf.push(key + '="' + exports.escape(val) + '"');
      } else {
        buf.push(key + '="' + val + '"');
      }
    }
  }

  return buf.join(' ');
};

/**
 * Escape the given string of `html`.
 *
 * @param {String} html
 * @return {String}
 * @api private
 */

exports.escape = function escape(html){
  return String(html)
    .replace(/&(?!(\w+|\#\d+);)/g, '&amp;')
    .replace(/</g, '&lt;')
    .replace(/>/g, '&gt;')
    .replace(/"/g, '&quot;');
};

/**
 * Re-throw the given `err` in context to the
 * the jade in `filename` at the given `lineno`.
 *
 * @param {Error} err
 * @param {String} filename
 * @param {String} lineno
 * @api private
 */

exports.rethrow = function rethrow(err, filename, lineno){
  if (!filename) throw err;

  var context = 3
    , str = require('fs').readFileSync(filename, 'utf8')
    , lines = str.split('\n')
    , start = Math.max(lineno - context, 0)
    , end = Math.min(lines.length, lineno + context);

  // Error context
  var context = lines.slice(start, end).map(function(line, i){
    var curr = i + start + 1;
    return (curr == lineno ? '  > ' : '    ')
      + curr
      + '| '
      + line;
  }).join('\n');

  // Alter exception message
  err.path = filename;
  err.message = (filename || 'Jade') + ':' + lineno
    + '\n' + context + '\n\n' + err.message;
  throw err;
};

  return exports;

})({});

jade.templates = {};
jade.render = function(node, template, data) {
  var tmp = jade.templates[template](data);
  node.innerHTML = tmp;
};

jade.templates["addBrowserPopUp"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="addBrowserPopUp"><div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = data.resources.title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div><label class="addBrowserTitle">');
var __val__ = data.resources.text
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label>');
if ( (data.resources.bullets))
{
buf.push('<div class="howToIntro"><ol>');
// iterate data.resources.bullets
;(function(){
  if ('number' == typeof data.resources.bullets.length) {
    for (var $index = 0, $$l = data.resources.bullets.length; $index < $$l; $index++) {
      var bullet = data.resources.bullets[$index];

buf.push('<li>');
var __val__ = bullet
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</li>');
    }
  } else {
    for (var $index in data.resources.bullets) {
      var bullet = data.resources.bullets[$index];

buf.push('<li>');
var __val__ = bullet
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</li>');
   }
  }
}).call(this);

buf.push('</ol></div>');
}
else
{
buf.push('<div class="addBrowserIntro"><label class="dialogBullet">');
var __val__ = data.resources.bullet1
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><label class="addBrowserLink">');
var __val__ = data.url
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><label class="dialogBulletTop">');
var __val__ = data.resources.bullet3
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><label class="dialogBulletSub">');
var __val__ = data.resources.bullet3s
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label></div>');
}
buf.push('<div class="addBrowserInputbox"><form onsubmit="return false" action="#"><input');
buf.push(attrs({ 'type':("text"), 'name':("code"), 'placeholder':('' + (data.resources.placeholder) + ''), 'autocomplete':("off"), 'autocorrect':("off"), 'autocapitalize':("off"), 'spellcheck':("false"), "class": ('ximformcontrol') + ' ' + ('dialogInputText') }, {"type":true,"name":true,"placeholder":true,"autocomplete":true,"autocorrect":true,"autocapitalize":true,"spellcheck":true}));
buf.push('/><label class="dialogError"></label><input');
buf.push(attrs({ 'type':("submit"), 'value':('' + (data.resources.button) + ''), "class": ('addBrowserButton') }, {"type":true,"value":true}));
buf.push('/></form></div><label class="addBrowserFooter underline">');
var __val__ = data.resources.subtext
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><div style="display:none" class="addBrowserWait"><i class="dot"></i><i class="dot"></i><i class="dot"></i><i class="dot"></i><i class="dot"></i></div></div>');
}
return buf.join("");
}
jade.templates["albums"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="dialog-body"><div id="album-items" class="section">');
if ( albums.length)
{
buf.push('<div class="items tile-list"></div>');
}
else
{
buf.push('<div class="message loading">' + escape((interp = resources.messages.loadingFacebook) == null ? '' : interp) + '</div>');
}
buf.push('</div>');
if ( (title))
{
buf.push('<div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div>');
}
buf.push('<div class="command-bar commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["appMasterScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<i style="display: none" class="glyphiconXIM glyphiconXIM-exclamationIcon connectionglyph"></i><div class="header hidden"><h1 class="apptitle">xim</h1><i class="glyphiconXIM glyphiconXIM-settingsCog headerglyph cogglyph"></i></div><!-- DISABLED  img.dot(src="img/public-state.png")--><div class="headerAlt hidden"><h1 class="apptitle alttitle"> </h1><i class="ios-only glyphiconXIM glyphiconXIM-backArrowIcon headerglyph backglyph"></i></div><div class="navMenuContent"></div><div class="privacyMenuContent"></div><div class="page"><div class="detail-screens"></div><div class="commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["camera"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="dialog-body"><!----><div');
buf.push(attrs({ 'id':(contentSource +'-items'), "class": ('section') }, {"id":true}));
buf.push('>');
if ( albums.length)
{
buf.push('<div class="items tile-list"></div>');
}
else
{
buf.push('<div class="message loading"> ' + escape((interp = resources.messages.loadingCamera) == null ? '' : interp) + '</div>');
}
buf.push('</div>');
if ( (title))
{
buf.push('<div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div>');
}
buf.push('<div class="command-bar commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["CastScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="castContent"></div><div class="debugInfo"></div><div class="debugEffect"></div><div class="castTopBar"></div><div class="castCommandBar"></div><div class="castAltBar"></div><div class="castPeoplePickerCommandBar"></div><div class="bubbleContent"></div><div class="hoverMenuContent"></div><div class="castMessagePopUp"></div><div class="castPeoplePicker hidden"></div><div class="castPeoplePopUp"></div><div class="castPhotoPopUp"></div>');
}
return buf.join("");
}
jade.templates["castViewerChoice"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="viewerChoiceHeader"><label class="viewerChoiceTitle">');
var __val__ = resources.scanning
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><div class="viewerChoiceWait small"><i class="dot"></i><i class="dot"></i><i class="dot"></i><i class="dot"></i><i class="dot"></i></div></div>');
// iterate castViewers
;(function(){
  if ('number' == typeof castViewers.length) {
    for (var index = 0, $$l = castViewers.length; index < $$l; index++) {
      var castViewer = castViewers[index];

buf.push('<div');
buf.push(attrs({ 'id':('viewer_' + (index) + ''), "class": ('viewerChoiceBox') }, {"id":true}));
buf.push('><div');
buf.push(attrs({ "class": ('viewerChoiceGlyph') + ' ' + ('' + (castViewer.glyph) + '') }, {"class":true}));
buf.push('></div><div class="viewerChoiceLabel">' + escape((interp = castViewer.label) == null ? '' : interp) + '</div></div>');
    }
  } else {
    for (var index in castViewers) {
      var castViewer = castViewers[index];

buf.push('<div');
buf.push(attrs({ 'id':('viewer_' + (index) + ''), "class": ('viewerChoiceBox') }, {"id":true}));
buf.push('><div');
buf.push(attrs({ "class": ('viewerChoiceGlyph') + ' ' + ('' + (castViewer.glyph) + '') }, {"class":true}));
buf.push('></div><div class="viewerChoiceLabel">' + escape((interp = castViewer.label) == null ? '' : interp) + '</div></div>');
   }
  }
}).call(this);

}
return buf.join("");
}
jade.templates["castViewerChoicePopUp"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="castViewerChoicePopUp"></div>');
}
return buf.join("");
}
jade.templates["chromeTopBar"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="chromeTopBase"><div class="castBackArrow"></div></div>');
}
return buf.join("");
}
jade.templates["CodeScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
 var cdn = client.baseUri || '';
buf.push('<div class="text-center codeScreen"><div class="codeBlock"><div class="codeText">' + escape((interp = text.line1) == null ? '' : interp) + '</div><div class="codeText">' + escape((interp = text.line2) == null ? '' : interp) + '</div><div class="codeTextPin">' + escape((interp = pin) == null ? '' : interp) + '</div><div class="codeText">' + escape((interp = text.line3) == null ? '' : interp) + '</div><div class="codeSmall">' + escape((interp = text.line4) == null ? '' : interp) + '</div><div class="codeLink">' + escape((interp = text.link) == null ? '' : interp) + '</div></div>');
if ( isWp)
{
buf.push('<object');
buf.push(attrs({ 'data':('' + (cdn) + 'svg/curiousSimpleSun.svg'), 'type':('image/svg+xml'), 'width':('460px'), 'height':('460px'), "class": ('codeImg') }, {"data":true,"type":true,"width":true,"height":true}));
buf.push('></object>');
}
else
{
buf.push('<img');
buf.push(attrs({ 'src':('' + (cdn) + 'svg/curiousSimpleSun.svg'), "class": ('codeImg') }, {"src":true}));
buf.push('/>');
}
buf.push('</div>');
}
return buf.join("");
}
jade.templates["commandBar"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div');
buf.push(attrs({ "class": ('commandBarBase') + ' ' + ('' + (style.bar) + '') }, {"class":true}));
buf.push('><ul class="commandBarNav">');
// iterate commands 
;(function(){
  if ('number' == typeof commands .length) {
    for (var $index = 0, $$l = commands .length; $index < $$l; $index++) {
      var cmd = commands [$index];

buf.push('<li');
buf.push(attrs({ 'id':('' + (cmd.id) + ''), 'style':("font-size: " + (style.textSize) + "px; width:" + (width) + "%;"), "class": ((cmd.enabled? '' : 'commandDisabled') + ' ' + (style.tab) + '') }, {"id":true,"class":true,"style":true}));
buf.push('>');
if ( (style.glyphSize == 0))
{
buf.push('<div>' + escape((interp = cmd.label) == null ? '' : interp) + '</div>');
}
else if ( (cmd.icon))
{
buf.push('<span><a href="#"><img');
buf.push(attrs({ 'src':('' + (cmd.icon) + '') }, {"src":true}));
buf.push('/></a></span><span>' + escape((interp = cmd.label) == null ? '' : interp) + '</span>');
}
else
{
if ( (cmd.glyph) )
{
buf.push('<div><span');
buf.push(attrs({ 'style':('font-size: ' + (style.glyphSize) + 'px'), "class": ('commandBarGlyph') + ' ' + ('' + (cmd.glyph) + '') }, {"class":true,"style":true}));
buf.push('></span></div><div class="commandBarLabel">' + escape((interp = cmd.label) == null ? '' : interp) + '</div>');
}
else
{
buf.push('<div>' + escape((interp = cmd.label) == null ? '' : interp) + '</div>');
}
}
buf.push('</li>');
    }
  } else {
    for (var $index in commands ) {
      var cmd = commands [$index];

buf.push('<li');
buf.push(attrs({ 'id':('' + (cmd.id) + ''), 'style':("font-size: " + (style.textSize) + "px; width:" + (width) + "%;"), "class": ((cmd.enabled? '' : 'commandDisabled') + ' ' + (style.tab) + '') }, {"id":true,"class":true,"style":true}));
buf.push('>');
if ( (style.glyphSize == 0))
{
buf.push('<div>' + escape((interp = cmd.label) == null ? '' : interp) + '</div>');
}
else if ( (cmd.icon))
{
buf.push('<span><a href="#"><img');
buf.push(attrs({ 'src':('' + (cmd.icon) + '') }, {"src":true}));
buf.push('/></a></span><span>' + escape((interp = cmd.label) == null ? '' : interp) + '</span>');
}
else
{
if ( (cmd.glyph) )
{
buf.push('<div><span');
buf.push(attrs({ 'style':('font-size: ' + (style.glyphSize) + 'px'), "class": ('commandBarGlyph') + ' ' + ('' + (cmd.glyph) + '') }, {"class":true,"style":true}));
buf.push('></span></div><div class="commandBarLabel">' + escape((interp = cmd.label) == null ? '' : interp) + '</div>');
}
else
{
buf.push('<div>' + escape((interp = cmd.label) == null ? '' : interp) + '</div>');
}
}
buf.push('</li>');
   }
  }
}).call(this);

buf.push('</ul></div><div class="commandBarBaseTarget"><ul class="commandBarNav">');
// iterate commands 
;(function(){
  if ('number' == typeof commands .length) {
    for (var $index = 0, $$l = commands .length; $index < $$l; $index++) {
      var cmd = commands [$index];

buf.push('<li');
buf.push(attrs({ 'id':('' + (cmd.id) + ''), 'style':("width:" + (width) + "%;"), "class": ('commandTarget') }, {"id":true,"style":true}));
buf.push('></li>');
    }
  } else {
    for (var $index in commands ) {
      var cmd = commands [$index];

buf.push('<li');
buf.push(attrs({ 'id':('' + (cmd.id) + ''), 'style':("width:" + (width) + "%;"), "class": ('commandTarget') }, {"id":true,"style":true}));
buf.push('></li>');
   }
  }
}).call(this);

buf.push('</ul></div>');
}
return buf.join("");
}
jade.templates["commentPopUp"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="commentPopUp"><textarea');
buf.push(attrs({ 'rows':("3"), 'placeholder':('' + (data.resources.placeholder) + ''), "class": ('form-control') + ' ' + ('commentPopUpInputForm') }, {"rows":true,"placeholder":true}));
buf.push('></textarea><ul class="confirmationPopUpMenu">');
if ( (data.resources.leftButton))
{
buf.push('<li id="confirmationPopUpLeft" class="confirmationPopUpLi">' + escape((interp = data.resources.leftButton) == null ? '' : interp) + '</li>');
}
if ( (data.resources.rightButton))
{
buf.push('<li id="confirmationPopUpRight" class="confirmationPopUpLi">' + escape((interp = data.resources.rightButton) == null ? '' : interp) + '</li>');
}
buf.push('</ul></div>');
}
return buf.join("");
}
jade.templates["confirmationPopUp"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="confirmationPopUp"><div class="confirmationTitle">' + escape((interp = data.title) == null ? '' : interp) + '</div><div class="confirmationMessage">' + escape((interp = data.message) == null ? '' : interp) + '</div><ul class="confirmationPopUpMenu">');
if ( (data.leftChoice))
{
buf.push('<li id="confirmationPopUpLeft" class="confirmationPopUpLi">' + escape((interp = data.leftChoice) == null ? '' : interp) + '</li>');
}
if ( (data.rightChoice))
{
buf.push('<li id="confirmationPopUpRight" class="confirmationPopUpLi">' + escape((interp = data.rightChoice) == null ? '' : interp) + '</li>');
}
buf.push('</ul></div>');
}
return buf.join("");
}
jade.templates["connecting"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="page"><div class="message fullscreen"><h3>Connecting to service...</h3></div></div>');
}
return buf.join("");
}
jade.templates["countryPopUp"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="countryPopUp"><div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = data.title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div><div class="loginPhone">');
// iterate data.countryArray
;(function(){
  if ('number' == typeof data.countryArray.length) {
    for (var $index = 0, $$l = data.countryArray.length; $index < $$l; $index++) {
      var val = data.countryArray[$index];

buf.push('<div');
buf.push(attrs({ 'id':('' + (val.code) + ''), "class": ('countryPopUpItem') }, {"id":true}));
buf.push('>');
var __val__ = val.name
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</div>');
    }
  } else {
    for (var $index in data.countryArray) {
      var val = data.countryArray[$index];

buf.push('<div');
buf.push(attrs({ 'id':('' + (val.code) + ''), "class": ('countryPopUpItem') }, {"id":true}));
buf.push('>');
var __val__ = val.name
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</div>');
   }
  }
}).call(this);

buf.push('</div></div>');
}
return buf.join("");
}
jade.templates["debugFooter"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="debugfooter"><form role="form" class="debug-server-response"><div class="form-group"><label class="debuglabel">Server Override:</label><select id="serverOverride" class="form-control debugformcontrol"><option value="0">- none -</option><option value="1">badRequest</option><option value="2">notAuthorized</option><option value="3">forbidden</option><option value="4">notFound</option><option value="5">upgradeRequired</option><option value="6">tokenExpired</option><option value="7">serverError</option><option value="8">connectionError</option><option value="9">otherError</option><option value="10">waitlisted</option></select></div></form><i id="debugButtonLogs" class="glyphicon glyphicon-book"></i></div>');
}
return buf.join("");
}
jade.templates["debugPopUp"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="errorPopUp popUpHide"><div class="debugText">Thanks beta tester.</div><div class="debugText">You found a bug!</div><hr/><div class="debugText">' + escape((interp = data.error) == null ? '' : interp) + '</div><div class="debugText">File: ' + escape((interp = data.uri) == null ? '' : interp) + '</div><div class="debugText">Line: ' + escape((interp = data.line) == null ? '' : interp) + '</div><div class="debugText">Column: ' + escape((interp = data.column) == null ? '' : interp) + '</div><input type="submit" value="Send Report" class="debugPopUpButton"/></div>');
}
return buf.join("");
}
jade.templates["debugScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="content"><div class="debugEffect"></div><div class="section"><form role="form" class="debug-setting"><div class="form-group"><label for="loginOverride" class="debugformlabel">');
var __val__ = resources.forms.loginOverridePrompt
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><select id="loginOverride" class="form-control debugformcontrol"><option value="">automatic</option><option value="fb">facebook</option><option value="test">test</option></select></div><div class="form-group"><label for="locationOverride" class="debugformlabel">');
var __val__ = resources.forms.locationOverridePrompt
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><select id="locationOverride" class="form-control debugformcontrol"><option value="">automatic</option><option value="47.640682,-122.141174">Bldg 112</option><option value="47.642124,-122.142021">Bldg 99</option><option value="47.641228,-122.123962">Bldg 36 (Conference Center)</option><option value="47.651302,-122.133377">Bldg 85</option><option value="47.619930,-122.348671">Space Needle</option></select></div><div class="form-group"><label for="loggerLevel" class="debugformlabel">');
var __val__ = resources.forms.loggerLevel
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><select id="loggerLevel" class="form-control debugformcontrol"><option value="0">info</option><option value="1">debug</option><option value="2">warning</option><option value="3">error</option></select></div><div class="form-group"><label for="discoverySetting" class="debugformlabel">');
var __val__ = resources.forms.discoverySetting
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><select id="discoverySetting" class="form-control debugformcontrol"><option');
buf.push(attrs({ 'value':(consts.NearbyDiscoveryMode.anyone) }, {"value":true}));
buf.push('>public</option><option');
buf.push(attrs({ 'value':(consts.NearbyDiscoveryModefriends) }, {"value":true}));
buf.push('>friends</option><option');
buf.push(attrs({ 'value':(consts.NearbyDiscoveryModeincognito) }, {"value":true}));
buf.push('>incognito</option></select></div>');
// iterate settings
;(function(){
  if ('number' == typeof settings.length) {
    for (var index = 0, $$l = settings.length; index < $$l; index++) {
      var val = settings[index];

buf.push('<!-- For boolean type (\'0\') only-->');
if ( (val['type'] == 0) )
{
buf.push('<div class="form-group debugcheckgroup">       <input');
buf.push(attrs({ 'id':("" + (index) + ""), 'type':("checkbox"), 'checked':(val['value']), "class": ('debugcheckbox') + ' ' + ("" + (val['name']) + "") }, {"class":true,"id":true,"type":true,"checked":true}));
buf.push('/><label for="val[\'name\']" class="debugchecklabel">');
var __val__ = val['description']
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label></div>');
}
    }
  } else {
    for (var index in settings) {
      var val = settings[index];

buf.push('<!-- For boolean type (\'0\') only-->');
if ( (val['type'] == 0) )
{
buf.push('<div class="form-group debugcheckgroup">       <input');
buf.push(attrs({ 'id':("" + (index) + ""), 'type':("checkbox"), 'checked':(val['value']), "class": ('debugcheckbox') + ' ' + ("" + (val['name']) + "") }, {"class":true,"id":true,"type":true,"checked":true}));
buf.push('/><label for="val[\'name\']" class="debugchecklabel">');
var __val__ = val['description']
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label></div>');
}
   }
  }
}).call(this);

buf.push('</form></div></div>');
}
return buf.join("");
}
jade.templates["DisplayScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="message-area text-center"><h2 id="message">');
var __val__ = message
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h2></div>');
}
return buf.join("");
}
jade.templates["dropbox"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="dialog-body"><div id="dropbox-items" class="section">');
if ( albums.length)
{
buf.push('<div class="items tile-list"></div>');
}
else
{
buf.push('<div class="message loading">' + escape((interp = resources.messages.loadingDropbox) == null ? '' : interp) + '</div>');
}
buf.push('</div>');
if ( (title))
{
buf.push('<div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div>');
}
buf.push('<div class="command-bar commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["errorPopUp"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="errorPopUp popUpHide">');
if ( data.isWp)
{
buf.push('<object');
buf.push(attrs({ 'data':('' + (data.message.screenType.svg) + ''), 'type':('image/svg+xml'), 'width':('460px'), 'height':('460px'), "class": ('errorImg') }, {"data":true,"type":true,"width":true,"height":true}));
buf.push('></object>');
}
else
{
buf.push('<img');
buf.push(attrs({ 'src':('' + (data.message.screenType.svg) + ''), "class": ('errorImg') }, {"src":true}));
buf.push('/>');
}
if ( (data.message.text1))
{
buf.push('<div');
buf.push(attrs({ 'style':("top:" + (data.message.screenType.text1top) + "px"), "class": ('errorText') }, {"style":true}));
buf.push('>' + escape((interp = data.message.text1) == null ? '' : interp) + '</div>');
}
if ( (data.message.text2))
{
buf.push('<div');
buf.push(attrs({ 'style':("top:" + (data.message.screenType.text2top) + "px"), "class": ('errorText') }, {"style":true}));
buf.push('>' + escape((interp = data.message.text2) == null ? '' : interp) + '</div>');
}
if ( (data.message.buttonText))
{
buf.push('<input');
buf.push(attrs({ 'style':("top:" + (data.message.screenType.buttontop) + "px"), 'type':('submit'), 'value':('' + (data.message.buttonText) + ''), "class": ('errorButton') }, {"style":true,"type":true,"value":true}));
buf.push('/>');
}
buf.push('</div>');
}
return buf.join("");
}
jade.templates["ErrorScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
 var cdn = client.baseUri || '';
buf.push('<div class="message-area text-center splash"><div class="codeBlock"><div class="lobbyTitle">');
var __val__ = resources.title
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</div><div id="line1" class="codeText">');
var __val__ = resources.line1
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</div><div id="line1" class="codeText">');
var __val__ = resources.line2
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</div><div id="line2" class="codeText">');
var __val__ = message
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</div></div>');
if ( isWp)
{
buf.push('<object');
buf.push(attrs({ 'data':('' + (cdn) + 'svg/failSimpleSun.svg'), 'type':('image/svg+xml'), 'width':('460px'), 'height':('460px'), "class": ('codeImg') }, {"data":true,"type":true,"width":true,"height":true}));
buf.push('></object>');
}
else
{
buf.push('<img');
buf.push(attrs({ 'src':('' + (cdn) + 'svg/failSimpleSun.svg'), "class": ('codeImg') }, {"src":true}));
buf.push('/>');
}
buf.push('</div>');
}
return buf.join("");
}
jade.templates["ExpiredScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
 var cdn = client.baseUri || '';
buf.push('<div class="text-center codeScreen"><div class="codeBlock"><div class="codeText">' + escape((interp = text.line1) == null ? '' : interp) + '</div><div class="codeText">' + escape((interp = text.line2) == null ? '' : interp) + '</div></div>');
if ( isWp)
{
buf.push('<object');
buf.push(attrs({ 'data':('' + (cdn) + 'svg/curiousSimpleSun.svg'), 'type':('image/svg+xml'), 'width':('460px'), 'height':('460px'), "class": ('codeImg') }, {"data":true,"type":true,"width":true,"height":true}));
buf.push('></object>');
}
else
{
buf.push('<img');
buf.push(attrs({ 'src':('' + (cdn) + 'svg/curiousSimpleSun.svg'), "class": ('codeImg') }, {"src":true}));
buf.push('/>');
}
buf.push('</div>');
}
return buf.join("");
}
jade.templates["flickr"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="dialog-body"><div id="flickr-items" class="section">');
if ( albums.length)
{
buf.push('<div class="items tile-list"></div>');
}
else
{
buf.push('<div class="message loading">' + escape((interp = resources.messages.loadingFlickr) == null ? '' : interp) + '</div>');
}
buf.push('</div>');
if ( (title))
{
buf.push('<div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div>');
}
buf.push('<div class="command-bar commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["FtuxScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
 var cdn = client.baseUri || '';
buf.push('<div class="ftux-slider"></div><button id="start" class="ftux-login hidden">');
var __val__ = resources.ftux.start
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</button><div class="ftux-slides"><div class="ftux-slide touch-slider-hidden"><img');
buf.push(attrs({ 'src':('' + (cdn) + 'img/ftux/asset1.png'), "class": ('ftux-img') }, {"src":true}));
buf.push('/><div class="message-area text-center"><div class="copy-text"><h2>');
var __val__ = resources.ftux.createNew
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h2><h3>');
var __val__ = resources.ftux.createNewSub
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h3></div></div></div><div class="ftux-slide touch-slider-hidden"><img');
buf.push(attrs({ 'src':('' + (cdn) + 'img/ftux/asset2.png'), "class": ('ftux-img') }, {"src":true}));
buf.push('/><div class="message-area text-center"><div class="copy-text"><h2>');
var __val__ = resources.ftux.pickPhotos
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h2><h3>');
var __val__ = resources.ftux.pickPhotosSub
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h3></div></div></div><div class="ftux-slide touch-slider-hidden"><img');
buf.push(attrs({ 'src':('' + (cdn) + 'img/ftux/asset3.png'), "class": ('ftux-img') }, {"src":true}));
buf.push('/><div class="message-area text-center"><div class="copy-text"><h2>');
var __val__ = resources.ftux.invitePeople
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h2><h3>');
var __val__ = resources.ftux.invitePeopleSub
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h3></div></div></div><div class="ftux-slide touch-slider-hidden"><img');
buf.push(attrs({ 'src':('' + (cdn) + 'img/ftux/asset4.png'), "class": ('ftux-img') }, {"src":true}));
buf.push('/><div class="message-area text-center"><div class="copy-text"><h2>');
var __val__ = resources.ftux.screens
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h2><h3>');
var __val__ = resources.ftux.screensSub
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h3></div></div></div><div class="ftux-slide touch-slider-hidden"><img');
buf.push(attrs({ 'src':('' + (cdn) + 'img/ftux/asset5.png'), "class": ('ftux-img') }, {"src":true}));
buf.push('/><div class="message-area text-center"><div class="copy-text"><h2>');
var __val__ = resources.ftux.longPress
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h2><h3>');
var __val__ = resources.ftux.longPressSub
buf.push(null == __val__ ? "" : __val__);
buf.push('</h3></div></div></div><div class="ftux-slide touch-slider-hidden"><img');
buf.push(attrs({ 'src':('' + (cdn) + 'img/ftux/asset6-10.png'), "class": ('ftux-img') }, {"src":true}));
buf.push('/><div class="message-area text-center"><div class="copy-text"><h2>');
var __val__ = resources.ftux.ready
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h2><h3>');
var __val__ = resources.ftux.readySub
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h3></div></div></div></div>');
}
return buf.join("");
}
jade.templates["howToAirPlay"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="howToPopUp"><div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = data.resources.title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div><label class="howToTitle">');
var __val__ = data.resources.text
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><div class="howToIntro"><ol>');
// iterate data.resources.bullets
;(function(){
  if ('number' == typeof data.resources.bullets.length) {
    for (var $index = 0, $$l = data.resources.bullets.length; $index < $$l; $index++) {
      var bullet = data.resources.bullets[$index];

buf.push('<li>');
var __val__ = bullet
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</li>');
    }
  } else {
    for (var $index in data.resources.bullets) {
      var bullet = data.resources.bullets[$index];

buf.push('<li>');
var __val__ = bullet
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</li>');
   }
  }
}).call(this);

buf.push('</ol><input');
buf.push(attrs({ 'type':("submit"), 'value':('' + (data.resources.button) + ''), "class": ('howToButton') }, {"type":true,"value":true}));
buf.push('/></div><label class="howToFooter underline">');
var __val__ = data.resources.subtext
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label></div>');
}
return buf.join("");
}
jade.templates["howToXBox"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="howToPopUp"><div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = data.resources.title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div><label class="howToTitle">');
var __val__ = data.resources.text
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><div class="howToIntro"><ol>');
// iterate data.resources.bullets
;(function(){
  if ('number' == typeof data.resources.bullets.length) {
    for (var $index = 0, $$l = data.resources.bullets.length; $index < $$l; $index++) {
      var bullet = data.resources.bullets[$index];

buf.push('<li>');
var __val__ = bullet
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</li>');
    }
  } else {
    for (var $index in data.resources.bullets) {
      var bullet = data.resources.bullets[$index];

buf.push('<li>');
var __val__ = bullet
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</li>');
   }
  }
}).call(this);

buf.push('</ol><input');
buf.push(attrs({ 'type':("submit"), 'value':('' + (data.resources.button) + ''), "class": ('howToButton') }, {"type":true,"value":true}));
buf.push('/></div><label class="howToFooter underline">');
var __val__ = data.resources.subtext
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label></div>');
}
return buf.join("");
}
jade.templates["instagram"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="dialog-body"><!--<div class="search-box inputbox"><i class="glyphicon glyphicon-search"></i><form role="form"><div class="form-group"><input');
buf.push(attrs({ 'id':('search'), 'type':('search'), 'placeholder':('' + (resources.forms.searchPlaceholder) + ''), 'value':('' + (searchTerm) + ''), "class": ('form-control') }, {"type":true,"placeholder":true,"value":true}));
buf.push('/></div></form></div>--><div id="instagram-items" class="section">');
if ( albums.length)
{
buf.push('<div class="items tile-list"></div>');
}
else
{
buf.push('<div class="message loading"> ' + escape((interp = resources.messages.loadingInstagram) == null ? '' : interp) + '</div>');
}
buf.push('</div>');
if ( (title))
{
buf.push('<div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div>');
}
buf.push('<div class="command-bar commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["LoadingScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
 var cdn = client.baseUri || '';
buf.push('<div class="message-area text-center splash"><h1>xim</h1><h3 id="message">');
var __val__ = message
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h3><div class="loading"><i class="dot"></i><i class="dot"></i><i class="dot"></i><i class="dot"></i><i class="dot"></i></div><img');
buf.push(attrs({ 'src':('' + (cdn) + 'img/msft-logo.png'), "class": ('msft-logo') }, {"src":true}));
buf.push('/></div>');
}
return buf.join("");
}
jade.templates["LobbyScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
 var cdn = client.baseUri || '';
buf.push('<div class="message-area text-center splash">');
if ( isWp)
{
buf.push('<object');
buf.push(attrs({ 'data':('' + (cdn) + 'svg/programmerSun.svg'), 'type':('image/svg+xml'), 'width':('460px'), 'height':('460px'), "class": ('lobbyImg') }, {"data":true,"type":true,"width":true,"height":true}));
buf.push('></object>');
}
else
{
buf.push('<img');
buf.push(attrs({ 'src':('' + (cdn) + 'svg/programmerSun.svg'), "class": ('lobbyImg') }, {"src":true}));
buf.push('/>');
}
buf.push('<div class="lobbyBlock"><div class="lobbyTitle">');
var __val__ = resources.title
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</div><div id="line1" class="lobbyText"></div><div id="line2" class="lobbyText"></div><div id="line3" class="lobbyText"></div></div></div>');
}
return buf.join("");
}
jade.templates["login"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="dialog-body"><div class="section"><a id="login-facebook" href="." class="btn btn-primary btn-large login-btn">' + escape((interp = resources.labels.login.facebook) == null ? '' : interp) + '</a></div>');
if ( (title))
{
buf.push('<div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div>');
}
buf.push('<div class="command-bar commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["loginChoiceScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="dialog-body"><div class="loginPhone"><div class="dialogIntro"><label>');
var __val__ = resources.intro1
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><br/><label class="loginChoicePhone">');
var __val__ = phone
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><br/><label>');
var __val__ = resources.intro2
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><label>');
var __val__ = resources.intro3
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label></div><div class="loginChoiceInputbox"><form onsubmit="return false" action="#"><input');
buf.push(attrs({ 'type':("submit"), 'value':('' + (resources.sms) + ''), "class": ('loginButtonSMS') }, {"type":true,"value":true}));
buf.push('/><input');
buf.push(attrs({ 'type':("submit"), 'value':('' + (resources.phone) + ''), "class": ('loginButtonPhone') }, {"type":true,"value":true}));
buf.push('/></form></div><div class="loginChangePhone"><label class="underline">');
var __val__ = resources.sub1
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label></div><div style="display:none" class="loginChoiceWait"><i class="dot"></i><i class="dot"></i><i class="dot"></i><i class="dot"></i><i class="dot"></i></div></div>');
if ( (title))
{
buf.push('<div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div>');
}
buf.push('<div class="command-bar commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["loginPhoneScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="dialog-body"><div class="loginPhone"><div class="dialogIntro"><label>');
var __val__ = resources.intro1
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><label>');
var __val__ = resources.intro2
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><label>');
var __val__ = resources.intro3
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label></div><div class="loginInputbox"><form onsubmit="return false" action="#"><div class="loginPhoneBox"> <input type="submit" value="" class="loginButtonCountry"/><input');
buf.push(attrs({ 'type':("tel"), 'name':("phone"), 'placeholder':('' + (resources.inputCue) + ''), "class": ('loginInputTextPhone') }, {"type":true,"name":true,"placeholder":true}));
buf.push('/><label class="loginLabelCountry"></label></div><label class="loginErrorPhone"></label><input');
buf.push(attrs({ 'type':("submit"), 'value':('' + (resources.next) + ''), "class": ('loginButton') }, {"type":true,"value":true}));
buf.push('/></form></div><div class="loginPhoneFooter"><label>');
var __val__ = resources.sub1
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><label><a id="terms" class="loginLink">' + escape((interp = resources.termsOfUse) == null ? '' : interp) + '</a>&nbspand&nbsp<a id="privacy" class="loginLink">' + escape((interp = resources.privacy) == null ? '' : interp) + '</a></label></div><div style="display:none" class="loginWait"><i class="dot"></i><i class="dot"></i><i class="dot"></i><i class="dot"></i><i class="dot"></i></div></div><div class="loginCountryPopUp"></div>');
if ( (title))
{
buf.push('<div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div>');
}
buf.push('<div class="command-bar commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["loginVerifyScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="dialog-body"><div class="loginPhone"><div class="dialogIntro"><label>');
var __val__ = resources.intro1
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><label>');
var __val__ = resources.intro2
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label></div><div class="loginInputbox"><form onsubmit="return false" action="#"><input');
buf.push(attrs({ 'type':("tel"), 'name':("code"), 'placeholder':('' + (resources.inputCue) + ''), "class": ('ximformcontrol') + ' ' + ('dialogInputText') }, {"type":true,"name":true,"placeholder":true}));
buf.push('/><label class="dialogError"></label><input');
buf.push(attrs({ 'type':("submit"), 'value':('' + (resources.next) + ''), "class": ('loginButton') }, {"type":true,"value":true}));
buf.push('/></form></div><div class="loginResentSMS"><label class="underline">');
var __val__ = resources.sub1
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><br/><label class="underline">');
var __val__ = resources.sub2
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label></div><div style="display:none" class="loginWait"><i class="dot"></i><i class="dot"></i><i class="dot"></i><i class="dot"></i><i class="dot"></i></div></div>');
if ( (title))
{
buf.push('<div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div>');
}
buf.push('<div class="command-bar commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["logScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="dialog-body"><div class="content"><div class="form-group"><label for="loggerLevel" class="logformlabel">');
var __val__ = resources.forms.loggerLevel
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><select id="loggerLevel" class="form-control logformcontrol"><option value="0">info</option><option value="1">debug</option><option value="2">warning</option><option value="3">error</option></select></div><div class="section log"></div></div>');
if ( (title))
{
buf.push('<div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div>');
}
buf.push('<div class="command-bar commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["map"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<!--bing map key --><div class="console hidden">this is the console</div><div');
buf.push(attrs({ 'id':('map'), 'style':("" + (positionStyle) + "") }, {"style":true}));
buf.push('></div>');
}
return buf.join("");
}
jade.templates["messageBubble"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
 var cdn = client.baseUri || '';
 var newStyle = ""
 if (animate) newStyle = " messageNew";
buf.push('<div');
buf.push(attrs({ "class": ('messageBlock') + ' ' + (newStyle) }, {"class":true}));
buf.push('>');
 if (data.thumbnailUrl)
{
buf.push('<div');
buf.push(attrs({ 'style':("background-image:url('" + (data.thumbnailUrl) + "');"), "class": ('messageFace') + ' ' + ("messageFace" + (data.boxStyle) + "") }, {"class":true,"style":true}));
buf.push('></div>');
}
else
{
buf.push('<div');
buf.push(attrs({ 'style':("background-color:" + (data.profileColor) + ";"), "class": ('messageFace') + ' ' + ("messageFace" + (data.boxStyle) + "") }, {"class":true,"style":true}));
buf.push('><figcaption class="messageLabel">' + escape((interp = data.initials) == null ? '' : interp) + '</figcaption></div>');
}
buf.push('<div');
buf.push(attrs({ 'style':("border-color:" + (data.boxColor) + "; background-color:" + (data.backColor) + ";"), "class": ('messageBoxHolder') + ' ' + ("messageBox" + (data.boxStyle) + "") }, {"class":true,"style":true}));
buf.push('><div class="messageText">');
var __val__ = data.text
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</div>');
 if (data.imageUrl && data.text)
{
buf.push('<div class="messageText">');
var __val__ = "&nbsp;"
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</div>');
}
buf.push('<div');
buf.push(attrs({ 'style':("border-color:transparent " + (data.boxColor) + ""), "class": ("messageNub" + (data.boxStyle) + "") }, {"class":true,"style":true}));
buf.push('></div></div>');
 if (data.imageUrl)
{
buf.push('<div');
buf.push(attrs({ "class": ('messageImageHolder') + ' ' + ("messageImageHolder" + (data.boxStyle) + "") }, {"class":true}));
buf.push('><div');
buf.push(attrs({ 'style':("background-image:url('" + (data.imageUrl) + "');"), "class": ('messageImage') + ' ' + ("messageImage" + (data.boxStyle) + "") }, {"class":true,"style":true}));
buf.push('></div></div>');
}
 if (data.sunUrl)
{
buf.push('<img');
buf.push(attrs({ 'src':("" + (cdn) + "" + (data.sunUrl) + ""), "class": ('messageSun') + ' ' + ("messageSun" + (data.boxStyle) + "") }, {"class":true,"src":true}));
buf.push('/>');
}
buf.push('</div>');
}
return buf.join("");
}
jade.templates["messagePopUp"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="messagePopUp"><div class="messagePopUpTitle">');
var __val__ = "Messages"
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</div><div class="messagePopUpButtonRow"><input');
buf.push(attrs({ 'type':("submit"), 'value':('' + (data.resources.button) + ''), "class": ('messagePopUpButton') }, {"type":true,"value":true}));
buf.push('/></div><form class="messagePopUpInput"><input type="submit" class="hidden"/><textarea');
buf.push(attrs({ 'rows':("2"), 'placeholder':('' + (data.resources.placeholder) + ''), "class": ('form-control') + ' ' + ('messagePopUpInputForm') }, {"rows":true,"placeholder":true}));
buf.push('></textarea></form><div class="messagePopUpContent"></div><div class="profilePopUpContent"></div></div>');
}
return buf.join("");
}
jade.templates["people"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="dialog-body"><div id="peoplePicker"></div>');
if ( (title))
{
buf.push('<div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div>');
}
buf.push('<div class="command-bar commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["peoplePicker"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
 var cdn = client.baseUri || '';
buf.push('<div class="inputbox"><form onsubmit="return false" action="#"><input');
buf.push(attrs({ 'id':('searchTerm'), 'type':('search'), 'title':('Search'), 'placeholder':('' + (resources.forms.search) + ''), "class": ('ximformcontrol') }, {"type":true,"title":true,"placeholder":true}));
buf.push('/></form></div><div class="peoplepickerscrollpage"><div class="searchresults"><div class="searchresultslist"></div><div class="searchmessage"></div></div><div class="nearby"><div><div class="peoplelayout"></div><div class="commandBarPadding"></div></div><div class="peoplepickerNoContacts"><i class="glyphiconXIM glyphiconXIM-helloMessage nocontactsglyph"></i><div class="noximsTitle1">');
var __val__ = messages.peoplepicker_Title1
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</div>');
if ( isWp)
{
buf.push('<object');
buf.push(attrs({ 'data':('' + (cdn) + 'svg/lonelyClouds.svg'), 'type':('image/svg+xml'), "class": ('peoplePickerLonely') }, {"data":true,"type":true}));
buf.push('></object>');
}
else
{
buf.push('<img');
buf.push(attrs({ 'src':('' + (cdn) + 'svg/lonelyClouds.svg'), "class": ('peoplePickerLonely') }, {"src":true}));
buf.push('/>');
}
buf.push('</div></div></div>');
}
return buf.join("");
}
jade.templates["peoplePopUp"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="peoplePopUp"><div class="peoplePopUpCount"></div><div class="peoplePopProfiles"></div><div class="peoplePopUpButtonRow"><input');
buf.push(attrs({ 'type':("submit"), 'value':('' + (data.resources.addpeople) + ''), "class": ('peoplePopUpButtonAdd') }, {"type":true,"value":true}));
buf.push('/></div><div class="profilePopUpContent"></div></div>');
}
return buf.join("");
}
jade.templates["person"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<figure');
buf.push(attrs({ 'style':("left:" + (position.left) + "px; top:" + (position.top) + "px; width:" + (position.width) + "px; height:" + (position.height) + "px"), 'id':("" + (id) + ""), "class": ('personFigure') }, {"style":true,"id":true}));
buf.push('>');
 var imageStyle = 'width:'+ position.imageSize +'px; height:' + position.imageSize + 'px;'
if ( (initials))
{
if ( (isSelected))
{
buf.push('<div');
buf.push(attrs({ 'style':(imageStyle), "class": ('personImage') + ' ' + ('personImageSelected') }, {"style":true}));
buf.push('><img');
buf.push(attrs({ 'src':('' + (image) + ''), "class": ('personImageImg') }, {"src":true}));
buf.push('/></div>');
}
else
{
buf.push('<div');
buf.push(attrs({ 'style':(imageStyle), "class": ('personImage') }, {"style":true}));
buf.push('><img');
buf.push(attrs({ 'src':('' + (image) + ''), "class": ('personImageImg') }, {"src":true}));
buf.push('/></div>');
}
if ( (position.imageSize >= 100))
{
buf.push('<figcaption class="personLabel">' + escape((interp = initials) == null ? '' : interp) + '</figcaption>');
}
else
{
buf.push('<figcaption class="personLabel personLabelSmall">' + escape((interp = initials) == null ? '' : interp) + '</figcaption>');
}
if ( (useCaption))
{
buf.push('<figcaption');
buf.push(attrs({ 'style':('color:' + (captionColor) + ''), "class": ('personCaptionOffset') }, {"style":true}));
buf.push('>' + escape((interp = caption1) == null ? '' : interp) + '</figcaption>');
}
}
else
{
if ( (isSelected))
{
buf.push('<div');
buf.push(attrs({ 'style':(imageStyle), "class": ('personImage') + ' ' + ('personImageSelected') }, {"style":true}));
buf.push('><img');
buf.push(attrs({ 'src':('' + (image) + ''), "class": ('personImageImg') }, {"src":true}));
buf.push('/></div>');
}
else
{
buf.push('<div');
buf.push(attrs({ 'style':(imageStyle), "class": ('personImage') }, {"style":true}));
buf.push('><img');
buf.push(attrs({ 'src':('' + (image) + ''), "class": ('personImageImg') }, {"src":true}));
buf.push('/></div>');
}
if ( (useCaption))
{
buf.push('<figcaption');
buf.push(attrs({ 'style':('color:' + (captionColor) + ''), "class": ('personCaption') }, {"style":true}));
buf.push('>' + escape((interp = caption1) == null ? '' : interp) + '</figcaption>');
}
}
if ( (isSelected) )
{
buf.push('<figcaption class="figuremark checkmark"></figcaption>');
}
else
{
buf.push('<figcaption class="figuremark"></figcaption>');
}
buf.push('</figure>');
}
return buf.join("");
}
jade.templates["photoPopUp"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="photoPopUp"><div class="photoPopUpCount"></div><div class="photoPopUpButtonRow"><input');
buf.push(attrs({ 'type':("submit"), 'value':('' + (data.resources.addphotos) + ''), "class": ('photoPopUpButtonAdd') }, {"type":true,"value":true}));
buf.push('/><input');
buf.push(attrs({ 'type':("submit"), 'value':('' + (data.resources.takephoto) + ''), "class": ('photoPopUpButtonTake') }, {"type":true,"value":true}));
buf.push('/></div></div>');
}
return buf.join("");
}
jade.templates["picker"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="dialog-body"><div id="photos-connections" class="section">');
if ( tiles.length        )
{
buf.push('<div class="items tile-list flush">');
// iterate tiles
;(function(){
  if ('number' == typeof tiles.length) {
    for (var $index = 0, $$l = tiles.length; $index < $$l; $index++) {
      var tile = tiles[$index];

buf.push('<a');
buf.push(attrs({ 'href':('#/' + (tile.id) + ''), 'target':('_self'), "class": ('tile-link') }, {"href":true,"target":true}));
buf.push('><div class="tile picker-source"><div');
buf.push(attrs({ 'style':('background-image:url("' + (tile.thumbnail) + '")'), "class": ('tile-image') }, {"style":true}));
buf.push('></div><div class="tile-title picker-source"><h5>');
var __val__ = tile.title
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h5></div><i class="glyphiconXIM glyphiconXIM-rightArrow"></i></div></a>');
    }
  } else {
    for (var $index in tiles) {
      var tile = tiles[$index];

buf.push('<a');
buf.push(attrs({ 'href':('#/' + (tile.id) + ''), 'target':('_self'), "class": ('tile-link') }, {"href":true,"target":true}));
buf.push('><div class="tile picker-source"><div');
buf.push(attrs({ 'style':('background-image:url("' + (tile.thumbnail) + '")'), "class": ('tile-image') }, {"style":true}));
buf.push('></div><div class="tile-title picker-source"><h5>');
var __val__ = tile.title
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h5></div><i class="glyphiconXIM glyphiconXIM-rightArrow"></i></div></a>');
   }
  }
}).call(this);

buf.push('</div>');
}
else
{
buf.push('<div class="message">no photo source configured</div>');
}
buf.push('</div>');
if ( (title))
{
buf.push('<div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div>');
}
buf.push('<div class="command-bar commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["profile"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="navbar navbar-inverse navbar-static-top"><div class="navbar-inner"><a id="back" href="#/" class="ios-only"></a><a href="#/" class="brand">People</a></div></div><div class="page"><div id="profile-bio"><img');
buf.push(attrs({ 'src':('' + (thumbnailUrl) + ''), 'alt':('' + (displayName) + ''), "class": ('profile-image') }, {"src":true,"alt":true}));
buf.push('/><h3 class="profile-name">' + escape((interp = displayName) == null ? '' : interp) + '</h3></div><div id="profile-action"><a id="photo-cast-action" href="#/albums" class="button navigation">Photo Cast</a></div></div>');
}
return buf.join("");
}
jade.templates["profileEditor"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="dialog-body">');
 var cdn = client.baseUri || '';
buf.push('<div class="loginPhone"><div class="dialogIntro"><label>');
var __val__ = resources.intro1
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><label>');
var __val__ = resources.intro2
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label></div><div class="loginProfileInputbox"><form onsubmit="return false" action="#" class="loginInputProfile"><div class="loginThumbnail"><div style="display:none" class="loginThumbnailInitial"><div');
buf.push(attrs({ 'style':('background-color:' + (backcolor) + ''), "class": ('loginProfileImage') }, {"style":true}));
buf.push('><figcaption class="loginProfileLabel"></figcaption></div><div class="loginEditMark"></div></div><div class="loginThumbnailImage"><img');
buf.push(attrs({ 'src':('' + (thumbnail) + ''), "class": ('loginProfileFigure') }, {"src":true}));
buf.push('/><div class="loginAddMark"></div></div></div>');
if ( (profile))
{
buf.push('<div class="loginInputWrap"><input');
buf.push(attrs({ 'type':("text"), 'name':("firstname"), 'placeholder':('' + (resources.inputCue1) + ''), 'value':(firstName), "class": ('ximformcontrol') + ' ' + ('dialogInputText') }, {"type":true,"name":true,"placeholder":true,"value":true}));
buf.push('/><label id="firstName" class="loginFirstNameError"></label></div><div class="loginInputWrap"><input');
buf.push(attrs({ 'type':("text"), 'name':("lastname"), 'placeholder':('' + (resources.inputCue2) + ''), 'value':(lastName), "class": ('ximformcontrol') + ' ' + ('dialogInputText') }, {"type":true,"name":true,"placeholder":true,"value":true}));
buf.push('/><label id="lastName" class="loginLastNameError"></label></div><input');
buf.push(attrs({ 'type':("submit"), 'value':('' + (resources.edit) + ''), "class": ('profileButton') }, {"type":true,"value":true}));
buf.push('/>');
}
else
{
buf.push('<div class="loginInputWrap"><input');
buf.push(attrs({ 'type':("text"), 'name':("firstname"), 'placeholder':('' + (resources.inputCue1) + ''), "class": ('ximformcontrol') + ' ' + ('dialogInputText') }, {"type":true,"name":true,"placeholder":true}));
buf.push('/><label id="firstName" class="loginFirstNameError"></label></div><div class="loginInputWrap"><input');
buf.push(attrs({ 'type':("text"), 'name':("lastname"), 'placeholder':('' + (resources.inputCue2) + ''), "class": ('ximformcontrol') + ' ' + ('dialogInputText') }, {"type":true,"name":true,"placeholder":true}));
buf.push('/><label id="lastName" class="loginLastNameError"></label></div><input');
buf.push(attrs({ 'type':("submit"), 'value':('' + (resources.next) + ''), "class": ('profileButton') }, {"type":true,"value":true}));
buf.push('/>');
}
buf.push('</form></div><div class="loginProfileAutofill"><div class="loginAutofillBreak"></div><label class="loginAutofillOr">');
var __val__ = resources.sub1
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><img');
buf.push(attrs({ 'src':('' + (cdn) + 'img/facebook-logo.png'), "class": ('loginAutofillImage') }, {"src":true}));
buf.push('/><label class="loginAutofillText">');
var __val__ = resources.fb
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label></div><div style="display:none" class="loginProfileWait"><i class="dot"></i><i class="dot"></i><i class="dot"></i><i class="dot"></i><i class="dot"></i></div></div>');
if ( (title))
{
buf.push('<div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div>');
}
buf.push('<div class="command-bar commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["profilePopUp"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="profilePopUp"><div class="profileMask"></div>');
if ( (data.initials))
{
buf.push('<div class="profileInitial"><div');
buf.push(attrs({ 'style':('background-color:' + (data.color) + ''), "class": ('personImage') }, {"style":true}));
buf.push('></div><figcaption class="personLabel">' + escape((interp = data.initials) == null ? '' : interp) + '</figcaption></div>');
}
else
{
buf.push('<img');
buf.push(attrs({ 'src':('' + (data.thumbnailUrl) + ''), "class": ('profileFigure') }, {"src":true}));
buf.push('/>');
}
buf.push('<div class="profileNameFirst">' + escape((interp = data.firstName) == null ? '' : interp) + '</div><div class="profileNameLast">' + escape((interp = data.lastName) == null ? '' : interp) + '</div></div>');
}
return buf.join("");
}
jade.templates["search"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="navbar navbar-inverse navbar-static-top"><div class="navbar-inner"><a href="#" class="brand">Search</a></div></div><div class="page"><div class="items tile-list"></div></div>');
}
return buf.join("");
}
jade.templates["searchResult"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div');
buf.push(attrs({ 'id':("" + (id) + ""), "class": ('searchResultBox') }, {"id":true}));
buf.push('><div class="searchResultIcon"><div class="searchInitials"><div');
buf.push(attrs({ 'style':('background-color:' + (backgroundColor) + ''), "class": ('searchResultImage') }, {"style":true}));
buf.push('></div><div class="searchResultInitial">' + escape((interp = initials) == null ? '' : interp) + '</div></div><div class="searchImage"><img');
buf.push(attrs({ 'src':('' + (image) + ''), "class": ('searchResultImage') }, {"src":true}));
buf.push('/></div></div><div class="searchResultName">' + escape((interp = name) == null ? '' : interp) + '</div><div class="searchResultContact">' + escape((interp = contact) == null ? '' : interp) + '</div><div style="display: none" class="searchResultPicker">');
if ( (email))
{
buf.push('<div id="email" class="searchResultChoice">' + escape((interp = email) == null ? '' : interp) + '</div>');
}
if ( (phone))
{
buf.push('<div id="phone" class="searchResultChoice">' + escape((interp = phone) == null ? '' : interp) + '</div>');
}
buf.push('</div>');
if ( (showChoice))
{
buf.push('<i class="searchResultGlyph glyphiconXIM glyphiconXIM-unselectedOption"></i>');
}
buf.push('</div>');
}
return buf.join("");
}
jade.templates["settingsScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="dialog-body"><div class="content"><div class="confirmationPopUpContent"></div><div class="userSettings">');
if ( (showPushSettings))
{
buf.push('<label class="settingsGroup">Push Notifications</label><input type="submit" value="Receive Notifications" class="settingsTogglerButton notificationsButton"/><div class="settingsToggler settingsOn"></div>');
}
buf.push('<label class="settingsGroup">Account</label><input type="submit" value="Deauthorize Device" class="settingsButton unlinkButton"/><input type="submit" value="Delete Account" class="settingsButton deleteAccountButton"/><label class="applicationVersion">Version ' + escape((interp = consts.buildFlavor) == null ? '' : interp) + ' ' + escape((interp = version) == null ? '' : interp) + '</label></div></div>');
if ( (title))
{
buf.push('<div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div>');
}
buf.push('<div class="command-bar commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["skydrive"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="dialog-body"><div id="skydrive-items" class="section">');
if ( items.length)
{
buf.push('<div class="items tile-list"></div>');
}
else
{
buf.push('<div class="message loading">' + escape((interp = resources.messages.loadingOneDrive) == null ? '' : interp) + '</div>');
}
buf.push('</div>');
if ( (title))
{
buf.push('<div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div>');
}
buf.push('<div class="command-bar commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["StartingCastScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
 var cdn = client.baseUri || '';
buf.push('<div class="loading-graphics">');
if ( isWp)
{
buf.push('<object');
buf.push(attrs({ 'data':('' + (cdn) + 'svg/emptySun.svg'), 'type':('image/svg+xml'), 'width':('460px'), 'height':('460px'), "class": ('loading-img') + ' ' + ('backdrop') }, {"data":true,"type":true,"width":true,"height":true}));
buf.push('></object><object');
buf.push(attrs({ 'data':('' + (cdn) + 'svg/emptySunOpen.svg'), 'type':('image/svg+xml'), 'width':('460px'), 'height':('460px'), "class": ('loading-img') + ' ' + ('blink') }, {"data":true,"type":true,"width":true,"height":true}));
buf.push('></object><object');
buf.push(attrs({ 'data':('' + (cdn) + 'svg/emptySunClosed.svg'), 'type':('image/svg+xml'), 'width':('460px'), 'height':('460px'), "class": ('loading-img') }, {"data":true,"type":true,"width":true,"height":true}));
buf.push('></object>');
}
else
{
buf.push('<img');
buf.push(attrs({ 'src':('' + (cdn) + 'svg/emptySun.svg'), "class": ('loading-img') }, {"src":true}));
buf.push('/><img');
buf.push(attrs({ 'src':('' + (cdn) + 'svg/emptySunOpen.svg'), "class": ('loading-img') + ' ' + ('blink') }, {"src":true}));
buf.push('/><img');
buf.push(attrs({ 'src':('' + (cdn) + 'svg/emptySunClosed.svg'), "class": ('loading-img') }, {"src":true}));
buf.push('/>');
}
buf.push('</div><div class="loading-text text-center"><h3 id="message">');
var __val__ = message
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h3></div>');
}
return buf.join("");
}
jade.templates["thumbnailPopUp"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="thumbnailPopUp"><div class="thumbnailMask"></div>');
if ( (data.thumbnail))
{
buf.push('<img');
buf.push(attrs({ 'src':('' + (data.thumbnail) + ''), "class": ('loginProfileFigure') + ' ' + ('thumbnailMargin') }, {"src":true}));
buf.push('/>');
}
else
{
buf.push('<div class="loginProfileInitial"><div');
buf.push(attrs({ 'style':('background-color:' + (data.color) + ''), "class": ('loginProfileImage') + ' ' + ('thumbnailMargin') }, {"style":true}));
buf.push('><figcaption class="loginProfileLabel">' + escape((interp = data.initials) == null ? '' : interp) + '    </figcaption></div></div>');
}
buf.push('<ul class="thumbnailPopUpMenu"><li id="thumbnailPopUpPhoto" class="thumbnailPopUpLi">' + escape((interp = data.resources.photo) == null ? '' : interp) + '</li><li id="thumbnailPopUpInitials" class="thumbnailPopUpLi">' + escape((interp = data.resources.initials) == null ? '' : interp) + '</li></ul></div>');
}
return buf.join("");
}
jade.templates["timelineScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
 var cdn = client.baseUri || '';
buf.push('<div class="timelineContent"><div class="serverMessagePopUpContent"></div><div class="timelineGroups"><div class="timelineGroupsXims loading"></div><div class="commandBarPadding"></div><div class="timelineNoXims hidden"><i class="glyphiconXIM glyphiconXIM-dotMessage noximsglyph"></i><div class="noximsTitle1">' + escape((interp = messages.timeline_Title1) == null ? '' : interp) + '\n' + escape((interp = messages.timeline_Title2) == null ? '' : interp) + '</div>');
if ( isWp)
{
buf.push('<object');
buf.push(attrs({ 'data':('' + (cdn) + 'svg/lonelyClouds.svg'), 'type':('image/svg+xml'), "class": ('timelineLonely') }, {"data":true,"type":true}));
buf.push('></object>');
}
else
{
buf.push('<img');
buf.push(attrs({ 'src':('' + (cdn) + 'svg/lonelyClouds.svg'), "class": ('timelineLonely') }, {"src":true}));
buf.push('/>');
}
buf.push('</div></div></div>');
}
return buf.join("");
}
jade.templates["tokenScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="navbar navbar-inverse navbar-static-top"><div class="navbar-inner"><a id="back" href="#/" class="ios-only"></a></div></div><div class="page"><Live>ID Refresh Token</Live><div class="tokenBox"><select id="tokenSelect"><option value="fb">facebook</option><option value="live">liveRefresh</option><option value="dropbox">dropbox</option><option value="instagram">instagram</option></select><textarea style="width:96%;height:400px" class="token">' + escape((interp = token) == null ? '' : interp) + '</textarea></div><div class="actions"><button style="height:40px;width:120px" class="set">set</button></div></div>');
}
return buf.join("");
}
jade.templates["upsellBar"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="chromeTopBase"><label class="upsellTitle">xim</label><label class="upsellDescription">');
var __val__ = text.description
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><label class="upsellLink">');
var __val__ = text.link
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label></div>');
}
return buf.join("");
}
jade.templates["userProfile"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="navbar navbar-inverse navbar-static-top"><div class="navbar-inner"><a href="#" class="brand">People</a></div></div><div class="page"><div class="profile-card"><div');
buf.push(attrs({ 'style':("background-image:url('" + (user.pic) + "');"), "class": ('pull-left') + ' ' + ('image-tile') }, {"style":true}));
buf.push('></div><div class="profile-body"><h4>');
var __val__ = user.name
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h4></div></div></div>');
}
return buf.join("");
}
jade.templates["YouTubeDialog"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="dialog-body"><div class="search-box inputbox"><i class="glyphicon glyphicon-search"></i><form role="form"><div class="form-group"><input');
buf.push(attrs({ 'id':('search'), 'type':('search'), 'placeholder':('' + (resources.forms.searchPlaceholder) + ''), 'value':('' + (searchTerm) + ''), "class": ('form-control') }, {"type":true,"placeholder":true,"value":true}));
buf.push('/></div></form></div><div id="youtube-items" class="section">');
if ( items.length)
{
buf.push('<div class="items tile-list"></div>');
}
buf.push('<div class="message"></div></div>');
if ( (title))
{
buf.push('<div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div>');
}
buf.push('<div class="command-bar commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["YouTubeScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="dialog-body"><div class="search-box inputbox"><i class="glyphicon glyphicon-search"></i><form role="form"><div class="form-group"><input');
buf.push(attrs({ 'id':('search'), 'type':('search'), 'placeholder':('' + (resources.forms.searchPlaceholder) + ''), 'value':('' + (searchTerm) + ''), "class": ('form-control') }, {"type":true,"placeholder":true,"value":true}));
buf.push('/></div></form></div><div id="youtube-items" class="section">');
if ( items.length)
{
buf.push('<div class="items tile-list"></div>');
}
buf.push('<div class="message"></div></div>');
if ( (title))
{
buf.push('<div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div>');
}
buf.push('<div class="command-bar commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["YouTubeTile"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
}
return buf.join("");
}
jade.templates["CastScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="castContent"></div><div class="debugInfo"></div><div class="debugEffect"></div><div class="castTopBar"></div><div class="castCommandBar"></div><div class="castAltBar"></div><div class="castPeoplePickerCommandBar"></div><div class="bubbleContent"></div><div class="hoverMenuContent"></div><div class="castMessagePopUp"></div><div class="castPeoplePicker hidden"></div><div class="castPeoplePopUp"></div><div class="castPhotoPopUp"></div>');
}
return buf.join("");
}
jade.templates["CodeScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
 var cdn = client.baseUri || '';
buf.push('<div class="text-center codeScreen"><div class="codeBlock"><div class="codeText">' + escape((interp = text.line1) == null ? '' : interp) + '</div><div class="codeText">' + escape((interp = text.line2) == null ? '' : interp) + '</div><div class="codeTextPin">' + escape((interp = pin) == null ? '' : interp) + '</div><div class="codeText">' + escape((interp = text.line3) == null ? '' : interp) + '</div><div class="codeSmall">' + escape((interp = text.line4) == null ? '' : interp) + '</div><div class="codeLink">' + escape((interp = text.link) == null ? '' : interp) + '</div></div>');
if ( isWp)
{
buf.push('<object');
buf.push(attrs({ 'data':('' + (cdn) + 'svg/curiousSimpleSun.svg'), 'type':('image/svg+xml'), 'width':('460px'), 'height':('460px'), "class": ('codeImg') }, {"data":true,"type":true,"width":true,"height":true}));
buf.push('></object>');
}
else
{
buf.push('<img');
buf.push(attrs({ 'src':('' + (cdn) + 'svg/curiousSimpleSun.svg'), "class": ('codeImg') }, {"src":true}));
buf.push('/>');
}
buf.push('</div>');
}
return buf.join("");
}
jade.templates["DisplayScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="message-area text-center"><h2 id="message">');
var __val__ = message
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h2></div>');
}
return buf.join("");
}
jade.templates["ErrorScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
 var cdn = client.baseUri || '';
buf.push('<div class="message-area text-center splash"><div class="codeBlock"><div class="lobbyTitle">');
var __val__ = resources.title
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</div><div id="line1" class="codeText">');
var __val__ = resources.line1
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</div><div id="line1" class="codeText">');
var __val__ = resources.line2
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</div><div id="line2" class="codeText">');
var __val__ = message
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</div></div>');
if ( isWp)
{
buf.push('<object');
buf.push(attrs({ 'data':('' + (cdn) + 'svg/failSimpleSun.svg'), 'type':('image/svg+xml'), 'width':('460px'), 'height':('460px'), "class": ('codeImg') }, {"data":true,"type":true,"width":true,"height":true}));
buf.push('></object>');
}
else
{
buf.push('<img');
buf.push(attrs({ 'src':('' + (cdn) + 'svg/failSimpleSun.svg'), "class": ('codeImg') }, {"src":true}));
buf.push('/>');
}
buf.push('</div>');
}
return buf.join("");
}
jade.templates["ExpiredScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
 var cdn = client.baseUri || '';
buf.push('<div class="text-center codeScreen"><div class="codeBlock"><div class="codeText">' + escape((interp = text.line1) == null ? '' : interp) + '</div><div class="codeText">' + escape((interp = text.line2) == null ? '' : interp) + '</div></div>');
if ( isWp)
{
buf.push('<object');
buf.push(attrs({ 'data':('' + (cdn) + 'svg/curiousSimpleSun.svg'), 'type':('image/svg+xml'), 'width':('460px'), 'height':('460px'), "class": ('codeImg') }, {"data":true,"type":true,"width":true,"height":true}));
buf.push('></object>');
}
else
{
buf.push('<img');
buf.push(attrs({ 'src':('' + (cdn) + 'svg/curiousSimpleSun.svg'), "class": ('codeImg') }, {"src":true}));
buf.push('/>');
}
buf.push('</div>');
}
return buf.join("");
}
jade.templates["FtuxScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
 var cdn = client.baseUri || '';
buf.push('<div class="ftux-slider"></div><button id="start" class="ftux-login hidden">');
var __val__ = resources.ftux.start
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</button><div class="ftux-slides"><div class="ftux-slide touch-slider-hidden"><img');
buf.push(attrs({ 'src':('' + (cdn) + 'img/ftux/asset1.png'), "class": ('ftux-img') }, {"src":true}));
buf.push('/><div class="message-area text-center"><div class="copy-text"><h2>');
var __val__ = resources.ftux.createNew
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h2><h3>');
var __val__ = resources.ftux.createNewSub
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h3></div></div></div><div class="ftux-slide touch-slider-hidden"><img');
buf.push(attrs({ 'src':('' + (cdn) + 'img/ftux/asset2.png'), "class": ('ftux-img') }, {"src":true}));
buf.push('/><div class="message-area text-center"><div class="copy-text"><h2>');
var __val__ = resources.ftux.pickPhotos
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h2><h3>');
var __val__ = resources.ftux.pickPhotosSub
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h3></div></div></div><div class="ftux-slide touch-slider-hidden"><img');
buf.push(attrs({ 'src':('' + (cdn) + 'img/ftux/asset3.png'), "class": ('ftux-img') }, {"src":true}));
buf.push('/><div class="message-area text-center"><div class="copy-text"><h2>');
var __val__ = resources.ftux.invitePeople
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h2><h3>');
var __val__ = resources.ftux.invitePeopleSub
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h3></div></div></div><div class="ftux-slide touch-slider-hidden"><img');
buf.push(attrs({ 'src':('' + (cdn) + 'img/ftux/asset4.png'), "class": ('ftux-img') }, {"src":true}));
buf.push('/><div class="message-area text-center"><div class="copy-text"><h2>');
var __val__ = resources.ftux.screens
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h2><h3>');
var __val__ = resources.ftux.screensSub
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h3></div></div></div><div class="ftux-slide touch-slider-hidden"><img');
buf.push(attrs({ 'src':('' + (cdn) + 'img/ftux/asset5.png'), "class": ('ftux-img') }, {"src":true}));
buf.push('/><div class="message-area text-center"><div class="copy-text"><h2>');
var __val__ = resources.ftux.longPress
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h2><h3>');
var __val__ = resources.ftux.longPressSub
buf.push(null == __val__ ? "" : __val__);
buf.push('</h3></div></div></div><div class="ftux-slide touch-slider-hidden"><img');
buf.push(attrs({ 'src':('' + (cdn) + 'img/ftux/asset6-10.png'), "class": ('ftux-img') }, {"src":true}));
buf.push('/><div class="message-area text-center"><div class="copy-text"><h2>');
var __val__ = resources.ftux.ready
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h2><h3>');
var __val__ = resources.ftux.readySub
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h3></div></div></div></div>');
}
return buf.join("");
}
jade.templates["LoadingScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
 var cdn = client.baseUri || '';
buf.push('<div class="message-area text-center splash"><h1>xim</h1><h3 id="message">');
var __val__ = message
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h3><div class="loading"><i class="dot"></i><i class="dot"></i><i class="dot"></i><i class="dot"></i><i class="dot"></i></div><img');
buf.push(attrs({ 'src':('' + (cdn) + 'img/msft-logo.png'), "class": ('msft-logo') }, {"src":true}));
buf.push('/></div>');
}
return buf.join("");
}
jade.templates["LobbyScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
 var cdn = client.baseUri || '';
buf.push('<div class="message-area text-center splash">');
if ( isWp)
{
buf.push('<object');
buf.push(attrs({ 'data':('' + (cdn) + 'svg/programmerSun.svg'), 'type':('image/svg+xml'), 'width':('460px'), 'height':('460px'), "class": ('lobbyImg') }, {"data":true,"type":true,"width":true,"height":true}));
buf.push('></object>');
}
else
{
buf.push('<img');
buf.push(attrs({ 'src':('' + (cdn) + 'svg/programmerSun.svg'), "class": ('lobbyImg') }, {"src":true}));
buf.push('/>');
}
buf.push('<div class="lobbyBlock"><div class="lobbyTitle">');
var __val__ = resources.title
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</div><div id="line1" class="lobbyText"></div><div id="line2" class="lobbyText"></div><div id="line3" class="lobbyText"></div></div></div>');
}
return buf.join("");
}
jade.templates["StartingCastScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
 var cdn = client.baseUri || '';
buf.push('<div class="loading-graphics">');
if ( isWp)
{
buf.push('<object');
buf.push(attrs({ 'data':('' + (cdn) + 'svg/emptySun.svg'), 'type':('image/svg+xml'), 'width':('460px'), 'height':('460px'), "class": ('loading-img') + ' ' + ('backdrop') }, {"data":true,"type":true,"width":true,"height":true}));
buf.push('></object><object');
buf.push(attrs({ 'data':('' + (cdn) + 'svg/emptySunOpen.svg'), 'type':('image/svg+xml'), 'width':('460px'), 'height':('460px'), "class": ('loading-img') + ' ' + ('blink') }, {"data":true,"type":true,"width":true,"height":true}));
buf.push('></object><object');
buf.push(attrs({ 'data':('' + (cdn) + 'svg/emptySunClosed.svg'), 'type':('image/svg+xml'), 'width':('460px'), 'height':('460px'), "class": ('loading-img') }, {"data":true,"type":true,"width":true,"height":true}));
buf.push('></object>');
}
else
{
buf.push('<img');
buf.push(attrs({ 'src':('' + (cdn) + 'svg/emptySun.svg'), "class": ('loading-img') }, {"src":true}));
buf.push('/><img');
buf.push(attrs({ 'src':('' + (cdn) + 'svg/emptySunOpen.svg'), "class": ('loading-img') + ' ' + ('blink') }, {"src":true}));
buf.push('/><img');
buf.push(attrs({ 'src':('' + (cdn) + 'svg/emptySunClosed.svg'), "class": ('loading-img') }, {"src":true}));
buf.push('/>');
}
buf.push('</div><div class="loading-text text-center"><h3 id="message">');
var __val__ = message
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h3></div>');
}
return buf.join("");
}
jade.templates["YouTubeDialog"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="dialog-body"><div class="search-box inputbox"><i class="glyphicon glyphicon-search"></i><form role="form"><div class="form-group"><input');
buf.push(attrs({ 'id':('search'), 'type':('search'), 'placeholder':('' + (resources.forms.searchPlaceholder) + ''), 'value':('' + (searchTerm) + ''), "class": ('form-control') }, {"type":true,"placeholder":true,"value":true}));
buf.push('/></div></form></div><div id="youtube-items" class="section">');
if ( items.length)
{
buf.push('<div class="items tile-list"></div>');
}
buf.push('<div class="message"></div></div>');
if ( (title))
{
buf.push('<div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div>');
}
buf.push('<div class="command-bar commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["YouTubeScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="dialog-body"><div class="search-box inputbox"><i class="glyphicon glyphicon-search"></i><form role="form"><div class="form-group"><input');
buf.push(attrs({ 'id':('search'), 'type':('search'), 'placeholder':('' + (resources.forms.searchPlaceholder) + ''), 'value':('' + (searchTerm) + ''), "class": ('form-control') }, {"type":true,"placeholder":true,"value":true}));
buf.push('/></div></form></div><div id="youtube-items" class="section">');
if ( items.length)
{
buf.push('<div class="items tile-list"></div>');
}
buf.push('<div class="message"></div></div>');
if ( (title))
{
buf.push('<div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div>');
}
buf.push('<div class="command-bar commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["YouTubeTile"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
}
return buf.join("");
}
jade.templates["addBrowserPopUp"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="addBrowserPopUp"><div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = data.resources.title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div><label class="addBrowserTitle">');
var __val__ = data.resources.text
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label>');
if ( (data.resources.bullets))
{
buf.push('<div class="howToIntro"><ol>');
// iterate data.resources.bullets
;(function(){
  if ('number' == typeof data.resources.bullets.length) {
    for (var $index = 0, $$l = data.resources.bullets.length; $index < $$l; $index++) {
      var bullet = data.resources.bullets[$index];

buf.push('<li>');
var __val__ = bullet
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</li>');
    }
  } else {
    for (var $index in data.resources.bullets) {
      var bullet = data.resources.bullets[$index];

buf.push('<li>');
var __val__ = bullet
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</li>');
   }
  }
}).call(this);

buf.push('</ol></div>');
}
else
{
buf.push('<div class="addBrowserIntro"><label class="dialogBullet">');
var __val__ = data.resources.bullet1
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><label class="addBrowserLink">');
var __val__ = data.url
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><label class="dialogBulletTop">');
var __val__ = data.resources.bullet3
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><label class="dialogBulletSub">');
var __val__ = data.resources.bullet3s
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label></div>');
}
buf.push('<div class="addBrowserInputbox"><form onsubmit="return false" action="#"><input');
buf.push(attrs({ 'type':("text"), 'name':("code"), 'placeholder':('' + (data.resources.placeholder) + ''), 'autocomplete':("off"), 'autocorrect':("off"), 'autocapitalize':("off"), 'spellcheck':("false"), "class": ('ximformcontrol') + ' ' + ('dialogInputText') }, {"type":true,"name":true,"placeholder":true,"autocomplete":true,"autocorrect":true,"autocapitalize":true,"spellcheck":true}));
buf.push('/><label class="dialogError"></label><input');
buf.push(attrs({ 'type':("submit"), 'value':('' + (data.resources.button) + ''), "class": ('addBrowserButton') }, {"type":true,"value":true}));
buf.push('/></form></div><label class="addBrowserFooter underline">');
var __val__ = data.resources.subtext
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><div style="display:none" class="addBrowserWait"><i class="dot"></i><i class="dot"></i><i class="dot"></i><i class="dot"></i><i class="dot"></i></div></div>');
}
return buf.join("");
}
jade.templates["albums"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="dialog-body"><div id="album-items" class="section">');
if ( albums.length)
{
buf.push('<div class="items tile-list"></div>');
}
else
{
buf.push('<div class="message loading">' + escape((interp = resources.messages.loadingFacebook) == null ? '' : interp) + '</div>');
}
buf.push('</div>');
if ( (title))
{
buf.push('<div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div>');
}
buf.push('<div class="command-bar commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["appMasterScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<i style="display: none" class="glyphiconXIM glyphiconXIM-exclamationIcon connectionglyph"></i><div class="header hidden"><h1 class="apptitle">xim</h1><i class="glyphiconXIM glyphiconXIM-settingsCog headerglyph cogglyph"></i></div><!-- DISABLED  img.dot(src="img/public-state.png")--><div class="headerAlt hidden"><h1 class="apptitle alttitle"> </h1><i class="ios-only glyphiconXIM glyphiconXIM-backArrowIcon headerglyph backglyph"></i></div><div class="navMenuContent"></div><div class="privacyMenuContent"></div><div class="page"><div class="detail-screens"></div><div class="commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["camera"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="dialog-body"><!----><div');
buf.push(attrs({ 'id':(contentSource +'-items'), "class": ('section') }, {"id":true}));
buf.push('>');
if ( albums.length)
{
buf.push('<div class="items tile-list"></div>');
}
else
{
buf.push('<div class="message loading"> ' + escape((interp = resources.messages.loadingCamera) == null ? '' : interp) + '</div>');
}
buf.push('</div>');
if ( (title))
{
buf.push('<div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div>');
}
buf.push('<div class="command-bar commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["castViewerChoice"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="viewerChoiceHeader"><label class="viewerChoiceTitle">');
var __val__ = resources.scanning
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><div class="viewerChoiceWait small"><i class="dot"></i><i class="dot"></i><i class="dot"></i><i class="dot"></i><i class="dot"></i></div></div>');
// iterate castViewers
;(function(){
  if ('number' == typeof castViewers.length) {
    for (var index = 0, $$l = castViewers.length; index < $$l; index++) {
      var castViewer = castViewers[index];

buf.push('<div');
buf.push(attrs({ 'id':('viewer_' + (index) + ''), "class": ('viewerChoiceBox') }, {"id":true}));
buf.push('><div');
buf.push(attrs({ "class": ('viewerChoiceGlyph') + ' ' + ('' + (castViewer.glyph) + '') }, {"class":true}));
buf.push('></div><div class="viewerChoiceLabel">' + escape((interp = castViewer.label) == null ? '' : interp) + '</div></div>');
    }
  } else {
    for (var index in castViewers) {
      var castViewer = castViewers[index];

buf.push('<div');
buf.push(attrs({ 'id':('viewer_' + (index) + ''), "class": ('viewerChoiceBox') }, {"id":true}));
buf.push('><div');
buf.push(attrs({ "class": ('viewerChoiceGlyph') + ' ' + ('' + (castViewer.glyph) + '') }, {"class":true}));
buf.push('></div><div class="viewerChoiceLabel">' + escape((interp = castViewer.label) == null ? '' : interp) + '</div></div>');
   }
  }
}).call(this);

}
return buf.join("");
}
jade.templates["castViewerChoicePopUp"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="castViewerChoicePopUp"></div>');
}
return buf.join("");
}
jade.templates["chromeTopBar"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="chromeTopBase"><div class="castBackArrow"></div></div>');
}
return buf.join("");
}
jade.templates["commandBar"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div');
buf.push(attrs({ "class": ('commandBarBase') + ' ' + ('' + (style.bar) + '') }, {"class":true}));
buf.push('><ul class="commandBarNav">');
// iterate commands 
;(function(){
  if ('number' == typeof commands .length) {
    for (var $index = 0, $$l = commands .length; $index < $$l; $index++) {
      var cmd = commands [$index];

buf.push('<li');
buf.push(attrs({ 'id':('' + (cmd.id) + ''), 'style':("font-size: " + (style.textSize) + "px; width:" + (width) + "%;"), "class": ((cmd.enabled? '' : 'commandDisabled') + ' ' + (style.tab) + '') }, {"id":true,"class":true,"style":true}));
buf.push('>');
if ( (style.glyphSize == 0))
{
buf.push('<div>' + escape((interp = cmd.label) == null ? '' : interp) + '</div>');
}
else if ( (cmd.icon))
{
buf.push('<span><a href="#"><img');
buf.push(attrs({ 'src':('' + (cmd.icon) + '') }, {"src":true}));
buf.push('/></a></span><span>' + escape((interp = cmd.label) == null ? '' : interp) + '</span>');
}
else
{
if ( (cmd.glyph) )
{
buf.push('<div><span');
buf.push(attrs({ 'style':('font-size: ' + (style.glyphSize) + 'px'), "class": ('commandBarGlyph') + ' ' + ('' + (cmd.glyph) + '') }, {"class":true,"style":true}));
buf.push('></span></div><div class="commandBarLabel">' + escape((interp = cmd.label) == null ? '' : interp) + '</div>');
}
else
{
buf.push('<div>' + escape((interp = cmd.label) == null ? '' : interp) + '</div>');
}
}
buf.push('</li>');
    }
  } else {
    for (var $index in commands ) {
      var cmd = commands [$index];

buf.push('<li');
buf.push(attrs({ 'id':('' + (cmd.id) + ''), 'style':("font-size: " + (style.textSize) + "px; width:" + (width) + "%;"), "class": ((cmd.enabled? '' : 'commandDisabled') + ' ' + (style.tab) + '') }, {"id":true,"class":true,"style":true}));
buf.push('>');
if ( (style.glyphSize == 0))
{
buf.push('<div>' + escape((interp = cmd.label) == null ? '' : interp) + '</div>');
}
else if ( (cmd.icon))
{
buf.push('<span><a href="#"><img');
buf.push(attrs({ 'src':('' + (cmd.icon) + '') }, {"src":true}));
buf.push('/></a></span><span>' + escape((interp = cmd.label) == null ? '' : interp) + '</span>');
}
else
{
if ( (cmd.glyph) )
{
buf.push('<div><span');
buf.push(attrs({ 'style':('font-size: ' + (style.glyphSize) + 'px'), "class": ('commandBarGlyph') + ' ' + ('' + (cmd.glyph) + '') }, {"class":true,"style":true}));
buf.push('></span></div><div class="commandBarLabel">' + escape((interp = cmd.label) == null ? '' : interp) + '</div>');
}
else
{
buf.push('<div>' + escape((interp = cmd.label) == null ? '' : interp) + '</div>');
}
}
buf.push('</li>');
   }
  }
}).call(this);

buf.push('</ul></div><div class="commandBarBaseTarget"><ul class="commandBarNav">');
// iterate commands 
;(function(){
  if ('number' == typeof commands .length) {
    for (var $index = 0, $$l = commands .length; $index < $$l; $index++) {
      var cmd = commands [$index];

buf.push('<li');
buf.push(attrs({ 'id':('' + (cmd.id) + ''), 'style':("width:" + (width) + "%;"), "class": ('commandTarget') }, {"id":true,"style":true}));
buf.push('></li>');
    }
  } else {
    for (var $index in commands ) {
      var cmd = commands [$index];

buf.push('<li');
buf.push(attrs({ 'id':('' + (cmd.id) + ''), 'style':("width:" + (width) + "%;"), "class": ('commandTarget') }, {"id":true,"style":true}));
buf.push('></li>');
   }
  }
}).call(this);

buf.push('</ul></div>');
}
return buf.join("");
}
jade.templates["commentPopUp"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="commentPopUp"><textarea');
buf.push(attrs({ 'rows':("3"), 'placeholder':('' + (data.resources.placeholder) + ''), "class": ('form-control') + ' ' + ('commentPopUpInputForm') }, {"rows":true,"placeholder":true}));
buf.push('></textarea><ul class="confirmationPopUpMenu">');
if ( (data.resources.leftButton))
{
buf.push('<li id="confirmationPopUpLeft" class="confirmationPopUpLi">' + escape((interp = data.resources.leftButton) == null ? '' : interp) + '</li>');
}
if ( (data.resources.rightButton))
{
buf.push('<li id="confirmationPopUpRight" class="confirmationPopUpLi">' + escape((interp = data.resources.rightButton) == null ? '' : interp) + '</li>');
}
buf.push('</ul></div>');
}
return buf.join("");
}
jade.templates["confirmationPopUp"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="confirmationPopUp"><div class="confirmationTitle">' + escape((interp = data.title) == null ? '' : interp) + '</div><div class="confirmationMessage">' + escape((interp = data.message) == null ? '' : interp) + '</div><ul class="confirmationPopUpMenu">');
if ( (data.leftChoice))
{
buf.push('<li id="confirmationPopUpLeft" class="confirmationPopUpLi">' + escape((interp = data.leftChoice) == null ? '' : interp) + '</li>');
}
if ( (data.rightChoice))
{
buf.push('<li id="confirmationPopUpRight" class="confirmationPopUpLi">' + escape((interp = data.rightChoice) == null ? '' : interp) + '</li>');
}
buf.push('</ul></div>');
}
return buf.join("");
}
jade.templates["connecting"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="page"><div class="message fullscreen"><h3>Connecting to service...</h3></div></div>');
}
return buf.join("");
}
jade.templates["countryPopUp"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="countryPopUp"><div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = data.title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div><div class="loginPhone">');
// iterate data.countryArray
;(function(){
  if ('number' == typeof data.countryArray.length) {
    for (var $index = 0, $$l = data.countryArray.length; $index < $$l; $index++) {
      var val = data.countryArray[$index];

buf.push('<div');
buf.push(attrs({ 'id':('' + (val.code) + ''), "class": ('countryPopUpItem') }, {"id":true}));
buf.push('>');
var __val__ = val.name
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</div>');
    }
  } else {
    for (var $index in data.countryArray) {
      var val = data.countryArray[$index];

buf.push('<div');
buf.push(attrs({ 'id':('' + (val.code) + ''), "class": ('countryPopUpItem') }, {"id":true}));
buf.push('>');
var __val__ = val.name
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</div>');
   }
  }
}).call(this);

buf.push('</div></div>');
}
return buf.join("");
}
jade.templates["debugFooter"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="debugfooter"><form role="form" class="debug-server-response"><div class="form-group"><label class="debuglabel">Server Override:</label><select id="serverOverride" class="form-control debugformcontrol"><option value="0">- none -</option><option value="1">badRequest</option><option value="2">notAuthorized</option><option value="3">forbidden</option><option value="4">notFound</option><option value="5">upgradeRequired</option><option value="6">tokenExpired</option><option value="7">serverError</option><option value="8">connectionError</option><option value="9">otherError</option><option value="10">waitlisted</option></select></div></form><i id="debugButtonLogs" class="glyphicon glyphicon-book"></i></div>');
}
return buf.join("");
}
jade.templates["debugPopUp"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="errorPopUp popUpHide"><div class="debugText">Thanks beta tester.</div><div class="debugText">You found a bug!</div><hr/><div class="debugText">' + escape((interp = data.error) == null ? '' : interp) + '</div><div class="debugText">File: ' + escape((interp = data.uri) == null ? '' : interp) + '</div><div class="debugText">Line: ' + escape((interp = data.line) == null ? '' : interp) + '</div><div class="debugText">Column: ' + escape((interp = data.column) == null ? '' : interp) + '</div><input type="submit" value="Send Report" class="debugPopUpButton"/></div>');
}
return buf.join("");
}
jade.templates["debugScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="content"><div class="debugEffect"></div><div class="section"><form role="form" class="debug-setting"><div class="form-group"><label for="loginOverride" class="debugformlabel">');
var __val__ = resources.forms.loginOverridePrompt
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><select id="loginOverride" class="form-control debugformcontrol"><option value="">automatic</option><option value="fb">facebook</option><option value="test">test</option></select></div><div class="form-group"><label for="locationOverride" class="debugformlabel">');
var __val__ = resources.forms.locationOverridePrompt
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><select id="locationOverride" class="form-control debugformcontrol"><option value="">automatic</option><option value="47.640682,-122.141174">Bldg 112</option><option value="47.642124,-122.142021">Bldg 99</option><option value="47.641228,-122.123962">Bldg 36 (Conference Center)</option><option value="47.651302,-122.133377">Bldg 85</option><option value="47.619930,-122.348671">Space Needle</option></select></div><div class="form-group"><label for="loggerLevel" class="debugformlabel">');
var __val__ = resources.forms.loggerLevel
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><select id="loggerLevel" class="form-control debugformcontrol"><option value="0">info</option><option value="1">debug</option><option value="2">warning</option><option value="3">error</option></select></div><div class="form-group"><label for="discoverySetting" class="debugformlabel">');
var __val__ = resources.forms.discoverySetting
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><select id="discoverySetting" class="form-control debugformcontrol"><option');
buf.push(attrs({ 'value':(consts.NearbyDiscoveryMode.anyone) }, {"value":true}));
buf.push('>public</option><option');
buf.push(attrs({ 'value':(consts.NearbyDiscoveryModefriends) }, {"value":true}));
buf.push('>friends</option><option');
buf.push(attrs({ 'value':(consts.NearbyDiscoveryModeincognito) }, {"value":true}));
buf.push('>incognito</option></select></div>');
// iterate settings
;(function(){
  if ('number' == typeof settings.length) {
    for (var index = 0, $$l = settings.length; index < $$l; index++) {
      var val = settings[index];

buf.push('<!-- For boolean type (\'0\') only-->');
if ( (val['type'] == 0) )
{
buf.push('<div class="form-group debugcheckgroup">       <input');
buf.push(attrs({ 'id':("" + (index) + ""), 'type':("checkbox"), 'checked':(val['value']), "class": ('debugcheckbox') + ' ' + ("" + (val['name']) + "") }, {"class":true,"id":true,"type":true,"checked":true}));
buf.push('/><label for="val[\'name\']" class="debugchecklabel">');
var __val__ = val['description']
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label></div>');
}
    }
  } else {
    for (var index in settings) {
      var val = settings[index];

buf.push('<!-- For boolean type (\'0\') only-->');
if ( (val['type'] == 0) )
{
buf.push('<div class="form-group debugcheckgroup">       <input');
buf.push(attrs({ 'id':("" + (index) + ""), 'type':("checkbox"), 'checked':(val['value']), "class": ('debugcheckbox') + ' ' + ("" + (val['name']) + "") }, {"class":true,"id":true,"type":true,"checked":true}));
buf.push('/><label for="val[\'name\']" class="debugchecklabel">');
var __val__ = val['description']
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label></div>');
}
   }
  }
}).call(this);

buf.push('</form></div></div>');
}
return buf.join("");
}
jade.templates["dropbox"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="dialog-body"><div id="dropbox-items" class="section">');
if ( albums.length)
{
buf.push('<div class="items tile-list"></div>');
}
else
{
buf.push('<div class="message loading">' + escape((interp = resources.messages.loadingDropbox) == null ? '' : interp) + '</div>');
}
buf.push('</div>');
if ( (title))
{
buf.push('<div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div>');
}
buf.push('<div class="command-bar commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["errorPopUp"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="errorPopUp popUpHide">');
if ( data.isWp)
{
buf.push('<object');
buf.push(attrs({ 'data':('' + (data.message.screenType.svg) + ''), 'type':('image/svg+xml'), 'width':('460px'), 'height':('460px'), "class": ('errorImg') }, {"data":true,"type":true,"width":true,"height":true}));
buf.push('></object>');
}
else
{
buf.push('<img');
buf.push(attrs({ 'src':('' + (data.message.screenType.svg) + ''), "class": ('errorImg') }, {"src":true}));
buf.push('/>');
}
if ( (data.message.text1))
{
buf.push('<div');
buf.push(attrs({ 'style':("top:" + (data.message.screenType.text1top) + "px"), "class": ('errorText') }, {"style":true}));
buf.push('>' + escape((interp = data.message.text1) == null ? '' : interp) + '</div>');
}
if ( (data.message.text2))
{
buf.push('<div');
buf.push(attrs({ 'style':("top:" + (data.message.screenType.text2top) + "px"), "class": ('errorText') }, {"style":true}));
buf.push('>' + escape((interp = data.message.text2) == null ? '' : interp) + '</div>');
}
if ( (data.message.buttonText))
{
buf.push('<input');
buf.push(attrs({ 'style':("top:" + (data.message.screenType.buttontop) + "px"), 'type':('submit'), 'value':('' + (data.message.buttonText) + ''), "class": ('errorButton') }, {"style":true,"type":true,"value":true}));
buf.push('/>');
}
buf.push('</div>');
}
return buf.join("");
}
jade.templates["flickr"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="dialog-body"><div id="flickr-items" class="section">');
if ( albums.length)
{
buf.push('<div class="items tile-list"></div>');
}
else
{
buf.push('<div class="message loading">' + escape((interp = resources.messages.loadingFlickr) == null ? '' : interp) + '</div>');
}
buf.push('</div>');
if ( (title))
{
buf.push('<div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div>');
}
buf.push('<div class="command-bar commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["howToAirPlay"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="howToPopUp"><div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = data.resources.title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div><label class="howToTitle">');
var __val__ = data.resources.text
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><div class="howToIntro"><ol>');
// iterate data.resources.bullets
;(function(){
  if ('number' == typeof data.resources.bullets.length) {
    for (var $index = 0, $$l = data.resources.bullets.length; $index < $$l; $index++) {
      var bullet = data.resources.bullets[$index];

buf.push('<li>');
var __val__ = bullet
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</li>');
    }
  } else {
    for (var $index in data.resources.bullets) {
      var bullet = data.resources.bullets[$index];

buf.push('<li>');
var __val__ = bullet
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</li>');
   }
  }
}).call(this);

buf.push('</ol><input');
buf.push(attrs({ 'type':("submit"), 'value':('' + (data.resources.button) + ''), "class": ('howToButton') }, {"type":true,"value":true}));
buf.push('/></div><label class="howToFooter underline">');
var __val__ = data.resources.subtext
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label></div>');
}
return buf.join("");
}
jade.templates["howToXBox"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="howToPopUp"><div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = data.resources.title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div><label class="howToTitle">');
var __val__ = data.resources.text
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><div class="howToIntro"><ol>');
// iterate data.resources.bullets
;(function(){
  if ('number' == typeof data.resources.bullets.length) {
    for (var $index = 0, $$l = data.resources.bullets.length; $index < $$l; $index++) {
      var bullet = data.resources.bullets[$index];

buf.push('<li>');
var __val__ = bullet
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</li>');
    }
  } else {
    for (var $index in data.resources.bullets) {
      var bullet = data.resources.bullets[$index];

buf.push('<li>');
var __val__ = bullet
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</li>');
   }
  }
}).call(this);

buf.push('</ol><input');
buf.push(attrs({ 'type':("submit"), 'value':('' + (data.resources.button) + ''), "class": ('howToButton') }, {"type":true,"value":true}));
buf.push('/></div><label class="howToFooter underline">');
var __val__ = data.resources.subtext
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label></div>');
}
return buf.join("");
}
jade.templates["content"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="content"></div>');
}
return buf.join("");
}
jade.templates["contentWithHeader"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="has-header"><div class="header"></div><div class="content"></div></div>');
}
return buf.join("");
}
jade.templates["dialog"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="dialog-body">');
if ( (title))
{
buf.push('<div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div>');
}
buf.push('<div class="command-bar commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["messages"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="message">');
// iterate messages
;(function(){
  if ('number' == typeof messages.length) {
    for (var index = 0, $$l = messages.length; index < $$l; index++) {
      var val = messages[index];

buf.push('<div');
buf.push(attrs({ "class": ('' + (index) + '') }, {"class":true}));
buf.push('>');
var __val__ = val
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</div>');
    }
  } else {
    for (var index in messages) {
      var val = messages[index];

buf.push('<div');
buf.push(attrs({ "class": ('' + (index) + '') }, {"class":true}));
buf.push('>');
var __val__ = val
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</div>');
   }
  }
}).call(this);

buf.push('</div>');
}
return buf.join("");
}
jade.templates["searchHeader"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<form role="form"><div class="form-group"><input');
buf.push(attrs({ 'id':('search'), 'type':('search'), 'placeholder':('' + (resources.forms.searchPlaceholder) + ''), 'value':('' + (searchTerm) + ''), "class": ('form-control') }, {"type":true,"placeholder":true,"value":true}));
buf.push('/></div></form>');
}
return buf.join("");
}
jade.templates["tileList"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
if ( title)
{
buf.push('<div class="section-heading">');
var __val__ = title
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</div>');
}
buf.push('<div class="items tile-list"></div>');
}
return buf.join("");
}
jade.templates["content"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="content"></div>');
}
return buf.join("");
}
jade.templates["contentWithHeader"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="has-header"><div class="header"></div><div class="content"></div></div>');
}
return buf.join("");
}
jade.templates["dialog"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="dialog-body">');
if ( (title))
{
buf.push('<div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div>');
}
buf.push('<div class="command-bar commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["messages"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="message">');
// iterate messages
;(function(){
  if ('number' == typeof messages.length) {
    for (var index = 0, $$l = messages.length; index < $$l; index++) {
      var val = messages[index];

buf.push('<div');
buf.push(attrs({ "class": ('' + (index) + '') }, {"class":true}));
buf.push('>');
var __val__ = val
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</div>');
    }
  } else {
    for (var index in messages) {
      var val = messages[index];

buf.push('<div');
buf.push(attrs({ "class": ('' + (index) + '') }, {"class":true}));
buf.push('>');
var __val__ = val
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</div>');
   }
  }
}).call(this);

buf.push('</div>');
}
return buf.join("");
}
jade.templates["searchHeader"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<form role="form"><div class="form-group"><input');
buf.push(attrs({ 'id':('search'), 'type':('search'), 'placeholder':('' + (resources.forms.searchPlaceholder) + ''), 'value':('' + (searchTerm) + ''), "class": ('form-control') }, {"type":true,"placeholder":true,"value":true}));
buf.push('/></div></form>');
}
return buf.join("");
}
jade.templates["tileList"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
if ( title)
{
buf.push('<div class="section-heading">');
var __val__ = title
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</div>');
}
buf.push('<div class="items tile-list"></div>');
}
return buf.join("");
}
jade.templates["instagram"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="dialog-body"><!--<div class="search-box inputbox"><i class="glyphicon glyphicon-search"></i><form role="form"><div class="form-group"><input');
buf.push(attrs({ 'id':('search'), 'type':('search'), 'placeholder':('' + (resources.forms.searchPlaceholder) + ''), 'value':('' + (searchTerm) + ''), "class": ('form-control') }, {"type":true,"placeholder":true,"value":true}));
buf.push('/></div></form></div>--><div id="instagram-items" class="section">');
if ( albums.length)
{
buf.push('<div class="items tile-list"></div>');
}
else
{
buf.push('<div class="message loading"> ' + escape((interp = resources.messages.loadingInstagram) == null ? '' : interp) + '</div>');
}
buf.push('</div>');
if ( (title))
{
buf.push('<div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div>');
}
buf.push('<div class="command-bar commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["logScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="dialog-body"><div class="content"><div class="form-group"><label for="loggerLevel" class="logformlabel">');
var __val__ = resources.forms.loggerLevel
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><select id="loggerLevel" class="form-control logformcontrol"><option value="0">info</option><option value="1">debug</option><option value="2">warning</option><option value="3">error</option></select></div><div class="section log"></div></div>');
if ( (title))
{
buf.push('<div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div>');
}
buf.push('<div class="command-bar commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["login"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="dialog-body"><div class="section"><a id="login-facebook" href="." class="btn btn-primary btn-large login-btn">' + escape((interp = resources.labels.login.facebook) == null ? '' : interp) + '</a></div>');
if ( (title))
{
buf.push('<div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div>');
}
buf.push('<div class="command-bar commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["loginChoiceScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="dialog-body"><div class="loginPhone"><div class="dialogIntro"><label>');
var __val__ = resources.intro1
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><br/><label class="loginChoicePhone">');
var __val__ = phone
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><br/><label>');
var __val__ = resources.intro2
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><label>');
var __val__ = resources.intro3
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label></div><div class="loginChoiceInputbox"><form onsubmit="return false" action="#"><input');
buf.push(attrs({ 'type':("submit"), 'value':('' + (resources.sms) + ''), "class": ('loginButtonSMS') }, {"type":true,"value":true}));
buf.push('/><input');
buf.push(attrs({ 'type':("submit"), 'value':('' + (resources.phone) + ''), "class": ('loginButtonPhone') }, {"type":true,"value":true}));
buf.push('/></form></div><div class="loginChangePhone"><label class="underline">');
var __val__ = resources.sub1
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label></div><div style="display:none" class="loginChoiceWait"><i class="dot"></i><i class="dot"></i><i class="dot"></i><i class="dot"></i><i class="dot"></i></div></div>');
if ( (title))
{
buf.push('<div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div>');
}
buf.push('<div class="command-bar commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["loginPhoneScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="dialog-body"><div class="loginPhone"><div class="dialogIntro"><label>');
var __val__ = resources.intro1
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><label>');
var __val__ = resources.intro2
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><label>');
var __val__ = resources.intro3
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label></div><div class="loginInputbox"><form onsubmit="return false" action="#"><div class="loginPhoneBox"> <input type="submit" value="" class="loginButtonCountry"/><input');
buf.push(attrs({ 'type':("tel"), 'name':("phone"), 'placeholder':('' + (resources.inputCue) + ''), "class": ('loginInputTextPhone') }, {"type":true,"name":true,"placeholder":true}));
buf.push('/><label class="loginLabelCountry"></label></div><label class="loginErrorPhone"></label><input');
buf.push(attrs({ 'type':("submit"), 'value':('' + (resources.next) + ''), "class": ('loginButton') }, {"type":true,"value":true}));
buf.push('/></form></div><div class="loginPhoneFooter"><label>');
var __val__ = resources.sub1
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><label><a id="terms" class="loginLink">' + escape((interp = resources.termsOfUse) == null ? '' : interp) + '</a>&nbspand&nbsp<a id="privacy" class="loginLink">' + escape((interp = resources.privacy) == null ? '' : interp) + '</a></label></div><div style="display:none" class="loginWait"><i class="dot"></i><i class="dot"></i><i class="dot"></i><i class="dot"></i><i class="dot"></i></div></div><div class="loginCountryPopUp"></div>');
if ( (title))
{
buf.push('<div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div>');
}
buf.push('<div class="command-bar commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["loginVerifyScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="dialog-body"><div class="loginPhone"><div class="dialogIntro"><label>');
var __val__ = resources.intro1
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><label>');
var __val__ = resources.intro2
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label></div><div class="loginInputbox"><form onsubmit="return false" action="#"><input');
buf.push(attrs({ 'type':("tel"), 'name':("code"), 'placeholder':('' + (resources.inputCue) + ''), "class": ('ximformcontrol') + ' ' + ('dialogInputText') }, {"type":true,"name":true,"placeholder":true}));
buf.push('/><label class="dialogError"></label><input');
buf.push(attrs({ 'type':("submit"), 'value':('' + (resources.next) + ''), "class": ('loginButton') }, {"type":true,"value":true}));
buf.push('/></form></div><div class="loginResentSMS"><label class="underline">');
var __val__ = resources.sub1
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><br/><label class="underline">');
var __val__ = resources.sub2
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label></div><div style="display:none" class="loginWait"><i class="dot"></i><i class="dot"></i><i class="dot"></i><i class="dot"></i><i class="dot"></i></div></div>');
if ( (title))
{
buf.push('<div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div>');
}
buf.push('<div class="command-bar commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["map"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<!--bing map key --><div class="console hidden">this is the console</div><div');
buf.push(attrs({ 'id':('map'), 'style':("" + (positionStyle) + "") }, {"style":true}));
buf.push('></div>');
}
return buf.join("");
}
jade.templates["messageBubble"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
 var cdn = client.baseUri || '';
 var newStyle = ""
 if (animate) newStyle = " messageNew";
buf.push('<div');
buf.push(attrs({ "class": ('messageBlock') + ' ' + (newStyle) }, {"class":true}));
buf.push('>');
 if (data.thumbnailUrl)
{
buf.push('<div');
buf.push(attrs({ 'style':("background-image:url('" + (data.thumbnailUrl) + "');"), "class": ('messageFace') + ' ' + ("messageFace" + (data.boxStyle) + "") }, {"class":true,"style":true}));
buf.push('></div>');
}
else
{
buf.push('<div');
buf.push(attrs({ 'style':("background-color:" + (data.profileColor) + ";"), "class": ('messageFace') + ' ' + ("messageFace" + (data.boxStyle) + "") }, {"class":true,"style":true}));
buf.push('><figcaption class="messageLabel">' + escape((interp = data.initials) == null ? '' : interp) + '</figcaption></div>');
}
buf.push('<div');
buf.push(attrs({ 'style':("border-color:" + (data.boxColor) + "; background-color:" + (data.backColor) + ";"), "class": ('messageBoxHolder') + ' ' + ("messageBox" + (data.boxStyle) + "") }, {"class":true,"style":true}));
buf.push('><div class="messageText">');
var __val__ = data.text
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</div>');
 if (data.imageUrl && data.text)
{
buf.push('<div class="messageText">');
var __val__ = "&nbsp;"
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</div>');
}
buf.push('<div');
buf.push(attrs({ 'style':("border-color:transparent " + (data.boxColor) + ""), "class": ("messageNub" + (data.boxStyle) + "") }, {"class":true,"style":true}));
buf.push('></div></div>');
 if (data.imageUrl)
{
buf.push('<div');
buf.push(attrs({ "class": ('messageImageHolder') + ' ' + ("messageImageHolder" + (data.boxStyle) + "") }, {"class":true}));
buf.push('><div');
buf.push(attrs({ 'style':("background-image:url('" + (data.imageUrl) + "');"), "class": ('messageImage') + ' ' + ("messageImage" + (data.boxStyle) + "") }, {"class":true,"style":true}));
buf.push('></div></div>');
}
 if (data.sunUrl)
{
buf.push('<img');
buf.push(attrs({ 'src':("" + (cdn) + "" + (data.sunUrl) + ""), "class": ('messageSun') + ' ' + ("messageSun" + (data.boxStyle) + "") }, {"class":true,"src":true}));
buf.push('/>');
}
buf.push('</div>');
}
return buf.join("");
}
jade.templates["messagePopUp"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="messagePopUp"><div class="messagePopUpTitle">');
var __val__ = "Messages"
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</div><div class="messagePopUpButtonRow"><input');
buf.push(attrs({ 'type':("submit"), 'value':('' + (data.resources.button) + ''), "class": ('messagePopUpButton') }, {"type":true,"value":true}));
buf.push('/></div><form class="messagePopUpInput"><input type="submit" class="hidden"/><textarea');
buf.push(attrs({ 'rows':("2"), 'placeholder':('' + (data.resources.placeholder) + ''), "class": ('form-control') + ' ' + ('messagePopUpInputForm') }, {"rows":true,"placeholder":true}));
buf.push('></textarea></form><div class="messagePopUpContent"></div><div class="profilePopUpContent"></div></div>');
}
return buf.join("");
}
jade.templates["people"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="dialog-body"><div id="peoplePicker"></div>');
if ( (title))
{
buf.push('<div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div>');
}
buf.push('<div class="command-bar commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["peoplePicker"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
 var cdn = client.baseUri || '';
buf.push('<div class="inputbox"><form onsubmit="return false" action="#"><input');
buf.push(attrs({ 'id':('searchTerm'), 'type':('search'), 'title':('Search'), 'placeholder':('' + (resources.forms.search) + ''), "class": ('ximformcontrol') }, {"type":true,"title":true,"placeholder":true}));
buf.push('/></form></div><div class="peoplepickerscrollpage"><div class="searchresults"><div class="searchresultslist"></div><div class="searchmessage"></div></div><div class="nearby"><div><div class="peoplelayout"></div><div class="commandBarPadding"></div></div><div class="peoplepickerNoContacts"><i class="glyphiconXIM glyphiconXIM-helloMessage nocontactsglyph"></i><div class="noximsTitle1">');
var __val__ = messages.peoplepicker_Title1
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</div>');
if ( isWp)
{
buf.push('<object');
buf.push(attrs({ 'data':('' + (cdn) + 'svg/lonelyClouds.svg'), 'type':('image/svg+xml'), "class": ('peoplePickerLonely') }, {"data":true,"type":true}));
buf.push('></object>');
}
else
{
buf.push('<img');
buf.push(attrs({ 'src':('' + (cdn) + 'svg/lonelyClouds.svg'), "class": ('peoplePickerLonely') }, {"src":true}));
buf.push('/>');
}
buf.push('</div></div></div>');
}
return buf.join("");
}
jade.templates["peoplePopUp"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="peoplePopUp"><div class="peoplePopUpCount"></div><div class="peoplePopProfiles"></div><div class="peoplePopUpButtonRow"><input');
buf.push(attrs({ 'type':("submit"), 'value':('' + (data.resources.addpeople) + ''), "class": ('peoplePopUpButtonAdd') }, {"type":true,"value":true}));
buf.push('/></div><div class="profilePopUpContent"></div></div>');
}
return buf.join("");
}
jade.templates["person"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<figure');
buf.push(attrs({ 'style':("left:" + (position.left) + "px; top:" + (position.top) + "px; width:" + (position.width) + "px; height:" + (position.height) + "px"), 'id':("" + (id) + ""), "class": ('personFigure') }, {"style":true,"id":true}));
buf.push('>');
 var imageStyle = 'width:'+ position.imageSize +'px; height:' + position.imageSize + 'px;'
if ( (initials))
{
if ( (isSelected))
{
buf.push('<div');
buf.push(attrs({ 'style':(imageStyle), "class": ('personImage') + ' ' + ('personImageSelected') }, {"style":true}));
buf.push('><img');
buf.push(attrs({ 'src':('' + (image) + ''), "class": ('personImageImg') }, {"src":true}));
buf.push('/></div>');
}
else
{
buf.push('<div');
buf.push(attrs({ 'style':(imageStyle), "class": ('personImage') }, {"style":true}));
buf.push('><img');
buf.push(attrs({ 'src':('' + (image) + ''), "class": ('personImageImg') }, {"src":true}));
buf.push('/></div>');
}
if ( (position.imageSize >= 100))
{
buf.push('<figcaption class="personLabel">' + escape((interp = initials) == null ? '' : interp) + '</figcaption>');
}
else
{
buf.push('<figcaption class="personLabel personLabelSmall">' + escape((interp = initials) == null ? '' : interp) + '</figcaption>');
}
if ( (useCaption))
{
buf.push('<figcaption');
buf.push(attrs({ 'style':('color:' + (captionColor) + ''), "class": ('personCaptionOffset') }, {"style":true}));
buf.push('>' + escape((interp = caption1) == null ? '' : interp) + '</figcaption>');
}
}
else
{
if ( (isSelected))
{
buf.push('<div');
buf.push(attrs({ 'style':(imageStyle), "class": ('personImage') + ' ' + ('personImageSelected') }, {"style":true}));
buf.push('><img');
buf.push(attrs({ 'src':('' + (image) + ''), "class": ('personImageImg') }, {"src":true}));
buf.push('/></div>');
}
else
{
buf.push('<div');
buf.push(attrs({ 'style':(imageStyle), "class": ('personImage') }, {"style":true}));
buf.push('><img');
buf.push(attrs({ 'src':('' + (image) + ''), "class": ('personImageImg') }, {"src":true}));
buf.push('/></div>');
}
if ( (useCaption))
{
buf.push('<figcaption');
buf.push(attrs({ 'style':('color:' + (captionColor) + ''), "class": ('personCaption') }, {"style":true}));
buf.push('>' + escape((interp = caption1) == null ? '' : interp) + '</figcaption>');
}
}
if ( (isSelected) )
{
buf.push('<figcaption class="figuremark checkmark"></figcaption>');
}
else
{
buf.push('<figcaption class="figuremark"></figcaption>');
}
buf.push('</figure>');
}
return buf.join("");
}
jade.templates["photoPopUp"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="photoPopUp"><div class="photoPopUpCount"></div><div class="photoPopUpButtonRow"><input');
buf.push(attrs({ 'type':("submit"), 'value':('' + (data.resources.addphotos) + ''), "class": ('photoPopUpButtonAdd') }, {"type":true,"value":true}));
buf.push('/><input');
buf.push(attrs({ 'type':("submit"), 'value':('' + (data.resources.takephoto) + ''), "class": ('photoPopUpButtonTake') }, {"type":true,"value":true}));
buf.push('/></div></div>');
}
return buf.join("");
}
jade.templates["picker"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="dialog-body"><div id="photos-connections" class="section">');
if ( tiles.length        )
{
buf.push('<div class="items tile-list flush">');
// iterate tiles
;(function(){
  if ('number' == typeof tiles.length) {
    for (var $index = 0, $$l = tiles.length; $index < $$l; $index++) {
      var tile = tiles[$index];

buf.push('<a');
buf.push(attrs({ 'href':('#/' + (tile.id) + ''), 'target':('_self'), "class": ('tile-link') }, {"href":true,"target":true}));
buf.push('><div class="tile picker-source"><div');
buf.push(attrs({ 'style':('background-image:url("' + (tile.thumbnail) + '")'), "class": ('tile-image') }, {"style":true}));
buf.push('></div><div class="tile-title picker-source"><h5>');
var __val__ = tile.title
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h5></div><i class="glyphiconXIM glyphiconXIM-rightArrow"></i></div></a>');
    }
  } else {
    for (var $index in tiles) {
      var tile = tiles[$index];

buf.push('<a');
buf.push(attrs({ 'href':('#/' + (tile.id) + ''), 'target':('_self'), "class": ('tile-link') }, {"href":true,"target":true}));
buf.push('><div class="tile picker-source"><div');
buf.push(attrs({ 'style':('background-image:url("' + (tile.thumbnail) + '")'), "class": ('tile-image') }, {"style":true}));
buf.push('></div><div class="tile-title picker-source"><h5>');
var __val__ = tile.title
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h5></div><i class="glyphiconXIM glyphiconXIM-rightArrow"></i></div></a>');
   }
  }
}).call(this);

buf.push('</div>');
}
else
{
buf.push('<div class="message">no photo source configured</div>');
}
buf.push('</div>');
if ( (title))
{
buf.push('<div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div>');
}
buf.push('<div class="command-bar commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["profile"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="navbar navbar-inverse navbar-static-top"><div class="navbar-inner"><a id="back" href="#/" class="ios-only"></a><a href="#/" class="brand">People</a></div></div><div class="page"><div id="profile-bio"><img');
buf.push(attrs({ 'src':('' + (thumbnailUrl) + ''), 'alt':('' + (displayName) + ''), "class": ('profile-image') }, {"src":true,"alt":true}));
buf.push('/><h3 class="profile-name">' + escape((interp = displayName) == null ? '' : interp) + '</h3></div><div id="profile-action"><a id="photo-cast-action" href="#/albums" class="button navigation">Photo Cast</a></div></div>');
}
return buf.join("");
}
jade.templates["profileEditor"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="dialog-body">');
 var cdn = client.baseUri || '';
buf.push('<div class="loginPhone"><div class="dialogIntro"><label>');
var __val__ = resources.intro1
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><label>');
var __val__ = resources.intro2
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label></div><div class="loginProfileInputbox"><form onsubmit="return false" action="#" class="loginInputProfile"><div class="loginThumbnail"><div style="display:none" class="loginThumbnailInitial"><div');
buf.push(attrs({ 'style':('background-color:' + (backcolor) + ''), "class": ('loginProfileImage') }, {"style":true}));
buf.push('><figcaption class="loginProfileLabel"></figcaption></div><div class="loginEditMark"></div></div><div class="loginThumbnailImage"><img');
buf.push(attrs({ 'src':('' + (thumbnail) + ''), "class": ('loginProfileFigure') }, {"src":true}));
buf.push('/><div class="loginAddMark"></div></div></div>');
if ( (profile))
{
buf.push('<div class="loginInputWrap"><input');
buf.push(attrs({ 'type':("text"), 'name':("firstname"), 'placeholder':('' + (resources.inputCue1) + ''), 'value':(firstName), "class": ('ximformcontrol') + ' ' + ('dialogInputText') }, {"type":true,"name":true,"placeholder":true,"value":true}));
buf.push('/><label id="firstName" class="loginFirstNameError"></label></div><div class="loginInputWrap"><input');
buf.push(attrs({ 'type':("text"), 'name':("lastname"), 'placeholder':('' + (resources.inputCue2) + ''), 'value':(lastName), "class": ('ximformcontrol') + ' ' + ('dialogInputText') }, {"type":true,"name":true,"placeholder":true,"value":true}));
buf.push('/><label id="lastName" class="loginLastNameError"></label></div><input');
buf.push(attrs({ 'type':("submit"), 'value':('' + (resources.edit) + ''), "class": ('profileButton') }, {"type":true,"value":true}));
buf.push('/>');
}
else
{
buf.push('<div class="loginInputWrap"><input');
buf.push(attrs({ 'type':("text"), 'name':("firstname"), 'placeholder':('' + (resources.inputCue1) + ''), "class": ('ximformcontrol') + ' ' + ('dialogInputText') }, {"type":true,"name":true,"placeholder":true}));
buf.push('/><label id="firstName" class="loginFirstNameError"></label></div><div class="loginInputWrap"><input');
buf.push(attrs({ 'type':("text"), 'name':("lastname"), 'placeholder':('' + (resources.inputCue2) + ''), "class": ('ximformcontrol') + ' ' + ('dialogInputText') }, {"type":true,"name":true,"placeholder":true}));
buf.push('/><label id="lastName" class="loginLastNameError"></label></div><input');
buf.push(attrs({ 'type':("submit"), 'value':('' + (resources.next) + ''), "class": ('profileButton') }, {"type":true,"value":true}));
buf.push('/>');
}
buf.push('</form></div><div class="loginProfileAutofill"><div class="loginAutofillBreak"></div><label class="loginAutofillOr">');
var __val__ = resources.sub1
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><img');
buf.push(attrs({ 'src':('' + (cdn) + 'img/facebook-logo.png'), "class": ('loginAutofillImage') }, {"src":true}));
buf.push('/><label class="loginAutofillText">');
var __val__ = resources.fb
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label></div><div style="display:none" class="loginProfileWait"><i class="dot"></i><i class="dot"></i><i class="dot"></i><i class="dot"></i><i class="dot"></i></div></div>');
if ( (title))
{
buf.push('<div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div>');
}
buf.push('<div class="command-bar commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["profilePopUp"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="profilePopUp"><div class="profileMask"></div>');
if ( (data.initials))
{
buf.push('<div class="profileInitial"><div');
buf.push(attrs({ 'style':('background-color:' + (data.color) + ''), "class": ('personImage') }, {"style":true}));
buf.push('></div><figcaption class="personLabel">' + escape((interp = data.initials) == null ? '' : interp) + '</figcaption></div>');
}
else
{
buf.push('<img');
buf.push(attrs({ 'src':('' + (data.thumbnailUrl) + ''), "class": ('profileFigure') }, {"src":true}));
buf.push('/>');
}
buf.push('<div class="profileNameFirst">' + escape((interp = data.firstName) == null ? '' : interp) + '</div><div class="profileNameLast">' + escape((interp = data.lastName) == null ? '' : interp) + '</div></div>');
}
return buf.join("");
}
jade.templates["search"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="navbar navbar-inverse navbar-static-top"><div class="navbar-inner"><a href="#" class="brand">Search</a></div></div><div class="page"><div class="items tile-list"></div></div>');
}
return buf.join("");
}
jade.templates["searchResult"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div');
buf.push(attrs({ 'id':("" + (id) + ""), "class": ('searchResultBox') }, {"id":true}));
buf.push('><div class="searchResultIcon"><div class="searchInitials"><div');
buf.push(attrs({ 'style':('background-color:' + (backgroundColor) + ''), "class": ('searchResultImage') }, {"style":true}));
buf.push('></div><div class="searchResultInitial">' + escape((interp = initials) == null ? '' : interp) + '</div></div><div class="searchImage"><img');
buf.push(attrs({ 'src':('' + (image) + ''), "class": ('searchResultImage') }, {"src":true}));
buf.push('/></div></div><div class="searchResultName">' + escape((interp = name) == null ? '' : interp) + '</div><div class="searchResultContact">' + escape((interp = contact) == null ? '' : interp) + '</div><div style="display: none" class="searchResultPicker">');
if ( (email))
{
buf.push('<div id="email" class="searchResultChoice">' + escape((interp = email) == null ? '' : interp) + '</div>');
}
if ( (phone))
{
buf.push('<div id="phone" class="searchResultChoice">' + escape((interp = phone) == null ? '' : interp) + '</div>');
}
buf.push('</div>');
if ( (showChoice))
{
buf.push('<i class="searchResultGlyph glyphiconXIM glyphiconXIM-unselectedOption"></i>');
}
buf.push('</div>');
}
return buf.join("");
}
jade.templates["settingsScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="dialog-body"><div class="content"><div class="confirmationPopUpContent"></div><div class="userSettings">');
if ( (showPushSettings))
{
buf.push('<label class="settingsGroup">Push Notifications</label><input type="submit" value="Receive Notifications" class="settingsTogglerButton notificationsButton"/><div class="settingsToggler settingsOn"></div>');
}
buf.push('<label class="settingsGroup">Account</label><input type="submit" value="Deauthorize Device" class="settingsButton unlinkButton"/><input type="submit" value="Delete Account" class="settingsButton deleteAccountButton"/><label class="applicationVersion">Version ' + escape((interp = consts.buildFlavor) == null ? '' : interp) + ' ' + escape((interp = version) == null ? '' : interp) + '</label></div></div>');
if ( (title))
{
buf.push('<div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div>');
}
buf.push('<div class="command-bar commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["skydrive"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="dialog-body"><div id="skydrive-items" class="section">');
if ( items.length)
{
buf.push('<div class="items tile-list"></div>');
}
else
{
buf.push('<div class="message loading">' + escape((interp = resources.messages.loadingOneDrive) == null ? '' : interp) + '</div>');
}
buf.push('</div>');
if ( (title))
{
buf.push('<div class="dialogTitleBox"><label class="dialogTitle">' + escape((interp = title) == null ? '' : interp) + '</label><i id="back" class="glyphiconXIM glyphiconXIM-backArrowIcon"></i></div>');
}
buf.push('<div class="command-bar commandBarContent"></div></div>');
}
return buf.join("");
}
jade.templates["thumbnailPopUp"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="thumbnailPopUp"><div class="thumbnailMask"></div>');
if ( (data.thumbnail))
{
buf.push('<img');
buf.push(attrs({ 'src':('' + (data.thumbnail) + ''), "class": ('loginProfileFigure') + ' ' + ('thumbnailMargin') }, {"src":true}));
buf.push('/>');
}
else
{
buf.push('<div class="loginProfileInitial"><div');
buf.push(attrs({ 'style':('background-color:' + (data.color) + ''), "class": ('loginProfileImage') + ' ' + ('thumbnailMargin') }, {"style":true}));
buf.push('><figcaption class="loginProfileLabel">' + escape((interp = data.initials) == null ? '' : interp) + '    </figcaption></div></div>');
}
buf.push('<ul class="thumbnailPopUpMenu"><li id="thumbnailPopUpPhoto" class="thumbnailPopUpLi">' + escape((interp = data.resources.photo) == null ? '' : interp) + '</li><li id="thumbnailPopUpInitials" class="thumbnailPopUpLi">' + escape((interp = data.resources.initials) == null ? '' : interp) + '</li></ul></div>');
}
return buf.join("");
}
jade.templates["appTile"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
var linkTarget = (typeof target == 'undefined') ? '_self' :  target;
buf.push('<a');
buf.push(attrs({ 'href':("" + (link) + ""), 'target':('' + (linkTarget) + ''), "class": ('tile-link') }, {"href":true,"target":true}));
buf.push('><div');
buf.push(attrs({ 'style':("background-image:url('" + (backgroundImage) + "'); " + (positionStyle) + ""), "class": ('tile') }, {"style":true}));
buf.push('><div');
buf.push(attrs({ "class": ('tile-title') + ' ' + ('' + (titleClass) + '') }, {"class":true}));
buf.push('>');
if ( typeof count != 'undefined')
{
buf.push('<h5');
buf.push(attrs({ 'id':("" + (counterId) + ""), "class": ('pull-right') }, {"id":true}));
buf.push('>');
var __val__ = count 
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h5>');
}
if ( typeof title != 'undefined')
{
buf.push('<h5');
buf.push(attrs({ 'style':("" + (titlePositionStyle) + "") }, {"style":true}));
buf.push('>');
var __val__ = title
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h5>');
}
buf.push('</div></div></a>');
}
return buf.join("");
}
jade.templates["clickTile"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div');
buf.push(attrs({ 'style':("" + (positionStyle) + ""), "class": ('tile') }, {"style":true}));
buf.push('><div');
buf.push(attrs({ 'style':("background-image:url('" + (backgroundImage) + "'); width:100%; height: 100%;"), 'id':('' + (id) + ''), "class": ('click-tile') }, {"style":true,"id":true}));
buf.push('>');
if ( typeof title != 'undefined')
{
buf.push('<div');
buf.push(attrs({ 'style':("" + (titlePositionStyle) + ""), "class": ('tile-title') + ' ' + ("" + (titleClass) + "") }, {"class":true,"style":true}));
buf.push('>');
if ( typeof title != 'undefined')
{
if ( (title == ""))
{
buf.push('<h5>&nbsp</h5>');
}
else
{
buf.push('<h5>');
var __val__ = title
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h5>');
}
}
if ( typeof title2 != 'undefined')
{
if ( (title2 == ""))
{
buf.push('<h5>&nbsp</h5>');
}
else
{
buf.push('<h5>');
var __val__ = title2
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h5>');
}
}
buf.push('</div>');
}
buf.push('</div></div>');
}
return buf.join("");
}
jade.templates["folderTile"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
var tileSelected = false;
var backgroundImage = thumbnail.source;
var hasThumbnail = backgroundImage.indexOf('./img') != 0;
if ( typeof selected != 'undefined')
{
tileSelected = selected;
}
buf.push('<div');
buf.push(attrs({ "class": ('tile') + ' ' + ('' + (type) + '-tile') }, {"class":true}));
buf.push('><a');
buf.push(attrs({ 'href':("" + (link) + ""), "class": ('tile-link') }, {"href":true}));
buf.push('><div');
buf.push(attrs({ 'style':("background-image:url('" + (backgroundImage) + "')"), "class": ('image-tile') + ' ' + ('' + (hasThumbnail? 'thumbnail':'') + '') }, {"class":true,"style":true}));
buf.push('><div');
buf.push(attrs({ 'id':('' + (id) + ''), "class": ('select-image-tile') + ' ' + (''+(tileSelected?'tile-selected':'')) }, {"class":true,"id":true}));
buf.push('>');
if ( typeof title != 'undefined')
{
buf.push('<div class="tile-title"><h5>');
var __val__ = title
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h5></div>');
}
buf.push('</div></div></a></div>');
}
return buf.join("");
}
jade.templates["listTile"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div');
buf.push(attrs({ 'style':("" + (positionStyle) + ""), "class": ('tile') }, {"style":true}));
buf.push('><div');
buf.push(attrs({ 'id':('' + (id) + ''), "class": ('list-tile') }, {"id":true}));
buf.push('><div');
buf.push(attrs({ 'style':("background-image:url('" + (backgroundImage) + "'); " + (imageStyle) + ""), "class": ('image-tile') }, {"style":true}));
buf.push('></div><div');
buf.push(attrs({ 'style':("" + (titlePositionStyle) + ""), "class": ('list-title') }, {"style":true}));
buf.push('>');
if ( typeof title != 'undefined')
{
if ( (title == ""))
{
buf.push('<h5 class="bold">&nbsp</h5>');
}
else
{
buf.push('<h5 class="bold">');
var __val__ = title
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h5>');
}
}
if ( typeof title2 != 'undefined')
{
if ( (title2 == ""))
{
buf.push('<h5>&nbsp</h5>');
}
else
{
buf.push('<h5>');
var __val__ = title2
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h5>');
}
}
if ( typeof title3 != 'undefined')
{
if ( (title3 == ""))
{
buf.push('<h5>&nbsp</h5>');
}
else
{
buf.push('<h5>');
var __val__ = title3
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h5>');
}
}
buf.push('</div></div></div>');
}
return buf.join("");
}
jade.templates["mockTile"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div');
buf.push(attrs({ 'style':("background-image:url('" + (backgroundImage) + "'); " + (positionStyle) + ""), "class": ('tile') + ' ' + ('mock-tile') }, {"style":true}));
buf.push('><div');
buf.push(attrs({ "class": ('tile-title') + ' ' + ('' + (titleClass) + '') }, {"class":true}));
buf.push('>');
if ( typeof title != 'undefined')
{
buf.push('<h5');
buf.push(attrs({ 'style':("" + (titlePositionStyle) + "") }, {"style":true}));
buf.push('>');
var __val__ = title
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h5>');
}
buf.push('</div></div>');
}
return buf.join("");
}
jade.templates["photoTile"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
 var cdn = client.baseUri || '';
 var backgroundImage = thumbnail.source.replace("'", "\\'");
 var tileSelected = false;
if ( typeof selected != 'undefined')
{
 tileSelected = selected;
}
buf.push('<div class="tile"><div');
buf.push(attrs({ 'style':("background-image:url('" + (cdn) + "" + (backgroundImage) + "');"), 'id':('' + (id) + ''), "class": ('image-tile') + ' ' + ('select-image-tile') + ' ' + (''+(backgroundImage.indexOf('img/dropbox') == 0 ? 'small-icon':'')+(tileSelected ?' tile-selected':'')) }, {"class":true,"style":true,"id":true}));
buf.push('><div class="checkmark"></div></div>');
if ( typeof title != 'undefined')
{
buf.push('<div class="tile-title"><h5>');
var __val__ = title
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h5></div>');
}
buf.push('</div>');
}
return buf.join("");
}
jade.templates["pickerSource"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
var linkTarget = (typeof target == 'undefined') ? '_self' :  target;
buf.push('<a');
buf.push(attrs({ 'href':("" + (link) + ""), 'target':('' + (linkTarget) + ''), "class": ('tile-link') }, {"href":true,"target":true}));
buf.push('><div');
buf.push(attrs({ 'style':("" + (positionStyle) + ""), "class": ('tile') + ' ' + ('picker-source') }, {"style":true}));
buf.push('><div');
buf.push(attrs({ 'style':("background-image:url('" + (backgroundImage) + "')"), "class": ('tile-image') }, {"style":true}));
buf.push('></div><div');
buf.push(attrs({ "class": ('tile-title') + ' ' + ('' + (titleClass) + '') }, {"class":true}));
buf.push('>');
if ( typeof count != 'undefined')
{
buf.push('<h5');
buf.push(attrs({ 'id':("" + (counterId) + ""), "class": ('pull-right') }, {"id":true}));
buf.push('>');
var __val__ = count 
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h5>');
}
if ( typeof title != 'undefined')
{
buf.push('<h5');
buf.push(attrs({ 'style':("" + (titlePositionStyle) + "") }, {"style":true}));
buf.push('>');
var __val__ = title
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h5>');
}
buf.push('</div><i class="glyphiconXIM glyphiconXIM-rightArrow"></i></div></a>');
}
return buf.join("");
}
jade.templates["textTile"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="tile"><div');
buf.push(attrs({ 'id':('' + (id) + ''), "class": ('text-tile') + ' ' + (''+(selected? 'tile-selected' : '')) }, {"class":true,"id":true}));
buf.push('><div class="tile-title-text">');
var __val__ = title
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</div></div></div>');
}
return buf.join("");
}
jade.templates["videoTile"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
var d = client.google.parseYouTubeDuration(duration);
buf.push('<a');
buf.push(attrs({ 'href':("" + (link) + ""), "class": ('tile-link') }, {"href":true}));
buf.push('><div');
buf.push(attrs({ 'style':("" + (positionStyle) + ""), "class": ('tile') }, {"style":true}));
buf.push('><div');
buf.push(attrs({ 'id':('' + (id) + ''), "class": ('video-tile') }, {"id":true}));
buf.push('><div');
buf.push(attrs({ 'style':("background-image:url('" + (backgroundImage) + "');"), "class": ('thumbnail') }, {"style":true}));
buf.push('></div><div class="content-details"><h5>');
var __val__ = title
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h5><p class="duration">Length: ' + escape((interp = d.h ? (d.h + ' hr ') : '') == null ? '' : interp) + ' ' + escape((interp = d.m ? (d.m + ' min ') : '') == null ? '' : interp) + ' ' + escape((interp = d.h ? '' : (d.s ? (d.s + ' sec') : '')) == null ? '' : interp) + '</p><p class="published">Uploaded: ' + escape((interp = moment(published).calendar()) == null ? '' : interp) + '</p></div></div></div></a>');
}
return buf.join("");
}
jade.templates["appTile"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
var linkTarget = (typeof target == 'undefined') ? '_self' :  target;
buf.push('<a');
buf.push(attrs({ 'href':("" + (link) + ""), 'target':('' + (linkTarget) + ''), "class": ('tile-link') }, {"href":true,"target":true}));
buf.push('><div');
buf.push(attrs({ 'style':("background-image:url('" + (backgroundImage) + "'); " + (positionStyle) + ""), "class": ('tile') }, {"style":true}));
buf.push('><div');
buf.push(attrs({ "class": ('tile-title') + ' ' + ('' + (titleClass) + '') }, {"class":true}));
buf.push('>');
if ( typeof count != 'undefined')
{
buf.push('<h5');
buf.push(attrs({ 'id':("" + (counterId) + ""), "class": ('pull-right') }, {"id":true}));
buf.push('>');
var __val__ = count 
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h5>');
}
if ( typeof title != 'undefined')
{
buf.push('<h5');
buf.push(attrs({ 'style':("" + (titlePositionStyle) + "") }, {"style":true}));
buf.push('>');
var __val__ = title
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h5>');
}
buf.push('</div></div></a>');
}
return buf.join("");
}
jade.templates["clickTile"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div');
buf.push(attrs({ 'style':("" + (positionStyle) + ""), "class": ('tile') }, {"style":true}));
buf.push('><div');
buf.push(attrs({ 'style':("background-image:url('" + (backgroundImage) + "'); width:100%; height: 100%;"), 'id':('' + (id) + ''), "class": ('click-tile') }, {"style":true,"id":true}));
buf.push('>');
if ( typeof title != 'undefined')
{
buf.push('<div');
buf.push(attrs({ 'style':("" + (titlePositionStyle) + ""), "class": ('tile-title') + ' ' + ("" + (titleClass) + "") }, {"class":true,"style":true}));
buf.push('>');
if ( typeof title != 'undefined')
{
if ( (title == ""))
{
buf.push('<h5>&nbsp</h5>');
}
else
{
buf.push('<h5>');
var __val__ = title
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h5>');
}
}
if ( typeof title2 != 'undefined')
{
if ( (title2 == ""))
{
buf.push('<h5>&nbsp</h5>');
}
else
{
buf.push('<h5>');
var __val__ = title2
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h5>');
}
}
buf.push('</div>');
}
buf.push('</div></div>');
}
return buf.join("");
}
jade.templates["folderTile"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
var tileSelected = false;
var backgroundImage = thumbnail.source;
var hasThumbnail = backgroundImage.indexOf('./img') != 0;
if ( typeof selected != 'undefined')
{
tileSelected = selected;
}
buf.push('<div');
buf.push(attrs({ "class": ('tile') + ' ' + ('' + (type) + '-tile') }, {"class":true}));
buf.push('><a');
buf.push(attrs({ 'href':("" + (link) + ""), "class": ('tile-link') }, {"href":true}));
buf.push('><div');
buf.push(attrs({ 'style':("background-image:url('" + (backgroundImage) + "')"), "class": ('image-tile') + ' ' + ('' + (hasThumbnail? 'thumbnail':'') + '') }, {"class":true,"style":true}));
buf.push('><div');
buf.push(attrs({ 'id':('' + (id) + ''), "class": ('select-image-tile') + ' ' + (''+(tileSelected?'tile-selected':'')) }, {"class":true,"id":true}));
buf.push('>');
if ( typeof title != 'undefined')
{
buf.push('<div class="tile-title"><h5>');
var __val__ = title
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h5></div>');
}
buf.push('</div></div></a></div>');
}
return buf.join("");
}
jade.templates["listTile"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div');
buf.push(attrs({ 'style':("" + (positionStyle) + ""), "class": ('tile') }, {"style":true}));
buf.push('><div');
buf.push(attrs({ 'id':('' + (id) + ''), "class": ('list-tile') }, {"id":true}));
buf.push('><div');
buf.push(attrs({ 'style':("background-image:url('" + (backgroundImage) + "'); " + (imageStyle) + ""), "class": ('image-tile') }, {"style":true}));
buf.push('></div><div');
buf.push(attrs({ 'style':("" + (titlePositionStyle) + ""), "class": ('list-title') }, {"style":true}));
buf.push('>');
if ( typeof title != 'undefined')
{
if ( (title == ""))
{
buf.push('<h5 class="bold">&nbsp</h5>');
}
else
{
buf.push('<h5 class="bold">');
var __val__ = title
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h5>');
}
}
if ( typeof title2 != 'undefined')
{
if ( (title2 == ""))
{
buf.push('<h5>&nbsp</h5>');
}
else
{
buf.push('<h5>');
var __val__ = title2
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h5>');
}
}
if ( typeof title3 != 'undefined')
{
if ( (title3 == ""))
{
buf.push('<h5>&nbsp</h5>');
}
else
{
buf.push('<h5>');
var __val__ = title3
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h5>');
}
}
buf.push('</div></div></div>');
}
return buf.join("");
}
jade.templates["mockTile"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div');
buf.push(attrs({ 'style':("background-image:url('" + (backgroundImage) + "'); " + (positionStyle) + ""), "class": ('tile') + ' ' + ('mock-tile') }, {"style":true}));
buf.push('><div');
buf.push(attrs({ "class": ('tile-title') + ' ' + ('' + (titleClass) + '') }, {"class":true}));
buf.push('>');
if ( typeof title != 'undefined')
{
buf.push('<h5');
buf.push(attrs({ 'style':("" + (titlePositionStyle) + "") }, {"style":true}));
buf.push('>');
var __val__ = title
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h5>');
}
buf.push('</div></div>');
}
return buf.join("");
}
jade.templates["photoTile"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
 var cdn = client.baseUri || '';
 var backgroundImage = thumbnail.source.replace("'", "\\'");
 var tileSelected = false;
if ( typeof selected != 'undefined')
{
 tileSelected = selected;
}
buf.push('<div class="tile"><div');
buf.push(attrs({ 'style':("background-image:url('" + (cdn) + "" + (backgroundImage) + "');"), 'id':('' + (id) + ''), "class": ('image-tile') + ' ' + ('select-image-tile') + ' ' + (''+(backgroundImage.indexOf('img/dropbox') == 0 ? 'small-icon':'')+(tileSelected ?' tile-selected':'')) }, {"class":true,"style":true,"id":true}));
buf.push('><div class="checkmark"></div></div>');
if ( typeof title != 'undefined')
{
buf.push('<div class="tile-title"><h5>');
var __val__ = title
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h5></div>');
}
buf.push('</div>');
}
return buf.join("");
}
jade.templates["pickerSource"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
var linkTarget = (typeof target == 'undefined') ? '_self' :  target;
buf.push('<a');
buf.push(attrs({ 'href':("" + (link) + ""), 'target':('' + (linkTarget) + ''), "class": ('tile-link') }, {"href":true,"target":true}));
buf.push('><div');
buf.push(attrs({ 'style':("" + (positionStyle) + ""), "class": ('tile') + ' ' + ('picker-source') }, {"style":true}));
buf.push('><div');
buf.push(attrs({ 'style':("background-image:url('" + (backgroundImage) + "')"), "class": ('tile-image') }, {"style":true}));
buf.push('></div><div');
buf.push(attrs({ "class": ('tile-title') + ' ' + ('' + (titleClass) + '') }, {"class":true}));
buf.push('>');
if ( typeof count != 'undefined')
{
buf.push('<h5');
buf.push(attrs({ 'id':("" + (counterId) + ""), "class": ('pull-right') }, {"id":true}));
buf.push('>');
var __val__ = count 
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h5>');
}
if ( typeof title != 'undefined')
{
buf.push('<h5');
buf.push(attrs({ 'style':("" + (titlePositionStyle) + "") }, {"style":true}));
buf.push('>');
var __val__ = title
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h5>');
}
buf.push('</div><i class="glyphiconXIM glyphiconXIM-rightArrow"></i></div></a>');
}
return buf.join("");
}
jade.templates["textTile"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="tile"><div');
buf.push(attrs({ 'id':('' + (id) + ''), "class": ('text-tile') + ' ' + (''+(selected? 'tile-selected' : '')) }, {"class":true,"id":true}));
buf.push('><div class="tile-title-text">');
var __val__ = title
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</div></div></div>');
}
return buf.join("");
}
jade.templates["videoTile"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
var d = client.google.parseYouTubeDuration(duration);
buf.push('<a');
buf.push(attrs({ 'href':("" + (link) + ""), "class": ('tile-link') }, {"href":true}));
buf.push('><div');
buf.push(attrs({ 'style':("" + (positionStyle) + ""), "class": ('tile') }, {"style":true}));
buf.push('><div');
buf.push(attrs({ 'id':('' + (id) + ''), "class": ('video-tile') }, {"id":true}));
buf.push('><div');
buf.push(attrs({ 'style':("background-image:url('" + (backgroundImage) + "');"), "class": ('thumbnail') }, {"style":true}));
buf.push('></div><div class="content-details"><h5>');
var __val__ = title
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h5><p class="duration">Length: ' + escape((interp = d.h ? (d.h + ' hr ') : '') == null ? '' : interp) + ' ' + escape((interp = d.m ? (d.m + ' min ') : '') == null ? '' : interp) + ' ' + escape((interp = d.h ? '' : (d.s ? (d.s + ' sec') : '')) == null ? '' : interp) + '</p><p class="published">Uploaded: ' + escape((interp = moment(published).calendar()) == null ? '' : interp) + '</p></div></div></div></a>');
}
return buf.join("");
}
jade.templates["timelineScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
 var cdn = client.baseUri || '';
buf.push('<div class="timelineContent"><div class="serverMessagePopUpContent"></div><div class="timelineGroups"><div class="timelineGroupsXims loading"></div><div class="commandBarPadding"></div><div class="timelineNoXims hidden"><i class="glyphiconXIM glyphiconXIM-dotMessage noximsglyph"></i><div class="noximsTitle1">' + escape((interp = messages.timeline_Title1) == null ? '' : interp) + '\n' + escape((interp = messages.timeline_Title2) == null ? '' : interp) + '</div>');
if ( isWp)
{
buf.push('<object');
buf.push(attrs({ 'data':('' + (cdn) + 'svg/lonelyClouds.svg'), 'type':('image/svg+xml'), "class": ('timelineLonely') }, {"data":true,"type":true}));
buf.push('></object>');
}
else
{
buf.push('<img');
buf.push(attrs({ 'src':('' + (cdn) + 'svg/lonelyClouds.svg'), "class": ('timelineLonely') }, {"src":true}));
buf.push('/>');
}
buf.push('</div></div></div>');
}
return buf.join("");
}
jade.templates["tokenScreen"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="navbar navbar-inverse navbar-static-top"><div class="navbar-inner"><a id="back" href="#/" class="ios-only"></a></div></div><div class="page"><Live>ID Refresh Token</Live><div class="tokenBox"><select id="tokenSelect"><option value="fb">facebook</option><option value="live">liveRefresh</option><option value="dropbox">dropbox</option><option value="instagram">instagram</option></select><textarea style="width:96%;height:400px" class="token">' + escape((interp = token) == null ? '' : interp) + '</textarea></div><div class="actions"><button style="height:40px;width:120px" class="set">set</button></div></div>');
}
return buf.join("");
}
jade.templates["upsellBar"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="chromeTopBase"><label class="upsellTitle">xim</label><label class="upsellDescription">');
var __val__ = text.description
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label><label class="upsellLink">');
var __val__ = text.link
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</label></div>');
}
return buf.join("");
}
jade.templates["userProfile"] = function(locals, attrs, escape, rethrow, merge
/**/) {
attrs = attrs || jade.attrs; escape = escape || jade.escape; rethrow = rethrow || jade.rethrow; merge = merge || jade.merge;
var buf = [];
with (locals || {}) {
var interp;
buf.push('<div class="navbar navbar-inverse navbar-static-top"><div class="navbar-inner"><a href="#" class="brand">People</a></div></div><div class="page"><div class="profile-card"><div');
buf.push(attrs({ 'style':("background-image:url('" + (user.pic) + "');"), "class": ('pull-left') + ' ' + ('image-tile') }, {"style":true}));
buf.push('></div><div class="profile-body"><h4>');
var __val__ = user.name
buf.push(escape(null == __val__ ? "" : __val__));
buf.push('</h4></div></div></div>');
}
return buf.join("");
}